@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
-    'name' : 'Invoicing',
-    'version' : '1.2',
-    'summary': 'Invoices & Payments',
+    'name': 'Invoicing',
+    'version': '1.3',
+    'summary': 'Invoices, Payments, Follow-ups & Bank Synchronization',
     'sequence': 10,
     'description': """
 Invoicing & Payments
@@ -14,8 +14,7 @@ You could use this simplified accounting in case you work with an (external) acc
     """,
     'category': 'Accounting/Accounting',
     'website': 'https://www.odoo.com/app/invoicing',
-    'images' : ['images/accounts.jpeg','images/bank_statement.jpeg','images/cash_register.jpeg','images/chart_of_accounts.jpeg','images/customer_invoice.jpeg','images/journal_entries.jpeg'],
-    'depends' : ['base_setup', 'product', 'analytic', 'portal', 'digest'],
+    'depends': ['base_setup', 'onboarding', 'product', 'analytic', 'portal', 'digest'],
     'data': [
         'security/account_security.xml',
         'security/ir.model.access.csv',
@@ -23,8 +22,12 @@ You could use this simplified accounting in case you work with an (external) acc
         'data/digest_data.xml',
         'views/account_report.xml',
         'data/mail_template_data.xml',
+        'data/onboarding_data.xml',
+        'data/account_tour.xml',
+        'data/ir_sequence.xml',
         'views/account_payment_view.xml',
         'wizard/account_automatic_entry_wizard_views.xml',
+        'wizard/account_autopost_bills_wizard.xml',
         'wizard/account_unreconcile_view.xml',
         'wizard/account_move_reversal_view.xml',
         'wizard/account_resequence_views.xml',
@@ -40,7 +43,7 @@ You could use this simplified accounting in case you work with an (external) acc
         'views/account_tax_views.xml',
         'views/account_full_reconcile_views.xml',
         'views/account_payment_term_views.xml',
-        'views/account_chart_template_views.xml',
+        'views/account_payment_method.xml',
         'views/res_partner_bank_views.xml',
         'views/report_statement.xml',
         'views/terms_template.xml',
@@ -54,25 +57,34 @@ You could use this simplified accounting in case you work with an (external) acc
         'views/report_invoice.xml',
         'report/account_invoice_report_view.xml',
         'views/account_cash_rounding_view.xml',
+        'views/ir_actions_views.xml',
         'views/ir_module_views.xml',
         'views/res_config_settings_views.xml',
         'views/partner_view.xml',
         'views/account_journal_dashboard_view.xml',
         'views/account_portal_templates.xml',
         'views/report_payment_receipt_templates.xml',
-        'views/account_onboarding_templates.xml',
         'data/service_cron.xml',
         'views/account_incoterms_view.xml',
         'data/account_incoterms_data.xml',
         'views/digest_views.xml',
-        'wizard/account_invoice_send_views.xml',
+        'wizard/account_move_send_wizard.xml',
+        'wizard/account_move_send_batch_wizard.xml',
         'report/account_hash_integrity_templates.xml',
         'views/res_currency.xml',
         'views/account_menuitem.xml',
-        'wizard/account_tour_upload_bill.xml',
+        'wizard/account_secure_entries_wizard.xml',
+        'views/mail_message_views.xml',
         'wizard/accrued_orders.xml',
         'views/bill_preview_template.xml',
         'data/account_reports_data.xml',
+        'views/uom_uom_views.xml',
+        'views/product_views.xml',
+        'views/tests_shared_js_python.xml',
+        'views/base_document_layout_views.xml',
+        'views/account_lock_exception_views.xml',
+        'views/report_templates.xml',
+        'wizard/account_merge_wizard_views.xml',
     ],
     'demo': [
         'demo/account_demo.xml',
@@ -87,29 +99,46 @@ You could use this simplified accounting in case you work with an (external) acc
         'web.assets_backend': [
             'account/static/src/css/account_bank_and_cash.css',
             'account/static/src/css/account.css',
+            'account/static/src/css/account_payment.scss',
             'account/static/src/scss/account_journal_dashboard.scss',
-            'account/static/src/scss/account_dashboard.scss',
             'account/static/src/scss/account_searchpanel.scss',
-            'account/static/src/scss/legacy_account_activity.scss',
-            'account/static/src/js/legacy_account_payment_field.js',
-            'account/static/src/js/legacy_mail_activity.js',
-            'account/static/src/js/legacy_tax_totals.js',
-            'account/static/src/js/legacy_section_and_note_fields_backend.js',
-            'account/static/src/js/legacy_account_selection.js',
-            'account/static/src/js/legacy_open_move_widget.js',
+            'account/static/src/scss/account_payment_term.scss',
+            'account/static/src/scss/account_reconcile_model.scss',
+            'account/static/src/scss/account_multi_ledger.scss',
             'account/static/src/components/**/*',
+            'account/static/src/services/*.js',
+            'account/static/src/views/**/*',
             'account/static/src/js/tours/account.js',
-            'account/static/src/xml/**/*',
+            'account/static/src/js/search/search_bar/search_bar.js',
+            'account/static/src/helpers/*.js',
+            'account/static/src/core/utils/*.js',
+        ],
+        # Unit test files
+        'web.assets_unit_tests': [
+            'account/static/tests/**/*',
+            ('remove', 'account/static/tests/legacy/**/*'),  # to remove when all legacy tests are ported
+            ('remove', 'account/static/tests/helpers/**/*'),
+            ('remove', 'account/static/tests/tours/**/*'),
         ],
         'web.assets_frontend': [
             'account/static/src/js/account_portal_sidebar.js',
             'account/static/src/js/account_portal.js',
+            'account/static/src/components/tests_shared_js_python/*',
+            'account/static/src/helpers/*.js',
+            'account/static/src/core/utils/*.js',
         ],
         'web.assets_tests': [
             'account/static/tests/tours/**/*',
         ],
         'web.qunit_suite_tests': [
-            'account/static/tests/*.js',
+            'account/static/tests/helpers/*.js',
+            'account/static/tests/legacy/*.js',
+        ],
+        'web.report_assets_common': [
+            'account/static/src/css/report_invoice.css',
+        ],
+        'web.report_assets_pdf': [
+            'account/static/src/css/report_invoice.css',
         ],
     },
     'license': 'LGPL-3',
