@@ -14,19 +14,24 @@ class Digest(models.Model):
     def _compute_kpi_account_total_revenue_value(self):
         if not self.env.user.has_group('account.group_account_invoice'):
             raise AccessError(_("Do not have access, skip this data for user's digest email"))
+
+        start, end, companies = self._get_kpi_compute_parameters()
+
+        total_per_companies = dict(self.env['account.move.line'].sudo()._read_group(
+            groupby=['company_id'],
+            aggregates=['balance:sum'],
+            domain=[
+                ('company_id', 'in', companies.ids),
+                ('date', '>', start),
+                ('date', '<=', end),
+                ('account_id.internal_group', '=', 'income'),
+                ('parent_state', '=', 'posted'),
+            ],
+        ))
+
         for record in self:
-            start, end, company = record._get_kpi_compute_parameters()
-            self._cr.execute('''
-                SELECT -SUM(line.balance)
-                FROM account_move_line line
-                JOIN account_move move ON move.id = line.move_id
-                JOIN account_account account ON account.id = line.account_id
-                WHERE line.company_id = %s AND line.date > %s::DATE AND line.date <= %s::DATE
-                AND account.internal_group = 'income'
-                AND move.state = 'posted'
-            ''', [company.id, start, end])
-            query_res = self._cr.fetchone()
-            record.kpi_account_total_revenue_value = query_res and query_res[0] or 0.0
+            company = record.company_id or self.env.company
+            record.kpi_account_total_revenue_value = -total_per_companies.get(company, 0)
 
     def _compute_kpis_actions(self, company, user):
         res = super(Digest, self)._compute_kpis_actions(company, user)
