@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
+from datetime import date
 
 from odoo import api, fields, models, _
 from odoo.exceptions import ValidationError
 from odoo.tools.misc import format_date
-from odoo.tools import frozendict
+from odoo.tools import frozendict, mute_logger, date_utils, SQL
 
 import re
 from collections import defaultdict
-from psycopg2 import sql
+from psycopg2 import errors as pgerrors
 
 
 class SequenceMixin(models.AbstractModel):
@@ -23,9 +24,22 @@ class SequenceMixin(models.AbstractModel):
     _sequence_field = "name"
     _sequence_date_field = "date"
     _sequence_index = False
-    _sequence_monthly_regex = r'^(?P<prefix1>.*?)(?P<year>((?<=\D)|(?<=^))((19|20|21)\d{2}|(\d{2}(?=\D))))(?P<prefix2>\D*?)(?P<month>(0[1-9]|1[0-2]))(?P<prefix3>\D+?)(?P<seq>\d*)(?P<suffix>\D*?)$'
-    _sequence_yearly_regex = r'^(?P<prefix1>.*?)(?P<year>((?<=\D)|(?<=^))((19|20|21)?\d{2}))(?P<prefix2>\D+?)(?P<seq>\d*)(?P<suffix>\D*?)$'
-    _sequence_fixed_regex = r'^(?P<prefix1>.*?)(?P<seq>\d{0,9})(?P<suffix>\D*?)$'
+
+    prefix = r'(?P<prefix1>.*?)'
+    prefix2 = r'(?P<prefix2>\D)'
+    prefix3 = r'(?P<prefix3>\D+?)'
+    seq = r'(?P<seq>\d*)'
+    month = r'(?P<month>(0[1-9]|1[0-2]))'
+    # `(19|20|21)` is for catching 19 20 and 21 century prefixes
+    year = r'(?P<year>((?<=\D)|(?<=^))((19|20|21)\d{2}|(\d{2}(?=\D))))'
+    year_end = r'(?P<year_end>((?<=\D)|(?<=^))((19|20|21)\d{2}|(\d{2}(?=\D))))'
+    suffix = r'(?P<suffix>\D*?)'
+
+    _sequence_year_range_monthly_regex = fr'^{prefix}{year}{prefix2}{year_end}(?P<prefix3>\D){month}(?P<prefix4>\D+?){seq}{suffix}$'
+    _sequence_year_range_regex = fr'^(?:{prefix}{year}{prefix2}{year_end}{prefix3})?{seq}{suffix}$'
+    _sequence_monthly_regex = fr'^{prefix}{year}(?P<prefix2>\D*?){month}{prefix3}{seq}{suffix}$'
+    _sequence_yearly_regex = fr'^{prefix}(?P<year>((?<=\D)|(?<=^))((19|20|21)?\d{{2}}))(?P<prefix2>\D+?){seq}{suffix}$'
+    _sequence_fixed_regex = fr'^{prefix}(?P<seq>\d{{0,9}}){suffix}$'
 
     sequence_prefix = fields.Char(compute='_compute_split_sequence', store=True)
     sequence_number = fields.Integer(compute='_compute_split_sequence', store=True)
@@ -34,22 +48,56 @@ class SequenceMixin(models.AbstractModel):
         # Add an index to optimise the query searching for the highest sequence number
         if not self._abstract and self._sequence_index:
             index_name = self._table + '_sequence_index'
-            self.env.cr.execute('SELECT indexname FROM pg_indexes WHERE indexname = %s', (index_name,))
+            self.env.cr.execute(SQL('SELECT indexname FROM pg_indexes WHERE indexname = %s', index_name))
             if not self.env.cr.fetchone():
-                self.env.cr.execute(sql.SQL("""
-                    CREATE INDEX {index_name} ON {table} ({sequence_index}, sequence_prefix desc, sequence_number desc, {field});
-                    CREATE INDEX {index2_name} ON {table} ({sequence_index}, id desc, sequence_prefix);
-                """).format(
-                    sequence_index=sql.Identifier(self._sequence_index),
-                    index_name=sql.Identifier(index_name),
-                    index2_name=sql.Identifier(index_name + "2"),
-                    table=sql.Identifier(self._table),
-                    field=sql.Identifier(self._sequence_field),
+                self.env.cr.execute(SQL("""
+                    CREATE INDEX %(index_name)s ON %(table)s (%(sequence_index)s, sequence_prefix desc, sequence_number desc, %(field)s);
+                    CREATE INDEX %(index2_name)s ON %(table)s (%(sequence_index)s, id desc, sequence_prefix);
+                    """,
+                    sequence_index=SQL.identifier(self._sequence_index),
+                    index_name=SQL.identifier(index_name),
+                    index2_name=SQL.identifier(index_name + "2"),
+                    table=SQL.identifier(self._table),
+                    field=SQL.identifier(self._sequence_field),
                 ))
 
+    def _get_sequence_date_range(self, reset):
+        ref_date = fields.Date.to_date(self[self._sequence_date_field])
+        if reset in ('year', 'year_range', 'year_range_month'):
+            return (date(ref_date.year, 1, 1), date(ref_date.year, 12, 31), None, None)
+        if reset == 'month':
+            return date_utils.get_month(ref_date) + (None, None)
+        if reset == 'never':
+            return (date(1, 1, 1), date(9999, 12, 31), None, None)
+        raise NotImplementedError(reset)
+
     def _must_check_constrains_date_sequence(self):
         return True
 
+    def _year_match(self, format_value, year):
+        return format_value == self._truncate_year_to_length(year, len(str(format_value)))
+
+    def _truncate_year_to_length(self, year, length):
+        return year % (10 ** length)
+
+    def _sequence_matches_date(self):
+        self.ensure_one()
+        date = fields.Date.to_date(self[self._sequence_date_field])
+        sequence = self[self._sequence_field]
+
+        if not sequence or not date:
+            return True
+
+        format_values = self._get_sequence_format_param(sequence)[1]
+        sequence_number_reset = self._deduce_sequence_number_reset(sequence)
+        date_start, date_end, forced_year_start, forced_year_end = self._get_sequence_date_range(sequence_number_reset)
+        year_match = (
+            (not format_values["year"] or self._year_match(format_values["year"], forced_year_start or date_start.year))
+            and (not format_values["year_end"] or self._year_match(format_values["year_end"], forced_year_end or date_end.year))
+        )
+        month_match = not format_values['month'] or format_values['month'] == date.month
+        return year_match and month_match
+
     @api.constrains(lambda self: (self._sequence_field, self._sequence_date_field))
     def _constrains_date_sequence(self):
         # Make it possible to bypass the constraint to allow edition of already messed up documents.
@@ -63,28 +111,26 @@ class SequenceMixin(models.AbstractModel):
                 continue
             date = fields.Date.to_date(record[record._sequence_date_field])
             sequence = record[record._sequence_field]
-            if sequence and date and date > constraint_date:
-                format_values = record._get_sequence_format_param(sequence)[1]
-                if (
-                    format_values['year'] and format_values['year'] != date.year % 10**len(str(format_values['year']))
-                    or format_values['month'] and format_values['month'] != date.month
-                ):
-                    raise ValidationError(_(
-                        "The %(date_field)s (%(date)s) doesn't match the sequence number of the related %(model)s (%(sequence)s)\n"
-                        "You will need to clear the %(model)s's %(sequence_field)s to proceed.\n"
-                        "In doing so, you might want to resequence your entries in order to maintain a continuous date-based sequence.",
-                        date=format_date(self.env, date),
-                        sequence=sequence,
-                        date_field=record._fields[record._sequence_date_field]._description_string(self.env),
-                        sequence_field=record._fields[record._sequence_field]._description_string(self.env),
-                        model=self.env['ir.model']._get(record._name).display_name,
-                    ))
+            if (
+                sequence
+                and date
+                and date > constraint_date
+                and not record._sequence_matches_date()
+            ):
+                raise ValidationError(_(
+                    "The %(date_field)s (%(date)s) you've entered isn't aligned with the existing sequence number (%(sequence)s). Clear the sequence number to proceed.\n"
+                    "To maintain date-based sequences, select entries and use the resequence option from the actions menu, available in developer mode.",
+                    date_field=record._fields[record._sequence_date_field]._description_string(self.env),
+                    date=format_date(self.env, date),
+                    sequence=sequence,
+                ))
 
     @api.depends(lambda self: [self._sequence_field])
     def _compute_split_sequence(self):
         for record in self:
             sequence = record[record._sequence_field] or ''
-            regex = re.sub(r"\?P<\w+>", "?:", record._sequence_fixed_regex.replace(r"?P<seq>", ""))  # make the seq the only matching group
+            # make the seq the only matching group
+            regex = self._make_regex_non_capturing(record._sequence_fixed_regex.replace(r"?P<seq>", ""))
             matching = re.match(regex, sequence)
             record.sequence_prefix = sequence[:matching.start(1)]
             record.sequence_number = int(matching.group(1) or 0)
@@ -98,20 +144,47 @@ class SequenceMixin(models.AbstractModel):
             sequence.
         """
         for regex, ret_val, requirements in [
+            (self._sequence_year_range_monthly_regex, 'year_range_month', ['seq', 'year', 'year_end', 'month']),
             (self._sequence_monthly_regex, 'month', ['seq', 'month', 'year']),
+            (self._sequence_year_range_regex, 'year_range', ['seq', 'year', 'year_end']),
             (self._sequence_yearly_regex, 'year', ['seq', 'year']),
             (self._sequence_fixed_regex, 'never', ['seq']),
         ]:
             match = re.match(regex, name or '')
             if match:
                 groupdict = match.groupdict()
-                if all(req in groupdict for req in requirements):
+                if (
+                    groupdict.get('year_end') and groupdict.get('year')
+                    and (
+                        len(groupdict['year']) < len(groupdict['year_end'])
+                        or self._truncate_year_to_length((int(groupdict['year']) + 1), len(groupdict['year_end'])) != int(groupdict['year_end'])
+                    )
+                ):
+                    # year and year_end are not compatible for range (the difference is not 1)
+                    continue
+                if all(groupdict.get(req) is not None for req in requirements):
                     return ret_val
         raise ValidationError(_(
             'The sequence regex should at least contain the seq grouping keys. For instance:\n'
-            '^(?P<prefix1>.*?)(?P<seq>\d*)(?P<suffix>\D*?)$'
+            r'^(?P<prefix1>.*?)(?P<seq>\d*)(?P<suffix>\D*?)$'
         ))
 
+    def _make_regex_non_capturing(self, regex):
+        r""" Replace the "named capturing group" found in the regex by
+        "non-capturing group" instead.
+
+        Example:
+        `^(?P<prefix1>.*?)(?P<seq>\d{0,9})(?P<suffix>\D*?)$` will become
+        `^(?:.*?)(?:\d{0,9})(?:\D*?)$`
+        - `(?P<name>...)` = Named capturing groups
+        - `(?:...)` = Non-capturing group
+
+        :param regex: the regex to modify
+
+        :return: the modified regex
+        """
+        return re.sub(r"\?P<\w+>", "?:", regex)
+
     def _get_last_sequence_domain(self, relaxed=False):
         """Get the sql domain to retreive the previous sequence number.
 
@@ -138,7 +211,7 @@ class SequenceMixin(models.AbstractModel):
         self.ensure_one()
         return "00000000"
 
-    def _get_last_sequence(self, relaxed=False, with_prefix=None, lock=True):
+    def _get_last_sequence(self, relaxed=False, with_prefix=None):
         """Retrieve the previous sequence.
 
         This is done by taking the number with the greatest alphabetical value within
@@ -163,29 +236,20 @@ class SequenceMixin(models.AbstractModel):
         if self._sequence_field not in self._fields or not self._fields[self._sequence_field].store:
             raise ValidationError(_('%s is not a stored field', self._sequence_field))
         where_string, param = self._get_last_sequence_domain(relaxed)
-        if self.id or self.id.origin:
+        if self._origin.id:
             where_string += " AND id != %(id)s "
-            param['id'] = self.id or self.id.origin
+            param['id'] = self._origin.id
         if with_prefix is not None:
             where_string += " AND sequence_prefix = %(with_prefix)s "
             param['with_prefix'] = with_prefix
 
         query = f"""
-                SELECT {{field}} FROM {self._table}
+                SELECT {self._sequence_field} FROM {self._table}
                 {where_string}
                 AND sequence_prefix = (SELECT sequence_prefix FROM {self._table} {where_string} ORDER BY id DESC LIMIT 1)
                 ORDER BY sequence_number DESC
                 LIMIT 1
         """
-        if lock:
-            query = f"""
-            UPDATE {self._table} SET write_date = write_date WHERE id = (
-                {query.format(field='id')}
-            )
-            RETURNING {self._sequence_field};
-            """
-        else:
-            query = query.format(field=self._sequence_field)
 
         self.flush_model([self._sequence_field, 'sequence_number', 'sequence_prefix'])
         self.env.cr.execute(query, param)
@@ -204,24 +268,29 @@ class SequenceMixin(models.AbstractModel):
         regex = self._sequence_fixed_regex
         if sequence_number_reset == 'year':
             regex = self._sequence_yearly_regex
+        elif sequence_number_reset == 'year_range':
+            regex = self._sequence_year_range_regex
         elif sequence_number_reset == 'month':
             regex = self._sequence_monthly_regex
-
+        elif sequence_number_reset == 'year_range_month':
+            regex = self._sequence_year_range_monthly_regex
         format_values = re.match(regex, previous).groupdict()
         format_values['seq_length'] = len(format_values['seq'])
-        format_values['year_length'] = len(format_values.get('year', ''))
+        format_values['year_length'] = len(format_values.get('year') or '')
+        format_values['year_end_length'] = len(format_values.get('year_end') or '')
         if not format_values.get('seq') and 'prefix1' in format_values and 'suffix' in format_values:
             # if we don't have a seq, consider we only have a prefix and not a suffix
             format_values['prefix1'] = format_values['suffix']
             format_values['suffix'] = ''
-        for field in ('seq', 'year', 'month'):
+        for field in ('seq', 'year', 'month', 'year_end'):
             format_values[field] = int(format_values.get(field) or 0)
 
-        placeholders = re.findall(r'(prefix\d|seq|suffix\d?|year|month)', regex)
+        placeholders = re.findall(r'\b(prefix\d|seq|suffix\d?|year|year_end|month)\b', regex)
         format = ''.join(
             "{seq:0{seq_length}d}" if s == 'seq' else
             "{month:02d}" if s == 'month' else
             "{year:0{year_length}d}" if s == 'year' else
+            "{year_end:0{year_end_length}d}" if s == 'year_end' else
             "{%s}" % s
             for s in placeholders
         )
@@ -242,15 +311,43 @@ class SequenceMixin(models.AbstractModel):
         if new:
             last_sequence = self._get_last_sequence(relaxed=True) or self._get_starting_sequence()
 
-        format, format_values = self._get_sequence_format_param(last_sequence)
+        format_string, format_values = self._get_sequence_format_param(last_sequence)
+        sequence_number_reset = self._deduce_sequence_number_reset(last_sequence)
         if new:
+            date_start, date_end, forced_year_start, forced_year_end = self._get_sequence_date_range(sequence_number_reset)
             format_values['seq'] = 0
-            format_values['year'] = self[self._sequence_date_field].year % (10 ** format_values['year_length'])
+            format_values['year'] = self._truncate_year_to_length(forced_year_start or date_start.year, format_values['year_length'])
+            format_values['year_end'] = self._truncate_year_to_length(forced_year_end or date_end.year, format_values['year_end_length'])
             format_values['month'] = self[self._sequence_date_field].month
-        format_values['seq'] = format_values['seq'] + 1
 
-        self[self._sequence_field] = format.format(**format_values)
+        # before flushing inside the savepoint (which may be rolled back!), make sure everything
+        # is already flushed, otherwise we could lose non-sequence fields values, as the ORM believes
+        # them to be flushed.
+        self.flush_recordset()
+        # because we are flushing, and because the business code might be flushing elsewhere (i.e. to
+        # validate constraints), the fields depending on the sequence field might be protected by the
+        # ORM. This is not desired, so we already reset them here.
+        registry = self.env.registry
+        triggers = registry._field_triggers[self._fields[self._sequence_field]]
+        for inverse_field, triggered_fields in triggers.items():
+            for triggered_field in triggered_fields:
+                if not triggered_field.store or not triggered_field.compute:
+                    continue
+                for field in registry.field_inverses[inverse_field[0]] if inverse_field else [None]:
+                    self.env.add_to_compute(triggered_field, self[field.name] if field else self)
+        while True:
+            format_values['seq'] = format_values['seq'] + 1
+            sequence = format_string.format(**format_values)
+            try:
+                with self.env.cr.savepoint(flush=False), mute_logger('odoo.sql_db'):
+                    self[self._sequence_field] = sequence
+                    self.flush_recordset([self._sequence_field])
+                    break
+            except (pgerrors.ExclusionViolation, pgerrors.UniqueViolation):
+                pass
         self._compute_split_sequence()
+        self.flush_recordset(['sequence_prefix', 'sequence_number'])
+
 
     def _is_last_from_seq_chain(self):
         """Tells whether or not this element is the last one of the sequence chain.
