@@ -2,10 +2,13 @@
 
 import { registry } from "@web/core/registry";
 import { useService } from "@web/core/utils/hooks";
-
-const { Component } = owl;
+import { standardFieldProps } from "@web/views/fields/standard_field_props";
+import { Component } from "@odoo/owl";
 
 class OpenMoveWidget extends Component {
+    static template = "account.OpenMoveWidget";
+    static props = { ...standardFieldProps };
+
     setup() {
         super.setup();
         this.action = useService("action");
@@ -21,5 +24,6 @@ class OpenMoveWidget extends Component {
     }
 }
 
-OpenMoveWidget.template = "account.OpenMoveWidget";
-registry.category("fields").add("open_move_widget", OpenMoveWidget);
+registry.category("fields").add("open_move_widget", {
+    component: OpenMoveWidget,
+});
