@@ -12,13 +12,15 @@ import math
 class TestPrintCheck(AccountTestInvoicingCommon):
 
     @classmethod
-    def setUpClass(cls, chart_template_ref=None):
-        super().setUpClass(chart_template_ref=chart_template_ref)
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.other_currency = cls.setup_other_currency('EUR')
 
         bank_journal = cls.company_data['default_journal_bank']
 
         cls.payment_method_line_check = bank_journal.outbound_payment_method_line_ids\
             .filtered(lambda l: l.code == 'check_printing')
+        cls.payment_method_line_check.payment_account_id = cls.inbound_payment_method_line.payment_account_id
 
     def test_in_invoice_check_manual_sequencing(self):
         ''' Test the check generation for vendor bills. '''
@@ -119,7 +121,7 @@ class TestPrintCheck(AccountTestInvoicingCommon):
             'invoice_date': '2016-01-01',
             'invoice_line_ids': [Command.create({
                 'product_id': self.product_a.id,
-                'price_unit': 100.0,
+                'price_unit': 150.0,
                 'tax_ids': []
             })]
         })
@@ -128,7 +130,7 @@ class TestPrintCheck(AccountTestInvoicingCommon):
         # Partial payment in foreign currency.
         payment = self.env['account.payment.register'].with_context(active_model='account.move', active_ids=invoice.ids).create({
             'payment_method_line_id': self.payment_method_line_check.id,
-            'currency_id': self.currency_data['currency'].id,
+            'currency_id': self.other_currency.id,
             'amount': 150.0,
             'payment_date': '2017-01-01',
         })._create_payments()
@@ -138,9 +140,9 @@ class TestPrintCheck(AccountTestInvoicingCommon):
         self.assertEqual(stub_pages, [[{
             'due_date': '01/01/2016',
             'number': invoice.name,
-            'amount_total': f'${NON_BREAKING_SPACE}100.00',
-            'amount_residual': f'${NON_BREAKING_SPACE}50.00',
-            'amount_paid': f'150.000{NON_BREAKING_SPACE}☺',
+            'amount_total': f'${NON_BREAKING_SPACE}150.00',
+            'amount_residual': f'${NON_BREAKING_SPACE}75.00',
+            'amount_paid': f'150.00{NON_BREAKING_SPACE}€',
             'currency': invoice.currency_id,
         }]])
 
@@ -175,12 +177,9 @@ class TestPrintCheck(AccountTestInvoicingCommon):
 
         self.assertEqual(set(payments.mapped('check_number')), {str(x) for x in range(11111, 11111 + nb_invoices_to_test)})
 
-    def test_print_great_pre_number_check(self):
-        """
-        Make sure we can use integer of more than 2147483647 in check sequence
-         limit of `integer` type in psql: https://www.postgresql.org/docs/current/datatype-numeric.html
-        """
+    def test_check_label(self):
         payment = self.env['account.payment'].create({
+            'check_number': '2147483647',
             'payment_type': 'outbound',
             'partner_type': 'supplier',
             'amount': 100.0,
@@ -188,5 +187,28 @@ class TestPrintCheck(AccountTestInvoicingCommon):
             'payment_method_line_id': self.payment_method_line_check.id,
         })
         payment.action_post()
+
+        for move in payment.move_id:
+            self.assertRecordValues(move.line_ids, [{'name': "Checks - 2147483647"}] * len(move.line_ids))
+
+    def test_print_great_pre_number_check(self):
+        """
+        Make sure we can use integer of more than 2147483647 in check sequence
+         limit of `integer` type in psql: https://www.postgresql.org/docs/current/datatype-numeric.html
+        """
+        vals = {
+            'payment_type': 'outbound',
+            'partner_type': 'supplier',
+            'amount': 100.0,
+            'journal_id': self.company_data['default_journal_bank'].id,
+            'payment_method_line_id': self.payment_method_line_check.id,
+        }
+        payment = self.env['account.payment'].create(vals)
+        payment.action_post()
         self.assertTrue(payment.write({'check_number': '2147483647'}))
         self.assertTrue(payment.write({'check_number': '2147483648'}))
+
+        payment_2 = self.env['account.payment'].create(vals)
+        payment_2.action_post()
+        action_window = payment_2.print_checks()
+        self.assertEqual(action_window['context']['default_next_check_number'], '2147483649', "Check number should have been incremented without error.")
