@@ -4,16 +4,15 @@
     'name': 'Accounting/Fleet bridge',
     'category': 'Accounting/Accounting',
     'summary': 'Manage accounting with fleets',
-    'description': "",
     'version': '1.0',
     'depends': ['fleet', 'account'],
     'data': [
         'data/fleet_service_type_data.xml',
         'views/account_move_views.xml',
         'views/fleet_vehicle_views.xml',
+        'views/fleet_vehicle_log_services_views.xml'
     ],
     'installable': True,
     'auto_install': True,
-    'application': False,
-    'license': 'OEEL-1',
+    'license': 'LGPL-3',
 }
