@@ -1,10 +1,16 @@
 /** @odoo-module **/
 
+import { registry } from "@web/core/registry";
+import { batchedOrmService } from "@analytic/services/batched_orm_service";
+
 import {
+    addRow,
     click,
     editInput,
     getFixture,
     nextTick,
+    selectDropdownItem,
+    triggerEvent,
     triggerHotkey,
 } from "@web/../tests/helpers/utils";
 import { makeView, setupViewRegistries } from "@web/../tests/views/helpers";
@@ -74,11 +80,12 @@ QUnit.module("Analytic", (hooks) => {
                         amount: { string: "Amount", type: "float" },
                         analytic_distribution: { string: "Analytic", type: "json" },
                         move_id: { string: "Account Move", type: "many2one", relation: "move" },
+                        analytic_precision: { string: "Analytic Precision", type: "integer" },
                     },
                     records: [
-                        { id: 1, label: "Developer Time", amount: 100.00, analytic_distribution: {"1": 30.3, "3": 69.7}},
+                        { id: 1, label: "Developer Time", amount: 100.00, analytic_distribution: {"1, 7": 30.3, "3": 69.704}, analytic_precision: 3},
                         { id: 2, label: "Coke", amount: 100.00, analytic_distribution: {}},
-                        { id: 3, label: "Sprite", amount: 100.00, analytic_distribution: {}},
+                        { id: 3, label: "Sprite", amount: 100.00, analytic_distribution: {}, analytic_precision: 3},
                         { id: 4, label: "", amount: 100.00, analytic_distribution: {}},
                     ],
                 },
@@ -99,27 +106,19 @@ QUnit.module("Analytic", (hooks) => {
                         { id: 2, display_name: "INV0002", line_ids: [3, 4]},
                     ],
                 },
-                "decimal.precision": {
-                    fields: {
-                        name: { string: "Name", type: "char" },
-                        digits: { string: "Digits", type: "int" },
-                    },
-                    records: [
-                        { id: 1, name: "Percentage Analytic", digits: 2}
-                    ]
-                }
             },
             views: {
                 "account.analytic.account,false,search": `<search/>`,
                 "account.analytic.account,analytic.view_account_analytic_account_list_select,list": `
-                    <tree>
+                    <list>
                         <field name="name"/>
-                    </tree>
+                    </list>
                 `,
             }
         };
 
         setupViewRegistries();
+        registry.category("services").add("batchedOrm", batchedOrmService);
     });
 
     QUnit.module("AnalyticDistribution");
@@ -140,7 +139,7 @@ QUnit.module("Analytic", (hooks) => {
                         </group>
                     </sheet>
                 </form>`,
-            mockRPC(route, { kwargs, method, model }) {
+            mockRPC(route, { method, model }) {
                 if (method === "get_relevant_plans" && model === "account.analytic.plan") {
                     return Promise.resolve(
                         serverData.models['plan'].records.filter((r) => !r.parent_id && r.applicability !== "unavailable")
@@ -148,65 +147,77 @@ QUnit.module("Analytic", (hooks) => {
                 }
             },
         });
-
+        // tags
         assert.containsOnce(target, ".o_field_analytic_distribution", "widget should be visible");
         assert.containsN(target, ".badge", 2, "should contain 2 tags");
-        assert.strictEqual(target.querySelector(".badge .o_tag_badge_text").textContent, "RD 30.3%",
-            "should have rendered tag 'RD 30.3%'"
+        assert.strictEqual(target.querySelector(".badge .o_tag_badge_text").textContent, "30.3% RD | 69.7% FI",
+            "should have rendered tag '30.3% RD | 69.7% FI'"
         );
-        assert.strictEqual(target.querySelectorAll(".badge .o_tag_badge_text")[1].textContent, "FI 69.7%",
-            "should have rendered tag 'FI 69.7%'"
+        assert.strictEqual(target.querySelectorAll(".badge .o_tag_badge_text")[1].textContent, "30.3% Belgium",
+            "should have rendered tag '30.3% Belgium'"
         );
 
-        assert.containsN(target, ".o_delete", 2, "tags should contain a delete button");
-
-        let badge1 = target.querySelector('.badge');
-        await click(badge1, ".o_tag_badge_text");
+        // open popup
+        let field = target.querySelector('.o_field_analytic_distribution');
+        await click(field, ".o_input_dropdown");
         assert.containsN(target, ".analytic_distribution_popup", 1, "popup should be visible");
-
+        
         let popup = target.querySelector('.analytic_distribution_popup');
         let planTable = popup.querySelectorAll('table')[0];
 
-        assert.strictEqual(planTable.id, "plan_2", "mandatory plan appears first");
+        // contents of popup
         assert.containsN(planTable, 'tr', 4,
-            "first plan contains 4 rows including: plan title, 2 tags, empty tag"
-        );
-        assert.strictEqual(document.activeElement, planTable.querySelector('input'),
-            "focus is on the first analytic account"
+        "table contains 4 rows including: header row, 2 lines, add a line"
         );
 
-        triggerHotkey("Tab");
-        const input = document.activeElement;
-        await editInput(input, null, "19");
+        assert.strictEqual(document.activeElement, planTable.querySelector('tr:first-of-type #x_plan1_id'),
+            "focus is on the first analytic account input of the first row"
+        );
 
-        assert.containsOnce(planTable, '.o_analytic_status_invalid', "Mandatory plan has invalid status");
+        // change percentage
+        await click(planTable, "tr:first-of-type .o_field_percentage input");
+        let input = document.activeElement;
+        await editInput(input, null, "19.7001");
+        
+        // mandatory plan is red
+        assert.containsOnce(planTable, 'th:contains("Departments") .text-danger:contains("50%")', "Mandatory plan has invalid status");
 
+        // close and open popup with keyboard
         triggerHotkey("Escape");
         await nextTick();
         assert.containsNone(target, '.analytic_distribution_popup', "The popup should be closed");
 
-        triggerHotkey("arrowdown"); //opens the popup again
+        triggerHotkey("arrowdown");
         await nextTick();
 
+        // add a line
         popup = target.querySelector('.analytic_distribution_popup');
         planTable = popup.querySelectorAll('table')[0];
-        let incompleteInputName = planTable.querySelector('tr.incomplete .o_analytic_account_name input');
-        assert.strictEqual(document.activeElement, incompleteInputName,
-            "focus returns to the first incomplete tag"
-        );
+        await click(planTable, '.o_field_x2many_list_row_add a');
+        const newRow = planTable.querySelector('tr:nth-of-type(3)');
+        assert.equal(newRow.querySelector('.o_field_percentage input').value, 50);
 
+        // choose an account for the mandatory plan using the keyboard
+        await click(newRow, "#x_plan2_id");
+        input = document.activeElement;
+        await triggerEvent(input, null, "keydown", { key: "enter" });
+        await triggerEvent(input, null, "keyup", { key: "enter" });
         triggerHotkey("arrowdown");
         await nextTick();
 
         triggerHotkey("Tab");
         await nextTick();
 
-        assert.strictEqual(document.activeElement.value, "11.3%", "remainder percentage is prepopulated");
+        // mandatory plan is green
+        assert.containsOnce(planTable, 'th:contains("Departments") .text-success:contains("100%")', "Mandatory plan in complete");
 
+        // tags
         await click(target, '.fa-close');
         assert.containsNone(target, '.analytic_distribution_popup', "The popup should be closed");
-        assert.containsNone(target, '.o_field_invalid', "Distribution is valid");
-        assert.containsN(target, ".badge", 3, "should contain 3 tags");
+        assert.containsN(target, ".badge", 2, "should contain 2 tags");
+        assert.strictEqual(target.querySelector(".badge .o_tag_badge_text").textContent, "30.3% RD | 50% HR | 19.7% FI",
+            "should have rendered tag '30.3% RD | 50% HR | 19.7% FI'"
+        );
     });
 
     QUnit.test("analytic field in multi_edit list view + search more", async function (assert) {
@@ -215,14 +226,14 @@ QUnit.module("Analytic", (hooks) => {
             resModel: "aml",
             serverData,
             arch: `
-                <tree multi_edit="1">
+                <list multi_edit="1">
                     <field name="label"/>
                     <field name="analytic_distribution" widget="analytic_distribution" options="{'force_applicability': 'optional'}"/>
                     <field name="amount"/>
-                </tree>`,
+                </list>`,
             mockRPC(route, { kwargs, method, model }) {
                 if (method === "get_relevant_plans" && model === "account.analytic.plan") {
-                    assert.equal(kwargs.applicability, "optional")
+                    assert.equal(kwargs.applicability, "optional");
                     return Promise.resolve(
                         serverData.models['plan'].records
                             .filter((r) => !r.parent_id && r.applicability !== "unavailable")
@@ -240,101 +251,152 @@ QUnit.module("Analytic", (hooks) => {
         const amlrows = target.querySelectorAll(".o_data_row");
         await click(amlrows[0].querySelector(".o_list_record_selector input"));
         await click(amlrows[1].querySelector(".o_list_record_selector input"));
+        await nextTick();
         await click(badge1, ".o_tag_badge_text");
         await nextTick();
         assert.containsN(target, ".analytic_distribution_popup", 1, "popup should be visible");
 
         let popup = target.querySelector('.analytic_distribution_popup');
-        assert.containsNone(popup, 'th span', "All plans are optional with no status indicator");
-
-        let incompleteCountryTag = popup.querySelector("table#plan_5 .incomplete .o_analytic_account_name input");
-        await click(incompleteCountryTag);
-        await click(target.querySelector(".o_m2o_dropdown_option_search_more"));
+        assert.containsNone(popup, '.text-success', "All plans are optional");
 
+        // add a line
+        let planTable = popup.querySelectorAll('table')[0];
+        await addRow(planTable)
+        await selectDropdownItem(planTable.querySelector("tr[name='line_2']"), "x_plan5_id", "Search More...");
         assert.containsN(target, ".modal-dialog .o_list_renderer", 1, "select create list dialog is visible");
+        
+        await click(target, ".modal-dialog .modal-title");
+        await click(target, ".modal-dialog .o_data_row:nth-of-type(4) .o_data_cell:first-of-type");
+        assert.containsNone(target, ".modal-dialog .o_list_renderer", "select create list dialog is closed");
 
-        // select 2 analytic accounts
-        let accountRows = [...target.querySelectorAll(".modal-dialog .o_data_row")];
-        for (const row of accountRows.slice(0,2)) {
-            await click(row.querySelector(".o_list_record_selector input"));
-        }
-        await click(target.querySelector(".o_select_button"));
-
-        let percentageEls = [...popup.querySelectorAll("table#plan_5 .o_analytic_percentage input")];
-        let expectedPercentages = ['100%', '0%', '0%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `1: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-        // modify the percentage of tag 1, tag 2 is filled
-        await editInput(percentageEls[0], null, "50");
-        expectedPercentages = ['50%', '50%', '0%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `2: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-        // modify the percentage of tag 1, last empty tag (tag 3) is filled
-        await editInput(percentageEls[0], null, "40");
-        expectedPercentages = ['40%', '50%', '10%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `3: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-
-        // replace the first analytic account with 4 accounts
-        triggerHotkey("shift+Tab");
-        await click(document.activeElement);
-        await click(target.querySelector(".o_m2o_dropdown_option_search_more"));
-        accountRows = [...target.querySelectorAll(".modal-dialog .o_data_row")];
-        for (const row of accountRows.slice(0,4)) {
-            await click(row.querySelector(".o_list_record_selector input"));
-        }
-        await click(target.querySelector(".o_select_button"));
-
-        percentageEls = [...popup.querySelectorAll("table#plan_5 .o_analytic_percentage input")];
-        expectedPercentages = ['40%', '50%', '10%', '0%', '0%', '0%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `4: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-
-        // modify percentage of the tag 1 (focused), balance goes to the first zero (tag 4)
-        await editInput(document.activeElement, null, "10");
-        expectedPercentages = ['10%', '50%', '10%', '30%', '0%', '0%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `5: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-
-        // modify percentage of tag 4, balance goes to the first zero (tag 5)
-        await editInput(percentageEls[3], null, "20");
-        expectedPercentages = ['10%', '50%', '10%', '20%', '10%', '0%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `6: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-
-        // change tag 4 to 0%, tag is removed and balance goes to last tag (tag 5)
-        let accountEls = [...popup.querySelectorAll("table#plan_5 .o_analytic_account_name input")];
-        await editInput(percentageEls[3], null, "0");
-        percentageEls = [...popup.querySelectorAll("table#plan_5 .o_analytic_percentage input")];
-        expectedPercentages = ['10%', '50%', '10%', '10%', '20%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `7: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-        assert.strictEqual(document.activeElement, accountEls[4], "Focus should be on the fifth tag (one was removed)");
-
-        // delete tag 3, balance goes to last empty tag (tag 4)
-        let trashIcons = [...document.querySelectorAll("table#plan_5 .fa-trash-o")];
-        assert.equal(trashIcons.length, 4, "1 tag should not have a trash icon");
-        await click(trashIcons[2]);
-        percentageEls = [...popup.querySelectorAll("table#plan_5 .o_analytic_percentage input")];
-        expectedPercentages = ['10%', '50%', '10%', '30%'];
-        for (const [i, el] of percentageEls.entries()) {
-            assert.equal(el.value, expectedPercentages[i], `7: Percentage Element ${i} should be ${expectedPercentages[i]}`);
-        }
-        assert.equal(popup.querySelector("table#plan_5 tr:last-of-type .o_analytic_account_name input").value, "",
-            "Last tag's account is empty");
-
-        // apply the changes to both move lines
-        triggerHotkey("Escape");
-        await nextTick();
+        await click(popup, ".fa-close");
         await click(target.querySelector(".modal-dialog .btn-primary"));
-        assert.containsN(target, ".badge", 10, "should contain 2 rows of 5 tags each");
-
+        await nextTick();
+        assert.containsN(target, ".o_data_row .badge", 4, "should contain 2 rows of 2 tags each");
+        assert.strictEqual(target.querySelector("tr:nth-of-type(2) .badge:nth-of-type(2) .o_tag_badge_text").textContent, "30.3% Belgium | 69.7% Berlin",
+            "should have rendered tag '30.3% Belgium | 69.7% Berlin'"
+        );
     });
+
+    QUnit.test("Rounding, value suggestions, keyboard only", async (assert) => {
+        await makeView({
+            type: "form",
+            resModel: "move",
+            serverData,
+            resId: 2,
+            arch: `
+                <form>
+                    <sheet>
+                        <field name="line_ids">
+                            <list editable="bottom">
+                                <field name="label"/>
+                                <field name="analytic_distribution" widget="analytic_distribution"/>
+                                <field name="amount"/>
+                            </list>
+                        </field>
+                    </sheet>
+                </form>`,
+            mockRPC(route, { kwargs, method, model }) {
+                if (method === "get_relevant_plans" && model === "account.analytic.plan") {
+                    return Promise.resolve(
+                        serverData.models['plan'].records.filter((r) => !r.parent_id && r.applicability !== "unavailable" && r.all_account_count)
+                    );
+                }
+            },
+        });
+        await click(target, ".o_data_row:nth-of-type(1) .o_list_char");
+        triggerHotkey("Tab");
+        await nextTick();
+        assert.containsN(target, ".analytic_distribution_popup", 1, "popup should be visible");
+
+        // department
+        triggerHotkey("Tab");
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose the RD account
+        await nextTick();
+        triggerHotkey("Tab");  // tab to country
+        triggerHotkey("Tab");  // tab to percentage
+        await editInput(document.activeElement, false, "99.9");
+
+        // internal
+        triggerHotkey("Tab");  // new line
+        await nextTick();
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose the Time off account
+        await nextTick();
+        triggerHotkey("Tab");  // tab to departments
+        triggerHotkey("Tab");  // tab to country
+        triggerHotkey("Tab");  // tab to percentage
+        await editInput(document.activeElement, false, "99.99");
+
+        // country
+        triggerHotkey("Tab");  // new line
+        await nextTick();
+        triggerHotkey("Tab");  // tab to departments
+        triggerHotkey("Tab");  // tab to country
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose the Belgium account
+        await nextTick();
+        triggerHotkey("Tab");  // tab to percentage
+        await editInput(document.activeElement, false, "99.999");
+
+        // tags
+        assert.containsOnce(target, ".badge:contains('99.9% RD')", "contains RD tag");
+        assert.containsOnce(target, ".badge:contains('99.99% Time Off')", "contains Time Off tag");
+        assert.containsOnce(target, ".badge:contains('100% Belgium')", "contains Belgium tag always rounded to 2 decimals");
+
+        // fill department
+        triggerHotkey("Tab");  // new line
+        await nextTick();
+        triggerHotkey("Tab");  // tab to departments
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose the HR account
+        await nextTick();
+        triggerHotkey("Tab");
+        triggerHotkey("Tab");
+        assert.equal(document.activeElement.value, 0.1, "Mandatory plan filled first with 0.1");
+        await editInput(document.activeElement, false, "0.0996");
+        assert.containsOnce(target, ".badge:contains('99.9% RD | 0.1% HR')", "contains RD|HR tag");
+        assert.containsOnce(target, ".text-success:contains('100%')", "Department header");
+
+        // fill country
+        triggerHotkey("Tab");  // new line
+        await nextTick();
+        triggerHotkey("Tab");  // tab to departments
+        triggerHotkey("Tab");  // tab to country
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose Belgium again
+        await nextTick();
+        triggerHotkey("Tab");  // tab to percentage
+        await nextTick();
+        assert.equal(document.activeElement.value, 0.001, "Country plan filled next with 0.001");
+        await editInput(document.activeElement, false, "0.0006");
+        assert.containsOnce(target, ".badge:contains('Belgium'):not(:contains('%'))", "contains Belgium tag");
+
+        // fill internal
+        triggerHotkey("Tab");  // new line
+        await nextTick();
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("arrowdown");
+        await nextTick();
+        triggerHotkey("Enter");  // choose Operating Costs
+        await nextTick();
+        triggerHotkey("Escape");  // close the popup
+        await nextTick();
+        assert.containsOnce(target, ".badge:contains('99.99% Time Off | 0.01% Operating Costs')", "the Time Off | Operating Costs tag");
+    });
+
+    QUnit.test("Analytic Account Deleted after json is saved", (assert) => { assert.expect(0) });
+    QUnit.test("No Plans Available", (assert) => { assert.expect(0) });
+    QUnit.test("amount_field Column", (assert) => { assert.expect(0) });
+    QUnit.test("save as model", (assert) => { assert.expect(0) });
+
 });
