@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from odoo.tests.common import TransactionCase, tagged
+from odoo.tools.misc import file_open
 from unittest import skipIf
 import os
 
@@ -17,7 +18,7 @@ class TestCaseIndexation(TransactionCase):
 
     @skipIf(PDFResourceManager is None, "pdfminer not installed")
     def test_attachment_pdf_indexation(self):
-        with open(os.path.join(directory, 'files', 'test_content.pdf'), 'rb') as file:
+        with file_open(os.path.join(directory, 'files', 'test_content.pdf'), 'rb') as file:
             pdf = file.read()
             text = self.env['ir.attachment']._index(pdf, 'application/pdf')
             self.assertEqual(text, 'TestContent!!\x0c', 'the index content should be correct')
