@@ -123,51 +123,54 @@ class OAuthController(http.Controller):
     @fragment_to_query_string
     def signin(self, **kw):
         state = json.loads(kw['state'])
+
+        # make sure request.session.db and state['d'] are the same,
+        # update the session and retry the request otherwise
         dbname = state['d']
         if not http.db_filter([dbname]):
             return BadRequest()
+        ensure_db(db=dbname)
+
         provider = state['p']
-        context = clean_context(state.get('c', {}))
-        registry = registry_get(dbname)
-        with registry.cursor() as cr:
-            try:
-                env = api.Environment(cr, SUPERUSER_ID, context)
-                db, login, key = env['res.users'].sudo().auth_oauth(provider, kw)
-                cr.commit()
-                action = state.get('a')
-                menu = state.get('m')
-                redirect = werkzeug.urls.url_unquote_plus(state['r']) if state.get('r') else False
-                url = '/web'
-                if redirect:
-                    url = redirect
-                elif action:
-                    url = '/web#action=%s' % action
-                elif menu:
-                    url = '/web#menu_id=%s' % menu
+        request.update_context(**clean_context(state.get('c', {})))
+        try:
+            # auth_oauth may create a new user, the commit makes it
+            # visible to authenticate()'s own transaction below
+            _, login, key = request.env['res.users'].with_user(SUPERUSER_ID).auth_oauth(provider, kw)
+            request.env.cr.commit()
 
-                pre_uid = request.session.authenticate(db, login, key)
-                resp = request.redirect(_get_login_redirect_url(pre_uid, url), 303)
-                resp.autocorrect_location_header = False
+            action = state.get('a')
+            menu = state.get('m')
+            redirect = werkzeug.urls.url_unquote_plus(state['r']) if state.get('r') else False
+            url = '/odoo'
+            if redirect:
+                url = redirect
+            elif action:
+                url = '/odoo/action-%s' % action
+            elif menu:
+                url = '/odoo?menu_id=%s' % menu
 
-                # Since /web is hardcoded, verify user has right to land on it
-                if werkzeug.urls.url_parse(resp.location).path == '/web' and not request.env.user._is_internal():
-                    resp.location = '/'
-                return resp
-            except AttributeError:
-                # auth_signup is not installed
-                _logger.error("auth_signup not installed on database %s: oauth sign up cancelled." % (dbname,))
-                url = "/web/login?oauth_error=1"
-            except AccessDenied:
-                # oauth credentials not valid, user could be on a temporary session
-                _logger.info('OAuth2: access denied, redirect to main page in case a valid session exists, without setting cookies')
-                url = "/web/login?oauth_error=3"
-                redirect = request.redirect(url, 303)
-                redirect.autocorrect_location_header = False
-                return redirect
-            except Exception as e:
-                # signup error
-                _logger.exception("OAuth2: %s" % str(e))
-                url = "/web/login?oauth_error=2"
+            credential = {'login': login, 'token': key, 'type': 'oauth_token'}
+            auth_info = request.session.authenticate(dbname, credential)
+            resp = request.redirect(_get_login_redirect_url(auth_info['uid'], url), 303)
+            resp.autocorrect_location_header = False
+
+            # Since /web is hardcoded, verify user has right to land on it
+            if werkzeug.urls.url_parse(resp.location).path == '/web' and not request.env.user._is_internal():
+                resp.location = '/'
+            return resp
+        except AttributeError:  # TODO juc master: useless since ensure_db()
+            # auth_signup is not installed
+            _logger.error("auth_signup not installed on database %s: oauth sign up cancelled.", dbname)
+            url = "/web/login?oauth_error=1"
+        except AccessDenied:
+            # oauth credentials not valid, user could be on a temporary session
+            _logger.info('OAuth2: access denied, redirect to main page in case a valid session exists, without setting cookies')
+            url = "/web/login?oauth_error=3"
+        except Exception:
+            # signup error
+            _logger.exception("Exception during request handling")
+            url = "/web/login?oauth_error=2"
 
         redirect = request.redirect(url, 303)
         redirect.autocorrect_location_header = False
