@@ -1,6 +1,6 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+import contextlib
 import logging
 
 from ast import literal_eval
@@ -10,10 +10,10 @@ from dateutil.relativedelta import relativedelta
 from odoo import api, fields, models, _
 from odoo.exceptions import UserError
 from odoo.osv import expression
-from odoo.tools.misc import ustr
+from odoo.http import request
 
 from odoo.addons.base.models.ir_mail_server import MailDeliveryException
-from odoo.addons.auth_signup.models.res_partner import SignupError, now
+from odoo.addons.auth_signup.models.res_partner import SignupError
 
 _logger = logging.getLogger(__name__)
 
@@ -66,8 +66,7 @@ class ResUsers(models.Model):
             # signup with a token: find the corresponding partner id
             partner = self.env['res.partner']._signup_retrieve_partner(token, check_validity=True, raise_exception=True)
             # invalidate signup token
-            partner.write({'signup_token': False, 'signup_type': False, 'signup_expiration': False})
-
+            partner.write({'signup_type': False})
             partner_user = partner.user_ids and partner.user_ids[0] or False
 
             # avoid overwriting existing (presumably correct) values with geolocation data
@@ -118,15 +117,24 @@ class ResUsers(models.Model):
                 raise SignupError(_('Signup is not allowed for uninvited users'))
         return self._create_user_from_template(values)
 
+    @classmethod
+    def authenticate(cls, db, credential, user_agent_env):
+        auth_info = super().authenticate(db, credential, user_agent_env)
+        try:
+            with cls.pool.cursor() as cr:
+                env = api.Environment(cr, auth_info['uid'], {})
+                if env.user._should_alert_new_device():
+                    env.user._alert_new_device()
+        except MailDeliveryException:
+            pass
+        return auth_info
+
     def _notify_inviter(self):
         for user in self:
-            invite_partner = user.create_uid.partner_id
-            if invite_partner:
-                # notify invite user that new user is connected
-                self.env['bus.bus']._sendone(invite_partner, 'res.users/connection', {
-                    'username': user.name,
-                    'partnerId': user.partner_id.id,
-                })
+            # notify invite user that new user is connected
+            user.create_uid._bus_send(
+                "res.users/connection", {"username": user.name, "partnerId": user.partner_id.id}
+            )
 
     def _create_user_from_template(self, values):
         template_user_id = literal_eval(self.env['ir.config_parameter'].sudo().get_param('base.template_portal_user_id', 'False'))
@@ -146,43 +154,51 @@ class ResUsers(models.Model):
                 return template_user.with_context(no_reset_password=True).copy(values)
         except Exception as e:
             # copy may failed if asked login is not available.
-            raise SignupError(ustr(e))
+            raise SignupError(str(e))
 
     def reset_password(self, login):
         """ retrieve the user corresponding to login (login or email),
             and reset their password
         """
-        users = self.search([('login', '=', login)])
+        users = self.search(self._get_login_domain(login))
+        if not users:
+            users = self.search(self._get_email_domain(login))
         if not users:
-            users = self.search([('email', '=', login)])
-        if len(users) != 1:
             raise Exception(_('No account found for this login'))
+        if len(users) > 1:
+            raise Exception(_('Multiple accounts found for this login'))
         return users.action_reset_password()
 
     def action_reset_password(self):
+        try:
+            if self.env.context.get('create_user') == 1:
+                return self._action_reset_password(signup_type="signup")
+            else:
+                return self._action_reset_password(signup_type="reset")
+        except MailDeliveryException as mde:
+            if len(mde.args) == 2 and isinstance(mde.args[1], ConnectionRefusedError):
+                raise UserError(_("Could not contact the mail server, please check your outgoing email server configuration")) from mde
+            else:
+                raise UserError(_("There was an error when trying to deliver your Email, please check your configuration")) from mde
+
+    def _action_reset_password(self, signup_type="reset"):
         """ create signup token for each user, and send their signup url by email """
-        if self.env.context.get('install_mode', False):
+        if self.env.context.get('install_mode') or self.env.context.get('import_file'):
             return
         if self.filtered(lambda user: not user.active):
             raise UserError(_("You cannot perform this action on an archived user."))
         # prepare reset password signup
         create_mode = bool(self.env.context.get('create_user'))
 
-        # no time limit for initial invitation, only for reset password
-        expiration = False if create_mode else now(days=+1)
-
-        self.mapped('partner_id').signup_prepare(signup_type="reset", expiration=expiration)
+        self.mapped('partner_id').signup_prepare(signup_type=signup_type)
 
         # send email to users with their signup url
-        template = False
+        account_created_template = None
         if create_mode:
-            try:
-                template = self.env.ref('auth_signup.set_password_email', raise_if_not_found=False)
-            except ValueError:
-                pass
-        if not template:
-            template = self.env.ref('auth_signup.reset_password_email')
-        assert template._name == 'mail.template'
+            account_created_template = self.env.ref('auth_signup.set_password_email', raise_if_not_found=False)
+            if account_created_template and account_created_template._name != 'mail.template':
+                _logger.error("Wrong set password template %r", account_created_template)
+                return
 
         email_values = {
             'email_cc': False,
@@ -197,22 +213,55 @@ class ResUsers(models.Model):
             if not user.email:
                 raise UserError(_("Cannot send email: user %s has no email address.", user.name))
             email_values['email_to'] = user.email
-            # TDE FIXME: make this template technical (qweb)
-            with self.env.cr.savepoint():
-                force_send = not(self.env.context.get('import_file', False))
-                template.send_mail(user.id, force_send=force_send, raise_exception=True, email_values=email_values)
-            _logger.info("Password reset email sent for user <%s> to <%s>", user.login, user.email)
+            with contextlib.closing(self.env.cr.savepoint()):
+                if account_created_template:
+                    account_created_template.send_mail(
+                        user.id, force_send=True,
+                        raise_exception=True, email_values=email_values)
+                else:
+                    user_lang = user.lang or self.env.lang or 'en_US'
+                    body = self.env['mail.render.mixin'].with_context(lang=user_lang)._render_template(
+                        self.env.ref('auth_signup.reset_password_email'),
+                        model='res.users', res_ids=user.ids,
+                        engine='qweb_view', options={'post_process': True})[user.id]
+                    mail = self.env['mail.mail'].sudo().create({
+                        'subject': self.with_context(lang=user_lang).env._('Password reset'),
+                        'email_from': user.company_id.email_formatted or user.email_formatted,
+                        'body_html': body,
+                        **email_values,
+                    })
+                    mail.send()
+            if signup_type == 'reset':
+                _logger.info("Password reset email sent for user <%s> to <%s>", user.login, user.email)
+                message = _('A reset password link was send by email')
+            else:
+                _logger.info("Signup email sent for user <%s> to <%s>", user.login, user.email)
+                message = _('A signup link was send by email')
+        return {
+            'type': 'ir.actions.client',
+            'tag': 'display_notification',
+            'params': {
+                'title': 'Notification',
+                'message': message,
+                'sticky': False
+            }
+        }
 
-    def send_unregistered_user_reminder(self, after_days=5):
+    def send_unregistered_user_reminder(self, after_days=5, batch_size=100):
+        email_template = self.env.ref('auth_signup.mail_template_data_unregistered_users', raise_if_not_found=False)
+        if not email_template:
+            _logger.warning("Template 'auth_signup.mail_template_data_unregistered_users' was not found. Cannot send reminder notifications.")
+            return
         datetime_min = fields.Datetime.today() - relativedelta(days=after_days)
         datetime_max = datetime_min + relativedelta(hours=23, minutes=59, seconds=59)
 
-        res_users_with_details = self.env['res.users'].search_read([
-            ('share', '=', False),
+        domain = [('share', '=', False),
             ('create_uid.email', '!=', False),
             ('create_date', '>=', datetime_min),
             ('create_date', '<=', datetime_max),
-            ('log_ids', '=', False)], ['create_uid', 'name', 'login'])
+            ('log_ids', '=', False)]
+
+        res_users_with_details = self.env['res.users'].search_read(domain, ['create_uid', 'name', 'login'], limit=batch_size)
 
         # group by invited by
         invited_users = defaultdict(list)
@@ -221,9 +270,74 @@ class ResUsers(models.Model):
 
         # For sending mail to all the invitors about their invited users
         for user in invited_users:
-            template = self.env.ref('auth_signup.mail_template_data_unregistered_users').with_context(dbname=self._cr.dbname, invited_users=invited_users[user])
+            template = email_template.with_context(dbname=self._cr.dbname, invited_users=invited_users[user])
             template.send_mail(user, email_layout_xmlid='mail.mail_notification_light', force_send=False)
 
+        done = len(res_users_with_details)
+        self.env['ir.cron']._notify_progress(
+            done=done,
+            remaining=0 if done < batch_size else self.env['res.users'].search_count(domain)
+        )
+
+    def _alert_new_device(self):
+        self.ensure_one()
+        if self.email:
+            email_values = {
+                'email_cc': False,
+                'auto_delete': True,
+                'message_type': 'user_notification',
+                'recipient_ids': [],
+                'partner_ids': [],
+                'scheduled_date': False,
+                'email_to': self.email
+            }
+
+            body = self.env['mail.render.mixin']._render_template(
+                    'auth_signup.alert_login_new_device',
+                    model='res.users', res_ids=self.ids,
+                    engine='qweb_view', options={'post_process': True},
+                    add_context=self._prepare_new_device_notice_values())[self.id]
+            mail = self.env['mail.mail'].sudo().create({
+                'subject': _('New Connection to your Account'),
+                'email_from': self.company_id.email_formatted or self.email_formatted,
+                'body_html': body,
+                **email_values,
+            })
+            mail.send()
+            _logger.info("New device alert email sent for user <%s> to <%s>", self.login, self.email)
+
+    def _prepare_new_device_notice_values(self):
+        values = {
+            'login_date': fields.Datetime.now(),
+            'location_address': False,
+            'ip_address': False,
+            'browser': False,
+            'useros': False,
+        }
+
+        if not request:
+            return values
+
+        city = request.geoip.get('city') or False
+        region = request.geoip.get('region_name') or False
+        country = request.geoip.get('country') or False
+        if country:
+            if region and city:
+                values['location_address'] = _("Near %(city)s, %(region)s, %(country)s", city=city, region=region, country=country)
+            elif region:
+                values['location_address'] = _("Near %(region)s, %(country)s", region=region, country=country)
+            else:
+                values['location_address'] = _("In %(country)s", country=country)
+        else:
+            values['location_address'] = False
+        values['ip_address'] = request.httprequest.environ['REMOTE_ADDR']
+        if request.httprequest.user_agent:
+            if request.httprequest.user_agent.browser:
+                values['browser'] = request.httprequest.user_agent.browser.capitalize()
+            if request.httprequest.user_agent.platform:
+                values['useros'] = request.httprequest.user_agent.platform.capitalize()
+        return values
+
     @api.model
     def web_create_users(self, emails):
         inactive_users = self.search([('state', '=', 'new'), '|', ('login', 'in', emails), ('email', 'in', emails)])
@@ -241,16 +355,14 @@ class ResUsers(models.Model):
             users_with_email = users.filtered('email')
             if users_with_email:
                 try:
-                    users_with_email.with_context(create_user=True).action_reset_password()
+                    users_with_email.with_context(create_user=True)._action_reset_password(signup_type='signup')
                 except MailDeliveryException:
                     users_with_email.partner_id.with_context(create_user=True).signup_cancel()
         return users
 
-    @api.returns('self', lambda value: value.id)
     def copy(self, default=None):
         self.ensure_one()
-        sup = super(ResUsers, self)
         if not default or not default.get('email'):
             # avoid sending email to the user we are duplicating
-            sup = super(ResUsers, self.with_context(no_reset_password=True))
-        return sup.copy(default=default)
+            self = self.with_context(no_reset_password=True)
+        return super().copy(default=default)
