@@ -1,7 +1,6 @@
-odoo.define('auth_signup.signup', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var publicWidget = require('web.public.widget');
+import publicWidget from "@web/legacy/js/public/public_widget";
 
 publicWidget.registry.SignUpForm = publicWidget.Widget.extend({
     selector: '.oe_signup_form',
@@ -22,4 +21,3 @@ publicWidget.registry.SignUpForm = publicWidget.Widget.extend({
         $btn.prepend('<i class="fa fa-refresh fa-spin"/> ');
     },
 });
-});
