@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
-from odoo import api, models
-from odoo.addons.auth_totp.controllers.home import TRUSTED_DEVICE_AGE
+from odoo import models
 
 import logging
 _logger = logging.getLogger(__name__)
@@ -21,11 +20,3 @@ class AuthTotpDevice(models.Model):
         """Return True if device key matches given `scope` for user ID `uid`"""
         assert uid, "uid is required"
         return self._check_credentials(scope=scope, key=key) == uid
-
-    @api.autovacuum
-    def _gc_device(self):
-        self._cr.execute("""
-            DELETE FROM auth_totp_device
-            WHERE create_date < (NOW() AT TIME ZONE 'UTC' - INTERVAL '%s SECONDS')
-        """, [TRUSTED_DEVICE_AGE])
-        _logger.info("GC'd %d totp devices entries", self._cr.rowcount)
