@@ -1,27 +1,27 @@
-odoo.define('auth_totp.tours', function(require) {
-"use strict";
+/** @odoo-module **/
 
-const tour = require('web_tour.tour');
-const ajax = require('web.ajax');
+import { queryAll } from "@odoo/hoot-dom";
+import { rpc } from "@web/core/network/rpc";
+import { registry } from "@web/core/registry";
+import { stepUtils } from "@web_tour/tour_service/tour_utils";
 
 function openRoot() {
     return [{
         content: "return to client root to avoid race condition",
         trigger: 'body',
         run() {
-            $('body').addClass('wait');
-            window.location = '/web';
+            document.querySelector("body").classList.add("wait");
+            window.location = '/odoo';
         }
     }, {
         content: "wait for client reload",
         trigger: 'body:not(.wait)',
-        run() {}
     }];
 }
 function openUserProfileAtSecurityTab() {
     return [{
         content: 'Open user account menu',
-        trigger: '.o_user_menu .oe_topbar_name',
+        trigger: '.o_user_menu .dropdown-toggle',
         run: 'click',
     }, {
         content: "Open preferences / profile screen",
@@ -53,62 +53,74 @@ function closeProfileDialog({content, totp_state}) {
     return [{
         content,
         trigger,
-        run() {
-            const $modal = this.$anchor.parents('.o_dialog_container');
-            if ($modal.length) {
-                $modal.find('button[name=preference_cancel]').click()
+        run(helpers) {
+            const modal = document.querySelector(".o_dialog");
+            if (modal) {
+                modal.querySelector("button[name=preference_cancel]").click();
             }
         }
     }, {
         trigger: 'body',
         async run() {
-            while (document.querySelector('.o_dialog_container .o_dialog')) {
+            while (document.querySelector('.o_dialog')) {
                 await Promise.resolve();
             }
-            this.$anchor.addClass('dialog-closed');
+            this.anchor.classList.add("dialog-closed");
         },
     }, {
         trigger: 'body.dialog-closed',
-        run() {},
     }];
 }
 
-tour.register('totp_tour_setup', {
-    test: true,
-    url: '/web'
-}, [...openUserProfileAtSecurityTab(), {
+registry.category("web_tour.tours").add('totp_tour_setup', {
+    url: '/odoo',
+    steps: () => [...openUserProfileAtSecurityTab(), {
     content: "Open totp wizard",
     trigger: 'button[name=action_totp_enable_wizard]',
-}, {
+    run: "click",
+},
+{
+    trigger: ".modal div:contains(enter your password)",
+},
+{
     content: "Check that we have to enter enhanced security mode and input password",
-    extra_trigger: 'div:contains("enter your password")',
     trigger: '[name=password] input',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: "Confirm",
     trigger: "button:contains(Confirm Password)",
+    run: "click",
 }, {
     content: "Check the wizard has opened",
-    trigger: 'li:contains("When requested to do so")',
-    run() {}
+    trigger: '.modal li:contains("When requested to do so")',
 }, {
     content: "Get secret from collapsed div",
-    trigger: 'a:contains("Cannot scan it?")',
+    trigger: `.modal a:contains("Cannot scan it?")`,
     async run(helpers) {
-        const $secret = this.$anchor.closest('div').find('[name=secret] span:first-child');
-        const $copyBtn = $secret.find('button');
-        $copyBtn.remove();
-        const token = await ajax.jsonRpc('/totphook', 'call', {
-            secret: $secret.text()
+        const secret = this.anchor
+            .closest("div")
+            .querySelector("[name=secret] span:first-child");
+        const copyBtn = secret.querySelector("button");
+        if (copyBtn) {
+            copyBtn.remove();
+        }
+        const token = await rpc('/totphook', {
+            secret: secret.textContent
         });
-        helpers.text(token, '[name=code] input');
-        helpers.click('button.btn-primary:contains(Activate)');
-        $('body').addClass('got-token')
+        await helpers.edit(token, '[name=code] input');
+        document.querySelector("body").classList.add("got-token");
     }
-}, {
+},
+{
+    trigger: ".modal button.btn-primary:contains(Activate)",
+    run: "click",
+},
+{
+    trigger: "body:not(:has(.modal))",
+},
+{
     content: 'wait for rpc',
     trigger: 'body.got-token',
-    run() {}
 },
 ...openRoot(),
 ...openUserProfileAtSecurityTab(),
@@ -116,104 +128,112 @@ tour.register('totp_tour_setup', {
     content: "Check that the button has changed",
     totp_state: true,
 }),
-]);
+]});
 
-tour.register('totp_login_enabled', {
-    test: true,
-    url: '/'
-}, [{
+registry.category("web_tour.tours").add('totp_login_enabled', {
+    url: '/',
+    steps: () => [{
     content: "check that we're on the login page or go to it",
-    trigger: 'input#login, a:contains(Sign in)'
+    trigger: 'input#login, a:contains(Sign in)',
+    run: "click",
 }, {
     content: "input login",
     trigger: 'input#login',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: 'input password',
     trigger: 'input#password',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: "click da button",
     trigger: 'button:contains("Log in")',
+    run: "click",
 }, {
     content: "expect totp screen",
     trigger: 'label:contains(Authentication Code)',
+    run: "click",
 }, {
     content: "input code",
     trigger: 'input[name=totp_token]',
     async run(helpers) {
-        // TODO: if tours are ever async-aware the click should get moved out,
-        //       but currently there's no great way to make the tour wait until
-        //       we've retrieved and set the token: `:empty()` is aboutthe text
-        //       content of the HTML element, not the JS value property. We
-        //       could set a class but that's really no better than
-        //       procedurally clicking the button after we've set the input.
-        const token = await ajax.jsonRpc('/totphook', 'call', {});
-        helpers.text(token);
-        helpers.click('button:contains("Log in")');
+        const token = await rpc('/totphook');
+        helpers.edit(token);
     }
-}, {
+},
+{
+    trigger: `button:contains("Log in")`,
+    run: "click",
+},
+{
     content: "check we're logged in",
-    trigger: ".o_user_menu .oe_topbar_name",
-    run() {}
-}]);
+    trigger: ".o_user_menu .dropdown-toggle",
+}]});
 
-tour.register('totp_login_device', {
-    test: true,
-    url: '/'
-}, [{
+registry.category("web_tour.tours").add('totp_login_device', {
+    url: '/',
+    steps: () => [{
     content: "check that we're on the login page or go to it",
-    trigger: 'input#login, a:contains(Sign in)'
+    trigger: 'input#login, a:contains(Sign in)',
+    run: "click",
 }, {
     content: "input login",
     trigger: 'input#login',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: 'input password',
     trigger: 'input#password',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: "click da button",
     trigger: 'button:contains("Log in")',
+    run: "click",
 }, {
     content: "expect totp screen",
     trigger: 'label:contains(Authentication Code)',
+    run: "click",
 }, {
     content: "check remember device box",
     trigger: 'label[for=switch-remember]',
+    run: "click",
 }, {
     content: "input code",
     trigger: 'input[name=totp_token]',
     async run(helpers) {
-        const token = await ajax.jsonRpc('/totphook', 'call', {})
-        helpers.text(token);
-        helpers.click('button:contains("Log in")');
+        const token = await rpc('/totphook')
+        helpers.edit(token);
     }
-}, {
+},
+{
+    trigger: "button:contains(Log in)",
+    run: "click",
+},
+{
     content: "check we're logged in",
-    trigger: ".o_user_menu .oe_topbar_name",
+    trigger: ".o_user_menu .dropdown-toggle",
     run: 'click',
 }, {
     content: "click the Log out button",
     trigger: '.dropdown-item[data-menu=logout]',
+    run: "click",
 }, {
     content: "check that we're back on the login page or go to it",
-    trigger: 'input#login, a:contains(Log in)'
+    trigger: 'input#login, a:contains(Log in)',
+    run: "edit Test",
 }, {
     content: "input login again",
     trigger: 'input#login',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: 'input password again',
     trigger: 'input#password',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: "click da button again",
     trigger: 'button:contains("Log in")',
+    run: "click",
 },  {
     content: "check we're logged in without 2FA",
-    trigger: ".o_user_menu .oe_topbar_name",
-    run() {}
+    trigger: ".o_user_menu .dropdown-toggle",
 },
 // now go and disable two-factor authentication would be annoying to do in a separate tour
 // because we'd need to login & totp again as HttpCase.authenticate can't
@@ -222,14 +242,22 @@ tour.register('totp_login_device', {
 {
     content: "Open totp wizard",
     trigger: 'button[name=action_totp_disable]',
-}, {
+    run: "click",
+},
+{
+    trigger: ".modal div:contains(enter your password)",
+},
+{
     content: "Check that we have to enter enhanced security mode and input password",
-    extra_trigger: 'div:contains("enter your password")',
-    trigger: '[name=password] input',
-    run: 'text demo',
+    trigger: ".modal [name=password] input",
+    run: "edit demo",
 }, {
     content: "Confirm",
-    trigger: "button:contains(Confirm Password)",
+    trigger: ".modal button:contains(Confirm Password)",
+    run: "click",
+},
+{
+    trigger: "body:not(:has(.modal))",
 },
 ...openRoot(),
 ...openUserProfileAtSecurityTab(),
@@ -237,25 +265,26 @@ tour.register('totp_login_device', {
     content: "Check that the button has changed",
     totp_state: false
 }),
-]);
+]});
 
-tour.register('totp_login_disabled', {
-    test: true,
-    url: '/'
-}, [{
+registry.category("web_tour.tours").add('totp_login_disabled', {
+    url: '/',
+    steps: () => [{
     content: "check that we're on the login page or go to it",
-    trigger: 'input#login, a:contains(Sign in)'
+    trigger: 'input#login, a:contains(Sign in)',
+    run: "click",
 }, {
     content: "input login",
     trigger: 'input#login',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: 'input password',
     trigger: 'input#password',
-    run: 'text demo',
+    run: "edit demo",
 }, {
     content: "click da button",
     trigger: 'button:contains("Log in")',
+    run: "click",
 },
 // normally we'd end the tour here as it's all we care about but there are a
 // bunch of ongoing queries from the loading of the web client which cause
@@ -264,76 +293,73 @@ tour.register('totp_login_disabled', {
 ...openUserProfileAtSecurityTab(),
 // close the dialog if that makes sense
 ...closeProfileDialog({})
-]);
+]});
 
 const columns = {};
-tour.register('totp_admin_disables', {
-    test: true,
-    url: '/web'
-}, [tour.stepUtils.showAppsMenuItem(), {
+registry.category("web_tour.tours").add('totp_admin_disables', {
+    url: '/odoo',
+    steps: () => [stepUtils.showAppsMenuItem(), {
     content: 'Go to settings',
-    trigger: '[data-menu-xmlid="base.menu_administration"]'
+    trigger: '[data-menu-xmlid="base.menu_administration"]',
+    run: "click",
 }, {
     content: 'Wait for page',
     trigger: '.o_menu_brand:contains("Settings")',
-    run() {}
 }, {
     content: "Open Users menu",
-    trigger: '[data-menu-xmlid="base.menu_users"]'
+    trigger: '[data-menu-xmlid="base.menu_users"]',
+    run: "click",
 }, {
     content: "Open Users view",
     trigger: '[data-menu-xmlid="base.menu_action_res_users"]',
-    run(helpers) {
-        // funny story: the users view we're trying to reach, sometimes we're
-        // already there, but if we re-click the next step executes before the
-        // action has the time to re-load, the one after that doesn't, and our
-        // selection get discarded by the action reloading, so here try to
-        // see if we're already on the users action through the breadcrumb and
-        // just close the menu if so
-        const $crumb = $('.breadcrumb');
-        if ($crumb.text().indexOf('Users') === -1) {
-            // on general settings page, click menu
-            helpers.click();
-        } else {
-            // else close menu
-            helpers.click($('[data-menu-xmlid="base.menu_users"]'));
-        }
-    }
+    run: "click",
 }, {
     content: "Find Demo User",
     trigger: 'td.o_data_cell:contains("demo")',
     run(helpers) {
-        const $titles = this.$anchor.closest('table').find('tr:first th');
-        for (let i=0; i<$titles.length; ++i) {
-            columns[$titles[i].getAttribute('data-name')] = i;
-        }
-        const $row = this.$anchor.closest('tr');
-        const sel = $row.find('.o_list_record_selector input[type=checkbox]');
+        const titles = queryAll("tr:first th", { root: this.anchor.closest("table") });
+        titles.forEach((el, i) => {
+            columns[el.getAttribute('data-name')] = i;
+        })
+        const row = this.anchor.closest('tr');
+        const sel = row.querySelector('.o_list_record_selector input[type=checkbox]');
         helpers.click(sel);
     }
 }, {
     content: "Open Actions menu",
-    trigger: 'button.dropdown-toggle:contains("Action")'
+    trigger: 'button.dropdown-toggle:contains("Action")',
+    run: "click",
 }, {
     content: "Select totp remover",
-    trigger: 'span.dropdown-item:contains(Disable two-factor authentication)'
-}, { // enhanced security yo
+    trigger: 'span.dropdown-item:contains(Disable two-factor authentication)',
+    run: "click",
+},
+{
+    trigger: ".modal div:contains(enter your password)",
+},
+{ // enhanced security yo
     content: "Check that we have to enter enhanced security mode & input password",
-    extra_trigger: 'div:contains("enter your password")',
-    trigger: '[name=password] input',
-    run: 'text admin',
+    trigger: '.modal [name=password] input',
+    run: "edit admin",
 }, {
     content: "Confirm",
-    trigger: "button:contains(Confirm Password)",
-}, {
+    trigger: ".modal button:contains(Confirm Password)",
+    run: "click",
+},
+{
+    content: "Wait the modal is closed",
+    trigger: "body:not(:has(.modal))",
+},
+{
     content: "open the user's form",
     trigger: "td.o_data_cell:contains(demo)",
+    run: "click",
 }, {
     content: "go to Account security Tab",
     trigger: "a.nav-link:contains(Account Security)",
+    run: "click",
 }, ...closeProfileDialog({
     content: "check that demo user has been de-totp'd",
     totp_state: false,
 }),
-])
-});
+]})
