@@ -1,52 +1,33 @@
-odoo.define('barcodes.BarcodeParser', function (require) {
-"use strict";
+/** @odoo-module **/
 
-var Class = require('web.Class');
-var rpc = require('web.rpc');
+export class BarcodeParser {
+    static barcodeNomenclatureFields = ["name", "rule_ids", "upc_ean_conv"];
+    static barcodeRuleFields = ["name", "sequence", "type", "encoding", "pattern", "alias"];
+    static async fetchNomenclature(orm, id) {
+        const [nomenclature] = await orm.read(
+            "barcode.nomenclature",
+            [id],
+            this.barcodeNomenclatureFields
+        );
+        let rules = await orm.searchRead(
+            "barcode.rule",
+            [["barcode_nomenclature_id", "=", id]],
+            this.barcodeRuleFields
+        );
+        rules = rules.sort((a, b) => {
+            return a.sequence - b.sequence;
+        });
+        nomenclature.rules = rules;
+        return nomenclature;
+    }
 
-// The BarcodeParser is used to detect what is the category
-// of a barcode (product, partner, ...) and extract an encoded value
-// (like weight, price, etc.)
-var BarcodeParser = Class.extend({
-    init: function(attributes) {
-        this.nomenclature_id = attributes.nomenclature_id;
-        this.nomenclature = attributes.nomenclature;
-        this.loaded = this.load();
-    },
+    constructor() {
+        this.setup(...arguments);
+    }
 
-    // This loads the barcode nomenclature and barcode rules which are
-    // necessary to parse the barcodes. The BarcodeParser is operational
-    // only when those data have been loaded
-    load: function(){
-        if (!this.nomenclature_id) {
-            return this.nomenclature ? Promise.resolve() : Promise.reject();
-        }
-        var id = this.nomenclature_id[0];
-        return rpc.query({
-                model: 'barcode.nomenclature',
-                method: 'read',
-                args: [[id], this._barcodeNomenclatureFields()],
-            }).then(nomenclatures => {
-                this.nomenclature = nomenclatures[0];
-                var args = [
-                    [['barcode_nomenclature_id', '=', this.nomenclature.id]],
-                    this._barcodeRuleFields(),
-                ];
-                return rpc.query({
-                    model: 'barcode.rule',
-                    method: 'search_read',
-                    args: args,
-                });
-            }).then(rules => {
-                rules = rules.sort(function(a, b){ return a.sequence - b.sequence; });
-                this.nomenclature.rules = rules;
-            });
-    },
-
-    // resolves when the barcode parser is operational.
-    is_loaded: function() {
-        return this.loaded;
-    },
+    setup({ nomenclature }) {
+        this.nomenclature = nomenclature;
+    }
 
     /**
      * This algorithm is identical for all fixed length numeric GS1 data structures.
@@ -77,7 +58,7 @@ var BarcodeParser = Class.extend({
         }
         total = evensum * 3 + oddsum;
         return (10 - total % 10) % 10;
-    },
+    }
 
     /**
      * Checks if the barcode string is encoded with the provided encoding.
@@ -86,7 +67,7 @@ var BarcodeParser = Class.extend({
      * @param {String} encoding could be 'any' (no encoding rules), 'ean8', 'upca' or 'ean13'
      * @returns {boolean}
      */
-    check_encoding: function(barcode, encoding) {
+    check_encoding(barcode, encoding) {
         if (encoding === 'any') {
             return true;
         }
@@ -97,7 +78,7 @@ var BarcodeParser = Class.extend({
         };
         return barcode.length === barcodeSizes[encoding] && /^\d+$/.test(barcode) &&
             this.get_barcode_check_digit(barcode) === parseInt(barcode[barcode.length - 1]);
-    },
+    }
 
     /**
      * Sanitizes a EAN-13 prefix by padding it with chars zero.
@@ -105,11 +86,11 @@ var BarcodeParser = Class.extend({
      * @param {String} ean
      * @returns {String}
      */
-    sanitize_ean: function(ean){
+    sanitize_ean(ean) {
         ean = ean.substr(0, 13);
         ean = "0".repeat(13 - ean.length) + ean;
         return ean.substr(0, 12) + this.get_barcode_check_digit(ean);
-    },
+    }
 
     /**
      * Sanitizes a UPC-A prefix by padding it with chars zero.
@@ -117,9 +98,9 @@ var BarcodeParser = Class.extend({
      * @param {String} upc
      * @returns {String}
      */
-    sanitize_upc: function(upc) {
+    sanitize_upc(upc) {
         return this.sanitize_ean(upc).substr(1, 12);
-    },
+    }
 
     // Checks if barcode matches the pattern
     // Additionnaly retrieves the optional numerical content in barcode
@@ -127,7 +108,7 @@ var BarcodeParser = Class.extend({
     // - value: the numerical value encoded in the barcode (0 if no value encoded)
     // - base_code: the barcode in which numerical content is replaced by 0's
     // - match: boolean
-    match_pattern: function (barcode, pattern, encoding){
+    match_pattern(barcode, pattern, encoding) {
         var match = {
             value: 0,
             base_code: barcode,
@@ -183,7 +164,7 @@ var BarcodeParser = Class.extend({
         match.match = match.base_code.match(base_pattern);
 
         return match;
-    },
+    }
 
     /**
      * Attempts to interpret a barcode (string encoding a barcode Code-128)
@@ -195,8 +176,15 @@ var BarcodeParser = Class.extend({
      *      - value: if the barcode encodes a numerical value, it will be put there
      *      - base_code: the barcode with all the encoding parts set to zero; the one put on the product in the backend
      */
-    parse_barcode: function(barcode){
-        var parsed_result = {
+    parse_barcode(barcode) {
+        if (barcode.match(/^urn:/)) {
+            return this.parseURI(barcode);
+        }
+        return this.parseBarcodeNomenclature(barcode);
+    }
+
+    parseBarcodeNomenclature(barcode) {
+        const parsed_result = {
             encoding: '',
             type:'error',
             code:barcode,
@@ -220,7 +208,7 @@ var BarcodeParser = Class.extend({
             } else if (rule.encoding === 'upca' &&
                     this.check_encoding(barcode,'ean13') &&
                     barcode[0] === '0' &&
-                    this.upc_ean_conv in {'ean2upc':'','always':''} ){
+                    this.nomenclature.upc_ean_conv in {'ean2upc':'','always':''} ){
                 cur_barcode = cur_barcode.substr(1,12);
             }
 
@@ -251,31 +239,70 @@ var BarcodeParser = Class.extend({
             }
         }
         return parsed_result;
-    },
+    }
 
-    //--------------------------------------------------------------------------
-    // Private
-    //--------------------------------------------------------------------------
+    // URI methods
+    /**
+     * Parse an URI into an object with either the product and its lot/serial
+     * number, either the package.
+     * @param {String} barcode
+     * @returns {Object}
+     */
+    parseURI(barcode) {
+        const uriParts = barcode.split(":").map(v => v.trim());
+        // URI should be formatted like that (number is the index once split):
+        // 0: urn, 1: epc, 2: id/tag, 3: identifier, 4: data
+        const identifier = uriParts[3];
+        const data = uriParts[4].split(".");
+        if (identifier === "lgtin" || identifier === "sgtin") {
+            return this.convertURIGTINDataIntoProductAndTrackingNumber(barcode, data);
+        } else if (identifier === "sgtin-96" || identifier === "sgtin-198") {
+            // Same compute then SGTIN but we have to remove the filter.
+            return this.convertURIGTINDataIntoProductAndTrackingNumber(barcode, data.slice(1));
+        } else if (identifier === "sscc") {
+            return this.convertURISSCCDataIntoPackage(barcode, data);
+        } else if (identifier === "sscc-96") {
+            // Same compute then SSCC but we have to remove the filter.
+            return this.convertURISSCCDataIntoPackage(barcode, data.slice(1));
+        }
+        return barcode;
+    }
 
-    _barcodeNomenclatureFields: function () {
+    convertURIGTINDataIntoProductAndTrackingNumber(base_code, data) {
+        const [gs1CompanyPrefix, itemRefAndIndicator, trackingNumber] = data;
+        const indicator = itemRefAndIndicator[0];
+        const itemRef = itemRefAndIndicator.slice(1);
+        let productBarcode = indicator + gs1CompanyPrefix + itemRef;
+        productBarcode += this.get_barcode_check_digit(productBarcode + "0");
         return [
-            'name',
-            'rule_ids',
-            'upc_ean_conv',
+            {
+                base_code,
+                code: productBarcode,
+                string_value: productBarcode,
+                type: "product",
+                value: productBarcode,
+            }, {
+                base_code,
+                code: trackingNumber,
+                string_value: trackingNumber,
+                type: "lot",
+                value: trackingNumber,
+            }
         ];
-    },
+    }
 
-    _barcodeRuleFields: function () {
-        return [
-            'name',
-            'sequence',
-            'type',
-            'encoding',
-            'pattern',
-            'alias',
-        ];
-    },
-});
-
-return BarcodeParser;
-});
+    convertURISSCCDataIntoPackage(base_code, data) {
+        const [gs1CompanyPrefix, serialReference] = data;
+        const extension = serialReference[0];
+        const serialRef = serialReference.slice(1);
+        let sscc = extension + gs1CompanyPrefix + serialRef;
+        sscc += this.get_barcode_check_digit(sscc + "0");
+        return [{
+            base_code,
+            code: sscc,
+            string_value: sscc,
+            type: "package",
+            value: sscc,
+        }];
+    }
+}
