@@ -15,9 +15,8 @@ class City(models.Model):
     country_id = fields.Many2one(comodel_name='res.country', string='Country', required=True)
     state_id = fields.Many2one(comodel_name='res.country.state', string='State', domain="[('country_id', '=', country_id)]")
 
-    def name_get(self):
-        res = []
+    @api.depends('zipcode')
+    def _compute_display_name(self):
         for city in self:
-            name = city.name if not city.zipcode else '%s (%s)' % (city.name, city.zipcode)
-            res.append((city.id, name))
-        return res
+            name = city.name if not city.zipcode else f'{city.name} ({city.zipcode})'
+            city.display_name = name
