@@ -10,7 +10,7 @@
     'installable': True,
     'auto_install': True,
     'assets': {
-        'web.assets_common': [
+        'web.assets_backend': [
             'bus/static/src/*.js',
             'bus/static/src/services/**/*.js',
             'bus/static/src/workers/websocket_worker.js',
@@ -20,18 +20,24 @@
             'bus/static/src/*.js',
             'bus/static/src/services/**/*.js',
             ('remove', 'bus/static/src/services/assets_watchdog_service.js'),
+            ('remove', 'bus/static/src/simple_notification_service.js'),
             'bus/static/src/workers/websocket_worker.js',
             'bus/static/src/workers/websocket_worker_utils.js',
         ],
-        'web.qunit_suite_tests': [
-            'bus/static/tests/**/*.js',
+        # Unit test files
+        'web.assets_unit_tests': [
+            'bus/static/tests/**/*',
+            ('remove', 'bus/static/tests/legacy/**/*'), # to remove when all legacy tests are ported
         ],
-        'web.qunit_mobile_suite_tests': [
-            'bus/static/tests/helpers/**/*.js',
+        'web.tests_assets': [
+            'bus/static/tests/legacy/helpers/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'bus/static/tests/legacy/**/*.js',
+            ('remove', 'bus/static/tests/legacy/helpers/**/*'),
         ],
         'bus.websocket_worker_assets': [
-            'web/static/src/legacy/js/promise_extension.js',
-            'web/static/src/boot.js',
+            'web/static/src/module_loader.js',
             'bus/static/src/workers/*',
         ],
     },
