@@ -1,17 +1,17 @@
 /** @odoo-module **/
 
-import { debounce } from '@bus/workers/websocket_worker_utils';
+import { debounce, Deferred } from "@bus/workers/websocket_worker_utils";
 
 /**
  * Type of events that can be sent from the worker to its clients.
  *
- * @typedef { 'connect' | 'reconnect' | 'disconnect' | 'reconnecting' | 'notification' | 'initialized' } WorkerEvent
+ * @typedef { 'connect' | 'reconnect' | 'disconnect' | 'reconnecting' | 'notification' | 'initialized' | 'outdated'|'update_state' | 'log_debug' } WorkerEvent
  */
 
 /**
  * Type of action that can be sent from the client to the worker.
  *
- * @typedef {'add_channel' | 'delete_channel' | 'force_update_channels' | 'initialize_connection' | 'send' | 'leave' | 'stop' | 'start' } WorkerAction
+ * @typedef {'add_channel' | 'delete_channel' | 'force_update_channels' | 'initialize_connection' | 'send' | 'leave' | 'stop' | 'start'} WorkerAction
  */
 
 export const WEBSOCKET_CLOSE_CODES = Object.freeze({
@@ -32,10 +32,12 @@ export const WEBSOCKET_CLOSE_CODES = Object.freeze({
     KEEP_ALIVE_TIMEOUT: 4002,
     RECONNECTING: 4003,
 });
-// Should be incremented on every worker update in order to force
-// update of the worker in browser cache.
-export const WORKER_VERSION = '1.0.4';
-const INITIAL_RECONNECT_DELAY = 1000;
+export const WORKER_STATE = Object.freeze({
+    CONNECTED: "CONNECTED",
+    DISCONNECTED: "DISCONNECTED",
+    IDLE: "IDLE",
+    CONNECTING: "CONNECTING",
+});
 const MAXIMUM_RECONNECT_DELAY = 60000;
 
 /**
@@ -46,20 +48,31 @@ const MAXIMUM_RECONNECT_DELAY = 60000;
  * for SharedWorker and this class implements it.
  */
 export class WebsocketWorker {
+    INITIAL_RECONNECT_DELAY = 1000;
+    RECONNECT_JITTER = 1000;
+
     constructor() {
+        // Timestamp of start of most recent bus service sender
+        this.newestStartTs = undefined;
         this.websocketURL = "";
         this.currentUID = null;
+        this.currentDB = null;
         this.isWaitingForNewUID = true;
         this.channelsByClient = new Map();
-        this.connectRetryDelay = INITIAL_RECONNECT_DELAY;
+        this.connectRetryDelay = this.INITIAL_RECONNECT_DELAY;
         this.connectTimeout = null;
         this.debugModeByClient = new Map();
         this.isDebug = false;
+        this.active = true;
+        this.state = WORKER_STATE.IDLE;
         this.isReconnecting = false;
         this.lastChannelSubscription = null;
+        this.firstSubscribeDeferred = new Deferred();
         this.lastNotificationId = 0;
         this.messageWaitQueue = [];
-        this._forceUpdateChannels = debounce(this._forceUpdateChannels, 300, true);
+        this._forceUpdateChannels = debounce(this._forceUpdateChannels, 300);
+        this._debouncedUpdateChannels = debounce(this._updateChannels, 300);
+        this._debouncedSendToServer = debounce(this._sendToServer, 300);
 
         this._onWebsocketClose = this._onWebsocketClose.bind(this);
         this._onWebsocketError = this._onWebsocketError.bind(this);
@@ -80,8 +93,9 @@ export class WebsocketWorker {
      * @param {Object} data
      */
     broadcast(type, data) {
+        this._logDebug("broadcast", type, data);
         for (const client of this.channelsByClient.keys()) {
-            client.postMessage({ type, data });
+            client.postMessage({ type, data: data ? JSON.parse(JSON.stringify(data)) : undefined });
         }
     }
 
@@ -91,7 +105,7 @@ export class WebsocketWorker {
      * @param {MessagePort} messagePort
      */
     registerClient(messagePort) {
-        messagePort.onmessage = ev => {
+        messagePort.onmessage = (ev) => {
             this._onClientMessage(messagePort, ev.data);
         };
         this.channelsByClient.set(messagePort, []);
@@ -105,7 +119,8 @@ export class WebsocketWorker {
      * @param {Object} data
      */
     sendToClient(client, type, data) {
-        client.postMessage({ type, data });
+        this._logDebug("sendToClient", type, data);
+        client.postMessage({ type, data: data ? JSON.parse(JSON.stringify(data)) : undefined });
     }
 
     //--------------------------------------------------------------------------
@@ -124,22 +139,29 @@ export class WebsocketWorker {
      * action.
      */
     _onClientMessage(client, { action, data }) {
+        this._logDebug("_onClientMessage", action, data);
         switch (action) {
-            case 'send':
-                return this._sendToServer(data);
-            case 'start':
+            case "send": {
+                if (data["event_name"] === "update_presence") {
+                    this._debouncedSendToServer(data);
+                } else {
+                    this._sendToServer(data);
+                }
+                return;
+            }
+            case "start":
                 return this._start();
-            case 'stop':
+            case "stop":
                 return this._stop();
-            case 'leave':
+            case "leave":
                 return this._unregisterClient(client);
-            case 'add_channel':
+            case "add_channel":
                 return this._addChannel(client, data);
-            case 'delete_channel':
+            case "delete_channel":
                 return this._deleteChannel(client, data);
-            case 'force_update_channels':
+            case "force_update_channels":
                 return this._forceUpdateChannels();
-            case 'initialize_connection':
+            case "initialize_connection":
                 return this._initializeConnection(client, data);
         }
     }
@@ -156,7 +178,7 @@ export class WebsocketWorker {
         if (!clientChannels.includes(channel)) {
             clientChannels.push(channel);
             this.channelsByClient.set(client, clientChannels);
-            this._updateChannels();
+            this._debouncedUpdateChannels();
         }
     }
 
@@ -175,7 +197,7 @@ export class WebsocketWorker {
         const channelIndex = clientChannels.indexOf(channel);
         if (channelIndex !== -1) {
             clientChannels.splice(channelIndex, 1);
-            this._updateChannels();
+            this._debouncedUpdateChannels();
         }
     }
 
@@ -197,14 +219,15 @@ export class WebsocketWorker {
     _unregisterClient(client) {
         this.channelsByClient.delete(client);
         this.debugModeByClient.delete(client);
-        this.isDebug = Object.values(this.debugModeByClient).some(debugValue => debugValue !== '');
-        this._updateChannels();
+        this.isDebug = [...this.debugModeByClient.values()].some(Boolean);
+        this._debouncedUpdateChannels();
     }
 
     /**
      * Initialize a client connection to this worker.
      *
      * @param {Object} param0
+     * @param {string} [param0.db] Database name.
      * @param {String} [param0.debug] Current debugging mode for the
      * given client.
      * @param {Number} [param0.lastNotificationId] Last notification id
@@ -214,25 +237,39 @@ export class WebsocketWorker {
      *     - Number: user is logged whether on the frontend/backend.
      *     - false: user is not logged.
      *     - undefined: not available (e.g. livechat support page)
+     * @param {Number} param0.startTs Timestamp of start of bus service sender.
      */
-    _initializeConnection(client, { debug, lastNotificationId, uid, websocketURL }) {
+    _initializeConnection(client, { db, debug, lastNotificationId, uid, websocketURL, startTs }) {
+        if (this.newestStartTs && this.newestStartTs > startTs) {
+            this.debugModeByClient.set(client, debug);
+            this.isDebug = [...this.debugModeByClient.values()].some(Boolean);
+            this.sendToClient(client, "initialized");
+            this.sendToClient(client, "update_state", this.state);
+            return;
+        }
+        this.newestStartTs = startTs;
         this.websocketURL = websocketURL;
         this.lastNotificationId = lastNotificationId;
-        this.debugModeByClient[client] = debug;
-        this.isDebug = Object.values(this.debugModeByClient).some(debugValue => debugValue !== '');
+        this.debugModeByClient.set(client, debug);
+        this.isDebug = [...this.debugModeByClient.values()].some(Boolean);
         const isCurrentUserKnown = uid !== undefined;
         if (this.isWaitingForNewUID && isCurrentUserKnown) {
             this.isWaitingForNewUID = false;
             this.currentUID = uid;
         }
-        if (this.currentUID !== uid && isCurrentUserKnown) {
+        if ((this.currentUID !== uid && isCurrentUserKnown) || this.currentDB !== db) {
             this.currentUID = uid;
+            this.currentDB = db;
             if (this.websocket) {
                 this.websocket.close(WEBSOCKET_CLOSE_CODES.CLEAN);
             }
             this.channelsByClient.forEach((_, key) => this.channelsByClient.set(key, []));
         }
-        this.sendToClient(client, 'initialized');
+        this.sendToClient(client, "initialized");
+        this.sendToClient(client, "update_state", this.state);
+        if (!this.active) {
+            this.sendToClient(client, "outdated");
+        }
     }
 
     /**
@@ -277,23 +314,28 @@ export class WebsocketWorker {
      * closed.
      */
     _onWebsocketClose({ code, reason }) {
-        if (this.isDebug) {
-            console.debug(`%c${new Date().toLocaleString()} - [onClose]`, 'color: #c6e; font-weight: bold;', code, reason);
-        }
+        this._logDebug("_onWebsocketClose", code, reason);
+        this._updateState(WORKER_STATE.DISCONNECTED);
         this.lastChannelSubscription = null;
+        this.firstSubscribeDeferred = new Deferred();
         if (this.isReconnecting) {
             // Connection was not established but the close event was
             // triggered anyway. Let the onWebsocketError method handle
             // this case.
             return;
         }
-        this.broadcast('disconnect', { code, reason });
+        this.broadcast("disconnect", { code, reason });
         if (code === WEBSOCKET_CLOSE_CODES.CLEAN) {
+            if (reason === "OUTDATED_VERSION") {
+                console.warn("Worker deactivated due to an outdated version.");
+                this.active = false;
+                this.broadcast("outdated");
+            }
             // WebSocket was closed on purpose, do not try to reconnect.
             return;
         }
         // WebSocket was not closed cleanly, let's try to reconnect.
-        this.broadcast('reconnecting', { closeCode: code });
+        this.broadcast("reconnecting", { closeCode: code });
         this.isReconnecting = true;
         if (code === WEBSOCKET_CLOSE_CODES.KEEP_ALIVE_TIMEOUT) {
             // Don't wait to reconnect on keep alive timeout.
@@ -309,24 +351,36 @@ export class WebsocketWorker {
      * Triggered when a connection failed or failed to established.
      */
     _onWebsocketError() {
-        if (this.isDebug) {
-            console.debug(`%c${new Date().toLocaleString()} - [onError]`, 'color: #c6e; font-weight: bold;');
-        }
+        this._logDebug("_onWebsocketError");
         this._retryConnectionWithDelay();
     }
 
     /**
-    * Handle data received from the bus.
-    *
-    * @param {MessageEvent} messageEv
-    */
+     * Handle data received from the bus.
+     *
+     * @param {MessageEvent} messageEv
+     */
     _onWebsocketMessage(messageEv) {
         const notifications = JSON.parse(messageEv.data);
-        if (this.isDebug) {
-            console.debug(`%c${new Date().toLocaleString()} - [onMessage]`, 'color: #c6e; font-weight: bold;', notifications);
-        }
+        this._logDebug("_onWebsocketMessage", notifications);
         this.lastNotificationId = notifications[notifications.length - 1].id;
-        this.broadcast('notification', notifications);
+        this.broadcast("notification", notifications);
+    }
+
+    _logDebug(title, ...args) {
+        const clientsInDebug = [...this.debugModeByClient.keys()].filter((client) =>
+            this.debugModeByClient.get(client)
+        );
+        for (const client of clientsInDebug) {
+            client.postMessage({
+                type: "log_debug",
+                data: [
+                    `%c${new Date().toLocaleString()} - [${title}]`,
+                    "color: #c6e; font-weight: bold;",
+                    ...args,
+                ],
+            });
+        }
     }
 
     /**
@@ -334,16 +388,17 @@ export class WebsocketWorker {
      * the connection to open.
      */
     _onWebsocketOpen() {
-        if (this.isDebug) {
-            console.debug(`%c${new Date().toLocaleString()} - [onOpen]`, 'color: #c6e; font-weight: bold;');
-        }
-        this._updateChannels();
-        this.messageWaitQueue.forEach(msg => this.websocket.send(msg));
-        this.messageWaitQueue = [];
-        this.broadcast(this.isReconnecting ? 'reconnect' : 'connect');
-        this.connectRetryDelay = INITIAL_RECONNECT_DELAY;
+        this._logDebug("_onWebsocketOpen");
+        this._updateState(WORKER_STATE.CONNECTED);
+        this.broadcast(this.isReconnecting ? "reconnect" : "connect");
+        this._debouncedUpdateChannels();
+        this.connectRetryDelay = this.INITIAL_RECONNECT_DELAY;
         this.connectTimeout = null;
         this.isReconnecting = false;
+        this.firstSubscribeDeferred.then(() => {
+            this.messageWaitQueue.forEach((msg) => this.websocket.send(msg));
+            this.messageWaitQueue = [];
+        });
     }
 
     /**
@@ -351,7 +406,10 @@ export class WebsocketWorker {
      * applied to the reconnect attempts.
      */
     _retryConnectionWithDelay() {
-        this.connectRetryDelay = Math.min(this.connectRetryDelay * 1.5, MAXIMUM_RECONNECT_DELAY) + 1000 * Math.random();
+        this.connectRetryDelay =
+            Math.min(this.connectRetryDelay * 1.5, MAXIMUM_RECONNECT_DELAY) +
+            this.RECONNECT_JITTER * Math.random();
+        this._logDebug("_retryConnectionWithDelay", this.connectRetryDelay);
         this.connectTimeout = setTimeout(this._start.bind(this), this.connectRetryDelay);
     }
 
@@ -360,14 +418,26 @@ export class WebsocketWorker {
      * If the websocket is not open, enqueue the message and send it
      * upon the next reconnection.
      *
-     * @param {any} message Message to send to the server.
+     * @param {{event_name: string, data: any }} message Message to send to the server.
      */
     _sendToServer(message) {
+        this._logDebug("_sendToServer", message);
         const payload = JSON.stringify(message);
         if (!this._isWebsocketConnected()) {
-            this.messageWaitQueue.push(payload);
+            if (message["event_name"] === "subscribe") {
+                this.messageWaitQueue = this.messageWaitQueue.filter(
+                    (msg) => JSON.parse(msg).event_name !== "subscribe"
+                );
+                this.messageWaitQueue.unshift(payload);
+            } else {
+                this.messageWaitQueue.push(payload);
+            }
         } else {
-            this.websocket.send(payload);
+            if (message["event_name"] === "subscribe") {
+                this.websocket.send(payload);
+            } else {
+                this.firstSubscribeDeferred.then(() => this.websocket.send(payload));
+            }
         }
     }
 
@@ -375,14 +445,15 @@ export class WebsocketWorker {
      * Start the worker by opening a websocket connection.
      */
     _start() {
-        if (this._isWebsocketConnected() || this._isWebsocketConnecting()) {
+        this._logDebug("_start");
+        if (!this.active || this._isWebsocketConnected() || this._isWebsocketConnecting()) {
             return;
         }
         if (this.websocket) {
-            this.websocket.removeEventListener('open', this._onWebsocketOpen);
-            this.websocket.removeEventListener('message', this._onWebsocketMessage);
-            this.websocket.removeEventListener('error', this._onWebsocketError);
-            this.websocket.removeEventListener('close', this._onWebsocketClose);
+            this.websocket.removeEventListener("open", this._onWebsocketOpen);
+            this.websocket.removeEventListener("message", this._onWebsocketMessage);
+            this.websocket.removeEventListener("error", this._onWebsocketError);
+            this.websocket.removeEventListener("close", this._onWebsocketClose);
         }
         if (this._isWebsocketClosing()) {
             // close event was not triggered and will never be, broadcast the
@@ -390,19 +461,21 @@ export class WebsocketWorker {
             this.lastChannelSubscription = null;
             this.broadcast("disconnect", { code: WEBSOCKET_CLOSE_CODES.ABNORMAL_CLOSURE });
         }
+        this._updateState(WORKER_STATE.CONNECTING);
         this.websocket = new WebSocket(this.websocketURL);
-        this.websocket.addEventListener('open', this._onWebsocketOpen);
-        this.websocket.addEventListener('error', this._onWebsocketError);
-        this.websocket.addEventListener('message', this._onWebsocketMessage);
-        this.websocket.addEventListener('close', this._onWebsocketClose);
+        this.websocket.addEventListener("open", this._onWebsocketOpen);
+        this.websocket.addEventListener("error", this._onWebsocketError);
+        this.websocket.addEventListener("message", this._onWebsocketMessage);
+        this.websocket.addEventListener("close", this._onWebsocketClose);
     }
 
     /**
      * Stop the worker.
      */
     _stop() {
+        this._logDebug("_stop");
         clearTimeout(this.connectTimeout);
-        this.connectRetryDelay = INITIAL_RECONNECT_DELAY;
+        this.connectRetryDelay = this.INITIAL_RECONNECT_DELAY;
         this.isReconnecting = false;
         this.lastChannelSubscription = null;
         if (this.websocket) {
@@ -418,12 +491,28 @@ export class WebsocketWorker {
      * event if the channels haven't change since last subscription.
      */
     _updateChannels({ force = false } = {}) {
-        const allTabsChannels = [...new Set([].concat.apply([], [...this.channelsByClient.values()]))].sort();
+        const allTabsChannels = [
+            ...new Set([].concat.apply([], [...this.channelsByClient.values()])),
+        ].sort();
         const allTabsChannelsString = JSON.stringify(allTabsChannels);
-        const shouldUpdateChannelSubscription = allTabsChannelsString !== this.lastChannelSubscription;
+        const shouldUpdateChannelSubscription =
+            allTabsChannelsString !== this.lastChannelSubscription;
         if (force || shouldUpdateChannelSubscription) {
             this.lastChannelSubscription = allTabsChannelsString;
-            this._sendToServer({ event_name: 'subscribe', data: { channels: allTabsChannels, last: this.lastNotificationId } });
+            this._sendToServer({
+                event_name: "subscribe",
+                data: { channels: allTabsChannels, last: this.lastNotificationId },
+            });
+            this.firstSubscribeDeferred.resolve();
         }
     }
+    /**
+     * Update the worker state and broadcast the new state to its clients.
+     *
+     * @param {WORKER_STATE[keyof WORKER_STATE]} newState
+     */
+    _updateState(newState) {
+        this.state = newState;
+        this.broadcast("update_state", newState);
+    }
 }
