@@ -8,7 +8,7 @@
  *
  * Inspired by https://davidwalsh.name/javascript-debounce-function
  */
- export function debounce(func, wait, immediate) {
+export function debounce(func, wait, immediate) {
     let timeout;
     return function () {
         const context = this;
@@ -27,3 +27,18 @@
         }
     };
 }
+
+/**
+ * Deferred is basically a resolvable/rejectable extension of Promise.
+ */
+export class Deferred extends Promise {
+    constructor() {
+        let resolve;
+        let reject;
+        const prom = new Promise((res, rej) => {
+            resolve = res;
+            reject = rej;
+        });
+        return Object.assign(prom, { resolve, reject });
+    }
+}
