@@ -15,4 +15,4 @@ class MailActivityMixin(models.AbstractModel):
         """This computes the calendar event of the next activity.
         It evaluates to false if there is no such event."""
         for record in self:
-            record.activity_calendar_event_id = record.activity_ids[:1].calendar_event_id
+            record.activity_calendar_event_id = fields.first(record.activity_ids).calendar_event_id
