@@ -15,5 +15,14 @@ You can track your vendors, customers and other contacts.
     'data': [
         'views/contact_views.xml',
     ],
+    'demo': [
+        'data/mail_demo.xml',
+    ],
     'application': True,
+    'license': 'LGPL-3',
+    'assets': {
+        'web.assets_tests': [
+            'contacts/static/tests/tours/**/*',
+        ],
+    }
 }
