@@ -7,10 +7,11 @@ import random
 import threading
 
 from ast import literal_eval
+from markupsafe import Markup
 
 from odoo import api, exceptions, fields, models, _
 from odoo.osv import expression
-from odoo.tools import float_compare, float_round
+from odoo.tools import float_compare, float_round, SQL
 from odoo.tools.safe_eval import safe_eval
 
 _logger = logging.getLogger(__name__)
@@ -23,9 +24,7 @@ class Team(models.Model):
 
     use_leads = fields.Boolean('Leads', help="Check this box to filter and qualify incoming requests as leads before converting them into opportunities and assigning them to a salesperson.")
     use_opportunities = fields.Boolean('Pipeline', default=True, help="Check this box to manage a presales process with opportunities.")
-    alias_id = fields.Many2one(
-        'mail.alias', string='Alias', ondelete="restrict", required=True,
-        help="The email address associated with this channel. New emails received will automatically create new leads assigned to the channel.")
+    alias_id = fields.Many2one(help="The email address associated with this channel. New emails received will automatically create new leads assigned to the channel.")
     # assignment
     assignment_enabled = fields.Boolean('Lead Assign', compute='_compute_assignment_enabled')
     assignment_auto_enabled = fields.Boolean('Auto Assignment', compute='_compute_assignment_enabled')
@@ -42,6 +41,9 @@ class Team(models.Model):
     lead_all_assigned_month_count = fields.Integer(
         string='# Leads/Opps assigned this month', compute='_compute_lead_all_assigned_month_count',
         help="Number of leads and opportunities assigned this last month.")
+    lead_all_assigned_month_exceeded = fields.Boolean('Exceed monthly lead assignement', compute="_compute_lead_all_assigned_month_count",
+        help="True if the monthly lead assignment count is greater than the maximum assignment limit, false otherwise."
+    )
     opportunities_count = fields.Integer(
         string='# Opportunities', compute='_compute_opportunities_data')
     opportunities_amount = fields.Monetary(
@@ -50,10 +52,6 @@ class Team(models.Model):
         string='# Overdue Opportunities', compute='_compute_opportunities_overdue_data')
     opportunities_overdue_amount = fields.Monetary(
         string='Overdue Opportunities Revenues', compute='_compute_opportunities_overdue_data',)
-    # alias: improve fields coming from _inherits, use inherited to avoid replacing them
-    alias_user_id = fields.Many2one(
-        'res.users', related='alias_id.alias_user_id', readonly=False, inherited=True,
-        domain=lambda self: [('groups_id', 'in', self.env.ref('sales_team.group_sale_salesman_all_leads').id)])
     # properties
     lead_properties_definition = fields.PropertiesDefinition('Lead Properties')
 
@@ -76,27 +74,26 @@ class Team(models.Model):
             ('team_id', 'in', self.ids),
             ('type', '=', 'lead'),
             ('user_id', '=', False),
-        ], ['team_id'], ['team_id'])
-        counts = {datum['team_id'][0]: datum['team_id_count'] for datum in leads_data}
+        ], ['team_id'], ['__count'])
+        counts = {team.id: count for team, count in leads_data}
         for team in self:
             team.lead_unassigned_count = counts.get(team.id, 0)
 
-    @api.depends('crm_team_member_ids.lead_month_count')
+    @api.depends('crm_team_member_ids.lead_month_count', 'assignment_max')
     def _compute_lead_all_assigned_month_count(self):
         for team in self:
             team.lead_all_assigned_month_count = sum(member.lead_month_count for member in team.crm_team_member_ids)
+            team.lead_all_assigned_month_exceeded = team.lead_all_assigned_month_count > team.assignment_max
 
     def _compute_opportunities_data(self):
         opportunity_data = self.env['crm.lead']._read_group([
             ('team_id', 'in', self.ids),
             ('probability', '<', 100),
             ('type', '=', 'opportunity'),
-        ], ['expected_revenue:sum', 'team_id'], ['team_id'])
-        counts = {datum['team_id'][0]: datum['team_id_count'] for datum in opportunity_data}
-        amounts = {datum['team_id'][0]: datum['expected_revenue'] for datum in opportunity_data}
+        ], ['team_id'], ['__count', 'expected_revenue:sum'])
+        counts_amounts = {team.id: (count, expected_revenue_sum) for team, count, expected_revenue_sum in opportunity_data}
         for team in self:
-            team.opportunities_count = counts.get(team.id, 0)
-            team.opportunities_amount = amounts.get(team.id, 0)
+            team.opportunities_count, team.opportunities_amount = counts_amounts.get(team.id, (0, 0))
 
     def _compute_opportunities_overdue_data(self):
         opportunity_data = self.env['crm.lead']._read_group([
@@ -104,12 +101,10 @@ class Team(models.Model):
             ('probability', '<', 100),
             ('type', '=', 'opportunity'),
             ('date_deadline', '<', fields.Date.to_string(fields.Datetime.now()))
-        ], ['expected_revenue', 'team_id'], ['team_id'])
-        counts = {datum['team_id'][0]: datum['team_id_count'] for datum in opportunity_data}
-        amounts = {datum['team_id'][0]: (datum['expected_revenue']) for datum in opportunity_data}
+        ], ['team_id'], ['__count', 'expected_revenue:sum'])
+        counts_amounts = {team.id: (count, expected_revenue_sum) for team, count, expected_revenue_sum in opportunity_data}
         for team in self:
-            team.opportunities_overdue_count = counts.get(team.id, 0)
-            team.opportunities_overdue_amount = amounts.get(team.id, 0)
+            team.opportunities_overdue_count, team.opportunities_overdue_amount = counts_amounts.get(team.id, (0, 0))
 
     @api.onchange('use_leads', 'use_opportunities')
     def _onchange_use_leads_opportunities(self):
@@ -198,51 +193,30 @@ class Team(models.Model):
     # ------------------------------------------------------------
 
     @api.model
-    def _cron_assign_leads(self, work_days=None):
+    def _cron_assign_leads(self, force_quota=False, creation_delta_days=7):
         """ Cron method assigning leads. Leads are allocated to all teams and
-        assigned to their members. It is based on either cron configuration
-        either forced through ``work_days`` parameter.
+        assigned to their members.
 
-        When based on cron configuration purpose of cron is to assign leads to
-        sales persons. Assigned workload is set to the workload those sales
-        people should perform between two cron iterations. If their maximum
-        capacity is reached assign process will not assign them any more lead.
-
-        e.g. cron is active with interval_number 3, interval_type days. This
-        means cron runs every 3 days. Cron will assign leads for 3 work days
-        to salespersons each 3 days unless their maximum capacity is reached.
-
-        If cron runs on an hour- or minute-based schedule minimum assignment
-        performed is equivalent to 0.2 workdays to avoid rounding issues.
-        Max assignment performed is for 30 days as it is better to run more
-        often than planning for more than one month. Assign process is best
-        designed to run every few hours (~4 times / day) or each few days.
+        The cron is designed to run at least once a day or more.
+        A number of leads will be assigned each time depending on the daily leads
+        already assigned.
+        This allows the assignation process based on the cron to work on a daily basis
+        without allocating too much leads on members if the cron is executed multiple
+        times a day.
+        The daily quota of leads can be forcefully assigned with force_quota
+        (ignoring the daily leads already assigned).
 
         See ``CrmTeam.action_assign_leads()`` and its sub methods for more
         details about assign process.
 
-        :param float work_days: see ``CrmTeam.action_assign_leads()``;
         """
-        assign_cron = self.sudo().env.ref('crm.ir_cron_crm_lead_assign', raise_if_not_found=False)
-        if not work_days and assign_cron and assign_cron.active:
-            if assign_cron.interval_type == 'months':
-                work_days = 30  # maximum one month of work
-            elif assign_cron.interval_type == 'weeks':
-                work_days = min(30, assign_cron.interval_number * 7)  # max at 30 (better lead repartition)
-            elif assign_cron.interval_type == 'days':
-                work_days = min(30, assign_cron.interval_number * 1)  # max at 30 (better lead repartition)
-            elif assign_cron.interval_type == 'hours':
-                work_days = max(0.2, assign_cron.interval_number / 24)    # min at 0.2 to avoid small numbers issues
-            elif assign_cron.interval_type == 'minutes':
-                work_days = max(0.2, assign_cron.interval_number / 1440)    # min at 0.2 to avoid small numbers issues
-        work_days = work_days if work_days else 1  # avoid void values
         self.env['crm.team'].search([
             '&', '|', ('use_leads', '=', True), ('use_opportunities', '=', True),
             ('assignment_optout', '=', False)
-        ])._action_assign_leads(work_days=work_days)
+        ])._action_assign_leads(force_quota=force_quota, creation_delta_days=creation_delta_days)
         return True
 
-    def action_assign_leads(self, work_days=1, log=True):
+    def action_assign_leads(self):
         """ Manual (direct) leads assignment. This method both
 
           * assigns leads to teams given by self;
@@ -250,26 +224,20 @@ class Team(models.Model):
 
         See sub methods for more details about assign process.
 
-        :param float work_days: number of work days to consider when assigning leads
-          to teams or salespersons. We consider that Member.assignment_max (or
-          its equivalent on team model) targets 30 work days. We make a ratio
-          between expected number of work days and maximum assignment for those
-          30 days to know lead count to assign.
-
         :return action: a client notification giving some insights on assign
           process;
         """
-        teams_data, members_data = self._action_assign_leads(work_days=work_days)
+        teams_data, members_data = self._action_assign_leads(force_quota=True)
 
         # format result messages
         logs = self._action_assign_leads_logs(teams_data, members_data)
-        html_message = '<br />'.join(logs)
+        html_message = Markup('<br />').join(logs)
         notif_message = ' '.join(logs)
 
         # log a note in case of manual assign (as this method will mainly be called
         # on singleton record set, do not bother doing a specific message per team)
         log_action = _("Lead Assignment requested by %(user_name)s", user_name=self.env.user.name)
-        log_message = "<p>%s<br /><br />%s</p>" % (log_action, html_message)
+        log_message = Markup("<p>%s<br /><br />%s</p>") % (log_action, html_message)
         self._message_log_batch(bodies=dict((team.id, log_message) for team in self))
 
         return {
@@ -285,7 +253,7 @@ class Team(models.Model):
             }
         }
 
-    def _action_assign_leads(self, work_days=1):
+    def _action_assign_leads(self, force_quota=False, creation_delta_days=7):
         """ Private method for lead assignment. This method both
 
           * assigns leads to teams given by self;
@@ -293,19 +261,26 @@ class Team(models.Model):
 
         See sub methods for more details about assign process.
 
-        :param float work_days: see ``CrmTeam.action_assign_leads()``;
+        :param bool force_quota: Assign the full daily quota without taking into account
+                                 the leads already assigned today
+        :param int creation_delta_days: Take into account all leads created in the last nb days (by default 7).
+                                        If set to zero we take all the past leads.
 
         :return teams_data, members_data: structure-based result of assignment
           process. For more details about data see ``CrmTeam._allocate_leads()``
-          and ``CrmTeamMember._assign_and_convert_leads``;
+          and ``CrmTeam._assign_and_convert_leads``;
         """
-        if not self.env.user.has_group('sales_team.group_sale_manager') and not self.env.user.has_group('base.group_system'):
+        if not (self.env.user.has_group('sales_team.group_sale_manager') or self.env.is_system()):
             raise exceptions.UserError(_('Lead/Opportunities automatic assignment is limited to managers or administrators'))
 
-        _logger.info('### START Lead Assignment (%d teams, %d sales persons, %.2f work_days)', len(self), len(self.crm_team_member_ids), work_days)
-        teams_data = self._allocate_leads(work_days=work_days)
+        _logger.info(
+            '### START Lead Assignment (%d teams, %d sales persons, force daily quota: %s)',
+            len(self),
+            len(self.crm_team_member_ids),
+            "ON" if force_quota else "OFF")
+        teams_data = self._allocate_leads(creation_delta_days=creation_delta_days)
         _logger.info('### Team repartition done. Starting salesmen assignment.')
-        members_data = self.crm_team_member_ids._assign_and_convert_leads(work_days=work_days)
+        members_data = self._assign_and_convert_leads(force_quota=force_quota)
         _logger.info('### END Lead Assignment')
         return teams_data, members_data
 
@@ -313,7 +288,7 @@ class Team(models.Model):
         """ Tool method to prepare notification about assignment process result.
 
         :param teams_data: see ``CrmTeam._allocate_leads()``;
-        :param members_data: see ``CrmTeamMember._assign_and_convert_leads()``;
+        :param members_data: see ``CrmTeam._assign_and_convert_leads()``;
 
         :return list: list of formatted logs, ready to be formatted into a nice
         plaintext or html message at caller's will
@@ -375,7 +350,7 @@ class Team(models.Model):
 
         return message_parts
 
-    def _allocate_leads(self, work_days=1):
+    def _allocate_leads(self, creation_delta_days=7):
         """ Allocate leads to teams given by self. This method sets ``team_id``
         field on lead records that are unassigned (no team and no responsible).
         No salesperson is assigned in this process. Its purpose is simply to
@@ -389,6 +364,8 @@ class Team(models.Model):
             * without team, without user -> not assigned;
             * not in a won stage, and not having False/0 (lost) or 100 (won)
               probability) -> live leads;
+            * created in the last creation_delta_days (in the last week by default)
+              This avoid to take into account old leads in the allocation.
             * if set, a delay after creation can be applied (see BUNDLE_HOURS_DELAY)
               parameter explanations here below;
             * matching the team's assignment domain (empty means
@@ -420,14 +397,14 @@ class Team(models.Model):
         :config int crm.assignment.commit.bundle: optional config parameter allowing
           to set size of lead batch to be committed together. By default 100
           which is a good trade-off between transaction time and speed
-        :config int crm.assignment.delay: optional config parameter giving a
+        :config float crm.assignment.delay: optional config parameter giving a
           delay before taking a lead into assignment process (BUNDLE_HOURS_DELAY)
-          given in hours. Purpose if to allow other crons or automated actions
+          given in hours. Purpose if to allow other crons or automation rules
           to make their job. This option is mainly historic as its purpose was
-          to let automated actions prepare leads and score before PLS was added
+          to let automation rules prepare leads and score before PLS was added
           into CRM. This is now not required anymore but still supported;
 
-        :param float work_days: see ``CrmTeam.action_assign_leads()``;
+        :param int creation_delta_days: see ``CrmTeam._action_assign_leads()``;
 
         :return teams_data: dict() with each team assignment result:
           team: {
@@ -440,12 +417,8 @@ class Team(models.Model):
               are already removed at return of this method;
           }, ...
         """
-        if work_days < 0.2 or work_days > 30:
-            raise ValueError(
-                _('Leads team allocation should be done for at least 0.2 or maximum 30 work days, not %.2f.', work_days)
-            )
 
-        BUNDLE_HOURS_DELAY = int(self.env['ir.config_parameter'].sudo().get_param('crm.assignment.delay', default=0))
+        BUNDLE_HOURS_DELAY = float(self.env['ir.config_parameter'].sudo().get_param('crm.assignment.delay', default=0))
         BUNDLE_COMMIT_SIZE = int(self.env['ir.config_parameter'].sudo().get_param('crm.assignment.commit.bundle', 100))
         auto_commit = not getattr(threading.current_thread(), 'testing', False)
 
@@ -465,6 +438,11 @@ class Team(models.Model):
                 ['&', ('team_id', '=', False), ('user_id', '=', False)],
                 ['|', ('stage_id', '=', False), ('stage_id.is_won', '=', False)]
             ])
+            if creation_delta_days > 0:
+                lead_domain = expression.AND([
+                    lead_domain,
+                    [('create_date', '>', self.env.cr.now() - datetime.timedelta(days=creation_delta_days))]
+                ])
 
             leads = self.env["crm.lead"].search(lead_domain)
             # Fill duplicate cache: search for duplicate lead before the assignation
@@ -587,6 +565,100 @@ class Team(models.Model):
             'duplicates': leads_dup_ids,
         }
 
+    def _get_lead_to_assign_domain(self):
+        return [
+            ('user_id', '=', False),
+            ('date_open', '=', False),
+            ('team_id', 'in', self.ids),
+        ]
+
+    def _assign_and_convert_leads(self, force_quota=False):
+        """ Main processing method to assign leads to sales team members. It also
+        converts them into opportunities. This method should be called after
+        ``_allocate_leads`` as this method assigns leads already allocated to
+        the member's team. Its main purpose is therefore to distribute team
+        workload on its members based on their capacity.
+
+        This method follows the following heuristic
+            * Get quota per member
+            * Find all leads to be assigned per team
+            * Sort list of members per number of leads received in the last 24h
+            * Assign the lead using round robin
+                * Find the first member with a compatible domain
+                * Assign the lead
+                * Move the member at the end of the list if quota is not reached
+                * Remove it otherwise
+                * Move to the next lead
+
+        :param bool force_quota: see ``CrmTeam._action_assign_leads()``;
+
+        :return members_data: dict() with each member assignment result:
+          membership: {
+            'assigned': set of lead IDs directly assigned to the member;
+          }, ...
+
+        """
+        auto_commit = not getattr(threading.current_thread(), 'testing', False)
+        result_data = {}
+        commit_bundle_size = int(self.env['ir.config_parameter'].sudo().get_param('crm.assignment.commit.bundle', 100))
+        teams_with_members = self.filtered(lambda team: team.crm_team_member_ids)
+        quota_per_member = {member: member._get_assignment_quota(force_quota=force_quota) for member in self.crm_team_member_ids}
+        counter = 0
+        leads_per_team = dict(self.env['crm.lead']._read_group(
+            teams_with_members._get_lead_to_assign_domain(),
+            ['team_id'],
+            # Do not use recordset aggregation to avoid fetching all the leads at once in memory
+            # We want to have in memory only leads for the current team
+            # and make sure we need them before fetching them
+            ['id:array_agg'],
+        ))
+        for team, leads_to_assign_ids in leads_per_team.items():
+            members_to_assign = list(team.crm_team_member_ids.filtered(lambda member:
+                not member.assignment_optout and quota_per_member.get(member, 0) > 0
+            ).sorted(key=lambda member: quota_per_member.get(member, 0), reverse=True))
+            if not members_to_assign:
+                continue
+            result_data.update({
+                member: {"assigned": self.env["crm.lead"], "quota": quota_per_member[member]}
+                for member in members_to_assign
+            })
+            # Need to check that record still exists since the ids have been fetched at the begining of the process
+            # Previous iteration has commited the change, records may have been deleted in the meanwhile
+            leads_to_assign = self.env['crm.lead'].browse(leads_to_assign_ids).exists()
+            leads_per_member = {
+                member: leads_to_assign.filtered_domain(literal_eval(member.assignment_domain or '[]'))
+                for member in members_to_assign
+            }
+            for lead in leads_to_assign.sorted(lambda lead: (-lead.probability, id)):
+                counter += 1
+                member_found = next((member for member in members_to_assign if lead in leads_per_member[member]), False)
+                if not member_found:
+                    continue
+                lead.with_context(mail_auto_subscribe_no_notify=True).convert_opportunity(
+                    lead.partner_id,
+                    user_ids=member_found.user_id.ids
+                )
+                result_data[member_found]['assigned'] += lead
+                members_to_assign.remove(member_found)
+                quota_per_member[member_found] -= 1
+                if quota_per_member[member_found] > 0:
+                    # If the member should receive more lead, send him back at the end of the list
+                    members_to_assign.append(member_found)
+
+                if auto_commit and counter % commit_bundle_size == 0:
+                    self.env.cr.commit()
+            # Make sure we commit at least at the end of the team
+            if auto_commit:
+                self.env.cr.commit()
+            # Once we are done with a team we don't need to keep the leads in memory
+            # Try to avoid to explode memory usage
+            self.env.invalidate_all()
+
+        _logger.info('Assigned %s leads to %s salesmen', sum(len(r['assigned']) for r in result_data.values()), len(result_data))
+        for member, member_info in result_data.items():
+            _logger.info('-> member %s of team %s: assigned %d/%d leads (%s)', member.id, member.crm_team_id.id, len(member_info["assigned"]), member_info["quota"], member_info["assigned"])
+        return result_data
+
     # ------------------------------------------------------------
     # ACTIONS
     # ------------------------------------------------------------
@@ -612,7 +684,7 @@ class Team(models.Model):
             user_team_id = self.search([], limit=1).id
             action['help'] = "<p class='o_view_nocontent_smiling_face'>%s</p><p>" % _("Create an Opportunity")
             if user_team_id:
-                if self.user_has_groups('sales_team.group_sale_manager'):
+                if self.env.user.has_group('sales_team.group_sale_manager'):
                     action['help'] += "<p>%s</p>" % _("""As you are a member of no Sales Team, you are showed the Pipeline of the <b>first team by default.</b>
                                         To work with the CRM, you should <a name="%d" type="action" tabindex="-1">join a team.</a>""",
                                         self.env.ref('sales_team.crm_team_action_config').id)
@@ -649,17 +721,17 @@ class Team(models.Model):
 
     def _graph_date_column(self):
         if self.use_opportunities:
-            return 'create_date'
+            return SQL('create_date')
         return super(Team,self)._graph_date_column()
 
     def _graph_y_query(self):
         if self.use_opportunities:
-            return 'count(*)'
+            return SQL('count(*)')
         return super(Team,self)._graph_y_query()
 
     def _extra_sql_conditions(self):
         if self.use_opportunities:
-            return "AND type LIKE 'opportunity'"
+            return SQL("type LIKE 'opportunity'")
         return super(Team,self)._extra_sql_conditions()
 
     def _graph_title_and_key(self):
