@@ -3,13 +3,16 @@
 
 from datetime import datetime
 from freezegun import freeze_time
+from unittest.mock import patch
 
+from odoo import fields
 from odoo.addons.base.tests.test_format_address_mixin import FormatAddressCase
 from odoo.addons.crm.models.crm_lead import PARTNER_FIELDS_TO_SYNC, PARTNER_ADDRESS_FIELDS_TO_SYNC
 from odoo.addons.crm.tests.common import TestCrmCommon, INCOMING_EMAIL
+from odoo.addons.mail.tests.mail_tracking_duration_mixin_case import MailTrackingDurationMixinCase
 from odoo.addons.phone_validation.tools.phone_validation import phone_format
 from odoo.exceptions import UserError
-from odoo.tests.common import Form, tagged, users
+from odoo.tests import Form, tagged, users
 from odoo.tools import mute_logger
 
 
@@ -246,17 +249,19 @@ class TestCRMLead(TestCrmCommon):
 
     @users('user_sales_manager')
     def test_crm_lead_currency_sync(self):
-        lead = self.env['crm.lead'].create({
+        lead_company = self.env['res.company'].sudo().create({
+            'name': 'EUR company',
+            'currency_id': self.env.ref('base.EUR').id,
+        })
+        lead = self.env['crm.lead'].with_company(lead_company).create({
             'name': 'Lead 1',
-            'company_id': self.company_main.id
+            'company_id': lead_company.id
         })
         self.assertEqual(lead.company_currency, self.env.ref('base.EUR'))
 
-        self.company_main.currency_id = self.env.ref('base.CHF')
-        lead.with_company(self.company_main).update({'company_id': False})
+        lead_company.currency_id = self.env.ref('base.CHF')
+        lead.update({'company_id': False})
         self.assertEqual(lead.company_currency, self.env.ref('base.CHF'))
-        #set back original currency
-        self.company_main.currency_id = self.env.ref('base.EUR')
 
     @users('user_sales_manager')
     def test_crm_lead_date_closed(self):
@@ -300,6 +305,36 @@ class TestCRMLead(TestCrmCommon):
         lead.action_set_lost()
         self.assertEqual(lead.date_closed, datetime.now(), "Closed date is updated after marking lead as lost")
 
+    @users('user_sales_manager')
+    def test_crm_lead_meeting_display_fields(self):
+        lead = self.env['crm.lead'].create({'name': 'Lead With Meetings'})
+        meeting_1, meeting_2, meeting_3 = self.env['calendar.event'].create([{
+            'name': 'Meeting 1 of Lead',
+            'opportunity_id': lead.id,
+            'start': '2022-07-12 08:00:00',
+            'stop': '2022-07-12 10:00:00',
+        }, {
+            'name': 'Meeting 2 of Lead',
+            'opportunity_id': lead.id,
+            'start': '2022-07-14 08:00:00',
+            'stop': '2022-07-14 10:00:00',
+        }, {
+            'name': 'Meeting 3 of Lead',
+            'opportunity_id': lead.id,
+            'start': '2022-07-15 08:00:00',
+            'stop': '2022-07-15 10:00:00',
+        }])
+
+        with freeze_time('2022-07-13 11:00:00'):
+            self.assertEqual(lead.meeting_display_date, fields.Date.from_string('2022-07-14'))
+            self.assertEqual(lead.meeting_display_label, 'Next Meeting')
+            (meeting_2 | meeting_3).unlink()
+            self.assertEqual(lead.meeting_display_date, fields.Date.from_string('2022-07-12'))
+            self.assertEqual(lead.meeting_display_label, 'Last Meeting')
+            meeting_1.unlink()
+            self.assertFalse(lead.meeting_display_date)
+            self.assertEqual(lead.meeting_display_label, 'No Meeting')
+
     @users('user_sales_manager')
     def test_crm_lead_partner_sync(self):
         lead, partner = self.lead_1.with_user(self.env.user), self.contact_2
@@ -324,11 +359,13 @@ class TestCRMLead(TestCrmCommon):
         self.assertEqual(lead.email_from, partner_email)
         self.assertEqual(lead.phone, '+1 202 555 6666')
 
-        # resetting lead values also resets partner
+        # resetting lead values should not reset partner: voiding lead info (because
+        # of some reasons) should not prevent from using the contact in other records
         lead.email_from, lead.phone = False, False
-        self.assertFalse(partner.email)
-        self.assertFalse(partner.email_normalized)
-        self.assertFalse(partner.phone)
+        self.assertFalse(lead.email_from)
+        self.assertFalse(lead.phone)
+        self.assertEqual(partner.email, partner_email)
+        self.assertEqual(partner.phone, '+1 202 555 6666')
 
     @users('user_sales_manager')
     def test_crm_lead_partner_sync_email_phone(self):
@@ -345,9 +382,9 @@ class TestCRMLead(TestCrmCommon):
 
             # reset partner phone to a local number and prepare formatted / sanitized values
             partner_phone, partner_mobile = self.test_phone_data[2], self.test_phone_data[1]
-            partner_phone_formatted = phone_format(partner_phone, 'US', '1')
+            partner_phone_formatted = phone_format(partner_phone, 'US', '1', force_format='INTERNATIONAL')
             partner_phone_sanitized = phone_format(partner_phone, 'US', '1', force_format='E164')
-            partner_mobile_formatted = phone_format(partner_mobile, 'US', '1')
+            partner_mobile_formatted = phone_format(partner_mobile, 'US', '1', force_format='INTERNATIONAL')
             partner_mobile_sanitized = phone_format(partner_mobile, 'US', '1', force_format='E164')
             partner_email, partner_email_normalized = self.test_email_data[2], self.test_email_data_normalized[2]
             self.assertEqual(partner_phone_formatted, '+1 202-555-0888')
@@ -404,7 +441,7 @@ class TestCRMLead(TestCrmCommon):
             lead_form.email_from = new_email
             self.assertTrue(lead_form.partner_email_update)
             new_phone = '+1 202 555 7799'
-            new_phone_formatted = phone_format(new_phone, 'US', '1')
+            new_phone_formatted = phone_format(new_phone, 'US', '1', force_format="INTERNATIONAL")
             lead_form.phone = new_phone
             self.assertEqual(lead_form.phone, new_phone_formatted)
             self.assertTrue(lead_form.partner_email_update)
@@ -417,21 +454,22 @@ class TestCRMLead(TestCrmCommon):
 
             # LEAD/PARTNER SYNC: mobile does not update partner
             new_mobile = '+1 202 555 6543'
-            new_mobile_formatted = phone_format(new_mobile, 'US', '1')
+            new_mobile_formatted = phone_format(new_mobile, 'US', '1', force_format="INTERNATIONAL")
             lead_form.mobile = new_mobile
             lead_form.save()
             self.assertEqual(lead.mobile, new_mobile_formatted)
             self.assertEqual(partner.mobile, partner_mobile)
 
-            # LEAD/PARTNER SYNC: reseting lead values also resets partner for email
-            # and phone, but not for mobile
+            # LEAD/PARTNER SYNC: resetting lead values should not reset partner
+            # # voiding lead info (because of some reasons) should not prevent
+            # # from using the contact in other records
             lead_form.email_from, lead_form.phone, lead.mobile = False, False, False
-            self.assertTrue(lead_form.partner_email_update)
-            self.assertTrue(lead_form.partner_phone_update)
+            self.assertFalse(lead_form.partner_email_update)
+            self.assertFalse(lead_form.partner_phone_update)
             lead_form.save()
-            self.assertFalse(partner.email)
-            self.assertFalse(partner.email_normalized)
-            self.assertFalse(partner.phone)
+            self.assertEqual(partner.email, new_email)
+            self.assertEqual(partner.email_normalized, new_email_normalized)
+            self.assertEqual(partner.phone, new_phone_formatted)
             self.assertFalse(lead.phone)
             self.assertFalse(lead.mobile)
             self.assertFalse(lead.phone_sanitized)
@@ -527,11 +565,23 @@ class TestCRMLead(TestCrmCommon):
 
     @users('user_sales_manager')
     def test_crm_lead_stages(self):
-        lead = self.lead_1.with_user(self.env.user)
-        self.assertEqual(lead.team_id, self.sales_team_1)
+        first_now = datetime(2023, 11, 6, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: first_now), \
+             freeze_time(first_now):
+            self.lead_1.write({'date_open': first_now})
 
-        lead.convert_opportunity(self.contact_1)
+        lead = self.lead_1.with_user(self.env.user)
+        self.assertEqual(lead.date_open, first_now)
         self.assertEqual(lead.team_id, self.sales_team_1)
+        self.assertEqual(lead.user_id, self.user_sales_leads)
+
+        second_now = datetime(2023, 11, 8, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: second_now), \
+             freeze_time(second_now):
+            lead.convert_opportunity(self.contact_1)
+        self.assertEqual(lead.date_open, first_now)
+        self.assertEqual(lead.team_id, self.sales_team_1)
+        self.assertEqual(lead.user_id, self.user_sales_leads)
 
         lead.action_set_won()
         self.assertEqual(lead.probability, 100.0)
@@ -558,7 +608,7 @@ class TestCRMLead(TestCrmCommon):
                 'stop': '2022-07-13 10:00:00',
             }
         ])
-        self.assertEqual(lead.calendar_event_count, 1)
+        self.assertEqual(len(lead.calendar_event_ids), 1)
         self.assertEqual(meetings.opportunity_id, lead)
         self.assertEqual(meetings.mapped('res_id'), [lead.id, lead.id])
         self.assertEqual(meetings.mapped('res_model'), ['crm.lead', 'crm.lead'])
@@ -602,6 +652,96 @@ class TestCRMLead(TestCrmCommon):
         self.assertEqual(self.contact_company_1.email, 'broken')
         self.assertEqual(self.contact_company_1.phone, 'alsobroken')
 
+    @users('user_sales_manager')
+    def test_crm_lead_update_dates(self):
+        """ Test date_open / date_last_stage_update update, check those dates
+        are not erased too often """
+        first_now = datetime(2023, 11, 6, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: first_now), \
+             freeze_time(first_now):
+            leads = self.env['crm.lead'].create([
+                {
+                    'email_from': 'testlead@customer.company.com',
+                    'name': 'Lead_1',
+                    'team_id': self.sales_team_1.id,
+                    'type': 'lead',
+                    'user_id': False,
+                }, {
+                    'email_from': 'testopp@customer.company.com',
+                    'name': 'Opp_1',
+                    'type': 'opportunity',
+                    'user_id': self.user_sales_salesman.id,
+                },
+            ])
+            leads.flush_recordset()
+        for lead in leads:
+            self.assertEqual(lead.date_last_stage_update, first_now,
+                             "Stage updated at create time with default value")
+            self.assertEqual(lead.stage_id, self.stage_team1_1)
+            self.assertEqual(lead.team_id, self.sales_team_1)
+        self.assertFalse(leads[0].date_open, "No user -> no assign date")
+        self.assertFalse(leads[0].user_id)
+        self.assertEqual(leads[1].date_open, first_now, "Default user assigned")
+        self.assertEqual(leads[1].user_id, self.user_sales_salesman, "Default user assigned")
+
+        # changing user_id may change team_id / stage_id; update date_open and
+        # maybe date_last_stage_update
+        updated_time = datetime(2023, 11, 23, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: updated_time), \
+             freeze_time(updated_time):
+            leads.write({"user_id": self.user_sales_salesman.id})
+            leads.flush_recordset()
+        for lead in leads:
+            self.assertEqual(lead.stage_id, self.stage_team1_1)
+            self.assertEqual(lead.team_id, self.sales_team_1)
+        self.assertEqual(
+            leads[0].date_last_stage_update, first_now,
+            'Setting same stage when changing user_id, should not update')
+        self.assertEqual(
+            leads[0].date_open, updated_time,
+            'User assigned -> assign date updated')
+        self.assertEqual(
+            leads[1].date_last_stage_update, first_now,
+            'Setting same stage when changing user_id, should not update')
+        self.assertEqual(
+            leads[1].date_open, updated_time,
+            'Should not update date_open, was already the same user_id, but done in batch so ...')
+
+        # set won changes stage -> update date_last_stage_update
+        newer_time = datetime(2023, 11, 26, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: newer_time), \
+             freeze_time(newer_time):
+            leads[1].action_set_won()
+            leads[1].flush_recordset()
+        self.assertEqual(
+            leads[1].date_last_stage_update, newer_time,
+            'Mark as won updates stage hence stage update date')
+        self.assertEqual(leads[1].stage_id, self.stage_gen_won)
+
+        # merge may change user_id and then may change team_id / stage_id; in this
+        # case no real value change is happening
+        last_time = datetime(2023, 11, 29, 8, 0, 0)
+        with patch.object(self.env.cr, 'now', lambda: last_time), \
+             freeze_time(last_time):
+            leads.merge_opportunity(
+                user_id=self.user_sales_salesman.id,
+                auto_unlink=False,
+            )
+            leads.flush_recordset()
+        self.assertEqual(leads[0].date_last_stage_update, first_now)
+        self.assertEqual(leads[0].date_open, updated_time)
+        self.assertEqual(leads[0].stage_id, self.stage_team1_1)
+        self.assertEqual(leads[0].team_id, self.sales_team_1)
+        self.assertEqual(
+            leads[1].date_last_stage_update, newer_time,
+            'Should not rewrite when setting same stage')
+        self.assertEqual(
+            leads[1].date_open, updated_time,
+            'Should not rewrite when setting same user_id')
+        self.assertEqual(leads[1].stage_id, self.stage_gen_won)
+        self.assertEqual(leads[1].team_id, self.sales_team_1)
+        self.assertEqual(leads[1].user_id, self.user_sales_salesman)
+
     @users('user_sales_manager')
     def test_crm_team_alias(self):
         new_team = self.env['crm.team'].create({
@@ -620,12 +760,80 @@ class TestCRMLead(TestCrmCommon):
         # self.assertFalse(new_team.alias_id.alias_name)
         # self.assertFalse(new_team.alias_name)
 
+    @users('user_sales_manager')
+    def test_crm_team_alias_helper(self):
+        """Test that the help message is the right one if we are on multiple team with different settings."""
+        # archive other teams
+        self.env['crm.team'].search([]).active = False
+
+        self._activate_multi_company()
+        team_other_comp = self.team_company2
+
+        user_team_leads, team_leads, user_team_opport, team_opport = self.env['crm.team'].create([{
+            'name': 'UserTeamLeads',
+            'company_id': self.env.company.id,
+            'use_leads': True,
+            'member_ids': [(6, 0, [self.env.user.id])],
+        }, {
+            'name': 'TeamLeads',
+            'company_id': self.env.company.id,
+            'use_leads': True,
+            'member_ids': [],
+        }, {
+            'name': 'UserTeamOpportunities',
+            'company_id': self.env.company.id,
+            'use_leads': False,
+            'member_ids': [(6, 0, [self.env.user.id])],
+        }, {
+            'name': 'TeamOpportunities',
+            'company_id': self.env.company.id,
+            'use_leads': False,
+            'member_ids': [],
+        }])
+
+        self.env['crm.lead'].create([{
+            'name': 'LeadOurTeam',
+            'team_id': user_team_leads.id,
+            'type': 'lead',
+        }, {
+            'name': 'LeadTeam',
+            'team_id': team_leads.id,
+            'type': 'lead',
+        }, {
+            'name': 'OpportunityOurTeam',
+            'team_id': user_team_opport.id,
+            'type': 'opportunity',
+        }, {
+            'name': 'OpportunityTeam',
+            'team_id': team_opport.id,
+            'type': 'opportunity',
+        }])
+        self.env['crm.lead'].with_user(self.user_sales_manager_mc).create({
+            'name': 'LeadOtherComp',
+            'team_id': team_other_comp.id,
+            'type': 'lead',
+            'company_id': self.company_2.id,
+        })
+
+        # archive our team one by one and check that we have the correct help message
+        teams = [user_team_leads, team_leads, user_team_opport, team_opport, team_other_comp]
+        for team in teams:
+            team.alias_id.sudo().write({'alias_name': team.name})
+
+        for team in teams:
+            with self.subTest(team=team):
+                if team != team_other_comp:
+                    self.assertIn(f"<a href='mailto:{team.display_name}'>{team.display_name}</a>", self.env['crm.lead'].sudo().get_empty_list_help(""))
+                else:
+                    self.assertNotIn(f"<a href='mailto:{team.display_name}'>{team.display_name}</a>", self.env['crm.lead'].sudo().get_empty_list_help(""))
+                team.active = False
+
     @mute_logger('odoo.addons.mail.models.mail_thread')
     def test_mailgateway(self):
         new_lead = self.format_and_process(
             INCOMING_EMAIL,
             'unknown.sender@test.example.com',
-            '%s@%s' % (self.sales_team_1.alias_name, self.alias_domain),
+            self.sales_team_1.alias_email,
             subject='Delivery cost inquiry',
             target_model='crm.lead',
         )
@@ -634,49 +842,12 @@ class TestCRMLead(TestCrmCommon):
         self.assertEqual(new_lead.name, 'Delivery cost inquiry')
 
         message = new_lead.with_user(self.user_sales_manager).message_post(
-            body='Here is my offer !',
+            body='Here is my offer!',
             subtype_xmlid='mail.mt_comment')
         self.assertEqual(message.author_id, self.user_sales_manager.partner_id)
 
         new_lead._handle_partner_assignment(create_missing=True)
         self.assertEqual(new_lead.partner_id.email, 'unknown.sender@test.example.com')
-        self.assertEqual(new_lead.partner_id.team_id, self.sales_team_1)
-
-    @users('user_sales_manager')
-    def test_message_get_suggested_recipients(self):
-        """This test checks that creating a contact from a lead with an inactive language will ignore the language
-            while creating a contact from a lead with an active language will take it into account """
-        ResLang = self.env['res.lang'].sudo().with_context(active_test=False)
-
-        # Create a lead with an inactive language -> should ignore the preset language
-        lang_fr = ResLang.search([('code', '=', 'fr_FR')])
-        if not lang_fr:
-            lang_fr = ResLang._create_lang('fr_FR')
-        # set French language as inactive then try to call "_message_get_suggested_recipients"
-        # -> lang code should be ignored
-        lang_fr.active = False
-        lead1 = self.env['crm.lead'].create({
-            'name': 'TestLead',
-            'email_from': self.test_email,
-            'lang_id': lang_fr.id,
-        })
-        data = lead1._message_get_suggested_recipients()[lead1.id]
-        self.assertEqual(data, [(False, self.test_email, None, 'Customer Email')])
-
-        # Create a lead with an active language -> should keep the preset language for recipients
-        lang_en = ResLang.search([('code', '=', 'en_US')])
-        if not lang_en:
-            lang_en = ResLang._create_lang('en_US')
-        # set American English language as active then try to call "_message_get_suggested_recipients"
-        # -> lang code should be kept
-        lang_en.active = True
-        lead2 = self.env['crm.lead'].create({
-            'name': 'TestLead',
-            'email_from': self.test_email,
-            'lang_id': lang_en.id,
-        })
-        data = lead2._message_get_suggested_recipients()[lead2.id]
-        self.assertEqual(data, [(False, self.test_email, "en_US", 'Customer Email')])
 
     @users('user_sales_manager')
     def test_phone_mobile_search(self):
@@ -812,6 +983,26 @@ class TestCRMLead(TestCrmCommon):
         self.assertFalse(lead.phone_sanitized)
 
 
-class TestLeadFormatAddress(FormatAddressCase):
+@tagged('lead_internals')
+class TestLeadFormTools(FormatAddressCase):
+
     def test_address_view(self):
+        self.env.company.country_id = self.env.ref('base.us')
         self.assertAddressView('crm.lead')
+
+
+@tagged('lead_internals')
+class TestCrmLeadMailTrackingDuration(MailTrackingDurationMixinCase):
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass('crm.lead')
+
+    def test_crm_lead_mail_tracking_duration(self):
+        self._test_record_duration_tracking()
+
+    def test_crm_lead_mail_tracking_duration_batch(self):
+        self._test_record_duration_tracking_batch()
+
+    def test_crm_lead_queries_batch_mail_tracking_duration(self):
+        self._test_queries_batch_duration_tracking()
