@@ -4,8 +4,7 @@
 from odoo import SUPERUSER_ID
 from odoo.addons.crm.tests import common as crm_common
 from odoo.fields import Datetime
-from odoo.tests.common import tagged, users
-from odoo.tests.common import Form
+from odoo.tests import Form, tagged, users
 
 @tagged('lead_manage')
 class TestLeadConvertForm(crm_common.TestLeadConvertCommon):
@@ -246,6 +245,27 @@ class TestLeadConvert(crm_common.TestLeadConvertCommon):
         # TDE FIXME: should take into account normalized email version, not encoded one
         # self.assertEqual(convert.partner_id, customer)
 
+    @users('user_sales_manager')
+    def test_lead_convert_no_lang(self):
+        """ Ensure converting a lead with an archived language correctly falls back on the default partner language. """
+        inactive_lang = self.env["res.lang"].sudo().create({
+            'code': 'en_ZZ',
+            'name': 'Inactive Lang',
+            'active': False,
+        })
+
+        lead = self.lead_1.with_user(self.env.user)
+        lead.lang_id = inactive_lang
+
+        convert = self.env['crm.lead2opportunity.partner'].with_context({
+            'active_model': 'crm.lead',
+            'active_id': self.lead_1.id,
+            'active_ids': self.lead_1.ids,
+        }).create({'action': 'create'})
+        convert.action_apply()
+        self.assertTrue(lead.partner_id)
+        self.assertEqual(lead.partner_id.lang, 'en_US')
+
     @users('user_sales_manager')
     def test_lead_convert_internals(self):
         """ Test internals of convert wizard """
@@ -375,19 +395,11 @@ class TestLeadConvert(crm_common.TestLeadConvertCommon):
         }]
         self.lead_1.convert_opportunity(False)
         self.assertEqual(self.lead_1.team_id, initial_team)
-        self.assertEqual(self.lead_1.lead_properties, [{
-            'name': 'test',
-            'type': 'char',
-            'value': 'test value',
-        }])
+        self.assertEqual(self.lead_1.lead_properties, {'test': 'test value'})
 
         # re-writing the team, but keeping the same value should not reset the properties
         self.lead_1.write({'team_id': self.lead_1.team_id.id})
-        self.assertEqual(self.lead_1.lead_properties, [{
-            'name': 'test',
-            'type': 'char',
-            'value': 'test value',
-        }])
+        self.assertEqual(self.lead_1.lead_properties, {'test': 'test value'})
 
     @users('user_sales_manager')
     def test_lead_convert_properties_reset(self):
