@@ -3,4 +3,4 @@
 
 from . import chatbot_script
 from . import chatbot_script_step
-from . import mail_channel
+from . import discuss_channel
