@@ -3,6 +3,7 @@
 
 from . import event_event
 from . import event_mail
+from . import event_mail_registration
 from . import event_registration
 from . import event_stage
 from . import event_tag
@@ -10,3 +11,6 @@ from . import event_ticket
 from . import mail_template
 from . import res_config_settings
 from . import res_partner
+from . import event_question_answer
+from . import event_registration_answer
+from . import event_question
