@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+import json
+
 from odoo import api, fields, models, _
 from odoo.exceptions import ValidationError, UserError
 from odoo.tools.misc import formatLang
@@ -9,7 +11,9 @@ from odoo.tools.misc import formatLang
 class EventTemplateTicket(models.Model):
     _name = 'event.type.ticket'
     _description = 'Event Template Ticket'
+    _order = 'sequence, name, id'
 
+    sequence = fields.Integer('Sequence', default=10)
     # description
     name = fields.Char(
         string='Name', default=lambda self: _('Registration'),
@@ -36,17 +40,18 @@ class EventTemplateTicket(models.Model):
     def _get_event_ticket_fields_whitelist(self):
         """ Whitelist of fields that are copied from event_type_ticket_ids to event_ticket_ids when
         changing the event_type_id field of event.event """
-        return ['name', 'description', 'seats_max']
+        return ['sequence', 'name', 'description', 'seats_max']
 
 
 class EventTicket(models.Model):
-    """ Ticket model allowing to have differnt kind of registrations for a given
+    """ Ticket model allowing to have different kind of registrations for a given
     event. Ticket are based on ticket type as they share some common fields
     and behavior. Those models come from <= v13 Odoo event.event.ticket that
     modeled both concept: tickets for event templates, and tickets for events. """
     _name = 'event.event.ticket'
     _inherit = 'event.type.ticket'
     _description = 'Event Ticket'
+    _order = "event_id, sequence, name, id"
 
     @api.model
     def default_get(self, fields):
@@ -73,10 +78,12 @@ class EventTicket(models.Model):
     # seats
     seats_reserved = fields.Integer(string='Reserved Seats', compute='_compute_seats', store=False)
     seats_available = fields.Integer(string='Available Seats', compute='_compute_seats', store=False)
-    seats_unconfirmed = fields.Integer(string='Unconfirmed Seats', compute='_compute_seats', store=False)
     seats_used = fields.Integer(string='Used Seats', compute='_compute_seats', store=False)
+    seats_taken = fields.Integer(string="Taken Seats", compute="_compute_seats", store=False)
     is_sold_out = fields.Boolean(
         'Sold Out', compute='_compute_is_sold_out', help='Whether seats are not available for this ticket.')
+    # reports
+    color = fields.Char('Color', default="#875A7B")
 
     @api.depends('end_sale_datetime', 'event_id.date_tz')
     def _compute_is_expired(self):
@@ -108,21 +115,20 @@ class EventTicket(models.Model):
 
     @api.depends('seats_max', 'registration_ids.state', 'registration_ids.active')
     def _compute_seats(self):
-        """ Determine reserved, available, reserved but unconfirmed and used seats. """
+        """ Determine available, reserved, used and taken seats. """
         # initialize fields to 0 + compute seats availability
         for ticket in self:
-            ticket.seats_unconfirmed = ticket.seats_reserved = ticket.seats_used = ticket.seats_available = 0
+            ticket.seats_reserved = ticket.seats_used = ticket.seats_available = 0
         # aggregate registrations by ticket and by state
         results = {}
         if self.ids:
             state_field = {
-                'draft': 'seats_unconfirmed',
                 'open': 'seats_reserved',
                 'done': 'seats_used',
             }
             query = """ SELECT event_ticket_id, state, count(event_id)
                         FROM event_registration
-                        WHERE event_ticket_id IN %s AND state IN ('draft', 'open', 'done') AND active = true
+                        WHERE event_ticket_id IN %s AND state IN ('open', 'done') AND active = true
                         GROUP BY event_ticket_id, state
                     """
             self.env['event.registration'].flush_model(['event_id', 'event_ticket_id', 'state', 'active'])
@@ -135,6 +141,7 @@ class EventTicket(models.Model):
             ticket.update(results.get(ticket._origin.id or ticket.id, {}))
             if ticket.seats_max > 0:
                 ticket.seats_available = ticket.seats_max - (ticket.seats_reserved + ticket.seats_used)
+            ticket.seats_taken = ticket.seats_reserved + ticket.seats_used
 
     @api.depends('seats_limited', 'seats_available')
     def _compute_is_sold_out(self):
@@ -160,11 +167,12 @@ class EventTicket(models.Model):
             raise ValidationError(_('There are not enough seats available for:')
                                   + '\n%s\n' % '\n'.join(sold_out_tickets))
 
-    def name_get(self):
+    @api.depends('seats_max', 'seats_available')
+    @api.depends_context('name_with_seats_availability')
+    def _compute_display_name(self):
         """Adds ticket seats availability if requested by context."""
         if not self.env.context.get('name_with_seats_availability'):
-            return super().name_get()
-        res = []
+            return super()._compute_display_name()
         for ticket in self:
             if not ticket.seats_max:
                 name = ticket.name
@@ -176,8 +184,7 @@ class EventTicket(models.Model):
                     ticket_name=ticket.name,
                     count=formatLang(self.env, ticket.seats_available, digits=0),
                 )
-            res.append((ticket.id, name))
-        return res
+            ticket.display_name = name
 
     def _get_ticket_multiline_description(self):
         """ Compute a multiline description of this ticket. It is used when ticket
@@ -195,3 +202,15 @@ class EventTicket(models.Model):
             raise UserError(_(
                 "The following tickets cannot be deleted while they have one or more registrations linked to them:\n- %s",
                 '\n- '.join(self.mapped('name'))))
+
+    def _get_ticket_printing_color(self):
+        self.ensure_one()
+        default_color = '#000000'
+        color_overrides_json = self.env['ir.config_parameter'].sudo().get_param('event.ticket_text_colors')
+        if color_overrides_json:
+            try:
+                color_overrides = json.loads(color_overrides_json)
+                return color_overrides.get(self.name, default_color)
+            except (json.JSONDecodeError, AttributeError):
+                pass
+        return default_color
