@@ -10,6 +10,19 @@ from odoo.tests import HttpCase, tagged
 class TestUi(HttpCase):
 
     def test_event_configurator(self):
+        # self.env["account.chart.template"].try_loading('generic_coa', self.env.company)
+
+        self.env['account.tax.group'].create(
+            {'name': 'Test Account Tax Group', 'company_id': self.env.company.id}
+        )
+
+        self.tax_10 = self.env['account.tax'].sudo().create({
+            'name': 'Tax 10',
+            'amount': 10,
+        })
+
+        self.product_event = self.env.ref('event_product.product_product_event')
+        self.product_event.taxes_id = [(6, 0, self.tax_10.ids)]
 
         event = self.env['event.event'].create({
             'name': 'Design Fair Los Angeles',
@@ -20,10 +33,10 @@ class TestUi(HttpCase):
         self.env['event.event.ticket'].create([{
             'name': 'Standard',
             'event_id': event.id,
-            'product_id': self.env.ref('event_sale.product_product_event').id,
+            'product_id': self.product_event.id,
         }, {
             'name': 'VIP',
             'event_id': event.id,
-            'product_id': self.env.ref('event_sale.product_product_event').id,
+            'product_id': self.product_event.id,
         }])
-        self.start_tour("/web", 'event_configurator_tour', login="admin")
+        self.start_tour("/odoo", 'event_configurator_tour', login="admin")
