@@ -9,14 +9,22 @@ class SmsTemplate(models.Model):
     _inherit = 'sms.template'
 
     @api.model
-    def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
+    def _search_display_name(self, operator, value):
         """Context-based hack to filter reference field in a m2o search box to emulate a domain the ORM currently does not support.
 
         As we can not specify a domain on a reference field, we added a context
         key `filter_template_on_event` on the template reference field. If this
-        key is set, we add our domain in the `args` in the `_name_search`
+        key is set, we add our domain in the `domain` in the `_search_display_name`
         method to filtrate the SMS templates.
         """
+        domain = super()._search_display_name(operator, value)
         if self.env.context.get('filter_template_on_event'):
-            args = expression.AND([[('model', '=', 'event.registration')], args])
-        return super(SmsTemplate, self)._name_search(name, args, operator, limit, name_get_uid)
+            domain = expression.AND([[('model', '=', 'event.registration')], domain])
+        return domain
+
+    def unlink(self):
+        res = super().unlink()
+        domain = ('template_ref', 'in', [f"{template._name},{template.id}" for template in self])
+        self.env['event.mail'].sudo().search([domain]).unlink()
+        self.env['event.type.mail'].sudo().search([domain]).unlink()
+        return res
