@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import fields, models
+from odoo import api, fields, models
 
 
 class FleetVehicleAssignationLog(models.Model):
@@ -13,3 +13,8 @@ class FleetVehicleAssignationLog(models.Model):
     driver_id = fields.Many2one('res.partner', string="Driver", required=True)
     date_start = fields.Date(string="Start Date")
     date_end = fields.Date(string="End Date")
+
+    @api.depends('driver_id', 'vehicle_id')
+    def _compute_display_name(self):
+        for rec in self:
+            rec.display_name = f'{rec.vehicle_id.name} - {rec.driver_id.name}'
