@@ -11,7 +11,7 @@ class FleetVehicleOdometer(models.Model):
 
     name = fields.Char(compute='_compute_vehicle_log_name', store=True)
     date = fields.Date(default=fields.Date.context_today)
-    value = fields.Float('Odometer Value', group_operator="max")
+    value = fields.Float('Odometer Value', aggregator="max")
     vehicle_id = fields.Many2one('fleet.vehicle', 'Vehicle', required=True)
     unit = fields.Selection(related='vehicle_id.odometer_unit', string="Unit", readonly=True)
     driver_id = fields.Many2one(related="vehicle_id.driver_id", string="Driver", readonly=False)
