@@ -7,5 +7,5 @@ from odoo import fields, models
 class ResPartner(models.Model):
     _inherit = 'res.partner'
 
-    plan_to_change_car = fields.Boolean('Plan To Change Car', default=False)
+    plan_to_change_car = fields.Boolean('Plan To Change Car', default=False, tracking=True)
     plan_to_change_bike = fields.Boolean('Plan To Change Bike', default=False)
