@@ -26,8 +26,8 @@ class KarmaRank(models.Model):
 
     @api.depends('user_ids')
     def _compute_rank_users_count(self):
-        requests_data = self.env['res.users']._read_group([('rank_id', '!=', False)], ['rank_id'], ['rank_id'])
-        requests_mapped_data = dict((data['rank_id'][0], data['rank_id_count']) for data in requests_data)
+        requests_data = self.env['res.users']._read_group([('rank_id', '!=', False)], ['rank_id'], ['__count'])
+        requests_mapped_data = {rank.id: count for rank, count in requests_data}
         for rank in self:
             rank.rank_users_count = requests_mapped_data.get(rank.id, 0)
 
