@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from datetime import date
+from datetime import datetime
 from dateutil.relativedelta import relativedelta
+from freezegun import freeze_time
+from itertools import chain, repeat
 from unittest.mock import patch
 
-from odoo import exceptions, fields
+from odoo import exceptions, fields, _
 from odoo.addons.mail.tests.common import mail_new_test_user
 from odoo.tests import common
 
@@ -29,7 +31,9 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         )
         cls.env['gamification.karma.tracking'].search([]).unlink()
 
-        cls.test_date = fields.Date.today() + relativedelta(month=4, day=1)
+        cls.test_date = datetime(2021, 6, 1)
+        cls.first_day_of_test_date_month = '2021-06-01'
+        cls.first_day_of_test_date_next_month = '2021-07-01'
 
     @classmethod
     def _create_trackings(cls, user, karma, steps, track_date, days_delta=1):
@@ -41,7 +45,7 @@ class TestKarmaTrackingCommon(common.TransactionCase):
                 'old_value': old_value,
                 'new_value': new_value,
                 'consolidated': False,
-                'tracking_date': fields.Date.to_string(track_date)
+                'tracking_date': fields.Datetime.to_string(track_date)
             }])
             old_value = new_value
             track_date = track_date + relativedelta(days=days_delta)
@@ -77,25 +81,67 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         results = self.env['res.users']._get_tracking_karma_gain_position([])
         self.assertEqual(len(results), 0)
 
+    @freeze_time('2021-02-02')
     def test_consolidation_cron(self):
-        self.patcher = patch('odoo.addons.gamification.models.gamification_karma_tracking.fields.Date', wraps=fields.Date)
-        self.mock_datetime = self.startPatcher(self.patcher)
-        self.mock_datetime.today.return_value = date(self.test_date.year, self.test_date.month + 1, self.test_date.day)
+        Tracking = self.env['gamification.karma.tracking']
 
-        self._create_trackings(self.test_user, 20, 2, self.test_date, days_delta=30)
-        self._create_trackings(self.test_user_2, 10, 20, self.test_date, days_delta=2)
-        self.env['gamification.karma.tracking']._consolidate_last_month()
-        consolidated = self.env['gamification.karma.tracking'].search([
-            ('user_id', 'in', (self.test_user | self.test_user_2).ids),
-            ('consolidated', '=', True),
-            ('tracking_date', '=', self.test_date)
+        # Sanity check
+        self.assertFalse(Tracking.search_count([('user_id', 'in', (self.test_user | self.test_user_2).ids)]))
+
+        test_date = datetime(2020, 12, 15)
+        first_day_of_test_date_month = '2020-12-01'
+        first_day_of_test_date_next_month = '2021-01-01'
+
+        self._create_trackings(self.test_user, karma=20, steps=2, track_date=test_date, days_delta=30)
+        self._create_trackings(self.test_user_2, karma=10, steps=20, track_date=test_date, days_delta=2)
+
+        # Sanity check
+        self.assertEqual(Tracking.search_count([('user_id', '=', self.test_user.id)]), 2)
+        self.assertEqual(Tracking.search_count([('user_id', '=', self.test_user_2.id)]), 20)
+        self.assertEqual(self.test_user.karma, 40)
+        self.assertEqual(self.test_user_2.karma, 200)
+
+        with self.assertQueryCount(8), patch.object(type(self.env['res.users']), 'write') as patched_user_write:
+            Tracking._consolidate_cron()
+
+        # consolidation should not change user karma
+        self.assertFalse(patched_user_write.called, "User karma didn't change during consolidation, it should not be updated")
+        self.assertEqual(self.test_user.karma, 40)
+        self.assertEqual(self.test_user_2.karma, 200)
+
+        consolidated_1 = Tracking.search([
+            ('user_id', '=', self.test_user.id),
+            ('tracking_date', '>=', first_day_of_test_date_month),
+            ('tracking_date', '<', first_day_of_test_date_next_month),
         ])
-        self.assertEqual(len(consolidated), 2)
-        unconsolidated = self.env['gamification.karma.tracking'].search([
-            ('user_id', 'in', (self.test_user | self.test_user_2).ids),
+        self.assertEqual(len(consolidated_1), 1)
+        self.assertTrue(consolidated_1.consolidated)
+        self.assertEqual(consolidated_1.old_value, 0)
+        self.assertEqual(consolidated_1.new_value, 20)
+        self.assertEqual(consolidated_1.reason, 'Consolidation from 2020-12-01 to 2020-12-31')
+
+        consolidated_2 = Tracking.search([
+            ('user_id', '=', self.test_user_2.id),
+            ('tracking_date', '>=', first_day_of_test_date_month),
+            ('tracking_date', '<', first_day_of_test_date_next_month),
+        ])
+        self.assertEqual(len(consolidated_2), 1)
+        self.assertTrue(consolidated_2.consolidated)
+        self.assertEqual(consolidated_2.old_value, 0)
+        self.assertEqual(consolidated_2.new_value, 10 * 9)  # 9 records have been consolidated
+        self.assertEqual(consolidated_2.reason, 'Consolidation from 2020-12-01 to 2020-12-31')
+
+        unconsolidated_1 = Tracking.search_count([
+            ('user_id', '=', self.test_user.id),
             ('consolidated', '=', False),
         ])
-        self.assertEqual(len(unconsolidated), 6)  # 5 for test user 2, 1 for test user
+        self.assertEqual(unconsolidated_1, 1)
+
+        unconsolidated_2 = Tracking.search_count([
+            ('user_id', '=', self.test_user_2.id),
+            ('consolidated', '=', False),
+        ])
+        self.assertEqual(unconsolidated_2, 11)
 
     def test_consolidation_monthly(self):
         Tracking = self.env['gamification.karma.tracking']
@@ -107,11 +153,12 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         Tracking._process_consolidate(self.test_date)
         consolidated = Tracking.search([
             ('user_id', '=', self.test_user_2.id),
-            ('consolidated', '=', True),
-            ('tracking_date', '=', self.test_date)
+            ('tracking_date', '>=', self.first_day_of_test_date_month),
+            ('tracking_date', '<', self.first_day_of_test_date_next_month),
         ])
         self.assertEqual(len(consolidated), 1)
-        self.assertEqual(consolidated.old_value, base_test_user_2_karma)  # 15 2-days span, from 1 to 29 included = 15 steps -> 150 karma
+        self.assertTrue(consolidated.consolidated)
+        self.assertEqual(consolidated.old_value, base_test_user_2_karma)
         self.assertEqual(consolidated.new_value, base_test_user_2_karma + 150)  # 15 2-days span, from 1 to 29 included = 15 steps -> 150 karma
 
         remaining = Tracking.search([
@@ -130,7 +177,7 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         self.assertEqual(len(consolidated), 2)
         self.assertEqual(consolidated[0].new_value, base_test_user_2_karma + 200)  # 5 remaining 2-days span, from 1 to 9 included = 5 steps -> 50 karma
         self.assertEqual(consolidated[0].old_value, base_test_user_2_karma + 150)  # coming from previous iteration
-        self.assertEqual(consolidated[0].tracking_date, self.test_date + relativedelta(months=1))  # tracking set at beginning of month
+        self.assertEqual(consolidated[0].tracking_date.date(), self.test_date.date() + relativedelta(months=1))  # tracking set at beginning of month
         self.assertEqual(consolidated[-1].new_value, base_test_user_2_karma + 150)  # previously created one still present
         self.assertEqual(consolidated[-1].old_value, base_test_user_2_karma)  # previously created one still present
 
@@ -160,11 +207,17 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         with self.assertRaises(exceptions.AccessError):
             user.read(['karma_tracking_ids'])
 
-        user.write({'karma': 60})
-        user.add_karma(10)
+        user._add_karma(38, source=self.test_user_2)
         self.assertEqual(user.karma, 70)
-        trackings = self.env['gamification.karma.tracking'].sudo().search([('user_id', '=', user.id)])
-        self.assertEqual(len(trackings), 3)  # create + write + add_karma
+        trackings = self.env['gamification.karma.tracking'].sudo().search(
+            [('user_id', '=', user.id)], order="create_date ASC, id ASC")
+        self.assertEqual(len(trackings), 2)  # create + add_karma
+        self.assertEqual(trackings[0].origin_ref, self.test_user)
+        self.assertEqual(trackings[0].reason, "User Creation (Test User #%i)" % self.test_user.id)
+        self.assertEqual(trackings[1].origin_ref, self.test_user_2)
+        self.assertIn("Add Manually", trackings[1].reason)
+        self.assertIn(self.test_user_2.display_name, trackings[1].reason)
+        self.assertIn(str(self.test_user_2.id), trackings[1].reason)
 
     def test_user_tracking(self):
         self.test_user.write({'groups_id': [
@@ -180,17 +233,61 @@ class TestKarmaTrackingCommon(common.TransactionCase):
         self.assertEqual(user.karma_tracking_ids.old_value, 0)
         self.assertEqual(user.karma_tracking_ids.new_value, 32)
 
-        user.write({'karma': 60})
-        user.add_karma(10)
+        user._add_karma(38)
         self.assertEqual(user.karma, 70)
-        self.assertEqual(len(user.karma_tracking_ids), 3)
-        self.assertEqual(user.karma_tracking_ids[2].old_value, 60)
-        self.assertEqual(user.karma_tracking_ids[2].new_value, 70)
+        self.assertEqual(len(user.karma_tracking_ids), 2)
         self.assertEqual(user.karma_tracking_ids[1].old_value, 32)
-        self.assertEqual(user.karma_tracking_ids[1].new_value, 60)
+        self.assertEqual(user.karma_tracking_ids[1].new_value, 70)
+        self.assertIn(_('Add Manually'), user.karma_tracking_ids[1].reason)
+        self.assertIn(self.test_user.display_name, user.karma_tracking_ids[1].reason)
+        self.assertIn(str(self.test_user.id), user.karma_tracking_ids[1].reason)
         self.assertEqual(user.karma_tracking_ids[0].old_value, 0)
         self.assertEqual(user.karma_tracking_ids[0].new_value, 32)
 
+        user._add_karma(69, user, _('Test Reason'))
+        self.assertEqual(len(user.karma_tracking_ids), 3)
+        self.assertIn(_('Test Reason'), user.karma_tracking_ids[2].reason)
+        self.assertEqual(user.karma, 139)
+
+        # add manually karma to a user (e.g. from the technical view)
+        tracking = self.env['gamification.karma.tracking'].create({
+            'user_id': user.id,
+            'new_value': 150,
+            'consolidated': False,
+        })
+        self.assertEqual(tracking.old_value, 139)
+        self.assertEqual(tracking.gain, 11)
+        self.assertEqual(user.karma, 150)
+
+        # write directly on the karma field, should generate <gamification.karma.tracking>
+        self.test_user_2.karma = 100  # won't change
+        last_tracking_3 = self.test_user_2.karma_tracking_ids[-1]
+
+        users = (user | self.test_user | self.test_user_2).with_user(self.test_user)
+        with self.assertQueryCount(11):
+            users.karma = 100
+
+        tracking_1 = user.karma_tracking_ids[-1]
+        tracking_2 = self.test_user.karma_tracking_ids[-1]
+        tracking_3 = self.test_user_2.karma_tracking_ids[-1]
+
+        self.assertEqual(user.karma, 100)
+        self.assertEqual(self.test_user.karma, 100)
+        self.assertEqual(tracking_1.new_value, 100)
+        self.assertEqual(tracking_1.old_value, 150)
+        self.assertEqual(tracking_1.gain, -50)
+        self.assertEqual(tracking_1.reason, "Add Manually (Test User #%i)" % self.test_user.id)
+        self.assertEqual(tracking_1.origin_ref, self.test_user)
+        self.assertEqual(tracking_2.new_value, 100)
+        self.assertEqual(tracking_2.old_value, 0)
+        self.assertEqual(tracking_2.gain, 100)
+        self.assertEqual(tracking_2.reason, "Add Manually (Test User #%i)" % self.test_user.id)
+        self.assertEqual(tracking_2.origin_ref, self.test_user)
+        self.assertEqual(last_tracking_3, tracking_3, "Shouldn't have created a new tracking for the third user")
+        self.assertEqual(tracking_3.new_value, 100)
+        self.assertEqual(tracking_3.old_value, 0)
+        self.assertEqual(tracking_3.gain, 100)
+
 
 class TestComputeRankCommon(common.TransactionCase):
 
@@ -307,3 +404,19 @@ class TestComputeRankCommon(common.TransactionCase):
         with self.assertRaises(Exception):
             self.users[0:50]._recompute_rank()
 
+    def test_get_next_rank(self):
+        """ Test the computation of the next user rank.
+
+        The test is based on the users and ranks defined in the setup ("|" represents rank switches (karma_min)):
+        (user idx, user karma): (0, -1) | (1, 25)...(8, 235) | (9, 265)...(16, 475) | (17, 505)...(33, 985) | (34, 1015)
+        """
+        # user idx, karma:
+        for user, expected_next_rank in chain(
+                ((self.users[0], self.rank_1),),
+                zip(self.users[1:8], repeat(self.rank_2)),
+                zip(self.users[9:16], repeat(self.rank_3)),
+                zip(self.users[17:33], repeat(self.rank_4)),
+                ((self.users[34], self.env['gamification.karma.rank']),),
+        ):
+            user.next_rank_id = False  # Force the computation of the next rank
+            self.assertEqual(user._get_next_rank(), expected_next_rank)
