@@ -6,5 +6,5 @@ from . import google_sync
 from . import calendar
 from . import calendar_recurrence_rule
 from . import res_users
+from . import res_users_settings
 from . import calendar_attendee
-from . import google_credentials
