@@ -7,14 +7,21 @@ from dateutil.relativedelta import relativedelta
 from uuid import uuid4
 
 from odoo import api, fields, models, tools, _
+from odoo.exceptions import ValidationError
 
+from odoo.addons.google_calendar.utils.google_calendar import GoogleCalendarService
 
 class Meeting(models.Model):
     _name = 'calendar.event'
     _inherit = ['calendar.event', 'google.calendar.sync']
 
+    MEET_ROUTE = 'meet.google.com'
+
     google_id = fields.Char(
         'Google Calendar Event Id', compute='_compute_google_id', store=True, readonly=False)
+    guests_readonly = fields.Boolean(
+        'Guests Event Modification Permission', default=False)
+    videocall_source = fields.Selection(selection_add=[('google_meet', 'Google Meet')], ondelete={'google_meet': 'set discuss'})
 
     @api.depends('recurrence_id.google_id')
     def _compute_google_id(self):
@@ -29,10 +36,16 @@ class Meeting(models.Model):
             elif not event.google_id:
                 event.google_id = False
 
+    @api.depends('videocall_location')
+    def _compute_videocall_source(self):
+        events_with_google_url = self.filtered(lambda event: self.MEET_ROUTE in (event.videocall_location or ''))
+        events_with_google_url.videocall_source = 'google_meet'
+        super(Meeting, self - events_with_google_url)._compute_videocall_source()
+
     @api.model
     def _get_google_synced_fields(self):
         return {'name', 'description', 'allday', 'start', 'date_end', 'stop',
-                'attendee_ids', 'alarm_ids', 'location', 'privacy', 'active'}
+                'attendee_ids', 'alarm_ids', 'location', 'privacy', 'active', 'show_as'}
 
     @api.model
     def _restart_google_sync(self):
@@ -48,16 +61,59 @@ class Meeting(models.Model):
             for vals in vals_list
         ])
 
+    @api.model
+    def _check_values_to_sync(self, values):
+        """ Return True if values being updated intersects with Google synced values and False otherwise. """
+        synced_fields = self._get_google_synced_fields()
+        values_to_sync = any(key in synced_fields for key in values)
+        return values_to_sync
+
+    @api.model
+    def _get_update_future_events_values(self):
+        """ Add parameters for updating events within the _update_future_events function scope. """
+        update_future_events_values = super()._get_update_future_events_values()
+        return {**update_future_events_values, 'need_sync': False}
+
+    @api.model
+    def _get_remove_sync_id_values(self):
+        """ Add parameters for removing event synchronization while updating the events in super class. """
+        remove_sync_id_values = super()._get_remove_sync_id_values()
+        return {**remove_sync_id_values, 'google_id': False}
+
+    @api.model
+    def _get_archive_values(self):
+        """ Return the parameters for archiving events. Do not synchronize events after archiving. """
+        archive_values = super()._get_archive_values()
+        return {**archive_values, 'need_sync': False}
+
     def write(self, values):
         recurrence_update_setting = values.get('recurrence_update')
         if recurrence_update_setting in ('all_events', 'future_events') and len(self) == 1:
             values = dict(values, need_sync=False)
         notify_context = self.env.context.get('dont_notify', False)
+        if not notify_context and ([self.env.user.id != record.user_id.id for record in self]):
+            self._check_modify_event_permission(values)
         res = super(Meeting, self.with_context(dont_notify=notify_context)).write(values)
         if recurrence_update_setting in ('all_events',) and len(self) == 1 and values.keys() & self._get_google_synced_fields():
             self.recurrence_id.need_sync = True
         return res
 
+    def _check_modify_event_permission(self, values):
+        # Check if event modification attempt by attendee is valid to avoid duplicate events creation.
+        for event in self:
+            # Edge case: when restarting the synchronization, guests can write 'need_sync=True' on events.
+            google_sync_restart = values.get('need_sync') and len(values)
+            if not google_sync_restart and (event.guests_readonly and self.env.user.id != event.user_id.id):
+                raise ValidationError(_("The following event can only be updated by the organizer "
+                                        "according to the event permissions set on Google Calendar."))
+
+    def _skip_send_mail_status_update(self):
+        """If a google calendar is not syncing with the user, don't send a mail."""
+        user_id = self._get_event_user()
+        if user_id.is_google_calendar_synced() and user_id.res_users_settings_id._is_google_calendar_valid():
+            return True
+        return super()._skip_send_mail_status_update()
+
     def _get_sync_domain(self):
         # in case of full sync, limit to a range of 1y in past and 1y in the future by default
         ICP = self.env['ir.config_parameter'].sudo()
@@ -92,13 +148,17 @@ class Meeting(models.Model):
             'description': google_event.description and tools.html_sanitize(google_event.description),
             'location': google_event.location,
             'user_id': google_event.owner(self.env).id,
-            'privacy': google_event.visibility or self.default_get(['privacy'])['privacy'],
+            'privacy': google_event.visibility or False,
             'attendee_ids': attendee_commands,
             'alarm_ids': alarm_commands,
             'recurrency': google_event.is_recurrent(),
             'videocall_location': google_event.get_meeting_url(),
-            'show_as': 'free' if google_event.is_available() else 'busy'
+            'show_as': 'free' if google_event.is_available() else 'busy',
+            'guests_readonly': not bool(google_event.guestsCanModify)
         }
+        # Remove 'videocall_location' when not sent by Google, otherwise the local videocall will be discarded.
+        if not values.get('videocall_location'):
+            values.pop('videocall_location', False)
         if partner_commands:
             # Add partner_commands only if set from Google. The write method on calendar_events will
             # override attendee commands if the partner_ids command is set but empty.
@@ -121,8 +181,10 @@ class Meeting(models.Model):
             if stop < start:
                 stop = parse(google_event.end.get('date'))
             values['allday'] = True
-        values['start'] = start
-        values['stop'] = stop
+        if related_event['start'] != start:
+            values['start'] = start
+        if related_event['stop'] != stop:
+            values['stop'] = stop
         return values
 
     @api.model
@@ -139,7 +201,8 @@ class Meeting(models.Model):
         emails = [a.get('email') for a in google_attendees]
         existing_attendees = self.env['calendar.attendee']
         if google_event.exists(self.env):
-            existing_attendees = self.browse(google_event.odoo_id(self.env)).attendee_ids
+            event = google_event.get_odoo_event(self.env)
+            existing_attendees = event.attendee_ids
         attendees_by_emails = {tools.email_normalize(a.email): a for a in existing_attendees}
         partners = self._get_sync_partner(emails)
         for attendee in zip(emails, partners, google_attendees):
@@ -209,22 +272,35 @@ class Meeting(models.Model):
                 commands += [(0, 0, {'duration': duration, 'interval': interval, 'name': name, 'alarm_type': alarm_type})]
         return commands
 
+    def action_mass_archive(self, recurrence_update_setting):
+        """ Delete recurrence in Odoo if in 'all_events' or in 'future_events' edge case, triggering one mail. """
+        self.ensure_one()
+        google_service = GoogleCalendarService(self.env['google.service'])
+        archive_future_events = recurrence_update_setting == 'future_events' and self == self.recurrence_id.base_event_id
+        if recurrence_update_setting == 'all_events' or archive_future_events:
+            self.recurrence_id.with_context(is_recurrence=True)._google_delete(google_service, self.recurrence_id.google_id)
+            # Increase performance handling 'future_events' edge case as it was an 'all_events' update.
+            if archive_future_events:
+                recurrence_update_setting = 'all_events'
+        super(Meeting, self).action_mass_archive(recurrence_update_setting)
+
     def _google_values(self):
         if self.allday:
-            start = {'date': self.start_date.isoformat()}
-            end = {'date': (self.stop_date + relativedelta(days=1)).isoformat()}
+            # For all-day events, 'dateTime' must be set to None to indicate that it's an all-day event.
+            # Otherwise, if both 'date' and 'dateTime' are set, Google may not recognize it as an all-day event.
+            start = {'date': self.start_date.isoformat(), 'dateTime': None}
+            end = {'date': (self.stop_date + relativedelta(days=1)).isoformat(), 'dateTime': None}
         else:
-            start = {'dateTime': pytz.utc.localize(self.start).isoformat()}
-            end = {'dateTime': pytz.utc.localize(self.stop).isoformat()}
+            # For timed events, 'date' must be set to None to indicate that it's not an all-day event.
+            # Otherwise, if both 'date' and 'dateTime' are set, Google may not recognize it as a timed event
+            start = {'dateTime': pytz.utc.localize(self.start).isoformat(), 'date': None}
+            end = {'dateTime': pytz.utc.localize(self.stop).isoformat(), 'date': None}
         reminders = [{
             'method': "email" if alarm.alarm_type == "email" else "popup",
             'minutes': alarm.duration_minutes
         } for alarm in self.alarm_ids]
 
         attendees = self.attendee_ids
-        if self.user_id and self.user_id != self.env.user and bool(self.user_id.sudo().google_calendar_token):
-            # We avoid updating the other attendee status if we are not the organizer
-            attendees = self.attendee_ids.filtered(lambda att: att.partner_id == self.env.user.partner_id)
         attendee_values = [{
             'email': attendee.partner_id.email_normalized,
             'responseStatus': attendee.state or 'needsAction',
@@ -236,9 +312,9 @@ class Meeting(models.Model):
             'start': start,
             'end': end,
             'summary': self.name,
-            'description': tools.html2plaintext(self.description) if not tools.is_html_empty(self.description) else '',
+            'description': self._get_customer_description(),
             'location': self.location or '',
-            'guestsCanModify': True,
+            'guestsCanModify': not self.guests_readonly,
             'organizer': {'email': self.user_id.email, 'self': self.user_id == self.env.user},
             'attendees': attendee_values,
             'extendedProperties': {
@@ -251,10 +327,12 @@ class Meeting(models.Model):
                 'useDefault': False,
             }
         }
-        if not self.google_id and not self.videocall_location:
+        if not self.google_id and not self.videocall_location and not self.location:
             values['conferenceData'] = {'createRequest': {'requestId': uuid4().hex}}
         if self.privacy:
             values['visibility'] = self.privacy
+        if self.show_as:
+            values['transparency'] = 'opaque' if self.show_as == 'busy' else 'transparent'
         if not self.active:
             values['status'] = 'cancelled'
         if self.user_id and self.user_id != self.env.user and not bool(self.user_id.sudo().google_calendar_token):
@@ -280,6 +358,15 @@ class Meeting(models.Model):
         # only owner can delete => others refuse the event
         user = self.env.user
         my_cancelled_records = self.filtered(lambda e: e.user_id == user)
+        for event in self:
+            # remove the tracking data to avoid calling _track_template in the pre-commit phase
+            self.env.cr.precommit.data.pop(f'mail.tracking.create.{event._name}.{event.id}', None)
         super(Meeting, my_cancelled_records)._cancel()
         attendees = (self - my_cancelled_records).attendee_ids.filtered(lambda a: a.partner_id == user.partner_id)
         attendees.state = 'declined'
+
+    def _get_event_user(self):
+        self.ensure_one()
+        if self.user_id and self.user_id.sudo().google_calendar_token:
+            return self.user_id
+        return self.env.user
