@@ -29,8 +29,10 @@ class GoogleCalendarService():
         self.google_service = google_service
 
     @requires_auth_token
-    def get_events(self, sync_token=None, token=None, timeout=TIMEOUT):
+    def get_events(self, sync_token=None, token=None, event_id=None, timeout=TIMEOUT):
         url = "/calendar/v3/calendars/primary/events"
+        if event_id:
+            url += f"/{event_id}"
         headers = {'Content-type': 'application/json'}
         params = {'access_token': token}
         if sync_token:
@@ -51,6 +53,11 @@ class GoogleCalendarService():
                 raise InvalidSyncToken("Invalid sync token. Full sync required")
             raise e
 
+        if event_id:
+            next_sync_token = None
+            default_reminders = ()
+            return GoogleEvent([data]), next_sync_token, default_reminders
+
         events = data.get('items', [])
         next_page_token = data.get('nextPageToken')
         while next_page_token:
@@ -65,14 +72,14 @@ class GoogleCalendarService():
         return GoogleEvent(events), next_sync_token, default_reminders
 
     @requires_auth_token
-    def insert(self, values, token=None, timeout=TIMEOUT):
+    def insert(self, values, token=None, timeout=TIMEOUT, need_video_call=True):
         send_updates = self.google_service._context.get('send_updates', True)
-        url = "/calendar/v3/calendars/primary/events?conferenceDataVersion=1&sendUpdates=%s" % ("all" if send_updates else "none")
+        url = "/calendar/v3/calendars/primary/events?conferenceDataVersion=%d&sendUpdates=%s" % (1 if need_video_call else 0, "all" if send_updates else "none")
         headers = {'Content-type': 'application/json', 'Authorization': 'Bearer %s' % token}
         if not values.get('id'):
             values['id'] = uuid4().hex
-        self.google_service._do_request(url, json.dumps(values), headers, method='POST', timeout=timeout)
-        return values['id']
+        _dummy, google_values, _dummy = self.google_service._do_request(url, json.dumps(values), headers, method='POST', timeout=timeout)
+        return google_values
 
     @requires_auth_token
     def patch(self, event_id, values, token=None, timeout=TIMEOUT):
@@ -85,6 +92,12 @@ class GoogleCalendarService():
         url = "/calendar/v3/calendars/primary/events/%s?sendUpdates=all" % event_id
         headers = {'Content-type': 'application/json'}
         params = {'access_token': token}
+        # Delete all events from recurrence in a single request to Google and triggering a single mail.
+        # The 'singleEvents' parameter is a trick that tells Google API to delete all recurrent events individually,
+        # making the deletion be handled entirely on their side, and then we archive the events in Odoo.
+        is_recurrence = self.google_service._context.get('is_recurrence', True)
+        if is_recurrence:
+            params['singleEvents'] = 'true'
         try:
             self.google_service._do_request(url, params, headers=headers, method='DELETE', timeout=timeout)
         except requests.HTTPError as e:
@@ -110,7 +123,8 @@ class GoogleCalendarService():
         state = {
             'd': self.google_service.env.cr.dbname,
             's': 'calendar',
-            'f': from_url
+            'f': from_url,
+            'u': self.google_service.env['ir.config_parameter'].sudo().get_param('database.uuid'),
         }
         base_url = self.google_service._context.get('base_url') or self.google_service.get_base_url()
         return self.google_service._get_authorize_uri(
