@@ -1,18 +1,23 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import hr_employee_base 
+from . import hr_contract_type
+from . import hr_employee_base
 from . import hr_employee
 from . import hr_employee_category
 from . import hr_employee_public
 from . import hr_department
+from . import hr_departure_reason
 from . import hr_job
-from . import hr_plan
+from . import hr_work_location
+from . import mail_activity_plan
+from . import mail_activity_plan_template
 from . import mail_alias
-from . import mail_channel
+from . import discuss_channel
 from . import models
 from . import res_config_settings
-from . import res_partner
 from . import res_users
 from . import res_company
+from . import res_partner
 from . import resource
+from . import ir_ui_menu
