@@ -12,8 +12,7 @@ class DepartureReason(models.Model):
 
     sequence = fields.Integer("Sequence", default=10)
     name = fields.Char(string="Reason", required=True, translate=True)
-    # YTI TODO: Move reason_code to hr + adapt _unlink_except_default_departure_reasons
-    # to use the codes instead of refs
+    reason_code = fields.Integer()
 
     def _get_default_departure_reasons(self):
         return {
@@ -24,6 +23,6 @@ class DepartureReason(models.Model):
 
     @api.ondelete(at_uninstall=False)
     def _unlink_except_default_departure_reasons(self):
-        master_reasons = [self.env.ref('hr.departure_fired', False), self.env.ref('hr.departure_resigned', False), self.env.ref('hr.departure_retired', False)]
-        if any(reason in master_reasons for reason in self):
+        master_departure_codes = self._get_default_departure_reasons().values()
+        if any(reason.reason_code in master_departure_codes for reason in self):
             raise UserError(_('Default departure reasons cannot be deleted.'))
