@@ -1,12 +1,12 @@
 /** @odoo-module **/
 
-import tour from 'web_tour.tour';
+import { registry } from "@web/core/registry";
+import { stepUtils } from "@web_tour/tour_service/tour_utils";
 
-tour.register('hr_employee_tour', {
-    test: true,
-    url: '/web',
-}, [
-    tour.stepUtils.showAppsMenuItem(),
+registry.category("web_tour.tours").add('hr_employee_tour', {
+    url: '/odoo',
+    steps: () => [
+    stepUtils.showAppsMenuItem(),
     {
         content: "Open Employees app",
         trigger: ".o_app[data-menu-xmlid='hr.menu_hr_root']",
@@ -14,16 +14,16 @@ tour.register('hr_employee_tour', {
     },
     {
         content: "Open an Employee Profile",
-        trigger: ".o_kanban_record_title:contains('Johnny H.')",
+        trigger: ".o_kanban_record:contains('Johnny H.')",
         run: 'click',
     },
     {
         content: "Open user account menu",
-        trigger: ".o_user_menu .oe_topbar_name",
+        trigger: ".o_user_menu .dropdown-toggle",
         run: 'click',
     }, {
         content: "Open My Profile",
         trigger: "[data-menu=settings]",
         run: 'click',
     },
-]);
+]});
