@@ -4,6 +4,8 @@
 from . import test_hr_employee
 from . import test_channel
 from . import test_self_user_access
+from . import test_mail_activity_plan
 from . import test_multi_company
 from . import test_resource
 from . import test_ui
+from . import test_scenario
