@@ -102,6 +102,12 @@ class TestSelfAccessProfile(TestHrCommon):
         available_actions = john.get_views([(view.id, 'form')], {'toolbar': True})['views']['form']['toolbar']['action']
         self.assertTrue(any(x['id'] == change_password_action.id for x in available_actions))
 
+    def test_employee_fields_groups(self):
+        # Note: If this tests is crashing, this is probably because the linked field on the error
+        # message is defined on hr.employee only (and not on hr.employee.public) and has no group
+        # defined on it (at least hr.group_hr_user).
+        internal_user = new_test_user(self.env, login='mireille', groups='base.group_user', name='Mireille', email='mireille@example.com')
+        self.env['hr.employee'].with_user(internal_user).search([]).read([])
 
 class TestSelfAccessRights(TestHrCommon):
 
@@ -112,13 +118,12 @@ class TestSelfAccessRights(TestHrCommon):
         cls.richard_emp = cls.env['hr.employee'].create({
             'name': 'Richard',
             'user_id': cls.richard.id,
-            'address_home_id': cls.env['res.partner'].create({'name': 'Richard', 'phone': '21454', 'type': 'private'}).id,
+            'private_phone': '21454',
         })
         cls.hubert = new_test_user(cls.env, login='hub', groups='base.group_user', name='Simple employee', email='hub@example.com')
         cls.hubert_emp = cls.env['hr.employee'].create({
             'name': 'Hubert',
             'user_id': cls.hubert.id,
-            'address_home_id': cls.env['res.partner'].create({'name': 'Hubert', 'type': 'private'}).id,
         })
 
         cls.protected_fields_emp = OrderedDict([(k, v) for k, v in cls.env['hr.employee']._fields.items() if v.groups == 'hr.group_hr_user'])
@@ -170,7 +175,7 @@ class TestSelfAccessRights(TestHrCommon):
         for f, v in self.self_protected_fields_user.items():
             val = None
             if v.type == 'char' or v.type == 'text':
-                val = '0000' if f == 'pin' else 'dummy'
+                val = '0000' if f in ['pin', 'barcode'] else 'dummy'
             if val is not None:
                 self.richard.with_user(self.richard).write({f: val})
 
@@ -180,7 +185,7 @@ class TestSelfAccessRights(TestHrCommon):
         self.env['ir.config_parameter'].set_param('hr.hr_employee_self_edit', False)
         # should not raise
         vals = [
-            {'tz': "Australia/ACT"},
+            {'tz': "Australia/Sydney"},
             {'email': "new@example.com"},
             {'signature': "<p>I'm Richard!</p>"},
             {'notification_type': "email"},
@@ -194,7 +199,7 @@ class TestSelfAccessRights(TestHrCommon):
         # they are in SELF_READABLE_FIELDS
         self.env['ir.config_parameter'].set_param('hr.hr_employee_self_edit', False)
         vals = [
-            {'tz': "Australia/ACT"},
+            {'tz': "Australia/Sydney"},
             {'email': "new@example.com"},
             {'signature': "<p>I'm Richard!</p>"},
             {'notification_type': "email"},
@@ -217,3 +222,17 @@ class TestSelfAccessRights(TestHrCommon):
     def testSearchUserEMployee(self):
         # Searching user based on employee_id field should not raise bad query error
         self.env['res.users'].with_user(self.richard).search([('employee_id', 'ilike', 'Hubert')])
+
+    def test_onchange_readable_fields_with_no_access(self):
+        """
+            The purpose is to test that the onchange logic takes into account `SELF_READABLE_FIELDS`.
+
+            The view contains fields that are in `SELF_READABLE_FIELDS` (example: `private_street`).
+            Even if the user does not have read access to the employee,
+            it should not cause an access error if these fields are in `SELF_READABLE_FIELDS`.
+        """
+        self.env['res.lang']._activate_lang("fr_FR")
+        with Form(self.richard.with_user(self.richard), view='hr.res_users_view_form_profile') as form:
+            # triggering an onchange should not trigger some access error
+            form.lang = "fr_FR"
+            form.tz = "Europe/Brussels"
