@@ -17,19 +17,26 @@ Add all information on the employee form to manage contracts.
 
 You can assign several contracts per employee.
     """,
-    'website': 'https://www.odoo.com/page/employees',
+    'website': 'https://www.odoo.com/app/employees',
     'depends': ['hr'],
     'data': [
         'security/security.xml',
         'security/ir.model.access.csv',
         'data/hr_contract_data.xml',
+        'report/hr_contract_history_report_views.xml',
         'views/hr_contract_views.xml',
-        'views/assets.xml',
+        'views/hr_employee_views.xml',
+        'views/resource_calendar_views.xml',
+        'views/res_config_settings_views.xml',
         'wizard/hr_departure_wizard_views.xml',
     ],
     'demo': ['data/hr_contract_demo.xml'],
     'installable': True,
-    'auto_install': False,
     'application': True,
+    'assets': {
+        'web.assets_backend': [
+            'hr_contract/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
