@@ -1,10 +1,12 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import hr_employee
-from . import hr_employee_public
 from . import hr_contract
+from . import hr_employee
+from . import hr_payroll_structure_type
+from . import ir_ui_menu
+from . import res_company
+from . import res_config_settings
 from . import res_users
 from . import resource
+from . import resource_calendar_leaves
 from . import resource_resource
-from . import hr_payroll_structure_type
