@@ -23,8 +23,9 @@ class Contract(models.Model):
 
     name = fields.Char('Contract Reference', required=True)
     active = fields.Boolean(default=True)
-    structure_type_id = fields.Many2one('hr.payroll.structure.type', string="Salary Structure Type")
-    employee_id = fields.Many2one('hr.employee', string='Employee', tracking=True, domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]")
+    structure_type_id = fields.Many2one('hr.payroll.structure.type', string="Salary Structure Type", compute="_compute_structure_type_id", readonly=False, store=True, tracking=True)
+    employee_id = fields.Many2one('hr.employee', string='Employee', tracking=True, domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]", index=True)
+    active_employee = fields.Boolean(related="employee_id.active", string="Active Employee")
     department_id = fields.Many2one('hr.department', compute='_compute_employee_contract', store=True, readonly=False,
         domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]", string="Department")
     job_id = fields.Many2one('hr.job', compute='_compute_employee_contract', store=True, readonly=False,
@@ -36,9 +37,9 @@ class Contract(models.Model):
         help="End date of the trial period (if there is one).")
     resource_calendar_id = fields.Many2one(
         'resource.calendar', 'Working Schedule', compute='_compute_employee_contract', store=True, readonly=False,
-        default=lambda self: self.env.company.resource_calendar_id.id, copy=False, index=True,
+        default=lambda self: self.env.company.resource_calendar_id.id, copy=False, index=True, tracking=True,
         domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]")
-    wage = fields.Monetary('Wage', required=True, tracking=True, help="Employee's monthly gross wage.")
+    wage = fields.Monetary('Wage', required=True, tracking=True, help="Employee's monthly gross wage.", aggregator="avg")
     contract_wage = fields.Monetary('Contract Wage', compute='_compute_contract_wage')
     notes = fields.Html('Notes')
     state = fields.Selection([
@@ -46,13 +47,14 @@ class Contract(models.Model):
         ('open', 'Running'),
         ('close', 'Expired'),
         ('cancel', 'Cancelled')
-    ], string='Status', group_expand='_expand_states', copy=False,
-       tracking=True, help='Status of the contract', default='draft')
+    ], string='Status', group_expand=True, copy=False,
+        tracking=True, help='Status of the contract', default='draft')
     company_id = fields.Many2one('res.company', compute='_compute_employee_contract', store=True, readonly=False,
         default=lambda self: self.env.company, required=True)
     company_country_id = fields.Many2one('res.country', string="Company country", related='company_id.country_id', readonly=True)
     country_code = fields.Char(related='company_country_id.code', depends=['company_country_id'], readonly=True)
-    contract_type_id = fields.Many2one('hr.contract.type', "Contract Type")
+    contract_type_id = fields.Many2one('hr.contract.type', "Contract Type", tracking=True)
+    contracts_count = fields.Integer(related='employee_id.contracts_count', groups="hr_contract.group_hr_contract_employee_manager")
 
     """
         kanban_state:
@@ -61,14 +63,13 @@ class Contract(models.Model):
             * red = Shows a warning on the employees kanban view
     """
     kanban_state = fields.Selection([
-        ('normal', 'Grey'),
-        ('done', 'Green'),
-        ('blocked', 'Red')
+        ('normal', 'Ongoing'),
+        ('done', 'Ready'),
+        ('blocked', 'Warning')
     ], string='Kanban State', default='normal', tracking=True, copy=False)
     currency_id = fields.Many2one(string="Currency", related='company_id.currency_id', readonly=True)
     permit_no = fields.Char('Work Permit No', related="employee_id.permit_no", readonly=False)
     visa_no = fields.Char('Visa No', related="employee_id.visa_no", readonly=False)
-    visa_expire = fields.Date('Visa Expire Date', related="employee_id.visa_expire", readonly=False)
 
     def _get_hr_responsible_domain(self):
         return "[('share', '=', False), ('company_ids', 'in', company_id), ('groups_id', 'in', %s)]" % self.env.ref('hr.group_hr_user').id
@@ -83,8 +84,9 @@ class Contract(models.Model):
         for contract in self:
             contract.calendar_mismatch = contract.resource_calendar_id != contract.employee_id.resource_calendar_id
 
-    def _expand_states(self, states, domain, order):
-        return [key for key, val in type(self).state.selection]
+    def _get_salary_costs_factor(self):
+        self.ensure_one()
+        return 12.0
 
     @api.depends('employee_id')
     def _compute_employee_contract(self):
@@ -94,23 +96,33 @@ class Contract(models.Model):
             contract.resource_calendar_id = contract.employee_id.resource_calendar_id
             contract.company_id = contract.employee_id.company_id
 
-    @api.onchange('company_id')
-    def _onchange_company_id(self):
-        if self.company_id:
-            structure_types = self.env['hr.payroll.structure.type'].search([
-                '|',
-                ('country_id', '=', self.company_id.country_id.id),
-                ('country_id', '=', False)])
-            if structure_types:
-                self.structure_type_id = structure_types[0]
-            elif self.structure_type_id not in structure_types:
-                self.structure_type_id = False
+    @api.depends('company_id')
+    def _compute_structure_type_id(self):
+
+        default_structure_by_country = {}
+
+        def _default_salary_structure(country_id):
+            default_structure = default_structure_by_country.get(country_id)
+            if default_structure is None:
+                default_structure = default_structure_by_country[country_id] = (
+                    self.env['hr.payroll.structure.type'].search([('country_id', '=', country_id)], limit=1)
+                    or self.env['hr.payroll.structure.type'].search([('country_id', '=', False)], limit=1)
+                )
+            return default_structure
+
+        for contract in self:
+            if not contract.structure_type_id or (contract.structure_type_id.country_id and contract.structure_type_id.country_id != contract.company_id.country_id):
+                contract.structure_type_id = _default_salary_structure(contract.company_id.country_id.id)
 
     @api.onchange('structure_type_id')
     def _onchange_structure_type_id(self):
         default_calendar = self.structure_type_id.default_resource_calendar_id
         if default_calendar and default_calendar.company_id == self.company_id:
-            self.resource_calendar_id = self.structure_type_id.default_resource_calendar_id
+            # If the form was opened from the action_open_contract action,
+            # suggest current employee's calendar for the new contract instead of the default_calendar.
+            if self.env.context.get('from_action_open_contract'):
+                return
+            self.resource_calendar_id = default_calendar
 
     @api.constrains('employee_id', 'state', 'kanban_state', 'date_start', 'date_end')
     def _check_current_contract(self):
@@ -155,31 +167,58 @@ class Contract(models.Model):
     @api.model
     def update_state(self):
         from_cron = 'from_cron' in self.env.context
-        contracts = self.search([
-            ('state', '=', 'open'), ('kanban_state', '!=', 'blocked'),
-            '|',
-            '&',
-            ('date_end', '<=', fields.Date.to_string(date.today() + relativedelta(days=7))),
-            ('date_end', '>=', fields.Date.to_string(date.today() + relativedelta(days=1))),
-            '&',
-            ('visa_expire', '<=', fields.Date.to_string(date.today() + relativedelta(days=60))),
-            ('visa_expire', '>=', fields.Date.to_string(date.today() + relativedelta(days=1))),
-        ])
+        companies = self.env['res.company'].search([])
+        contracts = self.env['hr.contract']
+        work_permit_contracts = self.env['hr.contract']
+        for company in companies:
+            contracts += self.search([
+                ('state', '=', 'open'), ('kanban_state', '!=', 'blocked'), ('company_id', '=', company.id),
+                '&',
+                ('date_end', '<=', fields.date.today() + relativedelta(days=company.contract_expiration_notice_period)),
+                ('date_end', '>=', fields.date.today() + relativedelta(days=1)),
+            ])
+
+            work_permit_contracts += self.search([
+                ('state', '=', 'open'), ('kanban_state', '!=', 'blocked'), ('company_id', '=', company.id),
+                '&',
+                ('employee_id.work_permit_expiration_date', '<=', fields.date.today() + relativedelta(days=company.work_permit_expiration_notice_period)),
+                ('employee_id.work_permit_expiration_date', '>=', fields.date.today() + relativedelta(days=1)),
+            ])
 
         for contract in contracts:
             contract.with_context(mail_activity_quick_update=True).activity_schedule(
                 'mail.mail_activity_data_todo', contract.date_end,
                 _("The contract of %s is about to expire.", contract.employee_id.name),
                 user_id=contract.hr_responsible_id.id or self.env.uid)
+            contract.message_post(
+                body=_(
+                    "According to the contract's end date, this contract has been put in red on the %s. Please advise and correct.",
+                    fields.Date.today()
+                )
+            )
+
+        for contract in work_permit_contracts:
+            contract.with_context(mail_activity_quick_update=True).activity_schedule(
+                'mail.mail_activity_data_todo', contract.date_end,
+                _("The work permit of %s is about to expire.", contract.employee_id.name),
+                user_id=contract.hr_responsible_id.id or self.env.uid)
+            contract.message_post(
+                body=_(
+                    "According to Employee's Working Permit Expiration Date, this contract has been put in red on the %s. Please advise and correct.",
+                    fields.Date.today()
+                )
+            )
 
         if contracts:
             contracts._safe_write_for_cron({'kanban_state': 'blocked'}, from_cron)
+        if work_permit_contracts:
+            work_permit_contracts._safe_write_for_cron({'kanban_state': 'blocked'}, from_cron)
 
         contracts_to_close = self.search([
             ('state', '=', 'open'),
             '|',
             ('date_end', '<=', fields.Date.to_string(date.today())),
-            ('visa_expire', '<=', fields.Date.to_string(date.today())),
+            ('employee_id.work_permit_expiration_date', '<=', fields.Date.to_string(date.today())),
         ])
 
         if contracts_to_close:
@@ -226,9 +265,19 @@ class Contract(models.Model):
         else:
             self.write(vals)
 
+    def _get_employee_vals_to_update(self):
+        self.ensure_one()
+        vals = {'contract_id': self.id}
+        if self.job_id and self.job_id != self.employee_id.job_id:
+            vals['job_id'] = self.job_id.id
+        if self.department_id:
+            vals['department_id'] = self.department_id.id
+        return vals
+
     def _assign_open_contract(self):
         for contract in self:
-            contract.employee_id.sudo().write({'contract_id': contract.id})
+            vals = contract._get_employee_vals_to_update()
+            contract.employee_id.sudo().write(vals)
 
     @api.depends('wage')
     def _compute_contract_wage(self):
@@ -242,8 +291,14 @@ class Contract(models.Model):
         return self[self._get_contract_wage_field()]
 
     def _get_contract_wage_field(self):
+        self.ensure_one()
         return 'wage'
 
+    def _is_fully_flexible(self):
+        """ return True if contract has a fully flexible working calendar """
+        self.ensure_one()
+        return not self.resource_calendar_id
+
     def write(self, vals):
         old_state = {c.id: c.state for c in self}
         res = super(Contract, self).write(vals)
@@ -252,7 +307,7 @@ class Contract(models.Model):
             self._assign_open_contract()
         today = fields.Date.today()
         for contract in self:
-            if contract == contract.employee_id.contract_id \
+            if contract == contract.sudo().employee_id.contract_id \
                 and old_state[contract.id] == 'open' \
                 and new_state[contract.id] != 'open':
                 running_contract = self.env['hr.contract'].search([
@@ -261,7 +316,7 @@ class Contract(models.Model):
                     ('state', '=', 'open'),
                 ]).filtered(lambda c: c.date_start <= today and (not c.date_end or c.date_end >= today))
                 if running_contract:
-                    contract.employee_id.contract_id = running_contract[0]
+                    contract.employee_id.sudo().contract_id = running_contract[0]
         if vals.get('state') == 'close':
             for contract in self.filtered(lambda c: not c.date_end):
                 contract.date_end = max(date.today(), contract.date_start)
@@ -270,9 +325,11 @@ class Contract(models.Model):
             for contract in self.filtered(lambda c: c.state == 'open'):
                 contract.state = 'close'
 
-        calendar = vals.get('resource_calendar_id')
-        if calendar:
-            self.filtered(lambda c: c.state == 'open' or (c.state == 'draft' and c.kanban_state == 'done')).mapped('employee_id').write({'resource_calendar_id': calendar})
+        if 'resource_calendar_id' in vals:
+            calendar = vals['resource_calendar_id']
+            self.filtered(
+                lambda c: c.state == 'open' or (c.state == 'draft' and c.kanban_state == 'done' and c.employee_id.contracts_count == 1)
+            ).employee_id.resource_calendar_id = calendar
 
         if 'state' in vals and 'kanban_state' not in vals:
             self.write({'kanban_state': 'normal'})
@@ -283,9 +340,11 @@ class Contract(models.Model):
     def create(self, vals_list):
         contracts = super().create(vals_list)
         contracts.filtered(lambda c: c.state == 'open')._assign_open_contract()
-        open_contracts = contracts.filtered(lambda c: c.state == 'open' or c.state == 'draft' and c.kanban_state == 'done')
+        open_contracts = contracts.filtered(
+            lambda c: c.state == 'open' or (c.state == 'draft' and c.kanban_state == 'done' and c.employee_id.contracts_count == 1)
+        )
         # sync contract calendar -> calendar employee
-        for contract in open_contracts.filtered(lambda c: c.employee_id and c.resource_calendar_id):
+        for contract in open_contracts.filtered(lambda c: c.employee_id):
             contract.employee_id.resource_calendar_id = contract.resource_calendar_id
         return contracts
 
@@ -297,6 +356,11 @@ class Contract(models.Model):
             return self.env.ref('hr_contract.mt_contract_close')
         return super(Contract, self)._track_subtype(init_values)
 
+    def _is_struct_from_country(self, country_code):
+        self.ensure_one()
+        self_sudo = self.sudo()
+        return self_sudo.structure_type_id and self_sudo.structure_type_id.country_id.code == country_code
+
     def action_open_contract_form(self):
         self.ensure_one()
         action = self.env['ir.actions.actions']._for_xml_id('hr_contract.action_hr_contract')
@@ -307,3 +371,17 @@ class Contract(models.Model):
             'res_id': self.id,
         })
         return action
+
+    def action_open_contract_history(self):
+        self.ensure_one()
+        action = self.env["ir.actions.actions"]._for_xml_id('hr_contract.hr_contract_history_view_form_action')
+        action['res_id'] = self.employee_id.id
+        return action
+
+    def action_open_contract_list(self):
+        self.ensure_one()
+        action = self.env["ir.actions.actions"]._for_xml_id('hr_contract.action_hr_contract')
+        action.update({'domain': [('employee_id', '=', self.employee_id.id)],
+                      'views':  [[False, 'list'], [False, 'kanban'], [False, 'activity'], [False, 'form']],
+                       'context': {'default_employee_id': self.employee_id.id}})
+        return action
