@@ -8,8 +8,8 @@ class HrDepartment(models.Model):
     _inherit = 'hr.department'
 
     def _compute_expense_sheets_to_approve(self):
-        expense_sheet_data = self.env['hr.expense.sheet']._read_group([('department_id', 'in', self.ids), ('state', '=', 'submit')], ['department_id'], ['department_id'])
-        result = dict((data['department_id'][0], data['department_id_count']) for data in expense_sheet_data)
+        expense_sheet_data = self.env['hr.expense.sheet']._read_group([('department_id', 'in', self.ids), ('state', '=', 'submit')], ['department_id'], ['__count'])
+        result = {department.id: count for department, count in expense_sheet_data}
         for department in self:
             department.expense_sheets_to_approve_count = result.get(department.id, 0)
 
