@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import datetime
+from datetime import datetime, timezone
 from dateutil.relativedelta import relativedelta
 
 from odoo import api, fields, models
+from odoo.osv import expression
+import ast
 
 
 class Department(models.Model):
@@ -21,26 +22,26 @@ class Department(models.Model):
     def _compute_leave_count(self):
         Requests = self.env['hr.leave']
         Allocations = self.env['hr.leave.allocation']
-        today_date = datetime.datetime.utcnow().date()
+        today_date = datetime.now(timezone.utc).date()
         today_start = fields.Datetime.to_string(today_date)  # get the midnight of the current utc day
         today_end = fields.Datetime.to_string(today_date + relativedelta(hours=23, minutes=59, seconds=59))
 
         leave_data = Requests._read_group(
             [('department_id', 'in', self.ids),
              ('state', '=', 'confirm')],
-            ['department_id'], ['department_id'])
+            ['department_id'], ['__count'])
         allocation_data = Allocations._read_group(
             [('department_id', 'in', self.ids),
              ('state', '=', 'confirm')],
-            ['department_id'], ['department_id'])
+            ['department_id'], ['__count'])
         absence_data = Requests._read_group(
-            [('department_id', 'in', self.ids), ('state', 'not in', ['cancel', 'refuse']),
+            [('department_id', 'in', self.ids), ('state', '=', 'validate'),
              ('date_from', '<=', today_end), ('date_to', '>=', today_start)],
-            ['department_id'], ['department_id'])
+            ['department_id'], ['__count'])
 
-        res_leave = dict((data['department_id'][0], data['department_id_count']) for data in leave_data)
-        res_allocation = dict((data['department_id'][0], data['department_id_count']) for data in allocation_data)
-        res_absence = dict((data['department_id'][0], data['department_id_count']) for data in absence_data)
+        res_leave = {department.id: count for department, count in leave_data}
+        res_allocation = {department.id: count for department, count in allocation_data}
+        res_absence = {department.id: count for department, count in absence_data}
 
         for department in self:
             department.leave_to_approve_count = res_leave.get(department.id, 0)
@@ -53,6 +54,7 @@ class Department(models.Model):
             'search_default_active_employee': 2,
             'search_default_department_id': self.id,
             'default_department_id': self.id,
+            'searchpanel_default_department_id': self.id,
         }
 
     def action_open_leave_department(self):
@@ -61,7 +63,7 @@ class Department(models.Model):
             **self._get_action_context(),
             'search_default_active_time_off': 3,
             'hide_employee_name': 1,
-            'holiday_status_name_get': False
+            'holiday_status_display_name': False
         }
         return action
 
@@ -69,4 +71,5 @@ class Department(models.Model):
         action = self.env["ir.actions.actions"]._for_xml_id("hr_holidays.hr_leave_allocation_action_approve_department")
         action['context'] = self._get_action_context()
         action['context']['search_default_second_approval'] = 3
+        action['domain'] = expression.AND([ast.literal_eval(action['domain']), [('state', '=', 'confirm')]])
         return action
