@@ -2,14 +2,13 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import fields, models, _
-from odoo.exceptions import ValidationError
 
 
 class HrHolidaysCancelLeave(models.TransientModel):
     _name = 'hr.holidays.cancel.leave'
-    _description = 'Cancel Leave Wizard'
+    _description = 'Cancel Time Off Wizard'
 
-    leave_id = fields.Many2one('hr.leave', required=True)
+    leave_id = fields.Many2one('hr.leave', string="Time Off Request", required=True)
     reason = fields.Text(required=True)
 
     def action_cancel_leave(self):
@@ -22,7 +21,7 @@ class HrHolidaysCancelLeave(models.TransientModel):
             'tag': 'display_notification',
             'params': {
                 'type': 'success',
-                'message': _("Your time off has been canceled."),
+                'message': _("Your time off has been cancelled."),
                 'next': {'type': 'ir.actions.act_window_close'},
             }
         }
