@@ -3,7 +3,7 @@
 
 from . import hr_attendance
 from . import hr_leave_allocation
+from . import hr_leave_accrual_plan_level
 from . import hr_leave_type
 from . import hr_leave
-from . import res_company
 from . import res_users
