@@ -6,15 +6,15 @@
     'sequence': 125,
     'category': 'Human Resources',
     'description': """
-        Bridge between HR and Maintenance.""",
+Bridge between HR and Maintenance.""",
     'depends': ['hr', 'maintenance'],
-    'summary': 'Equipments, Assets, Internal Hardware, Allocation Tracking',
+    'summary': 'Equipment, Assets, Internal Hardware, Allocation Tracking',
     'data': [
         'security/equipment.xml',
         'views/maintenance_views.xml',
         'views/hr_views.xml',
+        'wizard/hr_departure_wizard_views.xml',
     ],
-    'demo': [],
     'installable': True,
     'auto_install': True,
     'license': 'LGPL-3',
