@@ -1,11 +1,12 @@
 /** @odoo-module */
 
-import { CharField } from "@web/views/fields/char/char_field";
+import { CharField, charField } from "@web/views/fields/char/char_field";
 import { registry } from "@web/core/registry";
 
 import { useService } from "@web/core/utils/hooks";
 
 export class ApplicantCharField extends CharField {
+    static template = "hr_recruitment.ApplicantCharField";
     setup() {
         super.setup();
 
@@ -26,5 +27,10 @@ export class ApplicantCharField extends CharField {
         }
     }
 }
-ApplicantCharField.template = "hr_recruitment.ApplicantCharField";
-registry.category("fields").add("applicant_char", ApplicantCharField);
+
+export const applicantCharField = {
+    ...charField,
+    component: ApplicantCharField,
+};
+
+registry.category("fields").add("applicant_char", applicantCharField);
