@@ -7,18 +7,24 @@
     'sequence': 270,
     'version': '1.0',
     'summary': 'Manage skills of your employees',
-    'description': """""",
     'depends': ['hr_skills', 'hr_recruitment'],
     'data': [
         'security/hr_recruitment_skills_security.xml',
         'views/hr_applicant_views.xml',
-        'views/hr_applicant_skill_views.xml',
+        'views/hr_candidate_views.xml',
+        'views/hr_candidate_skill_views.xml',
+        'views/hr_job_views.xml',
         'security/ir.model.access.csv',
     ],
+    'assets': {
+        'web.assets_backend': [
+            'hr_recruitment_skills/static/src/**/*',
+        ],
+    },
     'demo': [
+        'data/hr_recruitment_skills_demo.xml',
     ],
     'installable': True,
-    'application': False,
     'auto_install': True,
     'license': 'LGPL-3',
 }
