@@ -17,12 +17,18 @@ This module adds certification to resume for employees.
     'data': [
         'views/hr_templates.xml',
         'data/hr_resume_data.xml',
+        'views/hr_employee_certification_views.xml',
+        'views/survey_survey_views.xml',
     ],
     'auto_install': True,
     'assets': {
         'web.assets_backend': [
+            'hr_skills_survey/static/src/fields/**/*',
             'hr_skills_survey/static/src/xml/**/*',
         ],
     },
+    'demo': [
+        'data/hr_resume_demo.xml',
+    ],
     'license': 'LGPL-3',
 }
