@@ -1,11 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import analytic_applicability
+from . import hr_employee
 from . import hr_timesheet
 from . import ir_http
 from . import ir_ui_menu
 from . import res_company
 from . import res_config_settings
-from . import project
+from . import project_project
+from . import project_task
+from . import project_update
 from . import project_collaborator
-from . import uom
+from . import uom_uom
