@@ -9,7 +9,7 @@ from odoo.addons.web.controllers.webclient import WebClient
 
 class Routing(Home):
 
-    @http.route('/website/translations/<string:unique>', type='http', auth="public", website=True)
+    @http.route('/website/translations/<string:unique>', type='http', auth="public", website=True, readonly=True)
     def get_website_translations(self, unique, lang=None, mods=None):
         IrHttp = request.env['ir.http'].sudo()
         modules = IrHttp.get_translation_frontend_modules()
@@ -20,6 +20,6 @@ class Routing(Home):
 
 class SessionWebsite(Session):
 
-    @http.route('/web/session/logout', type='http', auth="none", website=True, multilang=False, sitemap=False)
-    def logout(self, redirect='/web'):
+    @http.route('/web/session/logout', website=True, multilang=False, sitemap=False)
+    def logout(self, redirect='/odoo'):
         return super().logout(redirect=redirect)
