@@ -2,14 +2,17 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from base64 import b64decode
+from datetime import datetime
 import json
 import logging
 import os
 import subprocess
+from socket import gethostname
 import time
+from werkzeug.exceptions import InternalServerError
+from zlib import adler32
 
 from odoo import http, tools
-from odoo.modules.module import get_resource_path
 
 from odoo.addons.hw_drivers.event_manager import event_manager
 from odoo.addons.hw_drivers.main import iot_devices, manager
@@ -30,6 +33,15 @@ class DriverController(http.Controller):
         if iot_device:
             iot_device.data['owner'] = session_id
             data = json.loads(data)
+
+            # Skip the request if it was already executed (duplicated action calls)
+            iot_idempotent_id = data.get("iot_idempotent_id")
+            if iot_idempotent_id:
+                idempotent_session = iot_device._check_idempotency(iot_idempotent_id, session_id)
+                if idempotent_session:
+                    _logger.info("Ignored request from %s as iot_idempotent_id %s already received from session %s",
+                                 session_id, iot_idempotent_id, idempotent_session)
+                    return False
             iot_device.action(data)
             return True
         return False
@@ -38,9 +50,9 @@ class DriverController(http.Controller):
     def check_certificate(self):
         """
         This route is called when we want to check if certificate is up-to-date
-        Used in cron.daily
+        Used in iot-box cron.daily, deprecated since image 24_10 but needed for compatibility with the image 24_01
         """
-        helpers.check_certificate()
+        helpers.get_certificate_status()
 
     @http.route('/hw_drivers/event', type='json', auth='none', cors='*', csrf=False, save_session=False)
     def event(self, listener):
@@ -70,7 +82,25 @@ class DriverController(http.Controller):
         """
         Downloads the log file
         """
-        if tools.config['logfile']:
-            res = http.send_file(tools.config['logfile'], mimetype="text/plain", as_attachment=True)
-            res.headers['Cache-Control'] = 'no-cache'
-            return res
+        log_path = tools.config['logfile']
+        if not log_path:
+            raise InternalServerError("Log file configuration is not set")
+        try:
+            stat = os.stat(log_path)
+        except FileNotFoundError:
+            raise InternalServerError("Log file has not been found")
+        check = adler32(log_path.encode())
+        log_file_name = f"iot-odoo-{gethostname()}-{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}.log"
+        # intentionally don't use Stream.from_path as the path used is not in the addons path
+        # for instance, for the iot-box it will be in /var/log/odoo
+        return http.Stream(
+                type='path',
+                path=log_path,
+                download_name=log_file_name,
+                etag=f'{int(stat.st_mtime)}-{stat.st_size}-{check}',
+                last_modified=stat.st_mtime,
+                size=stat.st_size,
+                mimetype='text/plain',
+            ).get_response(
+            mimetype='text/plain', as_attachment=True
+        )
