@@ -4,6 +4,7 @@
 from threading import Thread, Event
 
 from odoo.addons.hw_drivers.main import drivers, iot_devices
+from odoo.tools.lru import LRU
 
 
 class DriverMetaClass(type):
@@ -35,6 +36,9 @@ class Driver(Thread, metaclass=DriverMetaClass):
         self._actions = {}
         self._stopped = Event()
 
+        # Least Recently Used (LRU) Cache that will store the idempotent keys already seen.
+        self._iot_idempotent_ids_cache = LRU(500)
+
     @classmethod
     def supported(cls, device):
         """
@@ -54,3 +58,18 @@ class Driver(Thread, metaclass=DriverMetaClass):
     def disconnect(self):
         self._stopped.set()
         del iot_devices[self.device_identifier]
+
+    def _check_idempotency(self, iot_idempotent_id, session_id):
+        """
+        Some IoT requests for the same action might be received several times.
+        To avoid duplicating the resulting actions, we check if the action was "recently" executed.
+        If this is the case, we will simply ignore the action
+
+        :return: the `session_id` of the same `iot_idempotent_id` if any. False otherwise,
+        which means that it is the first time that the IoT box received the request with this ID
+        """
+        cache = self._iot_idempotent_ids_cache
+        if iot_idempotent_id in cache:
+            return cache[iot_idempotent_id]
+        cache[iot_idempotent_id] = session_id
+        return False
