@@ -4,15 +4,21 @@ from traceback import format_exc
 import json
 import platform
 import logging
-import socket
 from threading import Thread
 import time
 import urllib3
 
 from odoo.addons.hw_drivers.tools import helpers
+from odoo.addons.hw_drivers.websocket_client import WebsocketClient
 
 _logger = logging.getLogger(__name__)
 
+try:
+    import schedule
+except ImportError:
+    schedule = None
+    _logger.warning('Could not import library schedule')
+
 try:
     from dbus.mainloop.glib import DBusGMainLoop
 except ImportError:
@@ -25,23 +31,24 @@ iot_devices = {}
 
 
 class Manager(Thread):
-    def send_alldevices(self):
+    server_url = None
+
+    def send_alldevices(self, iot_client=None):
         """
-        This method send IoT Box and devices informations to Odoo database
+        This method send IoT Box and devices information to Odoo database
         """
-        server = helpers.get_odoo_server_url()
-        if server:
-            subject = helpers.read_file_first_line('odoo-subject.conf')
+        if self.server_url:
+            subject = helpers.get_conf('subject')
             if subject:
                 domain = helpers.get_ip().replace('.', '-') + subject.strip('*')
             else:
                 domain = helpers.get_ip()
             iot_box = {
-                'name': socket.gethostname(),
+                'name': helpers.get_hostname(),
                 'identifier': helpers.get_mac_address(),
                 'ip': domain,
                 'token': helpers.get_token(),
-                'version': helpers.get_version(),
+                'version': helpers.get_version(detailed_version=True),
             }
             devices_list = {}
             for device in iot_devices:
@@ -51,43 +58,57 @@ class Manager(Thread):
                     'type': iot_devices[device].device_type,
                     'manufacturer': iot_devices[device].device_manufacturer,
                     'connection': iot_devices[device].device_connection,
+                    'subtype': iot_devices[device].device_subtype if iot_devices[device].device_type == 'printer' else '',
                 }
-            data = {'params': {'iot_box': iot_box, 'devices': devices_list,}}
+            devices_list_to_send = {
+                key: value for key, value in devices_list.items() if key != 'distant_display'
+            }
+            data = {
+                'params': {
+                    'iot_box': iot_box,
+                    'devices': devices_list_to_send,
+                }  # Don't send distant_display to the db
+            }
             # disable certifiacte verification
             urllib3.disable_warnings()
             http = urllib3.PoolManager(cert_reqs='CERT_NONE')
             try:
-                http.request(
+                resp = http.request(
                     'POST',
-                    server + "/iot/setup",
+                    self.server_url + "/iot/setup",
                     body=json.dumps(data).encode('utf8'),
                     headers={
                         'Content-type': 'application/json',
                         'Accept': 'text/plain',
                     },
                 )
-            except Exception as e:
-                _logger.error('Could not reach configured server')
-                _logger.error('A error encountered : %s ' % e)
+                if iot_client:
+                    iot_client.iot_channel = json.loads(resp.data).get('result', '')
+            except json.decoder.JSONDecodeError:
+                _logger.exception('Could not load JSON data: Received data is not in valid JSON format\ncontent:\n%s', resp.data)
+            except Exception:
+                _logger.exception('Could not reach configured server')
         else:
-            _logger.warning('Odoo server not set')
+            _logger.info('Ignoring sending the devices to the database: no associated database')
 
     def run(self):
-        """
-        Thread that will load interfaces and drivers and contact the odoo server with the updates
-        """
-
+        """Thread that will load interfaces and drivers and contact the odoo server with the updates"""
+        self.server_url = helpers.get_odoo_server_url()
         helpers.start_nginx_server()
-        if platform.system() == 'Linux':
+
+        _logger.info("IoT Box Image version: %s", helpers.get_version(detailed_version=True))
+        if platform.system() == 'Linux' and self.server_url:
             helpers.check_git_branch()
+            helpers.generate_password()
         is_certificate_ok, certificate_details = helpers.get_certificate_status()
-        if not is_certificate_ok:
+        if not is_certificate_ok and certificate_details != 'ERR_IOT_HTTPS_CHECK_NO_SERVER':
             _logger.warning("An error happened when trying to get the HTTPS certificate: %s",
                             certificate_details)
 
+        iot_client = self.server_url and WebsocketClient(self.server_url)
         # We first add the IoT Box to the connected DB because IoT handlers cannot be downloaded if
         # the identifier of the Box is not found in the DB. So add the Box to the DB.
-        self.send_alldevices()
+        self.send_alldevices(iot_client)
         helpers.download_iot_handlers()
         helpers.load_iot_handlers()
 
@@ -100,15 +121,23 @@ class Manager(Thread):
             except Exception as e:
                 _logger.error("Error in %s: %s", str(interface), e)
 
-        # Check every 3 secondes if the list of connected devices has changed and send the updated
+        # Set scheduled actions
+        schedule and schedule.every().day.at("00:00").do(helpers.get_certificate_status)
+        schedule and schedule.every().day.at("00:00").do(helpers.reset_log_level)
+
+        # Set up the websocket connection
+        if self.server_url and iot_client.iot_channel:
+            iot_client.start()
+        # Check every 3 seconds if the list of connected devices has changed and send the updated
         # list to the connected DB.
         self.previous_iot_devices = []
         while 1:
             try:
                 if iot_devices != self.previous_iot_devices:
                     self.previous_iot_devices = iot_devices.copy()
-                    self.send_alldevices()
+                    self.send_alldevices(iot_client)
                 time.sleep(3)
+                schedule and schedule.run_pending()
             except Exception:
                 # No matter what goes wrong, the Manager loop needs to keep running
                 _logger.error(format_exc())
