@@ -19,6 +19,11 @@ If you activate this module, you won't be able to access the
 regular Odoo interface anymore.
 
 """,
+    'assets': {
+        'web.assets_backend': [
+            'hw_posbox_homepage/static/**/*',
+        ],
+    },
     'installable': False,
     'license': 'LGPL-3',
 }
