@@ -11,8 +11,13 @@ class AccountMoveLine(models.Model):
         invoice = self.move_id
         included_taxes = self.tax_ids.filtered('tax_group_id.l10n_ar_vat_afip_code') if self.move_id._l10n_ar_include_vat() else False
         if not included_taxes:
-            price_unit = self.tax_ids.with_context(round=False).compute_all(
-                self.price_unit, invoice.currency_id, 1.0, self.product_id, invoice.partner_id)
+            price_unit = self.tax_ids.compute_all(
+                self.price_unit,
+                currency=invoice.currency_id,
+                product=self.product_id,
+                partner=invoice.partner_id,
+                rounding_method="round_globally",
+            )
             price_unit = price_unit['total_excluded']
             price_subtotal = self.price_subtotal
         else:
