@@ -6,4 +6,4 @@ class Uom(models.Model):
 
     _inherit = 'uom.uom'
 
-    l10n_ar_afip_code = fields.Char('AFIP Code', help='This code will be used on electronic invoice')
+    l10n_ar_afip_code = fields.Char('Code', help='Argentina: This code will be used on electronic invoice.')
