@@ -4,7 +4,7 @@
     'version': '1.0',
     'category': 'Accounting/Localizations/Website',
     'sequence': 14,
-    'author': 'Odoo, ADHOC SA',
+    'author': 'Odoo S.A., ADHOC SA',
     'description': """Be able to see Identification Type and AFIP Responsibility in ecommerce checkout form.""",
     'depends': [
         'website_sale',
