@@ -1,4 +1,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import template_au
 from . import account_move
-from . import chart_template
+from . import res_partner_bank
+from . import account_payment
