@@ -1,10 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from . import account_chart_template
-from . import account_journal
+from . import template_cl
 from . import account_move
+from . import account_move_line
 from . import account_tax
-from . import ir_sequence
 from . import l10n_latam_document_type
 from . import res_company
 from . import res_country
