@@ -1,20 +1,14 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
-# Copyright (C) 2008-2008 凯源吕鑫 lvxin@gmail.com   <basic chart data>
-#                         维智众源 oldrev@gmail.com  <states data>
-# Copyright (C) 2012-2012 南京盈通 ccdos@intoerp.com <small business chart>
-# Copyright (C) 2008-now  开阖软件 jeff@osbzr.com    < PM and LTS >
-# Copyright (C) 2018-now  jeffery9@gmail.com
-
 {
     'name': 'China - Accounting',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['cn'],
     'version': '1.8',
     'category': 'Accounting/Localizations/Account Charts',
-    'author': 'www.openerp-china.org',
+    'author': 'openerp-china',
     'maintainer': 'jeff@osbzr.com',
-    'website': 'http://openerp-china.org',
-    'description': """
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
+    'description': r"""
 Includes the following data for the Chinese localization
 ========================================================
 
@@ -32,18 +26,18 @@ State Data/省份数据
 
     修改小企业会计税率
 
-We added the option to print a voucher which will also 
+    增加大企业会计科目表
+
+We added the option to print a voucher which will also
 print the amount in words (special Chinese characters for numbers)
 correctly when the cn2an library is installed. (e.g. with pip3 install cn2an)
     """,
-    'depends': ['base', 'account', 'l10n_multilang'],
+    'depends': [
+        'base',
+        'account',
+    ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_tax_group_data.xml',
-        'data/l10n_cn_chart_data.xml',
-        'data/account.account.template.csv',
-        'data/l10n_cn_chart_post_data.xml',
-        'data/account_tax_template_data.xml',
-        'data/account_chart_template_data.xml',
         'views/account_move_view.xml',
         'views/account_report.xml',
         'views/report_voucher.xml',
@@ -51,6 +45,5 @@ correctly when the cn2an library is installed. (e.g. with pip3 install cn2an)
     'demo': [
         'demo/demo_company.xml',
     ],
-    'post_init_hook': 'load_translations',
     'license': 'LGPL-3',
 }
