@@ -4,8 +4,8 @@
 
 {
     'name': 'China - City Data',
+    'countries': ['cn'],
     'version': '1.8',
-    'icon': '/l10n_cn/static/description/icon.png',
     'category': 'Accounting/Localizations',
     'author': 'Jeffery Chen Fan<jeffery9@gmail.com>',
     'description': """
