@@ -1,8 +1,9 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 {
     'name': 'Algeria - Accounting',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['dz'],
     'version': '1.0',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
@@ -11,14 +12,16 @@ This is the module to manage the accounting chart for Algeria in Odoo.
 This module applies to companies based in Algeria.
 """,
     'author': 'Osis',
-    'depends': ['account'],
+    'depends': [
+        'base_vat',
+        'account',
+    ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_chart_template_data.xml',
-        'data/account.account.template.csv',
-        'data/account_chart_template_post_data.xml',
-        'data/account_tax_data.xml',
-        'data/account_fiscal_position_template_data.xml',
-        'data/account_chart_template_configuration_data.xml',
+        'data/tax_report.xml',
+    ],
+    'demo': [
+        'demo/demo_company.xml',
     ],
     'license': 'LGPL-3',
 }
