@@ -4,9 +4,12 @@
 from odoo import fields, models
 
 _TYPE_EC = [
+    ("vat05", "VAT 5%"),
     ("vat08", "VAT 8%"),
     ("vat12", "VAT 12%"),
+    ("vat13", "VAT 13%"),
     ("vat14", "VAT 14%"),
+    ("vat15", "VAT 15%"),
     ("zero_vat", "VAT 0%"),
     ("not_charged_vat", "VAT Not Charged"),
     ("exempt_vat", "VAT Exempt"),
@@ -18,8 +21,6 @@ _TYPE_EC = [
     ("withhold_income_purchase", "Profit Withhold on Purchases"),
     ("outflows_tax", "Exchange Outflows"),
     ("other", "Others"),
-    ("withhold_vat", "VAT Withhold (Deprecated)"),  # removed in master
-    ("withhold_income_tax", "Profit Withhold (Deprecated)"),  # removed in master
 ]
 
 
