@@ -1,8 +1,10 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 {
     'name': 'Estonia - Accounting',
-    'version': '1.0',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
+    'version': '1.1',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['ee'],
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
 This is the base module to manage the accounting chart for Estonia in Odoo.
@@ -10,18 +12,10 @@ This is the base module to manage the accounting chart for Estonia in Odoo.
     'author': 'Odoo SA',
     'depends': [
         'account',
-        'l10n_multilang',
     ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_chart_template_data.xml',
-        'data/account.account.template.csv',
-        'data/l10n_ee_chart_post_data.xml',
-        'data/account_tax_group_data.xml',
         'data/account_tax_report_data.xml',
-        'data/account_tax_template_data.xml',
-        'data/account_fiscal_position_template_data.xml',
-        'data/account.group.template.csv',
-        'data/account_chart_template_try_loading.xml',
         'views/account_tax_form.xml',
     ],
     'demo': [
