@@ -1,14 +1,12 @@
 # -*- coding: utf-8 -*-
 {
     'name': "Egyptian E-Invoice Integration",
-    'summary': """
-            Egyptian Tax Authority Invoice Integration
-        """,
+    'countries': ['eg'],
+    'summary': "Egyptian Tax Authority Invoice Integration",
     'description': """
-       This module integrate with the ETA Portal to automatically sign and send your invoices to the tax Authority.
-       Special thanks to Plementus <info@plementus.com> for their help in developing this module.
+This module integrate with the ETA Portal to automatically sign and send your invoices to the tax Authority.
+Special thanks to Plementus <info@plementus.com> for their help in developing this module.
     """,
-    'author': 'odoo',
     'website': 'https://www.odoo.com',
     'category': 'account',
     'version': '0.2',
@@ -32,7 +30,7 @@
     ],
     'assets': {
         'web.assets_backend': [
-            'l10n_eg_edi_eta/static/src/js/sign_invoice.js',
+            'l10n_eg_edi_eta/static/src/**/*.js',
         ],
     },
     'external_dependencies': {
