@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import chart_template
+from . import eu_account_map
 from . import eu_tax_map
 from . import eu_tag_map
 from . import res_company
