@@ -43,9 +43,9 @@ EU_TAX_MAP = {
     ('AT', 13.0, 'CZ'): 21.0,
     ('AT', 13.0, 'DE'): 19.0,
     ('AT', 13.0, 'DK'): 25.0,
-    ('AT', 13.0, 'EE'): 20.0,
+    ('AT', 13.0, 'EE'): 22.0,
     ('AT', 13.0, 'ES'): 21.0,
-    ('AT', 13.0, 'FI'): 24.0,
+    ('AT', 13.0, 'FI'): 25.5,
     ('AT', 13.0, 'FR'): 20.0,
     ('AT', 13.0, 'GR'): 24.0,
     ('AT', 13.0, 'HR'): 25.0,
@@ -69,9 +69,9 @@ EU_TAX_MAP = {
     ('AT', 20.0, 'CZ'): 21.0,
     ('AT', 20.0, 'DE'): 19.0,
     ('AT', 20.0, 'DK'): 25.0,
-    ('AT', 20.0, 'EE'): 20.0,
+    ('AT', 20.0, 'EE'): 22.0,
     ('AT', 20.0, 'ES'): 21.0,
-    ('AT', 20.0, 'FI'): 24.0,
+    ('AT', 20.0, 'FI'): 25.5,
     ('AT', 20.0, 'FR'): 20.0,
     ('AT', 20.0, 'GR'): 24.0,
     ('AT', 20.0, 'HR'): 25.0,
@@ -95,9 +95,9 @@ EU_TAX_MAP = {
     ('BE', 12.0, 'CZ'): 21.0,
     ('BE', 12.0, 'DE'): 19.0,
     ('BE', 12.0, 'DK'): 25.0,
-    ('BE', 12.0, 'EE'): 20.0,
+    ('BE', 12.0, 'EE'): 22.0,
     ('BE', 12.0, 'ES'): 21.0,
-    ('BE', 12.0, 'FI'): 24.0,
+    ('BE', 12.0, 'FI'): 25.5,
     ('BE', 12.0, 'FR'): 20.0,
     ('BE', 12.0, 'GR'): 24.0,
     ('BE', 12.0, 'HR'): 25.0,
@@ -121,9 +121,9 @@ EU_TAX_MAP = {
     ('BE', 21.0, 'CZ'): 21.0,
     ('BE', 21.0, 'DE'): 19.0,
     ('BE', 21.0, 'DK'): 25.0,
-    ('BE', 21.0, 'EE'): 20.0,
+    ('BE', 21.0, 'EE'): 22.0,
     ('BE', 21.0, 'ES'): 21.0,
-    ('BE', 21.0, 'FI'): 24.0,
+    ('BE', 21.0, 'FI'): 25.5,
     ('BE', 21.0, 'FR'): 20.0,
     ('BE', 21.0, 'GR'): 24.0,
     ('BE', 21.0, 'HR'): 25.0,
@@ -173,9 +173,9 @@ EU_TAX_MAP = {
     ('BG', 20.0, 'CZ'): 21.0,
     ('BG', 20.0, 'DE'): 19.0,
     ('BG', 20.0, 'DK'): 25.0,
-    ('BG', 20.0, 'EE'): 20.0,
+    ('BG', 20.0, 'EE'): 22.0,
     ('BG', 20.0, 'ES'): 21.0,
-    ('BG', 20.0, 'FI'): 24.0,
+    ('BG', 20.0, 'FI'): 25.5,
     ('BG', 20.0, 'FR'): 20.0,
     ('BG', 20.0, 'GR'): 24.0,
     ('BG', 20.0, 'HR'): 25.0,
@@ -225,9 +225,9 @@ EU_TAX_MAP = {
     ('CY', 19.0, 'CZ'): 21.0,
     ('CY', 19.0, 'DE'): 19.0,
     ('CY', 19.0, 'DK'): 25.0,
-    ('CY', 19.0, 'EE'): 20.0,
+    ('CY', 19.0, 'EE'): 22.0,
     ('CY', 19.0, 'ES'): 21.0,
-    ('CY', 19.0, 'FI'): 24.0,
+    ('CY', 19.0, 'FI'): 25.5,
     ('CY', 19.0, 'FR'): 20.0,
     ('CY', 19.0, 'GR'): 24.0,
     ('CY', 19.0, 'HR'): 25.0,
@@ -355,9 +355,9 @@ EU_TAX_MAP = {
     ('CZ', 21.0, 'CY'): 19.0,
     ('CZ', 21.0, 'DE'): 19.0,
     ('CZ', 21.0, 'DK'): 25.0,
-    ('CZ', 21.0, 'EE'): 20.0,
+    ('CZ', 21.0, 'EE'): 22.0,
     ('CZ', 21.0, 'ES'): 21.0,
-    ('CZ', 21.0, 'FI'): 24.0,
+    ('CZ', 21.0, 'FI'): 25.5,
     ('CZ', 21.0, 'FR'): 20.0,
     ('CZ', 21.0, 'GR'): 24.0,
     ('CZ', 21.0, 'HR'): 25.0,
@@ -381,9 +381,9 @@ EU_TAX_MAP = {
     ('DE', 19.0, 'CY'): 19.0,
     ('DE', 19.0, 'CZ'): 21.0,
     ('DE', 19.0, 'DK'): 25.0,
-    ('DE', 19.0, 'EE'): 20.0,
+    ('DE', 19.0, 'EE'): 22.0,
     ('DE', 19.0, 'ES'): 21.0,
-    ('DE', 19.0, 'FI'): 24.0,
+    ('DE', 19.0, 'FI'): 25.5,
     ('DE', 19.0, 'FR'): 20.0,
     ('DE', 19.0, 'GR'): 24.0,
     ('DE', 19.0, 'HR'): 25.0,
@@ -433,9 +433,9 @@ EU_TAX_MAP = {
     ('DK', 25.0, 'CY'): 19.0,
     ('DK', 25.0, 'CZ'): 21.0,
     ('DK', 25.0, 'DE'): 19.0,
-    ('DK', 25.0, 'EE'): 20.0,
+    ('DK', 25.0, 'EE'): 22.0,
     ('DK', 25.0, 'ES'): 21.0,
-    ('DK', 25.0, 'FI'): 24.0,
+    ('DK', 25.0, 'FI'): 25.5,
     ('DK', 25.0, 'FR'): 20.0,
     ('DK', 25.0, 'GR'): 24.0,
     ('DK', 25.0, 'HR'): 25.0,
@@ -453,6 +453,32 @@ EU_TAX_MAP = {
     ('DK', 25.0, 'SE'): 25.0,
     ('DK', 25.0, 'SI'): 22.0,
     ('DK', 25.0, 'SK'): 20.0,
+    ('EE', 22.0, 'AT'): 20.0,
+    ('EE', 22.0, 'BE'): 21.0,
+    ('EE', 22.0, 'BG'): 20.0,
+    ('EE', 22.0, 'CY'): 19.0,
+    ('EE', 22.0, 'CZ'): 21.0,
+    ('EE', 22.0, 'DE'): 19.0,
+    ('EE', 22.0, 'DK'): 25.0,
+    ('EE', 22.0, 'ES'): 21.0,
+    ('EE', 22.0, 'FI'): 25.5,
+    ('EE', 22.0, 'FR'): 20.0,
+    ('EE', 22.0, 'GR'): 24.0,
+    ('EE', 22.0, 'HR'): 25.0,
+    ('EE', 22.0, 'HU'): 27.0,
+    ('EE', 22.0, 'IE'): 23.0,
+    ('EE', 22.0, 'IT'): 22.0,
+    ('EE', 22.0, 'LT'): 21.0,
+    ('EE', 22.0, 'LU'): 16.0,
+    ('EE', 22.0, 'LV'): 21.0,
+    ('EE', 22.0, 'MT'): 18.0,
+    ('EE', 22.0, 'NL'): 21.0,
+    ('EE', 22.0, 'PL'): 23.0,
+    ('EE', 22.0, 'PT'): 23.0,
+    ('EE', 22.0, 'RO'): 19.0,
+    ('EE', 22.0, 'SE'): 25.0,
+    ('EE', 22.0, 'SI'): 22.0,
+    ('EE', 22.0, 'SK'): 20.0,
     ('EE', 20.0, 'AT'): 20.0,
     ('EE', 20.0, 'BE'): 21.0,
     ('EE', 20.0, 'BG'): 20.0,
@@ -461,7 +487,7 @@ EU_TAX_MAP = {
     ('EE', 20.0, 'DE'): 19.0,
     ('EE', 20.0, 'DK'): 25.0,
     ('EE', 20.0, 'ES'): 21.0,
-    ('EE', 20.0, 'FI'): 24.0,
+    ('EE', 20.0, 'FI'): 25.5,
     ('EE', 20.0, 'FR'): 20.0,
     ('EE', 20.0, 'GR'): 24.0,
     ('EE', 20.0, 'HR'): 25.0,
@@ -538,8 +564,8 @@ EU_TAX_MAP = {
     ('ES', 21.0, 'CZ'): 21.0,
     ('ES', 21.0, 'DE'): 19.0,
     ('ES', 21.0, 'DK'): 25.0,
-    ('ES', 21.0, 'EE'): 20.0,
-    ('ES', 21.0, 'FI'): 24.0,
+    ('ES', 21.0, 'EE'): 22.0,
+    ('ES', 21.0, 'FI'): 25.5,
     ('ES', 21.0, 'FR'): 20.0,
     ('ES', 21.0, 'GR'): 24.0,
     ('ES', 21.0, 'HR'): 25.0,
@@ -642,7 +668,7 @@ EU_TAX_MAP = {
     ('FI', 24.0, 'CZ'): 21.0,
     ('FI', 24.0, 'DE'): 19.0,
     ('FI', 24.0, 'DK'): 25.0,
-    ('FI', 24.0, 'EE'): 20.0,
+    ('FI', 24.0, 'EE'): 22.0,
     ('FI', 24.0, 'ES'): 21.0,
     ('FI', 24.0, 'FR'): 20.0,
     ('FI', 24.0, 'GR'): 24.0,
@@ -661,6 +687,32 @@ EU_TAX_MAP = {
     ('FI', 24.0, 'SE'): 25.0,
     ('FI', 24.0, 'SI'): 22.0,
     ('FI', 24.0, 'SK'): 20.0,
+    ('FI', 25.5, 'AT'): 20.0,
+    ('FI', 25.5, 'BE'): 21.0,
+    ('FI', 25.5, 'BG'): 20.0,
+    ('FI', 25.5, 'CY'): 19.0,
+    ('FI', 25.5, 'CZ'): 21.0,
+    ('FI', 25.5, 'DE'): 19.0,
+    ('FI', 25.5, 'DK'): 25.0,
+    ('FI', 25.5, 'EE'): 22.0,
+    ('FI', 25.5, 'ES'): 21.0,
+    ('FI', 25.5, 'FR'): 20.0,
+    ('FI', 25.5, 'GR'): 24.0,
+    ('FI', 25.5, 'HR'): 25.0,
+    ('FI', 25.5, 'HU'): 27.0,
+    ('FI', 25.5, 'IE'): 23.0,
+    ('FI', 25.5, 'IT'): 22.0,
+    ('FI', 25.5, 'LT'): 21.0,
+    ('FI', 25.5, 'LU'): 16.0,
+    ('FI', 25.5, 'LV'): 21.0,
+    ('FI', 25.5, 'MT'): 18.0,
+    ('FI', 25.5, 'NL'): 21.0,
+    ('FI', 25.5, 'PL'): 23.0,
+    ('FI', 25.5, 'PT'): 23.0,
+    ('FI', 25.5, 'RO'): 19.0,
+    ('FI', 25.5, 'SE'): 25.0,
+    ('FI', 25.5, 'SI'): 22.0,
+    ('FI', 25.5, 'SK'): 20.0,
     ('FR', 10.0, 'AT'): 10.0,
     ('FR', 10.0, 'BE'): 6.0,
     ('FR', 10.0, 'BG'): 20.0,
@@ -720,9 +772,9 @@ EU_TAX_MAP = {
     ('FR', 20.0, 'CZ'): 21.0,
     ('FR', 20.0, 'DE'): 19.0,
     ('FR', 20.0, 'DK'): 25.0,
-    ('FR', 20.0, 'EE'): 20.0,
+    ('FR', 20.0, 'EE'): 22.0,
     ('FR', 20.0, 'ES'): 21.0,
-    ('FR', 20.0, 'FI'): 24.0,
+    ('FR', 20.0, 'FI'): 25.5,
     ('FR', 20.0, 'GR'): 24.0,
     ('FR', 20.0, 'HR'): 25.0,
     ('FR', 20.0, 'HU'): 27.0,
@@ -798,9 +850,9 @@ EU_TAX_MAP = {
     ('GR', 24.0, 'CZ'): 21.0,
     ('GR', 24.0, 'DE'): 19.0,
     ('GR', 24.0, 'DK'): 25.0,
-    ('GR', 24.0, 'EE'): 20.0,
+    ('GR', 24.0, 'EE'): 22.0,
     ('GR', 24.0, 'ES'): 21.0,
-    ('GR', 24.0, 'FI'): 24.0,
+    ('GR', 24.0, 'FI'): 25.5,
     ('GR', 24.0, 'FR'): 20.0,
     ('GR', 24.0, 'HR'): 25.0,
     ('GR', 24.0, 'HU'): 27.0,
@@ -876,9 +928,9 @@ EU_TAX_MAP = {
     ('HR', 25.0, 'CZ'): 21.0,
     ('HR', 25.0, 'DE'): 19.0,
     ('HR', 25.0, 'DK'): 25.0,
-    ('HR', 25.0, 'EE'): 20.0,
+    ('HR', 25.0, 'EE'): 22.0,
     ('HR', 25.0, 'ES'): 21.0,
-    ('HR', 25.0, 'FI'): 24.0,
+    ('HR', 25.0, 'FI'): 25.5,
     ('HR', 25.0, 'FR'): 20.0,
     ('HR', 25.0, 'GR'): 24.0,
     ('HR', 25.0, 'HU'): 27.0,
@@ -954,9 +1006,9 @@ EU_TAX_MAP = {
     ('HU', 27.0, 'CZ'): 21.0,
     ('HU', 27.0, 'DE'): 19.0,
     ('HU', 27.0, 'DK'): 25.0,
-    ('HU', 27.0, 'EE'): 20.0,
+    ('HU', 27.0, 'EE'): 22.0,
     ('HU', 27.0, 'ES'): 21.0,
-    ('HU', 27.0, 'FI'): 24.0,
+    ('HU', 27.0, 'FI'): 25.5,
     ('HU', 27.0, 'FR'): 20.0,
     ('HU', 27.0, 'GR'): 24.0,
     ('HU', 27.0, 'HR'): 25.0,
@@ -1006,9 +1058,9 @@ EU_TAX_MAP = {
     ('IE', 13.5, 'CZ'): 21.0,
     ('IE', 13.5, 'DE'): 19.0,
     ('IE', 13.5, 'DK'): 25.0,
-    ('IE', 13.5, 'EE'): 20.0,
+    ('IE', 13.5, 'EE'): 22.0,
     ('IE', 13.5, 'ES'): 21.0,
-    ('IE', 13.5, 'FI'): 24.0,
+    ('IE', 13.5, 'FI'): 25.5,
     ('IE', 13.5, 'FR'): 20.0,
     ('IE', 13.5, 'GR'): 24.0,
     ('IE', 13.5, 'HR'): 25.0,
@@ -1033,9 +1085,9 @@ EU_TAX_MAP = {
     ('IE', 23.0, 'CZ'): 21.0,
     ('IE', 23.0, 'DE'): 19.0,
     ('IE', 23.0, 'DK'): 25.0,
-    ('IE', 23.0, 'EE'): 20.0,
+    ('IE', 23.0, 'EE'): 22.0,
     ('IE', 23.0, 'ES'): 21.0,
-    ('IE', 23.0, 'FI'): 24.0,
+    ('IE', 23.0, 'FI'): 25.5,
     ('IE', 23.0, 'FR'): 20.0,
     ('IE', 23.0, 'GR'): 24.0,
     ('IE', 23.0, 'HR'): 25.0,
@@ -1137,9 +1189,9 @@ EU_TAX_MAP = {
     ('IT', 22.0, 'CZ'): 21.0,
     ('IT', 22.0, 'DE'): 19.0,
     ('IT', 22.0, 'DK'): 25.0,
-    ('IT', 22.0, 'EE'): 20.0,
+    ('IT', 22.0, 'EE'): 22.0,
     ('IT', 22.0, 'ES'): 21.0,
-    ('IT', 22.0, 'FI'): 24.0,
+    ('IT', 22.0, 'FI'): 25.5,
     ('IT', 22.0, 'FR'): 20.0,
     ('IT', 22.0, 'GR'): 24.0,
     ('IT', 22.0, 'HR'): 25.0,
@@ -1215,9 +1267,9 @@ EU_TAX_MAP = {
     ('LT', 21.0, 'CZ'): 21.0,
     ('LT', 21.0, 'DE'): 19.0,
     ('LT', 21.0, 'DK'): 25.0,
-    ('LT', 21.0, 'EE'): 20.0,
+    ('LT', 21.0, 'EE'): 22.0,
     ('LT', 21.0, 'ES'): 21.0,
-    ('LT', 21.0, 'FI'): 24.0,
+    ('LT', 21.0, 'FI'): 25.5,
     ('LT', 21.0, 'FR'): 20.0,
     ('LT', 21.0, 'GR'): 24.0,
     ('LT', 21.0, 'HR'): 25.0,
@@ -1293,9 +1345,9 @@ EU_TAX_MAP = {
     ('LU', 13.0, 'CZ'): 21.0,
     ('LU', 13.0, 'DE'): 19.0,
     ('LU', 13.0, 'DK'): 25.0,
-    ('LU', 13.0, 'EE'): 20.0,
+    ('LU', 13.0, 'EE'): 22.0,
     ('LU', 13.0, 'ES'): 21.0,
-    ('LU', 13.0, 'FI'): 24.0,
+    ('LU', 13.0, 'FI'): 25.5,
     ('LU', 13.0, 'FR'): 20.0,
     ('LU', 13.0, 'GR'): 24.0,
     ('LU', 13.0, 'HR'): 25.0,
@@ -1319,9 +1371,9 @@ EU_TAX_MAP = {
     ('LU', 14.0, 'CZ'): 21.0,
     ('LU', 14.0, 'DE'): 19.0,
     ('LU', 14.0, 'DK'): 25.0,
-    ('LU', 14.0, 'EE'): 20.0,
+    ('LU', 14.0, 'EE'): 22.0,
     ('LU', 14.0, 'ES'): 21.0,
-    ('LU', 14.0, 'FI'): 24.0,
+    ('LU', 14.0, 'FI'): 25.5,
     ('LU', 14.0, 'FR'): 20.0,
     ('LU', 14.0, 'GR'): 24.0,
     ('LU', 14.0, 'HR'): 25.0,
@@ -1345,9 +1397,9 @@ EU_TAX_MAP = {
     ('LU', 16.0, 'CZ'): 21.0,
     ('LU', 16.0, 'DE'): 19.0,
     ('LU', 16.0, 'DK'): 25.0,
-    ('LU', 16.0, 'EE'): 20.0,
+    ('LU', 16.0, 'EE'): 22.0,
     ('LU', 16.0, 'ES'): 21.0,
-    ('LU', 16.0, 'FI'): 24.0,
+    ('LU', 16.0, 'FI'): 25.5,
     ('LU', 16.0, 'FR'): 20.0,
     ('LU', 16.0, 'GR'): 24.0,
     ('LU', 16.0, 'HR'): 25.0,
@@ -1371,9 +1423,9 @@ EU_TAX_MAP = {
     ('LU', 17.0, 'CZ'): 21.0,
     ('LU', 17.0, 'DE'): 19.0,
     ('LU', 17.0, 'DK'): 25.0,
-    ('LU', 17.0, 'EE'): 20.0,
+    ('LU', 17.0, 'EE'): 22.0,
     ('LU', 17.0, 'ES'): 21.0,
-    ('LU', 17.0, 'FI'): 24.0,
+    ('LU', 17.0, 'FI'): 25.5,
     ('LU', 17.0, 'FR'): 20.0,
     ('LU', 17.0, 'GR'): 24.0,
     ('LU', 17.0, 'HR'): 25.0,
@@ -1501,9 +1553,9 @@ EU_TAX_MAP = {
     ('LV', 21.0, 'CZ'): 21.0,
     ('LV', 21.0, 'DE'): 19.0,
     ('LV', 21.0, 'DK'): 25.0,
-    ('LV', 21.0, 'EE'): 20.0,
+    ('LV', 21.0, 'EE'): 22.0,
     ('LV', 21.0, 'ES'): 21.0,
-    ('LV', 21.0, 'FI'): 24.0,
+    ('LV', 21.0, 'FI'): 25.5,
     ('LV', 21.0, 'FR'): 20.0,
     ('LV', 21.0, 'GR'): 24.0,
     ('LV', 21.0, 'HR'): 25.0,
@@ -1553,9 +1605,9 @@ EU_TAX_MAP = {
     ('MT', 18.0, 'CZ'): 21.0,
     ('MT', 18.0, 'DE'): 19.0,
     ('MT', 18.0, 'DK'): 25.0,
-    ('MT', 18.0, 'EE'): 20.0,
+    ('MT', 18.0, 'EE'): 22.0,
     ('MT', 18.0, 'ES'): 21.0,
-    ('MT', 18.0, 'FI'): 24.0,
+    ('MT', 18.0, 'FI'): 25.5,
     ('MT', 18.0, 'FR'): 20.0,
     ('MT', 18.0, 'GR'): 24.0,
     ('MT', 18.0, 'HR'): 25.0,
@@ -1631,9 +1683,9 @@ EU_TAX_MAP = {
     ('NL', 21.0, 'CZ'): 21.0,
     ('NL', 21.0, 'DE'): 19.0,
     ('NL', 21.0, 'DK'): 25.0,
-    ('NL', 21.0, 'EE'): 20.0,
+    ('NL', 21.0, 'EE'): 22.0,
     ('NL', 21.0, 'ES'): 21.0,
-    ('NL', 21.0, 'FI'): 24.0,
+    ('NL', 21.0, 'FI'): 25.5,
     ('NL', 21.0, 'FR'): 20.0,
     ('NL', 21.0, 'GR'): 24.0,
     ('NL', 21.0, 'HR'): 25.0,
@@ -1683,9 +1735,9 @@ EU_TAX_MAP = {
     ('PL', 23.0, 'CZ'): 21.0,
     ('PL', 23.0, 'DE'): 19.0,
     ('PL', 23.0, 'DK'): 25.0,
-    ('PL', 23.0, 'EE'): 20.0,
+    ('PL', 23.0, 'EE'): 22.0,
     ('PL', 23.0, 'ES'): 21.0,
-    ('PL', 23.0, 'FI'): 24.0,
+    ('PL', 23.0, 'FI'): 25.5,
     ('PL', 23.0, 'FR'): 20.0,
     ('PL', 23.0, 'GR'): 24.0,
     ('PL', 23.0, 'HR'): 25.0,
@@ -1761,9 +1813,9 @@ EU_TAX_MAP = {
     ('PT', 13.0, 'CZ'): 21.0,
     ('PT', 13.0, 'DE'): 19.0,
     ('PT', 13.0, 'DK'): 25.0,
-    ('PT', 13.0, 'EE'): 20.0,
+    ('PT', 13.0, 'EE'): 22.0,
     ('PT', 13.0, 'ES'): 21.0,
-    ('PT', 13.0, 'FI'): 24.0,
+    ('PT', 13.0, 'FI'): 25.5,
     ('PT', 13.0, 'FR'): 20.0,
     ('PT', 13.0, 'GR'): 24.0,
     ('PT', 13.0, 'HR'): 25.0,
@@ -1787,9 +1839,9 @@ EU_TAX_MAP = {
     ('PT', 23.0, 'CZ'): 21.0,
     ('PT', 23.0, 'DE'): 19.0,
     ('PT', 23.0, 'DK'): 25.0,
-    ('PT', 23.0, 'EE'): 20.0,
+    ('PT', 23.0, 'EE'): 22.0,
     ('PT', 23.0, 'ES'): 21.0,
-    ('PT', 23.0, 'FI'): 24.0,
+    ('PT', 23.0, 'FI'): 25.5,
     ('PT', 23.0, 'FR'): 20.0,
     ('PT', 23.0, 'GR'): 24.0,
     ('PT', 23.0, 'HR'): 25.0,
@@ -1839,9 +1891,9 @@ EU_TAX_MAP = {
     ('RO', 19.0, 'CZ'): 21.0,
     ('RO', 19.0, 'DE'): 19.0,
     ('RO', 19.0, 'DK'): 25.0,
-    ('RO', 19.0, 'EE'): 20.0,
+    ('RO', 19.0, 'EE'): 22.0,
     ('RO', 19.0, 'ES'): 21.0,
-    ('RO', 19.0, 'FI'): 24.0,
+    ('RO', 19.0, 'FI'): 25.5,
     ('RO', 19.0, 'FR'): 20.0,
     ('RO', 19.0, 'GR'): 24.0,
     ('RO', 19.0, 'HR'): 25.0,
@@ -1943,9 +1995,9 @@ EU_TAX_MAP = {
     ('SE', 25.0, 'CZ'): 21.0,
     ('SE', 25.0, 'DE'): 19.0,
     ('SE', 25.0, 'DK'): 25.0,
-    ('SE', 25.0, 'EE'): 20.0,
+    ('SE', 25.0, 'EE'): 22.0,
     ('SE', 25.0, 'ES'): 21.0,
-    ('SE', 25.0, 'FI'): 24.0,
+    ('SE', 25.0, 'FI'): 25.5,
     ('SE', 25.0, 'FR'): 20.0,
     ('SE', 25.0, 'GR'): 24.0,
     ('SE', 25.0, 'HR'): 25.0,
@@ -1995,9 +2047,9 @@ EU_TAX_MAP = {
     ('SI', 22.0, 'CZ'): 21.0,
     ('SI', 22.0, 'DE'): 19.0,
     ('SI', 22.0, 'DK'): 25.0,
-    ('SI', 22.0, 'EE'): 20.0,
+    ('SI', 22.0, 'EE'): 22.0,
     ('SI', 22.0, 'ES'): 21.0,
-    ('SI', 22.0, 'FI'): 24.0,
+    ('SI', 22.0, 'FI'): 25.5,
     ('SI', 22.0, 'FR'): 20.0,
     ('SI', 22.0, 'GR'): 24.0,
     ('SI', 22.0, 'HR'): 25.0,
@@ -2099,9 +2151,9 @@ EU_TAX_MAP = {
     ('SK', 20.0, 'CZ'): 21.0,
     ('SK', 20.0, 'DE'): 19.0,
     ('SK', 20.0, 'DK'): 25.0,
-    ('SK', 20.0, 'EE'): 20.0,
+    ('SK', 20.0, 'EE'): 22.0,
     ('SK', 20.0, 'ES'): 21.0,
-    ('SK', 20.0, 'FI'): 24.0,
+    ('SK', 20.0, 'FI'): 25.5,
     ('SK', 20.0, 'FR'): 20.0,
     ('SK', 20.0, 'GR'): 24.0,
     ('SK', 20.0, 'HR'): 25.0,
@@ -2118,4 +2170,31 @@ EU_TAX_MAP = {
     ('SK', 20.0, 'RO'): 19.0,
     ('SK', 20.0, 'SE'): 25.0,
     ('SK', 20.0, 'SI'): 22.0,
+    ('GB', 20.0, 'AT'): 20.0,
+    ('GB', 20.0, 'BE'): 21.0,
+    ('GB', 20.0, 'BG'): 20.0,
+    ('GB', 20.0, 'CY'): 19.0,
+    ('GB', 20.0, 'CZ'): 21.0,
+    ('GB', 20.0, 'DE'): 19.0,
+    ('GB', 20.0, 'DK'): 25.0,
+    ('GB', 20.0, 'EE'): 22.0,
+    ('GB', 20.0, 'ES'): 21.0,
+    ('GB', 20.0, 'FI'): 24.0,
+    ('GB', 20.0, 'FR'): 20.0,
+    ('GB', 20.0, 'GR'): 24.0,
+    ('GB', 20.0, 'HR'): 25.0,
+    ('GB', 20.0, 'HU'): 27.0,
+    ('GB', 20.0, 'IE'): 23.0,
+    ('GB', 20.0, 'IT'): 22.0,
+    ('GB', 20.0, 'LT'): 21.0,
+    ('GB', 20.0, 'LU'): 16.0,
+    ('GB', 20.0, 'LV'): 21.0,
+    ('GB', 20.0, 'MT'): 18.0,
+    ('GB', 20.0, 'NL'): 21.0,
+    ('GB', 20.0, 'PL'): 23.0,
+    ('GB', 20.0, 'PT'): 23.0,
+    ('GB', 20.0, 'RO'): 19.0,
+    ('GB', 20.0, 'SE'): 25.0,
+    ('GB', 20.0, 'SI'): 22.0,
+    ('GB', 20.0, 'SK'): 20.0,
 }
