@@ -14,7 +14,7 @@ def ctx_tz(record, field):
     tz_name = pytz.timezone(ctx.get('tz') or record.env.user.tz or 'UTC')
     timestamp = Datetime.from_string(record[field])
     if ctx.get('lang'):
-        res_lang = record.env['res.lang']._lang_get(ctx['lang'])
+        res_lang = record.env['res.lang']._get_data(code=ctx['lang'])
     if res_lang:
         timestamp = pytz.utc.localize(timestamp, is_dst=False)
         return datetime.strftime(timestamp.astimezone(tz_name), res_lang.date_format + ' ' + res_lang.time_format)
@@ -65,6 +65,7 @@ class ResCompany(models.Model):
 
             if not orders:
                 msg_alert = (_('There isn\'t any order flagged for data inalterability yet for the company %s. This mechanism only runs for point of sale orders generated after the installation of the module France - Certification CGI 286 I-3 bis. - POS', self.env.company.name))
+                raise UserError(msg_alert)
 
             previous_hash = u''
             corrupted_orders = []
@@ -94,3 +95,5 @@ class ResCompany(models.Model):
                 'printing_date': format_date(self.env,  Date.to_string( Date.today())),
                 'corrupted_orders': corrupted_orders or 'None'
             }
+        else:
+            raise UserError(_('Accounting is not unalterable for the company %s. This mechanism is designed for companies where accounting is unalterable.', self.env.company.name))
