@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 {
     'name': 'Indonesian - Accounting',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['id'],
     'version': '1.0',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
@@ -11,18 +11,22 @@ This is the latest Indonesian Odoo localisation necessary to run Odoo accounting
     - generic Indonesian chart of accounts
     - tax structure""",
     'author': 'vitraining.com',
-    'website': 'http://www.vitraining.com',
-    'depends': ['account', 'base_iban', 'base_vat', 'l10n_multilang'],
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations/indonesia.html',
+    'depends': [
+        'account',
+        'base_iban',
+        'base_vat',
+    ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_chart_template_data.xml',
-        'data/account.account.template.csv',
-        'data/account_chart_template_post_data.xml',
+        'security/ir.model.access.csv',
         'data/account_tax_template_data.xml',
-        'data/account_chart_template_configuration_data.xml',
+        'data/ir_cron.xml',
+        'views/account_move_views.xml',
+        'views/res_bank.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
     ],
-    'post_init_hook': 'load_translations',
     'license': 'LGPL-3',
 }
