@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 {
     'name': 'Israel - Accounting',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['il'],
     'version': '1.0',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
@@ -14,19 +14,14 @@ This module consists of:
  - Taxes and tax report
  - Multiple Fiscal positions
  """,
-    'website': 'http://www.odoo.com/accounting',
-    'depends': ['l10n_multilang'],
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
+    'depends': [
+        'account',
+    ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_chart_template_data.xml',
-        'data/account_tax_group_data.xml',
         'data/account_account_tag.xml',
-        'data/account.account.template.csv',
-        'data/account.group.template.csv',
         'data/account_tax_report_data.xml',
-        'data/account_tax_template_data.xml',
-        'data/fiscal_templates_data.xml',
-        'data/account_chart_template_post_data.xml',
-        'data/account_chart_template_configure_data.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
