@@ -1,48 +1,21 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from odoo.addons.account.tests.common import AccountTestInvoicingCommon
+from odoo.fields import Command
 from odoo.tests import tagged
 
+from odoo.addons.l10n_in.tests.common import L10nInTestInvoicingCommon
+
 
 @tagged("post_install_l10n", "post_install", "-at_install")
-class TestEdiJson(AccountTestInvoicingCommon):
-
+class TestEdiJson(L10nInTestInvoicingCommon):
 
     @classmethod
-    def setUpClass(cls, chart_template_ref="l10n_in.indian_chart_template_standard"):
-        super().setUpClass(chart_template_ref=chart_template_ref)
+    def setUpClass(cls):
+        super().setUpClass()
         cls.env['ir.config_parameter'].set_param('l10n_in_edi.manage_invoice_negative_lines', True)
         cls.maxDiff = None
-        cls.company_data["company"].write({
-            "street": "Block no. 401",
-            "street2": "Street 2",
-            "city": "City 1",
-            "zip": "500001",
-            "state_id": cls.env.ref("base.state_in_ts").id,
-            "country_id": cls.env.ref("base.in").id,
-            "vat": "36AABCT1332L011",
-        })
-        cls.partner_a.write({
-            "vat": "36BBBFF5679L8ZR",
-            "street": "Block no. 401",
-            "street2": "Street 2",
-            "city": "City 2",
-            "zip": "500001",
-            "state_id": cls.env.ref("base.state_in_ts").id,
-            "country_id": cls.env.ref("base.in").id,
-            "l10n_in_gst_treatment": "regular",
-        })
-        cls.product_a.write({"l10n_in_hsn_code": "01111"})
-        cls.product_a2 = cls.env['product.product'].create({
-            'name': 'product_a2',
-            'uom_id': cls.env.ref('uom.product_uom_unit').id,
-            'lst_price': 1000.0,
-            'standard_price': 1000.0,
-            'property_account_income_id': cls.company_data['default_account_revenue'].id,
-            'property_account_expense_id': cls.company_data['default_account_expense'].id,
-            'taxes_id': [(6, 0, cls.tax_sale_a.ids)],
-            'supplier_taxes_id': [(6, 0, cls.tax_purchase_a.ids)],
-            "l10n_in_hsn_code": "01111",
-        })
+
+        cls.partner_a.l10n_in_gst_treatment = "regular"
+
         cls.product_a_discount = cls.env['product.product'].create({
             'name': 'product_a discount',
             'uom_id': cls.env.ref('uom.product_uom_unit').id,
@@ -50,12 +23,12 @@ class TestEdiJson(AccountTestInvoicingCommon):
             'standard_price': 400.0,
             'property_account_income_id': cls.company_data['default_account_revenue'].id,
             'property_account_expense_id': cls.company_data['default_account_expense'].id,
-            'taxes_id': [(6, 0, cls.tax_sale_a.ids)],
-            'supplier_taxes_id': [(6, 0, cls.tax_purchase_a.ids)],
-            "l10n_in_hsn_code": "01111",
+            'taxes_id': [Command.set(cls.sgst_sale_5.ids)],
+            'supplier_taxes_id': [Command.set(cls.sgst_purchase_5.ids)],
+            "l10n_in_hsn_code": "111111",
         })
-        gst_with_cess = cls.env.ref("l10n_in.%s_sgst_sale_12" % (cls.company_data["company"].id)
-            ) + cls.env.ref("l10n_in.%s_cess_5_plus_1591_sale" % (cls.company_data["company"].id))
+        gst_with_cess = cls.env.ref("account.%s_sgst_sale_12" % (cls.company_data["company"].id)
+            ) + cls.env.ref("account.%s_cess_5_plus_1591_sale" % (cls.company_data["company"].id))
         product_with_cess = cls.env["product.product"].create({
             "name": "product_with_cess",
             "uom_id": cls.env.ref("uom.product_uom_unit").id,
@@ -63,9 +36,9 @@ class TestEdiJson(AccountTestInvoicingCommon):
             "standard_price": 800.0,
             "property_account_income_id": cls.company_data["default_account_revenue"].id,
             "property_account_expense_id": cls.company_data["default_account_expense"].id,
-            "taxes_id": [(6, 0, gst_with_cess.ids)],
-            "supplier_taxes_id": [(6, 0, cls.tax_purchase_a.ids)],
-            "l10n_in_hsn_code": "02222",
+            "taxes_id": [Command.set(gst_with_cess.ids)],
+            "supplier_taxes_id": [Command.set(cls.sgst_purchase_5.ids)],
+            "l10n_in_hsn_code": "333333",
         })
         rounding = cls.env["account.cash.rounding"].create({
             "name": "half-up",
@@ -115,7 +88,7 @@ class TestEdiJson(AccountTestInvoicingCommon):
                 (1, cls.invoice_negative_with_discount.invoice_line_ids[1].id, {"price_unit": -400}),
             ]})
         cls.invoice_negative_with_discount.action_post()
-        cls.invoice_negative_more_than_max_line = cls.init_invoice("out_invoice", post=False, products=cls.product_a + cls.product_a2 + cls.product_a_discount)
+        cls.invoice_negative_more_than_max_line = cls.init_invoice("out_invoice", post=False, products=cls.product_a + cls.product_b + cls.product_a_discount)
         cls.invoice_negative_more_than_max_line.write({
             "invoice_line_ids": [
                 (1, cls.invoice_negative_more_than_max_line.invoice_line_ids[0].id, {"price_unit": 2000, 'discount': 50}),
@@ -131,47 +104,53 @@ class TestEdiJson(AccountTestInvoicingCommon):
         cls.invoice_cash_rounding.action_post()
 
     def test_edi_json(self):
+        # line1: 1000, 10% discount and a tax of 5%
+        # line2: 1000, 10% discount and 4 taxes: 5%, 1.591, 6% SGST 6% CGST, each one affecting the base:
+        # 900 * 1.05 = 945
+        # 945 + 1.591 ~= 946.59
+        # 946.59 * 0.06 = 56.80
+        # total tax: 160.19
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice)
         expected = {
             "Version": "1.1",
             "TranDtls": {"TaxSch": "GST", "SupTyp": "B2B", "RegRev": "N", "IgstOnIntra": "N"},
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00001", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0002", "Dt": "01/01/2019"},
             "SellerDtls": {
-                "LglNm": "company_1_data",
-                "Addr1": "Block no. 401",
-                "Addr2": "Street 2",
-                "Loc": "City 1",
-                "Pin": 500001,
-                "Stcd": "36",
-                "GSTIN": "36AABCT1332L011"},
+                "Addr1": "Khodiyar Chowk",
+                "Loc": "Amreli",
+                "Pin": 365220,
+                "Stcd": "24",
+                "Addr2": "Sala Number 3",
+                "LglNm": "Default Company",
+                "GSTIN": "24AAGCC7144L6ZE"},
             "BuyerDtls": {
-                "LglNm": "partner_a",
-                "Addr1": "Block no. 401",
-                "Addr2": "Street 2",
-                "Loc": "City 2",
-                "Pin": 500001,
-                "Stcd": "36",
-                "POS": "36",
-                "GSTIN": "36BBBFF5679L8ZR"},
+                "Addr1": "Karansinhji Rd",
+                "Loc": "Rajkot",
+                "Pin": 360001,
+                "Stcd": "24",
+                "Addr2": "Karanpara",
+                "POS": "24",
+                "LglNm": "Partner Intra State",
+                "GSTIN": "24ABCPM8965E1ZE"},
             "ItemList": [
                 {
-                    "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                    "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                     "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 100.0, "AssAmt": 900.0,
                     "GstRt": 5.0, "IgstAmt": 0.0, "CgstAmt": 22.5, "SgstAmt": 22.5, "CesRt": 0.0, "CesAmt": 0.0,
                     "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
                     "OthChrg": 0.0, "TotItemVal": 945.0
                 },
                 {
-                    "SlNo": "2", "PrdDesc": "product_with_cess", "IsServc": "N", "HsnCd": "02222", "Qty": 1.0,
+                    "SlNo": "2", "PrdDesc": "product_with_cess", "IsServc": "N", "HsnCd": "333333", "Qty": 1.0,
                     "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 100.0, "AssAmt": 900.0,
-                    "GstRt": 12.0, "IgstAmt": 0.0, "CgstAmt": 54.0, "SgstAmt": 54.0, "CesRt": 5.0, "CesAmt": 45.0,
+                    "GstRt": 12.0, "IgstAmt": 0.0, "CgstAmt": 56.8, "SgstAmt": 56.8, "CesRt": 5.0, "CesAmt": 45.0,
                     "CesNonAdvlAmt": 1.59, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
-                    "OthChrg": 0.0, "TotItemVal": 1054.59
+                    "OthChrg": 0.0, "TotItemVal": 1060.19
                 }
             ],
             "ValDtls": {
-                "AssVal": 1800.0, "CgstVal": 76.5, "SgstVal": 76.5, "IgstVal": 0.0, "CesVal": 46.59,
-                "StCesVal": 0.0, "RndOffAmt": 0.0, "TotInvVal": 1999.59
+                "AssVal": 1800.0, "CgstVal": 79.3, "SgstVal": 79.3, "IgstVal": 0.0, "CesVal": 46.59,
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": 0.0, "TotInvVal": 2005.19
             }
         }
         self.assertDictEqual(json_value, expected, "Indian EDI send json value is not matched")
@@ -179,24 +158,24 @@ class TestEdiJson(AccountTestInvoicingCommon):
         #=================================== Full discount test =====================================
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_full_discount)
         expected.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00002", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0003", "Dt": "01/01/2019"},
             "ItemList": [{
-                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                 "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 1000.0, "AssAmt": 0.0,
                 "GstRt": 0.0, "IgstAmt": 0.0, "CgstAmt": 0.0, "SgstAmt": 0.0, "CesRt": 0.0, "CesAmt": 0.0,
                 "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
                 "OthChrg": 0.0, "TotItemVal": 0.0}],
             "ValDtls": {"AssVal": 0.0, "CgstVal": 0.0, "SgstVal": 0.0, "IgstVal": 0.0, "CesVal": 0.0,
-                "StCesVal": 0.0, "RndOffAmt": 0.0, "TotInvVal": 0.0}
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": 0.0, "TotInvVal": 0.0}
         })
         self.assertDictEqual(json_value, expected, "Indian EDI with 100% discount sent json value is not matched")
 
         #=================================== Zero quantity test =============================================
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_zero_qty)
         expected.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00003", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0004", "Dt": "01/01/2019"},
             "ItemList": [{
-                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 0.0,
+                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 0.0,
                 "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 0.0, "Discount": 0.0, "AssAmt": 0.0,
                 "GstRt": 0.0, "IgstAmt": 0.0, "CgstAmt": 0.0, "SgstAmt": 0.0, "CesRt": 0.0, "CesAmt": 0.0,
                 "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
@@ -207,42 +186,42 @@ class TestEdiJson(AccountTestInvoicingCommon):
         #=================================== Negative unit price test =============================================
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_negative_unit_price)
         expected.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00004", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0005", "Dt": "01/01/2019"},
             "ItemList": [
                 {
-                    "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                    "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                     "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 400.0, "AssAmt": 600.0,
                     "GstRt": 5.0, "IgstAmt": 0.0, "CgstAmt": 15.0, "SgstAmt": 15.0, "CesRt": 0.0, "CesAmt": 0.0,
                     "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
                     "OthChrg": 0.0, "TotItemVal": 630.0
                 },
                 {
-                    "SlNo": "3", "PrdDesc": "product_with_cess", "IsServc": "N", "HsnCd": "02222", "Qty": 1.0,
+                    "SlNo": "3", "PrdDesc": "product_with_cess", "IsServc": "N", "HsnCd": "333333", "Qty": 1.0,
                     "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 0.0, "AssAmt": 1000.0,
-                    "GstRt": 12.0, "IgstAmt": 0.0, "CgstAmt": 60.0, "SgstAmt": 60.0, "CesRt": 5.0, "CesAmt": 50.0,
+                    "GstRt": 12.0, "IgstAmt": 0.0, "CgstAmt": 63.1, "SgstAmt": 63.1, "CesRt": 5.0, "CesAmt": 50.0,
                     "CesNonAdvlAmt": 1.59, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
-                    "OthChrg": 0.0, "TotItemVal": 1171.59
+                    "OthChrg": 0.0, "TotItemVal": 1177.79
                 }
             ],
             "ValDtls": {
-                "AssVal": 1600.0, "CgstVal": 75.0, "SgstVal": 75.0, "IgstVal": 0.0, "CesVal": 51.59,
-                "StCesVal": 0.0, "RndOffAmt": 0.0, "TotInvVal": 1801.59
+                "AssVal": 1600.0, "CgstVal": 78.1, "SgstVal": 78.1, "IgstVal": 0.0, "CesVal": 51.59,
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": 0.0, "TotInvVal": 1807.79
             },
         })
         self.assertDictEqual(json_value, expected, "Indian EDI with negative unit price sent json value is not matched")
 
-        expected.update({"DocDtls": {"Typ": "INV", "No": "INV/2019/00005", "Dt": "01/01/2019"}})
+        expected.update({"DocDtls": {"Typ": "INV", "No": "INV/18-19/0006", "Dt": "01/01/2019"}})
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_negative_qty)
         self.assertDictEqual(json_value, expected, "Indian EDI with negative quantity sent json value is not matched")
 
-        expected.update({"DocDtls": {"Typ": "INV", "No": "INV/2019/00006", "Dt": "01/01/2019"}})
+        expected.update({"DocDtls": {"Typ": "INV", "No": "INV/18-19/0007", "Dt": "01/01/2019"}})
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_negative_unit_price_and_qty)
         self.assertDictEqual(json_value, expected, "Indian EDI with negative unit price and quantity sent json value is not matched")
 
         expected.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00007", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0008", "Dt": "01/01/2019"},
             "ItemList": [{
-                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                 "Unit": "UNT", "UnitPrice": 2000.0, "TotAmt": 2000.0, "Discount": 1400.0, "AssAmt": 600.0,
                 "GstRt": 5.0, "IgstAmt": 0.0, "CgstAmt": 15.0, "SgstAmt": 15.0, "CesRt": 0.0, "CesAmt": 0.0,
                 "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
@@ -250,23 +229,23 @@ class TestEdiJson(AccountTestInvoicingCommon):
             }],
             "ValDtls": {
                 "AssVal": 600.0, "CgstVal": 15.0, "SgstVal": 15.0, "IgstVal": 0.0, "CesVal": 0.0,
-                "StCesVal": 0.0, "RndOffAmt": 0.0, "TotInvVal": 630.0
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": 0.0, "TotInvVal": 630.0
             },
         })
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_negative_with_discount)
         self.assertDictEqual(json_value, expected, "Indian EDI with negative unit price and quantity sent json value is not matched")
 
         expected.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00008", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0009", "Dt": "01/01/2019"},
             "ItemList": [{
-                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                "SlNo": "1", "PrdDesc": "product_a", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                 "Unit": "UNT", "UnitPrice": 2000.0, "TotAmt": 2000.0, "Discount": 2000.0, "AssAmt": 0.0,
                 "GstRt": 5.0, "IgstAmt": 0.0, "CgstAmt": 0.0, "SgstAmt": 0.0, "CesRt": 0.0, "CesAmt": 0.0,
                 "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
                 "OthChrg": 0.0, "TotItemVal": 0.0
             },
             {
-                "SlNo": "2", "PrdDesc": "product_a2", "IsServc": "N", "HsnCd": "01111", "Qty": 1.0,
+                "SlNo": "2", "PrdDesc": "product_b", "IsServc": "N", "HsnCd": "111111", "Qty": 1.0,
                 "Unit": "UNT", "UnitPrice": 1000.0, "TotAmt": 1000.0, "Discount": 100.0, "AssAmt": 900.0,
                 "GstRt": 5.0, "IgstAmt": 0.0, "CgstAmt": 22.5, "SgstAmt": 22.5, "CesRt": 0.0, "CesAmt": 0.0,
                 "CesNonAdvlAmt": 0.0, "StateCesRt": 0.0, "StateCesAmt": 0.0, "StateCesNonAdvlAmt": 0.0,
@@ -274,7 +253,7 @@ class TestEdiJson(AccountTestInvoicingCommon):
             }],
             "ValDtls": {
                 "AssVal": 900.0, "CgstVal": 22.5, "SgstVal": 22.5, "IgstVal": 0.0, "CesVal": 0.0,
-                "StCesVal": 0.0, "RndOffAmt": 0.0, "TotInvVal": 945.0
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": 0.0, "TotInvVal": 945.0
             },
         })
         json_value = self.env['account.edi.format']._l10n_in_edi_generate_invoice_json(self.invoice_negative_more_than_max_line)
@@ -282,9 +261,9 @@ class TestEdiJson(AccountTestInvoicingCommon):
 
         json_value = self.env["account.edi.format"]._l10n_in_edi_generate_invoice_json(self.invoice_cash_rounding)
         expected_copy_rounding.update({
-            "DocDtls": {"Typ": "INV", "No": "INV/2019/00009", "Dt": "01/01/2019"},
+            "DocDtls": {"Typ": "INV", "No": "INV/18-19/0010", "Dt": "01/01/2019"},
             "ValDtls": {
-                "AssVal": 1800.0, "CgstVal": 76.5, "SgstVal": 76.5, "IgstVal": 0.0, "CesVal": 46.59,
-                "StCesVal": 0.0, "RndOffAmt": 0.41, "TotInvVal": 2000.00
+                "AssVal": 1800.0, "CgstVal": 79.3, "SgstVal": 79.3, "IgstVal": 0.0, "CesVal": 46.59,
+                "StCesVal": 0.0, "Discount": 0.0, "RndOffAmt": -0.19, "TotInvVal": 2005.00
             }})
         self.assertDictEqual(json_value, expected_copy_rounding, "Indian EDI with cash rounding sent json value is not matched")
