@@ -2,4 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import pos_order
-from . import pos_session
+from . import pos_order_line
+from . import product_product
+from . import account_move
+from . import account_tax
