@@ -13,13 +13,11 @@
     'data': [
         'views/report_sale_order.xml',
         'views/sale_views.xml',
-        'views/res_partner_views.xml',
     ],
     'demo': [
         'data/product_demo.xml',
     ],
     'installable': True,
-    'application': False,
     'auto_install': True,
     'license': 'LGPL-3',
 }
