@@ -3,17 +3,12 @@
 
 {
     'name': 'Italy - E-invoicing',
-    'icon': '/l10n_it/static/description/icon.png',
-    'version': '0.3',
+    'version': '0.4',
     'depends': [
         'l10n_it',
-        # Although account_edi is a dependency of account_edi_proxy_client,
-        # it is here because it's in the auto-install
-        'account_edi',
         'account_edi_proxy_client',
     ],
-    'auto_install': ['l10n_it', 'account_edi'],
-    'author': 'Odoo',
+    'auto_install': ['l10n_it'],
     'description': """
 E-invoice implementation
     """,
@@ -21,16 +16,16 @@ E-invoice implementation
     'website': 'http://www.odoo.com/',
     'data': [
         'security/ir.model.access.csv',
-        'data/account_edi_data.xml',
         'data/invoice_it_template.xml',
         'data/invoice_it_simplified_template.xml',
         'data/ir_cron.xml',
+        'data/account.account.tag.csv',
         'views/res_config_settings_views.xml',
         'views/l10n_it_view.xml',
-        ],
+        'views/report_invoice.xml',
+    ],
     'demo': [
         'data/account_invoice_demo.xml',
     ],
-    'post_init_hook': '_l10n_it_edi_post_init',
     'license': 'LGPL-3',
 }
