@@ -4,3 +4,6 @@ from . import common
 from . import test_edi_export
 from . import test_edi_import
 from . import test_edi_reverse_charge
+from . import test_edi_pa
+from . import test_account_move_send
+from . import test_res_partner
