@@ -1,11 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from . import models
-from odoo import api, SUPERUSER_ID
 
-
-def _create_picking_seq(cr, registry):
-    env = api.Environment(cr, SUPERUSER_ID, {})
+def _create_picking_seq(env):
     ptypes = env['stock.picking.type'].search([('code', '=', 'outgoing'), ('warehouse_id', '!=', False)])
     for ptype in ptypes:
         wh = ptype.warehouse_id
@@ -14,4 +11,4 @@ def _create_picking_seq(cr, registry):
         'prefix': wh.code + '/' + ptype.sequence_code + '/DDT', 'padding': 5,
         'company_id': wh.company_id.id,
         'implementation': 'no_gap',
-    }).id
\ No newline at end of file
+    }).id
