@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 from odoo.addons.sale.tests.common import TestSaleCommon
-from odoo.tests import tagged, Form
+from odoo.tests import Form, tagged
 
 
 @tagged('post_install_l10n', 'post_install', '-at_install')
 class TestDDT(TestSaleCommon):
 
     @classmethod
-    def setUpClass(cls, chart_template_ref='l10n_it.l10n_it_chart_template_generic'):
-        super().setUpClass(chart_template_ref=chart_template_ref)
+    @TestSaleCommon.setup_country('it')
+    def setUpClass(cls):
+        super().setUpClass()
         cls.company_data['company'].write({
                         'vat':"IT12345670017",
-                        'country_id': cls.env.ref('base.it').id,
                         'l10n_it_codice_fiscale': '01234560157',
                         'l10n_it_tax_system': 'RF01',
                         'street': 'Via Giovanni Maria Platina 66',
@@ -32,16 +32,9 @@ class TestDDT(TestSaleCommon):
         })
 
         settings = cls.env['res.config.settings'].create({})
-        if hasattr(settings, 'button_create_proxy_user'):
+        if hasattr(settings, '_create_proxy_user'):
             # Needed when `l10n_it_edi_sdiscoop` is installed
-            settings.button_create_proxy_user()
-
-    @classmethod
-    def setup_company_data(cls, company_name, **kwargs):
-        return super().setup_company_data(company_name, **{
-            **kwargs,
-            'country_id': cls.env.ref('base.it').id,
-        })
+            settings._create_proxy_user(cls.company_data['company'], 'demo')
 
     def test_ddt_flow(self):
         """
@@ -72,10 +65,8 @@ class TestDDT(TestSaleCommon):
 
         # deliver partially
         pick = self.so.picking_ids
-        pick.move_ids.write({'quantity_done': 1})
-        wiz_act = pick.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        pick.move_ids.write({'quantity': 1, 'picked': True})
+        Form.from_action(self.env, pick.button_validate()).save().process()
 
         self.assertTrue(pick.l10n_it_ddt_number, 'The outgoing picking should have a DDT number')
         self.inv1 = self.so._create_invoices()
@@ -84,17 +75,13 @@ class TestDDT(TestSaleCommon):
 
         # deliver partially
         pickx1 = self.so.picking_ids.filtered(lambda p: p.state != 'done')
-        pickx1.move_ids.write({'quantity_done': 1})
-        wiz_act = pickx1.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        pickx1.move_ids.write({'quantity': 1, 'picked': True})
+        Form.from_action(self.env, pickx1.button_validate()).save().process()
 
         # and again
         pickx2 = self.so.picking_ids.filtered(lambda p: p.state != 'done')
-        pickx2.move_ids.write({'quantity_done': 2})
-        wiz_act = pickx2.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        pickx2.move_ids.write({'quantity': 2, 'picked': True})
+        Form.from_action(self.env, pickx2.button_validate()).save().process()
 
         self.inv2 = self.so._create_invoices()
         self.inv2.action_post()
@@ -129,17 +116,15 @@ class TestDDT(TestSaleCommon):
 
         # deliver partially
         picking_1 = so.picking_ids
-        picking_1.move_ids.write({'quantity_done': 1})
-        wiz_act = picking_1.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        picking_1.move_ids.write({'quantity': 1, 'picked': True})
+        Form.from_action(self.env, picking_1.button_validate()).save().process()
 
         invoice_1 = so._create_invoices()
         invoice_1.invoice_line_ids[0].quantity = 1.0
         invoice_1.action_post()
 
         picking_2 = so.picking_ids.filtered(lambda p: p.state != 'done')
-        picking_2.move_ids.write({'quantity_done': 2})
+        picking_2.move_ids.write({'quantity': 2, 'picked': True})
         picking_2.button_validate()
 
         invoice_2 = so._create_invoices()
