@@ -1,5 +1,5 @@
 from . import account_move
-from . import product
+from . import account_move_send
 from . import res_company
 from . import res_config_settings
 from . import res_partner
