@@ -1,11 +1,12 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
     'name': 'Peru - Accounting',
-    "version": "2.0",
-    'summary': "PCGE Simplified",
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['pe'],
+    'version': '3.0',
     'category': 'Accounting/Localizations/Account Charts',
-    'author': 'Vauxoo, Odoo',
-    'website': 'https://www.odoo.com/documentation/16.0/applications/finance/accounting/fiscal_localizations/localizations/peru.html',
+    'author': 'Vauxoo, Odoo S.A.',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations/peru.html',
     'license': 'LGPL-3',
     'depends': [
         'base_vat',
@@ -13,23 +14,19 @@
         'l10n_latam_base',
         'l10n_latam_invoice_document',
         'account_debit_note',
+        'account',
     ],
+    'auto_install': ['account'],
     'data': [
         'security/ir.model.access.csv',
         'views/account_tax_view.xml',
-        'data/l10n_pe_chart_data.xml',
-        'data/account.group.template.csv',
-        'data/account.account.template.csv',
-        'data/l10n_pe_chart_post_data.xml',
-        'data/account_tax_group_data.xml',
-        'data/account_tax_data.xml',
-        'data/fiscal_position_data.xml',
+        'views/res_bank_view.xml',
         'data/l10n_latam_document_type_data.xml',
-        'data/account_chart_template_data.xml',
         'data/res.city.csv',
         'data/l10n_pe.res.city.district.csv',
         'data/res_country_data.xml',
         'data/l10n_latam_identification_type_data.xml',
+        'data/res.bank.csv',
     ],
     'demo': [
         'demo/demo_company.xml',
