@@ -1,12 +1,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
-# Copyright (C) 2009 - now Grzegorz Grzelak grzegorz.grzelak@openglobe.pl
-
 {
-    'name' : 'Poland - Accounting',
-    'version' : '2.0',
-    'author': 'Odoo S.A., Grzegorz Grzelak (OpenGLOBE)',
-    'website': 'http://www.openglobe.pl',
+    'name': 'Poland - Accounting',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['pl'],
+    'version': '2.0',
+    'author': 'Odoo S.A., Grzegorz Grzelak (OpenGLOBE) (http://www.openglobe.pl)',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
 This is the module to manage the accounting chart and taxes for Poland in Odoo.
@@ -19,22 +18,22 @@ zakładając, że wszystkie towary są w obrocie hurtowym.
 Niniejszy moduł jest przeznaczony dla odoo 8.0.
 Wewnętrzny numer wersji OpenGLOBE 1.02
     """,
-    'depends' : [
-        'account',
+    'depends': [
         'base_iban',
         'base_vat',
+        'account',
     ],
+    'auto_install': ['account'],
     'data': [
-              'data/l10n_pl_chart_data.xml',
-              'data/account.account.template.csv',
-              'data/account.group.template.csv',
-              'data/res.country.state.csv',
-              'data/l10n_pl_chart_post_data.xml',
-              'data/account_tax_group_data.xml',
-              'data/account_tax_report_data.xml',
-              'data/account_tax_data.xml',
-              'data/account_fiscal_position_data.xml',
-              'data/account_chart_template_data.xml'
+        'security/ir.model.access.csv',
+        'data/l10n_pl.l10n_pl_tax_office.csv',
+        'data/res.country.state.csv',
+        'data/account.account.tag.csv',
+        'data/account_tax_report_data.xml',
+        'views/account_move_views.xml',
+        'views/product_views.xml',
+        'views/res_config_settings_views.xml',
+        'views/res_partner_views.xml',
     ],
     'demo': [
         'demo/demo_company.xml',
