@@ -3,20 +3,20 @@
 
 {
     'name': 'Saudi Arabia - E-invoicing',
-    'icon': '/l10n_sa/static/description/icon.png',
+    'countries': ['sa'],
     'version': '0.1',
     'depends': [
+        'account_edi',
         'account_edi_ubl_cii',
         'account_debit_note',
         'l10n_sa',
-        'base_vat'
+        'base_vat',
+        'certificate',
     ],
     'author': 'Odoo',
-    'summary': """
-        E-Invoicing, Universal Business Language
-    """,
+    'summary': "E-Invoicing, Universal Business Language",
     'description': """
-        E-invoice implementation for the Kingdom of Saudi Arabia
+E-invoice implementation for the Kingdom of Saudi Arabia
     """,
     'category': 'Accounting/Localizations/EDI',
     'license': 'LGPL-3',
@@ -26,6 +26,7 @@
         'data/ubl_21_zatca.xml',
         'data/res_country_data.xml',
         'wizard/l10n_sa_edi_otp_wizard.xml',
+        'wizard/account_move_reversal_views.xml',
         'views/account_tax_views.xml',
         'views/account_journal_views.xml',
         'views/res_partner_views.xml',
