@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
     'name': 'Saudi Arabia - Point of Sale',
-    'author': 'Odoo S.A',
     'category': 'Accounting/Localizations/Point of Sale',
     'description': """
 K.S.A. POS Localization
@@ -14,10 +13,9 @@ K.S.A. POS Localization
         'l10n_sa',
     ],
     'assets': {
-        'point_of_sale.assets': [
+        'point_of_sale._assets_pos': [
             'web/static/lib/zxing-library/zxing-library.js',
-            'l10n_sa_pos/static/src/js/models.js',
-            'l10n_sa_pos/static/src/xml/OrderReceipt.xml',
+            'l10n_sa_pos/static/src/**/*',
         ]
     },
     'auto_install': True,
