@@ -13,7 +13,7 @@ class ResCompany(models.Model):
     @api.depends('vat')
     def _compute_org_number(self):
         for company in self:
-            if company.country_id.code == "SE" and company.vat:
+            if company.account_fiscal_country_id.code == "SE" and company.vat:
                 org_number = re.sub(r'\D', '', company.vat)[:-2]
                 org_number = org_number[:6] + '-' + org_number[6:]
 
