@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
 {
     'name': 'Thailand - Accounting',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['th'],
     'version': '2.0',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
@@ -11,17 +11,15 @@ Chart of Accounts for Thailand.
 
 Thai accounting chart and localization.
     """,
-    'author': 'Almacom',
-    'website': 'http://almacom.co.th/',
-    'depends': ['account'],
+    'author': 'Almacom (http://almacom.co.th/)',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations/thailand.html',
+    'depends': [
+        'account_qr_code_emv',
+        'account',
+    ],
+    'auto_install': ['account'],
     'data': [
-        'data/account_tax_group_data.xml',
-        'data/l10n_th_chart_data.xml',
-        'data/account.account.template.csv',
-        'data/l10n_th_chart_post_data.xml',
         'data/account_tax_report_data.xml',
-        'data/account_tax_template_data.xml',
-        'data/account_chart_template_data.xml',
         'views/report_invoice.xml',
     ],
     'demo': [
