@@ -2,14 +2,15 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
-    'name': 'United States - Accounting',
+    'name': 'United States - Localizations',
+    'website': 'https://www.odoo.com/documentation/master/applications/finance/fiscal_localizations.html',
+    'icon': '/account/static/description/l10n.png',
+    'countries': ['us'],
     'version': '1.1',
     'category': 'Accounting/Localizations/Account Charts',
     'description': """
-United States - Chart of accounts.
-==================================
     """,
-    'depends': ['l10n_generic_coa'],
+    'depends': ['base'],
     'data': [
         'data/res_company_data.xml',
         'views/res_partner_bank_views.xml'
