@@ -2,8 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import loyalty_card
+from . import loyalty_history
 from . import loyalty_mail
 from . import loyalty_reward
 from . import loyalty_rule
 from . import loyalty_program
 from . import product_product
+from . import product_template
+from . import res_partner
