@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 {
     'name': 'Lunch',
     'sequence': 300,
@@ -51,8 +49,8 @@ If you want to save your employees' time and avoid them to always have coins in
         'web.assets_tests': [
             'lunch/static/tests/tours/*.js',
         ],
-        'web.qunit_suite_tests': [
-            'lunch/static/tests/lunch_kanban_tests.js',
+        'web.assets_unit_tests': [
+            'lunch/static/tests/**/*.test.js',
         ],
     },
     'license': 'LGPL-3',
