@@ -5,6 +5,7 @@ from collections import defaultdict
 import itertools
 
 from odoo import api, fields, models, Command
+from odoo.addons.mail.tools.discuss import Store
 
 
 class Followers(models.Model):
@@ -29,7 +30,7 @@ class Followers(models.Model):
     res_id = fields.Many2oneReference(
         'Related Document ID', index=True, help='Id of the followed resource', model_field='res_model')
     partner_id = fields.Many2one(
-        'res.partner', string='Related Partner', index=True, ondelete='cascade', required=True, domain=[('type', '!=', 'private')])
+        'res.partner', string='Related Partner', index=True, ondelete='cascade', required=True)
     subtype_ids = fields.Many2many(
         'mail.message.subtype', string='Subtype',
         help="Message subtypes followed, meaning subtypes that will be pushed onto the user's Wall.")
@@ -74,6 +75,33 @@ class Followers(models.Model):
     # Private tools methods to fetch followers data
     # --------------------------------------------------
 
+    @api.model
+    def _get_mail_recipients_follower_status(self, mail_ids):
+        """ Get partner mail recipients that follows the related record of the mails.
+
+        Note that followers for message related to discuss.channel are not fetched.
+
+        :param list mail_ids: mail_mail ids
+        :return: followers of the related record of the mails limited to the
+            recipients of the mails as a set of tuple (model, res_id, partner_id).
+        :rtype: set
+        """
+        self.env['mail.mail'].flush_model(['message_id', 'recipient_ids'])
+        self.env['mail.followers'].flush_model(['partner_id', 'res_model', 'res_id'])
+        self.env['mail.message'].flush_model(['model', 'res_id'])
+        # mail_mail_res_partner_rel is the join table for the m2m recipient_ids field
+        self.env.cr.execute("""
+            SELECT message.model, message.res_id, mail_partner.res_partner_id
+              FROM mail_mail mail        
+              JOIN mail_mail_res_partner_rel mail_partner ON mail_partner.mail_mail_id = mail.id
+              JOIN mail_message message ON mail.mail_message_id = message.id AND message.model != 'discuss.channel'
+              JOIN mail_followers follower ON message.model = follower.res_model 
+               AND message.res_id = follower.res_id 
+               AND mail_partner.res_partner_id = follower.partner_id
+             WHERE mail.id IN %(mail_ids)s
+        """, {'mail_ids': tuple(mail_ids)})
+        return set(self.env.cr.fetchall())
+
     def _get_recipient_data(self, records, message_type, subtype_id, pids=None):
         """ Private method allowing to fetch recipients data based on a subtype.
         Purpose of this method is to fetch all data necessary to notify recipients
@@ -85,9 +113,9 @@ class Followers(models.Model):
 
         :param records: fetch data from followers of ``records`` that follow
           ``subtype_id``;
-        :param message_type: mail.message.message_type in order to allow custom
+        :param str message_type: mail.message.message_type in order to allow custom
           behavior depending on it (SMS for example);
-        :param subtype_id: mail.message.subtype to check against followers;
+        :param int subtype_id: mail.message.subtype to check against followers;
         :param pids: additional set of partner IDs from which to fetch recipient
           data independently from following status;
 
@@ -322,6 +350,8 @@ class Followers(models.Model):
           share status of partner (returned only if include_pshare is True)
           active flag status of partner (returned only if include_active is True)
         """
+        self.env['mail.followers'].flush_model(['partner_id', 'res_id', 'res_model', 'subtype_ids'])
+        self.env['res.partner'].flush_model(['active', 'partner_share'])
         # base query: fetch followers of given documents
         where_clause = ' OR '.join(['fol.res_model = %s AND fol.res_id IN %s'] * len(doc_data))
         where_params = list(itertools.chain.from_iterable((rm, tuple(rids)) for rm, rids in doc_data))
@@ -481,3 +511,28 @@ GROUP BY fol.id%s%s""" % (
                         update[fol_id] = {'subtype_ids': update_cmd}
 
         return new, update
+
+    def _to_store(self, store: Store, fields=None):
+        if fields is None:
+            fields = {
+                "display_name": True,
+                "email": True,
+                "is_active": True,
+                "name": True,
+                "partner_id": True,
+                "partner": None,
+                "thread": [],
+            }
+        for follower in self:
+            data = follower._read_format(
+                [field for field in fields if field not in ["partner", "thread"]], load=False
+            )[0]
+            if "partner" in fields:
+                data["partner"] = Store.one(follower.partner_id, fields=fields["partner"])
+            if "thread" in fields:
+                data["thread"] = Store.one(
+                    self.env[follower.res_model].browse(follower.res_id),
+                    as_thread=True,
+                    only_id=True,
+                )
+            store.add(follower, data)
