@@ -3,6 +3,7 @@
 
 from odoo import api, fields, models, tools, _
 from odoo.exceptions import AccessError, UserError
+from odoo.tools import SQL
 
 
 class MailBlackListMixin(models.AbstractModel):
@@ -31,8 +32,7 @@ class MailBlackListMixin(models.AbstractModel):
     _primary_email = 'email'
 
     email_normalized = fields.Char(
-        string='Normalized Email', compute="_compute_email_normalized", compute_sudo=True,
-        store=True, invisible=True,
+        string='Normalized Email', compute="_compute_email_normalized", compute_sudo=True, store=True,
         help="This field is used to search on email address as the primary email field can contain more than strictly an email address.")
     # Note : is_blacklisted sould only be used for display. As the compute is not depending on the blacklist,
     # once read, it won't be re-computed again if the blacklist is modified in the same request.
@@ -47,7 +47,7 @@ class MailBlackListMixin(models.AbstractModel):
     def _compute_email_normalized(self):
         self._assert_primary_email()
         for record in self:
-            record.email_normalized = tools.email_normalize(record[self._primary_email])
+            record.email_normalized = tools.email_normalize(record[self._primary_email], strict=False)
 
     @api.model
     def _search_is_blacklisted(self, operator, value):
@@ -62,25 +62,26 @@ class MailBlackListMixin(models.AbstractModel):
                 raise NotImplementedError()
 
         if value:
-            query = """
+            sql = SQL("""
                 SELECT m.id
                     FROM mail_blacklist bl
                     JOIN %s m
                     ON m.email_normalized = bl.email AND bl.active
-            """
+            """, SQL.identifier(self._table))
         else:
-            query = """
+            sql = SQL("""
                 SELECT m.id
                     FROM %s m
                     LEFT JOIN mail_blacklist bl
                     ON m.email_normalized = bl.email AND bl.active
                     WHERE bl.id IS NULL
-            """
-        self._cr.execute(query % self._table)
+            """, SQL.identifier(self._table))
+
+        self._cr.execute(SQL("%s FETCH FIRST ROW ONLY", sql))
         res = self._cr.fetchall()
         if not res:
             return [(0, '=', 1)]
-        return [('id', 'in', [r[0] for r in res])]
+        return [('id', 'in', SQL("(%s)", sql))]
 
     @api.depends('email_normalized')
     def _compute_is_blacklisted(self):
@@ -113,7 +114,7 @@ class MailBlackListMixin(models.AbstractModel):
     def mail_action_blacklist_remove(self):
         # wizard access rights currently not working as expected and allows users without access to
         # open this wizard, therefore we check to make sure they have access before the wizard opens.
-        can_access = self.env['mail.blacklist'].check_access_rights('write', raise_exception=False)
+        can_access = self.env['mail.blacklist'].has_access('write')
         if can_access:
             return {
                 'name': _('Are you sure you want to unblacklist this Email Address?'),
