@@ -5,6 +5,7 @@ import base64
 import json
 import logging
 import requests
+from markupsafe import Markup
 from werkzeug.exceptions import Forbidden
 
 from odoo import http, tools, _
@@ -117,10 +118,10 @@ class MailPluginController(http.Controller):
 
         partner.write(partner_values)
 
-        partner.message_post_with_view(
+        partner.message_post_with_source(
             'iap_mail.enrich_company',
-            values=iap_data,
-            subtype_id=request.env.ref('mail.mt_note').id,
+            render_values=iap_data,
+            subtype_xmlid='mail.mt_note',
         )
 
         return {
@@ -152,6 +153,18 @@ class MailPluginController(http.Controller):
         if not normalized_email:
             return {'error': _('Bad Email.')}
 
+        notification_emails = request.env['mail.alias.domain'].sudo().search([]).mapped('default_from_email')
+        if normalized_email in notification_emails:
+            return {
+                'partner': {
+                    'name': _('Notification'),
+                    'email': normalized_email,
+                    'enrichment_info': {
+                        'type': 'odoo_custom_error', 'info': _('This is your notification address. Search the Contact manually to link this email to a record.'),
+                    },
+                },
+            }
+
         # Search for the partner based on the email.
         # If multiple are found, take the first one.
         partner = request.env['res.partner'].search(['|', ('email', 'in', [normalized_email, email]),
@@ -170,7 +183,7 @@ class MailPluginController(http.Controller):
             }
             company = self._find_existing_company(normalized_email)
 
-            can_create_partner = request.env['res.partner'].check_access_rights('create', raise_exception=False)
+            can_create_partner = request.env['res.partner'].has_access('create')
 
             if not company and can_create_partner:  # create and enrich company
                 company, enrichment_info = self._create_company_from_iap(normalized_email)
@@ -193,7 +206,7 @@ class MailPluginController(http.Controller):
         if normalized_email:
             filter_domain = [('email_normalized', 'ilike', search_term)]
         else:
-            filter_domain = ['|', '|', ('display_name', 'ilike', search_term), ('ref', '=', search_term),
+            filter_domain = ['|', '|', ('complete_name', 'ilike', search_term), ('ref', '=', search_term),
                              ('email', 'ilike', search_term)]
 
         # Search for the partner based on the email.
@@ -214,6 +227,9 @@ class MailPluginController(http.Controller):
         params name: name of the new partner
         params company: parent company id of the new partner
         """
+        notification_emails = request.env['mail.alias.domain'].sudo().search([]).mapped('default_from_email')
+        if tools.email_normalize(email) in notification_emails:
+            raise Forbidden()
         # old route name "/mail_client_extension/partner/create is deprecated as of saas-14.3,it is not needed for newer
         # versions of the mail plugin but necessary for supporting older versions
         # TODO search the company again instead of relying on the one provided here?
@@ -250,7 +266,7 @@ class MailPluginController(http.Controller):
                 for name, content in attachments
             ]
 
-        request.env[model].browse(res_id).message_post(body=message, attachments=attachments)
+        request.env[model].browse(res_id).message_post(body=Markup(message), attachments=attachments)
         return True
 
     @http.route('/mail_plugin/get_translations', type="json", auth="outlook", cors="*")
@@ -262,7 +278,7 @@ class MailPluginController(http.Controller):
         Returns enrichment data for a given domain, in case an error happens the response will
         contain an enrichment_info key explaining what went wrong
         """
-        if domain in iap_tools._MAIL_DOMAIN_BLACKLIST:
+        if domain in iap_tools._MAIL_PROVIDERS:
             # Can not enrich the provider domain names (gmail.com; outlook.com, etc)
             return {'enrichment_info': {'type': 'missing_data'}}
 
@@ -298,8 +314,7 @@ class MailPluginController(http.Controller):
             return {'id': -1}
 
         try:
-            company.check_access_rights('read')
-            company.check_access_rule('read')
+            company.check_access('read')
         except AccessError:
             return {'id': company.id, 'name': _('No Access')}
 
@@ -362,10 +377,10 @@ class MailPluginController(http.Controller):
 
         new_company = request.env['res.partner'].create(new_company_info)
 
-        new_company.message_post_with_view(
+        new_company.message_post_with_source(
             'iap_mail.enrich_company',
-            values=iap_data,
-            subtype_id=request.env.ref('mail.mt_note').id,
+            render_values=iap_data,
+            subtype_xmlid='mail.mt_note',
         )
 
         return new_company, {'type': 'company_created'}
@@ -380,17 +395,15 @@ class MailPluginController(http.Controller):
         partner_values['enrichment_info'] = None
 
         try:
-            partner.check_access_rights('write')
-            partner.check_access_rule('write')
+            partner.check_access('write')
             partner_values['can_write_on_partner'] = True
         except AccessError:
             partner_values['can_write_on_partner'] = False
 
         if not partner_values['name']:
             # Always ensure that the partner has a name
-            name, email = request.env['res.partner']._parse_partner_name(
-                partner_values['email'])
-            partner_values['name'] = name or email
+            name, email_normalized = tools.parse_contact_from_email(partner_values['email'])
+            partner_values['name'] = name or email_normalized
 
         return partner_values
 
@@ -413,8 +426,7 @@ class MailPluginController(http.Controller):
         return {
             'partner': partner_response,
             'user_companies': request.env.user.company_ids.ids,
-            'can_create_partner': request.env['res.partner'].check_access_rights(
-                'create', raise_exception=False),
+            'can_create_partner': request.env['res.partner'].has_access('create'),
         }
 
     def _mail_content_logging_models_whitelist(self):
