@@ -4,7 +4,7 @@
 {
     'name': 'Email Marketing',
     'summary': 'Design, send and track emails',
-    'version': '2.5',
+    'version': '2.7',
     'sequence': 60,
     'website': 'https://www.odoo.com/app/email-marketing',
     'category': 'Marketing/Email Marketing',
@@ -14,7 +14,6 @@
         'utm',
         'link_tracker',
         'web_editor',
-        'web_kanban_gauge',
         'social_media',
         'web_tour',
         'digest',
@@ -28,8 +27,11 @@
         'data/ir_cron_data.xml',
         'data/ir_module_data.xml',
         'data/mailing_data_templates.xml',
-        'data/mailing_list_data.xml',
+        'data/mailing_list_contact.xml',
+        'data/mailing_subscription_optout.xml',
+        'data/mailing_subscription.xml',
         'data/res_users_data.xml',
+        'data/mass_mailing_tour.xml',
         'wizard/mail_compose_message_views.xml',
         'wizard/mailing_contact_import_views.xml',
         'wizard/mailing_contact_to_list_views.xml',
@@ -37,17 +39,19 @@
         'wizard/mailing_mailing_test_views.xml',
         'wizard/mailing_mailing_schedule_date_views.xml',
         'report/mailing_trace_report_views.xml',
+        'views/mail_blacklist_views.xml',
         'views/mailing_filter_views.xml',
+        'views/mailing_mobile_preview_content.xml',
         'views/mailing_trace_views.xml',
         'views/link_tracker_views.xml',
         'views/mailing_contact_views.xml',
-        'views/mailing_contact_subscription_views.xml',
         'views/mailing_list_views.xml',
         'views/mailing_mailing_views.xml',
+        'views/mailing_subscription_optout_views.xml',
+        'views/mailing_subscription_views.xml',
         'views/res_config_settings_views.xml',
         'views/utm_campaign_views.xml',
         'views/mailing_menus.xml',
-        'views/assets.xml',
         'views/mailing_templates_portal_layouts.xml',
         'views/mailing_templates_portal_management.xml',
         'views/mailing_templates_portal_unsubscribe.xml',
@@ -81,30 +85,57 @@
         'views/snippets/s_title.xml',
     ],
     'demo': [
-        'data/mass_mailing_demo.xml',
+        'demo/utm.xml',
+        'demo/mailing_list_contact.xml',
+        'demo/mailing_subscription.xml',
+        'demo/mailing_mailing.xml',
+        'demo/mailing_trace.xml',
     ],
     'application': True,
     'assets': {
+        'mass_mailing.iframe_css_assets_edit': [
+            ('include', 'mass_mailing.assets_mail_themes'),
+            ('include', 'web.assets_frontend'),
+            ('after', 'web/static/lib/bootstrap/scss/_maps.scss', 'mass_mailing/static/src/scss/mass_mailing.ui.scss'),
+            ('include', 'web_editor.backend_assets_wysiwyg'),
+            ('include', 'mass_mailing.assets_snippets_menu'),
+
+            'mass_mailing/static/src/scss/mass_mailing_mail.scss',
+        ],
+        'mass_mailing.iframe_css_assets_readonly': [
+            'mass_mailing/static/src/scss/mass_mailing_mail.scss',
+            'mass_mailing/static/src/css/basic_theme_readonly.css'
+        ],
         'mass_mailing.mailing_assets': [
             'mass_mailing/static/src/scss/mailing_portal.scss',
-            'mass_mailing/static/src/js/mailing_portal.js',
+            'mass_mailing/static/src/js/mailing_portal_subscription.js',
+            'mass_mailing/static/src/js/mailing_portal_subscription_blocklist.js',
+            'mass_mailing/static/src/js/mailing_portal_subscription_feedback.js',
+            'mass_mailing/static/src/js/mailing_portal_subscription_form.js',
+            'mass_mailing/static/src/xml/mailing_portal_subscription_blocklist.xml',
+            'mass_mailing/static/src/xml/mailing_portal_subscription_feedback.xml',
+            'mass_mailing/static/src/xml/mailing_portal_subscription_form.xml',
+        ],
+        'web_editor.backend_assets_wysiwyg': [
+            'mass_mailing/static/src/js/mass_mailing_wysiwyg.js',
+            'mass_mailing/static/src/scss/mass_mailing.wysiwyg.scss',
         ],
         'web.assets_backend': [
             'mass_mailing/static/src/scss/mailing_filter_widget.scss',
             'mass_mailing/static/src/scss/mass_mailing.scss',
             'mass_mailing/static/src/scss/mass_mailing_mobile.scss',
             'mass_mailing/static/src/scss/mass_mailing_mobile_preview.scss',
-            'mass_mailing/static/src/css/email_template.css',
-            'mass_mailing/static/src/views/*.js',
             'mass_mailing/static/src/js/mailing_m2o_filter.js',
-            'mass_mailing/static/src/js/mass_mailing.js',
             'mass_mailing/static/src/js/mass_mailing_design_constants.js',
             'mass_mailing/static/src/js/mass_mailing_mobile_preview.js',
             'mass_mailing/static/src/js/mass_mailing_html_field.js',
-            'mass_mailing/static/src/js/mailing_mailing_view_form_full_width.js',
             'mass_mailing/static/src/xml/mailing_filter_widget.xml',
             'mass_mailing/static/src/xml/mass_mailing.xml',
-            'mass_mailing/static/src/views/*.xml',
+            'mass_mailing/static/src/xml/mass_mailing_mobile_preview.xml',
+            'mass_mailing/static/src/js/tours/**/*',
+        ],
+        'web.assets_backend_lazy': [
+            'mass_mailing/static/src/views/mass_mailing_subscription_graph_renderer.js',
         ],
         'mass_mailing.assets_mail_themes': [
             'mass_mailing/static/src/scss/themes/**/*',
@@ -113,20 +144,28 @@
             ('include', 'web._assets_helpers'),
             'web/static/src/scss/pre_variables.scss',
             'web/static/lib/bootstrap/scss/_variables.scss',
+            'web/static/lib/bootstrap/scss/_variables-dark.scss',
+            'web/static/lib/bootstrap/scss/_maps.scss',
             'mass_mailing/static/src/scss/mass_mailing.ui.scss',
         ],
-        'web_editor.assets_wysiwyg': [
-            'mass_mailing/static/src/js/snippets.editor.js',
-            'mass_mailing/static/src/js/wysiwyg.js',
-            'mass_mailing/static/src/xml/mass_mailing.editor.xml',
-            'mass_mailing/static/src/scss/mass_mailing.wysiwyg.scss',
+        'mass_mailing.assets_wysiwyg': [
+            'mass_mailing/static/src/js/mass_mailing_snippets.js',
+            'mass_mailing/static/src/snippets/s_masonry_block/options.js',
+            'mass_mailing/static/src/snippets/s_media_list/options.js',
+            'mass_mailing/static/src/snippets/s_showcase/options.js',
+            'mass_mailing/static/src/snippets/s_rating/options.js'
         ],
-        'web.assets_common': [
-            'mass_mailing/static/src/js/tours/**/*',
+        'mass_mailing.assets_snippets_menu': [
+            ('include', 'web_editor.assets_snippets_menu'),
+            'mass_mailing/static/src/js/snippets.editor.js',
+            'mass_mailing/static/src/xml/mass_mailing.editor.xml',
         ],
         'web.assets_frontend': [
             'mass_mailing/static/src/js/tours/**/*',
         ],
+        'web.assets_tests': [
+            'mass_mailing/static/tests/tours/**/*',
+        ],
         'web.qunit_suite_tests': [
             'mass_mailing/static/tests/mass_mailing_favourite_filter_tests.js',
             'mass_mailing/static/src/js/mass_mailing_snippets.js',
