@@ -13,5 +13,6 @@ class ResCompany(models.Model):
             'social_facebook': self.social_facebook,
             'social_linkedin': self.social_linkedin,
             'social_twitter': self.social_twitter,
-            'social_instagram': self.social_instagram
+            'social_instagram': self.social_instagram,
+            'social_tiktok': self.social_tiktok,
         }
