@@ -1,7 +1,6 @@
-odoo.define('mass_mailing.s_showcase_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const options = require('web_editor.snippets.options');
+import options from "@web_editor/js/editor/snippets.options";
 
 options.registry.Showcase = options.Class.extend({
     /**
@@ -16,4 +15,3 @@ options.registry.Showcase = options.Class.extend({
         $title.find('.s_showcase_icon').toggleClass('me-lg-0 ms-lg-3', isLeftCol);
     },
 });
-});
