@@ -1,8 +1,9 @@
-/** @odoo-module alias=mass_mailing.FieldMassMailingFavoriteFilter.test */
+/** @odoo-module */
 
 import { makeView, setupViewRegistries } from "@web/../tests/views/helpers";
 import * as testUtils from "@web/../tests/helpers/utils";
-import weTestUtils from "web_editor.test_utils";
+import weTestUtils from "@web_editor/../tests/test_utils";
+import * as dsHelpers from "@web/../tests/core/domain_selector_tests";
 
 let fixture;
 let serverData;
@@ -129,9 +130,9 @@ QUnit.module('favorite filter widget', (hooks) => {
                         widget="mailing_filter"
                         options="{'no_create': '1', 'no_open': '1', 'domain_field': 'mailing_domain', 'model': 'mailing_model_id'}"/>
                 </form>`,
-            mockRPC: function (route, args) {
-                if (args.method === 'create' && args.model === 'mailing.filter') {
-                    assert.deepEqual(args.args,
+            mockRPC: function (_, { args, model, method }) {
+                if (method === 'create' && model === 'mailing.filter') {
+                    assert.deepEqual(args[0],
                         [{mailing_domain: '[["new_user","=",True]]', mailing_model_id: 1, name: 'event promo - new users'}],
                         "should pass correct data in create");
                 }
@@ -145,8 +146,8 @@ QUnit.module('favorite filter widget', (hooks) => {
         assert.isVisible(fixture.querySelector('.o_mass_mailing_save_filter_container'),
             "should have option to save filter if no filter is set");
         await testUtils.click(fixture.querySelector('.o_field_mailing_filter input'));
-        assert.containsOnce($dropdown, 'li.ui-menu-item',
-            "there should be only one existing filter");
+        assert.containsN($dropdown, 'li.ui-menu-item', 2,
+            "there should be only one existing filter and a search more btn");
         // create a new filter
         await testUtils.click(fixture, '.o_mass_mailing_add_filter');
         fixture.querySelector('.o_mass_mailing_filter_name').value = 'event promo - new users';
@@ -166,8 +167,8 @@ QUnit.module('favorite filter widget', (hooks) => {
         // Ensures input is not focussed otherwise clicking on it will just close the dropdown instead of opening it
         fixture.querySelector('.o_field_mailing_filter .o_input_dropdown input').blur();
         await testUtils.click(fixture.querySelector('.o_field_mailing_filter input'));
-        assert.containsN($dropdown, 'li.ui-menu-item', 2,
-            "there should be two existing filters");
+        assert.containsN($dropdown, 'li.ui-menu-item', 3,
+            "there should be two existing filters and a search more btn");
         await testUtils.clickSave(fixture);
     });
 
@@ -193,7 +194,7 @@ QUnit.module('favorite filter widget', (hooks) => {
             mockRPC: function (route, args) {
                 if (args.method === 'unlink' && args.model === 'mailing.filter') {
                     assert.deepEqual(args.args[0], [1], "should pass correct filter ID for deletion");
-                } else if (args.method === 'write' && args.model === 'mailing.mailing') {
+                } else if (args.method === 'web_save' && args.model === 'mailing.mailing') {
                     assert.strictEqual(args.args[1].mailing_filter_id,
                         false, "filter id should be");
                     assert.strictEqual(args.args[1].mailing_domain,
@@ -289,7 +290,8 @@ QUnit.module('favorite filter widget', (hooks) => {
         await testUtils.click(fixture.querySelector('.o_field_mailing_filter input'));
         fixture.querySelector('.o_field_mailing_filter input').autocomplete = 'widget';
         const $dropdown = fixture.querySelector('.o_field_mailing_filter .dropdown');
-        await testUtils.click($dropdown.lastElementChild, 'li');
+        await testUtils.click($dropdown.lastElementChild, 'li:first-of-type');
+        await testUtils.nextTick();
         assert.equal(fixture.querySelector('.o_domain_show_selection_button').textContent.trim(), '1 record(s)',
             "applied filter should only display single record (only Azure)");
         await testUtils.clickSave(fixture);
@@ -377,7 +379,7 @@ QUnit.module('favorite filter widget', (hooks) => {
             "should show icon to save the filter because domain is set in the mailing");
 
         // If domain is not set on mailing and no filter available, both drop-down and icon container are hidden
-        await testUtils.click(fixture.querySelector('.o_domain_delete_node_button'));
+        await dsHelpers.clickOnButtonDeleteNode(fixture);
         assert.isNotVisible(fixture.querySelector('.o_field_mailing_filter .o_input_dropdown'),
             "should not display drop-down because there is still no filter available to select from");
         assert.isNotVisible(fixture.querySelector('.o_mass_mailing_filter_container'),
@@ -400,7 +402,7 @@ QUnit.module('favorite filter widget', (hooks) => {
         assert.isNotVisible(fixture.querySelector('.o_mass_mailing_save_filter_container'),
             "should not have option to save filter if filter is selected");
 
-        await testUtils.click(fixture.querySelector('.o_domain_add_node_button'));
+        await dsHelpers.clickOnButtonAddNewRule(fixture);
         await testUtils.nextTick();
         assert.isVisible(fixture.querySelector('.o_mass_mailing_save_filter_container'),
             "should have option to save filter because mailing domain is changed");
@@ -408,6 +410,44 @@ QUnit.module('favorite filter widget', (hooks) => {
             "should not have option to remove filter because mailing domain is changed");
     });
 
+    QUnit.test('filter widget does not raise traceback when losing focus with unexpected domain format', async (assert) => {
+        assert.expect(4);
+
+        await makeView({
+            type: "form",
+            resModel: "mailing.mailing",
+            resId: 2,
+            serverData,
+            arch: `<form>
+                    <field name="display_name"/>
+                    <field name="subject"/>
+                    <field name="mailing_domain"/>
+                    <field name="mailing_model_name" invisible="1"/>
+                    <field name="mailing_model_id"/>
+                    <field name="mailing_filter_count"/>
+                    <field name="mailing_filter_domain" invisible="1"/>
+                    <field name="mailing_filter_id"
+                        widget="mailing_filter"
+                        options="{'no_create': '1', 'no_open': '1', 'domain_field': 'mailing_domain', 'model': 'mailing_model_id'}"/>
+                </form>`,
+        });
+
+        // Initial state of icons with no filter
+        assert.isVisible(fixture.querySelector('.o_mass_mailing_save_filter_container'),
+            "should have option to save filter if no filter is set");
+        assert.isNotVisible(fixture.querySelector('.o_mass_mailing_remove_filter'));
+
+        // Set incorrect domain format
+        await testUtils.editInput(fixture, "div[name='mailing_domain'] input", "[");
+        // Wait to lose the focus
+        await testUtils.nextTick();
+
+        assert.isNotVisible(fixture.querySelector('.o_mass_mailing_save_filter_container'),
+            "should not have option to save filter if domain format is incorrect");
+        assert.isNotVisible(fixture.querySelector('.o_mass_mailing_remove_filter'),
+            "should still not be visible");
+    });
+
     QUnit.test('filter widget works in edit and readonly', async (assert) => {
         assert.expect(4);
 
@@ -458,7 +498,7 @@ QUnit.module('favorite filter widget', (hooks) => {
                     <field name="display_name"/>
                     <field name="subject"/>
                     <field name="mailing_model_name" invisible="1"/>
-                    <field name="mailing_model_id" attrs="{'readonly': [('state', '!=', 'draft')]}"/>
+                    <field name="mailing_model_id" readonly="state != 'draft'"/>
                     <field name="mailing_filter_count" />
                     <field name="mailing_filter_id" widget="mailing_filter" options="{'no_create': '1', 'no_open': '1', 'domain_field': 'mailing_domain', 'model': 'mailing_model_id'}"/>
                     <field name="state" widget="statusbar" options="{'clickable' : '1'}"/>
@@ -471,13 +511,13 @@ QUnit.module('favorite filter widget', (hooks) => {
         await testUtils.nextTick();
         const selectField = fixture.querySelector("button[data-value='running']");
         assert.containsOnce(fixture, "div[name='mailing_model_id']:not(.o_readonly_modifier)");
-        assert.ok(fixture.querySelector(".o_mass_mailing_save_filter_container").checkVisibility());
+        assert.ok(fixture.querySelector(".o_mass_mailing_save_filter_container:not(.d-none)"));
         // set to 'running'
         selectField.dispatchEvent(new Event('click'));
         selectField.dispatchEvent(new Event('change'));
         await testUtils.nextTick();
         assert.containsOnce(fixture, "div[name='mailing_model_id'].o_readonly_modifier");
-        assert.ok(fixture.querySelector(".o_mass_mailing_save_filter_container").checkVisibility());
+        assert.ok(fixture.querySelector(".o_mass_mailing_save_filter_container:not(.d-none)"));
     });
 });
 });
