@@ -30,7 +30,7 @@ class MailingSMSController(http.Controller):
     def blacklist_page(self, mailing_id, trace_code, **post):
         check_res = self._check_trace(mailing_id, trace_code)
         if not check_res.get('trace'):
-            return request.redirect('/web')
+            return request.redirect('/odoo')
         return request.render('mass_mailing_sms.blacklist_main', {
             'mailing_id': mailing_id,
             'trace_code': trace_code,
@@ -40,23 +40,28 @@ class MailingSMSController(http.Controller):
     def blacklist_number(self, mailing_id, trace_code, **post):
         check_res = self._check_trace(mailing_id, trace_code)
         if not check_res.get('trace'):
-            return request.redirect('/web')
-        country_code = request.geoip.get('country_code')
+            return request.redirect('/odoo')
         # parse and validate number
         sms_number = post.get('sms_number', '').strip(' ')
-        sanitize_res = phone_validation.phone_sanitize_numbers([sms_number], country_code, None)[sms_number]
-        tocheck_number = sanitize_res['sanitized'] or sms_number
-
-        trace = check_res['trace'].filtered(lambda r: r.sms_number == tocheck_number)[:1]
-        mailing_list_ids = trace.mass_mailing_id.contact_list_ids
+        country = request.env['res.country'].search([('code', '=', request.geoip.country_code)], limit=1)
+        sanitized = phone_validation.phone_format(
+            sms_number,
+            country.code,
+            country.phone_code,
+            force_format='E164',
+            raise_exception=False,
+        )
+        tocheck_number = sanitized or sms_number
 
+        trace = check_res['trace'].filtered(lambda r: r.sms_number == tocheck_number)[:1] if tocheck_number else False
         # compute opt-out / blacklist information
         lists_optout = request.env['mailing.list'].sudo()
         lists_optin = request.env['mailing.list'].sudo()
         unsubscribe_error = False
         if tocheck_number and trace:
+            mailing_list_ids = trace.mass_mailing_id.contact_list_ids
             if mailing_list_ids:
-                subscriptions = request.env['mailing.contact.subscription'].sudo().search([
+                subscriptions = request.env['mailing.subscription'].sudo().search([
                     ('list_id', 'in', mailing_list_ids.ids),
                     ('contact_id.phone_sanitized', '=', tocheck_number),
                 ])
@@ -65,9 +70,9 @@ class MailingSMSController(http.Controller):
             else:
                 blacklist_rec = request.env['phone.blacklist'].sudo().add(tocheck_number)
                 blacklist_rec._message_log(
-                    body=_('Blacklist through SMS Marketing unsubscribe (mailing ID: %s - model: %s)') %
-                          (trace.mass_mailing_id.id, trace.mass_mailing_id.mailing_model_id.display_name))
-            lists_optin = request.env['mailing.contact.subscription'].sudo().search([
+                    body=_('Blacklist through SMS Marketing unsubscribe (mailing ID: %(mailing_id)s - model: %(model)s)',
+                           mailing_id=trace.mass_mailing_id.id, model=trace.mass_mailing_id.mailing_model_id.display_name))
+            lists_optin = request.env['mailing.subscription'].sudo().search([
                 ('contact_id.phone_sanitized', '=', tocheck_number),
                 ('list_id', 'not in', mailing_list_ids.ids),
                 ('opt_out', '=', False),
@@ -75,7 +80,7 @@ class MailingSMSController(http.Controller):
         elif tocheck_number:
             unsubscribe_error = _('Number %s not found', tocheck_number)
         else:
-            unsubscribe_error = sanitize_res['msg']
+            unsubscribe_error = _('Invalid number %s', post.get('sms_number', ''))
 
         return request.render('mass_mailing_sms.blacklist_number', {
             'mailing_id': mailing_id,
@@ -86,18 +91,17 @@ class MailingSMSController(http.Controller):
             'unsubscribe_error': unsubscribe_error,
         })
 
-    @http.route('/r/<string:code>/s/<int:sms_sms_id>', type='http', auth="public")
-    def sms_short_link_redirect(self, code, sms_sms_id, **post):
-        country_code = request.geoip.get('country_code')
-        if sms_sms_id:
-            trace_id = request.env['mailing.trace'].sudo().search([('sms_sms_id_int', '=', int(sms_sms_id))]).id
+    @http.route('/r/<string:code>/s/<int:sms_id_int>', type='http', auth="public")
+    def sms_short_link_redirect(self, code, sms_id_int, **post):
+        if sms_id_int:
+            trace_id = request.env['mailing.trace'].sudo().search([('sms_id_int', '=', int(sms_id_int))]).id
         else:
             trace_id = False
 
         request.env['link.tracker.click'].sudo().add_click(
             code,
             ip=request.httprequest.remote_addr,
-            country_code=country_code,
+            country_code=request.geoip.country_code,
             mailing_trace_id=trace_id
         )
         redirect_url = request.env['link.tracker'].get_url_from_code(code)
