@@ -3,7 +3,10 @@
 
 import logging
 
+from urllib.parse import urljoin
+
 from odoo import api, fields, models, _
+from odoo.addons.link_tracker.models.link_tracker import LINK_TRACKER_MIN_CODE_LENGTH
 from odoo.exceptions import UserError
 from odoo.osv import expression
 
@@ -29,7 +32,7 @@ class Mailing(models.Model):
     # 'sms_subject' should have the same helper as 'subject' field when 'mass_mailing_sms' installed.
     # otherwise 'sms_subject' will get the old helper from 'mass_mailing' module.
     # overriding 'subject' field helper in this model is not working, since the helper will keep the new value
-    # even when 'mass_mailing_sms' removed (see 'mailing_mailing_view_form_sms' for more details).                    
+    # even when 'mass_mailing_sms' removed (see 'mailing_mailing_view_form_sms' for more details).
     sms_subject = fields.Char(
         'Title', related='subject',
         readonly=False, translate=False,
@@ -52,15 +55,16 @@ class Mailing(models.Model):
     ab_testing_sms_winner_selection = fields.Selection(
         related="campaign_id.ab_testing_sms_winner_selection",
         default="clicks_ratio", readonly=False, copy=True)
+    ab_testing_mailings_sms_count = fields.Integer(related="campaign_id.ab_testing_mailings_sms_count")
 
     @api.depends('mailing_type')
     def _compute_medium_id(self):
         super(Mailing, self)._compute_medium_id()
         for mailing in self:
-            if mailing.mailing_type == 'sms' and (not mailing.medium_id or mailing.medium_id == self.env.ref('utm.utm_medium_email')):
-                mailing.medium_id = self.env.ref('mass_mailing_sms.utm_medium_sms').id
-            elif mailing.mailing_type == 'mail' and (not mailing.medium_id or mailing.medium_id == self.env.ref('mass_mailing_sms.utm_medium_sms')):
-                mailing.medium_id = self.env.ref('utm.utm_medium_email').id
+            if mailing.mailing_type == 'sms' and (not mailing.medium_id or mailing.medium_id == self.env['utm.medium']._fetch_or_create_utm_medium('email')):
+                mailing.medium_id = self.env['utm.medium']._fetch_or_create_utm_medium("sms", module="mass_mailing_sms").id
+            elif mailing.mailing_type == 'mail' and (not mailing.medium_id or mailing.medium_id == self.env['utm.medium']._fetch_or_create_utm_medium("sms", module="mass_mailing_sms")):
+                mailing.medium_id = self.env['utm.medium']._fetch_or_create_utm_medium('email').id
 
     @api.depends('sms_template_id', 'mailing_type')
     def _compute_body_plaintext(self):
@@ -70,23 +74,18 @@ class Mailing(models.Model):
 
     @api.depends('mailing_trace_ids.failure_type')
     def _compute_sms_has_iap_failure(self):
-        failures = ['sms_acc', 'sms_credit'] 
-        if not self.ids:
-            self.sms_has_insufficient_credit = self.sms_has_unregistered_account = False
-        else:
-            traces = self.env['mailing.trace'].sudo().read_group([
-                        ('mass_mailing_id', 'in', self.ids),
-                        ('trace_type', '=', 'sms'),
-                        ('failure_type', 'in', failures)
-            ], ['mass_mailing_id', 'failure_type'], ['mass_mailing_id', 'failure_type'], lazy=False)
+        self.sms_has_insufficient_credit = self.sms_has_unregistered_account = False
+        traces = self.env['mailing.trace'].sudo()._read_group([
+                    ('mass_mailing_id', 'in', self.ids),
+                    ('trace_type', '=', 'sms'),
+                    ('failure_type', 'in', ['sms_acc', 'sms_credit'])
+        ], ['mass_mailing_id', 'failure_type'])
 
-            trace_dict = dict.fromkeys(self.ids, {key: False for key in failures})
-            for t in traces:
-                trace_dict[t['mass_mailing_id'][0]][t['failure_type']] = bool(t['__count'])
-
-            for mail in self:
-                mail.sms_has_insufficient_credit = trace_dict[mail.id]['sms_credit']
-                mail.sms_has_unregistered_account = trace_dict[mail.id]['sms_acc']
+        for mass_mailing, failure_type in traces:
+            if failure_type == 'sms_credit':
+                mass_mailing.sms_has_insufficient_credit = True
+            elif failure_type == 'sms_acc':
+                mass_mailing.sms_has_unregistered_account = True
 
     # --------------------------------------------------
     # ORM OVERRIDES
@@ -122,9 +121,9 @@ class Mailing(models.Model):
 
     def action_test(self):
         if self.mailing_type == 'sms':
-            ctx = dict(self.env.context, default_mailing_id=self.id)
+            ctx = dict(self.env.context, default_mailing_id=self.id, dialog_size='medium')
             return {
-                'name': _('Test SMS marketing'),
+                'name': _('Test Mailing'),
                 'type': 'ir.actions.act_window',
                 'view_mode': 'form',
                 'res_model': 'mailing.sms.test',
@@ -136,7 +135,7 @@ class Mailing(models.Model):
     def _action_view_traces_filtered(self, view_filter):
         action = super(Mailing, self)._action_view_traces_filtered(view_filter)
         if self.mailing_type == 'sms':
-            action['views'] = [(self.env.ref('mass_mailing_sms.mailing_trace_view_tree_sms').id, 'tree'),
+            action['views'] = [(self.env.ref('mass_mailing_sms.mailing_trace_view_tree_sms').id, 'list'),
                                (self.env.ref('mass_mailing_sms.mailing_trace_view_form_sms').id, 'form')]
         return action
 
@@ -173,20 +172,14 @@ class Mailing(models.Model):
         target = self.env[self.mailing_model_real]
 
         partner_fields = []
-        if issubclass(type(target), self.pool['mail.thread.phone']):
+        if isinstance(target, self.pool['mail.thread.phone']):
             phone_fields = ['phone_sanitized']
-        elif issubclass(type(target), self.pool['mail.thread']):
+        else:
             phone_fields = [
-                fname for fname in target._sms_get_number_fields()
+                fname for fname in target._phone_get_number_fields()
                 if fname in target._fields and target._fields[fname].store
             ]
-            partner_fields = target._sms_get_partner_fields()
-        else:
-            phone_fields = []
-            if 'mobile' in target._fields and target._fields['mobile'].store:
-                phone_fields.append('mobile')
-            if 'phone' in target._fields and target._fields['phone'].store:
-                phone_fields.append('phone')
+            partner_fields = target._mail_get_partner_fields()
         partner_field = next(
             (fname for fname in partner_fields if target._fields[fname].store and target._fields[fname].type == 'many2one'),
             False
@@ -244,11 +237,11 @@ class Mailing(models.Model):
             'mass_sms_allow_unsubscribe': self.sms_allow_unsubscribe,
         }
 
-    def action_send_mail(self, res_ids=None):
+    def _action_send_mail(self, res_ids=None):
         mass_sms = self.filtered(lambda m: m.mailing_type == 'sms')
         if mass_sms:
             mass_sms.action_send_sms(res_ids=res_ids)
-        return super(Mailing, self - mass_sms).action_send_mail(res_ids=res_ids)
+        return super(Mailing, self - mass_sms)._action_send_mail(res_ids=res_ids)
 
     def action_send_sms(self, res_ids=None):
         for mailing in self:
@@ -257,12 +250,6 @@ class Mailing(models.Model):
             if res_ids:
                 composer = self.env['sms.composer'].with_context(active_id=False).create(mailing._send_sms_get_composer_values(res_ids))
                 composer._action_send_sms()
-
-            mailing.write({
-                'state': 'done',
-                'sent_date': fields.Datetime.now(),
-                'kpi_mail_required': not mailing.sent_date,
-                })
         return True
 
     # ------------------------------------------------------
@@ -330,6 +317,39 @@ class Mailing(models.Model):
         res.update(super(Mailing, self - sms_mailings).convert_links())
         return res
 
+    def get_sms_link_replacements_placeholders(self):
+        """Get placeholders for replaced links in sms widget for accurate computation of sms counts.
+
+        Reminders and assumptions:
+          * Links wille be transformed to the format "[base_url]/r/[link_tracker_code]/s/[sms_id]".
+          * unsubscribe is formatted as: "\nSTOP SMS : [base_url]/sms/[mailing_id]/[trace_code]".
+
+        :return: Character counts used for links, formatted as `{link: str, unsubscribe: str}`.
+        """
+        if self:
+            self.ensure_one()
+
+        self.check_access('write')
+
+        max_sms = self.env['sms.sms'].sudo().search_read([], ['id'], order='id desc', limit=1)
+        sms_id_length = max(len(str(max_sms[0]['id'])), 5) if max_sms else 5  # Assumes a mailing won't be more than 10⁵ sms at once
+        max_code = self.env['link.tracker.code'].sudo().search_read([], ['code'], order='id DESC', limit=1)
+        code_length = len(max_code[0]['code']) + 1 if max_code else LINK_TRACKER_MIN_CODE_LENGTH
+
+        if self.id:
+            mailing_id_placeholder_length = len(str(self.id))
+        else:
+            max_mailing = self.env['mailing.mailing'].sudo().search_read([], ['id'], order='id DESC', limit=1)
+            mailing_id_placeholder_length = len(str(max_mailing[0]['id'] + 1)) if max_mailing else 1
+        mailing_id_placeholder = 'x' * mailing_id_placeholder_length
+
+        base_url = self.get_base_url()
+        opt_out_url = urljoin(base_url, f"sms/{mailing_id_placeholder}/{'x' * self.env['mailing.trace'].CODE_SIZE}")
+        return {
+            'link': urljoin(base_url, f"r/{'x' * code_length}/s/{'x' * sms_id_length}"),
+            'unsubscribe': f"\n{self.env['sms.composer']._get_unsubscribe_info(opt_out_url)}"
+        }
+
     # ------------------------------------------------------
     # A/B Test Override
     # ------------------------------------------------------
@@ -342,6 +362,7 @@ class Mailing(models.Model):
         values = super()._get_ab_testing_description_values()
         if self.mailing_type == 'sms':
             values.update({
+                'ab_testing_count': self.ab_testing_mailings_sms_count,
                 'ab_testing_winner_selection': self.ab_testing_sms_winner_selection,
             })
         return values
