@@ -11,12 +11,13 @@ class Users(models.Model):
     _inherit = ['res.users']
 
     @api.model
-    def systray_get_activities(self):
+    def _get_activity_groups(self):
         """ Split mass_mailing and mass_mailing_sms activities in systray by 
             removing the single mailing.mailing activity represented and
             doing a new query to split them by mailing_type.
         """
-        activities = super(Users, self).systray_get_activities()
+        activities = super()._get_activity_groups()
+        view_type = self.env['mailing.mailing']._systray_view
         for activity in activities:
             if activity.get('model') == 'mailing.mailing':
                 activities.remove(activity)
@@ -56,6 +57,7 @@ class Users(models.Model):
                             'icon': icon,
                             'total_count': 0, 'today_count': 0, 'overdue_count': 0, 'planned_count': 0,
                             'res_ids': res_ids,
+                            "view_type": view_type,
                         }
                     user_activities[act['mailing_type']]['res_ids'].add(act['res_id'])
                     user_activities[act['mailing_type']]['%s_count' % act['states']] += act['count']
@@ -64,7 +66,6 @@ class Users(models.Model):
 
                 for mailing_type in user_activities.keys():
                     user_activities[mailing_type].update({
-                        'actions': [{'icon': 'fa-clock-o', 'name': 'Summary',}],
                         'domain': json.dumps([['activity_ids.res_id', 'in', list(user_activities[mailing_type]['res_ids'])]])
                     })
                 activities.extend(list(user_activities.values()))
