@@ -59,3 +59,34 @@ class TestMassMailValues(MassSMSCommon):
             'contact_list_ids': [(4, self.mailing_list_1.id), (4, self.mailing_list_2.id)]
         })
         self.assertEqual(literal_eval(mailing.mailing_domain), [('list_ids', 'in', (self.mailing_list_1 | self.mailing_list_2).ids)])
+
+    @users('user_marketing')
+    def test_mailing_get_sms_link_replacements_placeholders(self):
+        """Test the extra chars counts for estimating total SMS characters length.
+
+          * links are transformed into "[base_url]/r/[link_tracker_code]/s/[sms_id]".
+          * opt-out link is formatted as "\nSTOP SMS : [base_url]/sms/[mailing_id]/[trace_code]".
+        See `get_sms_link_replacements_placeholders` for safety characters.
+        """
+        mailing = self.env['mailing.mailing'].create({
+            'body_plaintext': 'Coucou hibou',
+            'mailing_model_id': self.env['ir.model']._get('res.partner').id,
+            'mailing_type': 'sms',
+            'name': 'TestMailing',
+            'subject': 'Test',
+        })
+        base_url = self.env['mailing.mailing'].get_base_url()
+
+        link_trackers = bool(self.env['link.tracker'].search([], limit=1))  # depends on demo
+
+        expected = {
+            'link': f'{base_url}/r/xxx{"x" if link_trackers else ""}/s/xxxxx',
+            'unsubscribe': f"\nSTOP SMS: {base_url}/sms/{'x' * len(str(mailing.id))}/{'x' * self.env['mailing.trace'].CODE_SIZE}",
+        }
+        self.assertDictEqual(mailing.get_sms_link_replacements_placeholders(), expected)
+
+        no_mailing = self.env['mailing.mailing']
+        self.assertDictEqual(no_mailing.get_sms_link_replacements_placeholders(), expected)
+
+        new_mailing = self.env['mailing.mailing'].new()
+        self.assertDictEqual(new_mailing.get_sms_link_replacements_placeholders(), expected)
