@@ -4,14 +4,14 @@
 import logging
 import pytz
 import re
-from datetime import datetime
+from datetime import datetime, timedelta
 from dateutil.parser import parse
 from dateutil.relativedelta import relativedelta
 
 from odoo import api, fields, models, _
 from odoo.exceptions import UserError, ValidationError
-from odoo.tools import is_html_empty, email_normalize
-from odoo.addons.microsoft_calendar.utils.event_id_storage import combine_ids
+from odoo.tools import email_normalize
+from odoo.osv import expression
 
 ATTENDEE_CONVERTER_O2M = {
     'needsAction': 'notresponded',
@@ -38,8 +38,6 @@ class Meeting(models.Model):
     _name = 'calendar.event'
     _inherit = ['calendar.event', 'microsoft.calendar.sync']
 
-    # contains organizer event id and universal event id separated by a ':'
-    microsoft_id = fields.Char('Microsoft Calendar Event Id')
     microsoft_recurrence_master_id = fields.Char('Microsoft Recurrence Master Id')
 
     def _get_organizer(self):
@@ -49,11 +47,18 @@ class Meeting(models.Model):
     def _get_microsoft_synced_fields(self):
         return {'name', 'description', 'allday', 'start', 'date_end', 'stop',
                 'user_id', 'privacy',
-                'attendee_ids', 'alarm_ids', 'location', 'show_as', 'active'}
+                'attendee_ids', 'alarm_ids', 'location', 'show_as', 'active', 'videocall_location'}
 
     @api.model
     def _restart_microsoft_sync(self):
-        self.env['calendar.event'].search(self._get_microsoft_sync_domain()).write({
+        domain = self._get_microsoft_sync_domain()
+
+        # Sync only events created/updated after last sync date (with 5 min of time acceptance).
+        if self.env.user.microsoft_last_sync_date:
+            time_offset = timedelta(minutes=5)
+            domain = expression.AND([domain, [('write_date', '>=', self.env.user.microsoft_last_sync_date - time_offset)]])
+
+        self.env['calendar.event'].with_context(dont_notify=True).search(domain).write({
             'need_sync_m': True,
         })
 
@@ -62,8 +67,15 @@ class Meeting(models.Model):
         Returns True if synchronization with Outlook Calendar is active and False otherwise.
         The 'microsoft_synchronization_stopped' variable needs to be 'False' and Outlook account must be connected.
         """
-        outlook_connected = self.env.user._get_microsoft_calendar_token() and self.env.user._is_microsoft_calendar_valid()
-        return outlook_connected and self.env.user.microsoft_synchronization_stopped is False
+        outlook_connected = self.env.user._get_microsoft_calendar_token()
+        return outlook_connected and self.env.user.sudo().microsoft_synchronization_stopped is False
+
+    def _skip_send_mail_status_update(self):
+        """If microsoft calendar is not syncing, don't send a mail."""
+        user_id = self._get_event_user_m()
+        if self.with_user(user_id)._check_microsoft_sync_status() and user_id._get_microsoft_sync_status() == "sync_active":
+            return True
+        return super()._skip_send_mail_status_update()
 
     @api.model_create_multi
     def create(self, vals_list):
@@ -74,6 +86,12 @@ class Meeting(models.Model):
         if self._check_microsoft_sync_status() and not notify_context and recurrency_in_batch:
             self._forbid_recurrence_creation()
 
+        for vals in vals_list:
+            # If event has a different organizer, check its sync status and verify if the user is listed as attendee.
+            sender_user, partner_ids = self._get_organizer_user_change_info(vals)
+            partner_included = partner_ids and len(partner_ids) > 0 and sender_user.partner_id.id in partner_ids
+            self._check_organizer_validation(sender_user, partner_included)
+
         # for a recurrent event, we do not create events separately but we directly
         # create the recurrency from the corresponding calendar.recurrence.
         # That's why, events from a recurrency have their `need_sync_m` attribute set to False.
@@ -82,6 +100,21 @@ class Meeting(models.Model):
             for vals in vals_list
         ])
 
+    def _check_organizer_validation(self, sender_user, partner_included):
+        """ Check if the proposed event organizer can be set accordingly. """
+        # Edge case: events created or updated from Microsoft should not check organizer validation.
+        change_from_microsoft = self.env.context.get('dont_notify', False)
+        if sender_user and sender_user != self.env.user and not change_from_microsoft:
+            current_sync_status = self._check_microsoft_sync_status()
+            sender_sync_status = self.with_user(sender_user)._check_microsoft_sync_status()
+            if not sender_sync_status and current_sync_status:
+                raise ValidationError(
+                    _("For having a different organizer in your event, it is necessary that "
+                      "the organizer have its Odoo Calendar synced with Outlook Calendar."))
+            elif sender_sync_status and not partner_included:
+                raise ValidationError(
+                    _("It is necessary adding the proposed organizer as attendee before saving the event."))
+
     def _check_recurrence_overlapping(self, new_start):
         """
         Outlook does not allow to modify time fields of an event if this event crosses
@@ -121,7 +154,7 @@ class Meeting(models.Model):
         Suggest user to update recurrences in Outlook due to the Outlook Calendar spam limitation.
         """
         error_msg = _("Due to an Outlook Calendar limitation, recurrence updates must be done directly in Outlook Calendar.")
-        if any(not record.microsoft_id for record in self):
+        if any(not record.ms_universal_event_id for record in self):
             # If any event is not synced, suggest deleting it in Odoo and recreating it in Outlook.
             error_msg = _(
                 "Due to an Outlook Calendar limitation, recurrence updates must be done directly in Outlook Calendar.\n"
@@ -146,24 +179,95 @@ class Meeting(models.Model):
             if not notify_context and recurrence_update_attempt and not 'active' in values:
                 self._forbid_recurrence_update()
 
+        # When changing the organizer, check its sync status and verify if the user is listed as attendee.
+        # Updates from Microsoft must skip this check since changing the organizer on their side is not possible.
+        change_from_microsoft = self.env.context.get('dont_notify', False)
+        deactivated_events_ids = []
+        for event in self:
+            if values.get('user_id') and event.user_id.id != values['user_id'] and not change_from_microsoft:
+                sender_user, partner_ids = event._get_organizer_user_change_info(values)
+                partner_included = sender_user.partner_id in event.attendee_ids.partner_id or sender_user.partner_id.id in partner_ids
+                event._check_organizer_validation(sender_user, partner_included)
+                event._recreate_event_different_organizer(values, sender_user)
+                deactivated_events_ids.append(event.id)
+
         # check a Outlook limitation in overlapping the actual recurrence
         if recurrence_update_setting == 'self_only' and 'start' in values:
             self._check_recurrence_overlapping(values['start'])
 
         # if a single event becomes the base event of a recurrency, it should be first
-        # removed from the Outlook calendar.
-        if 'recurrency' in values and values['recurrency']:
-            for e in self.filtered(lambda e: not e.recurrency and not e.recurrence_id):
-                e._microsoft_delete(e._get_organizer(), e.ms_organizer_event_id, timeout=3)
-                e.microsoft_id = False
+        # removed from the Outlook calendar. Additionaly, checks if synchronization is not paused.
+        if self.env.user._get_microsoft_sync_status() != "sync_paused" and values.get('recurrency'):
+            for event in self:
+                if not event.recurrency and not event.recurrence_id:
+                    event._microsoft_delete(event._get_organizer(), event.microsoft_id, timeout=3)
+                    event.microsoft_id = False
+                    event.ms_universal_event_id = False
 
-        res = super(Meeting, self.with_context(dont_notify=notify_context)).write(values)
+        deactivated_events = self.browse(deactivated_events_ids)
+        # Update attendee status before 'values' variable is overridden in super.
+        attendee_ids = values.get('attendee_ids')
+        if attendee_ids and values.get('partner_ids'):
+            (self - deactivated_events)._update_attendee_status(attendee_ids)
+
+        res = super(Meeting, (self - deactivated_events).with_context(dont_notify=notify_context)).write(values)
+
+        # Deactivate events that were recreated after changing organizer.
+        if deactivated_events:
+            res |= super(Meeting, deactivated_events.with_context(dont_notify=notify_context)).write({**values, 'active': False})
 
         if recurrence_update_setting in ('all_events',) and len(self) == 1 \
            and values.keys() & self._get_microsoft_synced_fields():
             self.recurrence_id.need_sync_m = True
         return res
 
+    def unlink(self):
+        # Forbid recurrent events unlinking from calendar list view with sync active.
+        if self and self._check_microsoft_sync_status():
+            synced_events = self._get_synced_events()
+            change_from_microsoft = self.env.context.get('dont_notify', False)
+            recurrence_deletion = any(ev.recurrency and ev.recurrence_id and ev.follow_recurrence for ev in synced_events)
+            if not change_from_microsoft and recurrence_deletion:
+                self._forbid_recurrence_update()
+        return super().unlink()
+
+    def _recreate_event_different_organizer(self, values, sender_user):
+        """ Copy current event values, delete it and recreate it with the new organizer user. """
+        self.ensure_one()
+        event_copy = {**self.copy_data()[0], 'microsoft_id': False}
+        self.env['calendar.event'].with_user(sender_user).create({**event_copy, **values})
+        if self.ms_universal_event_id:
+            self._microsoft_delete(self._get_organizer(), self.microsoft_id)
+
+    @api.model
+    def _get_organizer_user_change_info(self, values):
+        """ Return the sender user of the event and the partner ids listed on the event values. """
+        sender_user_id = values.get('user_id')
+        if not sender_user_id:
+            sender_user_id = self.env.user.id
+        sender_user = self.env['res.users'].browse(sender_user_id)
+        attendee_values = self._attendees_values(values['partner_ids']) if 'partner_ids' in values else []
+        partner_ids = []
+        if attendee_values:
+            for command in attendee_values:
+                if len(command) == 3 and isinstance(command[2], dict):
+                    partner_ids.append(command[2].get('partner_id'))
+        return sender_user, partner_ids
+
+    def _update_attendee_status(self, attendee_ids):
+        """ Merge current status from 'attendees_ids' with new attendees values for avoiding their info loss in write().
+        Create a dict getting the state of each attendee received from 'attendee_ids' variable and then update their state.
+        :param attendee_ids: List of attendee commands carrying a dict with 'partner_id' and 'state' keys in its third position.
+        """
+        state_by_partner = {}
+        for cmd in attendee_ids:
+            if len(cmd) == 3 and isinstance(cmd[2], dict) and all(key in cmd[2] for key in ['partner_id', 'state']):
+                state_by_partner[cmd[2]['partner_id']] = cmd[2]['state']
+        for attendee in self.attendee_ids:
+            state_update = state_by_partner.get(attendee.partner_id.id)
+            if state_update:
+                attendee.state = state_update
+
     def action_mass_archive(self, recurrence_update_setting):
         # Do not allow archiving if recurrence is synced with Outlook. Suggest updating directly from Outlook.
         self.ensure_one()
@@ -177,6 +281,10 @@ class Meeting(models.Model):
         day_range = int(ICP.get_param('microsoft_calendar.sync.range_days', default=365))
         lower_bound = fields.Datetime.subtract(fields.Datetime.now(), days=day_range)
         upper_bound = fields.Datetime.add(fields.Datetime.now(), days=day_range)
+        # Define 'custom_lower_bound_range' param for limiting old events updates in Odoo and avoid spam on Microsoft.
+        custom_lower_bound_range = ICP.get_param('microsoft_calendar.sync.lower_bound_range')
+        if custom_lower_bound_range:
+            lower_bound = fields.Datetime.subtract(fields.Datetime.now(), days=int(custom_lower_bound_range))
         domain = [
             ('partner_ids.user_ids', 'in', self.env.user.id),
             ('stop', '>', lower_bound),
@@ -211,7 +319,7 @@ class Meeting(models.Model):
             'description': microsoft_event.body and microsoft_event.body['content'],
             'location': microsoft_event.location and microsoft_event.location.get('displayName') or False,
             'user_id': microsoft_event.owner_id(self.env),
-            'privacy': sensitivity_o2m.get(microsoft_event.sensitivity, self.default_get(['privacy'])['privacy']),
+            'privacy': sensitivity_o2m.get(microsoft_event.sensitivity, False),
             'attendee_ids': commands_attendee,
             'allday': microsoft_event.isAllDay,
             'start': start,
@@ -239,7 +347,9 @@ class Meeting(models.Model):
                 values['location'] = False
 
         if with_ids:
-            values['microsoft_id'] = combine_ids(microsoft_event.id, microsoft_event.iCalUId)
+            values['microsoft_id'] = microsoft_event.id
+            values['ms_universal_event_id'] = microsoft_event.iCalUId
+
 
         if microsoft_event.is_recurrent():
             values['microsoft_recurrence_master_id'] = microsoft_event.seriesMasterId
@@ -261,7 +371,8 @@ class Meeting(models.Model):
             stop = parse(microsoft_event.end.get('dateTime')).astimezone(timeZone_stop).replace(tzinfo=None)
         values = default_values or {}
         values.update({
-            'microsoft_id': combine_ids(microsoft_event.id, microsoft_event.iCalUId),
+            'microsoft_id': microsoft_event.id,
+            'ms_universal_event_id': microsoft_event.iCalUId,
             'microsoft_recurrence_master_id': microsoft_event.seriesMasterId,
             'start': start,
             'stop': stop,
@@ -290,7 +401,13 @@ class Meeting(models.Model):
         partners = self.env['mail.thread']._mail_find_partner_from_emails(emails, records=self, force_create=True)
         attendees_by_emails = {a.email: a for a in existing_attendees}
         for email, partner, attendee_info in zip(emails, partners, microsoft_attendees):
-            state = ATTENDEE_CONVERTER_M2O.get(attendee_info.get('status').get('response'), 'needsAction')
+            # Responses from external invitations are stored in the 'responseStatus' field.
+            # This field only carries the current user's event status because Microsoft hides other user's status.
+            if self.env.user.email == email and microsoft_event.responseStatus:
+                attendee_microsoft_status = microsoft_event.responseStatus.get('response', 'none')
+            else:
+                attendee_microsoft_status = attendee_info.get('status').get('response')
+            state = ATTENDEE_CONVERTER_M2O.get(attendee_microsoft_status, 'needsAction')
 
             if email in attendees_by_emails:
                 # Update existing attendees
@@ -385,7 +502,7 @@ class Meeting(models.Model):
 
         if 'description' in fields_to_sync:
             values['body'] = {
-                'content': self.description if not is_html_empty(self.description) else '',
+                'content': self._get_customer_description(),
                 'contentType': "html",
             }
 
@@ -404,6 +521,12 @@ class Meeting(models.Model):
         if 'location' in fields_to_sync:
             values['location'] = {'displayName': self.location or ''}
 
+        if not self.location and 'videocall_location' in fields_to_sync and self._need_video_call():
+            values['isOnlineMeeting'] = True
+            values['onlineMeetingProvider'] = 'teamsForBusiness'
+        else:
+            values['isOnlineMeeting'] = False
+
         if 'alarm_ids' in fields_to_sync:
             alarm_id = self.alarm_ids.filtered(lambda a: a.alarm_type == 'notification')[:1]
             values['isReminderOn'] = bool(alarm_id)
@@ -428,6 +551,11 @@ class Meeting(models.Model):
                 'private': 'private',
                 'confidential': 'confidential',
             }
+            # Set default privacy in event according to the organizer's calendar default privacy if defined.
+            if self.user_id:
+                sensitivity_o2m[False] = sensitivity_o2m.get(self.user_id.calendar_default_privacy)
+            else:
+                sensitivity_o2m[False] = 'normal'
             values['sensitivity'] = sensitivity_o2m.get(self.privacy)
 
         if 'active' in fields_to_sync and not self.active:
@@ -510,8 +638,8 @@ class Meeting(models.Model):
                                     "all attendees must have an email address. However, some events do "
                                     "not respect this condition. As long as the events are incorrect, "
                                     "the calendars will not be synchronized."
-                                    "\nEither update the events/attendees or archive these events %s:"
-                                    "\n%s", details, invalid_events))
+                                    "\nEither update the events/attendees or archive these events %(details)s:"
+                                    "\n%(invalid_events)s", details=details, invalid_events=invalid_events))
 
     def _microsoft_values_occurence(self, initial_values={}):
         values = initial_values
@@ -538,7 +666,22 @@ class Meeting(models.Model):
           2) the organizer is NOT an Odoo user: any attendee should remove the Odoo event.
         """
         user = self.env.user
-        records = self.filtered(lambda e: not e.user_id or e.user_id == user)
+        records = self.filtered(lambda e: not e.user_id or e.user_id == user or user.partner_id in e.partner_ids)
+        for event in records:
+            # remove the tracking data to avoid calling _track_template in the pre-commit phase
+            self.env.cr.precommit.data.pop(f'mail.tracking.create.{event._name}.{event.id}', None)
         super(Meeting, records)._cancel_microsoft()
         attendees = (self - records).attendee_ids.filtered(lambda a: a.partner_id == user.partner_id)
         attendees.do_decline()
+
+    def _get_event_user_m(self, user_id=None):
+        """ Get the user who will send the request to Microsoft (organizer if synchronized and current user otherwise). """
+        self.ensure_one()
+        # Current user must have access to token in order to access event properties (non-public user).
+        current_user_status = self.env.user._get_microsoft_calendar_token()
+        if user_id != self.env.user and current_user_status:
+            if user_id is None:
+                user_id = self.user_id
+            if user_id and self.with_user(user_id).sudo()._check_microsoft_sync_status():
+                return user_id
+        return self.env.user
