@@ -1,25 +1,14 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import models
 
-from odoo.addons.microsoft_calendar.models.microsoft_sync import microsoft_calendar_token
-from odoo.addons.microsoft_calendar.utils.microsoft_calendar import MicrosoftCalendarService
+from odoo import models
 
 
 class Attendee(models.Model):
     _name = 'calendar.attendee'
     _inherit = 'calendar.attendee'
 
-    def _send_mail_to_attendees(self, mail_template, force_send=False):
-        """ Override the super method
-        If not synced with Microsoft Outlook, let Odoo in charge of sending emails
-        Otherwise, Microsoft Outlook will send them
-        """
-        with microsoft_calendar_token(self.env.user.sudo()) as token:
-            if not token:
-                super()._send_mail_to_attendees(mail_template, force_send)
-
     def do_tentative(self):
         # Synchronize event after state change
         res = super().do_tentative()
@@ -43,9 +32,8 @@ class Attendee(models.Model):
         params = {"comment": "", "sendResponse": True}
         # Microsoft prevent user to answer the meeting when they are the organizer
         linked_events = self.event_id._get_synced_events()
-        for event in linked_events.filtered(lambda e: e.user_id != self.env.user):
-            event._microsoft_patch(
-                event._get_organizer(),
-                event.ms_organizer_event_id,
-                event._microsoft_values(["attendee_ids"]),
-            )
+        for event in linked_events:
+            if event._check_microsoft_sync_status() and self.env.user != event.user_id and self.env.user.partner_id in event.partner_ids:
+                if event.recurrency:
+                    event._forbid_recurrence_update()
+                event._microsoft_attendee_answer(answer, params)
