@@ -9,3 +9,5 @@ class ResConfigSettings(models.TransientModel):
 
     cal_microsoft_client_id = fields.Char("Microsoft Client_id", config_parameter='microsoft_calendar_client_id', default='')
     cal_microsoft_client_secret = fields.Char("Microsoft Client_key", config_parameter='microsoft_calendar_client_secret', default='')
+    cal_microsoft_sync_paused = fields.Boolean("Microsoft Synchronization Paused", config_parameter='microsoft_calendar_sync_paused',
+        help="Indicates if synchronization with Outlook Calendar is paused or not.")
