@@ -3,10 +3,11 @@ from datetime import datetime, timedelta
 from markupsafe import Markup
 from unittest.mock import patch, MagicMock
 
+from odoo import fields
+
 from odoo.tests.common import HttpCase
 
 from odoo.addons.microsoft_calendar.models.microsoft_sync import MicrosoftSync
-from odoo.addons.microsoft_calendar.utils.event_id_storage import combine_ids
 
 def mock_get_token(user):
     return f"TOKEN_FOR_USER_{user.id}"
@@ -35,6 +36,7 @@ class TestCommon(HttpCase):
     @patch_api
     def setUp(self):
         super(TestCommon, self).setUp()
+        self.env.user.unpause_microsoft_synchronization()
 
         # prepare users
         self.organizer_user = self.env["res.users"].search([("name", "=", "Mike Organizer")])
@@ -55,6 +57,10 @@ class TestCommon(HttpCase):
                 'partner_id': partner.id,
             })
 
+        # Add token validity with one hour of time window for properly checking the sync status.
+        for user in [self.organizer_user, self.attendee_user]:
+            user.microsoft_calendar_token_validity = fields.Datetime.now() + timedelta(hours=1)
+
         # -----------------------------------------------------------------------------------------
         # To create Odoo events
         # -----------------------------------------------------------------------------------------
@@ -241,7 +247,8 @@ class TestCommon(HttpCase):
             "start": self.start_date,
             "stop": self.end_date,
             "user_id": self.organizer_user,
-            "microsoft_id": combine_ids("123", "456"),
+            "microsoft_id": "123",
+            "ms_universal_event_id": "456",
             "partner_ids": [self.organizer_user.partner_id.id, self.attendee_user.partner_id.id],
         }
         self.expected_odoo_recurrency_from_outlook = {
@@ -258,7 +265,8 @@ class TestCommon(HttpCase):
             'fri': False,
             'interval': self.recurrent_event_interval,
             'month_by': 'date',
-            'microsoft_id': combine_ids('REC123', 'REC456'),
+            "microsoft_id": "REC123",
+            "ms_universal_event_id": "REC456",
             'name': "Every %s Days until %s" % (
                 self.recurrent_event_interval, self.recurrence_end_date.strftime("%Y-%m-%d")
             ),
@@ -397,7 +405,8 @@ class TestCommon(HttpCase):
                 "stop": self.end_date + timedelta(days=i * self.recurrent_event_interval),
                 "until": self.recurrence_end_date.date(),
                 "microsoft_recurrence_master_id": "REC123",
-                'microsoft_id': combine_ids(f"REC123_EVENT_{i+1}", f"REC456_EVENT_{i+1}"),
+                "microsoft_id": f"REC123_EVENT_{i+1}",
+                "ms_universal_event_id": f"REC456_EVENT_{i+1}",
                 "recurrency": True,
                 "follow_recurrence": True,
                 "active": True,
@@ -426,7 +435,8 @@ class TestCommon(HttpCase):
             self.simple_event = self.env["calendar.event"].with_user(self.organizer_user).create(
                 dict(
                     self.simple_event_values,
-                    microsoft_id=combine_ids("123", "456"),
+                    microsoft_id="123",
+                    ms_universal_event_id="456",
                 )
             )
 
@@ -437,7 +447,8 @@ class TestCommon(HttpCase):
                 dict(
                     self.simple_event_values,
                     name=f"event{i}",
-                    microsoft_id=combine_ids(f"e{i}", f"u{i}"),
+                    microsoft_id=f"e{i}",
+                    ms_universal_event_id=f"u{i}"
                 )
                 for i in range(1, 4)
             ])
@@ -464,11 +475,13 @@ class TestCommon(HttpCase):
         # set ids set by Outlook
         if not already_created:
             self.recurrence.with_context(dont_notify=True).write({
-                "microsoft_id": combine_ids("REC123", "REC456"),
+                "microsoft_id": "REC123",
+                "ms_universal_event_id": "REC456"
             })
             for i, e in enumerate(self.recurrence.calendar_event_ids.sorted(key=lambda r: r.start)):
                 e.with_context(dont_notify=True).write({
-                    "microsoft_id": combine_ids(f"REC123_EVENT_{i+1}", f"REC456_EVENT_{i+1}"),
+                    "microsoft_id": f"REC123_EVENT_{i+1}",
+                    "ms_universal_event_id": f"REC456_EVENT_{i+1}",
                     "microsoft_recurrence_master_id": "REC123",
                 })
             self.recurrence.invalidate_recordset()
