@@ -16,13 +16,13 @@
         'data/digest_data.xml',
         'data/mail_templates.xml',
         'data/mrp_data.xml',
+        'data/mail_message_subtype_data.xml',
         'wizard/change_production_qty_views.xml',
         'wizard/mrp_workcenter_block_view.xml',
         'wizard/stock_warn_insufficient_qty_views.xml',
         'wizard/mrp_production_backorder.xml',
         'wizard/mrp_consumption_warning_views.xml',
-        'wizard/mrp_immediate_production_views.xml',
-        'wizard/stock_assign_serial_numbers.xml',
+        'wizard/mrp_batch_produce.xml',
         'wizard/mrp_production_split.xml',
         'views/mrp_views_menus.xml',
         'views/stock_move_views.xml',
@@ -31,19 +31,19 @@
         'views/mrp_bom_views.xml',
         'views/mrp_production_views.xml',
         'views/mrp_routing_views.xml',
+        'views/product_document_views.xml',
         'views/product_views.xml',
         'views/stock_orderpoint_views.xml',
         'views/stock_warehouse_views.xml',
         'views/stock_picking_views.xml',
         'views/mrp_unbuild_views.xml',
-        'views/ir_attachment_view.xml',
         'views/res_config_settings_views.xml',
         'views/stock_scrap_views.xml',
         'report/report_deliveryslip.xml',
         'report/mrp_report_views_main.xml',
         'report/mrp_report_bom_structure.xml',
+        'report/mrp_report_mo_overview.xml',
         'report/mrp_production_templates.xml',
-        'report/report_stock_forecasted.xml',
         'report/report_stock_reception.xml',
         'report/report_stock_rule.xml',
         'report/mrp_zebra_production_templates.xml',
@@ -63,7 +63,7 @@
         'web.assets_tests': [
             'mrp/static/tests/tours/**/*',
         ],
-        'web.qunit_suite_tests': [
+        'web.assets_unit_tests': [
             'mrp/static/tests/**/*',
         ],
     },
