@@ -13,7 +13,6 @@ class ResConfigSettings(models.TransientModel):
         implied_group='mrp.group_mrp_byproducts')
     module_mrp_mps = fields.Boolean("Master Production Schedule")
     module_mrp_plm = fields.Boolean("Product Lifecycle Management (PLM)")
-    module_mrp_workorder = fields.Boolean("Work Orders")
     module_quality_control = fields.Boolean("Quality")
     module_quality_control_worksheet = fields.Boolean("Quality Worksheet")
     module_mrp_subcontracting = fields.Boolean("Subcontracting")
@@ -24,7 +23,16 @@ class ResConfigSettings(models.TransientModel):
     group_mrp_workorder_dependencies = fields.Boolean("Work Order Dependencies", implied_group="mrp.group_mrp_workorder_dependencies")
 
     def set_values(self):
+        routing_before = self.env.user.has_group('mrp.group_mrp_routings')
         super().set_values()
+        if routing_before and not self.group_mrp_routings:
+            self.env['mrp.routing.workcenter'].search([]).active = False
+        elif not routing_before and self.group_mrp_routings:
+            operations = self.env['mrp.routing.workcenter'].search_read([('active', '=', False)], ['id', 'write_date'])
+            last_updated = max((op['write_date'] for op in operations), default=0)
+            if last_updated:
+                op_to_update = self.env['mrp.routing.workcenter'].browse([op['id'] for op in operations if op['write_date'] == last_updated])
+                op_to_update.active = True
         if not self.group_mrp_workorder_dependencies:
             # Disabling this option should not interfere with currently planned productions
             self.env['mrp.bom'].sudo().search([('allow_operation_dependencies', '=', True)]).allow_operation_dependencies = False
@@ -34,19 +42,6 @@ class ResConfigSettings(models.TransientModel):
         if not self.use_manufacturing_lead:
             self.manufacturing_lead = 0.0
 
-    @api.onchange('group_mrp_routings')
-    def _onchange_group_mrp_routings(self):
-        # If we activate 'MRP Work Orders', it means that we need to install 'mrp_workorder'.
-        # The opposite is not always true: other modules (such as 'quality_mrp_workorder') may
-        # depend on 'mrp_workorder', so we should not automatically uninstall the module if 'MRP
-        # Work Orders' is deactivated.
-        # Long story short: if 'mrp_workorder' is already installed, we don't uninstall it based on
-        # group_mrp_routings
-        if self.group_mrp_routings:
-            self.module_mrp_workorder = True
-        else:
-            self.module_mrp_workorder = False
-
     @api.onchange('group_unlocked_by_default')
     def _onchange_group_unlocked_by_default(self):
         """ When changing this setting, we want existing MOs to automatically update to match setting. """
