@@ -2,10 +2,10 @@
 
 from odoo.tests import Form
 from odoo.addons.mail.tests.common import mail_new_test_user
-from odoo.addons.stock.tests import common2
+from odoo.addons.stock.tests.common import TestStockCommon
 
 
-class TestMrpCommon(common2.TestStockCommon):
+class TestMrpCommon(TestStockCommon):
 
     @classmethod
     def generate_mo(cls, tracking_final='none', tracking_base_1='none', tracking_base_2='none', qty_final=5, qty_base_1=4, qty_base_2=1, picking_type_id=False, consumption=False):
@@ -16,17 +16,20 @@ class TestMrpCommon(common2.TestStockCommon):
         """
         product_to_build = cls.env['product.product'].create({
             'name': 'Young Tom',
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
             'tracking': tracking_final,
         })
         product_to_use_1 = cls.env['product.product'].create({
             'name': 'Botox',
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
             'tracking': tracking_base_1,
         })
         product_to_use_2 = cls.env['product.product'].create({
             'name': 'Old Tom',
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
             'tracking': tracking_base_2,
         })
         bom_1 = cls.env['mrp.bom'].create({
@@ -37,8 +40,8 @@ class TestMrpCommon(common2.TestStockCommon):
             'type': 'normal',
             'consumption': consumption if consumption else 'flexible',
             'bom_line_ids': [
-                (0, 0, {'product_id': product_to_use_2.id, 'product_qty': qty_base_2}),
-                (0, 0, {'product_id': product_to_use_1.id, 'product_qty': qty_base_1})
+                (0, 0, {'product_id': product_to_use_2.id, 'product_qty': qty_base_2, 'manual_consumption': tracking_base_2 != 'none'}),
+                (0, 0, {'product_id': product_to_use_1.id, 'product_qty': qty_base_1, 'manual_consumption': tracking_base_1 != 'none'})
             ]})
         mo_form = Form(cls.env['mrp.production'])
         mo_form.product_id = product_to_build
@@ -73,7 +76,8 @@ class TestMrpCommon(common2.TestStockCommon):
 
         # Update demo products
         (cls.product_2 | cls.product_3 | cls.product_4 | cls.product_5 | cls.product_6 | cls.product_7_3 | cls.product_8).write({
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
         })
 
         # User Data: mrp user and mrp manager
@@ -93,9 +97,11 @@ class TestMrpCommon(common2.TestStockCommon):
             notification_type='inbox',
             groups='mrp.group_mrp_manager, stock.group_stock_user, mrp.group_mrp_byproducts, uom.group_uom',
         )
-        # Required for `product_uom_id` to be visible in the view
-        # This class is used by a lot of tests which sets `product_uom_id` on `mrp.production`
-        cls.env.user.groups_id += cls.env.ref('uom.group_uom')
+        # Both groups below are required to make fields `product_uom_id` and
+        # `workorder_ids` to be visible in the view of `mrp.production`. The
+        # field `product_uom_id` must be set by many tests, and subviews of
+        # `workorder_ids` must be present in many tests to create records.
+        cls.env.user.groups_id += cls.env.ref('uom.group_uom') + cls.env.ref('mrp.group_mrp_routings')
 
         cls.workcenter_1 = cls.env['mrp.workcenter'].create({
             'name': 'Nuclear Workcenter',
@@ -216,7 +222,8 @@ class TestMrpCommon(common2.TestStockCommon):
             'name': 'Acoustic Bloc Screens',
             'uom_id': cls.env.ref("uom.product_uom_unit").id,
             'uom_po_id': cls.env.ref("uom.product_uom_unit").id,
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
             'tracking': 'none',
             'categ_id': cls.env.ref('product.product_category_all').id,
         })
@@ -224,7 +231,37 @@ class TestMrpCommon(common2.TestStockCommon):
             'name': 'Individual Workplace',
             'uom_id': cls.env.ref("uom.product_uom_unit").id,
             'uom_po_id': cls.env.ref("uom.product_uom_unit").id,
-            'type': 'product',
+            'type': 'consu',
+            'is_storable': True,
             'tracking': 'none',
             'categ_id': cls.env.ref('product.product_category_all').id,
         })
+
+    @classmethod
+    def make_prods(cls, n):
+        return [
+            cls.env["product.product"].create(
+                {"name": f"p{k + 1}", 'is_storable': True}
+            )
+            for k in range(n)
+        ]
+
+    @classmethod
+    def make_bom(cls, p, *cs):
+        return cls.env["mrp.bom"].create(
+            {
+                "product_tmpl_id": p.product_tmpl_id.id,
+                "product_id": p.id,
+                "product_qty": 1,
+                "type": "phantom",
+                "product_uom_id": cls.uom_unit.id,
+                "bom_line_ids": [
+                    (0, 0, {
+                        "product_id": c.id,
+                        "product_qty": 1,
+                        "product_uom_id": cls.uom_unit.id
+                    })
+                    for c in cs
+                ],
+            }
+        )
