@@ -3,6 +3,7 @@
 
 from odoo.tests import Form, tagged
 from odoo.addons.mrp.tests.common import TestMrpCommon
+from odoo import Command
 
 
 @tagged('post_install', '-at_install')
@@ -29,7 +30,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         product_form.name = 'Stick'
         product_form.uom_id = cls.uom_unit
         product_form.uom_po_id = cls.uom_unit
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.route_ids.clear()
         product_form.route_ids.add(cls.warehouse.manufacture_pull_id.route_id)
         product_form.route_ids.add(cls.warehouse.mto_pull_id.route_id)
@@ -38,7 +39,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         # Create raw product for manufactured product
         product_form = Form(cls.env['product.product'])
         product_form.name = 'Raw Stick'
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.uom_id = cls.uom_unit
         product_form.uom_po_id = cls.uom_unit
         cls.raw_product = product_form.save()
@@ -85,12 +86,12 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.lot_stock_id.id)
 
     def test_02_warehouse_twostep_manufacturing(self):
-        """ Warehouse testing for picking ans store after manufacturing """
+        """ Warehouse testing for picking and store after manufacturing """
         with Form(self.warehouse) as warehouse:
             warehouse.manufacture_steps = 'pbm_sam'
         self._check_location_and_routes()
         self.assertEqual(len(self.warehouse.pbm_route_id.rule_ids), 3)
-        self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.sam_loc_id.id)
+        self.assertEqual(self.warehouse.manufacture_pull_id.location_dest_id.id, self.warehouse.lot_stock_id.id)
 
     def test_manufacturing_3_steps(self):
         """ Test MO/picking before manufacturing/picking after manufacturing
@@ -121,6 +122,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.assertEqual(len(move_finished_ids), 1)
         self.assertEqual(move_finished_ids.product_id, self.finished_product)
         self.assertEqual(move_finished_ids.picking_type_id, self.warehouse.manu_type_id)
+        production.button_mark_done()
         sam_move = move_finished_ids.move_dest_ids
         self.assertEqual(len(sam_move), 1)
         self.assertEqual(sam_move.location_id, self.warehouse.sam_loc_id)
@@ -137,7 +139,6 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         """
         with Form(self.warehouse) as warehouse:
             warehouse.manufacture_steps = 'pbm_sam'
-            warehouse.delivery_steps = 'pick_pack_ship'
         self.warehouse.flush_model()
         self.env.ref('stock.route_warehouse0_mto').active = True
         self.env['stock.quant']._update_available_quantity(self.raw_product, self.warehouse.lot_stock_id, 4.0)
@@ -146,14 +147,16 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             'location_dest_id': self.customer_location,
             'partner_id': self.env['ir.model.data']._xmlid_to_res_id('base.res_partner_4'),
             'picking_type_id': self.warehouse.out_type_id.id,
+            'state': 'draft',
         })
+
         self.env['stock.move'].create({
             'name': self.finished_product.name,
             'product_id': self.finished_product.id,
             'product_uom_qty': 2,
             'product_uom': self.uom_unit.id,
             'picking_id': picking_customer.id,
-            'location_id': self.warehouse.wh_output_stock_loc_id.id,
+            'location_id': self.warehouse.lot_stock_id.id,
             'location_dest_id': self.customer_location,
             'procure_method': 'make_to_order',
             'origin': 'SOURCEDOCUMENT',
@@ -179,14 +182,12 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         ]).picking_id
 
         self.assertTrue(picking_stock_preprod)
-        self.assertTrue(picking_stock_postprod)
+        self.assertFalse(picking_stock_postprod)
         self.assertEqual(picking_stock_preprod.state, 'assigned')
-        self.assertEqual(picking_stock_postprod.state, 'waiting')
         self.assertEqual(picking_stock_preprod.origin, production_order.name, 'The pre-prod origin should be the MO name')
-        self.assertEqual(picking_stock_postprod.origin, 'SOURCEDOCUMENT', 'The post-prod origin should be the SO name')
 
         picking_stock_preprod.action_assign()
-        picking_stock_preprod.move_line_ids.qty_done = 4
+        picking_stock_preprod.move_ids.write({'quantity': 4, 'picked': True})
         picking_stock_preprod._action_done()
 
         self.assertFalse(sum(self.env['stock.quant']._gather(self.raw_product, self.warehouse.lot_stock_id).mapped('quantity')))
@@ -194,7 +195,6 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
         production_order.action_assign()
         self.assertEqual(production_order.reservation_state, 'assigned')
-        self.assertEqual(picking_stock_postprod.state, 'waiting')
 
         produce_form = Form(production_order)
         produce_form.qty_producing = production_order.product_qty
@@ -203,15 +203,16 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
         self.assertFalse(sum(self.env['stock.quant']._gather(self.raw_product, self.warehouse.pbm_loc_id).mapped('quantity')))
 
+        picking_stock_postprod = self.env['stock.move'].search([
+            ('product_id', '=', self.finished_product.id),
+            ('location_id', '=', self.warehouse.sam_loc_id.id),
+            ('location_dest_id', '=', self.warehouse.lot_stock_id.id),
+            ('picking_type_id', '=', self.warehouse.sam_type_id.id)
+        ]).picking_id
+        self.assertTrue(picking_stock_postprod)
         self.assertEqual(picking_stock_postprod.state, 'assigned')
 
-        picking_stock_pick = self.env['stock.move'].search([
-            ('product_id', '=', self.finished_product.id),
-            ('location_id', '=', self.warehouse.lot_stock_id.id),
-            ('location_dest_id', '=', self.warehouse.wh_pack_stock_loc_id.id),
-            ('picking_type_id', '=', self.warehouse.pick_type_id.id)
-        ]).picking_id
-        self.assertEqual(picking_stock_pick.move_ids.move_orig_ids.picking_id, picking_stock_postprod)
+        self.assertEqual(picking_customer.move_ids.move_orig_ids.picking_id, picking_stock_postprod)
 
     def test_cancel_propagation(self):
         """ Test cancelling moves in a 'picking before
@@ -226,6 +227,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             'location_dest_id': self.customer_location,
             'partner_id': self.env['ir.model.data']._xmlid_to_res_id('base.res_partner_4'),
             'picking_type_id': self.warehouse.out_type_id.id,
+            'state': 'draft',
         })
         self.env['stock.move'].create({
             'name': self.finished_product.name,
@@ -247,22 +249,13 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             ('location_dest_id', '=', self.warehouse.pbm_loc_id.id),
             ('picking_type_id', '=', self.warehouse.pbm_type_id.id)
         ])
-        move_stock_postprod = self.env['stock.move'].search([
-            ('product_id', '=', self.finished_product.id),
-            ('location_id', '=', self.warehouse.sam_loc_id.id),
-            ('location_dest_id', '=', self.warehouse.lot_stock_id.id),
-            ('picking_type_id', '=', self.warehouse.sam_type_id.id)
-        ])
 
         self.assertTrue(move_stock_preprod)
-        self.assertTrue(move_stock_postprod)
         self.assertEqual(move_stock_preprod.state, 'assigned')
-        self.assertEqual(move_stock_postprod.state, 'waiting')
 
         move_stock_preprod._action_cancel()
         self.assertEqual(production_order.state, 'confirmed')
         production_order.action_cancel()
-        self.assertTrue(move_stock_postprod.state, 'cancel')
 
     def test_no_initial_demand(self):
         """ Test MO/picking before manufacturing/picking after manufacturing
@@ -294,7 +287,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         ])
         new_product = self.env['product.product'].create({
             'name': 'New product',
-            'type': 'product',
+            'is_storable': True,
         })
         bom.consumption = 'flexible'
         production_form = Form(self.env['mrp.production'])
@@ -340,11 +333,11 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         finished_product = self.env['product.product'].create({
             'name': 'Super Product',
             'route_ids': [(4, self.ref('mrp.route_warehouse0_manufacture'))],
-            'type': 'product',
+            'is_storable': True,
         })
         secondary_product = self.env['product.product'].create({
             'name': 'Secondary',
-            'type': 'product',
+            'is_storable': True,
         })
         component = self.env['product.product'].create({
             'name': 'Component',
@@ -378,12 +371,15 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.env['procurement.group'].run_scheduler()
         mo = self.env['mrp.production'].search([('product_id', '=', finished_product.id)])
         pickings = mo.picking_ids
-        self.assertEqual(len(pickings), 2)
+        self.assertEqual(len(pickings), 1)
 
         preprod_picking = pickings[0] if pickings[0].location_id == warehouse_stock_location else pickings[1]
         self.assertEqual(preprod_picking.location_id, warehouse_stock_location)
         self.assertEqual(preprod_picking.location_dest_id, pre_production_location)
 
+        mo.button_mark_done()
+        pickings = mo.picking_ids
+        self.assertEqual(len(pickings), 2)
         postprod_picking = pickings - preprod_picking
         self.assertEqual(postprod_picking.location_id, post_production_location)
         self.assertEqual(postprod_picking.location_dest_id, warehouse_stock_location)
@@ -393,7 +389,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             ('location_id', '=', post_production_location.id),
             ('location_dest_id', '=', warehouse_stock_location.id),
         ])
-        self.assertEqual(byproduct_postprod_move.state, 'waiting')
+        self.assertEqual(byproduct_postprod_move.state, 'assigned')
         self.assertEqual(byproduct_postprod_move.group_id.name, mo.name)
 
     def test_manufacturing_3_steps_trigger_reordering_rules(self):
@@ -407,7 +403,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         # Create an additional BoM for component
         product_form = Form(self.env['product.product'])
         product_form.name = 'Wood'
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
         self.wood_product = product_form.save()
@@ -478,114 +474,6 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
 
         self.assertEqual(self.bom.bom_line_ids.product_id + self.product_2, mo.picking_ids.move_ids.product_id)
 
-    def test_manufacturing_complex_product_3_steps(self):
-        """ Test MO/picking after manufacturing a complex product which uses
-        manufactured components. Ensure that everything is created and picked
-        correctly.
-        """
-
-        self.warehouse.mto_pull_id.route_id.active = True
-        # Creating complex product which trigger another manifacture
-
-        routes = self.warehouse.manufacture_pull_id.route_id + self.warehouse.mto_pull_id.route_id
-        self.complex_product = self.env['product.product'].create({
-            'name': 'Arrow',
-            'type': 'product',
-            'route_ids': [(6, 0, routes.ids)],
-        })
-
-        # Create raw product for manufactured product
-        self.raw_product_2 = self.env['product.product'].create({
-            'name': 'Raw Iron',
-            'type': 'product',
-            'uom_id': self.uom_unit.id,
-            'uom_po_id': self.uom_unit.id,
-        })
-
-        self.finished_product.route_ids = [(6, 0, routes.ids)]
-
-        # Create bom for manufactured product
-        bom_product_form = Form(self.env['mrp.bom'])
-        bom_product_form.product_id = self.complex_product
-        bom_product_form.product_tmpl_id = self.complex_product.product_tmpl_id
-        with bom_product_form.bom_line_ids.new() as line:
-            line.product_id = self.finished_product
-            line.product_qty = 1.0
-        with bom_product_form.bom_line_ids.new() as line:
-            line.product_id = self.raw_product_2
-            line.product_qty = 1.0
-
-        self.complex_bom = bom_product_form.save()
-
-        with Form(self.warehouse) as warehouse:
-            warehouse.manufacture_steps = 'pbm_sam'
-
-        production_form = Form(self.env['mrp.production'])
-        production_form.product_id = self.complex_product
-        production_form.picking_type_id = self.warehouse.manu_type_id
-        production = production_form.save()
-        production.action_confirm()
-
-        move_raw_ids = production.move_raw_ids
-        self.assertEqual(len(move_raw_ids), 2)
-        sfp_move_raw_id, raw_move_raw_id = move_raw_ids
-        self.assertEqual(sfp_move_raw_id.product_id, self.finished_product)
-        self.assertEqual(raw_move_raw_id.product_id, self.raw_product_2)
-
-        for move_raw_id in move_raw_ids:
-            self.assertEqual(move_raw_id.picking_type_id, self.warehouse.manu_type_id)
-
-            pbm_move = move_raw_id.move_orig_ids
-            self.assertEqual(len(pbm_move), 1)
-            self.assertEqual(pbm_move.location_id, self.warehouse.lot_stock_id)
-            self.assertEqual(pbm_move.location_dest_id, self.warehouse.pbm_loc_id)
-            self.assertEqual(pbm_move.picking_type_id, self.warehouse.pbm_type_id)
-
-        # Check move locations
-        move_finished_ids = production.move_finished_ids
-        self.assertEqual(len(move_finished_ids), 1)
-        self.assertEqual(move_finished_ids.product_id, self.complex_product)
-        self.assertEqual(move_finished_ids.picking_type_id, self.warehouse.manu_type_id)
-        sam_move = move_finished_ids.move_dest_ids
-        self.assertEqual(len(sam_move), 1)
-        self.assertEqual(sam_move.location_id, self.warehouse.sam_loc_id)
-        self.assertEqual(sam_move.location_dest_id, self.warehouse.lot_stock_id)
-        self.assertEqual(sam_move.picking_type_id, self.warehouse.sam_type_id)
-        self.assertFalse(sam_move.move_dest_ids)
-
-        subproduction = self.env['mrp.production'].browse(production.id+1)
-        sfp_pickings = subproduction.picking_ids.sorted('id')
-
-        # SFP Production: 2 pickings, 1 group
-        self.assertEqual(len(sfp_pickings), 2)
-        self.assertEqual(sfp_pickings.mapped('group_id'), subproduction.procurement_group_id)
-
-        # Move Raw Stick - Stock -> Preprocessing
-        picking = sfp_pickings[0]
-        self.assertEqual(len(picking.move_ids), 1)
-        picking.move_ids[0].product_id = self.raw_product
-
-        # Move SFP - PostProcessing -> Stock
-        picking = sfp_pickings[1]
-        self.assertEqual(len(picking.move_ids), 1)
-        picking.move_ids[0].product_id = self.finished_product
-
-        # Main production 2 pickings, 1 group
-        pickings = production.picking_ids.sorted('id')
-        self.assertEqual(len(pickings), 2)
-        self.assertEqual(pickings.mapped('group_id'), production.procurement_group_id)
-
-        # Move 2 components Stock -> Preprocessing
-        picking = pickings[0]
-        self.assertEqual(len(picking.move_ids), 2)
-        picking.move_ids[0].product_id = self.finished_product
-        picking.move_ids[1].product_id = self.raw_product_2
-
-        # Move FP PostProcessing -> Stock
-        picking = pickings[1]
-        self.assertEqual(len(picking.move_ids), 1)
-        picking.product_id = self.complex_product
-
     def test_child_parent_relationship_on_backorder_creation(self):
         """ Test Child Mo and Source Mo in 2/3-step production for reorder
             rules in backorder using order points with the help of run scheduler """
@@ -667,7 +555,7 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
             warehouse.manufacture_steps = 'pbm_sam'
         finished_product = self.env['product.product'].create({
             'name': 'Product',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': manufacturing_route,
         })
         self.env['mrp.bom'].create({
@@ -695,3 +583,223 @@ class TestMultistepManufacturingWarehouse(TestMrpCommon):
         self.assertEqual(len(mo), 1)
         self.assertEqual(mo.product_qty, 1.0)
         self.assertEqual(mo.bom_id, bom_2)
+
+    def test_manufacturing_bom_with_repetitions(self):
+        """
+            Checks that manufacturing orders created to manufacture the components of a BOM
+            are set with the correct quantities when products appear with repetitions.
+                - Create 5 products: product 1,2,3,4 (P1,P2,P3 and P4) and a final product (FP)
+                - Set routes to manifacture on each product
+                - For P1, P2, P3, P4 add a 0:0 reordering rule.
+                - Add a BOM for P2 with 1 unit of P1 as components
+                - Add a BOM for P3 with 1 unit of P2 as components
+                - Add a BOM for P4 with 1 unit of P3 as components
+                - Add a BOM for FP with 3 unit of P4 and 2 units of P3 as components
+        """
+        manufacturing_route = self.env['stock.rule'].search([
+            ('action', '=', 'manufacture')]).route_id
+        products = self.env['product.product'].create([
+            {
+            'name': 'FP',
+            'is_storable': True,
+            'route_ids': manufacturing_route,
+            },
+            {
+            'name': 'P1',
+            'is_storable': True,
+            'route_ids': manufacturing_route,
+            },
+            {
+            'name': 'P2',
+            'is_storable': True,
+            'route_ids': manufacturing_route,
+            },
+            {
+            'name': 'P3',
+            'is_storable': True,
+            'route_ids': manufacturing_route,
+            },
+            {
+            'name': 'P4',
+            'is_storable': True,
+            'route_ids': manufacturing_route,
+            },
+
+        ])
+        self.env['stock.warehouse.orderpoint'].create([
+            {
+            'name': 'My orderpoint',
+            'product_id': i,
+            'product_min_qty': 0,
+            'product_max_qty': 0,
+            } for i in products.ids[1:]
+        ])
+        self.env['mrp.bom'].create([
+            {
+            'product_tmpl_id': products[2].product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': products[2].uom_id.id,
+            'type': 'normal',
+            'bom_line_ids': [
+                Command.create({
+                    'product_id': products[1].id,
+                    'product_qty': 1,
+                })
+            ]},
+            {
+            'product_tmpl_id': products[3].product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': products[3].uom_id.id,
+            'type': 'normal',
+            'bom_line_ids': [
+                Command.create({
+                    'product_id': products[2].id,
+                    'product_qty': 1,
+                })
+            ]},
+            {
+            'product_tmpl_id': products[4].product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': products[4].uom_id.id,
+            'type': 'normal',
+            'bom_line_ids': [
+                Command.create({
+                    'product_id': products[3].id,
+                    'product_qty': 1,
+                })
+            ]},
+            {
+            'product_tmpl_id': products[0].product_tmpl_id.id,
+            'product_qty': 1,
+            'product_uom_id': products[0].uom_id.id,
+            'type': 'normal',
+            'bom_line_ids': [
+                Command.create({
+                    'product_id': products[4].id,
+                    'product_qty': 3,
+                }),
+                Command.create({
+                    'product_id': products[2].id,
+                    'product_qty': 2,
+                }),
+            ]},
+        ])
+        mo = self.env['mrp.production'].create({
+            'product_id': products[0].id,
+            'product_uom_qty': 1,
+        })
+        mo.action_confirm()
+        mo_P1 = self.env['mrp.production'].search([('product_id', '=', products[1].id)])
+        mo_P2 = self.env['mrp.production'].search([('product_id', '=', products[2].id)])
+        self.assertEqual(mo_P1.product_uom_qty, 5.0)
+        self.assertEqual(mo_P2.product_uom_qty, 5.0)
+
+    def test_update_component_qty(self):
+        self.warehouse.manufacture_steps = "pbm"
+        component = self.bom.bom_line_ids.product_id
+        mo = self.env['mrp.production'].create({
+            'product_id': self.bom.product_id.id,
+            'bom_id': self.bom.id,
+            'product_qty': 1,
+            'location_src_id': self.warehouse.pbm_loc_id.id,
+        })
+        mo.action_confirm()
+        self.assertEqual(mo.move_raw_ids.product_uom_qty, 2.0)
+        self.assertEqual(mo.picking_ids.move_ids.product_uom_qty, 2.0)
+        # we require a more components to complete the MO
+        mo_form = Form(mo)
+        with mo_form.move_raw_ids.new() as raw_move:
+            raw_move.product_id = component
+            raw_move.product_uom_qty = 1.0
+        mo = mo_form.save()
+        # check that the related moves qty is correctly updated
+        self.assertEqual(mo.move_raw_ids.product_uom_qty, 3.0)
+        self.assertEqual(mo.picking_ids.move_ids.product_uom_qty, 3.0)
+
+    def test_component_and_byproduct_on_transfers(self):
+        """
+            Checks if transfers is updated when we adding a new byproduct/component
+            after confirm the MO
+        """
+        self.env.user.groups_id += self.env.ref('mrp.group_mrp_byproducts')
+        demo = self.env['product.product'].create({
+            'name': 'DEMO',
+            'route_ids': [(4, self.ref('mrp.route_warehouse0_manufacture'))],
+            'is_storable': True,
+        })
+        comp1 = self.env['product.product'].create({
+            'name': 'COMP1'
+        })
+        comp2 = self.env['product.product'].create({
+            'name': 'COMP2'
+        })
+        bprod1 = self.env['product.product'].create({
+            'name': 'BPROD1'
+        })
+        bprod2 = self.env['product.product'].create({
+            'name': 'BPROD2'
+        })
+
+        warehouse = self.warehouse
+        warehouse.manufacture_steps = 'pbm_sam'
+        warehouse_stock_location = warehouse.lot_stock_id
+
+        self.env['mrp.bom'].create({
+            'product_tmpl_id': demo.product_tmpl_id.id,
+            'product_qty': 1,
+            'bom_line_ids': [(0, 0, {
+                'product_id': comp1.id,
+                'product_qty': 1,
+            })],
+            'byproduct_ids': [(0, 0, {
+                'product_id': bprod1.id,
+                'product_qty': 1,
+            })],
+        })
+
+        self.env['stock.warehouse.orderpoint'].create({
+            'warehouse_id': warehouse.id,
+            'location_id': warehouse_stock_location.id,
+            'product_id': demo.id,
+            'product_min_qty': 2,
+            'product_max_qty': 2,
+        })
+
+        self.env['procurement.group'].run_scheduler()
+        mo = self.env['mrp.production'].search([('product_id', '=', demo.id)])
+        mo.action_confirm()
+
+        mo_form = Form(mo)
+        with mo_form.move_raw_ids.new() as raw_move:
+            raw_move.product_id = comp2
+            raw_move.product_uom_qty = 1.0
+        with mo_form.move_byproduct_ids.new() as byprod_move:
+            byprod_move.product_id = bprod2
+            byprod_move.quantity = 1.0
+        mo = mo_form.save()
+        mo.with_context({'skip_consumption': True}).button_mark_done()
+
+        self.assertEqual(len(mo.picking_ids), 2, "Should have 2 pickings: Components + (Final product and byproducts)")
+        for picking in mo.picking_ids:
+            if demo in [m.product_id for m in picking.move_ids_without_package]:
+                self.assertEqual(len(picking.move_ids_without_package), 3, "Should have 3 moves for: Demo, Bprod1 and Bprod2")
+                self.assertEqual([move.product_id for move in picking.move_ids_without_package.sorted('product_id')], [demo, bprod1, bprod2])
+            else:
+                self.assertEqual(len(picking.move_ids_without_package), 2, "Should have 2 moves for: Comp1 and Comp2")
+                self.assertEqual([move.product_id for move in picking.move_ids_without_package.sorted('product_id')], [comp1, comp2])
+
+    def test_pick_components_uses_shipping_policy_from_picking_type(self):
+        self.warehouse.manufacture_steps = "pbm"
+        pick_components_type = self.warehouse.pbm_type_id
+
+        for move_type in ["direct", "one"]:
+            pick_components_type.move_type = move_type
+
+            mo = self.env["mrp.production"].create({
+                "bom_id": self.bom.id,
+                "location_src_id": self.warehouse.pbm_loc_id.id,
+            })
+            mo.action_confirm()
+
+            self.assertEqual(mo.picking_ids[0].picking_type_id, pick_components_type)
+            self.assertEqual(mo.picking_ids[0].move_type, move_type)
