@@ -3,7 +3,7 @@
 
 from odoo import _, fields, models, api
 from odoo.exceptions import UserError
-from odoo.tools import float_compare, float_round
+from odoo.tools import float_compare, float_is_zero
 
 
 class MrpConsumptionWarning(models.TransientModel):
@@ -33,26 +33,47 @@ class MrpConsumptionWarning(models.TransientModel):
     def action_confirm(self):
         ctx = dict(self.env.context)
         ctx.pop('default_mrp_production_ids', None)
-        action_from_do_finish = False
-        if self.env.context.get('from_workorder'):
-            if self.env.context.get('active_model') == 'mrp.workorder':
-                action_from_do_finish = self.env['mrp.workorder'].browse(self.env.context.get('active_id')).do_finish()
-        action_from_mark_done = self.mrp_production_ids.with_context(ctx, skip_consumption=True).button_mark_done()
-        return action_from_do_finish or action_from_mark_done
+        return self.mrp_production_ids.with_context(ctx, skip_consumption=True).button_mark_done()
 
     def action_set_qty(self):
-        self.mrp_production_ids.action_assign()
+        missing_move_vals = []
+        problem_tracked_products = self.env['product.product']
         for production in self.mrp_production_ids:
-            for move in production.move_raw_ids:
-                rounding = move.product_uom.rounding
-                if float_compare(move.quantity_done, move.should_consume_qty, precision_rounding=rounding) == 0:
+            for line in self.mrp_consumption_warning_line_ids:
+                if line.mrp_production_id != production:
                     continue
-                new_qty = float_round((production.qty_producing - production.qty_produced) * move.unit_factor, precision_rounding=move.product_uom.rounding)
-                if move.has_tracking in ('lot', 'serial'):
-                    if not (production.use_auto_consume_components_lots and
-                            float_compare(move.reserved_availability, new_qty, precision_rounding=move.product_uom.rounding) >= 0):
-                        raise UserError(_('You need to supply Lot/Serial Number'))
-                move.quantity_done = new_qty
+                for move in production.move_raw_ids:
+                    if line.product_id != move.product_id:
+                        continue
+                    qty_expected = line.product_uom_id._compute_quantity(line.product_expected_qty_uom, move.product_uom)
+                    qty_compare_result = float_compare(qty_expected, move.quantity, precision_rounding=move.product_uom.rounding)
+                    if qty_compare_result != 0:
+                        move.quantity = qty_expected
+                    # move should be set to picked to correctly consume the product
+                    move.picked = True
+                    # in case multiple lines with same product => set others to 0 since we have no way to know how to distribute the qty done
+                    line.product_expected_qty_uom = 0
+                # move was deleted before confirming MO or force deleted somehow
+                if not float_is_zero(line.product_expected_qty_uom, precision_rounding=line.product_uom_id.rounding):
+                    missing_move_vals.append({
+                        'product_id': line.product_id.id,
+                        'product_uom': line.product_uom_id.id,
+                        'product_uom_qty': line.product_expected_qty_uom,
+                        'quantity': line.product_expected_qty_uom,
+                        'raw_material_production_id': line.mrp_production_id.id,
+                        'additional': True,
+                        'picked': True,
+                    })
+        if problem_tracked_products:
+            products_list = "".join(f"\n- {product_name}" for product_name in problem_tracked_products.mapped("name"))
+            raise UserError(
+                _(
+                    "Values cannot be set and validated because a Lot/Serial Number needs to be specified for a tracked product that is having its consumed amount increased:%(products)s",
+                    products=products_list,
+                ),
+            )
+        if missing_move_vals:
+            self.env['stock.move'].create(missing_move_vals)
         return self.action_confirm()
 
     def action_cancel(self):
