@@ -12,7 +12,7 @@ class TestBomPriceCommon(common.TransactionCase):
     def _create_product(cls, name, price):
         return cls.Product.create({
             'name': name,
-            'type': 'product',
+            'is_storable': True,
             'standard_price': price,
         })
 
@@ -21,6 +21,8 @@ class TestBomPriceCommon(common.TransactionCase):
         super().setUpClass()
         # Required for `product_uom_id ` to be visible in the view
         cls.env.user.groups_id += cls.env.ref('uom.group_uom')
+        # Required for `product_id ` to be visible in the view
+        cls.env.user.groups_id += cls.env.ref('product.group_product_variant')
         cls.Product = cls.env['product.product']
         cls.Bom = cls.env['mrp.bom']
 
@@ -124,8 +126,8 @@ class TestBomPrice(TestBomPriceCommon):
         self.env['mrp.workcenter.capacity'].create({
             'product_id': self.dining_table.id,
             'workcenter_id': workcenter_1.id,
-            'time_start': 2,
-            'time_stop': 1,
+            'time_start': 17,
+            'time_stop': 16,
         })
 
         self.bom_1.write({
