@@ -4,8 +4,9 @@
 from collections import defaultdict
 
 from odoo import fields, models, api, _
-from odoo.exceptions import UserError, AccessError
-from odoo.tools.float_utils import float_compare, float_is_zero
+from odoo.exceptions import AccessError
+from odoo.tools.float_utils import float_compare, float_is_zero, float_round
+from odoo.tools.misc import OrderedSet
 
 
 class StockMove(models.Model):
@@ -16,18 +17,13 @@ class StockMove(models.Model):
         compute='_compute_show_subcontracting_details_visible'
     )
 
-    @api.depends('is_subcontract', 'move_orig_ids.production_id')
-    def _compute_is_quantity_done_editable(self):
-        super()._compute_is_quantity_done_editable()
-        self.filtered(lambda m: m.is_subcontract).is_quantity_done_editable = False
-
     def _compute_display_assign_serial(self):
         super(StockMove, self)._compute_display_assign_serial()
         for move in self:
             if not move.is_subcontract:
                 continue
             productions = move._get_subcontract_production()
-            if not productions or move.has_tracking != 'serial':
+            if not productions or move.has_tracking == 'none':
                 continue
             if productions._has_tracked_component() or productions[:1].consumption != 'strict':
                 move.display_assign_serial = False
@@ -38,7 +34,7 @@ class StockMove(models.Model):
         for move in self:
             if not move.is_subcontract:
                 continue
-            if float_is_zero(move.quantity_done, precision_rounding=move.product_uom.rounding):
+            if not move.picked or float_is_zero(move.quantity, precision_rounding=move.product_uom.rounding):
                 continue
             productions = move._get_subcontract_production()
             if not productions or (productions[:1].consumption == 'strict' and not productions[:1]._has_tracked_component()):
@@ -53,7 +49,7 @@ class StockMove(models.Model):
         for move in self:
             if not move.is_subcontract:
                 continue
-            if self.env.user.has_group('base.group_portal'):
+            if self.env.user._is_portal():
                 move.show_details_visible = any(not p._has_been_recorded() for p in move._get_subcontract_production())
                 continue
             productions = move._get_subcontract_production()
@@ -62,30 +58,92 @@ class StockMove(models.Model):
             move.show_details_visible = True
         return res
 
-    def copy(self, default=None):
+    def _set_quantity_done(self, qty):
+        to_set_moves = self
+        for move in self:
+            if move.is_subcontract and move._subcontracting_possible_record():
+                # If 'done' quantity is changed through the move, record components as if done through the wizard.
+                move._auto_record_components(qty)
+                to_set_moves -= move
+        if to_set_moves:
+            super(StockMove, to_set_moves)._set_quantity_done(qty)
+
+    def _set_quantity(self):
+        to_set_moves = self
+        for move in self:
+            if move.is_subcontract and move._subcontracting_possible_record():
+                move_line_quantities = sum(move.move_line_ids.filtered(lambda ml: ml.picked).mapped('quantity'))
+                delta_qty = move.quantity - move_line_quantities
+                if float_compare(delta_qty, 0, precision_rounding=move.product_uom.rounding) > 0:
+                    move._auto_record_components(delta_qty)
+                    to_set_moves -= move
+                elif float_compare(delta_qty, 0, precision_rounding=move.product_uom.rounding) < 0:
+                    move.with_context(transfer_qty=True)._reduce_subcontract_order_qty(abs(delta_qty))
+        if to_set_moves:
+            super(StockMove, to_set_moves)._set_quantity()
+
+    def _auto_record_components(self, qty):
         self.ensure_one()
-        if not self.is_subcontract or 'location_id' in default:
-            return super(StockMove, self).copy(default=default)
-        if not default:
-            default = {}
-        default['location_id'] = self.picking_id.location_id.id
-        return super(StockMove, self).copy(default=default)
+        subcontracted_productions = self._get_subcontract_production()
+        production = subcontracted_productions.filtered(lambda p: not p._has_been_recorded())[-1:]
+        if not production:
+            # If new quantity is over the already recorded quantity and we have no open production, then create a new one for the missing quantity.
+            production = subcontracted_productions[-1:]
+            production = production.sudo().with_context(allow_more=True)._split_productions({production: [production.qty_producing, qty]})[-1:]
+        qty = self.product_uom._compute_quantity(qty, production.product_uom_id)
+
+        if production.product_tracking == 'serial':
+            qty = float_round(qty, precision_digits=0, rounding_method='UP')  # Makes no sense to have partial quantities for serial number
+            if float_compare(qty, production.product_qty, precision_rounding=production.product_uom_id.rounding) < 0:
+                remaining_qty = production.product_qty - qty
+                productions = production.sudo()._split_productions({production: ([1] * int(qty)) + [remaining_qty]})[:-1]
+            else:
+                productions = production.sudo().with_context(allow_more=True)._split_productions({production: ([1] * int(qty))})
+
+            for production in productions:
+                production.qty_producing = 1
+                if not production.lot_producing_id:
+                    production.action_generate_serial()
+                production.with_context(cancel_backorder=False).subcontracting_record_component()
+        else:
+            production.qty_producing = qty
+            if float_compare(production.qty_producing, production.product_qty, precision_rounding=production.product_uom_id.rounding) > 0:
+                self.env['change.production.qty'].with_context(skip_activity=True).create({
+                    'mo_id': production.id,
+                    'product_qty': qty
+                }).change_prod_qty()
+            if production.product_tracking == 'lot' and not production.lot_producing_id:
+                production.action_generate_serial()
+            production._set_qty_producing()
+            production.with_context(cancel_backorder=False).subcontracting_record_component()
+
+    def copy_data(self, default=None):
+        default = dict(default or {})
+        vals_list = super().copy_data(default=default)
+        for move, vals in zip(self, vals_list):
+            if 'location_id' in default or not move.is_subcontract:
+                continue
+            vals['location_id'] = move.picking_id.location_id.id
+        return vals_list
 
     def write(self, values):
         """ If the initial demand is updated then also update the linked
         subcontract order to the new quantity.
         """
         self._check_access_if_subcontractor(values)
-        if 'product_uom_qty' in values and self.env.context.get('cancel_backorder') is not False:
-            self.filtered(lambda m: m.is_subcontract and m.state not in ['draft', 'cancel', 'done'])._update_subcontract_order_qty(values['product_uom_qty'])
+        if 'product_uom_qty' in values and self.env.context.get('cancel_backorder') is not False and not self._context.get('extra_move_mode'):
+            self.filtered(
+                lambda m: m.is_subcontract and m.state not in ['draft', 'cancel', 'done']
+                and float_compare(m.product_uom_qty, values['product_uom_qty'], precision_rounding=m.product_uom.rounding) != 0
+            )._update_subcontract_order_qty(values['product_uom_qty'])
         res = super().write(values)
         if 'date' in values:
             for move in self:
                 if move.state in ('done', 'cancel') or not move.is_subcontract:
                     continue
                 move.move_orig_ids.production_id.filtered(lambda p: p.state not in ('done', 'cancel')).write({
-                    'date_planned_finished': move.date,
-                    'date_planned_start': move.date,
+                    'date_start': move.date,
+                    'date_finished': move.date,
                 })
         return res
 
@@ -109,43 +167,43 @@ class StockMove(models.Model):
                 'show_lots_m2o': self.has_tracking != 'none',
                 'show_lots_text': False,
             })
-        elif self.env.user.has_group('base.group_portal'):
-            if self.picking_type_id.show_reserved:
-                action['views'] = [(self.env.ref('mrp_subcontracting.mrp_subcontracting_view_stock_move_operations').id, 'form')]
-            else:
-                action['views'] = [(self.env.ref('mrp_subcontracting.mrp_subcontracting_view_stock_move_nosuggest_operations').id, 'form')]
+        elif self.env.user._is_portal():
+            action['views'] = [(self.env.ref('mrp_subcontracting.mrp_subcontracting_view_stock_move_operations').id, 'form')]
         return action
 
     def action_show_subcontract_details(self):
         """ Display moves raw for subcontracted product self. """
-        moves = self._get_subcontract_production().move_raw_ids
-        tree_view = self.env.ref('mrp_subcontracting.mrp_subcontracting_move_tree_view')
+        moves = self._get_subcontract_production().move_raw_ids.filtered(lambda m: m.state != 'cancel')
+        list_view = self.env.ref('mrp_subcontracting.mrp_subcontracting_move_tree_view')
         form_view = self.env.ref('mrp_subcontracting.mrp_subcontracting_move_form_view')
         ctx = dict(self._context, search_default_by_product=True)
-        if self.env.user.has_group('base.group_portal'):
+        if self.env.user._is_portal():
             form_view = self.env.ref('mrp_subcontracting.mrp_subcontracting_portal_move_form_view')
             ctx.update(no_breadcrumbs=False)
         return {
-            'name': _('Raw Materials for %s') % (self.product_id.display_name),
+            'name': _('Raw Materials for %s', self.product_id.display_name),
             'type': 'ir.actions.act_window',
             'res_model': 'stock.move',
-            'views': [(tree_view.id, 'list'), (form_view.id, 'form')],
+            'views': [(list_view.id, 'list'), (form_view.id, 'form')],
             'target': 'current',
             'domain': [('id', 'in', moves.ids)],
             'context': ctx
         }
 
-    def _set_quantities_to_reservation(self):
-        move_untouchable = self.filtered(lambda m: m.is_subcontract and m._get_subcontract_production()._has_tracked_component())
-        return super(StockMove, self - move_untouchable)._set_quantities_to_reservation()
-
     def _action_cancel(self):
+        productions_to_cancel_ids = OrderedSet()
         for move in self:
             if move.is_subcontract:
-                active_production = move.move_orig_ids.production_id.filtered(lambda p: p.state not in ('done', 'cancel'))
-                moves = self.env.context.get('moves_todo')
-                if not moves or active_production not in moves.move_orig_ids.production_id:
-                    active_production.with_context(skip_activity=True).action_cancel()
+                active_productions = move.move_orig_ids.production_id.filtered(lambda p: p.state not in ('done', 'cancel'))
+                moves_todo = self.env.context.get('moves_todo')
+                not_todo_productions = active_productions.filtered(lambda p: p not in moves_todo.move_orig_ids.production_id) if moves_todo else active_productions
+                if not_todo_productions:
+                    productions_to_cancel_ids.update(not_todo_productions.ids)
+
+        if productions_to_cancel_ids:
+            productions_to_cancel = self.env['mrp.production'].browse(productions_to_cancel_ids)
+            productions_to_cancel.with_context(skip_activity=True).action_cancel()
+
         return super()._action_cancel()
 
     def _action_confirm(self, merge=True, merge_into=False):
@@ -158,17 +216,11 @@ class StockMove(models.Model):
             bom = move._get_subcontract_bom()
             if not bom:
                 continue
-            if float_is_zero(move.product_qty, precision_rounding=move.product_uom.rounding) and\
-                    move.picking_id.immediate_transfer is True:
-                raise UserError(_("To subcontract, use a planned transfer."))
             move.write({
                 'is_subcontract': True,
                 'location_id': move.picking_id.partner_id.with_company(move.company_id).property_stock_subcontractor.id
             })
-            if float_compare(move.product_qty, 0, precision_rounding=move.product_uom.rounding) <= 0:
-                # If a subcontracted amount is decreased, don't create a MO that would be for a negative value.
-                # We don't care if the MO decreases even when done since everything is handled through picking
-                continue
+            move._action_assign()  # Re-reserve as the write on location_id will break the link
         res = super()._action_confirm(merge=merge, merge_into=merge_into)
         for move in res:
             if move.is_subcontract:
@@ -184,8 +236,10 @@ class StockMove(models.Model):
         self.ensure_one()
         production = self._get_subcontract_production()[-1:]
         view = self.env.ref('mrp_subcontracting.mrp_production_subcontracting_form_view')
-        if self.env.user.has_group('base.group_portal'):
+        if self.env.user._is_portal():
             view = self.env.ref('mrp_subcontracting.mrp_production_subcontracting_portal_form_view')
+        context = dict(self._context)
+        context.pop('skip_consumption', False)
         return {
             'name': _('Subcontract'),
             'type': 'ir.actions.act_window',
@@ -195,7 +249,7 @@ class StockMove(models.Model):
             'view_id': view.id,
             'target': 'new',
             'res_id': production.id,
-            'context': self.env.context,
+            'context': context,
         }
 
     def _get_subcontract_bom(self):
@@ -215,6 +269,9 @@ class StockMove(models.Model):
     def _subcontrating_can_be_record(self):
         return self._get_subcontract_production().filtered(lambda p: not p._has_been_recorded() and p.consumption != 'strict')
 
+    def _subcontracting_possible_record(self):
+        return self._get_subcontract_production().filtered(lambda p: p._has_tracked_component() or p.consumption != 'strict')
+
     def _get_subcontract_production(self):
         return self.filtered(lambda m: m.is_subcontract).move_orig_ids.production_id
 
@@ -222,20 +279,16 @@ class StockMove(models.Model):
     def _has_tracked_subcontract_components(self):
         return any(m.has_tracking != 'none' for m in self._get_subcontract_production().move_raw_ids)
 
-    def _prepare_extra_move_vals(self, qty):
-        vals = super(StockMove, self)._prepare_extra_move_vals(qty)
-        vals['location_id'] = self.location_id.id
-        return vals
-
     def _prepare_move_split_vals(self, qty):
         vals = super(StockMove, self)._prepare_move_split_vals(qty)
         vals['location_id'] = self.location_id.id
         return vals
 
-    def _should_bypass_set_qty_producing(self):
-        if (self.production_id | self.raw_material_production_id)._get_subcontract_move():
-            return False
-        return super()._should_bypass_set_qty_producing()
+    def _prepare_procurement_values(self):
+        res = super()._prepare_procurement_values()
+        if self.raw_material_production_id.subcontractor_id:
+            res['warehouse_id'] = self.picking_type_id.warehouse_id
+        return res
 
     def _should_bypass_reservation(self, forced_location=False):
         """ If the move is subcontracted then ignore the reservation. """
@@ -247,19 +300,46 @@ class StockMove(models.Model):
     def _update_subcontract_order_qty(self, new_quantity):
         for move in self:
             quantity_to_remove = move.product_uom_qty - new_quantity
-            productions = move.move_orig_ids.production_id.filtered(lambda p: p.state not in ('done', 'cancel'))[::-1]
-            # Cancel productions until reach new_quantity
-            for production in productions:
-                if quantity_to_remove >= production.product_qty:
-                    quantity_to_remove -= production.product_qty
-                    production.with_context(skip_activity=True).action_cancel()
-                else:
-                    self.env['change.production.qty'].with_context(skip_activity=True).create({
-                        'mo_id': production.id,
-                        'product_qty': production.product_uom_qty - quantity_to_remove
-                    }).change_prod_qty()
+            if not float_is_zero(quantity_to_remove, precision_rounding=move.product_uom.rounding):
+                move._reduce_subcontract_order_qty(quantity_to_remove)
+
+    def _reduce_subcontract_order_qty(self, quantity_to_remove):
+        self.ensure_one()
+        productions = self.move_orig_ids.production_id.filtered(lambda p: p.state not in ('done', 'cancel'))[::-1]
+        wip_production = productions[0] if self._context.get('transfer_qty') and len(productions) > 1 else self.env['mrp.production']
+
+        # Transfer removed qty to WIP production
+        if wip_production:
+            self.env['change.production.qty'].with_context(skip_activity=True).create({
+                'mo_id': wip_production.id,
+                'product_qty': wip_production.product_qty + quantity_to_remove
+            }).change_prod_qty()
+
+        # Cancel productions until reach new_quantity
+        for production in (productions - wip_production):
+            if quantity_to_remove >= production.product_qty:
+                quantity_to_remove -= production.product_qty
+                production.with_context(skip_activity=True).action_cancel()
+            else:
+                if float_is_zero(quantity_to_remove, precision_rounding=production.product_uom_id.rounding):
+                    # No need to do change_prod_qty for no change at all.
+                    break
+                self.env['change.production.qty'].with_context(skip_activity=True).create({
+                    'mo_id': production.id,
+                    'product_qty': production.product_qty - quantity_to_remove
+                }).change_prod_qty()
+                break
 
     def _check_access_if_subcontractor(self, vals):
-        if self.env.user.has_group('base.group_portal') and not self.env.su:
+        if self.env.user._is_portal() and not self.env.su:
             if vals.get('state') == 'done':
                 raise AccessError(_("Portal users cannot create a stock move with a state 'Done' or change the current state to 'Done'."))
+
+    def _is_subcontract_return(self):
+        self.ensure_one()
+        subcontracting_location = self.picking_id.partner_id.with_company(self.company_id).property_stock_subcontractor
+        return (
+                not self.is_subcontract
+                and self.origin_returned_move_id.is_subcontract
+                and self.location_dest_id.id == subcontracting_location.id
+        )
