@@ -36,11 +36,34 @@ export function usePartnerAutocomplete() {
         return checkVATNumber(sanitizeVAT(value));
     }
 
+    function isGSTNumber(value) {
+        // Check if the input is a valid GST number.
+        let isGST = false;
+        if (value && value.length === 15) {
+            const allGSTinRe = [
+                /\d{2}[a-zA-Z]{5}\d{4}[a-zA-Z][1-9A-Za-z][Zz1-9A-Ja-j][0-9a-zA-Z]/, // Normal, Composite, Casual GSTIN
+                /\d{4}[A-Z]{3}\d{5}[UO]N[A-Z0-9]/, // UN/ON Body GSTIN
+                /\d{4}[a-zA-Z]{3}\d{5}NR[0-9a-zA-Z]/, // NRI GSTIN
+                /\d{2}[a-zA-Z]{4}[a-zA-Z0-9]\d{4}[a-zA-Z][1-9A-Za-z][DK][0-9a-zA-Z]/, // TDS GSTIN
+                /\d{2}[a-zA-Z]{5}\d{4}[a-zA-Z][1-9A-Za-z]C[0-9a-zA-Z]/ // TCS GSTIN
+            ];
+
+            isGST = allGSTinRe.some((re) => re.test(value));
+        }
+
+        return isGST;
+    }
+
+    async function isTAXNumber(value) {
+        const isVAT = await isVATNumber(value);
+        const isGST = isGSTNumber(value);
+        return isVAT || isGST;
+    }
+
     async function autocomplete(value) {
         value = value.trim();
 
-        const isVAT = await isVATNumber(value);
-
+        const isVAT = await isTAXNumber(value);
         let odooSuggestions = [];
         let clearbitSuggestions = [];
         return new Promise((resolve, reject) => {
@@ -130,7 +153,7 @@ export function usePartnerAutocomplete() {
     function getCreateData(company) {
         const removeUselessFields = (company) => {
             // Delete attribute to avoid "Field_changed" errors
-            const fields = ['label', 'description', 'domain', 'logo', 'legal_name', 'ignored', 'email', 'bank_ids', 'classList'];
+            const fields = ['label', 'description', 'domain', 'logo', 'legal_name', 'ignored', 'email', 'bank_ids', 'classList', 'skip_enrich'];
             fields.forEach((field) => {
                 delete company[field];
             });
@@ -146,7 +169,7 @@ export function usePartnerAutocomplete() {
 
         return new Promise((resolve) => {
             // Fetch additional company info via Autocomplete Enrichment API
-            const enrichPromise = enrichCompany(company);
+            const enrichPromise = !company.skip_enrich ? enrichCompany(company) : false;
 
             // Get logo
             const logoPromise = company.logo ? getCompanyLogo(company.logo) : false;
@@ -168,6 +191,15 @@ export function usePartnerAutocomplete() {
                     else {
                         notification.add(company_data.error_message);
                     }
+                    if (company_data.city !== undefined) {
+                        company.city = company_data.city;
+                    }
+                    if (company_data.street !== undefined) {
+                        company.street = company_data.street;
+                    }
+                    if (company_data.zip !== undefined) {
+                        company.zip = company_data.zip;
+                    }
                     company_data = company;
                 }
 
@@ -310,5 +342,5 @@ export function usePartnerAutocomplete() {
             notification.add(title);
         }
     }
-    return { autocomplete, getCreateData, isVATNumber };
+    return { autocomplete, getCreateData, isTAXNumber };
 }
