@@ -1,39 +1,32 @@
 /** @odoo-module **/
-/* global checkVATNumber */
 
 import { AutoComplete } from "@web/core/autocomplete/autocomplete";
 import { useChildRef } from "@web/core/utils/hooks";
 import { registry } from "@web/core/registry";
 import { _t } from "@web/core/l10n/translation";
-import { CharField } from "@web/views/fields/char/char_field";
+import { CharField, charField } from "@web/views/fields/char/char_field";
 import { useInputField } from "@web/views/fields/input_field_hook";
-import { loadJS } from "@web/core/assets";
 
 import { usePartnerAutocomplete } from "@partner_autocomplete/js/partner_autocomplete_core"
 
 export class PartnerAutoCompleteCharField extends CharField {
+    static template = "partner_autocomplete.PartnerAutoCompleteCharField";
+    static components = {
+        ...CharField.components,
+        AutoComplete,
+    };
     setup() {
         super.setup();
 
         this.partner_autocomplete = usePartnerAutocomplete();
 
         this.inputRef = useChildRef();
-        useInputField({ getValue: () => this.props.value || "", parse: (v) => this.parse(v), ref: this.inputRef});
+        useInputField({ getValue: () => this.props.record.data[this.props.name] || "", parse: (v) => this.parse(v), ref: this.inputRef});
     }
 
-    sanitizeVAT(request) {
-        return request ? request.replace(/[^A-Za-z0-9]/g, '') : '';
-    }
-
-    isVAT(request) {
-        // checkVATNumber is defined in library jsvat.
-        // It validates that the input has a valid VAT number format
-        return checkVATNumber(this.sanitizeVAT(request));
-    }
-
-    validateSearchTerm(request) {
+    async validateSearchTerm(request) {
         if (this.props.name == 'vat') {
-            return this.isVAT(request);
+            return this.partner_autocomplete.isTAXNumber(request);
         }
         else {
             return request && request.length > 2;
@@ -44,11 +37,8 @@ export class PartnerAutoCompleteCharField extends CharField {
         return [
             {
                 options: async (request) => {
-                    // Lazyload jsvat only if the component is being used.
-                    await loadJS("/partner_autocomplete/static/lib/jsvat.js");
-                    
-                    if (this.validateSearchTerm(request)) {
-                        const suggestions = await this.partner_autocomplete.autocomplete(request, this.isVAT(request));
+                    if (await this.validateSearchTerm(request)) {
+                        const suggestions = await this.partner_autocomplete.autocomplete(request);
                         suggestions.forEach((suggestion) => {
                             suggestion.classList = "partner_autocomplete_dropdown_char";
                         });
@@ -88,13 +78,15 @@ export class PartnerAutoCompleteCharField extends CharField {
             }
         });
         this.props.record.update(data.company);
+        if (this.props.setDirty) {
+            this.props.setDirty(false);
+        }
     }
 }
 
-PartnerAutoCompleteCharField.template = "partner_autocomplete.PartnerAutoCompleteCharField";
-PartnerAutoCompleteCharField.components = {
-    ...CharField.components,
-    AutoComplete,
+export const partnerAutoCompleteCharField = {
+    ...charField,
+    component: PartnerAutoCompleteCharField,
 };
 
-registry.category("fields").add("field_partner_autocomplete", PartnerAutoCompleteCharField);
+registry.category("fields").add("field_partner_autocomplete", partnerAutoCompleteCharField);
