@@ -1,7 +1,7 @@
 /** @odoo-module **/
 
 import { Many2XAutocomplete } from '@web/views/fields/relational_utils';
-import { Many2OneField } from '@web/views/fields/many2one/many2one_field';
+import { Many2OneField, many2OneField } from '@web/views/fields/many2one/many2one_field';
 import { _t } from "@web/core/l10n/translation";
 import { registry } from "@web/core/registry";
 
@@ -18,13 +18,18 @@ export class PartnerMany2XAutocomplete extends Many2XAutocomplete {
     }
 
     get sources() {
-        return super.sources.concat(
+        const sources = super.sources;
+        if (!this.props.canCreate) {
+            return sources;
+        }
+        return sources.concat(
             {
                 options: async (request) => {
                     if (this.validateSearchTerm(request)) {
                         const suggestions = await this.partner_autocomplete.autocomplete(request);
                         suggestions.forEach((suggestion) => {
                             suggestion.classList = "partner_autocomplete_dropdown_many2one";
+                            suggestion.isFromPartnerAutocomplete = true;
                         });
                         return suggestions;
                     }
@@ -38,8 +43,8 @@ export class PartnerMany2XAutocomplete extends Many2XAutocomplete {
         );
     }
 
-    async onSelect(option) {
-        if (option.partner_gid) {  // Checks that it is a partner autocomplete option
+    async onSelect(option, params) {
+        if (option.isFromPartnerAutocomplete) {  // Checks that it is a partner autocomplete option
             const data = await this.partner_autocomplete.getCreateData(Object.getPrototypeOf(option));
             let context = {
                 'default_is_company': true
@@ -55,17 +60,37 @@ export class PartnerMany2XAutocomplete extends Many2XAutocomplete {
             return this.openMany2X({ context });
         }
         else {
-            return super.onSelect(option);
+            return super.onSelect(option, params);
         }
     }
 
 }
 
-export class PartnerAutoCompleteMany2one extends Many2OneField {}
-
-PartnerAutoCompleteMany2one.components = {
-    ...Many2OneField.components,
-    Many2XAutocomplete: PartnerMany2XAutocomplete,
+PartnerMany2XAutocomplete.props = {
+    ...Many2XAutocomplete.props,
+    canCreate: { type: Boolean, optional: true },
 }
 
-registry.category("fields").add("res_partner_many2one", PartnerAutoCompleteMany2one);
+export class PartnerAutoCompleteMany2one extends Many2OneField {
+    static components = {
+        ...Many2OneField.components,
+        Many2XAutocomplete: PartnerMany2XAutocomplete,
+    };
+    static props = {
+        ...Many2OneField.props,
+        canCreate: this.props.canCreate,
+    };
+    get Many2XAutocompleteProps() {
+        return {
+            ...super.Many2XAutocompleteProps,
+            canCreate: this.props.canCreate,
+        };
+    }
+}
+
+export const partnerAutoCompleteMany2one = {
+    ...many2OneField,
+    component: PartnerAutoCompleteMany2one,
+};
+
+registry.category("fields").add("res_partner_many2one", partnerAutoCompleteMany2one);
