@@ -13,10 +13,10 @@ from werkzeug.exceptions import Forbidden
 from odoo import _, http
 from odoo.exceptions import ValidationError
 from odoo.http import request
+from odoo.tools import py_to_js_locale
 
 from odoo.addons.payment import utils as payment_utils
 from odoo.addons.payment_adyen import utils as adyen_utils
-from odoo.addons.payment_adyen.const import CURRENCY_DECIMALS
 
 _logger = logging.getLogger(__name__)
 
@@ -25,58 +25,35 @@ class AdyenController(http.Controller):
 
     _webhook_url = '/payment/adyen/notification'
 
-    @http.route('/payment/adyen/provider_info', type='json', auth='public')
-    def adyen_provider_info(self, provider_id):
-        """ Return public information on the provider.
-
-        :param int provider_id: The provider handling the transaction, as a `payment.provider` id
-        :return: Public information on the provider, namely: the state and client key
-        :rtype: str
-        """
-        provider_sudo = request.env['payment.provider'].sudo().browse(provider_id).exists()
-        return {
-            'state': provider_sudo.state,
-            'client_key': provider_sudo.adyen_client_key,
-        }
-
     @http.route('/payment/adyen/payment_methods', type='json', auth='public')
-    def adyen_payment_methods(self, provider_id, amount=None, currency_id=None, partner_id=None):
-        """ Query the available payment methods based on the transaction context.
+    def adyen_payment_methods(self, provider_id, formatted_amount=None, partner_id=None):
+        """ Query the available payment methods based on the payment context.
 
         :param int provider_id: The provider handling the transaction, as a `payment.provider` id
-        :param float amount: The transaction amount
-        :param int currency_id: The transaction currency, as a `res.currency` id
+        :param dict formatted_amount: The Adyen-formatted amount.
         :param int partner_id: The partner making the transaction, as a `res.partner` id
         :return: The JSON-formatted content of the response
         :rtype: dict
         """
         provider_sudo = request.env['payment.provider'].sudo().browse(provider_id)
-        currency = request.env['res.currency'].browse(currency_id)
-        currency_code = currency_id and currency.name
-        converted_amount = amount and currency_code and payment_utils.to_minor_currency_units(
-            amount, currency, CURRENCY_DECIMALS.get(currency_code)
-        )
         partner_sudo = partner_id and request.env['res.partner'].sudo().browse(partner_id).exists()
         # The lang is taken from the context rather than from the partner because it is not required
         # to be logged in to make a payment, and because the lang is not always set on the partner.
         # Adyen only supports a limited set of languages but, instead of looking for the closest
         # match in https://docs.adyen.com/checkout/components-web/localization-components, we simply
         # provide the lang string as is (after adapting the format) and let Adyen find the best fit.
-        lang_code = (request.context.get('lang') or 'en-US').replace('_', '-')
+        lang_code = py_to_js_locale(request.context.get('lang')) or 'en-US'
         shopper_reference = partner_sudo and f'ODOO_PARTNER_{partner_sudo.id}'
         data = {
             'merchantAccount': provider_sudo.adyen_merchant_account,
-            'amount': converted_amount,
+            'amount': formatted_amount,
             'countryCode': partner_sudo.country_id.code or None,  # ISO 3166-1 alpha-2 (e.g.: 'BE')
-            'shopperLocale': lang_code,  # IETF language tag (e.g.: 'fr-BE')
+            'shopperLocale': lang_code,  # IETF BCP 47 language tag (e.g.: 'fr-BE')
             'shopperReference': shopper_reference,
             'channel': 'Web',
         }
         response_content = provider_sudo._adyen_make_request(
-            url_field_name='adyen_checkout_api_url',
-            endpoint='/paymentMethods',
-            payload=data,
-            method='POST'
+            endpoint='/paymentMethods', payload=data, method='POST'
         )
         _logger.info("paymentMethods request response:\n%s", pprint.pformat(response_content))
         return response_content
@@ -102,7 +79,7 @@ class AdyenController(http.Controller):
         # Check that the transaction details have not been altered. This allows preventing users
         # from validating transactions by paying less than agreed upon.
         if not payment_utils.check_access_token(
-            access_token, reference, converted_amount, partner_id
+            access_token, reference, converted_amount, currency_id, partner_id
         ):
             raise ValidationError("Adyen: " + _("Received tampered payment request data."))
 
@@ -121,12 +98,12 @@ class AdyenController(http.Controller):
             'recurringProcessingModel': 'CardOnFile',  # Most susceptible to trigger a 3DS check
             'shopperIP': payment_utils.get_customer_ip_address(),
             'shopperInteraction': 'Ecommerce',
-            'shopperEmail': tx_sudo.partner_email,
+            'shopperEmail': tx_sudo.partner_email or "",
             'shopperName': adyen_utils.format_partner_name(tx_sudo.partner_name),
-            'telephoneNumber': tx_sudo.partner_phone,
+            'telephoneNumber': tx_sudo.partner_phone or "",
             'storePaymentMethod': tx_sudo.tokenize,  # True by default on Adyen side
             'additionalData': {
-                'allow3DS2': True
+                'authenticationData.threeDSRequestData.nativeThreeDS': True,
             },
             'channel': 'web',  # Required to support 3DS
             'origin': provider_sudo.get_base_url(),  # Required to support 3DS
@@ -152,11 +129,11 @@ class AdyenController(http.Controller):
             data.update(captureDelayHours=0)
 
         # Make the payment request to Adyen
+        idempotency_key = payment_utils.generate_idempotency_key(
+            tx_sudo, scope='payment_request_controller'
+        )
         response_content = provider_sudo._adyen_make_request(
-            url_field_name='adyen_checkout_api_url',
-            endpoint='/payments',
-            payload=data,
-            method='POST'
+            endpoint='/payments', payload=data, method='POST', idempotency_key=idempotency_key
         )
 
         # Handle the payment request response
@@ -169,7 +146,7 @@ class AdyenController(http.Controller):
         )
         return response_content
 
-    @http.route('/payment/adyen/payment_details', type='json', auth='public')
+    @http.route('/payment/adyen/payments/details', type='json', auth='public')
     def adyen_payment_details(self, provider_id, reference, payment_details):
         """ Submit the details of the additional actions and handle the notification data.
 
@@ -185,10 +162,7 @@ class AdyenController(http.Controller):
         # Make the payment details request to Adyen
         provider_sudo = request.env['payment.provider'].browse(provider_id).sudo()
         response_content = provider_sudo._adyen_make_request(
-            url_field_name='adyen_checkout_api_url',
-            endpoint='/payments/details',
-            payload=payment_details,
-            method='POST'
+            endpoint='/payments/details', payload=payment_details, method='POST'
         )
 
         # Handle the payment details request response
@@ -246,7 +220,7 @@ class AdyenController(http.Controller):
         # Redirect the user to the status page
         return request.redirect('/payment/status')
 
-    @http.route(_webhook_url, type='json', auth='public')
+    @http.route(_webhook_url, type='http', methods=['POST'], auth='public', csrf=False)
     def adyen_webhook(self):
         """ Process the data sent by Adyen to the webhook based on the event code.
 
@@ -256,7 +230,7 @@ class AdyenController(http.Controller):
         :return: The '[accepted]' string to acknowledge the notification
         :rtype: str
         """
-        data = request.dispatcher.jsonrequest
+        data = request.get_json_data()
         for notification_item in data['notificationItems']:
             notification_data = notification_item['NotificationRequestItem']
 
@@ -285,6 +259,9 @@ class AdyenController(http.Controller):
                     notification_data['resultCode'] = 'Cancelled' if success else 'Error'
                 elif event_code in ['REFUND', 'CAPTURE']:
                     notification_data['resultCode'] = 'Authorised' if success else 'Error'
+                elif event_code == 'CAPTURE_FAILED' and success:
+                    # The capture failed after a capture notification with success = True was sent
+                    notification_data['resultCode'] = 'Error'
                 else:
                     continue  # Don't handle unsupported event codes and failed events
                 try:
@@ -295,7 +272,7 @@ class AdyenController(http.Controller):
                         "unable to handle the notification data;skipping to acknowledge"
                     )
 
-        return '[accepted]'  # Acknowledge the notification
+        return request.make_json_response('[accepted]')  # Acknowledge the notification
 
     @staticmethod
     def _verify_notification_signature(notification_data, tx_sudo):
