@@ -17,8 +17,3 @@ class PaymentToken(models.Model):
     authorize_profile = fields.Char(
         string="Authorize.Net Profile ID",
         help="The unique reference for the partner/token combination in the Authorize.net backend.")
-    authorize_payment_method_type = fields.Selection(
-        string="Authorize.Net Payment Type",
-        help="The type of payment method this token is linked to.",
-        selection=[("credit_card", "Credit Card"), ("bank_account", "Bank Account (USA Only)")],
-    )
