@@ -6,9 +6,9 @@ from . import models
 from odoo.addons.payment import setup_provider, reset_payment_provider
 
 
-def post_init_hook(cr, registry):
-    setup_provider(cr, registry, 'flutterwave')
+def post_init_hook(env):
+    setup_provider(env, 'flutterwave')
 
 
-def uninstall_hook(cr, registry):
-    reset_payment_provider(cr, registry, 'flutterwave')
+def uninstall_hook(env):
+    reset_payment_provider(env, 'flutterwave')
