@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import phone_blacklist
-from . import phone_validation_mixin
 from . import mail_thread_phone
+from . import models
 from . import res_partner
+from . import res_users
