@@ -1,19 +1,19 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, models
+from odoo.addons.phone_validation.tools import phone_validation
 
 
 class Partner(models.Model):
     _name = 'res.partner'
-    _inherit = ['res.partner', 'phone.validation.mixin']
+    _inherit = ['res.partner']
 
     @api.onchange('phone', 'country_id', 'company_id')
     def _onchange_phone_validation(self):
         if self.phone:
-            self.phone = self.phone_format(self.phone)
+            self.phone = self._phone_format(fname='phone', force_format='INTERNATIONAL') or self.phone
 
     @api.onchange('mobile', 'country_id', 'company_id')
     def _onchange_mobile_validation(self):
         if self.mobile:
-            self.mobile = self.phone_format(self.mobile)
+            self.mobile = self._phone_format(fname='mobile', force_format='INTERNATIONAL') or self.mobile
