@@ -14,14 +14,14 @@ class Digest(models.Model):
     def _compute_kpi_pos_total_value(self):
         if not self.env.user.has_group('point_of_sale.group_pos_user'):
             raise AccessError(_("Do not have access, skip this data for user's digest email"))
-        for record in self:
-            start, end, company = record._get_kpi_compute_parameters()
-            record.kpi_pos_total_value = sum(self.env['pos.order'].search([
-                ('date_order', '>=', start),
-                ('date_order', '<', end),
-                ('state', 'not in', ['draft', 'cancel', 'invoiced']),
-                ('company_id', '=', company.id)
-            ]).mapped('amount_total'))
+
+        self._calculate_company_based_kpi(
+            'pos.order',
+            'kpi_pos_total_value',
+            date_field='date_order',
+            additional_domain=[('state', 'not in', ['draft', 'cancel', 'invoiced'])],
+            sum_field='amount_total',
+        )
 
     def _compute_kpis_actions(self, company, user):
         res = super(Digest, self)._compute_kpis_actions(company, user)
