@@ -28,13 +28,25 @@ __base="$(basename ${__file} .sh)"
 MOUNT_POINT="${__dir}/root_mount"
 OVERWRITE_FILES_BEFORE_INIT_DIR="${__dir}/overwrite_before_init"
 OVERWRITE_FILES_AFTER_INIT_DIR="${__dir}/overwrite_after_init"
-VERSION=16.0
-VERSION_IOTBOX=22.11
-REPO=https://github.com/odoo/odoo.git
+VERSION=17.0
+VERSION_IOTBOX=24.10
+
+
+# ask user for the branch/version
+current_branch="$(git branch --show-current)"
+read -p "Enter dev branch [${current_branch}]: " VERSION
+VERSION=${VERSION:-$current_branch}
+
+# ask user for the repository
+current_remote=$(git config branch.$current_branch.remote)
+current_repo="$(git remote get-url $current_remote | sed 's/.*github.com[\/:]//' | sed 's/\/odoo.git//')"
+read -p "Enter repo [${current_repo}]: " REPO
+REPO="https://github.com/${REPO:-$current_repo}/odoo.git"
+echo "Using repo: ${REPO}"
 
 if ! file_exists *raspios*.img ; then
-    wget 'https://downloads.raspberrypi.org/raspios_lite_armhf/images/raspios_lite_armhf-2021-05-28/2021-05-07-raspios-buster-armhf-lite.zip' -O raspios.img.zip
-    unzip raspios.img.zip
+    wget "https://downloads.raspberrypi.org/raspios_lite_armhf/images/raspios_lite_armhf-2024-07-04/2024-07-04-raspios-bookworm-armhf-lite.img.xz" -O raspios.img.xz
+    unxz --verbose raspios.img.xz
 fi
 
 RASPIOS=$(echo *raspios*.img)
@@ -62,24 +74,22 @@ cd "${__dir}"
 USR_BIN="${OVERWRITE_FILES_BEFORE_INIT_DIR}/usr/bin/"
 mkdir -pv "${USR_BIN}"
 cd "/tmp"
-curl 'https://bin.equinox.io/c/4VmDzA7iaHb/ngrok-stable-linux-arm.zip' > ngrok.zip
-unzip ngrok.zip
-rm -v ngrok.zip
+curl 'https://bin.equinox.io/c/bNyj1mQVY4c/ngrok-v3-stable-linux-arm.tgz' > ngrok.tgz
+tar xvzf ngrok.tgz -C "${USR_BIN}"
+rm -v ngrok.tgz
 cd "${__dir}"
-mv -v /tmp/ngrok "${USR_BIN}"
 
-# zero pad the image to be around 4.4 GiB, by default the image is only ~2.2 GiB
+# zero pad the image to be around ~5 GiB, by default the image is only ~2.2 GiB
 echo "Enlarging the image..."
-dd if=/dev/zero bs=1M count=2048 status=progress >> iotbox.img
+dd if=/dev/zero bs=1M count=2560 status=progress >> iotbox.img
 
 # resize partition table
 echo "Fdisking"
 
 SECTORS_BOOT_START=$(sudo fdisk -l iotbox.img | tail -n 2 | awk 'NR==1 {print $2}')
-SECTORS_BOOT_END=$((SECTORS_BOOT_START + 1048576)) # sectors to have a partition of ~512Mo
+SECTORS_BOOT_END=$((SECTORS_BOOT_START + 1056767)) # sectors to have a partition of ~512Mo
 SECTORS_ROOT_START=$((SECTORS_BOOT_END + 1))
 
-START_OF_ROOT_PARTITION=$(fdisk -l iotbox.img | tail -n 1 | awk '{print $2}')
 (echo 'p';                          # print
  echo 'd';                          # delete
  echo '2';                          #    number 2
@@ -103,12 +113,14 @@ LOOP_RASPIOS=$(kpartx -avs "${RASPIOS}")
 LOOP_RASPIOS_ROOT=$(echo "${LOOP_RASPIOS}" | tail -n 1 | awk '{print $3}')
 LOOP_RASPIOS_PATH="/dev/${LOOP_RASPIOS_ROOT::-2}"
 LOOP_RASPIOS_ROOT="/dev/mapper/${LOOP_RASPIOS_ROOT}"
+LOOP_RASPIOS_BOOT=$(echo "${LOOP_RASPIOS}" | head -n 1 | awk '{print $3}')
+LOOP_RASPIOS_BOOT="/dev/mapper/${LOOP_RASPIOS_BOOT}"
 
 LOOP_IOT=$(kpartx -avs iotbox.img)
 LOOP_IOT_ROOT=$(echo "${LOOP_IOT}" | tail -n 1 | awk '{print $3}')
 LOOP_IOT_PATH="/dev/${LOOP_IOT_ROOT::-2}"
 LOOP_IOT_ROOT="/dev/mapper/${LOOP_IOT_ROOT}"
-LOOP_IOT_BOOT=$(echo "${LOOP_IOT}" | tail -n 2 | awk 'NR==1 {print $3}')
+LOOP_IOT_BOOT=$(echo "${LOOP_IOT}" | head -n 1 | awk 'NR==1 {print $3}')
 LOOP_IOT_BOOT="/dev/mapper/${LOOP_IOT_BOOT}"
 
 mkfs.ext4 -v "${LOOP_IOT_ROOT}"
@@ -116,7 +128,7 @@ mkfs.ext4 -v "${LOOP_IOT_ROOT}"
 dd if="${LOOP_RASPIOS_ROOT}" of="${LOOP_IOT_ROOT}" bs=4M status=progress
 
 # resize filesystem
-e2fsck -fv "${LOOP_IOT_ROOT}" # resize2fs requires clean fs
+e2fsck -fvy "${LOOP_IOT_ROOT}" # resize2fs requires clean fs
 resize2fs "${LOOP_IOT_ROOT}"
 
 mkdir -pv "${MOUNT_POINT}" #-p: no error if existing
@@ -128,17 +140,23 @@ cp -v "${QEMU_ARM_STATIC}" "${MOUNT_POINT}/usr/bin/"
 
 # 'overlay' the overwrite directory onto the mounted image filesystem
 cp -av "${OVERWRITE_FILES_BEFORE_INIT_DIR}"/* "${MOUNT_POINT}"
-chroot "${MOUNT_POINT}" /bin/bash -c "sudo /etc/init_posbox_image.sh"
+
+# Reload network manager is mandatory in order to apply DNS configurations:
+# it needs to be reloaded after copying the 'overwrite_before_init' files in the new image
+# it needs to be performed in the classic filesystem, as 'systemctl' commands are not available in /root_bypass_ramdisks
+sudo systemctl reload NetworkManager
+
+chroot "${MOUNT_POINT}" /bin/bash -c "/etc/init_posbox_image.sh"
 
 # copy iotbox version
-mkdir -pv "${MOUNT_POINT}"/var/odoo
-echo "${VERSION_IOTBOX}" | tee "${MOUNT_POINT}"/var/odoo/iotbox_version "${MOUNT_POINT}"/home/pi/iotbox_version
+mkdir -pv "${MOUNT_POINT}"/var/odoo/
+echo "${VERSION_IOTBOX}" > "${MOUNT_POINT}"/var/odoo/iotbox_version
 
 # get rid of the git clone
-rm -rfv "${CLONE_DIR}"
+rm -rf "${CLONE_DIR}"
 # and the ngrok usr/bin
-rm -rfv "${OVERWRITE_FILES_BEFORE_INIT_DIR}/usr"
-cp -av "${OVERWRITE_FILES_AFTER_INIT_DIR}"/* "${MOUNT_POINT}"
+rm -rf "${OVERWRITE_FILES_BEFORE_INIT_DIR}/usr"
+cp -a "${OVERWRITE_FILES_AFTER_INIT_DIR}"/* "${MOUNT_POINT}"
 
 find "${MOUNT_POINT}"/ -type f -name "*.iotpatch"|while read iotpatch; do
     DIR=$(dirname "${iotpatch}")
@@ -150,7 +168,7 @@ done
 
 # cleanup
 umount -fv "${MOUNT_POINT}"/boot/
-umount -fv "${MOUNT_POINT}"/
+umount -lv "${MOUNT_POINT}"/
 rm -rfv "${MOUNT_POINT}"
 
 echo "Running zerofree..."
@@ -160,3 +178,5 @@ sleep 10
 
 kpartx -dv "${LOOP_IOT_PATH}"
 kpartx -dv "${LOOP_RASPIOS_PATH}"
+
+echo "Image build finished."
