@@ -1,6 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import web
-from . import portal
+from . import attachment
 from . import mail
+from . import message_reaction
+from . import portal
+from . import thread
+from . import web
