@@ -16,12 +16,15 @@ discount to a customer.
 """,
     'depends': ['point_of_sale'],
     'data': [
-        'views/pos_discount_views.xml',
-        'views/pos_discount_templates.xml'
-    ],
-    'qweb': [
-        'static/src/xml/DiscountButton.xml',
-    ],
+        'data/pos_discount_data.xml',
+        'views/res_config_settings_views.xml',
+        'views/pos_config_views.xml',
+        ],
     'installable': True,
+    'assets': {
+        'point_of_sale._assets_pos': [
+            'pos_discount/static/src/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
