@@ -2,7 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
-    'name': "pos_hr",
+    'name': "POS - HR",
     'category': "Hidden",
     'summary': 'Link module between Point of Sale and HR',
 
@@ -15,17 +15,26 @@ The actual till still requires one user but an unlimited number of employees can
 
     'data': [
         'views/pos_config.xml',
-        'views/point_of_sale.xml',
         'views/pos_order_view.xml',
+        'views/pos_payment_view.xml',
         'views/pos_order_report_view.xml',
+        'views/single_employee_sales_report.xml',
+        'views/multi_employee_sales_report.xml',
+        'views/res_config_settings_views.xml',
+        'wizard/pos_daily_sales_reports.xml',
     ],
     'installable': True,
     'auto_install': True,
-    'qweb': [
-        'static/src/xml/HeaderLockButton.xml',
-        'static/src/xml/Chrome.xml',
-        'static/src/xml/CashierName.xml',
-        'static/src/xml/LoginScreen.xml',
-    ],
+    'assets': {
+        'point_of_sale._assets_pos': [
+            'pos_hr/static/src/**/*',
+        ],
+        'web.assets_tests': [
+            'pos_hr/static/tests/**/*',
+        ],
+        'web.assets_backend': [
+            'pos_hr/static/src/app/print_report_button/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
