@@ -5,3 +5,6 @@ from . import pos_config
 from . import pos_order
 from . import pos_payment
 from . import pos_restaurant
+from . import pos_session
+from . import res_config_settings
+from . import account_fiscal_position
