@@ -22,8 +22,8 @@ class CrmTeam(models.Model):
         data = self.env['report.pos.order']._read_group([
             ('session_id.state', '=', 'opened'),
             ('config_id.crm_team_id', 'in', self.ids),
-        ], ['price_total:sum', 'config_id'], ['config_id'])
-        rg_results = dict((d['config_id'][0], d['price_total']) for d in data)
+        ], ['config_id'], ['price_total:sum'])
+        rg_results = {config.id: price_total_sum for config, price_total_sum in data}
         for team in self:
             team.pos_order_amount_total = sum([
                 rg_results.get(config.id, 0.0)
