@@ -6,7 +6,6 @@
     'category': 'Sales/Point of Sale',
     'sequence': 6,
     'summary': 'Integrate your POS with a Stripe payment terminal',
-    'description': '',
     'data': [
         'views/pos_payment_method_views.xml',
         'views/assets_stripe.xml',
@@ -14,7 +13,7 @@
     'depends': ['point_of_sale', 'payment_stripe'],
     'installable': True,
     'assets': {
-        'point_of_sale.assets': [
+        'point_of_sale._assets_pos': [
             'pos_stripe/static/**/*',
         ],
     },
