@@ -5,7 +5,6 @@
     'name': 'Privacy',
     'category': 'Hidden',
     'version': '1.0',
-    'description': """""",
     'depends': ['mail'],
     'data': [
         'wizard/privacy_lookup_wizard_views.xml',
@@ -14,6 +13,5 @@
         'data/ir_actions_server_data.xml',
     ],
     'auto_install': True,
-    'assets': {},
     'license': 'LGPL-3',
 }
