@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, models
@@ -17,19 +16,19 @@ class ProductPricelistReport(models.AbstractModel):
         return self.env['ir.qweb']._render('product.report_pricelist_page', render_values)
 
     def _get_report_data(self, data, report_type='html'):
-        quantities = data['quantities'] or [1]
-
-        pricelist_id = data['pricelist_id'] and int(data['pricelist_id']) or None
+        quantities = data.get('quantities', [1])
+        data_pricelist_id = data.get('pricelist_id')
+        pricelist_id = data_pricelist_id and int(data_pricelist_id)
         pricelist = self.env['product.pricelist'].browse(pricelist_id).exists()
         if not pricelist:
             pricelist = self.env['product.pricelist'].search([], limit=1)
 
-        active_model = data['active_model']
+        active_model = data.get('active_model', 'product.template')
         active_ids = data.get('active_ids') or []
         is_product_tmpl = active_model == 'product.template'
         ProductClass = self.env[active_model]
 
-        products = ProductClass.browse(active_ids) if active_ids else ProductClass.search([('sale_ok', '=', True)])
+        products = ProductClass.browse(active_ids) if active_ids else []
         products_data = [
             self._get_product_data(is_product_tmpl, product, pricelist, quantities)
             for product in products
@@ -38,7 +37,7 @@ class ProductPricelistReport(models.AbstractModel):
         return {
             'is_html_type': report_type == 'html',
             'is_product_tmpl': is_product_tmpl,
-            'is_visible_title': bool(data['is_visible_title']) or False,
+            'display_pricelist_title': data.get('display_pricelist_title', False) and bool(data['display_pricelist_title']),
             'pricelist': pricelist,
             'products': products_data,
             'quantities': quantities,
