@@ -4,6 +4,8 @@
 import base64
 from collections import OrderedDict
 from datetime import timedelta
+from unittest.mock import patch
+
 import io
 import unittest.mock
 
@@ -42,12 +44,29 @@ class TestVariantsSearch(ProductVariantsCommon):
     def test_name_search(self):
         self.product_slip_template = self.env['product.template'].create({
             'name': 'Slip',
+            'default_code': 'ABC',
         })
         res = self.env['product.product'].name_search('Shirt', [], 'not ilike', None)
         res_ids = [r[0] for r in res]
         self.assertIn(self.product_slip_template.product_variant_ids.id, res_ids,
                       'Slip should be found searching \'not ilike\'')
 
+        templates = self.product_slip_template.name_search(
+            "ABC",
+            [['id', '!=', -1]],
+        )
+        self.assertFalse(templates, "Should not return template when searching on code")
+        templates = self.product_slip_template.with_context(search_product_product=True).name_search(
+            "ABC",
+            [['id', '!=', -1]],
+        )
+        self.assertTrue(templates, "Should return template when searching on code")
+
+        templates = self.product_slip_template.with_context(search_product_product=True).name_search(
+            "ABC",
+            [['id', '!=', self.product_slip_template.id]],
+        )
+        self.assertFalse(templates, "Should not return template.")
 
 @tagged('post_install', '-at_install')
 class TestVariants(ProductVariantsCommon):
@@ -229,6 +248,10 @@ class TestVariants(ProductVariantsCommon):
         self.assertEqual(template_dyn.name, 'Test Dynamical')
 
         variant_dyn = template_dyn._create_product_variant(template_dyn._get_first_possible_combination())
+        if 'create_product_product' in variant_dyn._context:
+            new_context = dict(variant_dyn._context)
+            new_context.pop('create_product_product')
+            variant_dyn = variant_dyn.with_context(new_context)
         self.assertEqual(len(template_dyn.product_variant_ids), 1)
 
         variant_dyn_copy = variant_dyn.copy()
@@ -551,6 +574,40 @@ class TestVariantsNoCreate(ProductAttributesCommon):
         # no_variant attribute should not appear on the variant
         self.assertNotIn(self.size_attribute_s, template.product_variant_ids.product_template_attribute_value_ids.product_attribute_value_id)
 
+    def test_unarchive_multiple_products_with_variants(self):
+        product_attribut = self.env['product.attribute'].create({
+            'name': 'Color',
+            'sequence': 1,
+            'create_variant': 'dynamic',
+        })
+        attr_value = self.env['product.attribute.value'].create({
+            'name': 'Blue',
+            'attribute_id': product_attribut.id,
+            'sequence': 1,
+        })
+        first_product = self.env['product.template'].create({
+            'name': 'Sofa',
+            'attribute_line_ids': [(0, 0, {
+                'attribute_id': product_attribut.id,
+                'value_ids': [(6, 0, [attr_value.id])],
+            })]
+        })
+        second_product = first_product.copy({
+            'product_variant_ids': [(0, 0, {
+            'name': 'Sofa',
+            })]
+        })
+
+        products = first_product + second_product
+        products.action_archive()
+        self.assertFalse(first_product.active)
+        self.assertFalse(second_product.active)
+        self.assertFalse(second_product.product_variant_ids)
+        products.action_unarchive()
+        # check products should be unarchived successfully.
+        self.assertTrue(first_product.active)
+        self.assertTrue(second_product.active)
+        self.assertTrue(second_product.product_variant_ids)
 
 @tagged('post_install', '-at_install')
 class TestVariantsManyAttributes(TransactionCase):
@@ -732,11 +789,11 @@ class TestVariantsImages(ProductVariantsCommon):
         images = self.variants.mapped('image_1920')
         self.assertEqual(len(set(images)), 4)
         variant_no_image = self.variants[0]
-        old_last_update = variant_no_image['__last_update']
+        old_last_update = variant_no_image.write_date
 
         self.assertFalse(variant_no_image.image_1920)
         self.template.image_1920 = image_black
-        new_last_update = variant_no_image['__last_update']
+        new_last_update = variant_no_image.write_date
 
         # the first has no image variant, all the others do
         self.assertFalse(variant_no_image.image_variant_1920)
@@ -833,7 +890,7 @@ class TestVariantsArchive(ProductVariantsCommon):
 
         def unlink(self):
             raise Exception('just')
-        Product._patch_method('unlink', unlink)
+        self.patch(type(Product), 'unlink', unlink)
 
         variants_2x1 = self.template.product_variant_ids
         self._assert_2color_x_1size()
@@ -855,8 +912,6 @@ class TestVariantsArchive(ProductVariantsCommon):
         archived_variants = self._get_archived_variants()
         self.assertFalse(archived_variants)
 
-        Product._revert_method('unlink')
-
     def test_02_update_variant_archive_2_value(self):
         """We do the same operations on the template as in the previous tests,
            except we simulate that the variants can't be unlinked.
@@ -868,7 +923,7 @@ class TestVariantsArchive(ProductVariantsCommon):
 
         def unlink(slef):
             raise Exception('just')
-        Product._patch_method('unlink', unlink)
+        self.patch(type(Product), 'unlink', unlink)
 
         variants_2x2 = self.template.product_variant_ids
         self._assert_2color_x_2size()
@@ -924,8 +979,6 @@ class TestVariantsArchive(ProductVariantsCommon):
         self.assertEqual(archived_variants, variants_2x2)
         self._assert_2color_x_2size(archived_variants)
 
-        Product._revert_method('unlink')
-
     @mute_logger('odoo.models.unlink')
     def test_03_update_variant_archive_3_value(self):
         self._remove_ptal_size()
@@ -935,7 +988,7 @@ class TestVariantsArchive(ProductVariantsCommon):
 
         def unlink(slef):
             raise Exception('just')
-        Product._patch_method('unlink', unlink)
+        self.patch(type(Product), 'unlink', unlink)
 
         self._assert_2color_x_1size()
         archived_variants = self._get_archived_variants()
@@ -983,14 +1036,12 @@ class TestVariantsArchive(ProductVariantsCommon):
         archived_variants = self._get_archived_variants()
         self.assertEqual(archived_variants, variants_2x1 + variant_0x0)
 
-        Product._revert_method('unlink')
-
     def test_04_from_to_single_values(self):
         Product = self.env['product.product']
 
         def unlink(slef):
             raise Exception('just')
-        Product._patch_method('unlink', unlink)
+        self.patch(type(Product), 'unlink', unlink)
 
         # CASE: remove one value, line becoming single value
         variants_2x2 = self.template.product_variant_ids
@@ -1027,9 +1078,9 @@ class TestVariantsArchive(ProductVariantsCommon):
         self._assert_2color_x_0size(archived_variants)
         self.assertEqual(archived_variants, variants_2x0)
 
-        Product._revert_method('unlink')
-
     def test_name_search_dynamic_attributes(self):
+        # To be able to test dynamic variant "variants" feature must be set up
+        self.env.user.write({'groups_id': [(4, self.env.ref('product.group_product_variant').id)]})
         dynamic_attr = self.env['product.attribute'].create({
             'name': 'Dynamic',
             'create_variant': 'dynamic',
@@ -1079,7 +1130,7 @@ class TestVariantsArchive(ProductVariantsCommon):
         # Patch unlink method to force archiving instead deleting
         def unlink(self):
             self.active = False
-        Product._patch_method('unlink', unlink)
+        self.patch(type(Product), 'unlink', unlink)
 
         # Creating attributes
         pa_color = ProductAttribute.create({'sequence': 1, 'name': 'color', 'create_variant': 'dynamic'})
@@ -1184,8 +1235,6 @@ class TestVariantsArchive(ProductVariantsCommon):
         })
         self.assertTrue(product_white.active)
 
-        Product._revert_method('unlink')
-
     def test_set_barcode(self):
         tmpl = self.product.product_tmpl_id
         tmpl.barcode = '123'
@@ -1401,6 +1450,7 @@ class TestVariantsExclusion(ProductAttributesCommon):
             )
 
         cls.smartphone_s = get_ptav(cls.smartphone, cls.size_attribute_s)
+        cls.smartphone_l = get_ptav(cls.smartphone, cls.size_attribute_l)
         cls.smartphone_256 = get_ptav(cls.smartphone, cls.storage_attr_value_256)
         cls.smartphone_128 = get_ptav(cls.smartphone, cls.storage_attr_value_128)
 
@@ -1471,3 +1521,85 @@ class TestVariantsExclusion(ProductAttributesCommon):
             'exclude_for': [(2, self.smartphone_s.exclude_for.ids[0], 0)]
         })
         self.assertEqual(len(self.smartphone.product_variant_ids), 3, 'With one exclusion, the smartphone should have 3 active different variants')
+
+    @mute_logger('odoo.models.unlink')
+    def test_exclusions_crud(self):
+        """ Make sure that exclusions creation, update & delete are correctly handled.
+
+        Exclusions updates are not necessarily done from a specific template.
+        """
+        PTAE = self.env['product.template.attribute.exclusion']
+
+        exclude = PTAE.create({
+            'product_tmpl_id': self.smartphone.id,
+            'product_template_attribute_value_id': self.smartphone_s.id,
+            'value_ids': [Command.set(self.smartphone_256.ids)]
+        })
+        self.assertEqual(len(self.smartphone.product_variant_ids), 3)
+        self.assertNotIn(
+            self.smartphone_s + self.smartphone_256,
+            [product.product_template_attribute_value_ids for product in self.smartphone.product_variant_ids],
+        )
+
+        exclude.value_ids = [Command.set(self.smartphone_128.ids)]
+        self.assertEqual(len(self.smartphone.product_variant_ids), 3)
+        self.assertNotIn(
+            self.smartphone_s + self.smartphone_128,
+            [product.product_template_attribute_value_ids for product in self.smartphone.product_variant_ids],
+        )
+
+        exclude.unlink()
+        self.assertEqual(len(self.smartphone.product_variant_ids), 4)
+
+    @mute_logger('odoo.models.unlink')
+    def test_dynamic_variants_unarchive(self):
+        """ Make sure that exclusions creation, update & delete are correctly handled.
+
+        Exclusions updates are not necessarily done from a specific template.
+        """
+        product_template = self.env['product.template'].create({
+            'name': 'Test dynamic',
+            'attribute_line_ids': [
+                Command.create({
+                    'attribute_id': self.dynamic_attribute.id,
+                    'value_ids': [Command.set(self.dynamic_attribute.value_ids.ids)],
+                }),
+                Command.create({
+                    'attribute_id': self.dynamic_attribute.id,
+                    'value_ids': [Command.set(self.dynamic_attribute.value_ids.ids)],
+                })
+            ]
+        })
+        self.assertFalse(product_template.product_variant_ids)
+        first_line_ptavs = product_template.attribute_line_ids[0].product_template_value_ids
+        second_line_ptavs = product_template.attribute_line_ids[1].product_template_value_ids
+        for ptav1, ptav2 in zip(first_line_ptavs, second_line_ptavs, strict=True):
+            product_template._create_product_variant(ptav1 + ptav2)
+
+        self.assertEqual(len(product_template.product_variant_ids), 2)
+
+        pav_to_remove = self.dynamic_attribute.value_ids[:1]
+        variant_to_archive = product_template.product_variant_ids.filtered(
+            lambda pp:
+                pav_to_remove in pp.product_template_attribute_value_ids.product_attribute_value_id
+        )
+
+        # Removing one option will archive one variant
+        with patch(
+            'odoo.addons.product.models.product_product.ProductProduct._filter_to_unlink',
+            lambda products: products.filtered(
+                lambda pp: pp.product_tmpl_id.id != product_template.id
+            ),
+        ):
+            product_template.attribute_line_ids[1].value_ids = [
+                Command.unlink(self.dynamic_attribute.value_ids[:1].id)
+            ]
+        self.assertEqual(len(product_template.product_variant_ids), 1)
+        self.assertFalse(variant_to_archive.active)
+
+        # Putting it back should unarchive the archived variant
+        product_template.attribute_line_ids[1].value_ids = [
+            Command.link(self.dynamic_attribute.value_ids[:1].id)
+        ]
+        self.assertEqual(len(product_template.product_variant_ids), 2)
+        self.assertTrue(variant_to_archive.active)
