@@ -26,9 +26,9 @@ Also implements the removal strategy First Expiry First Out (FEFO) widely used,
              'wizard/confirm_expiry_view.xml',
              'report/report_deliveryslip.xml',
              'report/report_lot_barcode.xml',
-             'report/report_package_barcode.xml',
              'data/product_expiry_data.xml',
              'data/mail_activity_type_data.xml',
             ],
+    'post_init_hook': '_enable_tracking_numbers',
     'license': 'LGPL-3',
 }
