@@ -2,9 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
     'name': "Product Matrix",
-    'summary': """
-       Technical module: Matrix Implementation
-    """,
+    'summary': "Technical module: Matrix Implementation",
     'description': """
 Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     """,
@@ -20,6 +18,7 @@ Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     ],
     'assets': {
         'web.assets_backend': [
+            'product_matrix/static/src/js/product_matrix_dialog.js',
             'product_matrix/static/src/scss/product_matrix.scss',
             'product_matrix/static/src/xml/**/*',
         ],
