@@ -5,9 +5,7 @@ from . import controllers
 from . import models
 from . import report
 from . import wizard
-from . import populate
 
-from odoo import api, SUPERUSER_ID
 from odoo.tools.sql import create_index
 
 
@@ -22,16 +20,18 @@ def _check_exists_collaborators_for_project_sharing(env):
         env['project.collaborator']._toggle_project_sharing_portal_rules(True)
 
 
-def _project_post_init(cr, registry):
-    env = api.Environment(cr, SUPERUSER_ID, {})
+def _project_post_init(env):
     _check_exists_collaborators_for_project_sharing(env)
 
     # Index to improve the performance of burndown chart.
     project_task_stage_field_id = env['ir.model.fields']._get_ids('project.task').get('stage_id')
     create_index(
-        cr,
+        env.cr,
         'mail_tracking_value_mail_message_id_old_value_integer_task_stage',
         env['mail.tracking.value']._table,
         ['mail_message_id', 'old_value_integer'],
-        where=f'field={project_task_stage_field_id}'
+        where=f'field_id={project_task_stage_field_id}'
     )
+
+    # Create analytic plan fields on project model for existing plans
+    env['account.analytic.plan'].search([])._sync_plan_column('project.project')
