@@ -1,15 +1,21 @@
 /** @odoo-module */
 
-import { useService } from "@web/core/utils/hooks";
+import { rpc } from "@web/core/network/rpc";
 
 import { ChatterAttachmentsViewer } from "./chatter_attachments_viewer";
-
-const { Component } = owl;
+import { Component } from "@odoo/owl";
 
 export class ChatterMessages extends Component {
-    setup() {
-        this.rpc = useService('rpc');
-    }
+    static template = "project.ChatterMessages";
+    static props = {
+        messages: Array,
+        isUserEmployee: { type: Boolean, optional: true },
+        update: { type: Function, optional: true },
+    };
+    static defaultProps = {
+        update: (message_id, changes) => {},
+    };
+    static components = { ChatterAttachmentsViewer };
 
     /**
      * Toggle the visibility of the message.
@@ -17,21 +23,10 @@ export class ChatterMessages extends Component {
      * @param {Object} message message to change the visibility
      */
     async toggleMessageVisibility(message) {
-        const result = await this.rpc(
+        const result = await rpc(
             '/mail/update_is_internal',
             { message_id: message.id, is_internal: !message.is_internal },
         );
         this.props.update(message.id, { is_internal: result });
     }
 }
-
-ChatterMessages.template = 'project.ChatterMessages';
-ChatterMessages.props = {
-    messages: Array,
-    isUserEmployee: { type: Boolean, optional: true },
-    update: { type: Function, optional: true },
-};
-ChatterMessages.defaultProps = {
-    update: (message_id, changes) => {},
-};
-ChatterMessages.components = { ChatterAttachmentsViewer };
