@@ -11,7 +11,8 @@
     'data': [
         'views/project_stage_views.xml',
         'views/project_task_type_views.xml',
-        'views/project_views.xml',
+        'views/project_project_views.xml',
+        'views/project_task_views.xml',
         'security/ir.model.access.csv',
         'security/project_sms_security.xml',
     ],
