@@ -14,10 +14,22 @@ It is basically used when we want to keep track of production orders generated
 from purchase order.
     """,
     'data': [
+        'views/mrp_bom_views.xml',
         'views/purchase_order_views.xml',
-        'views/mrp_production_views.xml'
+        'views/mrp_production_views.xml',
+        'views/stock_orderpoint_views.xml',
+        'security/ir.model.access.csv',
+    ],
+    'demo': [
+        'data/purchase_mrp_demo.xml',
     ],
     'depends': ['mrp', 'purchase_stock'],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'mrp/static/src/**/*.js',
+        ],
+    },
+    'license': 'LGPL-3',
 }
