@@ -4,7 +4,7 @@
 from collections import defaultdict
 
 from odoo import api, fields, models, _, Command
-from odoo.tools import get_lang
+from odoo.tools import DEFAULT_SERVER_DATETIME_FORMAT, get_lang
 
 
 class PurchaseOrderGroup(models.Model):
@@ -23,8 +23,8 @@ class PurchaseOrderGroup(models.Model):
 class PurchaseOrder(models.Model):
     _inherit = 'purchase.order'
 
-    requisition_id = fields.Many2one('purchase.requisition', string='Purchase Agreement', copy=False)
-    is_quantity_copy = fields.Selection(related='requisition_id.is_quantity_copy', readonly=False)
+    requisition_id = fields.Many2one('purchase.requisition', string='Agreement', copy=False)
+    requisition_type = fields.Selection(related='requisition_id.requisition_type')
 
     purchase_group_id = fields.Many2one('purchase.order.group')
     alternative_po_ids = fields.One2many(
@@ -39,7 +39,8 @@ class PurchaseOrder(models.Model):
     @api.depends('purchase_group_id')
     def _compute_has_alternatives(self):
         self.has_alternatives = False
-        self.filtered(lambda po: po.purchase_group_id).has_alternatives = True
+        if self.env.user.has_group('purchase_requisition.group_purchase_alternatives'):
+            self.filtered(lambda po: po.purchase_group_id).has_alternatives = True
 
     @api.onchange('requisition_id')
     def _onchange_requisition_id(self):
@@ -59,7 +60,7 @@ class PurchaseOrder(models.Model):
 
         self.partner_id = partner.id
         self.fiscal_position_id = fpos.id
-        self.payment_term_id = payment_term.id,
+        self.payment_term_id = payment_term.id
         self.company_id = requisition.company_id.id
         self.currency_id = requisition.currency_id.id
         if not self.origin or requisition.name not in self.origin.split(', '):
@@ -69,10 +70,10 @@ class PurchaseOrder(models.Model):
             else:
                 self.origin = requisition.name
         self.notes = requisition.description
-        self.date_order = fields.Datetime.now()
-
-        if requisition.type_id.line_copy != 'copy':
-            return
+        if requisition.date_start:
+            self.date_order = max(fields.Datetime.now(), fields.Datetime.to_datetime(requisition.date_start))
+        else:
+            self.date_order = fields.Datetime.now()
 
         # Create PO lines if necessary
         order_lines = []
@@ -97,7 +98,7 @@ class PurchaseOrder(models.Model):
                 product_qty = line.product_qty
                 price_unit = line.price_unit
 
-            if requisition.type_id.quantity_copy != 'copy':
+            if requisition.requisition_type != 'purchase_template':
                 product_qty = 0
 
             # Create PO line
@@ -113,7 +114,7 @@ class PurchaseOrder(models.Model):
             if alternative_po_ids:
                 view = self.env.ref('purchase_requisition.purchase_requisition_alternative_warning_form')
                 return {
-                    'name': _('Alternative Warning'),
+                    'name': _("What about the alternative Requests for Quotations?"),
                     'type': 'ir.actions.act_window',
                     'view_mode': 'form',
                     'res_model': 'purchase.requisition.alternative.warning',
@@ -122,14 +123,6 @@ class PurchaseOrder(models.Model):
                     'context': dict(self.env.context, default_alternative_po_ids=alternative_po_ids.ids, default_po_ids=self.ids),
                 }
         res = super(PurchaseOrder, self).button_confirm()
-        for po in self:
-            if not po.requisition_id:
-                continue
-            if po.requisition_id.type_id.exclusive == 'exclusive':
-                others_po = po.requisition_id.mapped('purchase_ids').filtered(lambda r: r.id != po.id)
-                others_po.button_cancel()
-                if po.state not in ['draft', 'sent', 'to approve']:
-                    po.requisition_id.action_done()
         return res
 
     @api.model_create_multi
@@ -142,13 +135,13 @@ class PurchaseOrder(models.Model):
                 origin_po_id.purchase_group_id.order_ids |= orders
             else:
                 self.env['purchase.order.group'].create({'order_ids': [Command.set(origin_po_id.ids + orders.ids)]})
-        mt_note = self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
         for order in orders:
             if order.requisition_id:
-                order.message_post_with_view(
+                order.message_post_with_source(
                     'mail.message_origin_link',
-                    values={'self': order, 'origin': order.requisition_id},
-                    subtype_id=mt_note)
+                    render_values={'self': order, 'origin': order.requisition_id},
+                    subtype_xmlid='mail.mt_note',
+                )
         return orders
 
     def write(self, vals):
@@ -158,10 +151,10 @@ class PurchaseOrder(models.Model):
         result = super(PurchaseOrder, self).write(vals)
         if vals.get('requisition_id'):
             for order in self:
-                order.message_post_with_view(
+                order.message_post_with_source(
                     'mail.message_origin_link',
-                    values={'self': order, 'origin': order.requisition_id, 'edit': True},
-                    subtype_id=self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
+                    render_values={'self': order, 'origin': order.requisition_id, 'edit': True},
+                    subtype_xmlid='mail.mt_note',
                 )
         if vals.get('alternative_po_ids', False):
             if not self.purchase_group_id and len(self.alternative_po_ids + self) > len(self):
@@ -215,22 +208,36 @@ class PurchaseOrder(models.Model):
         product_to_best_price_line = defaultdict(lambda: self.env['purchase.order.line'])
         product_to_best_date_line = defaultdict(lambda: self.env['purchase.order.line'])
         product_to_best_price_unit = defaultdict(lambda: self.env['purchase.order.line'])
-        order_lines = self.order_line | self.alternative_po_ids.order_line
-        for line in order_lines:
-            if not line.product_qty or not line.price_subtotal or line.state in ['cancel', 'purchase', 'done']:
+        po_alternatives = self | self.alternative_po_ids
+
+        for line in po_alternatives.order_line:
+            if not line.product_qty or not line.price_total_cc or line.state in ['cancel', 'purchase', 'done']:
                 continue
-            if not product_to_best_price_line[line.product_id] or product_to_best_price_line[line.product_id][0].price_subtotal > line.price_subtotal:
+
+            # if no best price line => no best price unit line either
+            if not product_to_best_price_line[line.product_id]:
                 product_to_best_price_line[line.product_id] = line
-            elif product_to_best_price_line[line.product_id][0].price_subtotal == line.price_subtotal:
-                product_to_best_price_line[line.product_id] |= line
+                product_to_best_price_unit[line.product_id] = line
+            else:
+                price_subtotal = line.price_total_cc
+                price_unit = line.price_total_cc / line.product_qty
+                current_price_subtotal = product_to_best_price_line[line.product_id][0].price_total_cc
+                current_price_unit = product_to_best_price_unit[line.product_id][0].price_total_cc / product_to_best_price_unit[line.product_id][0].product_qty
+
+                if current_price_subtotal > price_subtotal:
+                    product_to_best_price_line[line.product_id] = line
+                elif current_price_subtotal == price_subtotal:
+                    product_to_best_price_line[line.product_id] |= line
+                if current_price_unit > price_unit:
+                    product_to_best_price_unit[line.product_id] = line
+                elif current_price_unit == price_unit:
+                    product_to_best_price_unit[line.product_id] |= line
+
             if not product_to_best_date_line[line.product_id] or product_to_best_date_line[line.product_id][0].date_planned > line.date_planned:
                 product_to_best_date_line[line.product_id] = line
             elif product_to_best_date_line[line.product_id][0].date_planned == line.date_planned:
                 product_to_best_date_line[line.product_id] |= line
-            if not product_to_best_price_unit[line.product_id] or product_to_best_price_unit[line.product_id][0].price_unit > line.price_unit:
-                product_to_best_price_unit[line.product_id] = line
-            elif product_to_best_price_unit[line.product_id][0].price_unit == line.price_unit:
-                product_to_best_price_unit[line.product_id] |= line
+
         best_price_ids = set()
         best_date_ids = set()
         best_price_unit_ids = set()
@@ -242,10 +249,27 @@ class PurchaseOrder(models.Model):
             best_price_unit_ids.update(lines.ids)
         return list(best_price_ids), list(best_date_ids), list(best_price_unit_ids)
 
+    def _prepare_grouped_data(self, rfq):
+        match_fields = super()._prepare_grouped_data(rfq)
+        return match_fields + (rfq.requisition_id.id,)
+
+    def _merge_alternative_po(self, rfqs):
+        if self.alternative_po_ids:
+            super()._merge_alternative_po(rfqs)
+            self.alternative_po_ids += rfqs.mapped('alternative_po_ids')
+
 
 class PurchaseOrderLine(models.Model):
     _inherit = 'purchase.order.line'
 
+    price_total_cc = fields.Monetary(compute='_compute_price_total_cc', string="Company Subtotal", currency_field="company_currency_id", store=True)
+    company_currency_id = fields.Many2one(related="company_id.currency_id", string="Company Currency")
+
+    @api.depends('price_subtotal', 'order_id.currency_rate')
+    def _compute_price_total_cc(self):
+        for line in self:
+            line.price_total_cc = line.price_subtotal / line.order_id.currency_rate
+
     def _compute_price_unit_and_date_planned_and_name(self):
         po_lines_without_requisition = self.env['purchase.order.line']
         for pol in self:
@@ -263,6 +287,10 @@ class PurchaseOrderLine(models.Model):
                         date=pol.order_id.date_order and pol.order_id.date_order.date(),
                         uom_id=line.product_uom_id,
                         params=params)
+
+                    if not pol.date_planned:
+                        pol.date_planned = pol._get_date_planned(seller).strftime(DEFAULT_SERVER_DATETIME_FORMAT)
+
                     product_ctx = {'seller_id': seller.id, 'lang': get_lang(pol.env, partner.lang).code}
                     name = pol._get_product_purchase_description(pol.product_id.with_context(product_ctx))
                     if line.product_description_variants:
