@@ -8,7 +8,10 @@ class PurchaseRequisition(models.Model):
     _inherit = 'purchase.requisition'
 
     def _default_picking_type_id(self):
-        return self.env['stock.picking.type'].search([('warehouse_id.company_id', '=', self.env.company.id), ('code', '=', 'incoming')], limit=1)
+        picking_type = self.env['stock.picking.type'].search([('warehouse_id.company_id', '=', self.env.company.id), ('code', '=', 'incoming')], limit=1)
+        if not picking_type:
+            self.env['stock.warehouse']._warehouse_redirect_warning()
+        return picking_type
 
     warehouse_id = fields.Many2one('stock.warehouse', string='Warehouse', domain="[('company_id', '=', company_id)]")
     picking_type_id = fields.Many2one(
