@@ -5,10 +5,24 @@ from odoo import api, fields, models
 from odoo.osv import expression
 
 
+class ProductCategory(models.Model):
+    _inherit = "product.category"
+
+    property_account_creditor_price_difference_categ = fields.Many2one(
+        'account.account', string="Price Difference Account",
+        company_dependent=True, ondelete='restrict',
+        help="This account will be used to value price difference between purchase price and accounting cost.")
+
+
 class ProductTemplate(models.Model):
     _name = 'product.template'
     _inherit = 'product.template'
 
+    property_account_creditor_price_difference = fields.Many2one(
+        'account.account', string="Price Difference Account", company_dependent=True, ondelete='restrict',
+        help="This account is used in automated inventory valuation to "\
+             "record the price difference between a purchase order and its related vendor bill when validating this vendor bill.")
+
     @api.model
     def _get_buy_route(self):
         buy_route = self.env.ref('purchase_stock.route_warehouse0_buy', raise_if_not_found=False)
@@ -34,45 +48,47 @@ class ProductProduct(models.Model):
         qty_by_product_location, qty_by_product_wh = super()._get_quantity_in_progress(location_ids, warehouse_ids)
         domain = self._get_lines_domain(location_ids, warehouse_ids)
         groups = self.env['purchase.order.line']._read_group(domain,
-            ['product_id', 'product_qty', 'order_id', 'product_uom', 'orderpoint_id'],
-            ['order_id', 'product_id', 'product_uom', 'orderpoint_id'], lazy=False)
-        for group in groups:
-            if group.get('orderpoint_id'):
-                location = self.env['stock.warehouse.orderpoint'].browse(group['orderpoint_id'][:1]).location_id
+            ['order_id', 'product_id', 'product_uom', 'orderpoint_id', 'location_final_id'],
+            ['product_qty:sum'])
+        for order, product, uom, orderpoint, location_final, product_qty_sum in groups:
+            if orderpoint:
+                location = orderpoint.location_id
+            elif location_final:
+                location = location_final
             else:
-                order = self.env['purchase.order'].browse(group['order_id'][0])
                 location = order.picking_type_id.default_location_dest_id
-            product = self.env['product.product'].browse(group['product_id'][0])
-            uom = self.env['uom.uom'].browse(group['product_uom'][0])
-            product_qty = uom._compute_quantity(group['product_qty'], product.uom_id, round=False)
+            product_qty = uom._compute_quantity(product_qty_sum, product.uom_id, round=False)
             qty_by_product_location[(product.id, location.id)] += product_qty
             qty_by_product_wh[(product.id, location.warehouse_id.id)] += product_qty
         return qty_by_product_location, qty_by_product_wh
 
     def _get_lines_domain(self, location_ids=False, warehouse_ids=False):
-        domain = []
+        domains = []
         rfq_domain = [
             ('state', 'in', ('draft', 'sent', 'to approve')),
             ('product_id', 'in', self.ids)
         ]
         if location_ids:
-            domain = expression.AND([rfq_domain, [
+            domains.append(expression.AND([rfq_domain, [
+                '|',
                 '|',
                     ('order_id.picking_type_id.default_location_dest_id', 'in', location_ids),
+                    '&',
+                        ('move_ids', '=', False),
+                        ('location_final_id', 'child_of', location_ids),
                     '&',
                         ('move_dest_ids', '=', False),
                         ('orderpoint_id.location_id', 'in', location_ids)
-            ]])
+            ]]))
         if warehouse_ids:
-            wh_domain = expression.AND([rfq_domain, [
+            domains.append(expression.AND([rfq_domain, [
                 '|',
                     ('order_id.picking_type_id.warehouse_id', 'in', warehouse_ids),
                     '&',
                         ('move_dest_ids', '=', False),
                         ('orderpoint_id.warehouse_id', 'in', warehouse_ids)
-            ]])
-            domain = expression.OR([domain, wh_domain])
-        return domain
+            ]]))
+        return expression.OR(domains) if domains else []
 
 
 class SupplierInfo(models.Model):
@@ -89,7 +105,7 @@ class SupplierInfo(models.Model):
             ('order_line.product_id', 'in',
              self.product_tmpl_id.product_variant_ids.ids),
             ('partner_id', 'in', self.partner_id.ids),
-        ], order='date_order')
+        ], order='date_order desc')
         for supplier in self:
             products = supplier.product_tmpl_id.product_variant_ids
             for purchase in purchases:
@@ -115,7 +131,20 @@ class SupplierInfo(models.Model):
         orderpoint = self.env['stock.warehouse.orderpoint'].browse(orderpoint_id)
         if not orderpoint:
             return
-        orderpoint.route_id = self.env['stock.rule'].search([('action', '=', 'buy')], limit=1).route_id.id
+        if 'buy' not in orderpoint.route_id.rule_ids.mapped('action'):
+            orderpoint.route_id = self.env['stock.rule'].search([('action', '=', 'buy')], limit=1).route_id.id
         orderpoint.supplier_id = self
-        if orderpoint.qty_to_order < self.min_qty:
-            orderpoint.qty_to_order = self.min_qty
+        supplier_min_qty = self.product_uom._compute_quantity(self.min_qty, orderpoint.product_id.uom_id)
+        if orderpoint.qty_to_order < supplier_min_qty:
+            orderpoint.qty_to_order = supplier_min_qty
+        if self._context.get('replenish_id'):
+            replenish = self.env['product.replenish'].browse(self._context.get('replenish_id'))
+            replenish.supplier_id = self
+            return {
+                'type': 'ir.actions.act_window',
+                'name': 'Replenish',
+                'res_model': 'product.replenish',
+                'res_id': replenish.id,
+                'target': 'new',
+                'view_mode': 'form',
+            }
