@@ -26,7 +26,7 @@ class ResPartner(models.Model):
             ('order_id.state', 'in', ['done', 'purchase']),
             ('product_id', 'in', self.env['product.product'].sudo()._search([('type', '!=', 'service')]))
         ])
-        lines_qty_done = defaultdict(lambda: 0)
+        lines_quantity = defaultdict(lambda: 0)
         moves = self.env['stock.move'].search([
             ('purchase_line_id', 'in', order_lines.ids),
             ('state', '=', 'done')])
@@ -34,13 +34,13 @@ class ResPartner(models.Model):
         order_lines.read(['date_planned', 'partner_id', 'product_uom_qty'], load='')
         moves.read(['purchase_line_id', 'date'], load='')
         moves = moves.filtered(lambda m: m.date.date() <= m.purchase_line_id.date_planned.date())
-        for move, qty_done in zip(moves, moves.mapped('quantity_done')):
-            lines_qty_done[move.purchase_line_id.id] += qty_done
+        for move, quantity in zip(moves, moves.mapped('quantity')):
+            lines_quantity[move.purchase_line_id.id] += quantity
         partner_dict = {}
         for line in order_lines:
             on_time, ordered = partner_dict.get(line.partner_id, (0, 0))
             ordered += line.product_uom_qty
-            on_time += lines_qty_done[line.id]
+            on_time += lines_quantity[line.id]
             partner_dict[line.partner_id] = (on_time, ordered)
         seen_partner = self.env['res.partner']
         for partner, numbers in partner_dict.items():
