@@ -6,6 +6,7 @@ from . import test_average_price
 from . import test_create_picking
 from . import test_fifo_price
 from . import test_fifo_returns
+from . import test_lot_valuation
 from . import test_onchange_product
 from . import test_purchase_delete_order
 from . import test_purchase_lead_time
@@ -19,3 +20,6 @@ from . import test_move_cancel_propagation
 from . import test_product_template
 from . import test_routes
 from . import test_purchase_stock_accrued_entries
+from . import test_uninstall
+from . import test_purchase_method
+from . import test_old_rules
