@@ -12,20 +12,20 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
         cls.customer = cls.env['res.partner'].create({'name': 'abc'})
         cls.group = cls.env['procurement.group'].create({'partner_id': cls.customer.id, 'name': 'New Group'})
         cls.warehouse = cls.env.ref('stock.warehouse0')
-        cust_location = cls.env.ref('stock.stock_location_customers')
+        cls.cust_location = cls.env.ref('stock.stock_location_customers')
         seller = cls.env['product.supplierinfo'].create({
             'partner_id': cls.customer.id,
             'price': 100.0,
         })
         product = cls.env['product.product'].create({
             'name': 'Geyser',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': [(4, cls.route_mto), (4, cls.route_buy)],
             'seller_ids': [(6, 0, [seller.id])],
         })
         cls.picking_out = cls.env['stock.picking'].create({
             'location_id': cls.warehouse.out_type_id.default_location_src_id.id,
-            'location_dest_id': cust_location.id,
+            'location_dest_id': cls.cust_location.id,
             'partner_id': cls.customer.id,
             'group_id': cls.group.id,
             'picking_type_id': cls.env.ref('stock.picking_type_out').id,
@@ -38,7 +38,7 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
             'picking_id': cls.picking_out.id,
             'group_id': cls.group.id,
             'location_id': cls.warehouse.out_type_id.default_location_src_id.id,
-            'location_dest_id': cust_location.id,
+            'location_dest_id': cls.cust_location.id,
             'procure_method': 'make_to_order',
         })
 
@@ -99,7 +99,7 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
         self.assertEqual(picking_in.state, 'cancel')
         self.assertNotEqual(self.picking_out.state, 'cancel')
 
-    def test_03_cancel_draft_purchase_order_two_steps(self):
+    def test_03_cancel_draft_purchase_order_two_steps_push(self):
         """ Check the picking and moves status related PO, When canceling purchase order
             in 'draft' state.
             Ex.
@@ -109,32 +109,27 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
                   but it should not cancel < PICK, Delivery >
         """
         self.warehouse.write({'delivery_steps': 'pick_ship', 'reception_steps': 'two_steps'})
-        self.picking_out.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.move.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.picking_out.action_confirm()
+        self.move.write({
+            'picking_id': False,
+            'picking_type_id': self.warehouse.pick_type_id.id,
+            'location_final_id': self.cust_location,
+        })
+        self.move._action_confirm()
+        self.assertEqual(self.move.picking_id.state, 'waiting')
 
         # Find purchase order related to picking.
         purchase_order = self.env['purchase.order'].search([('partner_id', '=', self.customer.id)])
         # Purchase order should be created for picking.
         self.assertTrue(purchase_order, 'No purchase order created.')
-
-        picking_ids = self.env['stock.picking'].search([('group_id', '=', self.group.id)])
-
-        internal = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.int_type_id and r.group_id.id == self.group.id)
-        pick = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pick_type_id and r.group_id.id == self.group.id)
-
         # Check status of Purchase Order
         self.assertEqual(purchase_order.state, 'draft', "Purchase order should be in 'draft' state.")
         # Cancel Purchase order.
         purchase_order.button_cancel()
 
         # Check the status of picking after canceling po.
-        for res in internal:
-            self.assertEqual(res.state, 'cancel')
-        self.assertNotEqual(pick.state, 'cancel')
-        self.assertNotEqual(self.picking_out.state, 'cancel')
+        self.assertNotEqual(self.move.picking_id.state, 'cancel')
 
-    def test_04_cancel_confirm_purchase_order_two_steps(self):
+    def test_04_cancel_confirm_purchase_order_two_steps_push(self):
         """ Check the picking and moves status related PO, When canceling purchase order
             Ex.
                 1) Set 2 steps of receiption and delivery on the warehouse.
@@ -143,20 +138,19 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
                   not < PICK, SHIP >
         """
         self.warehouse.write({'delivery_steps': 'pick_ship', 'reception_steps': 'two_steps'})
-        self.picking_out.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.move.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.picking_out.action_confirm()
+        self.move.write({
+            'picking_id': False,
+            'picking_type_id': self.warehouse.pick_type_id.id,
+            'location_final_id': self.cust_location,
+        })
+        self.move._action_confirm()
+        self.assertEqual(self.move.picking_id.state, 'waiting')
 
         # Find PO related to picking.
         purchase_order = self.env['purchase.order'].search([('partner_id', '=', self.customer.id)])
         # Po should be create related picking.
         self.assertTrue(purchase_order, 'purchase order is created.')
 
-        picking_ids = self.env['stock.picking'].search([('group_id', '=', self.group.id)])
-
-        internal = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.int_type_id)
-        pick = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pick_type_id)
-
         # Check status of Purchase Order
         self.assertEqual(purchase_order.state, 'draft', "Purchase order should be in 'draft' state.")
 
@@ -167,12 +161,9 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
 
         # Check the status of picking after canceling po.
         self.assertEqual(picking_in.state, 'cancel')
-        for res in internal:
-            self.assertEqual(res.state, 'cancel')
-        self.assertNotEqual(pick.state, 'cancel')
-        self.assertNotEqual(self.picking_out.state, 'cancel')
+        self.assertNotEqual(self.move.picking_id.state, 'cancel')
 
-    def test_05_cancel_draft_purchase_order_three_steps(self):
+    def test_05_cancel_draft_purchase_order_three_steps_push(self):
         """ Check the picking and moves status related PO, When canceling purchase order
             Ex.
                 1) Set 3 steps of receiption and delivery on the warehouse.
@@ -181,34 +172,27 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
                   not < PICK, PACK, SHIP >
         """
         self.warehouse.write({'delivery_steps': 'pick_pack_ship', 'reception_steps': 'three_steps'})
-        self.picking_out.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.move.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.picking_out.action_confirm()
+        self.move.write({
+            'picking_id': False,
+            'picking_type_id': self.warehouse.pick_type_id.id,
+            'location_final_id': self.cust_location,
+        })
+        self.move._action_confirm()
+        self.assertEqual(self.move.picking_id.state, 'waiting')
 
         # Find PO related to picking.
         purchase_order = self.env['purchase.order'].search([('partner_id', '=', self.customer.id)])
         # Po should be create related picking.
         self.assertTrue(purchase_order, 'No purchase order created.')
 
-        picking_ids = self.env['stock.picking'].search([('group_id', '=', self.group.id)])
-
-        internal = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.int_type_id)
-        pick = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pick_type_id)
-        pack = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pack_type_id)
-
         # Check status of Purchase Order
         self.assertEqual(purchase_order.state, 'draft', "Purchase order should be in 'draft' state.")
         # Cancel Purchase order.
         purchase_order.button_cancel()
 
-        # Check the status of picking after canceling po.
-        for res in internal:
-            self.assertEqual(res.state, 'cancel')
-        self.assertNotEqual(pick.state, 'cancel')
-        self.assertNotEqual(pack.state, 'cancel')
-        self.assertNotEqual(self.picking_out.state, 'cancel')
+        self.assertNotEqual(self.move.picking_id.state, 'cancel')
 
-    def test_06_cancel_confirm_purchase_order_three_steps(self):
+    def test_06_cancel_confirm_purchase_order_three_steps_push(self):
         """ Check the picking and moves status related PO, When canceling purchase order
             Ex.
                 1) Set 3 steps of receiption and delivery on the warehouse.
@@ -217,21 +201,19 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
                   not < PICK, PACK, SHIP >
         """
         self.warehouse.write({'delivery_steps': 'pick_pack_ship', 'reception_steps': 'three_steps'})
-        self.picking_out.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.move.write({'location_id': self.warehouse.out_type_id.default_location_src_id.id})
-        self.picking_out.action_confirm()
+        self.move.write({
+            'picking_id': False,
+            'picking_type_id': self.warehouse.pick_type_id.id,
+            'location_final_id': self.cust_location,
+        })
+        self.move._action_confirm()
+        self.assertEqual(self.move.picking_id.state, 'waiting')
 
         # Find PO related to picking.
         purchase_order = self.env['purchase.order'].search([('partner_id', '=', self.customer.id)])
         # Po should be create related picking.
         self.assertTrue(purchase_order, 'No purchase order created.')
 
-        picking_ids = self.env['stock.picking'].search([('group_id', '=', self.group.id)])
-
-        internal = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.int_type_id)
-        pick = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pick_type_id)
-        pack = picking_ids.filtered(lambda r: r.picking_type_id == self.warehouse.pack_type_id)
-
         # Check status of Purchase Order
         self.assertEqual(purchase_order.state, 'draft', "Purchase order should be in 'draft' state.")
 
@@ -242,11 +224,7 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
 
         # Check the status of picking after canceling po.
         self.assertEqual(picking_in.state, 'cancel')
-        for res in internal:
-            self.assertEqual(res.state, 'cancel')
-        self.assertNotEqual(pick.state, 'cancel')
-        self.assertNotEqual(pack.state, 'cancel')
-        self.assertNotEqual(self.picking_out.state, 'cancel')
+        self.assertNotEqual(self.move.picking_id.state, 'cancel')
 
     def test_cancel_move_lines_operation(self):
         """Check for done and cancelled moves. Ensure that the RFQ cancellation
@@ -265,7 +243,7 @@ class TestMoveCancelPropagation(PurchaseTestCommon):
         })
         product_car = self.env['product.product'].create({
             'name': 'Car',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': [(4, self.ref('stock.route_warehouse0_mto')), (4, self.ref('purchase_stock.route_warehouse0_buy'))],
             'seller_ids': [(6, 0, [seller.id])],
             'categ_id': self.env.ref('product.product_category_all').id,
