@@ -6,7 +6,7 @@ from unittest.mock import patch
 
 from odoo import fields
 from .common import PurchaseTestCommon
-from odoo.tests.common import Form
+from odoo.tests import Form
 
 
 class TestPurchaseLeadTime(PurchaseTestCommon):
@@ -94,71 +94,6 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         purchase_form.save()
         self.assertEqual(purchase2.picking_ids.date_deadline, purchase2.date_planned, "Deadline of pickings should be propagate")
 
-    def test_02_product_route_level_delays(self):
-        """ In order to check dates, set product's Delivery Lead Time
-            and warehouse route's delay."""
-
-        company = self.env.ref('base.main_company')
-        company.write({'po_lead': 1.00})
-
-        # Update warehouse_1 with Incoming Shipments 3 steps
-        self.warehouse_1.write({'reception_steps': 'three_steps'})
-
-        # Set delay on push rule
-        for push_rule in self.warehouse_1.reception_route_id.rule_ids:
-            push_rule.write({'delay': 2})
-
-        rule_delay = sum(self.warehouse_1.reception_route_id.rule_ids.mapped('delay'))
-
-        date_planned = fields.Datetime.to_string(fields.datetime.now() + timedelta(days=10))
-        # Create procurement order of product_1
-        self.env['procurement.group'].run([self.env['procurement.group'].Procurement(
-            self.product_1, 5.000, self.uom_unit, self.warehouse_1.lot_stock_id, 'Test scheduler for RFQ', '/', self.env.company,
-            {
-                'warehouse_id': self.warehouse_1,
-                'date_planned': date_planned,  # 10 days added to current date of procurement to get future schedule date and order date of purchase order.
-                'date_deadline': date_planned,  # 10 days added to current date of procurement to get future schedule date and order date of purchase order.
-                'rule_id': self.warehouse_1.buy_pull_id,
-                'group_id': False,
-                'route_ids': [],
-            }
-        )])
-
-        # Confirm purchase order
-        purchase = self.env['purchase.order.line'].search([('product_id', '=', self.product_1.id)], limit=1).order_id
-        purchase.button_confirm()
-
-        # Check order date of purchase order
-        order_date = fields.Datetime.from_string(date_planned) - timedelta(days=self.product_1.seller_ids.delay + rule_delay)
-        self.assertEqual(purchase.date_order, order_date, 'Order date should be equal to: Date of the procurement order - Delivery Lead Time(supplier and pull rules).')
-
-        # Check scheduled date of purchase order
-        schedule_date = order_date + timedelta(days=self.product_1.seller_ids.delay + rule_delay)
-        self.assertEqual(date_planned, str(schedule_date), 'Schedule date should be equal to: Order date of Purchase order + Delivery Lead Time(supplier and pull rules).')
-
-        # Check the picking crated or not
-        self.assertTrue(purchase.picking_ids, "Picking should be created.")
-
-        # Check scheduled date of Internal Type shipment
-        incoming_shipment1 = self.env['stock.picking'].search([('move_ids.product_id', 'in', (self.product_1.id, self.product_2.id)), ('picking_type_id', '=', self.warehouse_1.int_type_id.id), ('location_id', '=', self.warehouse_1.wh_input_stock_loc_id.id), ('location_dest_id', '=', self.warehouse_1.wh_qc_stock_loc_id.id)])
-        incoming_shipment1_date = order_date + timedelta(days=self.product_1.seller_ids.delay)
-        self.assertEqual(incoming_shipment1.scheduled_date, incoming_shipment1_date, 'Schedule date of Internal Type shipment for input stock location should be equal to: schedule date of purchase order + push rule delay.')
-        self.assertEqual(incoming_shipment1.date_deadline, incoming_shipment1_date)
-        old_deadline1 = incoming_shipment1.date_deadline
-
-        incoming_shipment2 = self.env['stock.picking'].search([('picking_type_id', '=', self.warehouse_1.int_type_id.id), ('location_id', '=', self.warehouse_1.wh_qc_stock_loc_id.id), ('location_dest_id', '=', self.warehouse_1.lot_stock_id.id)])
-        incoming_shipment2_date = schedule_date - timedelta(days=incoming_shipment2.move_ids[0].rule_id.delay)
-        self.assertEqual(incoming_shipment2.scheduled_date, incoming_shipment2_date, 'Schedule date of Internal Type shipment for quality control stock location should be equal to: schedule date of Internal type shipment for input stock location + push rule delay..')
-        self.assertEqual(incoming_shipment2.date_deadline, incoming_shipment2_date)
-        old_deadline2 = incoming_shipment2.date_deadline
-
-        # Modify the date_planned of the purchase -> propagate the deadline
-        purchase_form = Form(purchase)
-        purchase_form.date_planned = purchase.date_planned + timedelta(days=1)
-        purchase_form.save()
-        self.assertEqual(incoming_shipment2.date_deadline, old_deadline2 + timedelta(days=1), 'Deadline should be propagate')
-        self.assertEqual(incoming_shipment1.date_deadline, old_deadline1 + timedelta(days=1), 'Deadline should be propagate')
-
     def test_merge_po_line(self):
         """Change that merging po line for same procurement is done."""
 
@@ -288,7 +223,7 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         # Test if the orderpoint is created when opening the replenishment view
         prod = self.env['product.product'].create({
             'name': 'Carrot',
-            'type': 'product',
+            'is_storable': True,
             'seller_ids': [
                 (0, 0, {'partner_id': vendor.id, 'delay': 1.0, 'company_id': company.id})
             ]
@@ -313,7 +248,7 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         # Test if purchase orders are created according to the days to purchase
         product = self.env['product.product'].create({
             'name': 'Chicory',
-            'type': 'product',
+            'is_storable': True,
             'seller_ids': [
                 (0, 0, {'partner_id': vendor2.id, 'delay': 15.0, 'company_id': company2.id}),
                 (0, 0, {'partner_id': vendor.id, 'delay': 1.0, 'company_id': company.id})
@@ -344,7 +279,8 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         delivery_moves._action_confirm()
         self.env['procurement.group'].run_scheduler()
         po_line = self.env['purchase.order.line'].search([('product_id', '=', product.id)])
-        self.assertEqual(fields.Date.to_date(po_line.order_id.date_order), fields.Date.today() + timedelta(days=2))
+        expected_date_order = fields.Date.today() + timedelta(days=2)
+        self.assertEqual(fields.Date.to_date(po_line.order_id.date_order), expected_date_order)
         self.assertEqual(len(po_line), 1)
         self.assertEqual(po_line.product_uom_qty, 25.0)
         self.assertEqual(len(po_line.order_id), 1)
@@ -354,12 +290,10 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         self.mock_date.today.return_value = fields.Date.today() + timedelta(days=2)
         orderpoint._compute_qty()
         self.env['procurement.group'].run_scheduler()
-        po_line = self.env['purchase.order.line'].search([('product_id', '=', product.id)])
-        self.assertEqual(len(po_line), 2)
-        self.assertEqual(len(po_line.order_id), 2)
-        new_order = po_line.order_id.sorted('date_order')[-1]
-        self.assertEqual(fields.Date.to_date(new_order.date_order), fields.Date.today() + timedelta(days=2))
-        self.assertEqual(new_order.order_line.product_uom_qty, 5.0)
+        po_line02 = self.env['purchase.order.line'].search([('product_id', '=', product.id)])
+        self.assertEqual(po_line02, po_line, 'The orderpoint execution should not create a new POL')
+        self.assertEqual(fields.Date.to_date(po_line.order_id.date_order), expected_date_order, 'The Order Deadline should not change')
+        self.assertEqual(po_line.product_uom_qty, 30.0, 'The existing POL should be updated with the quantity of the last execution')
 
     def test_supplier_lead_time(self):
         """ Basic stock configuration and a supplier with a minimum qty and a lead time """
@@ -382,6 +316,6 @@ class TestPurchaseLeadTime(PurchaseTestCommon):
         self.env['procurement.group'].run_scheduler()
         purchase_order = self.env['purchase.order'].search([('partner_id', '=', self.partner_1.id)])
 
-        today = fields.Datetime.start_of(fields.Datetime.now(), 'day')
+        today = datetime.combine(fields.Datetime.now(), time(12))
         self.assertEqual(purchase_order.date_order, today)
-        self.assertEqual(fields.Datetime.start_of(purchase_order.date_planned, 'day'), today + timedelta(days=7))
+        self.assertEqual(purchase_order.date_planned, today + timedelta(days=7))
