@@ -2,11 +2,12 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import re
 from datetime import datetime, timedelta
-from freezegun import freeze_time
 
+from odoo import Command
 from odoo.tools import DEFAULT_SERVER_DATETIME_FORMAT
 from odoo.addons.stock_account.tests.test_anglo_saxon_valuation_reconciliation_common import ValuationReconciliationTestCommon
-from odoo.tests import Form, tagged
+from odoo.exceptions import UserError
+from odoo.tests import Form, tagged, freeze_time
 
 
 @freeze_time("2021-01-14 09:12:15")
@@ -14,8 +15,8 @@ from odoo.tests import Form, tagged
 class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
     @classmethod
-    def setUpClass(cls, chart_template_ref=None):
-        super().setUpClass(chart_template_ref=chart_template_ref)
+    def setUpClass(cls):
+        super().setUpClass()
 
         cls.product_id_1 = cls.env['product.product'].create({'name': 'Large Desk', 'purchase_method': 'purchase'})
         cls.product_id_2 = cls.env['product.product'].create({'name': 'Conference Chair', 'purchase_method': 'purchase'})
@@ -71,7 +72,8 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
         self.assertEqual(self.po.incoming_picking_count, 1, 'Purchase: one picking should be created"')
         self.picking = self.po.picking_ids[0]
-        self.picking.move_line_ids.write({'qty_done': 5.0})
+        self.picking.move_line_ids.write({'quantity': 5.0})
+        self.picking.move_ids.picked = True
         self.picking.button_validate()
         self.assertEqual(self.po.order_line.mapped('qty_received'), [5.0, 5.0], 'Purchase: all products should be received"')
 
@@ -102,7 +104,8 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         self.assertEqual(self.po.state, 'purchase', 'Purchase: PO state should be "Purchase')
         self.assertEqual(self.po.incoming_picking_count, 1, 'Purchase: one picking should be created"')
         self.picking = self.po.picking_ids[0]
-        self.picking.move_line_ids.write({'qty_done': 5.0})
+        self.picking.move_line_ids.write({'quantity': 5.0})
+        self.picking.move_ids.picked = True
         self.picking.button_validate()
         self.assertEqual(self.po.order_line.mapped('qty_received'), [5.0, 5.0], 'Purchase: all products should be received"')
 
@@ -127,12 +130,12 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
             active_model='stock.picking'))
         return_wiz = stock_return_picking_form.save()
         return_wiz.product_return_moves.write({'quantity': 2.0, 'to_refund': True})  # Return only 2
-        res = return_wiz.create_returns()
+        res = return_wiz.action_create_returns()
         return_pick = self.env['stock.picking'].browse(res['res_id'])
 
         # Validate picking
-        return_pick.move_line_ids.write({'qty_done': 2})
-
+        return_pick.move_line_ids.write({'quantity': 2})
+        return_pick.move_ids.picked = True
         return_pick.button_validate()
 
         # Check Received quantity
@@ -144,10 +147,10 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         # Not supposed to see/change the purchase order of a refund invoice by default
         # <field name="purchase_id" invisible="1"/>
         # <label for="purchase_vendor_bill_id" string="Auto-Complete" class="oe_edit_only"
-        #         attrs="{'invisible': ['|', ('state','!=','draft'), ('move_type', '!=', 'in_invoice')]}" />
+        #         invisible="state != 'draft' or move_type != 'in_invoice'" />
         # <field name="purchase_vendor_bill_id" nolabel="1"
-        #         attrs="{'invisible': ['|', ('state','!=','draft'), ('move_type', '!=', 'in_invoice')]}"
-        move_form._view['modifiers']['purchase_id']['invisible'] = False
+        #         invisible="state != 'draft' or move_type != 'in_invoice'"
+        move_form._view['modifiers']['purchase_id']['invisible'] = 'False'
         move_form.purchase_id = self.po
         self.invoice = move_form.save()
         move_form = Form(self.invoice)
@@ -188,9 +191,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         po1.button_confirm()
 
         picking = po1.picking_ids
-        wiz_act = picking.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        picking.button_validate()
 
         # Return 5 units
         stock_return_picking_form = Form(self.env['stock.return.picking'].with_context(
@@ -204,14 +205,15 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
                 'quantity': 5,
                 'to_refund': True
             })
-        res = return_wiz.create_returns()
+        res = return_wiz.action_create_returns()
         return_pick = self.env['stock.picking'].browse(res['res_id'])
-        wiz_act = return_pick.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        return_pick.button_validate()
 
         self.assertEqual(po1.order_line.qty_received, 5)
 
+        with self.assertRaises(UserError, msg="Shouldn't allow ordered qty be lower than received"):
+            po1.order_line.product_qty = po1.order_line.qty_received - 0.01
+
         # Deliver 15 instead of 10.
         po1.write({
             'order_line': [
@@ -223,6 +225,16 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         self.assertEqual(po1.order_line.qty_received, 5)
         self.assertEqual(po1.picking_ids[-1].move_ids.product_qty, 10)
 
+        # Modify after invoicing
+        po1.action_create_invoice()
+        self.assertEqual(po1.order_line.qty_invoiced, 15)
+        self.assertFalse(po1.invoice_ids.activity_ids)
+        po1.order_line.product_qty = 14.99
+        self.assertTrue(
+            po1.invoice_ids.activity_ids,
+            "Lowering product qty below invoiced qty should schedule an activity",
+        )
+
     def test_04_update_date_planned(self):
         today = datetime.today().replace(hour=9, microsecond=0)
         tomorrow = datetime.today().replace(hour=9, microsecond=0) + timedelta(days=1)
@@ -246,9 +258,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         )
 
         # receive products
-        wiz_act = po.picking_ids.button_validate()
-        wiz = Form(self.env[wiz_act['res_model']].with_context(wiz_act['context'])).save()
-        wiz.process()
+        po.picking_ids.button_validate()
 
         # update second line
         old_date = po.order_line[1].date_planned
@@ -300,12 +310,13 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         po.button_confirm()
         picking = po.picking_ids[0]
         # Process 9.0 out of the 10.0 ordered qty
-        picking.move_line_ids.write({'qty_done': 9.0})
+        picking.move_line_ids.write({'quantity': 9.0})
+        picking.move_ids.picked = True
         res_dict = picking.button_validate()
         # No backorder
         self.env['stock.backorder.confirmation'].with_context(res_dict['context']).process_cancel_backorder()
         # `on_time_rate` should be equals to the ratio of quantity received against quantity ordered
-        expected_rate = sum(picking.move_line_ids.mapped("qty_done")) / sum(po.order_line.mapped("product_qty")) * 100
+        expected_rate = sum(picking.move_line_ids.mapped("quantity")) / sum(po.order_line.mapped("product_qty")) * 100
         self.assertEqual(expected_rate, po.on_time_rate)
 
         # Create a purchase order with 80% qty received for company B
@@ -319,12 +330,13 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         po.button_confirm()
         picking = po.picking_ids[0]
         # Process 8.0 out of the 10.0 ordered qty
-        picking.move_line_ids.write({'qty_done': 8.0})
+        picking.move_line_ids.write({'quantity': 8.0})
+        picking.move_ids.picked = True
         res_dict = picking.button_validate()
         # No backorder
         self.env['stock.backorder.confirmation'].with_context(res_dict['context']).process_cancel_backorder()
         # `on_time_rate` should be equal to the ratio of quantity received against quantity ordered
-        expected_rate = sum(picking.move_line_ids.mapped("qty_done")) / sum(po.order_line.mapped("product_qty")) * 100
+        expected_rate = sum(picking.move_line_ids.mapped("quantity")) / sum(po.order_line.mapped("product_qty")) * 100
         self.assertEqual(expected_rate, po.on_time_rate)
 
         # Tricky corner case
@@ -372,7 +384,8 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         })
         po.button_confirm()
         picking = po.picking_ids[0]
-        picking.move_line_ids.write({'qty_done': 3.66})
+        picking.move_line_ids.write({'quantity': 3.66})
+        picking.move_ids.picked = True
         picking.button_validate()
         self.assertEqual(po.order_line.mapped('qty_received'), [4.0], 'Purchase: no conversion error on receipt in different uom"')
 
@@ -392,13 +405,13 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
         super_product = self.env['product.product'].create({
             'name': 'Super Product',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': self.stock_account_product_categ.id,
             'standard_price': 9.876543,
         })
 
         purchase_order = self.env['purchase.order'].create({
-            'partner_id': self.env.ref('base.res_partner_3').id,
+            'partner_id': self.env['res.partner'].create({'name': 'Test Partner'}).id,
             'order_line': [(0, 0, {
                 'name': super_product.name,
                 'product_id': super_product.id,
@@ -413,7 +426,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         self.assertEqual(purchase_order.state, 'purchase')
         self.assertEqual(len(purchase_order.picking_ids), 1)
         self.assertEqual(len(purchase_order.picking_ids.move_line_ids), 1)
-        self.assertEqual(purchase_order.picking_ids.move_line_ids.reserved_qty, 7)
+        self.assertEqual(purchase_order.picking_ids.move_line_ids.quantity_product_uom, 7)
 
 
         purchase_order.order_line.product_qty = 4
@@ -422,7 +435,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         purchase_order.button_confirm()
         self.assertEqual(len(purchase_order.picking_ids), 1)
         self.assertEqual(len(purchase_order.picking_ids.move_line_ids), 1)
-        self.assertEqual(purchase_order.picking_ids.move_line_ids.reserved_qty, 4)
+        self.assertEqual(purchase_order.picking_ids.move_line_ids.quantity_product_uom, 4)
 
     def test_message_qty_already_received(self):
         self.env.user.write({'company_id': self.company_data['company'].id})
@@ -441,22 +454,16 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         _purchase_order.button_confirm()
 
         first_picking = _purchase_order.picking_ids[0]
-        first_picking.move_ids.quantity_done = 5
-        backorder_wizard_dict = first_picking.button_validate()
-        backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
-        backorder_wizard.process()
+        first_picking.move_ids.quantity = 5
+        Form.from_action(self.env, first_picking.button_validate()).save().process()
 
         second_picking = _purchase_order.picking_ids[1]
-        second_picking.move_ids.quantity_done = 5
-        backorder_wizard_dict = second_picking.button_validate()
-        backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
-        backorder_wizard.process()
+        second_picking.move_ids.quantity = 5
+        Form.from_action(self.env, second_picking.button_validate()).save().process()
 
         third_picking = _purchase_order.picking_ids[2]
-        third_picking.move_ids.quantity_done = 5
-        backorder_wizard_dict = third_picking.button_validate()
-        backorder_wizard = Form(self.env[backorder_wizard_dict['res_model']].with_context(backorder_wizard_dict['context'])).save()
-        backorder_wizard.process()
+        third_picking.move_ids.quantity = 5
+        Form.from_action(self.env, third_picking.button_validate()).save().process()
 
         _message_content = _purchase_order.message_ids.mapped("body")[0]
         self.assertIsNotNone(re.search(r"Received Quantity: 5.0 -&gt; 10.0", _message_content), "Already received quantity isn't correctly taken into consideration")
@@ -490,7 +497,7 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         orderpoint_form = Form(self.env['stock.warehouse.orderpoint'])
         orderpoint_form.product_id = product
         orderpoint_form.product_min_qty = 1
-        orderpoint_form.product_max_qty = 0.000
+        orderpoint_form.product_max_qty = 1.000
         orderpoint_form.save()
 
         self.env['procurement.group'].run_scheduler()
@@ -526,6 +533,51 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
 
         self.assertEqual(po.picking_ids.move_ids.product_uom_qty, 8)
 
+    def test_packaging_propagation(self):
+        """ Editing the packaging on an purchase.order.line
+        should propagate to the delivery order, so that
+        when we are editing the packaging, the lines can be merged
+        with the new packaging and quantity.
+        """
+        # set the 3 step route
+        warehouse = self.company_data['default_warehouse']
+        warehouse.reception_steps = 'three_steps'
+        packOf10 = self.env['product.packaging'].create({
+            'name': 'PackOf10',
+            'product_id': self.product_a.id,
+            'qty': 10
+        })
+
+        packOf20 = self.env['product.packaging'].create({
+            'name': 'PackOf20',
+            'product_id': self.product_a.id,
+            'qty': 20
+        })
+
+        po = self.env['purchase.order'].create({
+            'partner_id': self.partner_a.id,
+            'order_line': [
+                (0, 0, {
+                    'product_id': self.product_a.id,
+                    'product_uom_qty': 10.0,
+                    'product_uom': self.product_a.uom_id.id,
+                    'product_packaging_id': packOf10.id,
+                })],
+        })
+        po.button_confirm()
+        # the 3 moves for the 3 steps
+        step_1 = po.order_line.move_ids
+        self.assertEqual(step_1.product_packaging_id, packOf10)
+
+        po.order_line[0].write({
+            'product_packaging_id': packOf20.id,
+            'product_uom_qty': 20
+        })
+        self.assertEqual(step_1.product_packaging_id, packOf20)
+
+        po.order_line[0].write({'product_packaging_id': False})
+        self.assertFalse(step_1.product_packaging_id)
+
     def test_putaway_strategy_in_backorder(self):
         stock_location = self.company_data['default_warehouse'].lot_stock_id
         sub_loc_01 = self.env['stock.location'].create([{
@@ -550,8 +602,148 @@ class TestPurchaseOrder(ValuationReconciliationTestCommon):
         picking = po.picking_ids
         self.assertEqual(po.state, "purchase")
         self.assertEqual(picking.move_line_ids_without_package.location_dest_id.id, sub_loc_01.id)
-        picking.move_line_ids_without_package.write({'qty_done': 1})
+        picking.move_line_ids_without_package.write({'quantity': 1})
+        picking.move_ids.write({'picked': True})
         res_dict = picking.button_validate()
         self.env[res_dict['res_model']].with_context(res_dict['context']).process()
         backorder = picking.backorder_ids
         self.assertEqual(backorder.move_line_ids_without_package.location_dest_id.id, sub_loc_01.id)
+
+    def test_inventory_adjustments_with_po(self):
+        """ check that the quant created by a PO can be applied in an inventory adjustment correctly """
+        product = self.env['product.product'].create({
+            'name': 'Product A',
+            'is_storable': True,
+        })
+        po_form = Form(self.env['purchase.order'])
+        po_form.partner_id = self.partner_a
+        with po_form.order_line.new() as line:
+            line.product_id = product
+            line.product_qty = 5
+        po = po_form.save()
+        po.button_confirm()
+        po.picking_ids.move_ids.quantity = 5
+        po.picking_ids.move_ids.picked = True
+        po.picking_ids.button_validate()
+        self.assertEqual(po.picking_ids.state, 'done')
+        quant = self.env['stock.quant'].search([('product_id', '=', product.id), ('location_id.usage', '=', 'internal')])
+        wizard = self.env['stock.inventory.adjustment.name'].create({'quant_ids': quant})
+        wizard.action_apply()
+        self.assertEqual(quant.quantity, 5)
+
+    def test_po_edit_after_receive(self):
+        self.po = self.env['purchase.order'].create(self.po_vals)
+        self.po.button_confirm()
+        self.po.picking_ids.move_ids.quantity = 5
+        self.po.picking_ids.move_ids.picked = True
+        self.po.picking_ids.button_validate()
+        self.assertEqual(self.po.picking_ids.move_ids.mapped('product_uom_qty'), [5.0, 5.0])
+        self.po.with_context(import_file=True).order_line[0].product_qty = 10
+        self.assertEqual(self.po.picking_ids.move_ids.mapped('product_uom_qty'), [5.0, 5.0, 5.0])
+
+    def test_receive_returned_product_without_po_update(self):
+        """
+        Receive again the returned qty, but with the option "Update PO" disabled
+        At the end, the received qty of the POL should be correct
+        """
+        po = self.env['purchase.order'].create(self.po_vals)
+        po.button_confirm()
+
+        receipt01 = po.picking_ids
+        receipt01.move_ids.quantity = 5
+        receipt01.button_validate()
+
+        wizard = Form(self.env['stock.return.picking'].with_context(active_ids=receipt01.ids, active_id=receipt01.id, active_model='stock.picking')).save()
+        wizard.product_return_moves.quantity = 5
+        wizard.product_return_moves.to_refund = False
+        res = wizard.action_create_returns()
+
+        return_pick = self.env['stock.picking'].browse(res['res_id'])
+        return_pick.move_ids.quantity = 5
+        return_pick.button_validate()
+
+        wizard = Form(self.env['stock.return.picking'].with_context(active_ids=return_pick.ids, active_id=return_pick.id, active_model='stock.picking')).save()
+        wizard.product_return_moves.quantity = 5
+        wizard.product_return_moves.to_refund = False
+        res = wizard.action_create_returns()
+
+        receipt02 = self.env['stock.picking'].browse(res['res_id'])
+        receipt02.move_ids.quantity = 5
+        receipt02.button_validate()
+
+        self.assertEqual(po.order_line[0].qty_received, 5)
+        self.assertEqual(po.order_line[1].qty_received, 5)
+
+    def test_receive_qty_invoiced_but_no_posted(self):
+        """
+        Create a purchase order, confirm it, invoice it, but don't post the invoice.
+        Receive the products.
+        """
+        self.product_id_1.is_storable = True
+        self.product_id_1.categ_id.property_cost_method = 'average'
+        po = self.env['purchase.order'].create(self.po_vals)
+        po.button_confirm()
+        self.assertEqual(po.order_line[0].product_id, self.product_id_1)
+        # Invoice the PO
+        action = po.action_create_invoice()
+        invoice = self.env['account.move'].browse(action['res_id'])
+        self.assertTrue(invoice)
+        # Receive the products
+        receipt01 = po.picking_ids
+        receipt01.button_validate()
+        self.assertEqual(receipt01.state, 'done')
+        self.assertEqual(po.order_line[0].qty_received, 5)
+        self.assertEqual(po.order_line[0].price_unit, 500)
+        layers = self.env['stock.valuation.layer'].search([('product_id', '=', self.product_id_1.id)])
+        self.assertEqual(len(layers), 1)
+        self.assertEqual(layers.quantity, 5)
+        self.assertEqual(layers.value, 2500)
+
+    def test_stock_picking_type_for_deliveries_generated_from_po(self):
+        """
+        Checks that the stock picking type of a PO generated by an orderpoint
+        influence the destination of the delivery, if the stock picking type
+        is more precise than the orderpoint.
+        """
+        product = self.env['product.product'].create({
+            'name': 'Super product',
+            'is_storable': True,
+            'seller_ids': [Command.create({
+                'partner_id': self.partner_a.id,
+                'price': 100.0,
+            })]
+        })
+        stock_location = self.company_data['default_warehouse'].lot_stock_id
+        intermediate_location = self.env['stock.location'].create([{
+            'name': 'intermediate Location',
+            'usage': 'internal',
+            'location_id': stock_location.id,
+        }])
+        sub_location = self.env['stock.location'].create([{
+            'name': 'Sub Location',
+            'usage': 'internal',
+            'location_id': intermediate_location.id,
+        }])
+        super_receipt = self.env['stock.picking.type'].create({
+            'name': 'Super receipt',
+            'code': 'incoming',
+            'sequence_code': 'SR',
+            'default_location_src_id': self.env.ref("stock.stock_location_suppliers").id,
+            'default_location_dest_id': sub_location.id,
+        })
+        orderpoint = self.env['stock.warehouse.orderpoint'].create({
+            'product_id': product.id,
+            'qty_to_order': 1.0,
+            'location_id': stock_location.id
+        })
+        orderpoint.action_replenish()
+        po = self.env['purchase.order'].search([("product_id", "=", product.id)], limit=1)
+        po.picking_type_id = super_receipt
+        po.button_confirm()
+        picking = po.picking_ids
+        self.assertEqual(picking.location_dest_id, sub_location)
+        stock_move = picking.move_ids
+        self.assertEqual(stock_move.location_dest_id, sub_location)
+        stock_move.quantity = 1.0
+        picking.button_validate()
+        self.assertEqual(picking.move_ids.location_dest_id, sub_location)
