@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 
 from . import rating
+from . import rating_data
 from . import rating_mixin
+from . import rating_parent_mixin
 from . import mail_thread
 from . import mail_message
