@@ -16,26 +16,34 @@ The following topics are covered by this module:
 ------------------------------------------------------
     * Add/remove products in the reparation
     * Impact for stocks
-    * Invoicing (products and/or services)
     * Warranty concept
     * Repair quotation report
     * Notes for the technician and for the final customer
 """,
-    'depends': ['stock', 'sale_management', 'account'],
+    'depends': ['stock', 'sale_management'],
     'data': [
         'security/ir.model.access.csv',
         'security/repair_security.xml',
-        'wizard/repair_make_invoice_views.xml',
         'wizard/stock_warn_insufficient_qty_views.xml',
+        'views/product_views.xml',
+        'views/stock_move_views.xml',
         'views/repair_views.xml',
+        'views/sale_order_views.xml',
+        'views/stock_lot_views.xml',
+        'views/stock_picking_views.xml',
+        'views/stock_warehouse_views.xml',
         'report/repair_reports.xml',
         'report/repair_templates_repair_order.xml',
-        'data/ir_sequence_data.xml',
         'data/repair_data.xml',
     ],
     'demo': ['data/repair_demo.xml'],
-    'test': [],
+    'post_init_hook': '_create_warehouse_data',
     'installable': True,
-    'auto_install': False,
     'application': True,
+    'assets': {
+        'web.assets_backend': [
+            'repair/static/src/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
