@@ -2,9 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import repair
+from . import stock_move
+from . import stock_move_line
 from . import stock_picking
 from . import stock_traceability
 from . import stock_lot
-from . import account_move
 from . import product
-from . import mail_compose_message
+from . import sale_order
+from . import stock_warehouse
