@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import onboarding
+from . import combo_configurator
 from . import portal
-from . import variant
+from . import product_configurator
