@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import fields, models
+from odoo.tools import SQL
 
 
 class AccountInvoiceReport(models.Model):
@@ -9,5 +10,5 @@ class AccountInvoiceReport(models.Model):
 
     team_id = fields.Many2one(comodel_name='crm.team', string="Sales Team")
 
-    def _select(self):
-        return super()._select() + ", move.team_id as team_id"
+    def _select(self) -> SQL:
+        return SQL("%s, move.team_id as team_id", super()._select())
