@@ -4,9 +4,6 @@
 from . import models
 from . import wizard
 
-from odoo import api, SUPERUSER_ID
-
-def uninstall_hook(cr, registry):
-    env = api.Environment(cr, SUPERUSER_ID, {})
+def uninstall_hook(env):
     teams = env['crm.team'].search([('use_opportunities', '=', False)])
     teams.write({'use_opportunities': True})
\ No newline at end of file
