@@ -15,13 +15,21 @@ This module allow to reinvoice employee expense, by setting the SO directly on t
 """,
     'depends': ['sale_management', 'hr_expense'],
     'data': [
-        'views/assets.xml',
+        'data/sale_expense_data.xml',
         'views/product_view.xml',
         'views/hr_expense_views.xml',
         'views/sale_order_views.xml',
+        'views/hr_expense_sheet_views.xml',
     ],
-    'demo': ['data/sale_expense_demo.xml'],
-    'test': [],
     'installable': True,
     'auto_install': True,
+    'assets': {
+        'web.assets_backend': [
+            'sale_expense/static/src/**/*',
+        ],
+        'web.qunit_suite_tests': [
+            'sale_expense/static/tests/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
