@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import account_move_line
 from . import hr_expense
 from . import hr_expense_sheet
 from . import hr_expense_split
-from . import sale_order
 from . import product_template
-from . import account_move
+from . import sale_order
