@@ -7,7 +7,10 @@ from odoo import fields, models
 class LoyaltyCard(models.Model):
     _inherit = 'loyalty.card'
 
-    order_id = fields.Many2one('sale.order', 'Order Reference', readonly=True,
+    order_id = fields.Many2one(
+        comodel_name='sale.order',
+        string="Order Reference",
+        readonly=True,
         help="The sales order from which coupon is generated")
 
     def _get_default_template(self):
@@ -25,8 +28,8 @@ class LoyaltyCard(models.Model):
     def _compute_use_count(self):
         super()._compute_use_count()
         read_group_res = self.env['sale.order.line']._read_group(
-            [('coupon_id', 'in', self.ids)], ['id'], ['coupon_id'])
-        count_per_coupon = {r['coupon_id'][0]: r['coupon_id_count'] for r in read_group_res}
+            [('coupon_id', 'in', self.ids)], ['coupon_id'], ['__count'])
+        count_per_coupon = {coupon.id: count for coupon, count in read_group_res}
         for card in self:
             card.use_count += count_per_coupon.get(card.id, 0)
 
