@@ -3,12 +3,13 @@
 
 from odoo import fields, models
 
+
 class SaleOrderCouponPoints(models.Model):
     _name = 'sale.order.coupon.points'
     _description = 'Sale Order Coupon Points - Keeps track of how a sale order impacts a coupon'
 
-    order_id = fields.Many2one('sale.order', required=True, ondelete='cascade')
-    coupon_id = fields.Many2one('loyalty.card', required=True, ondelete='cascade')
+    order_id = fields.Many2one(comodel_name='sale.order', required=True, ondelete='cascade')
+    coupon_id = fields.Many2one(comodel_name='loyalty.card', required=True, ondelete='cascade')
     points = fields.Float(required=True)
 
     _sql_constraints = [
