@@ -14,14 +14,14 @@ class Digest(models.Model):
     def _compute_kpi_sale_total_value(self):
         if not self.env.user.has_group('sales_team.group_sale_salesman_all_leads'):
             raise AccessError(_("Do not have access, skip this data for user's digest email"))
-        for record in self:
-            start, end, company = record._get_kpi_compute_parameters()
-            all_channels_sales = self.env['sale.report'].read_group([
-                ('date', '>=', start),
-                ('date', '<', end),
-                ('state', 'not in', ['draft', 'cancel', 'sent']),
-                ('company_id', '=', company.id)], ['price_total'], ['price_total'])
-            record.kpi_all_sale_total_value = sum([channel_sale['price_total'] for channel_sale in all_channels_sales])
+
+        self._calculate_company_based_kpi(
+            'sale.report',
+            'kpi_all_sale_total_value',
+            date_field='date',
+            additional_domain=[('state', 'not in', ['draft', 'cancel', 'sent'])],
+            sum_field='price_total',
+        )
 
     def _compute_kpis_actions(self, company, user):
         res = super(Digest, self)._compute_kpis_actions(company, user)
