@@ -12,6 +12,11 @@ class MrpBom(models.Model):
         self.filtered(lambda bom: bom.active)._ensure_bom_is_free()
         return super().toggle_active()
 
+    def write(self, vals):
+        if 'phantom' in self.mapped('type') and vals.get('type', 'phantom') != 'phantom':
+            self._ensure_bom_is_free()
+        return super().write(vals)
+
     def unlink(self):
         self._ensure_bom_is_free()
         return super().unlink()
@@ -25,7 +30,7 @@ class MrpBom(models.Model):
         if not product_ids:
             return
         lines = self.env['sale.order.line'].sudo().search([
-            ('state', 'in', ('sale', 'done')),
+            ('state', '=', 'sale'),
             ('invoice_status', 'in', ('no', 'to invoice')),
             ('product_id', 'in', product_ids),
             ('move_ids.state', '!=', 'cancel'),
