@@ -15,6 +15,8 @@ class TestMultistepManufacturing(TestMrpCommon):
         cls.env.user.groups_id += cls.env.ref('uom.group_uom')
         # Required for `manufacture_steps` to be visible in the view
         cls.env.user.groups_id += cls.env.ref('stock.group_adv_location')
+        # Required for `product_id` to be visible in the view
+        cls.env.user.groups_id += cls.env.ref('product.group_product_variant')
 
         cls.env.ref('stock.route_warehouse0_mto').active = True
         cls.MrpProduction = cls.env['mrp.production']
@@ -23,6 +25,7 @@ class TestMultistepManufacturing(TestMrpCommon):
         warehouse_form.name = 'Test'
         warehouse_form.code = 'Test'
         cls.warehouse = warehouse_form.save()
+        cls.warehouse.mto_pull_id.route_id.rule_ids.procure_method = "make_to_order"
 
         cls.uom_unit = cls.env.ref('uom.product_uom_unit')
 
