@@ -3,12 +3,18 @@
 
 import time
 
-from odoo.tests.common import TransactionCase, Form
+from odoo.tests import Form, TransactionCase
 from odoo.tools import mute_logger
+from odoo import Command
 
 
 class TestSaleMrpProcurement(TransactionCase):
 
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.env.ref('base.group_user').write({'implied_ids': [(4, cls.env.ref('product.group_product_variant').id)]})
+
     def test_sale_mrp(self):
         # Required for `uom_id` to be visible in the view
         self.env.user.groups_id += self.env.ref('uom.group_uom')
@@ -34,7 +40,7 @@ class TestSaleMrpProcurement(TransactionCase):
         product.categ_id = product_category_allproductssellable0
         product.list_price = 200.0
         product.name = 'Slider Mobile'
-        product.detailed_type = 'product'
+        product.is_storable = True
         product.uom_id = uom_unit
         product.uom_po_id = uom_unit
         product.route_ids.clear()
@@ -96,7 +102,7 @@ class TestSaleMrpProcurement(TransactionCase):
         # Create raw product for manufactured product
         product_form = Form(self.env['product.product'])
         product_form.name = 'Raw Stick'
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
         self.raw_product = product_form.save()
@@ -106,7 +112,7 @@ class TestSaleMrpProcurement(TransactionCase):
         product_form.name = 'Stick'
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.route_ids.clear()
         product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
         product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
@@ -115,7 +121,7 @@ class TestSaleMrpProcurement(TransactionCase):
         # Create manifactured product which uses another manifactured
         product_form = Form(self.env['product.product'])
         product_form.name = 'Arrow'
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.route_ids.clear()
         product_form.route_ids.add(self.warehouse.manufacture_pull_id.route_id)
         product_form.route_ids.add(self.warehouse.mto_pull_id.route_id)
@@ -124,7 +130,7 @@ class TestSaleMrpProcurement(TransactionCase):
         ## Create raw product for manufactured product
         product_form = Form(self.env['product.product'])
         product_form.name = 'Raw Iron'
-        product_form.detailed_type = 'product'
+        product_form.is_storable = True
         product_form.uom_id = self.uom_unit
         product_form.uom_po_id = self.uom_unit
         self.raw_product_2 = product_form.save()
@@ -196,7 +202,7 @@ class TestSaleMrpProcurement(TransactionCase):
 
         product, component = self.env['product.product'].create([{
             'name': 'Finished',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': [(6, 0, manufacture_route.ids)],
         }, {
             'name': 'Component',
@@ -243,7 +249,7 @@ class TestSaleMrpProcurement(TransactionCase):
     def test_so_reordering_rule(self):
         kit_1, component_1 = self.env['product.product'].create([{
             'name': n,
-            'type': 'product',
+            'is_storable': True,
         } for n in ['Kit 1', 'Compo 1']])
 
         self.env['mrp.bom'].create([{
@@ -271,3 +277,114 @@ class TestSaleMrpProcurement(TransactionCase):
         orderpoint_product = self.env['stock.warehouse.orderpoint'].search(
             [('product_id', '=', kit_1.id)])
         self.assertFalse(orderpoint_product)
+
+    def test_so_reordering_rule_02(self):
+        """
+        Have a manufactured product in kg unit of measure with the manufacturing route
+        and a reordering rule (RR) set to min=max=0, and a BoM in grams.
+        Confirm a SO with that product in 510 grams -> It should generate a MO with 510g.
+        Create a second SO with 510g -> It should update the MO to 1020g.
+        """
+        warehouse = self.env['stock.warehouse'].search([('company_id', '=', self.env.company.id)], limit=1)
+        manufacture_route = warehouse.manufacture_pull_id.route_id
+
+        uom_kg = self.env.ref('uom.product_uom_kgm')
+        uom_gram = self.env.ref('uom.product_uom_gram')
+
+        product, component = self.env['product.product'].create([{
+            'name': 'Finished',
+            'is_storable': True,
+            'uom_id': uom_kg.id,
+            'uom_po_id': uom_kg.id,
+            'route_ids': [(6, 0, manufacture_route.ids)],
+        }, {
+            'name': 'Component',
+            'type': 'consu',
+        }])
+
+        self.env['mrp.bom'].create({
+            'product_id': product.id,
+            'product_tmpl_id': product.product_tmpl_id.id,
+            'product_uom_id': uom_gram.id,
+            'product_qty': 1.0,
+            'type': 'normal',
+            'bom_line_ids': [
+                (0, 0, {'product_id': component.id, 'product_qty': 1}),
+            ],
+        })
+
+        self.env['stock.warehouse.orderpoint'].create({
+            'name': product.name,
+            'location_id': warehouse.lot_stock_id.id,
+            'product_id': product.id,
+            'product_min_qty': 0,
+            'product_max_qty': 0,
+            'trigger': 'auto',
+            'qty_multiple': 0.01,
+        })
+
+        so = self.env['sale.order'].create({
+            'partner_id': self.env['res.partner'].create({'name': 'Super Partner'}).id,
+            'order_line': [
+                (0, 0, {
+                    'name': product.name,
+                    'product_id': product.id,
+                    'product_uom_qty': 510,
+                    'product_uom': uom_gram.id,
+                    'price_unit': 1,
+                })],
+        })
+        so.action_confirm()
+        self.assertEqual(so.state, 'sale')
+
+        mo = self.env['mrp.production'].search([('product_id', '=', product.id)], order='id desc', limit=1)
+        self.assertIn(so.name, mo.origin)
+        self.assertEqual(mo.product_uom_id, uom_gram)
+        self.assertEqual(mo.product_qty, 510)
+
+        so_2 = self.env['sale.order'].create({
+            'partner_id': self.env['res.partner'].create({'name': 'Super Partner'}).id,
+            'order_line': [
+                (0, 0, {
+                    'name': product.name,
+                    'product_id': product.id,
+                    'product_uom_qty': 510,
+                    'product_uom': uom_gram.id,
+                    'price_unit': 1,
+                })],
+        })
+        so_2.action_confirm()
+        self.assertEqual(so_2.state, 'sale')
+        self.assertEqual(mo.product_uom_id, uom_gram)
+        self.assertEqual(mo.product_qty, 1020)
+
+    def test_sale_mrp_avoid_multiple_pickings(self):
+        """
+        Test sale of multiple products. Avoid multiple pickings being
+        generated when we are not in 3 steps manufacturing.
+        """
+
+        warehouse = self.env['stock.warehouse'].search([('company_id', '=', self.env.company.id)], limit=1)
+        warehouse.sam_loc_id = warehouse.lot_stock_id
+
+        so = self.env['sale.order'].create({
+            'partner_id': self.env['res.partner'].create({'name': 'My Partner'}).id,
+            'order_line': [
+                Command.create({
+                    'name': 'sol_p1',
+                    'product_id': self.env['product.product'].create({'name': 'p1'}).id,
+                    'product_uom_qty': 1,
+                    'product_uom': self.env.ref('uom.product_uom_unit').id,
+                }),
+                Command.create({
+                    'name': 'sol_p2',
+                    'product_id': self.env['product.product'].create({'name': 'p2'}).id,
+                    'product_uom_qty': 1,
+                    'product_uom': self.env.ref('uom.product_uom_unit').id,
+                }),
+            ],
+        })
+
+        so.action_confirm()
+        self.assertEqual(len(so.picking_ids), 1)
+        self.assertEqual(so.picking_ids.picking_type_id, warehouse.out_type_id)
