@@ -16,12 +16,11 @@ by external providers and will automatically generate purchase orders directed t
         'purchase',
     ],
     'data': [
-        'data/mail_data.xml',
+        'data/mail_templates.xml',
         'views/product_views.xml',
         'views/sale_order_views.xml',
         'views/purchase_order_views.xml',
-    ],
-    'demo': [
+        'wizards/sale_order_cancel_views.xml',
     ],
     'auto_install': True,
     'license': 'LGPL-3',
