@@ -34,7 +34,12 @@ class StockMove(models.Model):
 
     def _get_source_document(self):
         res = super()._get_source_document()
-        return self.sale_line_id.order_id or res
+        return self.sudo().sale_line_id.order_id or res
+
+    def _get_sale_order_lines(self):
+        """ Return all possible sale order lines for one stock move. """
+        self.ensure_one()
+        return (self + self.browse(self._rollup_move_origs() | self._rollup_move_dests())).sale_line_id
 
     def _assign_picking_post_process(self, new=False):
         super(StockMove, self)._assign_picking_post_process(new=new)
@@ -42,10 +47,21 @@ class StockMove(models.Model):
             picking_id = self.mapped('picking_id')
             sale_order_ids = self.mapped('sale_line_id.order_id')
             for sale_order_id in sale_order_ids:
-                picking_id.message_post_with_view(
+                picking_id.message_post_with_source(
                     'mail.message_origin_link',
-                    values={'self': picking_id, 'origin': sale_order_id},
-                    subtype_id=self.env.ref('mail.mt_note').id)
+                    render_values={'self': picking_id, 'origin': sale_order_id},
+                    subtype_xmlid='mail.mt_note',
+                )
+
+    def _get_all_related_sm(self, product):
+        return super()._get_all_related_sm(product) | self.filtered(lambda m: m.sale_line_id.product_id == product)
+
+
+class StockMoveLine(models.Model):
+    _inherit = "stock.move.line"
+
+    def _should_show_lot_in_invoice(self):
+        return 'customer' in {self.location_id.usage, self.location_dest_id.usage}
 
 
 class ProcurementGroup(models.Model):
@@ -59,14 +75,34 @@ class StockRule(models.Model):
 
     def _get_custom_move_fields(self):
         fields = super(StockRule, self)._get_custom_move_fields()
-        fields += ['sale_line_id', 'partner_id', 'sequence']
+        fields += ['sale_line_id', 'partner_id', 'sequence', 'to_refund']
         return fields
 
 
 class StockPicking(models.Model):
     _inherit = 'stock.picking'
 
-    sale_id = fields.Many2one(related="group_id.sale_id", string="Sales Order", store=True, readonly=False, index='btree_not_null')
+    sale_id = fields.Many2one('sale.order', compute="_compute_sale_id", inverse="_set_sale_id", string="Sales Order", store=True, index='btree_not_null')
+
+    @api.depends('group_id')
+    def _compute_sale_id(self):
+        for picking in self:
+            picking.sale_id = picking.group_id.sale_id
+
+    def _set_sale_id(self):
+        if self.group_id:
+            self.group_id.sale_id = self.sale_id
+        else:
+            if self.sale_id:
+                vals = {
+                    'sale_id': self.sale_id.id,
+                    'name': self.sale_id.name,
+                }
+            else:
+                vals = {}
+
+            pg = self.env['procurement.group'].create(vals)
+            self.group_id = pg
 
     def _auto_init(self):
         """
@@ -87,16 +123,23 @@ class StockPicking(models.Model):
             sale_order = move.picking_id.sale_id
             # Creates new SO line only when pickings linked to a sale order and
             # for moves with qty. done and not already linked to a SO line.
-            if not sale_order or move.location_dest_id.usage != 'customer' or move.sale_line_id or not move.quantity_done:
+            if not sale_order \
+                or (move.location_dest_id.usage != 'customer' and not (move.location_id.usage == 'customer' and move.to_refund)) \
+                or move.sale_line_id \
+                or not move.picked:
                 continue
             product = move.product_id
+            quantity = move.quantity
+            if move.to_refund:
+                quantity *= -1
+
             so_line_vals = {
                 'move_ids': [(4, move.id, 0)],
                 'name': product.display_name,
                 'order_id': sale_order.id,
                 'product_id': product.id,
                 'product_uom_qty': 0,
-                'qty_delivered': move.quantity_done,
+                'qty_delivered': quantity,
                 'product_uom': move.product_uom.id,
             }
             if product.invoice_policy == 'delivery':
@@ -147,11 +190,16 @@ class StockPicking(models.Model):
             }
             return self.env['ir.qweb']._render('sale_stock.exception_on_picking', values)
 
-        documents = self._log_activity_get_documents(moves, 'sale_line_id', 'DOWN', _keys_in_groupby)
+        documents = self.sudo()._log_activity_get_documents(moves, 'sale_line_id', 'DOWN', _keys_in_groupby)
         self._log_activity(_render_note_exception_quantity, documents)
 
         return super(StockPicking, self)._log_less_quantities_than_expected(moves)
 
+    def _can_return(self):
+        self.ensure_one()
+        return super()._can_return() or self.sale_id
+
+
 class StockLot(models.Model):
     _inherit = 'stock.lot'
 
@@ -163,7 +211,7 @@ class StockLot(models.Model):
         sale_orders = defaultdict(lambda: self.env['sale.order'])
         for move_line in self.env['stock.move.line'].search([('lot_id', 'in', self.ids), ('state', '=', 'done')]):
             move = move_line.move_id
-            if move.picking_id.location_dest_id.usage == 'customer' and move.sale_line_id.order_id:
+            if move.picking_id.location_dest_id.usage in ('customer', 'transit') and move.sale_line_id.order_id:
                 sale_orders[move_line.lot_id.id] |= move.sale_line_id.order_id
         for lot in self:
             lot.sale_order_ids = sale_orders[lot.id]
