@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import project_create_sale_order
 from . import project_create_invoice
 from . import sale_make_invoice_advance
-from . import project_task_create_sale_order
