@@ -2,11 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from ast import literal_eval
+from uuid import uuid4
 
 from odoo import api, fields, models, _
-from odoo.addons.phone_validation.tools import phone_validation
+from odoo.addons.sms.tools.sms_tools import sms_content_to_rendered_html
 from odoo.exceptions import UserError
-from odoo.tools import html2plaintext
 
 
 class SendSMS(models.TransientModel):
@@ -51,10 +51,10 @@ class SendSMS(models.TransientModel):
     # recipients
     recipient_valid_count = fields.Integer('# Valid recipients', compute='_compute_recipients', compute_sudo=False)
     recipient_invalid_count = fields.Integer('# Invalid recipients', compute='_compute_recipients', compute_sudo=False)
-    recipient_single_description = fields.Text('Recipients (Partners)', compute='_compute_recipient_single', compute_sudo=False)
-    recipient_single_number = fields.Char('Stored Recipient Number', compute='_compute_recipient_single', compute_sudo=False)
+    recipient_single_description = fields.Text('Recipients (Partners)', compute='_compute_recipient_single_non_stored', compute_sudo=False)
+    recipient_single_number = fields.Char('Stored Recipient Number', compute='_compute_recipient_single_non_stored', compute_sudo=False)
     recipient_single_number_itf = fields.Char(
-        'Recipient Number', compute='_compute_recipient_single',
+        'Recipient Number', compute='_compute_recipient_single_stored',
         readonly=False, compute_sudo=False, store=True,
         help='Phone number of the recipient. If changed, it will be recorded on recipient\'s profile.')
     recipient_single_valid = fields.Boolean("Is valid", compute='_compute_recipient_single_valid', compute_sudo=False)
@@ -103,7 +103,7 @@ class SendSMS(models.TransientModel):
                 continue
 
             records = composer._get_records()
-            if records and issubclass(type(records), self.pool['mail.thread']):
+            if records and isinstance(records, self.pool['mail.thread']):
                 res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=not composer.comment_single_recipient)
                 composer.recipient_valid_count = len([rid for rid, rvalues in res.items() if rvalues['sanitized']])
                 composer.recipient_invalid_count = len([rid for rid, rvalues in res.items() if not rvalues['sanitized']])
@@ -113,31 +113,39 @@ class SendSMS(models.TransientModel):
                 ) else 1
 
     @api.depends('res_model', 'number_field_name')
-    def _compute_recipient_single(self):
+    def _compute_recipient_single_stored(self):
         for composer in self:
             records = composer._get_records()
-            if not records or not issubclass(type(records), self.pool['mail.thread']) or not composer.comment_single_recipient:
-                composer.recipient_single_description = False
-                composer.recipient_single_number = ''
+            if not records or not isinstance(records, self.pool['mail.thread']) or not composer.comment_single_recipient:
                 composer.recipient_single_number_itf = ''
                 continue
             records.ensure_one()
             res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=False)
-            composer.recipient_single_description = res[records.id]['partner'].name or records._sms_get_default_partners().display_name
-            composer.recipient_single_number = res[records.id]['number'] or ''
             if not composer.recipient_single_number_itf:
                 composer.recipient_single_number_itf = res[records.id]['number'] or ''
             if not composer.number_field_name:
                 composer.number_field_name = res[records.id]['field_store']
 
+    @api.depends('res_model', 'number_field_name')
+    def _compute_recipient_single_non_stored(self):
+        for composer in self:
+            records = composer._get_records()
+            if not records or not isinstance(records, self.pool['mail.thread']) or not composer.comment_single_recipient:
+                composer.recipient_single_description = False
+                composer.recipient_single_number = ''
+                continue
+            records.ensure_one()
+            res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=True)
+            composer.recipient_single_description = res[records.id]['partner'].name or records._mail_get_partners()[records[0].id].display_name
+            composer.recipient_single_number = res[records.id]['number'] or ''
+
     @api.depends('recipient_single_number', 'recipient_single_number_itf')
     def _compute_recipient_single_valid(self):
         for composer in self:
             value = composer.recipient_single_number_itf or composer.recipient_single_number
             if value:
                 records = composer._get_records()
-                sanitized = phone_validation.phone_sanitize_numbers_w_record([value], records)[value]['sanitized']
-                composer.recipient_single_valid = bool(sanitized)
+                composer.recipient_single_valid = bool(records._phone_format(number=value)) if len(records) == 1 else False
             else:
                 composer.recipient_single_valid = False
 
@@ -147,9 +155,8 @@ class SendSMS(models.TransientModel):
             if composer.numbers:
                 record = composer._get_records() if composer.res_model and composer.res_id else self.env.user
                 numbers = [number.strip() for number in composer.numbers.split(',')]
-                sanitize_res = phone_validation.phone_sanitize_numbers_w_record(numbers, record)
-                sanitized_numbers = [info['sanitized'] for info in sanitize_res.values() if info['sanitized']]
-                invalid_numbers = [number for number, info in sanitize_res.items() if info['code']]
+                sanitized_numbers = [record._phone_format(number=number) for number in numbers]
+                invalid_numbers = [number for sanitized, number in zip(sanitized_numbers, numbers) if not sanitized]
                 if invalid_numbers:
                     raise UserError(_('Following numbers are not correctly encoded: %s', repr(invalid_numbers)))
                 composer.sanitized_numbers = ','.join(sanitized_numbers)
@@ -187,7 +194,7 @@ class SendSMS(models.TransientModel):
         if self.composition_mode == 'numbers':
             return self._action_send_sms_numbers()
         elif self.composition_mode == 'comment':
-            if records is None or not issubclass(type(records), self.pool['mail.thread']):
+            if records is None or not isinstance(records, self.pool['mail.thread']):
                 return self._action_send_sms_numbers()
             if self.comment_single_recipient:
                 return self._action_send_sms_comment_single(records)
@@ -197,11 +204,8 @@ class SendSMS(models.TransientModel):
             return self._action_send_sms_mass(records)
 
     def _action_send_sms_numbers(self):
-        self.env['sms.api']._send_sms_batch([{
-            'res_id': 0,
-            'number': number,
-            'content': self.body,
-        } for number in self.sanitized_numbers.split(',')])
+        sms_values = [{'body': self.body, 'number': number} for number in self.sanitized_numbers.split(',')]
+        self.env['sms.sms'].sudo().create(sms_values).send()
         return True
 
     def _action_send_sms_comment_single(self, records=None):
@@ -210,7 +214,7 @@ class SendSMS(models.TransientModel):
         # on the numbers in the database.
         records = records if records is not None else self._get_records()
         records.ensure_one()
-        if not self.number_field_name:
+        if not self.number_field_name or self.number_field_name not in records:
             self.numbers = self.recipient_single_number_itf or self.recipient_single_number
         elif self.recipient_single_number_itf and self.recipient_single_number_itf != self.recipient_single_number:
             records.write({self.number_field_name: self.recipient_single_number_itf})
@@ -236,8 +240,7 @@ class SendSMS(models.TransientModel):
 
         sms_record_values = self._prepare_mass_sms_values(records)
         sms_all = self._prepare_mass_sms(records, sms_record_values)
-
-        if sms_all and self.mass_keep_log and records and issubclass(type(records), self.pool['mail.thread']):
+        if sms_all and self.mass_keep_log and records and isinstance(records, self.pool['mail.thread']):
             log_values = self._prepare_mass_log_values(records, sms_record_values)
             records._message_log_batch(**log_values)
 
@@ -315,10 +318,11 @@ class SendSMS(models.TransientModel):
 
             result[record.id] = {
                 'body': all_bodies[record.id],
-                'partner_id': recipients['partner'].id,
-                'number': sanitized if sanitized else recipients['number'],
-                'state': state,
                 'failure_type': failure_type,
+                'number': sanitized if sanitized else recipients['number'],
+                'partner_id': recipients['partner'].id,
+                'state': state,
+                'uuid': uuid4().hex,
             }
         return result
 
@@ -329,7 +333,7 @@ class SendSMS(models.TransientModel):
     def _prepare_log_body_values(self, sms_records_values):
         result = {}
         for record_id, sms_values in sms_records_values.items():
-            result[record_id] = html2plaintext(sms_values['body'])
+            result[record_id] = sms_content_to_rendered_html(sms_values['body'])
         return result
 
     def _prepare_mass_log_values(self, records, sms_records_values):
