@@ -8,7 +8,6 @@
 The purpose of this technical module is to provide a front for
 social media configuration for any other module that might need it.
     """,
-    'author': "Odoo S.A.",
     'category': 'Hidden',
     'version': '0.1',
     'depends': ['base'],
@@ -18,5 +17,6 @@ social media configuration for any other module that might need it.
     ],
     'demo': [
         'demo/res_company_demo.xml',
-    ]
+    ],
+    'license': 'LGPL-3',
 }
