@@ -2,20 +2,24 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import barcode
+from . import ir_actions_report
 from . import product_strategy
 from . import res_company
 from . import res_partner
+from . import res_users
 from . import res_config_settings
-from . import stock_inventory
 from . import stock_location
 from . import stock_move
 from . import stock_move_line
 from . import stock_orderpoint
-from . import stock_production_lot
+from . import stock_lot
 from . import stock_picking
 from . import stock_quant
+from . import stock_replenish_mixin
 from . import stock_rule
 from . import stock_warehouse
 from . import stock_scrap
 from . import product
 from . import stock_package_level
+from . import stock_package_type
+from . import stock_storage_category
