@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, models, _
-from odoo.tools import config
 from odoo.tools import format_datetime
 from markupsafe import Markup
 
@@ -35,7 +34,7 @@ class MrpStockReport(models.TransientModel):
                     lambda m: m.lot_id == move_line.lot_id and m.state == 'done'
                 ) - lines_seen
             # if MTS
-            elif move_line.location_id.usage == 'internal':
+            elif move_line.location_id.usage in ('internal', 'transit'):
                 lines = self.env['stock.move.line'].search([
                     ('product_id', '=', move_line.product_id.id),
                     ('lot_id', '=', move_line.lot_id.id),
@@ -52,7 +51,7 @@ class MrpStockReport(models.TransientModel):
         return lines_seen - move_lines
 
     @api.model
-    def get_lines(self, line_id=None, **kw):
+    def get_lines(self, line_id=False, **kw):
         context = dict(self.env.context)
         model = kw and kw['model_name'] or context.get('model')
         rec_id = kw and kw['model_id'] or context.get('active_id')
@@ -94,10 +93,10 @@ class MrpStockReport(models.TransientModel):
             res_model = 'stock.move'
             res_id = move_line.move_id.id
             ref = 'Inventory Adjustment'
-        elif move_line.move_id.scrapped and move_line.move_id.scrap_ids:
+        elif move_line.move_id.scrapped and move_line.move_id.scrap_id:
             res_model = 'stock.scrap'
-            res_id = move_line.move_id.scrap_ids[0].id
-            ref = move_line.move_id.scrap_ids[0].name
+            res_id = move_line.move_id.scrap_id.id
+            ref = move_line.move_id.scrap_id.name
         return res_model, res_id, ref
 
     @api.model
@@ -129,7 +128,7 @@ class MrpStockReport(models.TransientModel):
             'model_id': move_line.id,
             'model': 'stock.move.line',
             'product_id': move_line.product_id.display_name,
-            'product_qty_uom': "%s %s" % (self._quantity_to_str(move_line.product_uom_id, move_line.product_id.uom_id, move_line.qty_done), move_line.product_id.uom_id.name),
+            'product_qty_uom': "%s %s" % (self._quantity_to_str(move_line.product_uom_id, move_line.product_id.uom_id, move_line.quantity), move_line.product_id.uom_id.name),
             'lot_name': move_line.lot_id.name,
             'lot_id': move_line.lot_id.id,
             'location_source': move_line.location_id.name,
@@ -172,7 +171,7 @@ class MrpStockReport(models.TransientModel):
         return False, False
 
     @api.model
-    def _lines(self, line_id=None, model_id=False, model=False, level=0, move_lines=[], **kw):
+    def _lines(self, line_id=False, model_id=False, model=False, level=0, move_lines=None, **kw):
         final_vals = []
         lines = move_lines or []
         if model and line_id:
@@ -211,8 +210,6 @@ class MrpStockReport(models.TransientModel):
         }
 
         context = dict(self.env.context)
-        if not config['test_enable']:
-            context['commit_assetsbundle'] = True
         if context.get('active_id') and context.get('active_model'):
             rcontext['reference'] = self.env[context.get('active_model')].browse(int(context.get('active_id'))).display_name
 
@@ -231,17 +228,13 @@ class MrpStockReport(models.TransientModel):
             specific_paperformat_args={'data-report-margin-top': 17, 'data-report-header-spacing': 12}
         )
 
-    def _get_html(self):
-        result = {}
-        rcontext = {}
+    def _get_main_lines(self):
         context = dict(self.env.context)
-        rcontext['lines'] = self.with_context(context).get_lines()
-        result['html'] = self.env['ir.qweb']._render('stock.report_stock_inventory', rcontext)
-        return result
+        return self.with_context(context).get_lines()
 
     @api.model
-    def get_html(self, given_context=None):
+    def get_main_lines(self, given_context=None):
         res = self.search([('create_uid', '=', self.env.uid)], limit=1)
         if not res:
-            return self.create({}).with_context(given_context)._get_html()
-        return res.with_context(given_context)._get_html()
+            return self.create({}).with_context(given_context)._get_main_lines()
+        return res.with_context(given_context)._get_main_lines()
