@@ -1,12 +1,23 @@
 /** @odoo-module */
 
 import { ListController } from '@web/views/list/list_controller';
+import { Dropdown } from "@web/core/dropdown/dropdown";
+import { DropdownItem } from "@web/core/dropdown/dropdown_item";
 
 export class StockOrderpointListController extends ListController {
-    async onClickOrder() {
+    static template = "stock.StockOrderpoint.listView";
+
+    static components = {
+        ...super.components,
+        Dropdown,
+        DropdownItem,
+    }
+
+    async onClickOrder(force_to_max) {
         const resIds = await this.getSelectedResIds();
         const action = await this.model.orm.call(this.props.resModel, 'action_replenish', [resIds], {
             context: this.props.context,
+            force_to_max: force_to_max,
         });
         if (action) {
             await this.actionService.doAction(action);
