@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.exceptions import UserError
-from odoo.tests.common import TransactionCase, Form
+from odoo.exceptions import UserError, ValidationError
+from odoo.tests import Form, TransactionCase
 
 
 class TestMultiCompany(TransactionCase):
@@ -108,13 +108,24 @@ class TestMultiCompany(TransactionCase):
         with self.assertRaises(UserError):
             shared_partner.with_user(self.user_b).property_stock_customer = self.stock_location_a
 
+    def test_partner_2(self):
+        """On the partners of companies A and B:
+        - As a user of Company A, the customer/vendor location of Company B should be the inter-company location
+        - As a user of Company B, the customer/vendor location of Company A should be the inter-company location
+        """
+        inter_company_loc = self.env.ref('stock.stock_location_inter_company')
+        self.assertEqual(self.company_a.partner_id.with_user(self.user_b).property_stock_customer, inter_company_loc)
+        self.assertEqual(self.company_a.partner_id.with_user(self.user_b).property_stock_supplier, inter_company_loc)
+        self.assertEqual(self.company_b.partner_id.with_user(self.user_a).property_stock_customer, inter_company_loc)
+        self.assertEqual(self.company_b.partner_id.with_user(self.user_a).property_stock_supplier, inter_company_loc)
+
     def test_inventory_1(self):
         """Create a quant (inventory adjustment) in Company A for a product limited to Company A and
         as a user of company B, apply the inventory adjustment and set its counted quantity to 10
         before validating. The quant and stock moves should belong to Company A.
         """
         product = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'company_id': self.company_a.id,
             'name': 'Product limited to company A',
         })
@@ -129,7 +140,7 @@ class TestMultiCompany(TransactionCase):
         last_move_id = self.env['stock.move'].search([('is_inventory', '=', True)])[-1]
         self.assertEqual(inventory_quant.company_id, self.company_a)
         self.assertEqual(last_move_id.company_id, self.company_a)
-        self.assertEqual(last_move_id.quantity_done, 10)
+        self.assertEqual(last_move_id.quantity, 10)
         self.assertEqual(last_move_id.location_id.company_id, self.company_a)
 
     def test_inventory_2(self):
@@ -139,7 +150,7 @@ class TestMultiCompany(TransactionCase):
         product = self.env['product.product'].create({
             'name': 'product limited to company b',
             'company_id': self.company_b.id,
-            'type': 'product'
+            'is_storable': True
         })
 
         with self.assertRaises(UserError):
@@ -164,38 +175,21 @@ class TestMultiCompany(TransactionCase):
         with self.assertRaises(UserError):
             self.stock_location_b.location_id = self.stock_location_a
 
-    def test_lot_1(self):
-        """Check it is possible to create a stock.production.lot with the same name in Company A and
-        Company B"""
-        product_lot = self.env['product.product'].create({
-            'type': 'product',
-            'tracking': 'lot',
-            'name': 'product lot',
-        })
-        self.env['stock.lot'].create({
-            'name': 'lotA',
-            'company_id': self.company_a.id,
-            'product_id': product_lot.id,
-        })
-        self.env['stock.lot'].create({
-            'name': 'lotA',
-            'company_id': self.company_b.id,
-            'product_id': product_lot.id,
-        })
-
     def test_lot_2(self):
         """Validate a picking of Company A receiving lot1 while being logged into Company B. Check
-        the lot is created in Company A.
+        the lot is created in Company A since the product belongs to Company A.
         """
         product = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'tracking': 'serial',
             'name': 'product',
+            'company_id': self.company_a.id,
         })
         picking = self.env['stock.picking'].with_user(self.user_a).create({
             'picking_type_id': self.warehouse_a.in_type_id.id,
             'location_id': self.env.ref('stock.stock_location_suppliers').id,
             'location_dest_id': self.stock_location_a.id,
+            'state': 'draft',
         })
         self.assertEqual(picking.company_id, self.company_a)
         move1 = self.env['stock.move'].create({
@@ -211,9 +205,10 @@ class TestMultiCompany(TransactionCase):
         })
         picking.with_user(self.user_b).action_confirm()
         self.assertEqual(picking.state, 'assigned')
-        move1.with_user(self.user_b).move_line_ids[0].qty_done = 1
+        move1.with_user(self.user_b).move_line_ids[0].quantity = 1
         move1.with_user(self.user_b).move_line_ids[0].lot_name = 'receipt_serial'
         self.assertEqual(move1.move_line_ids[0].company_id, self.company_a)
+        picking.with_user(self.user_b).move_ids.picked = True
         picking.with_user(self.user_b).button_validate()
         self.assertEqual(picking.state, 'done')
         created_serial = self.env['stock.lot'].search([
@@ -221,13 +216,44 @@ class TestMultiCompany(TransactionCase):
         ])
         self.assertEqual(created_serial.company_id, self.company_a)
 
+    def test_lot_3(self):
+        """ Checks that with a lot created in company A, it's not possible to create the same lot without
+            a company from company B, while it's possible to create it with company B set as its company.
+        """
+        product = self.env['product.product'].create({
+            'type': 'consu',
+            'is_storable': True,
+            'tracking': 'serial',
+            'name': 'Cross-Company Product',
+        })
+        lot = self.env['stock.lot'].create({
+            'name': 'unique',
+            'product_id': product.id,
+            'company_id': self.company_a.id,
+        })
+        self.assertTrue(lot)
+        # Even without having access to it, it shouldn't be possible to duplicate the lot between a company & no-company.
+        with self.assertRaises(ValidationError):
+            self.env['stock.lot'].with_user(self.user_b).with_context(allowed_company_ids=self.company_b.ids).create({
+                'name': 'unique',
+                'product_id': product.id,
+                'company_id': False,
+            })
+        # But it should be possible to create it in another company.
+        lot_b = self.env['stock.lot'].with_user(self.user_b).create({
+            'name': 'unique',
+            'product_id': product.id,
+            'company_id': self.company_b.id,
+        })
+        self.assertTrue(lot_b)
+
     def test_orderpoint_1(self):
         """As a user of company A, create an orderpoint for company B. Check itsn't possible to
         use a warehouse of companny A"""
         # Required for `warehouse_id` and `location_id` to be visible in the view
         self.user_a.groups_id += self.env.ref("stock.group_stock_multi_locations")
         product = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'name': 'shared product',
         })
         orderpoint = Form(self.env['stock.warehouse.orderpoint'].with_user(self.user_a))
@@ -248,7 +274,7 @@ class TestMultiCompany(TransactionCase):
         # Required for `warehouse_id` and `location_id` to be visible in the view
         self.user_a.groups_id += self.env.ref("stock.group_stock_multi_locations")
         product = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'name': 'shared product',
         })
         orderpoint = Form(self.env['stock.warehouse.orderpoint'].with_user(self.user_a))
@@ -267,7 +293,7 @@ class TestMultiCompany(TransactionCase):
         # to test the change of location when changing of warehouse within a same company
         warehouse_a2 = self.env['stock.warehouse'].with_user(self.user_a).sudo().create({'name': 'foo', 'code': 'foo'})
         product = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'name': 'shared product',
         })
         orderpoint = self.env['stock.warehouse.orderpoint'].with_user(self.user_a).create({
@@ -342,7 +368,7 @@ class TestMultiCompany(TransactionCase):
         """
         product = self.env['product.product'].create({
             'name': 'p1',
-            'type': 'product'
+            'is_storable': True
         })
         picking_type_b = self.env['stock.picking.type'].search([
             ('company_id', '=', self.company_b.id),
@@ -365,7 +391,7 @@ class TestMultiCompany(TransactionCase):
         """
         product = self.env['product.product'].create({
             'name': 'p1',
-            'type': 'product'
+            'is_storable': True
         })
         picking_type_b = self.env['stock.picking.type'].search([
             ('company_id', '=', self.company_b.id),
@@ -388,7 +414,7 @@ class TestMultiCompany(TransactionCase):
         """
         product = self.env['product.product'].create({
             'name': 'p1',
-            'type': 'product',
+            'is_storable': True,
             'company_id': self.company_b.id,
         })
         picking_type_b = self.env['stock.picking.type'].search([
@@ -413,11 +439,12 @@ class TestMultiCompany(TransactionCase):
         with previous move, and no product are reserved from inter-company
         transit. """
         supplier_location = self.env.ref('stock.stock_location_suppliers')
-        intercom_location = self.env.ref('stock.stock_location_inter_wh')
+        intercom_location = self.env.ref('stock.stock_location_inter_company')
         intercom_location.write({'active': True})
 
+        self.user_a.company_ids = [(6, 0, [self.company_a.id])]
         product_lot = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'tracking': 'lot',
             'name': 'product lot',
         })
@@ -451,22 +478,23 @@ class TestMultiCompany(TransactionCase):
             })],
         })
 
-        move_from_supplier = self.env['stock.move'].create({
+        move_from_supplier = self.env['stock.move'].with_user(self.user_a).create({
             'company_id': self.company_a.id,
             'name': 'test_from_supplier',
             'location_id': supplier_location.id,
             'location_dest_id': self.stock_location_a.id,
             'product_id': product_lot.id,
             'product_uom': product_lot.uom_id.id,
-            'product_uom_qty': 1.0,
+            'product_uom_qty': 0.1,
             'picking_type_id': self.warehouse_a.in_type_id.id,
         })
         move_from_supplier._action_confirm()
         move_line_1 = move_from_supplier.move_line_ids[0]
         move_line_1.lot_name = 'lot 1'
-        move_line_1.qty_done = 1.0
+        move_line_1.quantity = 0.1
+        move_from_supplier.picked = True
         move_from_supplier._action_done()
-        lot_1 = move_line_1.lot_id
+        lot = move_line_1.lot_id
 
         move_to_transit = self.env['stock.move'].create({
             'company_id': self.company_a.id,
@@ -475,58 +503,52 @@ class TestMultiCompany(TransactionCase):
             'location_dest_id': intercom_location.id,
             'product_id': product_lot.id,
             'product_uom': product_lot.uom_id.id,
-            'product_uom_qty': 1.0,
+            'product_uom_qty': 0.1,
             'picking_type_id': picking_type_to_transit.id,
             'route_ids': [(4, route.id)],
         })
-        move_to_transit._action_confirm()
-        move_to_transit._action_assign()
+        move_to_transit.with_user(self.user_a)._action_confirm()
+        move_to_transit.with_user(self.user_a)._action_assign()
         move_line_2 = move_to_transit.move_line_ids[0]
         self.assertTrue(move_line_2.lot_id, move_line_1.lot_id)
-        move_line_2.qty_done = 1.0
-        move_to_transit._action_done()
+        move_line_2.quantity = 0.1
+        move_to_transit.picked = True
+        move_to_transit.with_user(self.user_a)._action_done()
 
         move_push = self.env['stock.move'].search([('location_id', '=', intercom_location.id),
                                                    ('product_id', '=', product_lot.id)])
         self.assertTrue(move_push, 'No move created from push rules')
         self.assertEqual(move_push.state, "assigned")
         self.assertTrue(move_push.move_line_ids, "No move line created for the move")
-        self.assertFalse(move_push in move_to_transit.move_dest_ids,
-                         "Chained move created in transit location")
-        self.assertNotEqual(move_push.move_line_ids.lot_id, move_line_2.lot_id,
-                            "Reserved from transit location")
+        self.assertTrue(move_push in move_to_transit.move_dest_ids,
+                         "Moves are not chained")
+        self.assertEqual(move_push.move_line_ids.lot_id, move_line_2.lot_id,
+                            "Should be reserved from transit location")
         picking_receipt = move_push.picking_id
-        with self.assertRaises(UserError):
-            picking_receipt.button_validate()
-
         move_line_3 = move_push.move_line_ids[0]
-        move_line_3.lot_name = 'lot 2'
-        move_line_3.qty_done = 1.0
+        picking_receipt.move_ids.picked = True
         picking_receipt.button_validate()
-        lot_2 = move_line_3.lot_id
-        self.assertEqual(lot_1.company_id, self.company_a)
-        self.assertEqual(lot_1.name, 'lot 1')
-        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, intercom_location, lot_1), 1.0)
-        self.assertEqual(lot_2.company_id, self.company_b)
-        self.assertEqual(lot_2.name, 'lot 2')
-        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, self.stock_location_b, lot_2), 1.0)
+        self.assertEqual(move_line_3.lot_id, lot)
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, intercom_location, lot), 0)
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, self.stock_location_b, lot), 0.1)
 
     def test_intercom_lot_pull(self):
-        """Use warehouse of comany a to resupply warehouse of company b. Check
-        pull rule works correctly in two companies and moves are unchained from
-        inter-company transit location."""
+        """Use warehouse of company a to resupply warehouse of company b. Check
+        pull rule works correctly in two companies and moves are chained all the way through."""
         customer_location = self.env.ref('stock.stock_location_customers')
         supplier_location = self.env.ref('stock.stock_location_suppliers')
-        intercom_location = self.env.ref('stock.stock_location_inter_wh')
+        intercom_location = self.env.ref('stock.stock_location_inter_company')
         intercom_location.write({'active': True})
         partner = self.env['res.partner'].create({'name': 'Deco Addict'})
         self.warehouse_a.resupply_wh_ids = [(6, 0, [self.warehouse_b.id])]
-        resupply_route = self.env['stock.route'].search([('supplier_wh_id', '=', self.warehouse_b.id),
-                                                                  ('supplied_wh_id', '=', self.warehouse_a.id)])
+        resupply_route = self.env['stock.route'].search([
+            ('supplier_wh_id', '=', self.warehouse_b.id),
+            ('supplied_wh_id', '=', self.warehouse_a.id)
+        ])
         self.assertTrue(resupply_route, "Resupply route not found")
 
         product_lot = self.env['product.product'].create({
-            'type': 'product',
+            'is_storable': True,
             'tracking': 'lot',
             'name': 'product lot',
             'route_ids': [(4, resupply_route.id), (4, self.env.ref('stock.route_warehouse0_mto').id)],
@@ -544,10 +566,11 @@ class TestMultiCompany(TransactionCase):
         })
         move_sup_to_whb._action_confirm()
         move_line_1 = move_sup_to_whb.move_line_ids[0]
-        move_line_1.lot_name = 'lot b'
-        move_line_1.qty_done = 1.0
+        move_line_1.lot_name = 'lot a'
+        move_line_1.quantity = 1.0
+        move_sup_to_whb.picked = True
         move_sup_to_whb._action_done()
-        lot_b = move_line_1.lot_id
+        lot_a = move_line_1.lot_id
 
         picking_out = self.env['stock.picking'].create({
             'company_id': self.company_a.id,
@@ -555,6 +578,7 @@ class TestMultiCompany(TransactionCase):
             'picking_type_id': self.warehouse_a.out_type_id.id,
             'location_id': self.stock_location_a.id,
             'location_dest_id': customer_location.id,
+            'state': 'draft',
         })
         move_wha_to_cus = self.env['stock.move'].create({
             'name': "WH_A to Customer",
@@ -578,40 +602,58 @@ class TestMultiCompany(TransactionCase):
         self.assertTrue(move_transit_to_wha, "No move created by pull rule")
         self.assertTrue(move_wha_to_cus in move_transit_to_wha.move_dest_ids,
                         "Moves are not chained")
-        self.assertFalse(move_transit_to_wha in move_whb_to_transit.move_dest_ids,
-                         "Chained move created in transit location")
+        self.assertTrue(move_transit_to_wha in move_whb_to_transit.move_dest_ids,
+                         "Moves are not chained")
         self.assertEqual(move_wha_to_cus.state, "waiting")
         self.assertEqual(move_transit_to_wha.state, "waiting")
         self.assertEqual(move_whb_to_transit.state, "assigned")
 
         (move_wha_to_cus + move_whb_to_transit + move_transit_to_wha).picking_id.action_assign()
         self.assertEqual(move_wha_to_cus.state, "waiting")
-        self.assertEqual(move_transit_to_wha.state, "assigned")
+        self.assertEqual(move_transit_to_wha.state, "waiting")
         self.assertEqual(move_whb_to_transit.state, "assigned")
-
-        res_dict = move_whb_to_transit.picking_id.button_validate()
-        self.assertEqual(res_dict.get('res_model'), 'stock.immediate.transfer')
-        wizard = Form(self.env[res_dict['res_model']].with_context(res_dict['context'])).save()
-        wizard.process()
-        self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, intercom_location, lot_b), 1.0)
-        with self.assertRaises(UserError):
-            move_transit_to_wha.picking_id.button_validate()
+        move_whb_to_transit.picking_id.button_validate()
+        intercom_quant = self.env['stock.quant'].search([('lot_id', '=', lot_a.id), ('product_id', '=', product_lot.id), ('location_id', '=', intercom_location.id)])
+        self.assertRecordValues(intercom_quant, [{'quantity': 1, 'reserved_quantity': 1}])
 
         move_line_2 = move_transit_to_wha.move_line_ids[0]
-        move_line_2.lot_name = 'lot a'
-        move_line_2.qty_done = 1.0
+        self.assertEqual(move_line_2.lot_id, lot_a)
+        move_line_2.quantity = 1.0
+        move_transit_to_wha.picked = True
         move_transit_to_wha._action_done()
-        lot_a = move_line_2.lot_id
 
         move_wha_to_cus._action_assign()
         self.assertEqual(move_wha_to_cus.state, "assigned")
-        res_dict = move_wha_to_cus.picking_id.button_validate()
-        self.assertEqual(res_dict.get('res_model'), 'stock.immediate.transfer')
-        wizard = Form(self.env[res_dict['res_model']].with_context(res_dict['context'])).save()
-        wizard.process()
+        move_wha_to_cus.picking_id.button_validate()
         self.assertEqual(self.env['stock.quant']._get_available_quantity(product_lot, customer_location, lot_a), 1.0)
 
-        self.assertEqual(lot_a.company_id, self.company_a)
         self.assertEqual(lot_a.name, 'lot a')
-        self.assertEqual(lot_b.company_id, self.company_b)
-        self.assertEqual(lot_b.name, 'lot b')
+
+    def test_route_rules_company_consistency(self):
+        route = self.env['stock.route'].create({
+            'name': 'Test Route',
+            'company_id': self.company_a.id,
+            'rule_ids': [
+                (0, 0, {
+                    'name': 'Buy',
+                    'action': 'pull_push',
+                    'company_id': self.company_a.id,
+                    'location_dest_id': self.stock_location_a.id,
+                    'picking_type_id': self.warehouse_a.in_type_id.id,
+                })
+            ]
+        })
+
+        with self.assertRaises(ValidationError):
+            route.write({'company_id': self.company_b.id})
+
+        with self.assertRaises(ValidationError):
+            route.write({'rule_ids': [
+                (0, 0, {
+                    'name': 'Buy',
+                    'action': 'pull_push',
+                    'company_id': self.company_b.id,
+                    'location_dest_id': self.stock_location_b.id,
+                    'picking_type_id': self.warehouse_b.in_type_id.id,
+                })
+            ]})
