@@ -16,7 +16,7 @@ class TestRobustness(TransactionCase):
         cls.uom_dozen = cls.env.ref('uom.product_uom_dozen')
         cls.product1 = cls.env['product.product'].create({
             'name': 'Product A',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': cls.env.ref('product.product_category_all').id,
         })
 
@@ -67,10 +67,9 @@ class TestRobustness(TransactionCase):
         move1._do_unreserve()
 
     def test_location_usage(self):
-        """ Changing the usage of a location shouldn't be allowed while
-        quantities are reserved, else the existing move lines won't be
-        consistent with the `reserved_quantity` on the quants.
-        """
+        """Changing the usage of a location shouldn't be allowed
+        or changing a location from scrap to non-scrap or vice versa
+        shouldn't be allowed when stock is available in a location"""
         # change stock usage
         test_stock_location = self.env['stock.location'].create({
             'name': "Test Location",
@@ -86,7 +85,7 @@ class TestRobustness(TransactionCase):
         )
 
         # reserve a unit
-        move1 = self.env['stock.move'].create({
+        move = self.env['stock.move'].create({
             'name': 'test_location_archive',
             'location_id': test_stock_location.id,
             'location_dest_id': self.customer_location.id,
@@ -94,25 +93,27 @@ class TestRobustness(TransactionCase):
             'product_uom': self.uom_unit.id,
             'product_uom_qty': 1,
         })
-        move1._action_confirm()
-        move1._action_assign()
-        self.assertEqual(move1.state, 'assigned')
-        quant = self.env['stock.quant']._gather(
-            self.product1,
-            test_stock_location,
-        )
-
-        # assert the reservation
-        self.assertEqual(quant.reserved_quantity, 0)  # reservation is bypassed in scrap location
-        self.assertEqual(move1.product_qty, 1)
+        move._action_confirm()
+        move._action_assign()
+        move.picked = True
+        move._action_done()
+        self.assertEqual(move.state, 'done')
 
         # change the stock usage
+        with self.cr.savepoint():
+            test_stock_location.scrap_location = False
+
+        # make some stock again
+        self.env['stock.quant']._update_available_quantity(
+            self.product1,
+            test_stock_location,
+            1,
+        )
+
+        # change the stock usage again
         with self.assertRaises(UserError):
             with self.cr.savepoint():
-                test_stock_location.scrap_location = False
-
-        # unreserve
-        move1._do_unreserve()
+                test_stock_location.scrap_location = True
 
     def test_package_unpack(self):
         """ Unpack a package that contains quants with a reservation
@@ -158,13 +159,13 @@ class TestRobustness(TransactionCase):
         """
         product1 = self.env['product.product'].create({
             'name': 'Product 1',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': self.env.ref('product.product_category_all').id,
             'tracking': 'lot',
         })
         product2 = self.env['product.product'].create({
             'name': 'Product 2',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': self.env.ref('product.product_category_all').id,
             'tracking': 'lot',
         })
@@ -172,13 +173,11 @@ class TestRobustness(TransactionCase):
         lot1 = self.env['stock.lot'].create({
             'name': 'lot1',
             'product_id': product1.id,
-            'company_id': self.env.company.id,
 
         })
         lot2 = self.env['stock.lot'].create({
             'name': 'lot2',
             'product_id': product2.id,
-            'company_id': self.env.company.id,
         })
 
         self.env['stock.quant']._update_available_quantity(product1, self.stock_location, 1, lot_id=lot1)
@@ -206,7 +205,7 @@ class TestRobustness(TransactionCase):
             move1.write({'move_line_ids': [(0, 0, {
                 'product_id': product1.id,
                 'product_uom_id': self.uom_unit.id,
-                'qty_done': 1,
+                'quantity': 1,
                 'lot_id': lot2.id,
                 'location_id': move1.location_id.id,
                 'location_dest_id': move1.location_dest_id.id,
@@ -216,7 +215,7 @@ class TestRobustness(TransactionCase):
             move2.write({'move_line_ids': [(0, 0, {
                 'product_id': product2.id,
                 'product_uom_id': self.uom_unit.id,
-                'qty_done': 1,
+                'quantity': 1,
                 'lot_id': lot1.id,
                 'location_id': move2.location_id.id,
                 'location_dest_id': move2.location_dest_id.id,
@@ -227,14 +226,13 @@ class TestRobustness(TransactionCase):
         """
         productA = self.env['product.product'].create({
             'name': 'ProductA',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': self.env.ref('product.product_category_all').id,
             'tracking': 'lot',
         })
         lotA = self.env['stock.lot'].create({
             'name': 'lotA',
             'product_id': productA.id,
-            'company_id': self.env.company.id,
 
         })
         self.env['stock.quant']._update_available_quantity(productA, self.stock_location, 5, lot_id=lotA)
@@ -251,7 +249,7 @@ class TestRobustness(TransactionCase):
         moveA.write({'move_line_ids': [(0, 0, {
             'product_id': productA.id,
             'product_uom_id': self.uom_unit.id,
-            'qty_done': 5,
+            'quantity': 5,
             'lot_id': lotA.id,
             'location_id': moveA.location_id.id,
             'location_dest_id': moveA.location_dest_id.id,
@@ -259,4 +257,4 @@ class TestRobustness(TransactionCase):
         moveA._action_done()
         moveA._set_lot_ids()
 
-        self.assertEqual(moveA.quantity_done, 5)
+        self.assertEqual(moveA.quantity, 5)
