@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
 
-from odoo.addons.stock.tests.common2 import TestStockCommon
+from odoo import Command
+from odoo.addons.stock.tests.common import TestStockCommon
 from odoo.tests import Form
-from odoo.exceptions import UserError
-from odoo.tools import mute_logger
 
 
 class TestWarehouse(TestStockCommon):
@@ -14,7 +13,7 @@ class TestWarehouse(TestStockCommon):
         cls.partner = cls.env['res.partner'].create({'name': 'Deco Addict'})
 
     def test_inventory_product(self):
-        self.product_1.type = 'product'
+        self.product_1.is_storable = True
         product_1_quant = self.env['stock.quant'].with_context(inventory_mode=True).create({
             'product_id': self.product_1.id,
             'inventory_quantity': 50.0,
@@ -54,11 +53,35 @@ class TestWarehouse(TestStockCommon):
         self.assertEqual(self.env['stock.quant']._gather(self.product_1, self.warehouse_1.wh_input_stock_loc_id).quantity, 0.0)
         self.assertEqual(self.env['stock.quant']._gather(self.product_1, self.env.ref('stock.stock_location_stock')).quantity, 0.0)
 
-    def test_inventory_wizard_as_manager(self):
-        """ Using the "Update Quantity" wizard as stock manager.
+    def test_initial_quant_location(self):
         """
-        self.product_1.type = 'product'
-        InventoryWizard = self.env['stock.change.product.qty'].with_user(self.user_stock_manager)
+        When updating product quantity, new quant should have its location set
+        to the stock location of the top warehouse.
+        """
+        stock_location = self.env.ref('stock.stock_location_stock')
+        suppliers_location = self.env.ref('stock.stock_location_suppliers')
+
+        warehouse = self.env['stock.warehouse'].create({
+            'name': 'Mixed locations',
+            'code': 'TEST',
+            'sequence': 0,
+        })
+        warehouse.in_type_id.default_location_dest_id = suppliers_location
+        warehouse.lot_stock_id = stock_location
+
+        quant = self.env['stock.quant'].new({
+            'product_id': self.product_1.id,
+            'inventory_quantity': 1,
+        })
+        quant._onchange_product_id()
+
+        self.assertEqual(quant.location_id, stock_location)
+
+    def test_inventory_wizard_as_user(self):
+        """ Using the "Update Quantity" wizard as stock user.
+        """
+        self.product_1.is_storable = True
+        InventoryWizard = self.env['stock.change.product.qty'].with_user(self.user_stock_user)
         inventory_wizard = InventoryWizard.create({
             'product_id': self.product_1.id,
             'product_tmpl_id': self.product_1.product_tmpl_id.id,
@@ -73,23 +96,9 @@ class TestWarehouse(TestStockCommon):
         quant = self.env['stock.quant'].search([('id', 'not in', self.existing_quants.ids)])
         self.assertEqual(len(quant), 2)
 
-    def test_inventory_wizard_as_user(self):
-        """ Using the "Update Quantity" wizard as stock user.
-        """
-        self.product_1.type = 'product'
-        InventoryWizard = self.env['stock.change.product.qty'].with_user(self.user_stock_user)
-        inventory_wizard = InventoryWizard.create({
-            'product_id': self.product_1.id,
-            'product_tmpl_id': self.product_1.product_tmpl_id.id,
-            'new_quantity': 50.0,
-        })
-        # User has no right on quant, must raise an AccessError
-        with self.assertRaises(UserError):
-            inventory_wizard.change_product_qty()
-
     def test_basic_move(self):
         product = self.product_3.with_user(self.user_stock_manager)
-        product.type = 'product'
+        product.is_storable = True
         picking_out = self.env['stock.picking'].create({
             'partner_id': self.partner.id,
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
@@ -116,7 +125,8 @@ class TestWarehouse(TestStockCommon):
         self.assertEqual(product.qty_available, 0.0)
         self.assertEqual(product.virtual_available, -5.0)
 
-        customer_move.quantity_done = 5
+        customer_move.quantity = 5
+        customer_move.picked = True
         customer_move._action_done()
         self.assertEqual(product.qty_available, -5.0)
 
@@ -124,7 +134,8 @@ class TestWarehouse(TestStockCommon):
         receive_move = self._create_move(product, self.env.ref('stock.stock_location_suppliers'), self.warehouse_1.lot_stock_id, product_uom_qty=15)
 
         receive_move._action_confirm()
-        receive_move.quantity_done = 15
+        receive_move.quantity = 15
+        receive_move.picked = True
         receive_move._action_done()
 
         product._compute_quantities()
@@ -139,14 +150,15 @@ class TestWarehouse(TestStockCommon):
         self.assertEqual(product.qty_available, 10.0)
         self.assertEqual(product.virtual_available, 8.0)
 
-        customer_move_2.quantity_done = 2.0
+        customer_move_2.quantity = 2.0
+        customer_move_2.picked = True
         customer_move_2._action_done()
         product._compute_quantities()
         self.assertEqual(product.qty_available, 8.0)
 
     def test_inventory_adjustment_and_negative_quants_1(self):
         """Make sure negative quants from returns get wiped out with an inventory adjustment"""
-        productA = self.env['product.product'].create({'name': 'Product A', 'type': 'product'})
+        productA = self.env['product.product'].create({'name': 'Product A', 'is_storable': True})
         stock_location = self.env.ref('stock.stock_location_stock')
         customer_location = self.env.ref('stock.stock_location_customers')
 
@@ -167,7 +179,8 @@ class TestWarehouse(TestStockCommon):
             'location_dest_id': customer_location.id,
         })
         picking_out.action_confirm()
-        picking_out.move_ids.quantity_done = 1
+        picking_out.move_ids.quantity = 1
+        picking_out.move_ids.picked = True
         picking_out._action_done()
 
         quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
@@ -177,10 +190,11 @@ class TestWarehouse(TestStockCommon):
             active_model='stock.picking'))
         stock_return_picking = stock_return_picking_form.save()
         stock_return_picking.product_return_moves.quantity = 1.0
-        stock_return_picking_action = stock_return_picking.create_returns()
+        stock_return_picking_action = stock_return_picking.action_create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
         return_pick.action_assign()
-        return_pick.move_ids.quantity_done = 1
+        return_pick.move_ids.quantity = 1
+        return_pick.move_ids.picked = True
         return_pick._action_done()
 
         quant = self.env['stock.quant'].search([('product_id', '=', productA.id), ('location_id', '=', stock_location.id)])
@@ -188,7 +202,7 @@ class TestWarehouse(TestStockCommon):
 
     def test_inventory_adjustment_and_negative_quants_2(self):
         """Make sure negative quants get wiped out with an inventory adjustment"""
-        productA = self.env['product.product'].create({'name': 'Product A', 'type': 'product'})
+        productA = self.env['product.product'].create({'name': 'Product A', 'is_storable': True})
         stock_location = self.env.ref('stock.stock_location_stock')
         customer_location = self.env.ref('stock.stock_location_customers')
         location_loss = productA.property_stock_inventory
@@ -210,7 +224,8 @@ class TestWarehouse(TestStockCommon):
             'location_dest_id': customer_location.id,
         })
         picking_out.action_confirm()
-        picking_out.move_ids.quantity_done = 1
+        picking_out.move_ids.quantity = 1
+        picking_out.move_ids.picked = True
         picking_out._action_done()
 
         # Make an inventory adjustment to set the quantity to 0
@@ -273,7 +288,7 @@ class TestWarehouse(TestStockCommon):
 
         product = self.env['product.product'].create({
             'name': 'Fakir',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': [(4, route_id) for route_id in [route_stock_to_dist.id, route_dist_to_shop.id, self.env.ref('stock.route_warehouse0_mto').id]],
         })
 
@@ -350,7 +365,7 @@ class TestWarehouse(TestStockCommon):
         # The product contains the 2 resupply routes.
         product = self.env['product.product'].create({
             'name': 'Fakir',
-            'type': 'product',
+            'is_storable': True,
             'route_ids': [(4, route_id) for route_id in [route_shop_namur.id, route_shop_wavre.id, self.env.ref('stock.route_warehouse0_mto').id]],
         })
 
@@ -365,6 +380,7 @@ class TestWarehouse(TestStockCommon):
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': warehouse_shop_namur.lot_stock_id.id,
             'location_dest_id': customer_location.id,
+            'state': 'draft',
         })
         self.env['stock.move'].create({
             'name': product.name,
@@ -384,17 +400,20 @@ class TestWarehouse(TestStockCommon):
         picking_stock_transit = self.env['stock.picking'].search([('location_id', '=', warehouse_distribution_namur.lot_stock_id.id)])
         self.assertTrue(picking_stock_transit)
         picking_stock_transit.action_assign()
-        picking_stock_transit.move_ids[0].quantity_done = 1.0
+        picking_stock_transit.move_ids[0].quantity = 1.0
+        picking_stock_transit.move_ids[0].picked = True
         picking_stock_transit._action_done()
 
         picking_transit_shop_namur = self.env['stock.picking'].search([('location_dest_id', '=', warehouse_shop_namur.lot_stock_id.id)])
         self.assertTrue(picking_transit_shop_namur)
         picking_transit_shop_namur.action_assign()
-        picking_transit_shop_namur.move_ids[0].quantity_done = 1.0
+        picking_transit_shop_namur.move_ids[0].quantity = 1.0
+        picking_transit_shop_namur.move_ids[0].picked = True
         picking_transit_shop_namur._action_done()
 
         picking_out_namur.action_assign()
-        picking_out_namur.move_ids[0].quantity_done = 1.0
+        picking_out_namur.move_ids[0].picked = True
+        picking_out_namur.move_ids[0].quantity = 1.0
         picking_out_namur._action_done()
 
         # Check that the correct quantity has been provided to customer
@@ -409,6 +428,7 @@ class TestWarehouse(TestStockCommon):
             'picking_type_id': self.env.ref('stock.picking_type_out').id,
             'location_id': warehouse_shop_wavre.lot_stock_id.id,
             'location_dest_id': customer_location.id,
+            'state': 'draft',
         })
         self.env['stock.move'].create({
             'name': product.name,
@@ -428,17 +448,20 @@ class TestWarehouse(TestStockCommon):
         picking_stock_transit = self.env['stock.picking'].search([('location_id', '=', warehouse_distribution_wavre.lot_stock_id.id)])
         self.assertTrue(picking_stock_transit)
         picking_stock_transit.action_assign()
-        picking_stock_transit.move_ids[0].quantity_done = 1.0
+        picking_stock_transit.move_ids[0].quantity = 1.0
+        picking_stock_transit.move_ids[0].picked = True
         picking_stock_transit._action_done()
 
         picking_transit_shop_wavre = self.env['stock.picking'].search([('location_dest_id', '=', warehouse_shop_wavre.lot_stock_id.id)])
         self.assertTrue(picking_transit_shop_wavre)
         picking_transit_shop_wavre.action_assign()
-        picking_transit_shop_wavre.move_ids[0].quantity_done = 1.0
+        picking_transit_shop_wavre.move_ids[0].quantity = 1.0
+        picking_transit_shop_wavre.move_ids[0].picked = True
         picking_transit_shop_wavre._action_done()
 
         picking_out_wavre.action_assign()
-        picking_out_wavre.move_ids[0].quantity_done = 1.0
+        picking_out_wavre.move_ids[0].quantity = 1.0
+        picking_out_wavre.move_ids[0].picked = True
         picking_out_wavre._action_done()
 
         # Check that the correct quantity has been provided to customer
@@ -446,6 +469,152 @@ class TestWarehouse(TestStockCommon):
         # Ensure there still no quants in distribution warehouse
         self.assertEqual(sum(self.env['stock.quant']._gather(product, warehouse_distribution_wavre.lot_stock_id).mapped('quantity')), 0)
 
+    def test_add_resupply_warehouse_one_by_one(self):
+        """ Checks that selecting a warehouse as a resupply warehouse one after another correctly sets the routes as well.
+        """
+        warehouse_A, warehouse_B, warehouse_C = self.env['stock.warehouse'].create([{
+            'name': code,
+            'code': code,
+        } for code in ['WH_A', 'WH_B', 'WH_C']])
+        warehouse_A.resupply_wh_ids = [Command.link(warehouse_B.id)]
+        # Assign Warehouse B as supplier warehouse
+        self.assertEqual(len(warehouse_A.resupply_route_ids), 1)
+        self.assertEqual(warehouse_A.resupply_route_ids.supplier_wh_id, warehouse_B)
+        # Assign Warehouse C as supplier warehouse
+        warehouse_A.resupply_wh_ids = [Command.link(warehouse_C.id)]
+        self.assertEqual(len(warehouse_A.resupply_route_ids), 2)
+        self.assertRecordValues(warehouse_A.resupply_route_ids.sorted('id'), [
+            {'supplier_wh_id': warehouse_B.id},
+            {'supplier_wh_id': warehouse_C.id},
+        ])
+
+    def test_toggle_resupply_warehouse(self):
+        """ Checks that selecting then unselecting a warehouse as resupply correctly archives/unarchives the related route.
+        """
+        warehouse_A = self.env['stock.warehouse'].create({
+            'name': 'Warehouse A',
+            'code': 'WH_A',
+        })
+        warehouse_B = self.env['stock.warehouse'].create({
+            'name': 'Warehouse B',
+            'code': 'WH_B',
+            'resupply_wh_ids': [Command.set(warehouse_A.ids)],
+        })
+        resupply_route = warehouse_B.resupply_route_ids
+        self.assertTrue(resupply_route.active, 'Route should be active')
+        # Un-select Warehouse A as a resupply warehouse
+        warehouse_B.resupply_wh_ids = [Command.set([])]
+        self.assertFalse(warehouse_B.resupply_route_ids)
+        self.assertFalse(resupply_route.active, 'Route should now be inactive')
+        # Re-select Warehouse A as a resupply warehouse
+        warehouse_B.resupply_wh_ids = [Command.set(warehouse_A.ids)]
+        self.assertEqual(warehouse_B.resupply_route_ids, resupply_route)
+        self.assertTrue(resupply_route.active, 'Route should now be active')
+
+    def test_muti_step_resupply_warehouse(self):
+        """ Simulate the following situation:
+        - First warehouse has a 3-steps delivery
+        - Second warehouse has a 3-steps reception
+        - Second warehouse is resupplied by the first warehouse
+        - A product has some stock in the first warehouse
+        - A reordering rule is set on the product to fill the second warehouse
+        Ensure that the product can move all the way from the first to the second warehouse.
+        """
+        warehouse_A = self.env['stock.warehouse'].create({
+            'name': 'Warehouse A',
+            'code': 'WH_A',
+            'delivery_steps': 'pick_pack_ship',
+        })
+        warehouse_B = self.env['stock.warehouse'].create({
+            'name': 'Warehouse B',
+            'code': 'WH_B',
+            'reception_steps': 'three_steps',
+            'resupply_wh_ids': [Command.link(warehouse_A.id)],
+        })
+        self.product_3.write({
+            'type': 'consu',
+            'is_storable': True,
+            'route_ids': [Command.link(warehouse_B.resupply_route_ids.id)],
+        })
+        self.env['stock.quant']._update_available_quantity(self.product_3, warehouse_A.lot_stock_id, 1.0)
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(self.product_3, warehouse_A.lot_stock_id), 1)
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(self.product_3, warehouse_B.lot_stock_id), 0)
+
+        orderpoint = self.env['stock.warehouse.orderpoint'].create({
+            'location_id': warehouse_B.lot_stock_id.id,
+            'product_id': self.product_3.id,
+            'qty_to_order': 1.0,
+        })
+        orderpoint.action_replenish()
+        # Check that the orderpoint generated the source move from the furthest location.
+        move = self.env['stock.move'].search([('location_id', '=', warehouse_A.lot_stock_id.id), ('origin', '=', orderpoint.name)])
+        self.assertTrue(move, 'No move created from WH_A/Stock')
+
+        # Validate each intermediate transfers towards resupply of WH_B/Stock
+        inter_wh_loc = self.env.company.internal_transit_location_id
+        step_location_ids = [
+            (warehouse_A.lot_stock_id.id, warehouse_A.wh_pack_stock_loc_id.id),             # WH_A/Stock -> WH_A/Packing Zone
+            (warehouse_A.wh_pack_stock_loc_id.id, warehouse_A.wh_output_stock_loc_id.id),   # WH_A/Packing Zone -> WH_A/Output
+            (warehouse_A.wh_output_stock_loc_id.id, inter_wh_loc.id),                       # WH_A/Output -> Inter-warehouse transit
+            (inter_wh_loc.id, warehouse_B.wh_input_stock_loc_id.id),                        # Inter-warehouse transit -> WH_B/Input
+            (warehouse_B.wh_input_stock_loc_id.id, warehouse_B.wh_qc_stock_loc_id.id),      # WH_B/Input -> WH_B/Quality Control
+            (warehouse_B.wh_qc_stock_loc_id.id, warehouse_B.lot_stock_id.id),               # WH_B/Quality Control -> WH_B/Stock
+        ]
+        for loc_src_id, loc_dest_id in step_location_ids:
+            self.assertEqual(move.location_id.id, loc_src_id)
+            self.assertEqual(move.location_dest_id.id, loc_dest_id)
+            move.picked = True
+            move._action_done()
+            self.assertEqual(move.state, 'done')
+            move = move.move_dest_ids
+        # Verify that the quantity has been properly transfered from WH_A/Stock to WH_B/Stock
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(self.product_3, warehouse_A.lot_stock_id), 0)
+        self.assertEqual(self.env['stock.quant']._get_available_quantity(self.product_3, warehouse_B.lot_stock_id), 1)
+
+    def test_change_delivery_step_resupply_warehouse(self):
+        """ Verifies that when changing the delivery steps of a warehouse, it correctly adds/removes the extra rule
+        that is required to resupply the Output location.
+        """
+        warehouse_A = self.env['stock.warehouse'].create({
+            'name': 'Warehouse X',
+            'code': 'WH_X',
+        })
+        warehouse_B = self.env['stock.warehouse'].create({
+            'name': 'Warehouse Y',
+            'code': 'WH_Y',
+            'resupply_wh_ids': [Command.link(warehouse_A.id)],
+        })
+        resupply_rules = warehouse_B.resupply_route_ids.rule_ids
+        self.assertEqual(len(resupply_rules), 2)
+        stock_A_to_transit = resupply_rules.filtered(lambda r: r.location_dest_id == self.env.company.internal_transit_location_id)
+        self.assertEqual(stock_A_to_transit.location_src_id, warehouse_A.lot_stock_id)
+
+        # Set Warehouse A to 3 steps, a new rule should be created to resupply Output.
+        warehouse_A.delivery_steps = 'pick_pack_ship'
+        new_resupply_rules = warehouse_B.resupply_route_ids.rule_ids
+        self.assertEqual(len(new_resupply_rules), 3)
+        self.assertEqual(stock_A_to_transit.location_src_id, warehouse_A.wh_output_stock_loc_id)
+        stock_to_output = new_resupply_rules - resupply_rules
+        self.assertEqual(stock_to_output.location_src_id, warehouse_A.lot_stock_id)
+        self.assertEqual(stock_to_output.location_dest_id, warehouse_A.wh_output_stock_loc_id)
+
+        # Set Warehouse A to 2 steps, no change should have been made.
+        warehouse_A.delivery_steps = 'pick_ship'
+        self.assertEqual(warehouse_B.resupply_route_ids.rule_ids, new_resupply_rules)
+        self.assertEqual(stock_A_to_transit.location_src_id, warehouse_A.wh_output_stock_loc_id)
+        self.assertEqual(stock_to_output.location_dest_id, warehouse_A.wh_output_stock_loc_id)
+
+        # Set Warehouse A to 1 step, the rule to resupply Output should be archived.
+        warehouse_A.delivery_steps = 'ship_only'
+        self.assertEqual(warehouse_B.resupply_route_ids.rule_ids, resupply_rules)
+        self.assertEqual(stock_A_to_transit.location_src_id, warehouse_A.lot_stock_id)
+        self.assertFalse(stock_to_output.active, "The intermediate rule should have been archived.")
+
+        # Set Warehouse A back to 2 steps, the rule to resupply Output should be unarchived.
+        warehouse_A.delivery_steps = 'pick_ship'
+        self.assertTrue(stock_to_output.active, "The intermediate rule should have been unarchived.")
+        self.assertEqual(warehouse_B.resupply_route_ids.rule_ids, new_resupply_rules, "No new rule should have been created.")
+
     def test_noleak(self):
         # non-regression test to avoid company_id leaking to other warehouses (see blame)
         partner = self.env['res.partner'].create({'name': 'Chicago partner'})
@@ -504,7 +673,6 @@ class TestWarehouse(TestStockCommon):
 
         # Picking Type
         self.assertFalse(warehouse.in_type_id.active)
-        self.assertFalse(warehouse.in_type_id.show_operations)
         self.assertFalse(warehouse.out_type_id.active)
         self.assertFalse(warehouse.int_type_id.active)
         self.assertFalse(warehouse.pick_type_id.active)
@@ -529,7 +697,6 @@ class TestWarehouse(TestStockCommon):
 
         # Picking Type
         self.assertTrue(warehouse.in_type_id.active)
-        self.assertFalse(warehouse.in_type_id.show_operations)
         self.assertTrue(warehouse.out_type_id.active)
         self.assertTrue(warehouse.int_type_id.active)
         self.assertFalse(warehouse.pick_type_id.active)
@@ -621,13 +788,13 @@ class TestWarehouse(TestStockCommon):
         wh.name = "Chicago"
         wh.code = "chic"
         warehouse = wh.save()
-        self.assertEqual(warehouse.int_type_id.barcode, 'CHIC-INTERNAL')
+        self.assertEqual(warehouse.int_type_id.barcode, 'CHICINT')
         self.assertEqual(warehouse.int_type_id.sequence_id.prefix, 'chic/INT/')
 
         wh = Form(warehouse)
         wh.code = 'CH'
         wh.save()
-        self.assertEqual(warehouse.int_type_id.barcode, 'CH-INTERNAL')
+        self.assertEqual(warehouse.int_type_id.barcode, 'CHINT')
         self.assertEqual(warehouse.int_type_id.sequence_id.prefix, 'CH/INT/')
 
     def test_location_warehouse(self):
@@ -649,3 +816,37 @@ class TestWarehouse(TestStockCommon):
         test_warehouse.sequence = 100
         location._compute_warehouse_id()
         self.assertEqual(location.warehouse_id, test_warehouse)
+
+    def test_location_updates_wh(self):
+        warehouse_A = self.env['stock.warehouse'].create({
+            'name': 'Warehouse X',
+            'code': 'WH_X',
+            'delivery_steps': 'pick_pack_ship'
+        })
+        warehouse_B = self.env['stock.warehouse'].create({
+            'name': 'Warehouse Y',
+            'code': 'WH_Y',
+            'delivery_steps': 'pick_pack_ship'
+        })
+        picking_out = self.env['stock.picking'].create({
+            'partner_id': self.partner.id,
+            'picking_type_id': warehouse_A.pick_type_id.id,
+            'location_id': warehouse_A.lot_stock_id.id,
+            'location_dest_id': self.env.ref('stock.stock_location_customers').id,
+        })
+        customer_move = self.env['stock.move'].create({
+            'name': self.product.name,
+            'product_id': self.product.id,
+            'product_uom_qty': 1,
+            'product_uom': self.product.uom_id.id,
+            'picking_id': picking_out.id,
+            'location_id': warehouse_A.lot_stock_id.id,
+            'location_dest_id': self.env.ref('stock.stock_location_customers').id,
+        })
+        picking_form = Form(picking_out)
+        picking_form.picking_type_id = warehouse_B.pick_type_id
+        picking_form.save()
+        self.assertEqual(customer_move.warehouse_id, warehouse_B)
+        self.assertEqual(picking_out.picking_type_id, warehouse_B.pick_type_id)
+        picking_out.button_validate()
+        self.assertEqual(customer_move.move_dest_ids.warehouse_id, warehouse_B)
