@@ -15,8 +15,12 @@ class ChooseDestinationLocation(models.TransientModel):
 
     @api.depends('picking_id')
     def _compute_move_line_ids(self):
+        # specific move lines selected from move line view
+        move_lines_to_pack_ids = self.env.context.get('move_lines_to_pack_ids')
         for destination in self:
-            destination.move_line_ids = destination.picking_id.move_line_ids.filtered(lambda l: l.qty_done > 0 and not l.result_package_id)
+            destination.move_line_ids = destination.picking_id.move_line_ids.filtered(lambda l: l.quantity > 0 and not l.result_package_id)
+            if move_lines_to_pack_ids:
+                destination.move_line_ids = destination.move_line_ids.filtered(lambda l: l.id in move_lines_to_pack_ids)
 
     @api.depends('move_line_ids')
     def _filter_location(self):
@@ -26,4 +30,4 @@ class ChooseDestinationLocation(models.TransientModel):
     def action_done(self):
         # set the same location on each move line and pass again in action_put_in_pack
         self.move_line_ids.location_dest_id = self.location_dest_id
-        return self.picking_id.action_put_in_pack()
+        return self.move_line_ids.action_put_in_pack()
