@@ -22,6 +22,9 @@ Dashboard / Reports for Warehouse Management includes:
     'depends': ['stock', 'account'],
     'category': 'Hidden',
     'sequence': 16,
+    'demo': [
+        'data/stock_account_demo.xml',
+    ],
     'data': [
         'security/stock_account_security.xml',
         'security/ir.model.access.csv',
@@ -29,15 +32,24 @@ Dashboard / Reports for Warehouse Management includes:
         'views/stock_account_views.xml',
         'views/res_config_settings_views.xml',
         'data/product_data.xml',
-        'views/product_views.xml',
-        'views/stock_quant_views.xml',
+        'views/report_invoice.xml',
         'views/stock_valuation_layer_views.xml',
+        'views/stock_quant_views.xml',
+        'views/product_views.xml',
+        'views/stock_lot_views.xml',
+        'views/stock_picking_views.xml',
+        'wizard/stock_request_count.xml',
         'wizard/stock_valuation_layer_revaluation_views.xml',
-        'report/report_stock_forecasted.xml',
-    ],
-    'test': [
+        'wizard/stock_quantity_history.xml',
+        'report/account_invoice_report_view.xml',
     ],
     'installable': True,
     'auto_install': True,
     'post_init_hook': '_configure_journals',
+    'assets': {
+        'web.assets_backend': [
+            'stock_account/static/src/**/*',
+        ],
+    },
+    'license': 'LGPL-3',
 }
