@@ -3,12 +3,15 @@
 
 from . import account_chart_template
 from . import account_move
+from . import analytic_account
 from . import product
+from . import res_company
 from . import stock_move
-from . import stock_inventory
 from . import stock_location
+from . import stock_lot
 from . import stock_move_line
 from . import stock_picking
 from . import stock_quant
 from . import stock_valuation_layer
 from . import res_config_settings
+from . import template_generic_coa
