@@ -21,8 +21,15 @@ internal transfer document is needed.
 
 """,
     'depends': ['sale_purchase_stock'],
-    'data': ['data/stock_data.xml', 'views/sale_order_views.xml'],
+    'data': [
+        'data/stock_data.xml',
+        'views/sale_order_views.xml',
+        'views/stock_picking_views.xml',
+        'views/purchase_order_views.xml'
+    ],
+    'demo': [
+        'data/stock_dropshipping_demo.xml',
+    ],
     'installable': True,
-    'auto_install': False,
     'license': 'LGPL-3',
 }
