@@ -8,15 +8,15 @@ from odoo.tests import Form, tagged
 class TestStockValuation(ValuationReconciliationTestCommon):
 
     @classmethod
-    def setUpClass(cls, chart_template_ref=None):
-        super().setUpClass(chart_template_ref=chart_template_ref)
+    def setUpClass(cls):
+        super().setUpClass()
 
         cls.supplier_location = cls.env.ref('stock.stock_location_suppliers')
         cls.stock_location = cls.company_data['default_warehouse'].lot_stock_id
         cls.partner_id = cls.env['res.partner'].create({'name': 'My Test Partner'})
         cls.product1 = cls.env['product.product'].create({
             'name': 'Large Desk',
-            'type': 'product',
+            'is_storable': True,
             'categ_id': cls.stock_account_product_categ.id,
             'taxes_id': [(6, 0, [])],
         })
@@ -49,7 +49,6 @@ class TestStockValuation(ValuationReconciliationTestCommon):
                 'price_unit': 12,
                 'tax_id': [(6, 0, [])],
             })],
-            'pricelist_id': self.env.ref('product.list0').id,
             'picking_policy': 'direct',
         })
         self.sale_order1.action_confirm()
@@ -60,9 +59,7 @@ class TestStockValuation(ValuationReconciliationTestCommon):
 
         # validate the dropshipping picking
         self.assertEqual(len(self.sale_order1.picking_ids), 1)
-        wizard = self.sale_order1.picking_ids.button_validate()
-        immediate_transfer = Form(self.env[wizard['res_model']].with_context(wizard['context'])).save()
-        immediate_transfer.process()
+        self.sale_order1.picking_ids.button_validate()
         self.assertEqual(self.sale_order1.picking_ids.state, 'done')
 
         # create the vendor bill
@@ -281,9 +278,11 @@ class TestStockValuation(ValuationReconciliationTestCommon):
             .with_context(active_ids=self.sale_order1.picking_ids.ids, active_id=self.sale_order1.picking_ids.ids[0],
             active_model='stock.picking'))
         stock_return_picking = stock_return_picking_form.save()
-        stock_return_picking_action = stock_return_picking.create_returns()
+        stock_return_picking.product_return_moves.quantity = 1.0
+        stock_return_picking_action = stock_return_picking.action_create_returns()
         return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
-        return_pick.move_ids[0].move_line_ids[0].qty_done = 1.0
+        return_pick.move_ids[0].move_line_ids[0].quantity = 1.0
+        return_pick.move_ids[0].picked = True
         return_pick._action_done()
         self.assertEqual(return_pick.move_ids._is_dropshipped_returned(), True)
 
@@ -298,3 +297,46 @@ class TestStockValuation(ValuationReconciliationTestCommon):
         }
 
         self._check_results(expected_aml, 4, all_amls_return - all_amls)
+
+    def test_dropship_fifo_return(self):
+        """Test the return of a dropship order with a product set to FIFO costing
+        method. The unit price is correctly computed on the return picking svl.
+        """
+        self.env.company.anglo_saxon_accounting = True
+        self.product1.product_tmpl_id.categ_id.property_cost_method = 'fifo'
+        self.product1.product_tmpl_id.categ_id.property_valuation = 'real_time'
+        self.product1.product_tmpl_id.invoice_policy = 'order'
+
+        self._dropship_product1()
+        self.assertTrue(8 in self.purchase_order1.picking_ids.move_ids.stock_valuation_layer_ids.mapped('value'))
+        self.assertTrue(-8 in self.purchase_order1.picking_ids.move_ids.stock_valuation_layer_ids.mapped('value'))
+
+        # return what we've done
+        stock_return_picking_form = Form(self.env['stock.return.picking']
+            .with_context(active_ids=self.sale_order1.picking_ids.ids, active_id=self.sale_order1.picking_ids.ids[0],
+            active_model='stock.picking'))
+        stock_return_picking = stock_return_picking_form.save()
+        stock_return_picking.product_return_moves.quantity = 1.0
+        stock_return_picking_action = stock_return_picking.action_create_returns()
+        return_pick = self.env['stock.picking'].browse(stock_return_picking_action['res_id'])
+        return_pick.move_ids[0].move_line_ids[0].quantity = 1.0
+        return_pick.move_ids[0].picked = True
+        return_pick._action_done()
+
+        self.assertTrue(8 in return_pick.move_ids.stock_valuation_layer_ids.mapped('value'))
+        self.assertTrue(-8 in return_pick.move_ids.stock_valuation_layer_ids.mapped('value'))
+
+        # return again to have a new dropship picking from a dropship return
+        stock_return_picking_form_2 = Form(self.env['stock.return.picking']
+            .with_context(active_ids=return_pick.ids, active_id=return_pick.ids[0],
+            active_model='stock.picking'))
+        stock_return_picking_2 = stock_return_picking_form_2.save()
+        stock_return_picking_2.product_return_moves.quantity = 1.0
+        stock_return_picking_action_2 = stock_return_picking_2.action_create_returns()
+        return_pick_2 = self.env['stock.picking'].browse(stock_return_picking_action_2['res_id'])
+        return_pick_2.move_ids[0].move_line_ids[0].quantity = 1.0
+        return_pick_2.move_ids[0].picked = True
+        return_pick_2._action_done()
+
+        self.assertTrue(8 in return_pick_2.move_ids.stock_valuation_layer_ids.mapped('value'))
+        self.assertTrue(-8 in return_pick_2.move_ids.stock_valuation_layer_ids.mapped('value'))
