@@ -25,13 +25,13 @@ class AccountMove(models.Model):
         self.ensure_one()
         landed_costs_lines = self.line_ids.filtered(lambda line: line.is_landed_costs_line)
 
-        landed_costs = self.env['stock.landed.cost'].create({
+        landed_costs = self.env['stock.landed.cost'].with_company(self.company_id).create({
             'vendor_bill_id': self.id,
             'cost_lines': [(0, 0, {
                 'product_id': l.product_id.id,
                 'name': l.product_id.name,
                 'account_id': l.product_id.product_tmpl_id.get_product_accounts()['stock_input'].id,
-                'price_unit': l.currency_id._convert(l.price_subtotal, l.company_currency_id, l.company_id, l.move_id.date),
+                'price_unit': l.currency_id._convert(l.price_subtotal, l.company_currency_id, l.company_id, self.invoice_date or fields.Date.context_today(l)),
                 'split_method': l.product_id.split_method_landed_cost or 'equal',
             }) for l in landed_costs_lines],
         })
@@ -43,23 +43,21 @@ class AccountMove(models.Model):
         action = self.env["ir.actions.actions"]._for_xml_id("stock_landed_costs.action_stock_landed_cost")
         domain = [('id', 'in', self.landed_costs_ids.ids)]
         context = dict(self.env.context, default_vendor_bill_id=self.id)
-        views = [(self.env.ref('stock_landed_costs.view_stock_landed_cost_tree2').id, 'tree'), (False, 'form'), (False, 'kanban')]
+        views = [(self.env.ref('stock_landed_costs.view_stock_landed_cost_tree2').id, 'list'), (False, 'form'), (False, 'kanban')]
         return dict(action, domain=domain, context=context, views=views)
 
+    def _post(self, soft=True):
+        posted = super()._post(soft)
+        posted.sudo().landed_costs_ids.reconcile_landed_cost()
+        return posted
+
 
 class AccountMoveLine(models.Model):
     _inherit = 'account.move.line'
 
-    product_type = fields.Selection(related='product_id.detailed_type', readonly=True)
+    product_type = fields.Selection(related='product_id.type', readonly=True)
     is_landed_costs_line = fields.Boolean()
 
-    @api.depends('is_landed_costs_line')
-    def _compute_account_id(self):
-        landed_costs_lines = self.filtered(lambda l: l.is_landed_costs_line and l.move_id.company_id.anglo_saxon_accounting)
-        for line in landed_costs_lines:
-            line.account_id = line.product_id.product_tmpl_id._get_product_accounts()['stock_input']
-        super(AccountMoveLine, self - landed_costs_lines)._compute_account_id()
-
     @api.onchange('product_id')
     def _onchange_product_id_landed_costs(self):
         if self.product_id.landed_cost_ok:
@@ -75,3 +73,10 @@ class AccountMoveLine(models.Model):
     def _get_stock_valuation_layers(self, move):
         layers = super()._get_stock_valuation_layers(move)
         return layers.filtered(lambda svl: not svl.stock_landed_cost_id)
+
+    def _eligible_for_cogs(self):
+        return super()._eligible_for_cogs() or (
+            self.product_id.type == "service"
+            and self.product_id.landed_cost_ok
+            and self.product_id.valuation == "real_time"
+        )
