@@ -5,6 +5,7 @@ from odoo.addons.stock_landed_costs.tests.common import TestStockLandedCostsComm
 from odoo.addons.stock_landed_costs.tests.test_stockvaluationlayer import TestStockValuationLCCommon
 from odoo.addons.stock_account.tests.test_stockvaluation import _create_accounting_data
 
+from odoo.fields import Date
 from odoo.tests import tagged, Form
 
 
@@ -19,6 +20,7 @@ class TestLandedCosts(TestStockLandedCostsCommon):
             'partner_id': cls.supplier_id,
             'picking_type_id': cls.warehouse.in_type_id.id,
             'location_id': cls.supplier_location_id,
+            'state': 'draft',
             'location_dest_id': cls.warehouse.lot_stock_id.id})
         cls.Move.create({
             'name': cls.product_refrigerator.name,
@@ -41,6 +43,7 @@ class TestLandedCosts(TestStockLandedCostsCommon):
             'partner_id': cls.customer_id,
             'picking_type_id': cls.warehouse.out_type_id.id,
             'location_id': cls.warehouse.lot_stock_id.id,
+            'state': 'draft',
             'location_dest_id': cls.customer_location_id})
         cls.Move.create({
             'name': cls.product_refrigerator.name,
@@ -70,6 +73,8 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         #         3.transportation    250       By Weight
         #         4.packaging         20        By Volume
 
+        self.landed_cost.categ_id.property_valuation = 'real_time'
+
         # Process incoming shipment
         income_ship = self._process_incoming_shipment()
         # Create landed costs
@@ -115,10 +120,8 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         ])
 
     def test_00_landed_costs_on_incoming_shipment_without_real_time(self):
-        chart_of_accounts = self.env.company.chart_template_id
-        generic_coa = self.env.ref('l10n_generic_coa.configurable_chart_template')
-        if chart_of_accounts != generic_coa:
-            raise unittest.SkipTest('Skip this test as it works only with %s (%s loaded)' % (generic_coa.name, chart_of_accounts.name))
+        if self.env.company.chart_template != 'generic_coa':
+            raise unittest.SkipTest('Skip this test as it works only with `generic_coa`')
         # Test landed cost on incoming shipment
         #
         # (A) Purchase product
@@ -196,6 +199,8 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         #         3.transportation    -50       By Weight
         #         4.packaging         -5        By Volume
 
+        self.landed_cost.categ_id.property_valuation = 'real_time'
+
         # Process incoming shipment
         income_ship = self._process_incoming_shipment()
         # Refrigerator outgoing shipment.
@@ -243,9 +248,9 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         stock_negative_landed_cost.button_validate()
         self.assertEqual(stock_negative_landed_cost.state, 'done', 'Negative landed costs should be in done state')
         self.assertTrue(stock_negative_landed_cost.account_move_id, 'Landed costs should be available account move lines')
-        account_entry = self.env['account.move.line'].read_group(
-            [('move_id', '=', stock_negative_landed_cost.account_move_id.id)], ['balance', 'move_id'], ['move_id'])[0]
-        self.assertEqual(account_entry['balance'], 0, 'Move is not balanced')
+        [balance] = self.env['account.move.line']._read_group(
+            [('move_id', '=', stock_negative_landed_cost.account_move_id.id)], aggregates=['balance:sum'])[0]
+        self.assertEqual(balance, 0, 'Move is not balanced')
         move_lines = [
             {'name': 'split by volume - Microwave Oven',                    'debit': 3.75,  'credit': 0.0},
             {'name': 'split by volume - Microwave Oven',                    'debit': 0.0,   'credit': 3.75},
@@ -284,7 +289,7 @@ class TestLandedCosts(TestStockLandedCostsCommon):
                 {'name': 'equal split - Refrigerator: 2.0 already out',         'debit': 0.0,   'credit': 1.0},
             ]
         self.assertRecordValues(
-            sorted(stock_negative_landed_cost.account_move_id.line_ids, key=lambda d: (d['name'], d['debit'])),
+            stock_negative_landed_cost.account_move_id.line_ids.sorted(lambda d: (d['name'], d['debit'])),
             sorted(move_lines, key=lambda d: (d['name'], d['debit'])),
         )
 
@@ -293,9 +298,7 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         # Confirm incoming shipment.
         self.picking_in.action_confirm()
         # Transfer incoming shipment
-        res_dict = self.picking_in.button_validate()
-        wizard = Form(self.env[(res_dict.get('res_model'))].with_context(res_dict.get('context'))).save()
-        wizard.process()
+        self.picking_in.button_validate()
         return self.picking_in
 
     def _process_outgoing_shipment(self):
@@ -306,9 +309,7 @@ class TestLandedCosts(TestStockLandedCostsCommon):
         self.picking_out.action_assign()
         # Transfer picking.
 
-        res_dict = self.picking_out.button_validate()
-        wizard = Form(self.env[(res_dict.get('res_model'))].with_context(res_dict['context'])).save()
-        wizard.process()
+        self.picking_out.button_validate()
 
     def _create_landed_costs(self, value, picking_in):
         return self.LandedCost.create(dict(
@@ -382,7 +383,7 @@ class TestLandedCostsWithPurchaseAndInv(TestStockValuationLCCommon):
 
         # Receive the goods
         receipt = order.picking_ids[0]
-        receipt.move_ids.quantity_done = 1
+        receipt.move_ids.quantity = 1
         receipt.button_validate()
 
         # Check SVL and AML
@@ -425,3 +426,70 @@ class TestLandedCostsWithPurchaseAndInv(TestStockValuationLCCommon):
         # Check nothing was posted in the stock valuation account.
         price_diff_aml = self.env['account.move.line'].search([('account_id', '=', stock_valuation_account.id), ('move_id', '=', move.id)])
         self.assertEqual(len(price_diff_aml), 0, "No line should have been generated in the stock valuation account about the price difference.")
+
+    def test_invoice_after_lc_amls(self):
+        self.env.company.anglo_saxon_accounting = True
+        self.landed_cost.landed_cost_ok = True
+        self.landed_cost.categ_id.property_cost_method = 'fifo'
+        self.landed_cost.categ_id.property_valuation = 'real_time'
+
+        # Create PO
+        po = self.env['purchase.order'].create({
+            'partner_id': self.partner_a.id,
+            'currency_id': self.company_data['currency'].id,
+            'order_line': [
+                (0, 0, {
+                    'name': self.product_a.name,
+                    'product_id': self.product_a.id,
+                    'product_qty': 1.0,
+                    'product_uom': self.product_a.uom_po_id.id,
+                    'price_unit': 100.0,
+                    'taxes_id': False,
+                }),
+                (0, 0, {
+                    'name': self.landed_cost.name,
+                    'product_id': self.landed_cost.id,
+                    'product_qty': 1.0,
+                    'price_unit': 100.0,
+                }),
+            ],
+        })
+        po.button_confirm()
+
+        receipt = po.picking_ids
+        receipt.move_ids.quantity = 1
+        receipt.button_validate()
+        po.order_line[1].qty_received = 1
+
+        po.action_create_invoice()
+        bill = po.invoice_ids
+
+        # Create and validate LC
+        lc = self.env['stock.landed.cost'].create(dict(
+            picking_ids=[(6, 0, [receipt.id])],
+            account_journal_id=self.stock_journal.id,
+            cost_lines=[
+                (0, 0, {
+                    'name': 'equal split',
+                    'split_method': 'equal',
+                    'price_unit': 100,
+                    'product_id': self.landed_cost.id,
+                }),
+            ],
+        ))
+        lc.compute_landed_cost()
+        lc.button_validate()
+
+        user = self.env['res.users'].create({
+            'name': 'User h',
+            'login': 'usher',
+            'email': 'usher@yourcompany.com',
+            'groups_id': [(6, 0, [self.env.ref('account.group_account_invoice').id])]
+        })
+        # Post the bill
+        bill.landed_costs_ids = [(6, 0, lc.id)]
+        bill.invoice_date = Date.today()
+        bill.with_user(user)._post()
+
+        landed_cost_aml = bill.invoice_line_ids.filtered(lambda l: l.product_id == self.landed_cost)
+        self.assertTrue(landed_cost_aml.reconciled)
