@@ -2,8 +2,9 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import survey_survey
+from . import survey_survey_template
 from . import survey_question
-from . import survey_user
+from . import survey_user_input
 from . import badge
 from . import challenge
 from . import res_partner
