@@ -8,7 +8,7 @@ import uuid
 from dateutil.relativedelta import relativedelta
 
 from odoo import api, fields, models, _
-from odoo.exceptions import ValidationError
+from odoo.exceptions import ValidationError, UserError
 from odoo.tools import float_is_zero
 
 _logger = logging.getLogger(__name__)
@@ -23,13 +23,13 @@ class SurveyUserInput(models.Model):
     _inherit = ['mail.thread', 'mail.activity.mixin']
 
     # answer description
-    survey_id = fields.Many2one('survey.survey', string='Survey', required=True, readonly=True, ondelete='cascade')
+    survey_id = fields.Many2one('survey.survey', string='Survey', required=True, readonly=True, index=True, ondelete='cascade')
     scoring_type = fields.Selection(string="Scoring", related="survey_id.scoring_type")
     start_datetime = fields.Datetime('Start date and time', readonly=True)
     end_datetime = fields.Datetime('End date and time', readonly=True)
     deadline = fields.Datetime('Deadline', help="Datetime until customer can open the survey and submit answers")
     state = fields.Selection([
-        ('new', 'Not started yet'),
+        ('new', 'New'),
         ('in_progress', 'In Progress'),
         ('done', 'Completed')], string='Status', default='new', readonly=True)
     test_entry = fields.Boolean(readonly=True)
@@ -43,15 +43,16 @@ class SurveyUserInput(models.Model):
     # identification / access
     access_token = fields.Char('Identification token', default=lambda self: str(uuid.uuid4()), readonly=True, required=True, copy=False)
     invite_token = fields.Char('Invite token', readonly=True, copy=False)  # no unique constraint, as it identifies a pool of attempts
-    partner_id = fields.Many2one('res.partner', string='Contact', readonly=True)
+    partner_id = fields.Many2one('res.partner', string='Contact', readonly=True, index='btree_not_null')
     email = fields.Char('Email', readonly=True)
     nickname = fields.Char('Nickname', help="Attendee nickname, mainly used to identify them in the survey session leaderboard.")
     # questions / answers
     user_input_line_ids = fields.One2many('survey.user_input.line', 'user_input_id', string='Answers', copy=True)
     predefined_question_ids = fields.Many2many('survey.question', string='Predefined Questions', readonly=True)
     scoring_percentage = fields.Float("Score (%)", compute="_compute_scoring_values", store=True, compute_sudo=True)  # stored for perf reasons
-    scoring_total = fields.Float("Total Score", compute="_compute_scoring_values", store=True, compute_sudo=True)  # stored for perf reasons
+    scoring_total = fields.Float("Total Score", compute="_compute_scoring_values", store=True, compute_sudo=True, digits=(10, 2))  # stored for perf reasons
     scoring_success = fields.Boolean('Quizz Passed', compute='_compute_scoring_success', store=True, compute_sudo=True)  # stored for perf reasons
+    survey_first_submitted = fields.Boolean(string='Survey First Submitted')
     # live sessions
     is_session_answer = fields.Boolean('Is in a Session', help="Is that user input part of a survey session or not.")
     question_time_limit_reached = fields.Boolean("Question Time Limit Reached", compute='_compute_question_time_limit_reached')
@@ -234,14 +235,16 @@ class SurveyUserInput(models.Model):
         - The survey is a certification
         - It has a certification_mail_template_id set
         - The user succeeded the test
+        3. Notify survey subtype subscribers of the newly completed input
         Will also run challenge Cron to give the certification badge if any."""
         self.write({
             'end_datetime': fields.Datetime.now(),
             'state': 'done',
         })
 
-        Challenge = self.env['gamification.challenge'].sudo()
+        Challenge_sudo = self.env['gamification.challenge'].sudo()
         badge_ids = []
+        self._notify_new_participation_subscribers()
         for user_input in self:
             if user_input.survey_id.certification and user_input.scoring_success:
                 if user_input.survey_id.certification_mail_template_id and not user_input.test_entry:
@@ -253,9 +256,9 @@ class SurveyUserInput(models.Model):
             user_input.predefined_question_ids -= user_input._get_inactive_conditional_questions()
 
         if badge_ids:
-            challenges = Challenge.search([('reward_id', 'in', badge_ids)])
+            challenges = Challenge_sudo.search([('reward_id', 'in', badge_ids)])
             if challenges:
-                Challenge._cron_update(ids=challenges.ids, commit=False)
+                Challenge_sudo._cron_update(ids=challenges.ids, commit=False)
 
     def get_start_url(self):
         self.ensure_one()
@@ -269,18 +272,21 @@ class SurveyUserInput(models.Model):
     # CREATE / UPDATE LINES FROM SURVEY FRONTEND INPUT
     # ------------------------------------------------------------
 
-    def save_lines(self, question, answer, comment=None):
-        """ Save answers to questions, depending on question type
+    def _save_lines(self, question, answer, comment=None, overwrite_existing=True):
+        """ Save answers to questions, depending on question type.
 
-            If an answer already exists for question and user_input_id, it will be
-            overwritten (or deleted for 'choice' questions) (in order to maintain data consistency).
+        :param bool overwrite_existing: if an answer already exists for question and user_input_id
+        it will be overwritten (or deleted for 'choice' questions) in order to maintain data consistency.
+        :raises UserError: if line exists and overwrite_existing is False
         """
         old_answers = self.env['survey.user_input.line'].search([
             ('user_input_id', '=', self.id),
             ('question_id', '=', question.id)
         ])
+        if old_answers and not overwrite_existing:
+            raise UserError(_("This answer cannot be overwritten."))
 
-        if question.question_type in ['char_box', 'text_box', 'numerical_box', 'date', 'datetime']:
+        if question.question_type in ['char_box', 'text_box', 'scale', 'numerical_box', 'date', 'datetime']:
             self._save_line_simple_answer(question, old_answers, answer)
             if question.save_as_email and answer:
                 self.write({'email': answer})
@@ -361,6 +367,8 @@ class SurveyUserInput(models.Model):
             vals['suggested_answer_id'] = int(answer)
         elif answer_type == 'numerical_box':
             vals['value_numerical_box'] = float(answer)
+        elif answer_type == 'scale':
+            vals['value_scale'] = int(answer)
         else:
             vals['value_%s' % answer_type] = answer
         return vals
@@ -430,14 +438,20 @@ class SurveyUserInput(models.Model):
         scored_questions = self.mapped('predefined_question_ids').filtered(lambda question: question.is_scored_question)
 
         for question in scored_questions:
+            if question.question_type == 'simple_choice':
+                question_incorrect_scored_answers = question.suggested_answer_ids.filtered(lambda answer: not answer.is_correct and answer.answer_score > 0)
+
             if question.question_type in ['simple_choice', 'multiple_choice']:
                 question_correct_suggested_answers = question.suggested_answer_ids.filtered(lambda answer: answer.is_correct)
 
             question_section = question.page_id.title or _('Uncategorized')
             for user_input in self:
-                user_input_lines = user_input.user_input_line_ids.filtered(lambda line: line.question_id == question)
-                if question.question_type in ['simple_choice', 'multiple_choice']:
-                    answer_result_key = self._choice_question_answer_result(user_input_lines, question_correct_suggested_answers)
+                user_input_lines = user_input.user_input_line_ids.filtered(lambda line:
+                    line.question_id == question and (line.answer_type != 'char_box' or question.comment_count_as_answer))
+                if question.question_type == 'simple_choice':
+                    answer_result_key = self._simple_choice_question_answer_result(user_input_lines, question_correct_suggested_answers, question_incorrect_scored_answers)
+                elif question.question_type == 'multiple_choice':
+                    answer_result_key = self._multiple_choice_question_answer_result(user_input_lines, question_correct_suggested_answers)
                 else:
                     answer_result_key = self._simple_question_answer_result(user_input_lines)
 
@@ -474,7 +488,7 @@ class SurveyUserInput(models.Model):
 
         return res
 
-    def _choice_question_answer_result(self, user_input_lines, question_correct_suggested_answers):
+    def _multiple_choice_question_answer_result(self, user_input_lines, question_correct_suggested_answers):
         correct_user_input_lines = user_input_lines.filtered(lambda line: line.answer_is_correct and not line.skipped).mapped('suggested_answer_id')
         incorrect_user_input_lines = user_input_lines.filtered(lambda line: not line.answer_is_correct and not line.skipped)
         if question_correct_suggested_answers and correct_user_input_lines == question_correct_suggested_answers:
@@ -486,6 +500,17 @@ class SurveyUserInput(models.Model):
         else:
             return 'skipped'
 
+    def _simple_choice_question_answer_result(self, user_input_line, question_correct_suggested_answers, question_incorrect_scored_answers):
+        user_answer = user_input_line.suggested_answer_id if not user_input_line.skipped else self.env['survey.question.answer']
+        if user_answer in question_correct_suggested_answers:
+            return 'correct'
+        elif user_answer in question_incorrect_scored_answers:
+            return 'partial'
+        elif user_answer:
+            return 'incorrect'
+        else:
+            return 'skipped'
+
     def _simple_question_answer_result(self, user_input_line):
         if user_input_line.skipped:
             return 'skipped'
@@ -519,20 +544,21 @@ class SurveyUserInput(models.Model):
                    that is the next in sequence and that is either not triggered by another question's answer, or that
                    is triggered by an already selected answer.
          To do all this, we need to return:
-            - list of all selected answers: [answer_id1, answer_id2, ...] (for survey reloading, otherwise, this list is
-              updated at client side)
+            - triggering_answers_by_question: dict -> for a given question, the answers that triggers it
+                Used mainly to ease template rendering
             - triggered_questions_by_answer: dict -> for a given answer, list of questions triggered by this answer;
                 Used mainly for dynamic show/hide behaviour at client side
-            - triggering_answer_by_question: dict -> for a given question, the answer that triggers it
-                Used mainly to ease template rendering
+            - list of all selected answers: [answer_id1, answer_id2, ...] (for survey reloading, otherwise, this list is
+              updated at client side)
         """
-        triggering_answer_by_question, triggered_questions_by_answer = {}, {}
+        triggering_answers_by_question = {}
+        triggered_questions_by_answer = {}
         # Ignore conditional configuration if randomised questions selection
         if self.survey_id.questions_selection != 'random':
-            triggering_answer_by_question, triggered_questions_by_answer = self.survey_id._get_conditional_maps()
+            triggering_answers_by_question, triggered_questions_by_answer = self.survey_id._get_conditional_maps()
         selected_answers = self._get_selected_suggested_answers()
 
-        return triggering_answer_by_question, triggered_questions_by_answer, selected_answers
+        return triggering_answers_by_question, triggered_questions_by_answer, selected_answers
 
     def _get_selected_suggested_answers(self):
         """
@@ -568,14 +594,13 @@ class SurveyUserInput(models.Model):
         answers_to_delete.unlink()
 
     def _get_inactive_conditional_questions(self):
-        triggering_answer_by_question, triggered_questions_by_answer, selected_answers = self._get_conditional_values()
+        triggering_answers_by_question, _, selected_answers = self._get_conditional_values()
 
         # get questions that should not be answered
         inactive_questions = self.env['survey.question']
-        for answer in triggered_questions_by_answer.keys():
-            if answer not in selected_answers:
-                for question in triggered_questions_by_answer[answer]:
-                    inactive_questions |= question
+        for question, triggering_answers in triggering_answers_by_question.items():
+            if triggering_answers and not triggering_answers & selected_answers:
+                inactive_questions |= question
         return inactive_questions
 
     def _get_print_questions(self):
@@ -591,21 +616,90 @@ class SurveyUserInput(models.Model):
             inactive_questions = self._get_inactive_conditional_questions()
         return survey.question_ids - inactive_questions
 
+    def _get_next_skipped_page_or_question(self):
+        """Get next skipped question or page in case the option 'can_go_back' is set on the survey
+        It loops to the first skipped question or page if 'last_displayed_page_id' is the last
+        skipped question or page."""
+        self.ensure_one()
+        skipped_mandatory_answer_ids = self.user_input_line_ids.filtered(
+            lambda answer: answer.skipped and answer.question_id.constr_mandatory)
+
+        if not skipped_mandatory_answer_ids:
+            return self.env['survey.question']
+
+        page_or_question_key = 'page_id' if self.survey_id.questions_layout == 'page_per_section' else 'question_id'
+        page_or_question_ids = skipped_mandatory_answer_ids.mapped(page_or_question_key).sorted()
+
+        if self.last_displayed_page_id not in page_or_question_ids\
+            or self.last_displayed_page_id == page_or_question_ids[-1]:
+            return page_or_question_ids[0]
+
+        current_page_index = page_or_question_ids.ids.index(self.last_displayed_page_id.id)
+        return page_or_question_ids[current_page_index + 1]
+
+    def _get_skipped_questions(self):
+        self.ensure_one()
+
+        return self.user_input_line_ids.filtered(
+            lambda answer: answer.skipped and answer.question_id.constr_mandatory).question_id
+
+    def _is_last_skipped_page_or_question(self, page_or_question):
+        """In case of a submitted survey tells if the question or page is the last
+        skipped page or question.
+
+        This is used to :
+
+        - Display a Submit button if the actual question is the last skipped question.
+        - Avoid displaying a Submit button on the last survey question if there are
+          still skipped questions before.
+        - Avoid displaying the next page if submitting the latest skipped question.
+
+        :param page_or_question: page if survey's layout is page_per_section, question if page_per_question.
+        """
+        if self.survey_id.questions_layout == 'one_page':
+            return True
+        skipped = self._get_skipped_questions()
+        if not skipped:
+            return True
+        if self.survey_id.questions_layout == 'page_per_section':
+            skipped = skipped.page_id
+        return skipped == page_or_question
+
     # ------------------------------------------------------------
     # MESSAGING
     # ------------------------------------------------------------
 
     def _message_get_suggested_recipients(self):
         recipients = super()._message_get_suggested_recipients()
-        for user_input in self:
-            if user_input.partner_id:
-                user_input._message_add_suggested_recipient(
-                    recipients,
-                    partner=user_input.partner_id,
-                    reason=_('Survey Participant')
-                )
+        if self.partner_id:
+            self._message_add_suggested_recipient(
+                recipients,
+                partner=self.partner_id,
+                reason=_('Survey Participant')
+            )
         return recipients
 
+    def _notify_new_participation_subscribers(self):
+        subtype_id = self.env.ref('survey.mt_survey_survey_user_input_completed', raise_if_not_found=False)
+        if not self.ids or not subtype_id:
+            return
+        author_id = self.env.ref('base.partner_root').id if self.env.user.is_public else self.env.user.partner_id.id
+        # Only post if there are any followers
+        recipients_data = self.env['mail.followers']._get_recipient_data(self.survey_id, 'notification', subtype_id.id)
+        followed_survey_ids = [survey_id for survey_id, followers in recipients_data.items() if followers]
+        for user_input in self.filtered(lambda user_input_: user_input_.survey_id.id in followed_survey_ids):
+            survey_title = user_input.survey_id.title
+            if user_input.partner_id:
+                body = _(
+                    '%(participant)s just participated in "%(survey_title)s".',
+                    participant=user_input.partner_id.display_name,
+                    survey_title=survey_title,
+                )
+            else:
+                body = _('Someone just participated in "%(survey_title)s".', survey_title=survey_title)
+
+            user_input.message_post(author_id=author_id, body=body, subtype_xmlid='survey.mt_survey_user_input_completed')
+
 
 class SurveyUserInputLine(models.Model):
     _name = 'survey.user_input.line'
@@ -616,7 +710,7 @@ class SurveyUserInputLine(models.Model):
     # survey data
     user_input_id = fields.Many2one('survey.user_input', string='User Input', ondelete='cascade', required=True, index=True)
     survey_id = fields.Many2one(related='user_input_id.survey_id', string='Survey', store=True, readonly=False)
-    question_id = fields.Many2one('survey.question', string='Question', ondelete='cascade', required=True)
+    question_id = fields.Many2one('survey.question', string='Question', ondelete='cascade', required=True, index=True)
     page_id = fields.Many2one(related='question_id.page_id', string="Section", readonly=False)
     question_sequence = fields.Integer('Sequence', related='question_id.sequence', store=True)
     # answer
@@ -625,11 +719,13 @@ class SurveyUserInputLine(models.Model):
         ('text_box', 'Free Text'),
         ('char_box', 'Text'),
         ('numerical_box', 'Number'),
+        ('scale', 'Number'),
         ('date', 'Date'),
         ('datetime', 'Datetime'),
         ('suggestion', 'Suggestion')], string='Answer Type')
     value_char_box = fields.Char('Text answer')
     value_numerical_box = fields.Float('Numerical answer')
+    value_scale = fields.Integer('Scale value')
     value_date = fields.Date('Date answer')
     value_datetime = fields.Datetime('Datetime answer')
     value_text_box = fields.Text('Free Text answer')
@@ -639,7 +735,11 @@ class SurveyUserInputLine(models.Model):
     answer_score = fields.Float('Score')
     answer_is_correct = fields.Boolean('Correct')
 
-    @api.depends('answer_type')
+    @api.depends(
+        'answer_type', 'value_text_box', 'value_numerical_box',
+        'value_char_box', 'value_date', 'value_datetime',
+        'suggested_answer_id.value', 'matrix_row_id.value',
+    )
     def _compute_display_name(self):
         for line in self:
             if line.answer_type == 'char_box':
@@ -652,11 +752,11 @@ class SurveyUserInputLine(models.Model):
                 line.display_name = fields.Date.to_string(line.value_date)
             elif line.answer_type == 'datetime':
                 line.display_name = fields.Datetime.to_string(line.value_datetime)
+            elif line.answer_type == 'scale':
+                line.display_name = line.value_scale
             elif line.answer_type == 'suggestion':
                 if line.matrix_row_id:
-                    line.display_name = '%s: %s' % (
-                        line.suggested_answer_id.value,
-                        line.matrix_row_id.value)
+                    line.display_name = f'{line.suggested_answer_id.value}: {line.matrix_row_id.value}'
                 else:
                     line.display_name = line.suggested_answer_id.value
 
@@ -669,9 +769,12 @@ class SurveyUserInputLine(models.Model):
             if (line.skipped == bool(line.answer_type)):
                 raise ValidationError(_('A question can either be skipped or answered, not both.'))
 
-            # allow 0 for numerical box
+            # allow 0 for numerical box and scale
             if line.answer_type == 'numerical_box' and float_is_zero(line['value_numerical_box'], precision_digits=6):
                 continue
+            if line.answer_type == 'scale' and line['value_scale'] == 0:
+                continue
+
             if line.answer_type == 'suggestion':
                 field_name = 'suggested_answer_id'
             elif line.answer_type:
@@ -706,6 +809,29 @@ class SurveyUserInputLine(models.Model):
             res = super(SurveyUserInputLine, line).write(vals_copy) and res
         return res
 
+    def _get_answer_matching_domain(self):
+        self.ensure_one()
+        if self.answer_type in ('char_box', 'text_box', 'numerical_box', 'scale', 'date', 'datetime'):
+            value_field = {
+                'char_box': 'value_char_box',
+                'text_box': 'value_text_box',
+                'numerical_box': 'value_numerical_box',
+                'scale': 'value_scale',
+                'date': 'value_date',
+                'datetime': 'value_datetime',
+            }
+            operators = {
+                'char_box': 'ilike',
+                'text_box': 'ilike',
+                'numerical_box': '=',
+                'scale': '=',
+                'date': '=',
+                'datetime': '=',
+            }
+            return ['&', ('question_id', '=', self.question_id.id), (value_field[self.answer_type], operators[self.answer_type], self._get_answer_value())]
+        elif self.answer_type == 'suggestion':
+            return self.suggested_answer_id._get_answer_matching_domain(self.matrix_row_id.id if self.matrix_row_id else False)
+
     @api.model
     def _get_answer_score_values(self, vals, compute_speed_score=True):
         """ Get values for: answer_is_correct and associated answer_score.
@@ -764,7 +890,7 @@ class SurveyUserInputLine(models.Model):
         if compute_speed_score and answer_score > 0:
             user_input = self.env['survey.user_input'].browse(user_input_id)
             session_speed_rating = user_input.exists() and user_input.is_session_answer and user_input.survey_id.session_speed_rating
-            if session_speed_rating:
+            if session_speed_rating and question.is_time_limited:
                 max_score_delay = 2
                 time_limit = question.time_limit
                 now = fields.Datetime.now()
@@ -773,12 +899,28 @@ class SurveyUserInputLine(models.Model):
                 # if answered within the max_score_delay => leave score as is
                 if question_remaining_time < 0:  # if no time left
                     answer_score /= 2
-                elif seconds_to_answer > max_score_delay:
-                    time_limit -= max_score_delay  # we remove the max_score_delay to have all possible values
-                    score_proportion = (time_limit - seconds_to_answer) / time_limit
+                elif seconds_to_answer > max_score_delay:  # linear decrease in score after 2 sec
+                    score_proportion = (time_limit - seconds_to_answer) / (time_limit - max_score_delay)
                     answer_score = (answer_score / 2) * (1 + score_proportion)
 
         return {
             'answer_is_correct': answer_is_correct,
             'answer_score': answer_score
         }
+
+    def _get_answer_value(self):
+        self.ensure_one()
+        if self.answer_type == 'char_box':
+            return self.value_char_box
+        elif self.answer_type == 'text_box':
+            return self.value_text_box
+        elif self.answer_type == 'numerical_box':
+            return self.value_numerical_box
+        elif self.answer_type == 'scale':
+            return self.value_scale
+        elif self.answer_type == 'date':
+            return self.value_date
+        elif self.answer_type == 'datetime':
+            return self.value_datetime
+        elif self.answer_type == 'suggestion':
+            return self.suggested_answer_id.value
