@@ -1,9 +1,9 @@
-odoo.define('survey.result', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var _t = require('web.core')._t;
-const { loadJS } = require('@web/core/assets');
-var publicWidget = require('web.public.widget');
+import { _t } from "@web/core/l10n/translation";
+import { loadBundle } from "@web/core/assets";
+import { SurveyImageZoomer } from "@survey/js/survey_image_zoomer";
+import publicWidget from "@web/legacy/js/public/public_widget";
 
 // The given colors are the same as those used by D3
 var D3_COLORS = ["#1f77b4","#ff7f0e","#aec7e8","#ffbb78","#2ca02c","#98df8a","#d62728",
@@ -15,6 +15,7 @@ var D3_COLORS = ["#1f77b4","#ff7f0e","#aec7e8","#ffbb78","#2ca02c","#98df8a","#d
 publicWidget.registry.SurveyResultPagination = publicWidget.Widget.extend({
     events: {
         'click li.o_survey_js_results_pagination a': '_onPageClick',
+        "click .o_survey_question_answers_show_btn": "_onShowAllAnswers",
     },
 
     //--------------------------------------------------------------------------
@@ -55,18 +56,32 @@ publicWidget.registry.SurveyResultPagination = publicWidget.Widget.extend({
 
         var $target = $(ev.currentTarget);
         $target.closest('li').addClass('active');
-        this.$questionsEl.find('tbody tr').addClass('d-none');
+        this.$questionsEl.find("tbody tr").addClass("d-none");
 
         var num = $target.text();
-        var min = (this.limit * (num-1))-1;
-        if (min === -1){
-            this.$questionsEl.find('tbody tr:lt('+ this.limit * num +')')
-                .removeClass('d-none');
+        var min = this.limit * (num - 1) - 1;
+        if (min === -1) {
+            this.$questionsEl
+                .find("tbody tr:lt(" + this.limit * num + ")")
+                .removeClass("d-none");
         } else {
-            this.$questionsEl.find('tbody tr:lt('+ this.limit * num +'):gt(' + min + ')')
-                .removeClass('d-none');
+            this.$questionsEl
+                .find("tbody tr:lt(" + this.limit * num + "):gt(" + min + ")")
+                .removeClass("d-none");
         }
+    },
 
+    /**
+     * @private
+     * @param {MouseEvent} ev
+     */
+    _onShowAllAnswers: function (ev) {
+        const btnEl = ev.currentTarget;
+        const pager = btnEl.previousElementSibling;
+        btnEl.classList.add("d-none");
+        this.$questionsEl.find("tbody tr").removeClass("d-none");
+        pager.classList.add("d-none");
+        this.$questionsEl.parent().addClass("h-auto");
     },
 });
 
@@ -75,9 +90,6 @@ publicWidget.registry.SurveyResultPagination = publicWidget.Widget.extend({
  *
  */
 publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
-    jsLibs: [
-        '/web/static/lib/Chart/Chart.js',
-    ],
 
     //--------------------------------------------------------------------------
     // Widget
@@ -113,12 +125,15 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                         self.chartConfig = self._getSectionResultsChartConfig();
                         break;
                 }
-
-                self._loadChart();
+                self.chart = self._loadChart();
             }
         });
     },
 
+    willStart: async function () {
+        await loadBundle("web.chartjs_lib");
+    },
+
     // -------------------------------------------------------------------------
     // Private
     // -------------------------------------------------------------------------
@@ -146,23 +161,33 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
             },
             options: {
                 scales: {
-                    xAxes: [{
+                    x: {
                         ticks: {
-                            callback: this._customTick(25),
+                            callback: function (val, index) {
+                                // For a category axis, the val is the index so the lookup via getLabelForValue is needed
+                                const value = this.getLabelForValue(val);
+                                const tickLimit = 25;
+                                return value?.length > tickLimit
+                                    ? `${value.slice(0, tickLimit)}...`
+                                    : value;
+                            },
                         },
-                    }],
-                    yAxes: [{
+                    },
+                    y: {
                         ticks: {
                             precision: 0,
                         },
-                    }],
+                        beginAtZero: true,
+                    },
                 },
-                tooltips: {
-                    callbacks: {
-                        title: function (tooltipItem, data) {
-                            return data.labels[tooltipItem[0].index];
-                        }
-                    }
+                plugins: {
+                    tooltip: {
+                        callbacks: {
+                            title: function (tooltipItem) {
+                                return tooltipItem.label;
+                            },
+                        },
+                    },
                 },
             },
         };
@@ -192,24 +217,34 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                 })
             },
             options: {
-                legend: {
-                    display: false,
+                plugins: {
+                    legend: {
+                        display: false,
+                    },
+                    tooltip: {
+                        enabled: false,
+                    },
                 },
                 scales: {
-                    xAxes: [{
+                    x: {
                         ticks: {
-                            callback: this._customTick(35),
+                            callback: function (val, index) {
+                                // For a category axis, the val is the index so the lookup via getLabelForValue is needed
+                                const value = this.getLabelForValue(val);
+                                const tickLimit = 35;
+                                return value?.length > tickLimit
+                                    ? `${value.slice(0, tickLimit)}...`
+                                    : value;
+                            },
                         },
-                    }],
-                    yAxes: [{
+                    },
+                    y: {
                         ticks: {
-                                precision: 0,
+                            precision: 0,
                         },
-                    }],
+                        beginAtZero: true,
+                    },
                 },
-                tooltips: {
-                    enabled: false,
-                }
             },
         };
     },
@@ -235,7 +270,10 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                         return D3_COLORS[index % 20];
                     }),
                 }]
-            }
+            },
+            options: {
+                aspectRatio: 2,
+            },
         };
     },
 
@@ -259,10 +297,13 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                 }]
             },
             options: {
-                title: {
-                    display: true,
-                    text: _t("Overall Performance"),
+                plugins: {
+                    title: {
+                        display: true,
+                        text: _t("Overall Performance"),
+                    },
                 },
+                aspectRatio: 2,
             }
         };
     },
@@ -323,20 +364,38 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                 datasets: datasets
             },
             options: {
-                title: {
-                    display: true,
-                    text: _t("Performance by Section"),
-                },
-                legend: {
-                    display: true,
+                plugins: {
+                    title: {
+                        display: true,
+                        text: _t("Performance by Section"),
+                    },
+                    legend: {
+                        display: true,
+                    },
+                    tooltip: {
+                        callbacks: {
+                            label: (tooltipItem) => {
+                                const xLabel = tooltipItem.label;
+                                var roundedValue = Math.round(tooltipItem.parsed.y * 100) / 100;
+                                return `${xLabel}: ${roundedValue}%`;
+                            },
+                        },
+                    },
                 },
                 scales: {
-                    xAxes: [{
+                    x: {
                         ticks: {
-                            callback: this._customTick(20),
+                            callback: function (val, index) {
+                                // For a category axis, the val is the index so the lookup via getLabelForValue is needed
+                                const value = this.getLabelForValue(val);
+                                const tickLimit = 20;
+                                return value?.length > tickLimit
+                                    ? `${value.slice(0, tickLimit)}...`
+                                    : value;
+                            },
                         },
-                    }],
-                    yAxes: [{
+                    },
+                    y: {
                         gridLines: {
                             display: false,
                         },
@@ -345,42 +404,18 @@ publicWidget.registry.SurveyResultChart = publicWidget.Widget.extend({
                             callback: function (label) {
                                 return label + '%';
                             },
-                            suggestedMin: 0,
-                            suggestedMax: 100,
                             maxTicksLimit: 5,
-                            stepSize: 25
+                            stepSize: 25,
                         },
-                    }],
+                        beginAtZero: true,
+                        suggestedMin: 0,
+                        suggestedMax: 100,
+                    },
                 },
-                tooltips: {
-                    callbacks: {
-                        label: function (tooltipItem, data) {
-                            var datasetLabel = data.datasets[tooltipItem.datasetIndex].label || '';
-                            var roundedValue = Math.round(tooltipItem.yLabel * 100) / 100;
-                            return `${datasetLabel}: ${roundedValue}%`;
-                        }
-                    }
-                }
             },
         };
     },
 
-    /**
-     * Custom Tick function to replace overflowing text with '...'
-     *
-     * @private
-     * @param {Integer} tickLimit
-     */
-    _customTick: function (tickLimit) {
-        return function (label) {
-            if (label.length <= tickLimit) {
-                return label;
-            } else {
-                return label.slice(0, tickLimit) + '...';
-            }
-        };
-    },
-
     /**
      * Loads the chart using the provided Chart library.
      *
@@ -409,28 +444,22 @@ publicWidget.registry.SurveyResultWidget = publicWidget.Widget.extend({
     selector: '.o_survey_result',
     events: {
         'click .o_survey_results_topbar_clear_filters': '_onClearFiltersClick',
-        'click i.filter-add-answer': '_onFilterAddAnswerClick',
+        'click .filter-add-answer': '_onFilterAddAnswerClick',
         'click i.filter-remove-answer': '_onFilterRemoveAnswerClick',
         'click a.filter-finished-or-not': '_onFilterFinishedOrNotClick',
         'click a.filter-finished': '_onFilterFinishedClick',
         'click a.filter-failed': '_onFilterFailedClick',
         'click a.filter-passed': '_onFilterPassedClick',
         'click a.filter-passed-and-failed': '_onFilterPassedAndFailedClick',
+        'click .o_survey_answer_image': '_onAnswerImgClick',
+        "click .o_survey_results_print": "_onPrintResultsClick",
+        "click .o_survey_results_data_tab": "_onDataViewChange",
     },
 
     //--------------------------------------------------------------------------
     // Widget
     //--------------------------------------------------------------------------
 
-    /**
-    * @override
-    */
-    willStart: function () {
-        var url = '/web/webclient/locale/' + (document.documentElement.getAttribute('lang') || 'en_US').replace('-', '_');
-        var localeReady = loadJS(url);
-        return Promise.all([this._super.apply(this, arguments), localeReady]);
-    },
-
     /**
     * @override
     */
@@ -438,19 +467,22 @@ publicWidget.registry.SurveyResultWidget = publicWidget.Widget.extend({
         var self = this;
         return this._super.apply(this, arguments).then(function () {
             var allPromises = [];
-
-            self.$('.pagination').each(function (){
+            self.$('.pagination_wrapper').each(function (){
                 var questionId = $(this).data("question_id");
                 allPromises.push(new publicWidget.registry.SurveyResultPagination(self, {
                     'questionsEl': self.$('#survey_table_question_'+ questionId)
                 }).attachTo($(this)));
             });
-
             self.$('.survey_graph').each(function () {
                 allPromises.push(new publicWidget.registry.SurveyResultChart(self)
                     .attachTo($(this)));
             });
 
+            // Set the size of results tables so that they do not resize when switching pages.
+            document.querySelectorAll('.o_survey_results_table_wrapper').forEach((table) => {
+                table.style.height = table.clientHeight + 'px';
+            })
+
             if (allPromises.length !== 0) {
                 return Promise.all(allPromises);
             } else {
@@ -463,7 +495,20 @@ publicWidget.registry.SurveyResultWidget = publicWidget.Widget.extend({
     // Handlers
     // -------------------------------------------------------------------------
 
-     /**
+    /**
+     * Recompute the table height as the table could have been hidden when its height was initially computed (see 'start').
+     * @private
+     * @param {Event} ev
+     */
+    _onDataViewChange: function (ev) {
+        const tableWrapper = document.querySelector(`div[id="${ev.currentTarget.getAttribute('aria-controls')}"] .o_survey_results_table_wrapper`);
+        if (tableWrapper) {
+            tableWrapper.style.height = 'auto';
+            tableWrapper.style.height = tableWrapper.clientHeight + 'px';
+        }
+    },
+
+    /**
      * Add an answer filter by updating the URL and redirecting.
      * @private
      * @param {Event} ev
@@ -557,25 +602,55 @@ publicWidget.registry.SurveyResultWidget = publicWidget.Widget.extend({
     },
 
     /**
-     * Returns the modified pathname string for filters after adding or removing an
-     * answer filter (from click event). Filters are formatted as `"rowX,ansX", where
-     * the row is used for matrix-type questions and set to 0 otherwise.
+     * Called when an image on an answer in multi-answers question is clicked.
+     * Starts a widget opening a dialog to display the now zoomable image.
+     * this.imgZoomer is the zoomer widget linked to the survey result widget, if any.
+     *
      * @private
-     * @param {String} filters Existing answer filters, formatted as `rowX,ansX|rowY,ansY...`.
+     * @param {Event} ev
+     */
+    _onAnswerImgClick: function (ev) {
+        ev.preventDefault();
+        new SurveyImageZoomer({
+            sourceImage: $(ev.currentTarget).attr('src')
+        }).appendTo(document.body);
+    },
+
+    /**
+     * Call print dialog
+     * @private
+     */
+    _onPrintResultsClick: function () {
+        window.print();
+    },
+
+    /**
+     * Returns the modified pathname string for filters after adding or removing an
+     * answer filter (from click event).
+     * @private
+     * @param {String} filters Existing answer filters, formatted as
+     * `modelX,rowX,ansX|modelY,rowY,ansY...` - row is used for matrix-type questions row id, 0 for others
+     * "model" specifying the model to query depending on the question type we filter on.
+       - 'A': 'survey.question.answer' ids: simple_choice, multiple_choice, matrix
+       - 'L': 'survey.user_input.line' ids: char_box, text_box, numerical_box, date, datetime
      * @param {"add" | "remove"} operation Whether to add or remove the filter.
      * @param {Event} ev Event defining the filter.
      * @returns {String} Updated filters.
      */
     _prepareAnswersFilters(filters, operation, ev) {
-        const cell = $(ev.target);
-        const eventFilter = `${cell.data('rowId') || 0},${cell.data('answerId')}`;
+        const cellDataset = ev.currentTarget.dataset;
+        const filter = `${cellDataset.modelShortKey},${cellDataset.rowId || 0},${cellDataset.recordId}`;
 
         if (operation === 'add') {
-            filters = filters ? filters + `|${eventFilter}` : eventFilter;
+            if (filters) {
+                filters = !filters.split("|").includes(filter) ? filters += `|${filter}` : filters;
+            } else {
+                filters = filter;
+            }
         } else if (operation === 'remove') {
             filters = filters
                 .split("|")
-                .filter(filterItem => filterItem !== eventFilter)
+                .filter(filterItem => filterItem !== filter)
                 .join("|");
         } else {
             throw new Error('`operation` parameter for `_prepareAnswersFilters` must be either "add" or "remove".')
@@ -584,10 +659,8 @@ publicWidget.registry.SurveyResultWidget = publicWidget.Widget.extend({
     }
 });
 
-return {
+export default {
     resultWidget: publicWidget.registry.SurveyResultWidget,
     chartWidget: publicWidget.registry.SurveyResultChart,
     paginationWidget: publicWidget.registry.SurveyResultPagination
 };
-
-});
