@@ -1,15 +1,22 @@
-# -*- coding: utf-8 -*-
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import test_invite
 from . import test_ir_actions
+from . import test_ir_attachment
 from . import test_mail_activity
+from . import test_mail_activity_plan
+from . import test_mail_alias
 from . import test_mail_composer
 from . import test_mail_composer_mixin
 from . import test_mail_followers
-from . import test_mail_message
-from . import test_mail_mail
 from . import test_mail_gateway
+from . import test_mail_mail
+from . import test_mail_management
+from . import test_mail_message
 from . import test_mail_multicompany
+from . import test_mail_push
+from . import test_mail_scheduled_message
+from . import test_mail_security
 from . import test_mail_thread_internals
 from . import test_mail_thread_mixins
 from . import test_mail_template
@@ -18,6 +25,4 @@ from . import test_message_management
 from . import test_message_post
 from . import test_message_track
 from . import test_performance
-from . import test_ui
-from . import test_mail_management
-from . import test_mail_security
+from . import test_public_binary_controller
