@@ -9,21 +9,23 @@ from unittest.mock import patch
 from unittest.mock import DEFAULT
 
 import pytz
+import random
 
 from odoo import fields, exceptions, tests
-from odoo.addons.mail.tests.common import mail_new_test_user
-from odoo.addons.test_mail.tests.common import TestMailCommon
+from odoo.addons.mail.tests.common import mail_new_test_user, MailCommon
 from odoo.addons.test_mail.models.test_mail_models import MailTestActivity
+from odoo.tests import Form, HttpCase, users
 from odoo.tools import mute_logger
-from odoo.tests.common import Form, users
 
 
-class TestActivityCommon(TestMailCommon):
+class TestActivityCommon(MailCommon):
 
     @classmethod
     def setUpClass(cls):
         super(TestActivityCommon, cls).setUpClass()
-        cls.test_record = cls.env['mail.test.activity'].with_context(cls._test_context).create({'name': 'Test'})
+        cls.test_record, cls.test_record_2 = cls.env['mail.test.activity'].with_context(cls._test_context).create([
+            {'name': 'Test'}, {'name': 'Test_2'},
+        ])
         # reset ctx
         cls._reset_mail_context(cls.test_record)
 
@@ -48,14 +50,13 @@ class TestActivityRights(TestActivityCommon):
 
     @mute_logger('odoo.addons.mail.models.mail_mail')
     def test_activity_security_user_noaccess_automated(self):
-        def _employee_crash(*args, **kwargs):
+        def _employee_crash(records, operation):
             """ If employee is test employee, consider they have no access on document """
-            recordset = args[0]
-            if recordset.env.uid == self.user_employee.id:
-                raise exceptions.AccessError('Hop hop hop Ernest, please step back.')
+            if records.env.uid == self.user_employee.id and not records.env.su:
+                return records, lambda: exceptions.AccessError('Hop hop hop Ernest, please step back.')
             return DEFAULT
 
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             activity = self.test_record.activity_schedule(
                 'test_mail.mail_act_test_todo',
                 user_id=self.user_employee.id)
@@ -64,10 +65,9 @@ class TestActivityRights(TestActivityCommon):
             activity2.write({'user_id': self.user_employee.id})
 
     def test_activity_security_user_noaccess_manual(self):
-        def _employee_crash(*args, **kwargs):
+        def _employee_crash(records, operation):
             """ If employee is test employee, consider they have no access on document """
-            recordset = args[0]
-            if recordset.env.uid == self.user_employee.id:
+            if records.env.uid == self.user_employee.id and not records.env.su:
                 raise exceptions.AccessError('Hop hop hop Ernest, please step back.')
             return DEFAULT
 
@@ -82,13 +82,13 @@ class TestActivityRights(TestActivityCommon):
 
         # can _search activities if access to the document
         self.env['mail.activity'].with_user(self.user_employee)._search(
-            [('id', '=', test_activity.id)], count=False)
+            [('id', '=', test_activity.id)])
 
         # cannot _search activities if no access to the document
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             with self.assertRaises(exceptions.AccessError):
                 searched_activity = self.env['mail.activity'].with_user(self.user_employee)._search(
-                    [('id', '=', test_activity.id)], count=False)
+                    [('id', '=', test_activity.id)])
 
         # can read_group activities if access to the document
         read_group_result = self.env['mail.activity'].with_user(self.user_employee).read_group(
@@ -100,7 +100,7 @@ class TestActivityRights(TestActivityCommon):
         self.assertEqual('Summary', read_group_result[0]['summary'])
 
         # cannot read_group activities if no access to the document
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             with self.assertRaises(exceptions.AccessError):
                 self.env['mail.activity'].with_user(self.user_employee).read_group(
                     [('id', '=', test_activity.id)],
@@ -109,36 +109,55 @@ class TestActivityRights(TestActivityCommon):
                 )
 
         # cannot read activities if no access to the document
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             with self.assertRaises(exceptions.AccessError):
                 searched_activity = self.env['mail.activity'].with_user(self.user_employee).search(
                     [('id', '=', test_activity.id)])
                 searched_activity.read(['summary'])
 
         # cannot search_read activities if no access to the document
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             with self.assertRaises(exceptions.AccessError):
                 self.env['mail.activity'].with_user(self.user_employee).search_read(
                     [('id', '=', test_activity.id)],
                     ['summary'])
 
-        # cannot create activities for people that cannot access record
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
-            with self.assertRaises(exceptions.UserError):
-                activity = self.env['mail.activity'].create({
-                    'activity_type_id': self.env.ref('test_mail.mail_act_test_todo').id,
-                    'res_model_id': self.env.ref('test_mail.model_mail_test_activity').id,
-                    'res_id': self.test_record.id,
-                    'user_id': self.user_employee.id,
-                })
+        # can create activities for people that cannot access record
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
+            self.env['mail.activity'].create({
+                'activity_type_id': self.env.ref('test_mail.mail_act_test_todo').id,
+                'res_model_id': self.env.ref('test_mail.model_mail_test_activity').id,
+                'res_id': self.test_record.id,
+                'user_id': self.user_employee.id,
+            })
 
         # cannot create activities if no access to the document
-        with patch.object(MailTestActivity, 'check_access_rights', autospec=True, side_effect=_employee_crash):
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
             with self.assertRaises(exceptions.AccessError):
                 activity = self.test_record.with_user(self.user_employee).activity_schedule(
                     'test_mail.mail_act_test_todo',
                     user_id=self.user_admin.id)
 
+        test_activity.user_id = self.user_employee
+        test_activity.flush_recordset()
+
+        # user can read activities assigned to him even if he has no access to the document
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
+            found = self.env['mail.activity'].with_user(self.user_employee).search(
+                [('id', '=', test_activity.id)])
+            self.assertEqual(found, test_activity)
+            found.read(['summary'])
+
+        # user can read_group activities assigned to him even if he has no access to the document
+        with patch.object(MailTestActivity, '_check_access', autospec=True, side_effect=_employee_crash):
+            read_group_result = self.env['mail.activity'].with_user(self.user_employee).read_group(
+                [('id', '=', test_activity.id)],
+                ['summary'],
+                ['summary'],
+            )
+            self.assertEqual(1, read_group_result[0]['summary_count'])
+            self.assertEqual('Summary', read_group_result[0]['summary'])
+
 
 @tests.tagged('mail_activity')
 class TestActivityFlow(TestActivityCommon):
@@ -284,7 +303,7 @@ class TestActivityMixin(TestActivityCommon):
             name='user Australia',
             login='user Australia',
         )
-        cls.user_australia.tz = 'Australia/ACT'
+        cls.user_australia.tz = 'Australia/Sydney'
 
     @mute_logger('odoo.addons.mail.models.mail_mail')
     def test_activity_mixin(self):
@@ -341,6 +360,7 @@ class TestActivityMixin(TestActivityCommon):
                 user_id=self.user_admin.id,
                 feedback='Test feedback',)
             self.assertEqual(self.test_record.activity_ids, act2 | act3)
+            self.assertFalse(act1.exists())
 
             # Reschedule all activities, should update the record state
             self.assertEqual(self.test_record.activity_state, 'overdue')
@@ -354,6 +374,7 @@ class TestActivityMixin(TestActivityCommon):
             self.test_record.activity_feedback(
                 ['test_mail.mail_act_test_todo'],
                 feedback='Test feedback')
+            self.assertFalse(act3.exists())
 
             # Setting activities as done should delete them and post messages
             self.assertEqual(self.test_record.activity_ids, act2)
@@ -366,6 +387,8 @@ class TestActivityMixin(TestActivityCommon):
             # Canceling activities should simply remove them
             self.assertEqual(self.test_record.activity_ids, self.env['mail.activity'])
             self.assertEqual(len(self.test_record.message_ids), 2)
+            self.assertFalse(self.test_record.activity_state)
+            self.assertFalse(act2.exists())
 
     @mute_logger('odoo.addons.mail.models.mail_mail')
     def test_activity_mixin_archive(self):
@@ -381,6 +404,29 @@ class TestActivityMixin(TestActivityCommon):
         self.assertEqual(rec.active, True)
         self.assertEqual(rec.activity_ids, self.env['mail.activity'])
 
+    @mute_logger('odoo.addons.mail.models.mail_mail')
+    def test_activity_mixin_archive_user(self):
+        """
+        Test when archiving an user, we unlink all his related activities
+        """
+        test_users = self.env['res.users']
+        for i in range(5):
+            test_users += mail_new_test_user(self.env, name=f'test_user_{i}', login=f'test_password_{i}')
+        for user in test_users:
+            self.test_record.activity_schedule(user_id=user.id)
+        archived_users = self.env['res.users'].browse(map(lambda x: x.id, random.sample(test_users, 2)))  # pick 2 users to archive
+        archived_users.action_archive()
+        active_users = test_users - archived_users
+
+        activities = self.env['mail.activity'].search([('user_id', 'in', archived_users.ids)])
+        self.assertFalse(activities, "Activities of archived users should be deleted.")
+
+        # activities of active users shouldn't be touched, each has exactly 1 activity present
+        activities = self.env['mail.activity'].search([('user_id', 'in', active_users.ids)])
+        self.assertEqual(len(activities), 3, "We should have only 3 activities in total linked to our active users")
+        self.assertEqual(activities.mapped('user_id'), active_users,
+                         "We should have 3 different users linked to the activities of the active users")
+
     @mute_logger('odoo.addons.mail.models.mail_mail')
     def test_activity_mixin_reschedule_user(self):
         rec = self.test_record.with_user(self.user_employee)
@@ -530,6 +576,22 @@ class TestActivityMixin(TestActivityCommon):
             self.assertEqual(activity_2.state, 'overdue')
             self.assertEqual(activity_3.state, 'today')
 
+    @users('employee')
+    def test_mail_activity_mixin_search_activity_user_id_false(self):
+        """Test the search method on the "activity_user_id" when searching for non-set user"""
+        MailTestActivity = self.env['mail.test.activity']
+        test_records = self.test_record | self.test_record_2
+        self.assertFalse(test_records.activity_ids)
+        self.assertEqual(MailTestActivity.search([('activity_user_id', '=', False)]), test_records)
+
+        self.env['mail.activity'].create({
+            'summary': 'Test',
+            'activity_type_id': self.env.ref('test_mail.mail_act_test_todo').id,
+            'res_model_id': self.env.ref('test_mail.model_mail_test_activity').id,
+            'res_id': self.test_record.id,
+        })
+        self.assertEqual(MailTestActivity.search([('activity_user_id', '!=', True)]), self.test_record_2)
+
     @mute_logger('odoo.addons.mail.models.mail_mail', 'odoo.tests')
     def test_mail_activity_mixin_search_state_basic(self):
         """Test the search method on the "activity_state".
@@ -552,12 +614,14 @@ class TestActivityMixin(TestActivityCommon):
         ])
 
         origin_1, origin_2 = self.env['mail.test.activity'].search([], limit=2)
+        activity_type = self.env.ref('test_mail.mail_act_test_todo')
+        activity_type.sudo().keep_done = True
 
         with patch('odoo.addons.mail.models.mail_activity.datetime', MockedDatetime), \
             patch('odoo.addons.mail.models.mail_activity_mixin.datetime', MockedDatetime):
             origin_1_activity_1 = self.env['mail.activity'].create({
                 'summary': 'Test',
-                'activity_type_id': 1,
+                'activity_type_id': activity_type.id,
                 'res_model_id': self.env.ref('test_mail.model_mail_test_activity').id,
                 'res_id': origin_1.id,
                 'date_deadline': today_utc,
@@ -575,7 +639,7 @@ class TestActivityMixin(TestActivityCommon):
 
             origin_2_activity_1 = self.env['mail.activity'].create({
                 'summary': 'Test',
-                'activity_type_id': 1,
+                'activity_type_id': activity_type.id,
                 'res_model_id': self.env.ref('test_mail.model_mail_test_activity').id,
                 'res_id': origin_2.id,
                 'date_deadline': today_utc + relativedelta(hours=8),
@@ -626,6 +690,22 @@ class TestActivityMixin(TestActivityCommon):
             self.assertTrue(len(result) > 0)
             self.assertEqual(result, all_activity_mixin_record.filtered(lambda p: p.activity_state in ('today', False)))
 
+            # Check that activity done are not taken into account by group and search by activity_state.
+            Model = self.env['mail.test.activity']
+            search_params = {
+                'domain': [('id', 'in', (origin_1 | origin_2).ids), ('activity_state', '=', 'overdue')]}
+            read_group_params = {'domain': [('id', 'in', (origin_1 | origin_2).ids)], 'fields': ['id:array_agg'],
+                                 'groupby': ['activity_state']}
+            self.assertEqual(Model.search(**search_params), origin_1)
+            self.assertEqual(
+                {(e['activity_state'], e['activity_state_count']) for e in Model.read_group(**read_group_params)},
+                {('today', 1), ('overdue', 1)})
+            origin_1_activity_2.action_feedback(feedback='Done')
+            self.assertFalse(Model.search(**search_params))
+            self.assertEqual(
+                {(e['activity_state'], e['activity_state_count']) for e in Model.read_group(**read_group_params)},
+                {('today', 2)})
+
     @mute_logger('odoo.addons.mail.models.mail_mail', 'odoo.tests')
     def test_mail_activity_mixin_search_state_different_day_but_close_time(self):
         """Test the case where there's less than 24 hours between the deadline and now_tz,
@@ -694,59 +774,239 @@ class TestActivityMixin(TestActivityCommon):
             self.assertEqual(test_record_1, record)
 
 
-class TestReadProgressBar(tests.TransactionCase):
+@tests.tagged("mail_activity")
+class TestActivitySystray(TestActivityCommon, HttpCase):
+    """Test for systray_get_activities"""
+
+    @freeze_time("2024-01-15 14:00:00 UTC")
+    @users("employee")
+    def test_systray_activities_for_archived_records(self):
+        """Check that activities made on archived records are shown in the
+        systray activities."""
+        self.test_record.action_archive()
+        self.test_record.activity_schedule(
+            "test_mail.mail_act_test_todo",
+            user_id=self.user_employee.id,
+        )
+        self.authenticate(self.user_employee.login, self.user_employee.login)
+        data = self.make_jsonrpc_request("/mail/data", {"systray_get_activities": True})
+        total_count = sum(
+            record["total_count"]
+            for record in data["Store"]["activityGroups"]
+            if record.get("model") == self.test_record._name
+        )
+        self.assertEqual(total_count, 1)
+
+
+@tests.tagged('mail_activity')
+class TestActivityViewHelpers(TestActivityCommon):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.type_todo = cls.env.ref('test_mail.mail_act_test_todo')
+        cls.type_call = cls.env.ref('test_mail.mail_act_test_call')
+        cls.type_upload = cls.env.ref('test_mail.mail_act_test_upload_document')
+        cls.user_employee_2 = mail_new_test_user(
+            cls.env,
+            name='Employee2',
+            login='employee2',
+        )
+        cls.attachment_1, cls.attachment_2 = cls.env['ir.attachment'].create([{
+            'name': f"Uploaded doc_{idx + 1}",
+            'raw': b'bar',
+            'res_model': cls.test_record_2._name,
+            'res_id': cls.test_record_2.id,
+        } for idx in range(2)])
+        cls.upload_type = cls.env.ref('test_mail.mail_act_test_upload_document')
+        cls.upload_type.sudo().keep_done = True
+        cls.user_employee.tz = cls.user_admin.tz
+
+    @freeze_time("2023-10-18 06:00:00")
+    def test_get_activity_data(self):
+        get_activity_data = self.env['mail.activity'].get_activity_data
+        with self.with_user('employee'):
+            # Setup activities: 3 for the first record, 2 "done" and 2 ongoing for the second
+            test_record, test_record_2 = self.env['mail.test.activity'].browse(
+                (self.test_record + self.test_record_2).ids
+            )
+            now_utc = datetime.now(pytz.UTC)
+            now_user = now_utc.astimezone(pytz.timezone(self.env.user.tz or 'UTC'))
+            today_user = now_user.date()
+
+            for days, user_id in ((-1, self.user_employee_2), (0, self.user_employee), (1, self.user_admin)):
+                test_record.activity_schedule(
+                    'test_mail.mail_act_test_upload_document',
+                    today_user + relativedelta(days=days),
+                    user_id=user_id.id)
+            for days, user_id in ((-2, self.user_admin), (0, self.user_employee), (2, self.user_employee_2),
+                                  (3, self.user_admin)):
+                test_record_2.activity_schedule(
+                    'test_mail.mail_act_test_upload_document',
+                    today_user + relativedelta(days=days),
+                    user_id=user_id.id)
+            record_activities = test_record.activity_ids
+            record_2_activities = test_record_2.activity_ids
+            record_2_activities[0].action_feedback(feedback='Done', attachment_ids=self.attachment_1.ids)
+            record_2_activities[1].action_feedback(feedback='Done', attachment_ids=self.attachment_2.ids)
+
+            # Check get activity data
+            activity_data = get_activity_data('mail.test.activity', None, fetch_done=True)
+            self.assertEqual(activity_data['activity_res_ids'], [test_record.id, test_record_2.id])
+            self.assertDictEqual(
+                next((t for t in activity_data['activity_types'] if t['id'] == self.upload_type.id), {}),
+                {
+                    'id': self.upload_type.id,
+                    'name': 'Upload Document',
+                    'template_ids': [],
+                    'keep_done': True,
+                })
+
+            grouped = activity_data['grouped_activities'][test_record.id][self.upload_type.id]
+            grouped['ids'] = set(grouped['ids'])  # ids order doesn't matter
+            self.assertDictEqual(grouped, {
+                'state': 'overdue',
+                'count_by_state': {'overdue': 1, 'planned': 1, 'today': 1},
+                'ids': set(record_activities.ids),
+                'reporting_date': record_activities[0].date_deadline,
+                'user_assigned_ids': record_activities.user_id.ids,
+            })
+
+            grouped = activity_data['grouped_activities'][test_record_2.id][self.upload_type.id]
+            grouped['ids'] = set(grouped['ids'])
+            self.assertDictEqual(grouped, {
+                'state': 'planned',
+                'count_by_state': {'done': 2, 'planned': 2},
+                'ids': set(record_2_activities.ids),
+                'reporting_date': record_2_activities[2].date_deadline,
+                'user_assigned_ids': record_2_activities[2:].user_id.ids,
+                'attachments_info': {
+                    'count': 2, 'most_recent_id': self.attachment_2.id, 'most_recent_name': 'Uploaded doc_2'}
+            })
+
+            # Mark all first record activities as "done" and check activity data
+            record_activities.action_feedback(feedback='Done', attachment_ids=self.attachment_1.ids)
+            self.assertEqual(record_activities[2].date_done, date.today())  # Thanks to freeze_time
+            activity_data = get_activity_data('mail.test.activity', None, fetch_done=True)
+            grouped = activity_data['grouped_activities'][test_record.id][self.upload_type.id]
+            grouped['ids'] = set(grouped['ids'])
+            self.assertDictEqual(grouped, {
+                'state': 'done',
+                'count_by_state': {'done': 3},
+                'ids': set(record_activities.ids),
+                'reporting_date': record_activities[2].date_done,
+                'user_assigned_ids': [],
+                'attachments_info': {
+                    'count': 1,  # 1 instead of 3 because all attachments are the same one
+                    'most_recent_id': self.attachment_1.id,
+                    'most_recent_name': self.attachment_1.name,
+                }
+            })
+            self.assertEqual(activity_data['activity_res_ids'], [test_record_2.id, test_record.id])
+
+            # Check filters (domain, pagination and fetch_done)
+            self.assertEqual(
+                get_activity_data('mail.test.activity', domain=[('id', 'in', test_record.ids)],
+                                  fetch_done=True)['activity_res_ids'],
+                [test_record.id])
+            self.assertEqual(get_activity_data('mail.test.activity', None, fetch_done=False)['activity_res_ids'],
+                             [test_record_2.id])
+            # Note that the records are ordered by ids not by deadline (so we get the "wrong" order)
+            self.assertEqual(
+                get_activity_data('mail.test.activity', None, offset=1, fetch_done=True)['activity_res_ids'],
+                [test_record_2.id])
+            self.assertEqual(
+                get_activity_data('mail.test.activity', None, limit=1, fetch_done=True)['activity_res_ids'],
+                [test_record.id])
+
+            # Unset keep done and check activity data: record with only "done" activities must not be returned
+            self.upload_type.sudo().keep_done = False
+            activity_data = get_activity_data('mail.test.activity', None, fetch_done=True)
+            self.assertDictEqual(
+                next((t for t in activity_data['activity_types'] if t['id'] == self.upload_type.id), {}),
+                {
+                    'id': self.upload_type.id,
+                    'name': 'Upload Document',
+                    'template_ids': [],
+                    'keep_done': False,
+                })
+            self.assertEqual(activity_data['activity_res_ids'], [test_record_2.id])
+
+            # Unarchiving activities should restore the activity
+            record_activities.action_unarchive()
+            self.assertFalse(any(act.date_done for act in record_activities))
+            self.assertTrue(all(act.date_deadline for act in record_activities))
+            self.upload_type.sudo().keep_done = True
+            activity_data = get_activity_data('mail.test.activity', None, fetch_done=True)
+            grouped = activity_data['grouped_activities'][test_record.id][self.upload_type.id]
+            self.assertEqual(grouped['state'], 'overdue')
+            self.assertEqual(grouped['count_by_state'], {'overdue': 1, 'planned': 1, 'today': 1})
+            self.assertEqual(grouped['reporting_date'], record_activities[0].date_deadline)
+            self.assertEqual(activity_data['activity_res_ids'], [test_record.id, test_record_2.id])
+            grouped['ids'] = set(grouped['ids'])
+            self.assertDictEqual(grouped, {
+                'state': 'overdue',
+                'count_by_state': {'overdue': 1, 'planned': 1, 'today': 1},
+                'ids': set(record_activities.ids),
+                'reporting_date': record_activities[0].date_deadline,
+                'user_assigned_ids': record_activities.user_id.ids,
+            })
+
+
+@tests.tagged('mail_activity')
+class TestORM(TestActivityCommon):
     """Test for read_progress_bar"""
 
     def test_week_grouping(self):
         """The labels associated to each record in read_progress_bar should match
         the ones from read_group, even in edge cases like en_US locale on sundays
         """
-        model = self.env['mail.test.activity'].with_context(lang='en_US')
+        MailTestActivityCtx = self.env['mail.test.activity'].with_context({"lang": "en_US"})
 
         # Don't mistake fields date and date_deadline:
         # * date is just a random value
         # * date_deadline defines activity_state
-        model.create({
-            'date': '2021-05-02',
-            'name': "Yesterday, all my troubles seemed so far away",
-        }).activity_schedule(
-            'test_mail.mail_act_test_todo',
-            summary="Make another test super asap (yesterday)",
-            date_deadline=fields.Date.context_today(model) - timedelta(days=7),
-        )
-        model.create({
-            'date': '2021-05-09',
-            'name': "Things we said today",
-        }).activity_schedule(
-            'test_mail.mail_act_test_todo',
-            summary="Make another test asap",
-            date_deadline=fields.Date.context_today(model),
-        )
-        model.create({
-            'date': '2021-05-16',
-            'name': "Tomorrow Never Knows",
-        }).activity_schedule(
-            'test_mail.mail_act_test_todo',
-            summary="Make a test tomorrow",
-            date_deadline=fields.Date.context_today(model) + timedelta(days=7),
-        )
+        with freeze_time("2024-09-24 10:00:00"):
+            self.env['mail.test.activity'].create({
+                'date': '2021-05-02',
+                'name': "Yesterday, all my troubles seemed so far away",
+            }).activity_schedule(
+                'test_mail.mail_act_test_todo',
+                summary="Make another test super asap (yesterday)",
+                date_deadline=fields.Date.context_today(MailTestActivityCtx) - timedelta(days=7),
+            )
+            self.env['mail.test.activity'].create({
+                'date': '2021-05-09',
+                'name': "Things we said today",
+            }).activity_schedule(
+                'test_mail.mail_act_test_todo',
+                summary="Make another test asap",
+                date_deadline=fields.Date.context_today(MailTestActivityCtx),
+            )
+            self.env['mail.test.activity'].create({
+                'date': '2021-05-16',
+                'name': "Tomorrow Never Knows",
+            }).activity_schedule(
+                'test_mail.mail_act_test_todo',
+                summary="Make a test tomorrow",
+                date_deadline=fields.Date.context_today(MailTestActivityCtx) + timedelta(days=7),
+            )
 
-        domain = [('date', "!=", False)]
-        groupby = "date:week"
-        progress_bar = {
-            'field': 'activity_state',
-            'colors': {
-                "overdue": 'danger',
-                "today": 'warning',
-                "planned": 'success',
+            domain = [('date', "!=", False)]
+            groupby = "date:week"
+            progress_bar = {
+                'field': 'activity_state',
+                'colors': {
+                    "overdue": 'danger',
+                    "today": 'warning',
+                    "planned": 'success',
+                }
             }
-        }
 
-        # call read_group to compute group names
-        groups = model.read_group(domain, fields=['date'], groupby=[groupby])
-        progressbars = model.read_progress_bar(domain, group_by=groupby, progress_bar=progress_bar)
-        self.assertEqual(len(groups), 3)
-        self.assertEqual(len(progressbars), 3)
+            # call read_group to compute group names
+            groups = MailTestActivityCtx.read_group(domain, fields=['date'], groupby=[groupby])
+            progressbars = MailTestActivityCtx.read_progress_bar(domain, group_by=groupby, progress_bar=progress_bar)
+            self.assertEqual(len(groups), 3)
+            self.assertEqual(len(progressbars), 3)
 
         # format the read_progress_bar result to get a dictionary under this
         # format: {activity_state: group_name}; the original format
@@ -759,3 +1019,63 @@ class TestReadProgressBar(tests.TransactionCase):
         self.assertEqual(groups[0][groupby], pg_groups["overdue"])
         self.assertEqual(groups[1][groupby], pg_groups["today"])
         self.assertEqual(groups[2][groupby], pg_groups["planned"])
+
+
+@tests.tagged('post_install', '-at_install')
+class TestTours(HttpCase):
+    def test_activity_view_data_with_offset(self):
+        self.patch(MailTestActivity, '_order', 'date desc, id desc')
+        MailTestActivityModel = self.env['mail.test.activity']
+        MailTestActivityCtx = MailTestActivityModel.with_context({"lang": "en_US"})
+        MailTestActivityModel.create({
+            'date': '2021-05-02',
+            'name': "Task 1",
+        }).activity_schedule(
+            'test_mail.mail_act_test_todo',
+            summary="Activity 1",
+            date_deadline=fields.Date.context_today(MailTestActivityCtx) - timedelta(days=7),
+        )
+        MailTestActivityModel.create({
+            'date': '2021-05-16',
+            'name': "Task 1 without activity",
+        })
+        MailTestActivityModel.create({
+            'date': '2021-05-09',
+            'name': "Task 2",
+        }).activity_schedule(
+            'test_mail.mail_act_test_todo',
+            summary="Activity 2",
+            date_deadline=fields.Date.context_today(MailTestActivityCtx),
+        )
+        MailTestActivityModel.create({
+            'date': '2021-05-16',
+            'name': "Task 3",
+        }).activity_schedule(
+            'test_mail.mail_act_test_todo',
+            summary="Activity 3",
+            date_deadline=fields.Date.context_today(MailTestActivityCtx) + timedelta(days=7),
+        )
+        MailTestActivityModel.create({
+            'date': '2021-05-16',
+            'name': "Task 2 without activity",
+        })
+
+        self.env["ir.ui.view"].create({
+            "name": "Test Activity View",
+            "model": "mail.test.activity",
+            "type": 'activity',
+            "arch": """
+                <activity string="OrderedMailTestActivity">
+                    <templates>
+                        <div t-name="activity-box">
+                            <field name="name"/>
+                        </div>
+                    </templates>
+                </activity>
+            """,
+        })
+        self.start_tour(
+            "/odoo?debug=1",
+            "mail_activity_view",
+            login="admin",
+        )
