@@ -11,21 +11,29 @@ and mail-related sub modules. Those tests are present in a separate module as it
 contains models used only to perform tests independently to functional aspects of
 real applications. """,
     'depends': [
-        'test_mail',
-        'test_mass_mailing',
         'mail',
         'mail_bot',
+        'portal',
+        'rating',
         # 'snailmail',
         'mass_mailing',
-        'mass_mailing_sms',
+        'mass_mailing_sms',  # adds portal
         'phone_validation',
         'sms',
+        'test_mail',
+        'test_mail_sms',
+        'test_mass_mailing',
     ],
     'data': [
+        'data/mail_message_subtype_data.xml',
         'security/ir.model.access.csv',
+        'security/ir_rule_data.xml',
     ],
-    'demo': [
-    ],
+    'assets': {
+        'web.assets_unit_tests': [
+            'test_mail_full/static/tests/**/*',
+        ],
+    },
     'installable': True,
-    'application': False,
+    'license': 'LGPL-3',
 }
