@@ -2,13 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.addons.phone_validation.tools import phone_validation
-from odoo.addons.sms.tests.common import SMSCommon
-from odoo.addons.test_mail.tests.common import TestMailCommon, TestRecipients
-
-
-class TestSMSCommon(SMSCommon, TestMailCommon):
-    """ Main entry point for functional tests. Kept to ease backward
-    compatibility and updating common. """
+from odoo.addons.test_mail.tests.common import TestRecipients
 
 
 class TestSMSRecipients(TestRecipients):
