@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.test_mail_sms.tests.common import TestSMSCommon, TestSMSRecipients
+from odoo.addons.sms.tests.common import SMSCommon
+from odoo.addons.test_mail_sms.tests.common import TestSMSRecipients
 
 
-class TestSMSComposerComment(TestSMSCommon, TestSMSRecipients):
+class TestSMSComposerComment(SMSCommon, TestSMSRecipients):
     """ TODO LIST
 
      * add test for default_res_model / default_res_id and stuff like that;
@@ -42,12 +43,11 @@ class TestSMSComposerComment(TestSMSCommon, TestSMSRecipients):
                 'numbers': ','.join(self.random_numbers),
             })
 
-            with self.mockSMSGateway():
+            with self.mockSMSGateway(sms_allow_unlink=True):
                 composer._action_send_sms()
 
-        # use sms.api directly, does not create sms.sms
-        self.assertNoSMS()
-        self.assertSMSIapSent(self.random_numbers_san, self._test_body)
+        for number in self.random_numbers_san:
+            self.assertSMS(self.env['res.partner'], number, 'pending', content=self._test_body, fields_values={'to_delete': True})
 
     def test_composer_comment_default(self):
         with self.with_user('employee'):
@@ -224,7 +224,7 @@ class TestSMSComposerComment(TestSMSCommon, TestSMSRecipients):
             'phone_nbr': False,
             'mobile_nbr': False,
         })
-        default_field_name = self.env['mail.test.sms']._sms_get_number_fields()[0]
+        default_field_name = self.env['mail.test.sms']._phone_get_number_fields()[0]
 
         with self.with_user('employee'):
             composer = self.env['sms.composer'].with_context(
@@ -258,12 +258,11 @@ class TestSMSComposerComment(TestSMSCommon, TestSMSRecipients):
                 'numbers': ','.join(self.random_numbers),
             })
 
-            with self.mockSMSGateway():
+            with self.mockSMSGateway(sms_allow_unlink=True):
                 composer._action_send_sms()
 
-        # use sms.api directly, does not create sms.sms
-        self.assertNoSMS()
-        self.assertSMSIapSent(self.random_numbers_san, self._test_body)
+        for number in self.random_numbers_san:
+            self.assertSMS(self.env['res.partner'], number, 'pending', content=self._test_body, fields_values={'to_delete': True})
 
     def test_composer_sending_with_no_number_field(self):
         test_record = self.env['mail.test.sms.partner'].create({'name': 'Test'})
@@ -283,7 +282,7 @@ class TestSMSComposerComment(TestSMSCommon, TestSMSRecipients):
         self.assertSMSNotification([{'number': self.random_numbers_san[0]}], self._test_body)
 
 
-class TestSMSComposerBatch(TestSMSCommon):
+class TestSMSComposerBatch(SMSCommon):
     @classmethod
     def setUpClass(cls):
         super(TestSMSComposerBatch, cls).setUpClass()
@@ -333,7 +332,7 @@ class TestSMSComposerBatch(TestSMSCommon):
             )
 
 
-class TestSMSComposerMass(TestSMSCommon):
+class TestSMSComposerMass(SMSCommon):
 
     @classmethod
     def setUpClass(cls):
@@ -585,7 +584,7 @@ class TestSMSComposerMass(TestSMSCommon):
             with self.mockSMSGateway():
                 messages = composer._action_send_sms()
 
-        number = self.partners[2].phone_get_sanitized_number()
+        number = self.partners[2]._phone_format()
         self.assertSMSNotification(
             [{'partner': test_record_2.customer_id, 'number': number}],
             "Hello %s ceci est en français." % test_record_2.display_name, messages
