@@ -2,12 +2,9 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import odoo
 from odoo.tests import HttpCase, tagged
-from odoo.tests.common import HOST
 from odoo.tools import mute_logger
-from odoo.addons.http_routing.models.ir_http import slug
 
 from unittest.mock import patch
-from urllib.parse import urlparse
 
 
 @tagged('-at_install', 'post_install')
@@ -39,7 +36,7 @@ class TestRedirect(HttpCase):
                 - Correct & working redirect as logged in user
                 - Correct replace of url_for() URLs in DOM
         """
-        url = '/test_website/country/' + slug(country_ad)
+        url = '/test_website/country/' + self.env['ir.http']._slug(country_ad)
         redirect_url = url.replace('test_website', 'redirected')
 
         # [Public User] Open the original url and check redirect OK
@@ -82,27 +79,27 @@ class TestRedirect(HttpCase):
             # published
             resp = self.url_open(f"/test_website/200/name-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(urlparse(resp.headers.get('Location', '')).path, f"/test_website/308/name-{rec_published.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/name-{rec_published.id}")
 
             resp = self.url_open(f"/test_website/308/name-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 200)
 
             resp = self.url_open(f"/test_website/200/xx-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/xx-{rec_published.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/xx-{rec_published.id}")
 
             resp = self.url_open(f"/test_website/308/xx-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 301)
-            self.assertEqual(urlparse(resp.headers.get('Location'), '').path, f"/test_website/308/name-{rec_published.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/name-{rec_published.id}")
 
             resp = self.url_open(f"/test_website/200/xx-{rec_published.id}", allow_redirects=True)
             self.assertEqual(resp.status_code, 200)
-            self.assertEqual(resp.url, f"{self.base_url()}/test_website/308/name-{rec_published.id}")
+            self.assertURLEqual(resp.url, f"/test_website/308/name-{rec_published.id}")
 
             # unexisting
             resp = self.url_open("/test_website/200/name-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/name-100")
+            self.assertURLEqual(resp.headers.get('Location'), "/test_website/308/name-100")
 
             resp = self.url_open("/test_website/308/name-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 404)
@@ -110,7 +107,7 @@ class TestRedirect(HttpCase):
 
             resp = self.url_open("/test_website/200/xx-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/xx-100")
+            self.assertURLEqual(resp.headers.get('Location'), "/test_website/308/xx-100")
 
             resp = self.url_open("/test_website/308/xx-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 404)
@@ -119,7 +116,7 @@ class TestRedirect(HttpCase):
             # unpublish
             resp = self.url_open(f"/test_website/200/name-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/name-{rec_unpublished.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/name-{rec_unpublished.id}")
 
             resp = self.url_open(f"/test_website/308/name-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 403)
@@ -127,7 +124,7 @@ class TestRedirect(HttpCase):
 
             resp = self.url_open(f"/test_website/200/xx-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/xx-{rec_unpublished.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/xx-{rec_unpublished.id}")
 
             resp = self.url_open(f"/test_website/308/xx-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 403)
@@ -139,14 +136,14 @@ class TestRedirect(HttpCase):
 
             resp = self.url_open(f"/test_website/200/seo-name-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/seo-name-{rec_published.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/seo-name-{rec_published.id}")
 
             resp = self.url_open(f"/test_website/308/seo-name-{rec_published.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 200)
 
             resp = self.url_open(f"/test_website/200/xx-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/xx-{rec_unpublished.id}")
+            self.assertURLEqual(resp.headers.get('Location'), f"/test_website/308/xx-{rec_unpublished.id}")
 
             resp = self.url_open(f"/test_website/308/xx-{rec_unpublished.id}", allow_redirects=False)
             self.assertEqual(resp.status_code, 403)
@@ -154,7 +151,7 @@ class TestRedirect(HttpCase):
 
             resp = self.url_open("/test_website/200/xx-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 308)
-            self.assertEqual(resp.headers.get('Location'), f"{self.base_url()}/test_website/308/xx-100")
+            self.assertURLEqual(resp.headers.get('Location'), "/test_website/308/xx-100")
 
             resp = self.url_open("/test_website/308/xx-100", allow_redirects=False)
             self.assertEqual(resp.status_code, 404)
@@ -181,7 +178,7 @@ class TestRedirect(HttpCase):
             'name': '301 test record',
             'is_published': True,
         })
-        url_rec1 = '/test_website/200/' + slug(rec1)
+        url_rec1 = '/test_website/200/' + self.env['ir.http']._slug(rec1)
         r = self.url_open(url_rec1)
         self.assertEqual(r.status_code, 200)
 
@@ -203,10 +200,98 @@ class TestRedirect(HttpCase):
         # 4. Accessing unpublished record with redirect to another published
         # record: expecting redirect to that record
         rec2 = rec1.copy({'is_published': True})
-        url_rec2 = '/test_website/200/' + slug(rec2)
+        url_rec2 = '/test_website/200/' + self.env['ir.http']._slug(rec2)
         redirect.url_to = url_rec2
         r = self.url_open(url_rec1)
         self.assertEqual(r.status_code, 200)
         self.assertTrue(
             r.url.endswith(url_rec2),
             "Unpublished record should redirect to published record set in redirect")
+
+    @mute_logger('odoo.http')
+    def test_05_redirect_404_notfound_record(self):
+        # 1. Accessing unexisting record: raise 404
+        url_rec1 = '/test_website/200/unexisting-100000'
+        r = self.url_open(url_rec1)
+        self.assertEqual(r.status_code, 404)
+
+        # 2. Accessing unpublished record with redirect to a 404: expecting 404
+        redirect = self.env['website.rewrite'].create({
+            'name': 'Test 301 Redirect route unexisting record',
+            'redirect_type': '301',
+            'url_from': url_rec1,
+            'url_to': '/get',
+        })
+        r = self.url_open(url_rec1, allow_redirects=False)
+        self.assertEqual(r.status_code, 301)
+        self.assertURLEqual(r.headers.get('Location'), redirect.url_to)
+
+        r = self.url_open(url_rec1, allow_redirects=True)
+        self.assertEqual(r.status_code, 200)
+        self.assertURLEqual(r.url, redirect.url_to)
+
+    def test_redirect_308_multiple_url_endpoint(self):
+        self.env['website.rewrite'].create({
+            'name': 'Test Multi URL 308',
+            'redirect_type': '308',
+            'url_from': '/test_countries_308',
+            'url_to': '/test_countries_308_redirected',
+        })
+        rec1 = self.env['test.model'].create({
+            'name': '301 test record',
+            'is_published': True,
+        })
+        url_rec1 = f"/test_countries_308/{self.env['ir.http']._slug(rec1)}"
+
+        resp = self.url_open("/test_countries_308", allow_redirects=False)
+        self.assertEqual(resp.status_code, 308)
+        self.assertURLEqual(resp.headers.get('Location'), "/test_countries_308_redirected")
+
+        resp = self.url_open(url_rec1)
+        self.assertEqual(resp.status_code, 200)
+        self.assertTrue(resp.url.endswith(url_rec1))
+
+    def test_redirect_with_qs(self):
+        self.env['website.rewrite'].create({
+            'name': 'Test 301 Redirect with qs',
+            'redirect_type': '301',
+            'url_from': '/foo?bar=1',
+            'url_to': '/new-page-01',
+        })
+        self.env['website.rewrite'].create({
+            'name': 'Test 301 Redirect with qs',
+            'redirect_type': '301',
+            'url_from': '/foo?bar=2',
+            'url_to': '/new-page-10?qux=2',
+        })
+        self.env['website.rewrite'].create({
+            'name': 'Test 301 Redirect without qs',
+            'redirect_type': '301',
+            'url_from': '/foo',
+            'url_to': '/new-page-11',
+        })
+
+        # should match qs first
+        resp = self.url_open("/foo?bar=1", allow_redirects=False)
+        self.assertEqual(resp.status_code, 301)
+        self.assertURLEqual(resp.headers.get('Location'), "/new-page-01?bar=1")
+
+        # should match qs first
+        resp = self.url_open("/foo?bar=2", allow_redirects=False)
+        self.assertEqual(resp.status_code, 301)
+        self.assertURLEqual(resp.headers.get('Location'), "/new-page-10?qux=2&bar=2")
+
+        # should match no qs
+        resp = self.url_open("/foo?bar=3", allow_redirects=False)
+        self.assertEqual(resp.status_code, 301)
+        self.assertURLEqual(resp.headers.get('Location'), "/new-page-11?bar=3")
+
+        resp = self.url_open("/foo", allow_redirects=False)
+        self.assertEqual(resp.status_code, 301)
+        self.assertURLEqual(resp.headers.get('Location'), "/new-page-11")
+
+        # we dont support wrong get order
+        # purpose is to support simple case like content.asp?id=xx
+        resp = self.url_open("/foo?oups=1&bar=2", allow_redirects=False)
+        self.assertEqual(resp.status_code, 301)
+        self.assertURLEqual(resp.headers.get('Location'), "/new-page-11?oups=1&bar=2")
