@@ -10,18 +10,22 @@ It will install the e-learning, survey and e-commerce apps and make a complete
 certification flow including purchase, certification, failure and success.
 """,
     'depends': [
-        'website_sale_product_configurator',
         'website_sale_slides',
         'website_slides_forum',
         'website_slides_survey',
-        'payment_test'
+        'payment_demo'
+    ],
+    'data': [
+        'data/res_groups_data.xml',
     ],
     'demo': [
         'data/product_demo.xml',
     ],
-    'data': [
-        'views/assets.xml',
-    ],
     'installable': True,
+    'assets': {
+        'web.assets_tests': [
+            'test_website_slides_full/static/tests/tours/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
