@@ -17,8 +17,8 @@ class IrHttp(models.AbstractModel):
         response = Response.load(response)
         domain = cls.get_utm_domain_cookies()
         for url_parameter, __, cookie_name in request.env['utm.mixin'].tracking_fields():
-            if url_parameter in request.params and request.httprequest.cookies.get(cookie_name) != request.params[url_parameter]:
-                response.set_cookie(cookie_name, request.params[url_parameter], domain=domain, cookie_type='optional')
+            if url_parameter in request.params and request.cookies.get(cookie_name) != request.params[url_parameter]:
+                response.set_cookie(cookie_name, request.params[url_parameter], max_age=31 * 24 * 3600, domain=domain, cookie_type='optional')
 
     @classmethod
     def _post_dispatch(cls, response):
