@@ -2,10 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import odoo.tests
+from odoo.addons.base.tests.common import HttpCaseWithUserDemo
 
 
 @odoo.tests.tagged('post_install', '-at_install')
-class TestRoutes(odoo.tests.HttpCase):
+class TestRoutes(HttpCaseWithUserDemo):
 
     def test_01_web_session_destroy(self):
         base_url = self.env['ir.config_parameter'].sudo().get_param('web.base.url')
