@@ -1,5 +1,3 @@
-/** @odoo-module **/
-
 import { browser } from "./browser";
 
 // -----------------------------------------------------------------------------
@@ -7,12 +5,23 @@ import { browser } from "./browser";
 // -----------------------------------------------------------------------------
 
 /**
- * true if the browser is based on Chromium (Google Chrome, Opera, Edge)
+ * True if the browser is based on Chromium (Google Chrome, Opera, Edge).
+ */
+export function isBrowserChrome() {
+    return /Chrome/i.test(browser.navigator.userAgent);
+}
+
+export function isBrowserFirefox() {
+    return /Firefox/i.test(browser.navigator.userAgent);
+}
+
+/**
+ * true if the browser is based on Safari (Safari, Epiphany)
  *
  * @returns {boolean}
  */
-export function isBrowserChrome() {
-    return browser.navigator.userAgent.includes("Chrome");
+export function isBrowserSafari() {
+    return !isBrowserChrome() && browser.navigator.userAgent.includes("Safari");
 }
 
 export function isAndroid() {
@@ -31,7 +40,7 @@ export function isOtherMobileOS() {
 }
 
 export function isMacOS() {
-    return Boolean(browser.navigator.userAgent.match(/Mac/i));
+    return /Mac/i.test(browser.navigator.userAgent);
 }
 
 export function isMobileOS() {
@@ -42,8 +51,16 @@ export function isIosApp() {
     return /OdooMobile \(iOS\)/i.test(browser.navigator.userAgent);
 }
 
+export function isAndroidApp() {
+    return /OdooMobile.+Android/i.test(browser.navigator.userAgent);
+}
+
+export function isDisplayStandalone() {
+    return browser.matchMedia("(display-mode: standalone)").matches;
+}
+
 export function hasTouch() {
-    return browser.ontouchstart !== undefined;
+    return browser.ontouchstart !== undefined || browser.matchMedia("(pointer:coarse)").matches;
 }
 
 export function maxTouchPoints() {
