@@ -1,5 +1,3 @@
-/** @odoo-module **/
-
 import { registry } from "@web/core/registry";
 import { CalendarRenderer } from "./calendar_renderer";
 import { CalendarArchParser } from "./calendar_arch_parser";
@@ -9,9 +7,6 @@ import { CalendarController } from "./calendar_controller";
 export const calendarView = {
     type: "calendar",
 
-    display_name: "Calendar",
-    icon: "fa fa-calendar",
-    multiRecord: true,
     searchMenuTypes: ["filter", "favorite"],
 
     ArchParser: CalendarArchParser,
