@@ -1,9 +1,14 @@
-/** @odoo-module **/
-
 import { Component } from "@odoo/owl";
 import { getColor } from "../colors";
 
 export class CalendarMobileFilterPanel extends Component {
+    static components = {};
+    static template = "web.CalendarMobileFilterPanel";
+    static props = {
+        model: Object,
+        sideBarShown: Boolean,
+        toggleSideBar: Function,
+    };
     get caretDirection() {
         return this.props.sideBarShown ? "down" : "left";
     }
@@ -18,15 +23,18 @@ export class CalendarMobileFilterPanel extends Component {
             if (a.type === b.type) {
                 const va = a.value ? -1 : 0;
                 const vb = b.value ? -1 : 0;
+                //Condition to put unvaluable item (eg: Open Shifts) at the end of the sorted list.
                 if (a.type === "dynamic" && va !== vb) {
                     return va - vb;
                 }
-                return b.label.localeCompare(a.label);
+                return a.label.localeCompare(b.label, undefined, {
+                    numeric: true,
+                    sensitivity: "base",
+                    ignorePunctuation: true,
+                });
             } else {
                 return this.getFilterTypePriority(a.type) - this.getFilterTypePriority(b.type);
             }
         });
     }
 }
-CalendarMobileFilterPanel.components = {};
-CalendarMobileFilterPanel.template = "web.CalendarMobileFilterPanel";
