@@ -1,5 +1,3 @@
-/** @odoo-module **/
-
 import { registry } from "@web/core/registry";
 import { SIZES } from "@web/core/ui/ui_service";
 import {
@@ -18,7 +16,7 @@ import {
     makeSeparator,
 } from "@web/views/view_compiler";
 import { ViewCompiler } from "../view_compiler";
-import { localization } from "@web/core/l10n/localization";
+import { exprToBoolean } from "@web/core/utils/strings";
 
 const compilersRegistry = registry.category("form_compilers");
 
@@ -53,19 +51,23 @@ export class FormCompiler extends ViewCompiler {
         this.compilers.push(
             ...compilersRegistry.getAll(),
             { selector: "div[name='button_box']", fn: this.compileButtonBox },
-            { selector: "form", fn: this.compileForm },
+            { selector: "footer", fn: this.compileFooter },
+            { selector: "form", fn: this.compileForm, doNotCopyAttributes: true },
             { selector: "group", fn: this.compileGroup },
             { selector: "header", fn: this.compileHeader },
             { selector: "label", fn: this.compileLabel, doNotCopyAttributes: true },
             { selector: "notebook", fn: this.compileNotebook },
+            { selector: "setting", fn: this.compileSetting },
             { selector: "separator", fn: this.compileSeparator },
             { selector: "sheet", fn: this.compileSheet }
         );
     }
 
-    compile() {
+    compile(key, params = {}) {
         const compiled = super.compile(...arguments);
-        compiled.children[0].setAttribute("t-ref", "compiled_view_root");
+        if (!params.isSubView) {
+            compiled.children[0].setAttribute("t-ref", "compiled_view_root");
+        }
         return compiled;
     }
 
@@ -76,13 +78,13 @@ export class FormCompiler extends ViewCompiler {
         } else {
             labelText = labelText
                 ? toStringExpression(labelText)
-                : `props.record.fields['${fieldName}'].string`;
+                : `__comp__.props.record.fields['${fieldName}'].string`;
         }
         const formLabel = createElement("FormLabel", {
             id: `'${fieldId}'`,
             fieldName: `'${fieldName}'`,
-            record: `props.record`,
-            fieldInfo: `props.archInfo.fieldNodes['${fieldId}']`,
+            record: `__comp__.props.record`,
+            fieldInfo: `__comp__.props.archInfo.fieldNodes['${fieldId}']`,
             className: `"${label.className}"`,
             string: labelText,
         });
@@ -122,25 +124,30 @@ export class FormCompiler extends ViewCompiler {
      * @returns {Element}
      */
     compileButtonBox(el, params) {
-        if (!el.childNodes.length) {
+        if (!el.children.length) {
             return this.compileGenericNode(el, params);
         }
 
         el.classList.remove("oe_button_box");
         const buttonBox = createElement("ButtonBox");
+        buttonBox.setAttribute("t-if", "!__comp__.env.inDialog");
         let slotId = 0;
         let hasContent = false;
         for (const child of el.children) {
             const invisible = getModifier(child, "invisible");
-            if (this.isAlwaysInvisible(invisible, params)) {
+            if (!params.compileInvisibleNodes && (invisible === "True" || invisible === "1")) {
                 continue;
             }
             hasContent = true;
             let isVisibleExpr;
-            if (typeof invisible === "boolean") {
-                isVisibleExpr = `${invisible ? false : true}`;
+            if (!invisible || invisible === "False" || invisible === "0") {
+                isVisibleExpr = "true";
+            } else if (invisible === "True" || invisible === "1") {
+                isVisibleExpr = "false";
             } else {
-                isVisibleExpr = `!evalDomainFromRecord(props.record,${JSON.stringify(invisible)})`;
+                isVisibleExpr = `!__comp__.evaluateBooleanExpr(${JSON.stringify(
+                    invisible
+                )},__comp__.props.record.evalContextWithVirtualIds)`;
             }
             const mainSlot = createElement("t", {
                 "t-set-slot": `slot_${slotId++}`,
@@ -149,34 +156,24 @@ export class FormCompiler extends ViewCompiler {
             if (child.tagName === "button" || child.children.tagName === "button") {
                 child.classList.add(
                     "oe_stat_button",
-                    "btn-light",
-                    "flex-shrink-0",
-                    "mb-0",
-                    "py-0",
-                    "border-0",
-                    "border-start",
-                    "border-bottom",
-                    "rounded-0",
-                    "text-start",
-                    "text-nowrap",
-                    "text-capitalize"
+                    "btn",
+                    "btn-outline-secondary",
+                    "flex-grow-1",
+                    "flex-lg-grow-0"
                 );
             }
             if (child.tagName === "field") {
-                child.classList.add("d-inline-block", "mb-0");
+                child.classList.add("d-inline-block", "mb-0", "z-0");
             }
             append(mainSlot, this.compileNode(child, params, false));
             append(buttonBox, mainSlot);
         }
 
-        return hasContent ? buttonBox : null;
+        return hasContent ? buttonBox : "";
     }
 
     compileButton(el, params) {
-        const compiled = super.compileButton(el, params);
-        compiled.setAttribute("disable", "props.disableViewButtons");
-        compiled.setAttribute("enable", "props.enableViewButtons");
-        return compiled;
+        return super.compileButton(el, params);
     }
 
     /**
@@ -187,7 +184,7 @@ export class FormCompiler extends ViewCompiler {
 
         const fieldName = el.getAttribute("name");
         const fieldString = el.getAttribute("string");
-        const fieldId = el.getAttribute("field_id") || fieldName;
+        const fieldId = el.getAttribute("field_id");
         const labelsForAttr = el.getAttribute("id") || fieldName;
         const labels = this.getLabels(labelsForAttr);
         const dynamicLabel = (label) => {
@@ -206,7 +203,6 @@ export class FormCompiler extends ViewCompiler {
             dynamicLabel(label);
         }
         this.encounteredFields[fieldName] = dynamicLabel;
-        field.setAttribute("setDirty", `props.setFieldAsDirty`);
         return field;
     }
 
@@ -216,21 +212,32 @@ export class FormCompiler extends ViewCompiler {
      * @returns {Element}
      */
     compileForm(el, params) {
-        const sheetNode = el.querySelector("sheet");
+        let sheetNode = null;
+        for (const sheet of el.querySelectorAll("sheet")) {
+            if (sheet.closest("form") === el) {
+                sheetNode = sheet;
+                break;
+            }
+        }
         const displayClasses = sheetNode
-            ? `d-flex {{ uiService.size < ${SIZES.XXL} ? "flex-column" : "flex-nowrap h-100" }}`
+            ? `d-flex d-print-block {{ __comp__.uiService.size < ${SIZES.XXL} ? "flex-column" : "flex-nowrap h-100" }}`
             : "d-block";
         const stateClasses =
-            "{{ props.record.isDirty ? 'o_form_dirty' : !props.record.isVirtual ? 'o_form_saved' : '' }}";
+            "{{ __comp__.props.record.dirty ? 'o_form_dirty' : !__comp__.props.record.isNew ? 'o_form_saved' : '' }}";
         const form = createElement("div", {
-            "t-att-class": "props.class",
-            "t-attf-class": `{{props.record.isInEdition ? 'o_form_editable' : 'o_form_readonly'}} ${displayClasses} ${stateClasses}`,
+            class: "o_form_renderer",
+            "t-att-class": "__comp__.props.class",
+            "t-attf-class": `{{__comp__.props.record.isInEdition ? 'o_form_editable' : 'o_form_readonly'}} ${displayClasses} ${stateClasses}`,
         });
         if (!sheetNode) {
             for (const child of el.childNodes) {
-                append(form, this.compileNode(child, params));
+                // ButtonBox are already compiled for the control panel and should not
+                // be recompiled for the renderer of the view
+                if (child.attributes?.name?.value !== "button_box") {
+                    append(form, this.compileNode(child, params));
+                }
             }
-            form.className = "o_form_nosheet";
+            form.classList.add("o_form_nosheet");
         } else {
             let compiledList = [];
             for (const child of el.childNodes) {
@@ -245,21 +252,33 @@ export class FormCompiler extends ViewCompiler {
             }
             append(form, compiledList);
         }
-        if (localization.multiLang) {
-            const statusBar = form.querySelector(".o_form_statusbar");
-            const translateAlert = createElement("t", {
-                "t-if": "props.translateAlert",
-                "t-call": "web.TranslateAlert",
-            });
-            if (statusBar) {
-                statusBar.parentElement.insertBefore(translateAlert, statusBar.nextSibling);
-            } else if (form.querySelector(".o_form_sheet_bg")) {
-                form.querySelector(".o_form_sheet_bg").prepend(translateAlert);
-            } else {
-                form.prepend(translateAlert);
+        return form;
+    }
+
+    /**
+     * @param {Element} el
+     * @param {Record<string, any>} params
+     * @returns {Element}
+     */
+    compileFooter(el, params) {
+        const footer = createElement("t");
+        const replace = el.getAttribute("replace");
+        if (replace && !exprToBoolean(replace)) {
+            footer.append(
+                createElement("t", {
+                    "t-call": "web.DefaultButtonsSlot",
+                    "t-call-context": "{ props: __comp__.props }",
+                })
+            );
+        }
+        copyAttributes(el, footer);
+        for (const child of el.childNodes) {
+            const compiled = this.compileNode(child, params);
+            if (compiled) {
+                footer.append(compiled);
             }
         }
-        return form;
+        return footer;
     }
 
     /**
@@ -293,7 +312,7 @@ export class FormCompiler extends ViewCompiler {
             }
 
             const invisible = getModifier(child, "invisible");
-            if (this.isAlwaysInvisible(invisible, params)) {
+            if (!params.compileInvisibleNodes && (invisible === "True" || invisible === "1")) {
                 continue;
             }
 
@@ -331,14 +350,14 @@ export class FormCompiler extends ViewCompiler {
                     const props = {
                         id: `${fieldId}`,
                         fieldName: `'${fieldName}'`,
-                        record: `props.record`,
+                        record: `__comp__.props.record`,
                         string: child.hasAttribute("string")
                             ? toStringExpression(child.getAttribute("string"))
-                            : `props.record.fields.${fieldName}.string`,
-                        fieldInfo: `props.archInfo.fieldNodes[${fieldId}]`,
+                            : `__comp__.props.record.fields.${fieldName}.string`,
+                        fieldInfo: `__comp__.props.archInfo.fieldNodes[${fieldId}]`,
                     };
                     mainSlot.setAttribute("props", objectToString(props));
-                    mainSlot.setAttribute("Component", "constructor.components.FormLabel");
+                    mainSlot.setAttribute("Component", "__comp__.constructor.components.FormLabel");
                     mainSlot.setAttribute("subType", "'item_component'");
                 }
             } else {
@@ -356,12 +375,14 @@ export class FormCompiler extends ViewCompiler {
 
             if (slotContent && !isTextNode(slotContent)) {
                 let isVisibleExpr;
-                if (typeof invisible === "boolean") {
-                    isVisibleExpr = `${invisible ? false : true}`;
+                if (!invisible || invisible === "False" || invisible === "0") {
+                    isVisibleExpr = "true";
+                } else if (invisible === "True" || invisible === "1") {
+                    isVisibleExpr = "false";
                 } else {
-                    isVisibleExpr = `!evalDomainFromRecord(props.record,${JSON.stringify(
+                    isVisibleExpr = `!__comp__.evaluateBooleanExpr(${JSON.stringify(
                         invisible
-                    )})`;
+                    )},__comp__.props.record.evalContextWithVirtualIds)`;
                 }
                 mainSlot.setAttribute("isVisible", isVisibleExpr);
                 if (itemSpan > 0) {
@@ -419,7 +440,7 @@ export class FormCompiler extends ViewCompiler {
     compileHeader(el, params) {
         const statusBar = createElement("div");
         statusBar.className =
-            "o_form_statusbar position-relative d-flex justify-content-between border-bottom";
+            "o_form_statusbar position-relative d-flex justify-content-between mb-0 mb-md-2 pb-2 pb-md-0";
         const buttons = [];
         const others = [];
         for (const child of el.childNodes) {
@@ -427,7 +448,7 @@ export class FormCompiler extends ViewCompiler {
             if (!compiled || isTextNode(compiled)) {
                 continue;
             }
-            if (getTag(child, true) === "field") {
+            if (getTag(child, true) === "field" && !child.classList.contains("btn")) {
                 compiled.setAttribute("showTooltip", true);
                 others.push(compiled);
             } else {
@@ -438,8 +459,14 @@ export class FormCompiler extends ViewCompiler {
             }
         }
         let slotId = 0;
-        const statusBarButtons = createElement("StatusBarButtons");
-        statusBarButtons.setAttribute("readonly", "!props.record.isInEdition");
+        let statusBarButtons;
+        if (params.asDropdownItems) {
+            statusBarButtons = createElement("StatusBarDropdownItems");
+        } else {
+            statusBarButtons = createElement("StatusBarButtons", {
+                "t-if": "!__comp__.env.isSmall or __comp__.env.inDialog",
+            });
+        }
         for (const button of buttons) {
             const slot = createElement("t", {
                 "t-set-slot": `button_${slotId++}`,
@@ -448,6 +475,9 @@ export class FormCompiler extends ViewCompiler {
             append(slot, button);
             append(statusBarButtons, slot);
         }
+        if (params.asDropdownItems) {
+            return statusBarButtons;
+        }
         append(statusBar, statusBarButtons);
         append(statusBar, others);
         return statusBar;
@@ -504,11 +534,11 @@ export class FormCompiler extends ViewCompiler {
 
         noteBook.setAttribute(
             "defaultPage",
-            `props.record.isNew ? undefined : props.activeNotebookPages[${noteBookId}]`
+            `__comp__.props.record.isNew ? undefined : __comp__.props.activeNotebookPages[${noteBookId}]`
         );
         noteBook.setAttribute(
             "onPageUpdate",
-            `(page) => this.props.onNotebookPageChange(${noteBookId}, page)`
+            `(page) => __comp__.props.onNotebookPageChange(${noteBookId}, page)`
         );
 
         for (const child of el.children) {
@@ -516,7 +546,7 @@ export class FormCompiler extends ViewCompiler {
                 continue;
             }
             const invisible = getModifier(child, "invisible");
-            if (this.isAlwaysInvisible(invisible, params)) {
+            if (!params.compileInvisibleNodes && (invisible === "True" || invisible === "1")) {
                 continue;
             }
 
@@ -539,7 +569,7 @@ export class FormCompiler extends ViewCompiler {
             if (child.getAttribute("autofocus") === "autofocus") {
                 noteBook.setAttribute(
                     "defaultPage",
-                    `props.record.isNew ? "${pageId}" : (props.activeNotebookPages[${noteBookId}] || "${pageId}")`
+                    `__comp__.props.record.isNew ? "${pageId}" : (__comp__.props.activeNotebookPages[${noteBookId}] || "${pageId}")`
                 );
             }
 
@@ -554,13 +584,17 @@ export class FormCompiler extends ViewCompiler {
                 };
             }
 
-            let isVisible;
-            if (typeof invisible === "boolean") {
-                isVisible = `${!invisible}`;
+            let isVisibleExpr;
+            if (!invisible || invisible === "False" || invisible === "0") {
+                isVisibleExpr = "true";
+            } else if (invisible === "True" || invisible === "1") {
+                isVisibleExpr = "false";
             } else {
-                isVisible = `!evalDomainFromRecord(props.record,${JSON.stringify(invisible)})`;
+                isVisibleExpr = `!__comp__.evaluateBooleanExpr(${JSON.stringify(
+                    invisible
+                )},__comp__.props.record.evalContextWithVirtualIds)`;
             }
-            pageSlot.setAttribute("isVisible", isVisible);
+            pageSlot.setAttribute("isVisible", isVisibleExpr);
 
             for (const contents of child.children) {
                 append(pageSlot, this.compileNode(contents, { ...params, currentSlot: pageSlot }));
@@ -588,6 +622,55 @@ export class FormCompiler extends ViewCompiler {
         return noteBook;
     }
 
+    /**
+     * @param {Element} el
+     * @param {Record<string, any>} params
+     * @returns {Element}
+     */
+    compileSetting(el, params) {
+        const setting = createElement(params.componentName || "Setting", {
+            title: toStringExpression(el.getAttribute("title") || ""),
+            help: toStringExpression(el.getAttribute("help") || ""),
+            companyDependent: el.getAttribute("company_dependent") === "1" || "false",
+            documentation: toStringExpression(el.getAttribute("documentation") || ""),
+            record: `__comp__.props.record`,
+        });
+        if (el.getAttribute("id")) {
+            setting.setAttribute("id", toStringExpression(el.getAttribute("id")));
+        }
+        let string = toStringExpression(el.getAttribute("string") || "");
+        let addLabel = true;
+        Array.from(el.children).forEach((child, index) => {
+            if (getTag(child, true) === "field" && index === 0) {
+                const fieldSlot = createElement("t", { "t-set-slot": "fieldSlot" });
+                const field = this.compileNode(child, params);
+                if (field) {
+                    append(fieldSlot, field);
+                    setting.setAttribute("fieldInfo", field.getAttribute("fieldInfo"));
+
+                    addLabel = child.hasAttribute("nolabel")
+                        ? child.getAttribute("nolabel") !== "1"
+                        : true;
+                    const fieldName = child.getAttribute("name");
+                    string = child.hasAttribute("string")
+                        ? toStringExpression(child.getAttribute("string"))
+                        : string;
+                    setting.setAttribute("fieldName", toStringExpression(fieldName));
+                    setting.setAttribute(
+                        "fieldId",
+                        toStringExpression(child.getAttribute("field_id"))
+                    );
+                }
+                append(setting, fieldSlot);
+            } else {
+                append(setting, this.compileNode(child, params));
+            }
+        });
+        setting.setAttribute("string", string);
+        setting.setAttribute("addLabel", addLabel);
+        return setting;
+    }
+
     /**
      * @param {Element} el
      * @param {Record<string, any>} params
@@ -609,7 +692,7 @@ export class FormCompiler extends ViewCompiler {
         sheetBG.className = "o_form_sheet_bg";
 
         const sheetFG = createElement("div");
-        sheetFG.className = "o_form_sheet position-relative clearfix";
+        sheetFG.className = "o_form_sheet position-relative";
 
         append(sheetBG, sheetFG);
         for (const child of el.childNodes) {
@@ -617,6 +700,11 @@ export class FormCompiler extends ViewCompiler {
             if (!compiled) {
                 continue;
             }
+            if (compiled.nodeName === "ButtonBox") {
+                // in form views with a sheet, the button box is moved to the
+                // control panel, and in dialogs, there's no button box
+                continue;
+            }
             if (getTag(child, true) === "field") {
                 compiled.setAttribute("showTooltip", true);
             }
@@ -624,13 +712,4 @@ export class FormCompiler extends ViewCompiler {
         }
         return sheetBG;
     }
-
-    /**
-     * @override
-     */
-    compileWidget(el) {
-        const widget = super.compileWidget(el);
-        widget.setAttribute("readonly", `!props.record.isInEdition`);
-        return widget;
-    }
 }
