@@ -1,21 +1,19 @@
-/** @odoo-module */
-
 import { registry } from "@web/core/registry";
-import { RelationalModel } from "../relational_model";
+import { RelationalModel } from "@web/model/relational_model/relational_model";
 import { ListArchParser } from "./list_arch_parser";
 import { ListController } from "./list_controller";
 import { ListRenderer } from "./list_renderer";
 
 export const listView = {
     type: "list",
-    display_name: "List",
-    icon: "oi oi-view-list",
-    multiRecord: true,
     Controller: ListController,
     Renderer: ListRenderer,
     ArchParser: ListArchParser,
     Model: RelationalModel,
     buttonTemplate: "web.ListView.Buttons",
+    canOrderByCount: true,
+
+    limit: 80,
 
     props: (genericProps, view) => {
         const { ArchParser } = view;
