@@ -1,20 +1,39 @@
-/** @odoo-module **/
-
+import { _t } from "@web/core/l10n/translation";
 import { Domain } from "@web/core/domain";
 import { cartesian, sections, sortBy, symmetricalDifference } from "@web/core/utils/arrays";
 import { KeepLast, Race } from "@web/core/utils/concurrency";
-import { computeVariation } from "@web/core/utils/numbers";
 import { DEFAULT_INTERVAL } from "@web/search/utils/dates";
-import { Model } from "@web/views/model";
+import { Model } from "@web/model/model";
 import { computeReportMeasures, processMeasure } from "@web/views/utils";
 
+/**
+ * @param {number} value
+ * @param {number} comparisonValue
+ * @returns {number}
+ */
+function computeVariation(value, comparisonValue) {
+    if (isNaN(value) || isNaN(comparisonValue)) {
+        return NaN;
+    }
+    if (comparisonValue === 0) {
+        if (value === 0) {
+            return 0;
+        } else if (value > 0) {
+            return 1;
+        } else {
+            return -1;
+        }
+    }
+    return (value - comparisonValue) / Math.abs(comparisonValue);
+}
+
 /**
  * Pivot Model
  *
  * The pivot model keeps an in-memory representation of the pivot table that is
  * displayed on the screen.  The exact layout of this representation is not so
  * simple, because a pivot table is at its core a 2-dimensional object, but
- * with a 'tree' component: some rows/cols can be expanded so we zoom into the
+ * with a 'list' component: some rows/cols can be expanded so we zoom into the
  * structure.
  *
  * However, we need to be able to manipulate the data in a somewhat efficient
@@ -681,9 +700,8 @@ export class PivotModel extends Model {
      */
     async load(searchParams) {
         this.searchParams = searchParams;
-
-        const activeMeasures =
-            processMeasure(searchParams.context.pivot_measures) || this.metaData.activeMeasures;
+        const processedMeasures = processMeasure(searchParams.context.pivot_measures);
+        const activeMeasures = processedMeasures || this.metaData.activeMeasures;
         const metaData = this._buildMetaData({ activeMeasures });
         if (!this.reload) {
             metaData.rowGroupBys =
@@ -705,11 +723,14 @@ export class PivotModel extends Model {
             metaData.expandedColGroupBys = [];
         }
 
-        metaData.measures = computeReportMeasures(
-            metaData.fields,
-            metaData.fieldAttrs,
-            metaData.activeMeasures
-        );
+        const allActivesMeasures = new Set(this.metaData.activeMeasures);
+        if (processedMeasures) {
+            processedMeasures.forEach((e) => allActivesMeasures.add(e));
+        }
+
+        metaData.measures = computeReportMeasures(metaData.fields, metaData.fieldAttrs, [
+            ...allActivesMeasures,
+        ]);
         const config = { metaData, data: this.data };
         return this._loadData(config);
     }
@@ -886,7 +907,7 @@ export class PivotModel extends Model {
      * index, returns the given measure for a group determined by the id
      * groupId and the origin index.
      * If originIndexes is an array of length 2, we compute the variation
-     * ot the measure values for the groups determined by groupId and the
+     * of the measure values for the groups determined by groupId and the
      * different origin indexes.
      *
      * @protected
@@ -910,6 +931,19 @@ export class PivotModel extends Model {
             return values[0];
         }
     }
+    /**
+     * @protected
+     * @param {string[]} rowGroupBy
+     * @param {string[]} colGroupBy
+     * @returns {string[]}
+     */
+    _getGroupBySpecs(rowGroupBy, colGroupBy) {
+        const set = rowGroupBy.concat(colGroupBy).reduce((acc, gb) => {
+            acc.add(this._normalize(gb));
+            return acc;
+        }, new Set());
+        return [...set];
+    }
     /**
      * Returns a domain representation of a group
      *
@@ -936,7 +970,8 @@ export class PivotModel extends Model {
      * @returns {string[]}
      */
     _getGroupLabels(group, groupBys, config) {
-        return groupBys.map((groupBy) => {
+        return groupBys.map((gb) => {
+            const groupBy = this._normalize(gb);
             return this._sanitizeLabel(group[groupBy], groupBy, config);
         });
     }
@@ -949,27 +984,19 @@ export class PivotModel extends Model {
      * @param {Object} group
      * @param {string[]} rowGroupBy
      * @param {string[]} colGroupBy
-     * @param {Config} config
+     * @param {Object} params
      */
-    async _getGroupSubdivision(group, rowGroupBy, colGroupBy, config) {
-        const groupDomain = this._getGroupDomain(group, config);
-        const measureSpecs = this._getMeasureSpecs(config);
-        const groupBy = rowGroupBy.concat(colGroupBy);
-        const kwargs = { lazy: false, context: this.searchParams.context };
-        const subGroups = await this.orm.readGroup(
-            config.metaData.resModel,
-            groupDomain,
-            measureSpecs,
-            groupBy,
-            kwargs
-        );
+    async _getGroupSubdivision(group, rowGroupBy, colGroupBy, params) {
+        const groupBy = this._getGroupBySpecs(rowGroupBy, colGroupBy);
+        const subGroups = await this._getSubGroups(groupBy, params);
         return {
-            group: group,
-            subGroups: subGroups,
+            group,
+            subGroups,
             rowGroupBy: rowGroupBy,
             colGroupBy: colGroupBy,
         };
     }
+
     /**
      * Returns the group sanitized values.
      *
@@ -979,7 +1006,8 @@ export class PivotModel extends Model {
      * @returns {Array}
      */
     _getGroupValues(group, groupBys) {
-        return groupBys.map((groupBy) => {
+        return groupBys.map((gb) => {
+            const groupBy = this._normalize(gb);
             return this._sanitizeValue(group[groupBy]);
         });
     }
@@ -1070,9 +1098,9 @@ export class PivotModel extends Model {
     }
     /**
      * Returns the list of measure specs associated with metaData.activeMeasures, i.e.
-     * a measure 'fieldName' becomes 'fieldName:groupOperator' where
-     * groupOperator is the value specified on the field 'fieldName' for
-     * the key group_operator.
+     * a measure 'fieldName' becomes 'fieldName:aggregator' where
+     * aggregator is the value specified on the field 'fieldName' for
+     * the key aggregator.
      *
      * @protected
      * @param {Config} config
@@ -1087,14 +1115,14 @@ export class PivotModel extends Model {
             }
             const field = this.metaData.fields[measure];
             if (field.type === "many2one") {
-                field.group_operator = "count_distinct";
+                field.aggregator = "count_distinct";
             }
-            if (field.group_operator === undefined) {
+            if (field.aggregator === undefined) {
                 throw new Error(
                     "No aggregate function has been provided for the measure '" + measure + "'"
                 );
             }
-            acc.push(measure + ":" + field.group_operator);
+            acc.push(measure + ":" + field.aggregator);
             return acc;
         }, []);
     }
@@ -1158,7 +1186,7 @@ export class PivotModel extends Model {
                         height: 1,
                         measure: measure,
                         originIndexes: [originIndex - 1, originIndex],
-                        title: this.env._t("Variation"),
+                        title: _t("Variation"),
                         width: 1,
                     };
                     if (isSortedByVariation && sortedColumn.originIndexes[1] === originIndex) {
@@ -1171,6 +1199,20 @@ export class PivotModel extends Model {
 
         return originRow;
     }
+    /**
+     * @protected
+     * @param {string[]} groupBy
+     * @param {Object} params
+     * @returns {Promise<Object[]>}
+     */
+    async _getSubGroups(groupBy, params) {
+        const { resModel, groupDomain, measureSpecs, kwargs, mapping } = params;
+        const key = JSON.stringify(groupBy);
+        if (!mapping[key]) {
+            mapping[key] = this.orm.readGroup(resModel, groupDomain, measureSpecs, groupBy, kwargs);
+        }
+        return mapping[key];
+    }
     /**
      * Returns the list of header rows of the pivot table: the col group rows
      * (depending on the col groupbys), the measures row and optionnaly the
@@ -1180,7 +1222,6 @@ export class PivotModel extends Model {
      * @returns {Object[]}
      */
     _getTableHeaders() {
-        const _t = this.env._t;
         const colGroupBys = this.metaData.fullColGroupBys;
         const height = colGroupBys.length + 1;
         const measureCount = this.metaData.activeMeasures.length;
@@ -1221,10 +1262,7 @@ export class PivotModel extends Model {
                     rowIndex === 0
                         ? undefined
                         : fields[colGroupBys[rowIndex - 1].split(":")[0]].string,
-                title:
-                    group.labels.length
-                        ? group.labels[group.labels.length - 1]
-                        : _t("Total"),
+                title: group.labels.length ? group.labels[group.labels.length - 1] : _t("Total"),
                 width: leafCount * measureCount * (2 * originCount - 1),
             };
             row.push(cell);
@@ -1275,10 +1313,7 @@ export class PivotModel extends Model {
         let rows = [];
         const group = tree.root;
         const rowGroupId = [group.values, []];
-        const title =
-            group.labels.length
-                ? group.labels[group.labels.length - 1]
-                : this.env._t("Total");
+        const title = group.labels.length ? group.labels[group.labels.length - 1] : _t("Total");
         const indent = group.labels.length;
         const isLeaf = !tree.directSubTrees.size;
         const rowGroupBys = this.metaData.fullRowGroupBys;
@@ -1395,6 +1430,20 @@ export class PivotModel extends Model {
         this.data = config.data;
         this.metaData = config.metaData;
     }
+    /**
+     * @protected
+     * @param {string} gb
+     * @returns {string}
+     */
+    _normalize(gb) {
+        const [fieldName, interval] = gb.split(":");
+        const field = this.metaData.fields[fieldName];
+        if (["date", "datetime"].includes(field.type)) {
+            return `${fieldName}:${interval || "month"}`;
+        } else {
+            return fieldName;
+        }
+    }
     /**
      * Extract the information in the read_group results (groupSubdivisions)
      * and develop this.data.rowGroupTree, colGroupTree, measurements, counts, and
@@ -1502,13 +1551,18 @@ export class PivotModel extends Model {
             const subTree = tree.directSubTrees.get(subTreeKey);
             if (!oldTree.directSubTrees.has(subTreeKey)) {
                 subTree.directSubTrees.clear();
-                delete subTreeKey.sortedKeys;
+                delete subTree.sortedKeys;
             } else {
                 const oldSubTree = oldTree.directSubTrees.get(subTreeKey);
                 this._pruneTree(subTree, oldSubTree);
             }
         });
     }
+
+    _getEmptyGroupLabel(fieldName) {
+        return _t("None");
+    }
+
     /**
      * Extract from a groupBy value a label.
      *
@@ -1526,14 +1580,10 @@ export class PivotModel extends Model {
             metaData.fields[fieldName] &&
             metaData.fields[fieldName].type === "boolean"
         ) {
-            return value === undefined
-                ? this.env._t("None")
-                : value
-                ? this.env._t("Yes")
-                : this.env._t("No");
+            return value === undefined ? _t("None") : value ? _t("Yes") : _t("No");
         }
         if (value === false) {
-            return this.env._t("None");
+            return this._getEmptyGroupLabel(fieldName);
         }
         if (value instanceof Array) {
             return this._getNumberedLabel(value, fieldName, config);
@@ -1586,8 +1636,21 @@ export class PivotModel extends Model {
                     colValues: group.colValues,
                     originIndex: originIndex,
                 };
+                const groupDomain = this._getGroupDomain(subGroup, config);
+                const measureSpecs = this._getMeasureSpecs(config);
+                const resModel = config.metaData.resModel;
+                const kwargs = { lazy: false, context: this.searchParams.context };
+                const mapping = {};
                 divisors.forEach((divisor) => {
-                    acc.push(this._getGroupSubdivision(subGroup, divisor[0], divisor[1], config));
+                    acc.push(
+                        this._getGroupSubdivision(subGroup, divisor[0], divisor[1], {
+                            resModel,
+                            groupDomain,
+                            measureSpecs,
+                            kwargs,
+                            mapping,
+                        })
+                    );
                 });
             }
             return acc;
