@@ -1,5 +1,3 @@
-/** @odoo-module **/
-
 import { ActionDialog } from "./action_dialog";
 
 import { Component, xml, onWillDestroy } from "@odoo/owl";
@@ -8,6 +6,15 @@ import { Component, xml, onWillDestroy } from "@odoo/owl";
 // ActionContainer (Component)
 // -----------------------------------------------------------------------------
 export class ActionContainer extends Component {
+    static components = { ActionDialog };
+    static props = {};
+    static template = xml`
+        <t t-name="web.ActionContainer">
+          <div class="o_action_manager">
+            <t t-if="info.Component" t-component="info.Component" className="'o_action'" t-props="info.componentProps" t-key="info.id"/>
+          </div>
+        </t>`;
+
     setup() {
         this.info = {};
         this.onActionManagerUpdate = ({ detail: info }) => {
@@ -20,10 +27,3 @@ export class ActionContainer extends Component {
         });
     }
 }
-ActionContainer.components = { ActionDialog };
-ActionContainer.template = xml`
-    <t t-name="web.ActionContainer">
-      <div class="o_action_manager">
-        <t t-if="info.Component" t-component="info.Component" className="'o_action'" t-props="info.componentProps" t-key="info.id"/>
-      </div>
-    </t>`;
