@@ -1,18 +1,27 @@
-/** @odoo-module */
-
 import { registry } from "@web/core/registry";
 import { useService } from "@web/core/utils/hooks";
-import { SettingsContainer } from "../settings/settings_container";
-import { Setting } from "../settings/setting";
+import { SettingsBlock } from "../settings/settings_block";
+import { Setting } from "../../../views/form/setting/setting";
 
 import { Component, onWillStart } from "@odoo/owl";
+import { standardWidgetProps } from "@web/views/widgets/standard_widget_props";
+import { router } from "@web/core/browser/router";
 
 /**
  * Widget in the settings that handles the "Developer Tools" section.
  * Can be used to enable/disable the debug modes.
  * Can be used to load the demo data.
  */
-class ResConfigDevTool extends Component {
+export class ResConfigDevTool extends Component {
+    static template = "res_config_dev_tool";
+    static components = {
+        SettingsBlock,
+        Setting,
+    };
+    static props = {
+        ...standardWidgetProps,
+    };
+
     setup() {
         this.isDebug = Boolean(odoo.debug);
         this.isAssets = odoo.debug.includes("assets");
@@ -26,6 +35,10 @@ class ResConfigDevTool extends Component {
         });
     }
 
+    activateDebug(value) {
+        router.pushState({ debug: value }, { reload: true });
+    }
+
     /**
      * Forces demo data to be installed in a database without demo data installed.
      */
@@ -34,10 +47,8 @@ class ResConfigDevTool extends Component {
     }
 }
 
-ResConfigDevTool.template = "res_config_dev_tool";
-ResConfigDevTool.components = {
-    SettingsContainer,
-    Setting,
+export const resConfigDevTool = {
+    component: ResConfigDevTool,
 };
 
-registry.category("view_widgets").add("res_config_dev_tool", ResConfigDevTool);
+registry.category("view_widgets").add("res_config_dev_tool", resConfigDevTool);
