@@ -1,28 +1,33 @@
-/** @odoo-module **/
-
 import { Dropdown } from "@web/core/dropdown/dropdown";
+import { DropdownGroup } from "@web/core/dropdown/dropdown_group";
 import { DropdownItem } from "@web/core/dropdown/dropdown_item";
 import { CheckBox } from "@web/core/checkbox/checkbox";
-import { browser } from "@web/core/browser/browser";
 import { registry } from "@web/core/registry";
-import { useService } from "@web/core/utils/hooks";
+import { user } from "@web/core/user";
+import { session } from "@web/session";
 
 import { Component } from "@odoo/owl";
+import { imageUrl } from "@web/core/utils/urls";
 
 const userMenuRegistry = registry.category("user_menuitems");
 
 export class UserMenu extends Component {
+    static template = "web.UserMenu";
+    static components = { DropdownGroup, Dropdown, DropdownItem, CheckBox };
+    static props = {};
+
     setup() {
-        this.user = useService("user");
-        const { origin } = browser.location;
-        const { userId } = this.user;
-        this.source = `${origin}/web/image?model=res.users&field=avatar_128&id=${userId}`;
+        this.userName = user.name;
+        this.dbName = session.db;
+        const { partnerId, writeDate } = user;
+        this.source = imageUrl("res.partner", partnerId, "avatar_128", { unique: writeDate });
     }
 
     getElements() {
         const sortedItems = userMenuRegistry
             .getAll()
             .map((element) => element(this.env))
+            .filter((element) => (element.show ? element.show() : true))
             .sort((x, y) => {
                 const xSeq = x.sequence ? x.sequence : 100;
                 const ySeq = y.sequence ? y.sequence : 100;
@@ -31,8 +36,6 @@ export class UserMenu extends Component {
         return sortedItems;
     }
 }
-UserMenu.template = "web.UserMenu";
-UserMenu.components = { Dropdown, DropdownItem, CheckBox };
 
 export const systrayItem = {
     Component: UserMenu,
