@@ -1,11 +1,12 @@
-odoo.define('web_editor.image_processing', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const {getAffineApproximation, getProjective} = require('@web_editor/js/editor/perspective_utils');
+import { rpc } from "@web/core/network/rpc";
+import { pick } from "@web/core/utils/objects";
+import {getAffineApproximation, getProjective} from "@web_editor/js/editor/perspective_utils";
 
 // Fields returned by cropperjs 'getData' method, also need to be passed when
 // initializing the cropper to reuse the previous crop.
-const cropperDataFields = ['x', 'y', 'width', 'height', 'rotate', 'scaleX', 'scaleY'];
+export const cropperDataFields = ['x', 'y', 'width', 'height', 'rotate', 'scaleX', 'scaleY'];
 const modifierFields = [
     'filter',
     'quality',
@@ -15,8 +16,10 @@ const modifierFields = [
     'originalSrc',
     'resizeWidth',
     'aspectRatio',
+    "bgSrc",
+    "mimetypeBeforeConversion",
 ];
-const isGif = (mimetype) => mimetype === 'image/gif';
+export const isGif = (mimetype) => mimetype === 'image/gif';
 
 // webgl color filters
 const _applyAll = (result, filter, filters) => {
@@ -200,7 +203,7 @@ const glFilters = {
  * @param {HTMLImageElement} img the image to which modifications are applied
  * @returns {string} dataURL of the image with the applied modifications
  */
-async function applyModifications(img, dataOptions = {}) {
+export async function applyModifications(img, dataOptions = {}) {
     const data = Object.assign({
         glFilter: '',
         filter: '#0000',
@@ -233,6 +236,8 @@ async function applyModifications(img, dataOptions = {}) {
     // Crop
     const container = document.createElement('div');
     const original = await loadImage(originalSrc);
+    // loadImage may have ended up loading a different src (see: LOAD_IMAGE_404)
+    originalSrc = original.getAttribute('src');
     container.appendChild(original);
     await activateCropper(original, 0, data);
     let croppedImg = $(original).cropper('getCroppedCanvas', {width, height});
@@ -334,7 +339,13 @@ async function applyModifications(img, dataOptions = {}) {
     ctx.fillRect(0, 0, result.width, result.height);
 
     // Quality
-    return result.toDataURL(mimetype, quality / 100);
+    const dataURL = result.toDataURL(mimetype, quality / 100);
+    const newSize = getDataURLBinarySize(dataURL);
+    const originalSize = _getImageSizeFromCache(originalSrc);
+    const isChanged = !!perspective || !!glFilter ||
+        original.width !== result.width || original.height !== result.height ||
+        original.width !== croppedImg.width || original.height !== croppedImg.height;
+    return (isChanged || originalSize >= newSize) ? dataURL : await _loadImageDataURL(originalSrc);
 }
 
 /**
@@ -343,12 +354,30 @@ async function applyModifications(img, dataOptions = {}) {
  * @param {String} src URL of the image to load
  * @param {HTMLImageElement} [img] img element in which to load the image
  * @returns {Promise<HTMLImageElement>} Promise that resolves to the loaded img
+ *     or a placeholder image if the src is not found.
  */
-function loadImage(src, img = new Image()) {
+export function loadImage(src, img = new Image()) {
+    const handleImage = (source, resolve, reject) => {
+        img.addEventListener("load", () => resolve(img), {once: true});
+        img.addEventListener("error", reject, {once: true});
+        img.src = source;
+    };
+    // The server will return a placeholder image with the following src.
+    // grep: LOAD_IMAGE_404
+    const placeholderHref = "/web/image/__odoo__unknown__src__/";
+
     return new Promise((resolve, reject) => {
-        img.addEventListener('load', () => resolve(img), {once: true});
-        img.addEventListener('error', reject, {once: true});
-        img.src = src;
+        fetch(src)
+            .then(response => {
+                if (!response.ok) {
+                    src = placeholderHref;
+                }
+                handleImage(src, resolve, reject);
+            })
+            .catch(error => {
+                src = placeholderHref;
+                handleImage(src, resolve, reject);
+            });
     });
 }
 
@@ -391,9 +420,20 @@ async function _updateImageData(src, key = 'objectURL') {
     } else {
         value = URL.createObjectURL(blob);
     }
-    imageCache.set(src, Object.assign(currentImageData || {}, {[key]: value}));
+    imageCache.set(src, Object.assign(currentImageData || {}, {[key]: value, size: blob.size}));
     return value;
 }
+/**
+ * Returns the size of a cached image.
+ * Warning: this supposes that the image is already in the cache, i.e. that
+ * _updateImageData was called before.
+ *
+ * @param {String} src used as a key on the image cache map.
+ * @returns {Number} size of the image in bytes.
+ */
+function _getImageSizeFromCache(src) {
+    return imageCache.get(src).size;
+}
 /**
  * Activates the cropper on a given image.
  *
@@ -401,46 +441,77 @@ async function _updateImageData(src, key = 'objectURL') {
  * @param {Number} aspectRatio the aspectRatio of the crop box
  * @param {DOMStringMap} dataset dataset containing the cropperDataFields
  */
-async function activateCropper(image, aspectRatio, dataset) {
-    image.src = await _loadImageObjectURL(image.getAttribute('src'));
+export async function activateCropper(image, aspectRatio, dataset) {
+    const oldSrc = image.src;
+    const newSrc = await _loadImageObjectURL(image.getAttribute('src'));
+    image.src = newSrc;
     $(image).cropper({
         viewMode: 2,
         dragMode: 'move',
         autoCropArea: 1.0,
         aspectRatio: aspectRatio,
-        data: _.mapObject(_.pick(dataset, ...cropperDataFields), value => parseFloat(value)),
+        data: Object.fromEntries(Object.entries(pick(dataset, ...cropperDataFields))
+            .map(([key, value]) => [key, parseFloat(value)])),
         // Can't use 0 because it's falsy and cropperjs will then use its defaults (200x100)
         minContainerWidth: 1,
         minContainerHeight: 1,
     });
+    if (oldSrc === newSrc && image.complete) {
+        return;
+    }
     return new Promise(resolve => image.addEventListener('ready', resolve, {once: true}));
 }
 /**
  * Marks an <img> with its attachment data (originalId, originalSrc, mimetype)
  *
  * @param {HTMLImageElement} img the image whose attachment data should be found
- * @param {Function} rpc a function that can be used to make the RPC. Typically
- *   this would be passed as 'this._rpc.bind(this)' from widgets.
  * @param {string} [attachmentSrc=''] specifies the URL of the corresponding
  * attachment if it can't be found in the 'src' attribute.
  */
-async function loadImageInfo(img, rpc, attachmentSrc = '') {
+export async function loadImageInfo(img, attachmentSrc = '') {
     const src = attachmentSrc || img.getAttribute('src');
     // If there is a marked originalSrc, the data is already loaded.
-    if (img.dataset.originalSrc || !src) {
+    // If the image does not have the "mimetypeBeforeConversion" attribute, it
+    // has to be added.
+    if ((img.dataset.originalSrc && img.dataset.mimetypeBeforeConversion) || !src) {
         return;
     }
+    // In order to be robust to absolute, relative and protocol relative URLs,
+    // the src of the img is first converted to an URL object. To do so, the URL
+    // of the document in which the img is located is used as a base to build
+    // the URL object if the src of the img is a relative or protocol relative
+    // URL. The original attachment linked to the img is then retrieved thanks
+    // to the path of the built URL object.
+    let docHref = img.ownerDocument.defaultView.location.href;
+    if (docHref === "about:srcdoc") {
+        docHref = window.location.href;
+    }
 
-    const {original} = await rpc({
-        route: '/web_editor/get_image_info',
-        params: {src: src.split(/[?#]/)[0]},
-    });
-    // Check that url is local.
-    const isLocal = original && new URL(original.image_src, window.location.origin).origin === window.location.origin;
-    if (isLocal && original.image_src) {
+    const srcUrl = new URL(src, docHref);
+    const relativeSrc = srcUrl.pathname;
+
+    const {original} = await rpc('/web_editor/get_image_info', {src: relativeSrc});
+    // If src was an absolute "external" URL, we consider unlikely that its
+    // relative part matches something from the DB and even if it does, nothing
+    // bad happens, besides using this random image as the original when using
+    // the options, instead of having no option. Note that we do not want to
+    // check if the image is local or not here as a previous bug converted some
+    // local (relative src) images to absolute URL... and that before users had
+    // setup their website domain. That means they can have an absolute URL that
+    // looks like "https://mycompany.odoo.com/web/image/123" that leads to a
+    // "local" image even if the domain name is now "mycompany.be".
+    //
+    // The "redirect" check is for when it is a redirect image attachment due to
+    // an external URL upload.
+    if (original && original.image_src && !/\/web\/image\/\d+-redirect\//.test(original.image_src)) {
+        if (!img.dataset.mimetype) {
+            // The mimetype has to be added only if it is not already present as
+            // we want to avoid to reset a mimetype set by the user.
+            img.dataset.mimetype = original.mimetype;
+        }
         img.dataset.originalId = original.id;
         img.dataset.originalSrc = original.image_src;
-        img.dataset.mimetype = original.mimetype;
+        img.dataset.mimetypeBeforeConversion = original.mimetype;
     }
 }
 
@@ -450,17 +521,17 @@ async function loadImageInfo(img, rpc, attachmentSrc = '') {
  *     won't be accepted.
  * @returns {Boolean}
  */
-function isImageSupportedForProcessing(mimetype, strict = false) {
+export function isImageSupportedForProcessing(mimetype, strict = false) {
     if (isGif(mimetype)) {
         return !strict;
     }
-    return ['image/jpeg', 'image/png'].includes(mimetype);
+    return ['image/jpeg', 'image/png', 'image/webp'].includes(mimetype);
 }
 /**
  * @param {HTMLImageElement} img
  * @returns {Boolean}
  */
-function isImageSupportedForStyle(img) {
+export function isImageSupportedForStyle(img) {
     if (!img.parentElement) {
         return false;
     }
@@ -484,7 +555,7 @@ function isImageSupportedForStyle(img) {
  * @param {Blob} blob
  * @returns {Promise}
  */
-function createDataURL(blob) {
+export function createDataURL(blob) {
     return new Promise((resolve, reject) => {
         const reader = new FileReader();
         reader.addEventListener('load', () => resolve(reader.result));
@@ -494,16 +565,27 @@ function createDataURL(blob) {
     });
 }
 
-return {
+/**
+ * @param {String} dataURL
+ * @returns {Number} number of bytes represented with base64
+ */
+export function getDataURLBinarySize(dataURL) {
+    // Every 4 bytes of base64 represent 3 bytes.
+    return dataURL.split(',')[1].length / 4 * 3;
+}
+
+export const removeOnImageChangeAttrs = [...cropperDataFields, ...modifierFields];
+
+export default {
     applyModifications,
     cropperDataFields,
     activateCropper,
     loadImageInfo,
     loadImage,
-    removeOnImageChangeAttrs: [...cropperDataFields, ...modifierFields, 'aspectRatio'],
+    removeOnImageChangeAttrs,
     isImageSupportedForProcessing,
     isImageSupportedForStyle,
     createDataURL,
     isGif,
+    getDataURLBinarySize,
 };
-});
