@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
+from . import ir_attachment
+from . import ir_qweb_fields
 from . import res_config_settings
 from . import res_users
-from . import ir_qweb
