@@ -1,7 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import backend
+from . import binary
 from . import form
 from . import main
+from . import model_page
 from . import webclient
