@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import logging
@@ -7,6 +6,7 @@ import werkzeug
 
 from odoo import api, fields, models
 from odoo import tools
+from odoo.addons.website.tools import add_form_signature
 from odoo.exceptions import AccessError
 from odoo.osv import expression
 from odoo.http import request
@@ -21,9 +21,18 @@ class View(models.Model):
 
     website_id = fields.Many2one('website', ondelete='cascade', string="Website")
     page_ids = fields.One2many('website.page', 'view_id')
+    controller_page_ids = fields.One2many('website.controller.page', 'view_id')
     first_page_id = fields.Many2one('website.page', string='Website Page', help='First page linked to this view', compute='_compute_first_page_id')
     track = fields.Boolean(string='Track', default=False, help="Allow to specify for one page of the website to be trackable or not")
-    visibility = fields.Selection([('', 'All'), ('connected', 'Signed In'), ('restricted_group', 'Restricted Group'), ('password', 'With Password')], default='')
+    visibility = fields.Selection(
+        [
+            ('', 'Public'),
+            ('connected', 'Signed In'),
+            ('restricted_group', 'Restricted Group'),
+            ('password', 'With Password')
+        ],
+        default='',
+    )
     visibility_password = fields.Char(groups='base.group_system', copy=False)
     visibility_password_display = fields.Char(compute='_get_pwd', inverse='_set_pwd', groups='website.group_website_designer')
 
@@ -36,7 +45,7 @@ class View(models.Model):
         crypt_context = self.env.user._crypt_context()
         for r in self:
             if r.type == 'qweb':
-                r.sudo().visibility_password = r.visibility_password_display and crypt_context.encrypt(r.visibility_password_display) or ''
+                r.sudo().visibility_password = (r.visibility_password_display and crypt_context.hash(r.visibility_password_display)) or ''
                 r.visibility = r.visibility  # double check access
 
     def _compute_first_page_id(self):
@@ -69,19 +78,19 @@ class View(models.Model):
                     raise ValueError(f"Trying to create a view for website {new_website_id} from a website {website_id} environment")
         return super().create(vals_list)
 
-    def name_get(self):
+    @api.depends('website_id', 'key')
+    @api.depends_context('display_key', 'display_website')
+    def _compute_display_name(self):
         if not (self._context.get('display_key') or self._context.get('display_website')):
-            return super(View, self).name_get()
+            return super()._compute_display_name()
 
-        res = []
         for view in self:
             view_name = view.name
             if self._context.get('display_key'):
                 view_name += ' <%s>' % view.key
             if self._context.get('display_website') and view.website_id:
                 view_name += ' [%s]' % view.website_id.name
-            res.append((view.id, view_name))
-        return res
+            view.display_name = view_name
 
     def write(self, vals):
         '''COW for ir.ui.view. This way editing websites does not impact other
@@ -224,7 +233,7 @@ class View(models.Model):
                 specific_views += view._get_specific_views()
 
         result = super(View, self + specific_views).unlink()
-        self.clear_caches()
+        self.env.registry.clear_cache('templates')
         return result
 
     def _create_website_specific_pages_for_view(self, new_view, website):
@@ -236,9 +245,31 @@ class View(models.Model):
             })
             page.menu_ids.filtered(lambda m: m.website_id.id == website.id).page_id = new_page.id
 
-    def _get_top_level_view(self):
+    def get_view_hierarchy(self):
         self.ensure_one()
-        return self.inherit_id._get_top_level_view() if self.inherit_id else self
+        top_level_view = self
+        while top_level_view.inherit_id:
+            top_level_view = top_level_view.inherit_id
+        top_level_view = top_level_view.with_context(active_test=False)
+        sibling_views = top_level_view.search_read([('key', '=', top_level_view.key), ('id', '!=', top_level_view.id)])
+        return {
+            'sibling_views': sibling_views,
+            'hierarchy': top_level_view._build_hierarchy_datastructure()
+        }
+
+    def _build_hierarchy_datastructure(self):
+        inherit_children = []
+        for child in self.inherit_children_ids:
+            inherit_children.append(child._build_hierarchy_datastructure())
+        return {
+            'id': self.id,
+            'name': self.name,
+            'inherit_children': inherit_children,
+            'arch_updated': self.arch_updated,
+            'website_name': self.website_id.name if self.website_id else False,
+            'active': self.active,
+            'key': self.key,
+        }
 
     @api.model
     def get_related_views(self, key, bundles=False):
@@ -341,7 +372,7 @@ class View(models.Model):
                     """
 
     @api.model
-    @tools.ormcache_context('self.env.uid', 'self.env.su', 'xml_id', keys=('website_id',))
+    @tools.ormcache('self.env.uid', 'self.env.su', 'xml_id', 'self._context.get("website_id")', cache='templates')
     def _get_view_id(self, xml_id):
         """If a website_id is in the context and the given xml_id is not an int
         then try to get the id of the specific view for that website, but
@@ -352,7 +383,7 @@ class View(models.Model):
         method. `viewref` is probably more suitable.
 
         Archived views are ignored (unless the active_test context is set, but
-        then the ormcache_context will not work as expected).
+        then the ormcache will not work as expected).
         """
         website_id = self._context.get('website_id')
         if website_id and not isinstance(xml_id, int):
@@ -366,7 +397,7 @@ class View(models.Model):
             return view.id
         return super(View, self.sudo())._get_view_id(xml_id)
 
-    @tools.ormcache('self.id')
+    @tools.ormcache('self.id', cache='templates')
     def _get_cached_visibility(self):
         return self.visibility
 
@@ -470,6 +501,11 @@ class View(models.Model):
             'data-bg-video-src', 'data-shape', 'data-scroll-background-ratio',
         ]
 
+    def _get_combined_arch(self):
+        root = super()._get_combined_arch()
+        add_form_signature(root, self.sudo().env)
+        return root
+
     # --------------------------------------------------------------------------
     # Snippet saving
     # --------------------------------------------------------------------------
@@ -482,8 +518,8 @@ class View(models.Model):
             res['website_id'] = website_id
         return res
 
-    def _update_field_translations(self, fname, translations, digest=None):
-        return super(View, self.with_context(no_cow=True))._update_field_translations(fname, translations, digest)
+    def _update_field_translations(self, fname, translations, digest=None, source_lang=None):
+        return super(View, self.with_context(no_cow=True))._update_field_translations(fname, translations, digest=digest, source_lang=source_lang)
 
     def _get_base_lang(self):
         """ Returns the default language of the website as the base language if the record is bound to it """
