@@ -1,7 +1,7 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import api, fields, models
+from odoo import api, fields, models, _
+from odoo.exceptions import ValidationError
 
 
 class Company(models.Model):
@@ -19,6 +19,18 @@ class Company(models.Model):
         action['target'] = 'new'
         return action
 
+    @api.constrains('active')
+    def _check_active(self):
+        super()._check_active()
+        for company in self:
+            if not company.active and company.website_id:
+                raise ValidationError(_(
+                    'The company “%(company_name)s” cannot be archived because it has a linked website “%(website_name)s”.'
+                    '\nChange that website\'s company first.',
+                    company_name=company.name,
+                    website_name=company.website_id.name
+                ))
+
     def google_map_img(self, zoom=8, width=298, height=298):
         partner = self.sudo().partner_id
         return partner and partner.google_map_img(zoom, width, height) or None
