@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 import re
@@ -20,12 +19,12 @@ class WebsiteRoute(models.Model):
     path = fields.Char('Route')
 
     @api.model
-    def _name_search(self, name='', args=None, operator='ilike', limit=100, name_get_uid=None):
-        res = super(WebsiteRoute, self)._name_search(name=name, args=args, operator=operator, limit=limit, name_get_uid=name_get_uid)
-        if not len(res):
+    def _search_display_name(self, operator, value):
+        # in case we don't have results, refresh before returning the domain
+        domain = super()._search_display_name(operator, value)
+        if not self.search_count(domain, limit=1):
             self._refresh()
-            return super(WebsiteRoute, self)._name_search(name=name, args=args, operator=operator, limit=limit, name_get_uid=name_get_uid)
-        return res
+        return domain
 
     def _refresh(self):
         _logger.debug("Refreshing website.route")
@@ -93,24 +92,34 @@ class WebsiteRewrite(models.Model):
                     raise ValidationError(_('"URL to" must start with a leading slash.'))
                 for param in re.findall('/<.*?>', rewrite.url_from):
                     if param not in rewrite.url_to:
-                        raise ValidationError(_('"URL to" must contain parameter %s used in "URL from".') % param)
+                        raise ValidationError(_('"URL to" must contain parameter %s used in "URL from".', param))
                 for param in re.findall('/<.*?>', rewrite.url_to):
                     if param not in rewrite.url_from:
-                        raise ValidationError(_('"URL to" cannot contain parameter %s which is not used in "URL from".') % param)
+                        raise ValidationError(_('"URL to" cannot contain parameter %s which is not used in "URL from".', param))
+
+                if rewrite.url_to == '/':
+                    raise ValidationError(_('"URL to" cannot be set to "/". To change the homepage content, use the "Homepage URL" field in the website settings or the page properties on any custom page.'))
+
+                if any(
+                    rule for rule in self.env['ir.http'].routing_map().iter_rules()
+                    # Odoo routes are normally always defined without trailing
+                    # slashes + strict_slashes=False, but there are exceptions.
+                    if rule.rule.rstrip('/') == rewrite.url_to.rstrip('/')
+                ):
+                    raise ValidationError(_('"URL to" cannot be set to an existing page.'))
+
                 try:
                     converters = self.env['ir.http']._get_converters()
                     routing_map = werkzeug.routing.Map(strict_slashes=False, converters=converters)
                     rule = werkzeug.routing.Rule(rewrite.url_to)
                     routing_map.add(rule)
                 except ValueError as e:
-                    raise ValidationError(_('"URL to" is invalid: %s') % e)
+                    raise ValidationError(_('"URL to" is invalid: %s', e)) from e
 
-    def name_get(self):
-        result = []
+    @api.depends('redirect_type')
+    def _compute_display_name(self):
         for rewrite in self:
-            name = "%s - %s" % (rewrite.redirect_type, rewrite.name)
-            result.append((rewrite.id, name))
-        return result
+            rewrite.display_name = f"{rewrite.redirect_type} - {rewrite.name}"
 
     @api.model_create_multi
     def create(self, vals_list):
@@ -135,12 +144,12 @@ class WebsiteRewrite(models.Model):
         return res
 
     def _invalidate_routing(self):
-        # Call clear_caches on this worker to reload routing table.
+        # Call clear_cache for routing on all workers to reload routing table.
         # Note that only 404 and 308 redirection alter the routing map:
         # - 404: remove entry from routing map
         # - 301/302: served as fallback later if path not found in routing map
         # - 308: add "alias" (`redirect_to`) in routing map
-        self.env['ir.http'].clear_caches()
+        self.env.registry.clear_cache('routing')
 
     def refresh_routes(self):
         self.env['website.route']._refresh()
