@@ -1,29 +1,53 @@
 /** @odoo-module **/
 
+import { _t } from "@web/core/l10n/translation";
+import { browser } from '@web/core/browser/browser';
 import { registry } from '@web/core/registry';
+import { ResizablePanel } from '@web/core/resizable_panel/resizable_panel';
 import { useService, useBus } from '@web/core/utils/hooks';
-import core from 'web.core';
-import { AceEditorAdapterComponent } from '../../components/ace_editor/ace_editor';
+import { redirect } from "@web/core/utils/urls";
+import { ResourceEditor } from '../../components/resource_editor/resource_editor';
 import { WebsiteEditorComponent } from '../../components/editor/editor';
 import { WebsiteTranslator } from '../../components/translator/translator';
 import { unslugHtmlDataObject } from '../../services/website_service';
 import {OptimizeSEODialog} from '@website/components/dialog/seo';
-import { routeToUrl } from "@web/core/browser/router_service";
+import { WebsiteDialog } from "@website/components/dialog/dialog";
 import { getActiveHotkey } from "@web/core/hotkeys/hotkey_service";
-import wUtils from 'website.utils';
+import wUtils from '@website/js/utils';
+import { renderToElement } from "@web/core/utils/render";
+import { SIZES, utils as uiUtils } from "@web/core/ui/ui_service";
+import { standardActionServiceProps } from "@web/webclient/actions/action_service";
+import {
+    Component,
+    onWillStart,
+    onMounted,
+    onWillUnmount,
+    useRef,
+    useEffect,
+    useState,
+    useExternalListener,
+} from "@odoo/owl";
+import { getScrollingElement } from "@web/core/utils/scrolling";
 
-const { Component, onWillStart, onMounted, onWillUnmount, useRef, useEffect, useState } = owl;
-
-class BlockPreview extends Component {}
-BlockPreview.template = 'website.BlockPreview';
+class BlockPreview extends Component {
+    static template = "website.BlockPreview";
+    static props = {};
+}
 
 export class WebsitePreview extends Component {
+    static template = "website.WebsitePreview";
+    static components = {
+        WebsiteEditorComponent,
+        BlockPreview,
+        WebsiteTranslator,
+        ResourceEditor,
+        ResizablePanel,
+    };
+    static props = { ...standardActionServiceProps };
     setup() {
         this.websiteService = useService('website');
         this.dialogService = useService('dialog');
         this.title = useService('title');
-        this.user = useService('user');
-        this.router = useService('router');
         this.action = useService('action');
         this.orm = useService('orm');
 
@@ -44,6 +68,7 @@ export class WebsitePreview extends Component {
 
         useBus(this.websiteService.bus, 'BLOCK', (event) => this.block(event.detail));
         useBus(this.websiteService.bus, 'UNBLOCK', () => this.unblock());
+        useExternalListener(window, "keydown", this._onKeydownRefresh.bind(this));
 
         onWillStart(async () => {
             const [backendWebsiteRepr] = await Promise.all([
@@ -64,9 +89,17 @@ export class WebsitePreview extends Component {
                 // URL (event if it wasn't, it wouldn't be an issue as those are
                 // really considered as the same domain, the user will share the
                 // same session and CORS errors won't be a thing in such a case)
-                window.location.href = `${this.websiteDomain}/web#action=website.website_preview&path=${encodedPath}&website_id=${this.websiteId}`;
+                this.dialogService.add(WebsiteDialog, {
+                    title: _t("Redirecting..."),
+                    body: _t("You are about to be redirected to the domain configured for your website ( %s ). This is necessary to edit or view your website from the Website app. You might need to log back in.", this.websiteDomain),
+                    showSecondaryButton: false,
+                }, {
+                    onClose: () => {
+                        window.location.href = `${encodeURI(this.websiteDomain)}/odoo/action-website.website_preview?path=${encodedPath}&website_id=${encodeURIComponent(this.websiteId)}`;
+                    }
+                });
             } else {
-                this.initialUrl = `/website/force/${this.websiteId}?path=${encodedPath}`;
+                this.initialUrl = `/website/force/${encodeURIComponent(this.websiteId)}?path=${encodedPath}`;
             }
         });
 
@@ -75,6 +108,14 @@ export class WebsitePreview extends Component {
             if (this.isRestored) {
                 return;
             }
+
+            const isScreenLargeEnoughForEdit =
+                uiUtils.getSize() >= SIZES.MD;
+            if (!isScreenLargeEnoughForEdit && this.props.action.context.params) {
+                this.props.action.context.params.enable_editor = false;
+                this.props.action.context.params.with_loader = false;
+            }
+
             this.websiteService.context.showNewContentModal = this.props.action.context.params && this.props.action.context.params.display_new_content;
             this.websiteService.context.edition = this.props.action.context.params && !!this.props.action.context.params.enable_editor;
             this.websiteService.context.translation = this.props.action.context.params && !!this.props.action.context.params.edit_translations;
@@ -89,6 +130,14 @@ export class WebsitePreview extends Component {
             }
         }, () => [this.props.action.context.params]);
 
+        useEffect(() => {
+            this.websiteContext.showResourceEditor = false;
+        }, () => [
+            this.websiteContext.showNewContentModal,
+            this.websiteContext.edition,
+            this.websiteContext.translation,
+        ]);
+
         onMounted(() => {
             this.websiteService.blockPreview(true, 'load-iframe');
             this.iframe.el.addEventListener('load', () => this.websiteService.unblockPreview('load-iframe'), { once: true });
@@ -96,15 +145,10 @@ export class WebsitePreview extends Component {
             // OdooFrameContentLoaded event to unblock the iframe, as it is
             // triggered faster than the load event.
             this.iframe.el.addEventListener('OdooFrameContentLoaded', () => this.websiteService.unblockPreview('load-iframe'), { once: true });
-            this.env.services.messaging.modelManager.messagingCreatedPromise.then(() => {
-                this.env.services.messaging.modelManager.messaging.update({ isWebsitePreviewOpen: true });
-            });
         });
 
         onWillUnmount(() => {
-            this.env.services.messaging.modelManager.messagingCreatedPromise.then(() => {
-                this.env.services.messaging.modelManager.messaging.update({ isWebsitePreviewOpen: false });
-            });
+            this.websiteService.context.showResourceEditor = false;
             const { pathname, search, hash } = this.iframe.el.contentWindow.location;
             this.websiteService.lastUrl = `${pathname}${search}${hash}`;
             this.websiteService.currentWebsiteId = null;
@@ -122,10 +166,8 @@ export class WebsitePreview extends Component {
             const backendIconEl = document.querySelector("link[rel~='icon']");
             // Save initial backend values.
             const backendIconHref = backendIconEl.href;
-            const { zopenerp } = this.title.getParts();
             this.iframe.el.addEventListener('load', () => {
                 // Replace backend values with frontend's ones.
-                this.title.setParts({ zopenerp: null });
                 const frontendIconEl = this.iframe.el.contentDocument.querySelector("link[rel~='icon']");
                 if (frontendIconEl) {
                     backendIconEl.href = frontendIconEl.href;
@@ -133,49 +175,19 @@ export class WebsitePreview extends Component {
             }, { once: true });
             return () => {
                 // Restore backend initial values when leaving.
-                this.title.setParts({ zopenerp, action: null });
                 backendIconEl.href = backendIconHref;
             };
         }, () => []);
 
-        useEffect(() => {
-            let leftOnBackNavigation = false;
-            // When reaching a "regular" url of the webclient's router, an
-            // hashchange event should be dispatched to properly display the
-            // content of the previous URL before reaching the client action,
-            // which was lost after being replaced for the frontend's URL.
-            const handleBackNavigation = () => {
-                if (!window.location.pathname.startsWith('/@')) {
-                    window.dispatchEvent(new HashChangeEvent('hashchange', {
-                        newURL: window.location.href.toString()
-                    }));
-                    leftOnBackNavigation = true;
-                }
-            };
-            window.addEventListener('popstate', handleBackNavigation);
-            return () => {
-                window.removeEventListener('popstate', handleBackNavigation);
-                // When leaving the client action, its original url is pushed
-                // so that the router can replay the action on back navigation
-                // from other screens.
-                if (!leftOnBackNavigation) {
-                    history.pushState({}, null, this.backendUrl);
-                }
-            };
-        }, () => []);
-
         const toggleIsMobile = () => {
-            const wrapwrapEl = this.iframe.el.contentDocument.querySelector('#wrapwrap');
-            if (wrapwrapEl) {
-                wrapwrapEl.classList.toggle('o_is_mobile', this.websiteContext.isMobile);
-            }
+            this.iframe.el.contentDocument.documentElement
+                .classList.toggle('o_is_mobile', this.websiteContext.isMobile);
         };
-        // Toggle the 'o_is_mobile' class on the wrapwrap when 'isMobile'
-        // changes in the context. (e.g. Click on mobile preview buttons)
+        // Toggle the 'o_is_mobile' class when the context 'isMobile' changes
+        // (e.g. Click on mobile preview buttons).
         useEffect(toggleIsMobile, () => [this.websiteContext.isMobile]);
 
-        // Toggle the 'o_is_mobile' class on the wrapwrap according to
-        // 'isMobile' on iframe load.
+        // Toggle the 'o_is_mobile' class according to 'isMobile' on iframe load
         useEffect(() => {
             this.iframe.el.addEventListener('OdooFrameContentLoaded', toggleIsMobile);
             return () => this.iframe.el.removeEventListener('OdooFrameContentLoaded', toggleIsMobile);
@@ -218,7 +230,7 @@ export class WebsitePreview extends Component {
                 // ... otherwise, the path still needs to be normalized (as it
                 // would be if the given path was used as an href of a  <a/>
                 // element).
-                path = url.pathname + url.search + url.hash;
+                path = url.pathname + url.search;
             }
         } else {
             path = '/';
@@ -230,6 +242,11 @@ export class WebsitePreview extends Component {
         return false;
     }
 
+    get aceEditorWidth() {
+        const storedWidth = browser.localStorage.getItem("ace_editor_width");
+        return storedWidth ? parseInt(storedWidth) : 720;
+    }
+
     reloadIframe(url) {
         return new Promise((resolve, reject) => {
             this.websiteService.websiteRootInstance = undefined;
@@ -254,19 +271,21 @@ export class WebsitePreview extends Component {
 
     addWelcomeMessage() {
         if (this.websiteService.isRestrictedEditor) {
-            const $wrap = $(this.iframe.el.contentDocument.querySelector('#wrapwrap.homepage')).find('#wrap');
-            if ($wrap.length && $wrap.html().trim() === '') {
-                this.$welcomeMessage = $(core.qweb.render('website.homepage_editor_welcome_message'));
-                this.$welcomeMessage.addClass('o_homepage_editor_welcome_message');
-                this.$welcomeMessage.css('min-height', $wrap.parent('main').height() - ($wrap.outerHeight(true) - $wrap.height()));
-                $wrap.empty().append(this.$welcomeMessage);
+            const wrap = this.iframe.el.contentDocument.querySelector('#wrapwrap.homepage #wrap');
+            if (wrap && !wrap.innerHTML.trim()) {
+                this.welcomeMessage = renderToElement('website.homepage_editor_welcome_message');
+                this.welcomeMessage.classList.add('o_homepage_editor_welcome_message', 'h-100');
+                while (wrap.firstChild) {
+                    wrap.removeChild(wrap.lastChild);
+                }
+                wrap.append(this.welcomeMessage);
             }
         }
     }
 
     removeWelcomeMessage() {
-        if (this.$welcomeMessage) {
-            this.$welcomeMessage.detach();
+        if (this.welcomeMessage) {
+            this.welcomeMessage.remove();
         }
     }
 
@@ -279,11 +298,12 @@ export class WebsitePreview extends Component {
      * @private
      */
     _isTopWindowURL({ host, pathname }) {
-        const backendRoutes = ['/web', '/web/session/logout'];
+        const backendRoutes = ['/web', '/web/session/logout', '/odoo'];
         return host !== window.location.host
             || (pathname
                 && (backendRoutes.includes(pathname)
                     || pathname.startsWith('/@/')
+                    || pathname.startsWith('/odoo/')
                     || pathname.startsWith('/web/content/')
                     // This is defined here to avoid creating a
                     // website_documents module for just one patch.
@@ -291,27 +311,74 @@ export class WebsitePreview extends Component {
     }
 
     /**
-     * This replaces the browser url (/web#action=website...) with
+     * This replaces the browser url (/odoo/action-website...) with
      * the iframe's url (it is clearer for the user).
      */
     _replaceBrowserUrl() {
-        // The original /web#action=... url is saved to be pushed on top of the
-        // history when leaving the component, so that the webclient can
-        // correctly find back and replay the client action.
-        if (!this.backendUrl) {
-            this.backendUrl = routeToUrl(this.router.current);
+        if (!wUtils.isHTTPSorNakedDomainRedirection(this.iframe.el.contentWindow.location.origin, window.location.origin)) {
+            // If another domain ends up loading in the iframe (for example,
+            // if the iframe is being redirected and has no initial URL, so it
+            // loads "about:blank"), do not push that into the history
+            // state as that could prevent the user from going back and could
+            // trigger a traceback.
+            history.replaceState(history.state, document.title, '/odoo');
+            return;
         }
-        const currentUrl = new URL(this.iframe.el.contentDocument.location.href);
-        currentUrl.pathname = `/@${currentUrl.pathname}`;
-        this.currentTitle = this.iframe.el.contentDocument.title;
-        history.replaceState({}, this.currentTitle, currentUrl.href);
-        this.title.setParts({ action: this.currentTitle });
+        const currentTitle = this.iframe.el.contentDocument.title;
+        history.replaceState(history.state, currentTitle, this.iframe.el.contentDocument.location.href);
+        this.title.setParts({ action: currentTitle });
     }
 
-    _onPageLoaded() {
+    _onPageLoaded(ev) {
+        // FIX Chrome-only. If you have the backend in a language A but the
+        // website in English only, you can 1) modify a record's (event,
+        // product...) name in language A (say "New Name").
+        // 2) visit the page `/new-name-11` => the server will redirect you to
+        // the English page `/origin-11`, which is the only one existing.
+        // Chrome caches the redirection.
+        // 3) give the same name in English as in language A, try to visit
+        // => the server now wants to access `/new-name-11`
+        // => Chrome uses the cache to redirect `/new-name-11` to `/origin-11`,
+        // => the server tries to redirect to `/new-name-11` => loop.
+        // Chrome injects a "Too many redirects" layout in the iframe, which in
+        // turn raises a CORS error when the app tries to update the iframe.
+        // If we detect that behavior, we reload the iframe with a new query
+        // parameter, so that it's not cached for Chrome.
+        if (
+            navigator.userAgent.toLowerCase().includes("chrome")
+            && !this.iframe.el.src.includes("iframe_reload")
+        ) {
+            try {
+                /* eslint-disable no-unused-expressions */
+                this.iframe.el.contentWindow.location.href;
+            } catch (err) {
+                if (err.name === "SecurityError") {
+                    ev.stopImmediatePropagation();
+                    // Note that iframe's `src` is the URL used to start the
+                    // website preview, it's not sync'd with iframe navigation.
+                    const srcUrl = new URL(this.iframe.el.src);
+                    const pathUrl = new URL(srcUrl.searchParams.get("path"), srcUrl.origin);
+                    pathUrl.searchParams.set("iframe_reload", "1");
+                    srcUrl.searchParams.set("path", `${pathUrl.pathname}${pathUrl.search}`);
+                    // We could inject `pathUrl` directly but keep the same
+                    // expected URL format `/website/force/1?path=..`
+                    this.iframe.el.src = srcUrl.toString();
+                    return;
+                }
+            }
+        }
+        if (this.lastHiddenPageURL !== this.iframe.el.contentWindow.location.href) {
+            // Hide Ace Editor when moving to another page.
+            this.websiteService.context.showResourceEditor = false;
+            this.lastHiddenPageURL = undefined;
+        }
+        if (this.props.action.context.params?.with_loader) {
+            this.websiteService.hideLoader();
+            this.props.action.context.params.with_loader = false;
+        }
         this.iframe.el.contentWindow.addEventListener('beforeunload', this._onPageUnload.bind(this));
         this._replaceBrowserUrl();
-        this.iframe.el.contentWindow.addEventListener('hashchange', this._replaceBrowserUrl.bind(this));
+        this.iframe.el.contentWindow.addEventListener('popstate', this._replaceBrowserUrl.bind(this));
         this.iframe.el.contentWindow.addEventListener('pagehide', this._onPageHide.bind(this));
 
         this.websiteService.pageDocument = this.iframe.el.contentDocument;
@@ -340,6 +407,14 @@ export class WebsitePreview extends Component {
                 // Forward clicks to close backend client action's navbar
                 // dropdowns.
                 this.iframe.el.dispatchEvent(new MouseEvent('click', ev));
+            } else {
+                // When in edit mode, prevent the default behaviours of clicks
+                // as to avoid DOM changes not handled by the editor.
+                // (Such as clicking on a link that triggers navigating to
+                // another page.)
+                if (!ev.target.closest('#oe_manipulators')) {
+                    ev.preventDefault();
+                }
             }
 
             const linkEl = ev.target.closest('[href]');
@@ -350,14 +425,13 @@ export class WebsitePreview extends Component {
             const { href, target, classList } = linkEl;
             if (classList.contains('o_add_language')) {
                 ev.preventDefault();
-                // TODO: in master adapt the href in template to only be the
-                // return URL and use it directly here to pass to url_return
+                const searchParams = new URLSearchParams(href);
                 this.action.doAction('base.action_view_base_language_install', {
                     target: 'new',
                     additionalContext: {
                         params: {
                             website_id: this.websiteId,
-                            url_return: $.deparam(href).url_return || '/[lang]',
+                            url_return: searchParams.get("url_return"),
                         },
                     },
                 });
@@ -375,11 +449,14 @@ export class WebsitePreview extends Component {
                     },
                     reloadIframe: false,
                 });
-            } else if (href && target !== '_blank' && !isEditing && this._isTopWindowURL(linkEl)) {
-                ev.preventDefault();
-                this.router.redirect(href);
-            } else if (this.iframe.el.contentWindow.location.pathname !== new URL(href).pathname) {
-                this.websiteService.websiteRootInstance = undefined;
+            } else if (href && target !== '_blank' && !isEditing) {
+                if (this._isTopWindowURL(linkEl)) {
+                    ev.preventDefault();
+                    browser.location.assign(href);
+                } else if (this.iframe.el.contentWindow.location.pathname !== new URL(href).pathname) {
+                    // This scenario triggers a navigation inside the iframe.
+                    this.websiteService.websiteRootInstance = undefined;
+                }
             }
         });
         this.iframe.el.contentDocument.addEventListener('keydown', ev => {
@@ -388,6 +465,8 @@ export class WebsitePreview extends Component {
                 // CTRL-K from within the iframe.
                 ev.preventDefault();
             }
+            // Check if it's a refresh first as we want to prevent default in that case.
+            this._onKeydownRefresh(ev);
             this.iframe.el.dispatchEvent(new KeyboardEvent('keydown', ev));
         });
         this.iframe.el.contentDocument.addEventListener('keyup', ev => {
@@ -395,6 +474,25 @@ export class WebsitePreview extends Component {
         });
     }
 
+    /**
+     * This method is called when the page is unloaded to clean
+     * the iframefallback content.
+     */
+    _cleanIframeFallback() {
+        // Remove autoplay in all iframes urls so videos are not
+        // playing in the background
+        const iframesEl = this.iframefallback.el.contentDocument.querySelectorAll('iframe[src]:not([src=""])');
+        for (const iframeEl of iframesEl) {
+            const url = new URL(iframeEl.src);
+            url.searchParams.delete('autoplay');
+            iframeEl.src = url.toString();
+        }
+    }
+
+    _onResourceEditorResize(width) {
+        browser.localStorage.setItem("ace_editor_width", width);
+    }
+
     _onPageUnload() {
         this.iframe.el.setAttribute('is-ready', 'false');
         // Before leaving the iframe, its content is replicated on an
@@ -406,10 +504,12 @@ export class WebsitePreview extends Component {
         if (!this.websiteContext.edition && this.iframe.el.contentDocument.body && this.iframefallback.el) {
             this.iframefallback.el.contentDocument.body.replaceWith(this.iframe.el.contentDocument.body.cloneNode(true));
             this.iframefallback.el.classList.remove('d-none');
-            $().getScrollingElement(this.iframefallback.el.contentDocument)[0].scrollTop = $().getScrollingElement(this.iframe.el.contentDocument)[0].scrollTop;
+            getScrollingElement(this.iframefallback.el.contentDocument).scrollTop = getScrollingElement(this.iframe.el.contentDocument).scrollTop;
+            this._cleanIframeFallback();
         }
     }
     _onPageHide() {
+        this.lastHiddenPageURL = this.iframe.el && this.iframe.el.contentWindow.location.href;
         // Normally, at this point, the websiteRootInstance is already set to
         // `undefined`, as we want to do that as early as possible to prevent
         // the editor to be in an unstable state. But some events not managed
@@ -417,13 +517,27 @@ export class WebsitePreview extends Component {
         // it is set to undefined again.
         this.websiteService.websiteRootInstance = undefined;
     }
+    /**
+     * Handles refreshing while the website preview is active.
+     * Makes it possible to stay in the backend after an F5 or CTRL-R keypress.
+     *
+     * @param  {KeyboardEvent} ev
+     * @private
+     */
+    _onKeydownRefresh(ev) {
+        const hotkey = getActiveHotkey(ev);
+        if (hotkey !== 'control+r' && hotkey !== 'f5') {
+            return;
+        }
+        // The iframe isn't loaded yet: fallback to default refresh.
+        if (this.websiteService.contentWindow === undefined) {
+            return;
+        }
+        ev.preventDefault();
+        const path = this.websiteService.contentWindow.location;
+        const debugMode = this.env.debug ? `&debug=${this.env.debug}` : "";
+        redirect(`/odoo/action-website.website_preview?path=${encodeURIComponent(path)}${debugMode}`);
+    }
 }
-WebsitePreview.template = 'website.WebsitePreview';
-WebsitePreview.components = {
-    WebsiteEditorComponent,
-    BlockPreview,
-    WebsiteTranslator,
-    AceEditorAdapterComponent,
-};
 
 registry.category('actions').add('website_preview', WebsitePreview);
