@@ -1,9 +1,11 @@
-odoo.define('website.utils', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-
-const { qweb, _t } = core;
+import { intersection } from "@web/core/utils/arrays";
+import { _t } from "@web/core/l10n/translation";
+import { renderToElement } from "@web/core/utils/render";
+import { App, Component } from "@odoo/owl";
+import { getTemplate } from "@web/core/templates";
+import { UrlAutoComplete } from "@website/components/autocomplete_with_pages/url_autocomplete";
 
 /**
  * Allows to load anchors from a page.
@@ -22,9 +24,20 @@ function loadAnchors(url, body) {
             resolve();
         }
     }).then(function (response) {
-        return _.map($(response).find('[id][data-anchor=true]'), function (el) {
+        const anchors = $(response).find('[id][data-anchor=true], .modal[id][data-display="onClick"]').toArray().map((el) => {
             return '#' + el.id;
         });
+        // Always suggest the top and the bottom of the page as internal link
+        // anchor even if the header and the footer are not in the DOM. Indeed,
+        // the "scrollTo" function handles the scroll towards those elements
+        // even when they are not in the DOM.
+        if (!anchors.includes('#top')) {
+            anchors.unshift('#top');
+        }
+        if (!anchors.includes('#bottom')) {
+            anchors.push('#bottom');
+        }
+        return anchors;
     }).catch(error => {
         console.debug(error);
         return [];
@@ -34,77 +47,30 @@ function loadAnchors(url, body) {
 /**
  * Allows the given input to propose existing website URLs.
  *
- * @param {ServicesMixin|Widget} self - an element capable to trigger an RPC
- * @param {jQuery} $input
+ * @param {HTMLInputElement} input
  */
-function autocompleteWithPages(self, $input, options) {
-    $.widget("website.urlcomplete", $.ui.autocomplete, {
-        options: options || {},
-        _create: function () {
-            this._super();
-            this.widget().menu("option", "items", "> :not(.ui-autocomplete-category)");
-        },
-        _renderMenu: function (ul, items) {
-            const self = this;
-            items.forEach(item => {
-                if (item.separator) {
-                    self._renderSeparator(ul, item);
-                }
-                else {
-                    self._renderItem(ul, item);
-                }
-            });
-        },
-        _renderSeparator: function (ul, item) {
-            return $("<li class='ui-autocomplete-category fw-bold text-capitalize p-2'>")
-                   .append(`<div>${item.separator}</div>`)
-                   .appendTo(ul);
-        },
-        _renderItem: function (ul, item) {
-            return $("<li>")
-                   .data('ui-autocomplete-item', item)
-                   .append(`<div>${item.label}</div>`)
-                   .appendTo(ul);
-        },
-    });
-    $input.urlcomplete({
-        source: function (request, response) {
-            if (request.term[0] === '#') {
-                loadAnchors(request.term, options && options.body).then(function (anchors) {
-                    response(anchors);
-                });
-            } else if (request.term.startsWith('http') || request.term.length === 0) {
-                // avoid useless call to /website/get_suggested_links
-                response();
-            } else {
-                return self._rpc({
-                    route: '/website/get_suggested_links',
-                    params: {
-                        needle: request.term,
-                        limit: 15,
-                    }
-                }).then(function (res) {
-                    let choices = res.matching_pages;
-                    res.others.forEach(other => {
-                        if (other.values.length) {
-                            choices = choices.concat(
-                                [{separator: other.title}],
-                                other.values,
-                            );
-                        }
-                    });
-                    response(choices);
-                });
-            }
-        },
-        select: function (ev, ui) {
-            // choose url in dropdown with arrow change ev.target.value without trigger_up
-            // so cannot check here if value has been updated
-            ev.target.value = ui.item.value;
-            self.trigger_up('website_url_chosen');
-            ev.preventDefault();
+function autocompleteWithPages(input, options= {}) {
+    const owlApp = new App(UrlAutoComplete, {
+        env: Component.env,
+        dev: Component.env.debug,
+        getTemplate,
+        props: {
+            options,
+            loadAnchors,
+            targetDropdown: input,
         },
+        translatableAttributes: ["data-tooltip"],
+        translateFn: _t,
     });
+
+    const container = document.createElement("div");
+    container.classList.add("ui-widget", "ui-autocomplete", "ui-widget-content", "border-0");
+    document.body.appendChild(container);
+    owlApp.mount(container)
+    return () => {
+        owlApp.destroy();
+        container.remove();
+    }
 }
 
 /**
@@ -112,7 +78,7 @@ function autocompleteWithPages(self, $input, options) {
  * @param {jQuery} [$excluded]
  */
 function onceAllImagesLoaded($element, $excluded) {
-    var defs = _.map($element.find('img').addBack('img'), function (img) {
+    var defs = Array.from($element.find("img").addBack("img")).map((img) => {
         if (img.complete || $excluded && ($excluded.is(img) || $excluded.has(img).length)) {
             return; // Already loaded
         }
@@ -138,7 +104,7 @@ function prompt(options, _qweb) {
      *
      * Usage Ex:
      *
-     * website.prompt("What... is your quest ?").then(function (answer) {
+     * website.prompt("What... is your quest?").then(function (answer) {
      *     arthur.reply(answer || "To seek the Holy Grail.");
      * });
      *
@@ -164,10 +130,10 @@ function prompt(options, _qweb) {
             text: options
         };
     }
-    if (_.isUndefined(_qweb)) {
+    if (typeof _qweb === "undefined") {
         _qweb = 'website.prompt';
     }
-    options = _.extend({
+    options = Object.assign({
         window_title: '',
         field_name: '',
         'default': '', // dict notation for IE<9
@@ -176,13 +142,13 @@ function prompt(options, _qweb) {
         btn_secondary_title: _t('Cancel'),
     }, options || {});
 
-    var type = _.intersection(Object.keys(options), ['input', 'textarea', 'select']);
+    var type = intersection(Object.keys(options), ['input', 'textarea', 'select']);
     type = type.length ? type[0] : 'input';
     options.field_type = type;
     options.field_name = options.field_name || options[type];
 
     var def = new Promise(function (resolve, reject) {
-        var dialog = $(qweb.render(_qweb, options)).appendTo('body');
+        var dialog = $(renderToElement(_qweb, options)).appendTo('body');
         options.$dialog = dialog;
         var field = dialog.find(options.field_type).first();
         field.val(options['default']); // dict notation for IE<9
@@ -218,7 +184,7 @@ function prompt(options, _qweb) {
         });
         if (field.is('input[type="text"], select')) {
             field.keypress(function (e) {
-                if (e.which === 13) {
+                if (e.key === "Enter") {
                     e.preventDefault();
                     dialog.find('.btn-primary').trigger('click');
                 }
@@ -273,13 +239,14 @@ function sendRequest(route, params) {
     let form = document.createElement('form');
     form.setAttribute('action', route);
     form.setAttribute('method', params.method || 'POST');
-    const isInIframe = window.frameElement && window.frameElement.classList.contains('o_iframe');
-    if (isInIframe) {
+    // This is an exception for the 404 page create page button, in backend we
+    // want to open the response in the top window not in the iframe.
+    if (params.forceTopWindow) {
         form.setAttribute('target', '_top');
     }
 
-    if (core.csrf_token) {
-        _addInput(form, 'csrf_token', core.csrf_token);
+    if (odoo.csrf_token) {
+        _addInput(form, 'csrf_token', odoo.csrf_token);
     }
 
     for (const key in params) {
@@ -305,7 +272,33 @@ function sendRequest(route, params) {
  *      efficient in that second case.
  * @returns {Promise<string>} a base64 PNG (as result of a Promise)
  */
-async function svgToPNG(src) {
+export async function svgToPNG(src) {
+    return _exportToPNG(src, "svg+xml");
+}
+
+/**
+ * Converts a base64 WEBP into a base64 PNG.
+ *
+ * @param {string|HTMLImageElement} src - an URL to a WEBP or a *loaded* image
+ *     with such an URL. This allows the call to potentially be a bit more
+ *     efficient in that second case.
+ * @returns {Promise<string>} a base64 PNG (as result of a Promise)
+ */
+export async function webpToPNG(src) {
+    return _exportToPNG(src, "webp");
+}
+
+/**
+ * Converts a formatted base64 image into a base64 PNG.
+ *
+ * @private
+ * @param {string|HTMLImageElement} src - an URL to a image or a *loaded* image
+ *     with such an URL. This allows the call to potentially be a bit more
+ *     efficient in that second case.
+ * @param {string} format - the format of the image
+ * @returns {Promise<string>} a base64 PNG (as result of a Promise)
+ */
+async function _exportToPNG(src, format) {
     function checkImg(imgEl) {
         // Firefox does not support drawing SVG to canvas unless it has width
         // and height attributes set on the root <svg>.
@@ -334,7 +327,7 @@ async function svgToPNG(src) {
     return new Promise(resolve => {
         const imgEl = new Image();
         imgEl.onload = () => {
-            if (checkImg(imgEl)) {
+            if (format !== "svg+xml" || checkImg(imgEl)) {
                 resolve(imgEl);
                 return;
             }
@@ -378,7 +371,7 @@ async function svgToPNG(src) {
  *
  * @returns {HTMLIframeElement}
  */
-function generateGMapIframe() {
+export function generateGMapIframe() {
     const iframeEl = document.createElement('iframe');
     iframeEl.classList.add('s_map_embedded', 'o_not_editable');
     iframeEl.setAttribute('width', '100%');
@@ -388,6 +381,7 @@ function generateGMapIframe() {
     iframeEl.setAttribute('marginheight', '0');
     iframeEl.setAttribute('marginwidth', '0');
     iframeEl.setAttribute('src', 'about:blank');
+    iframeEl.setAttribute('aria-label', _t("Map"));
     return iframeEl;
 }
 
@@ -397,14 +391,112 @@ function generateGMapIframe() {
  * @param {DOMStringMap} dataset
  * @returns {string} a Google Maps URL
  */
-function generateGMapLink(dataset) {
+export function generateGMapLink(dataset) {
     return 'https://maps.google.com/maps?q=' + encodeURIComponent(dataset.mapAddress)
         + '&t=' + encodeURIComponent(dataset.mapType)
         + '&z=' + encodeURIComponent(dataset.mapZoom)
         + '&ie=UTF8&iwloc=&output=embed';
 }
 
-return {
+/**
+ * Checks if the edited content is currently previewed as in a mobile device.
+ *
+ * @param {Object} self - context object ("this")
+ * @returns {boolean}
+ */
+function isMobile(self) {
+    let isMobile;
+    self.trigger_up("service_context_get", {
+        callback: (ctx) => {
+            isMobile = ctx["isMobile"];
+        },
+    });
+
+    return isMobile;
+}
+
+/**
+ * Returns the parsed data coming from the data-for element for the given form.
+ *
+ * @param {string} formId
+ * @param {HTMLElement} parentEl
+ * @returns {Object|undefined} the parsed data
+ */
+function getParsedDataFor(formId, parentEl) {
+    const dataForEl = parentEl.querySelector(`[data-for='${formId}']`);
+    if (!dataForEl) {
+        return;
+    }
+    return JSON.parse(dataForEl.dataset.values
+        // replaces `True` by `true` if they are after `,` or `:` or `[`
+        .replace(/([,:\[]\s*)True/g, '$1true')
+        // replaces `False` and `None` by `""` if they are after `,` or `:` or `[`
+        .replace(/([,:\[]\s*)(False|None)/g, '$1""')
+        // replaces the `'` by `"` if they are before `,` or `:` or `]` or `}`
+        .replace(/'(\s*[,:\]}])/g, '"$1')
+        // replaces the `'` by `"` if they are after `{` or `[` or `,` or `:`
+        .replace(/([{\[:,]\s*)'/g, '$1"')
+    );
+}
+
+/**
+ * Deep clones children or parses a string into elements, with or without
+ * <script> elements.
+ *
+ * @param {DocumentFragment|HTMLElement|String} content
+ * @param {Boolean} [keepScripts=false] - whether to keep script tags or not.
+ * @returns {DocumentFragment}
+ */
+export function cloneContentEls(content, keepScripts = false) {
+    let copyFragment;
+    if (typeof content === "string") {
+        copyFragment = new Range().createContextualFragment(content);
+    } else {
+        copyFragment = new DocumentFragment();
+        const els = [...content.children].map(el => el.cloneNode(true));
+        copyFragment.append(...els);
+    }
+    if (!keepScripts) {
+        copyFragment.querySelectorAll("script").forEach(scriptEl => scriptEl.remove());
+    }
+    return copyFragment;
+}
+
+/**
+ * Checks SEO data and notifies if either the page title or description is not
+ * set.
+ *
+ * @param {Object} seo_data - The SEO data to check.
+ * @param {Component} OptimizeSEODialog - Dialog to be displayed
+ * @param {Object} services - Services object which will be used to display
+ * notifications and dialog.
+ */
+export function checkAndNotifySEO(seo_data, OptimizeSEODialog, services) {
+    if (seo_data) {
+        let message;
+        if (!seo_data.website_meta_title) {
+            message = _t("Page title not set.");
+        } else if (!seo_data.website_meta_description) {
+            message = _t("Page description not set.");
+        }
+        if (message) {
+            services.notification.add(message, {
+                type: "warning",
+                sticky: false,
+                buttons: [
+                    {
+                        name: _t("Optimize SEO"),
+                        onClick: () => {
+                            services.dialog.add(OptimizeSEODialog);
+                        },
+                    },
+                ],
+            });
+        }
+    }
+}
+
+export default {
     loadAnchors: loadAnchors,
     autocompleteWithPages: autocompleteWithPages,
     onceAllImagesLoaded: onceAllImagesLoaded,
@@ -413,7 +505,11 @@ return {
     websiteDomain: websiteDomain,
     isHTTPSorNakedDomainRedirection: isHTTPSorNakedDomainRedirection,
     svgToPNG: svgToPNG,
+    webpToPNG: webpToPNG,
     generateGMapIframe: generateGMapIframe,
     generateGMapLink: generateGMapLink,
+    isMobile: isMobile,
+    getParsedDataFor: getParsedDataFor,
+    cloneContentEls: cloneContentEls,
+    checkAndNotifySEO: checkAndNotifySEO,
 };
-});
