@@ -1,23 +1,20 @@
-odoo.define('website.s_chart_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-const {ColorpickerWidget} = require('web.Colorpicker');
-var options = require('web_editor.snippets.options');
-const weUtils = require('web_editor.utils');
-
-var _t = core._t;
+import { _t } from "@web/core/l10n/translation";
+import options from "@web_editor/js/editor/snippets.options";
+import weUtils from "@web_editor/js/common/utils";
+import { isCSSColor } from '@web/core/utils/colors';
 
 options.registry.InnerChart = options.Class.extend({
-    custom_events: _.extend({}, options.Class.prototype.custom_events, {
+    custom_events: Object.assign({}, options.Class.prototype.custom_events, {
         'get_custom_colors': '_onGetCustomColors',
     }),
-    events: _.extend({}, options.Class.prototype.events, {
+    events: Object.assign({}, options.Class.prototype.events, {
         'click we-button.add_column': '_onAddColumnClick',
         'click we-button.add_row': '_onAddRowClick',
         'click we-button.o_we_matrix_remove_col': '_onRemoveColumnClick',
         'click we-button.o_we_matrix_remove_row': '_onRemoveRowClick',
-        'blur we-matrix input': '_onMatrixInputFocusOut',
+        'input we-matrix input': '_onMatrixInputInput',
         'focus we-matrix input': '_onMatrixInputFocus',
     }),
 
@@ -69,9 +66,6 @@ options.registry.InnerChart = options.Class.extend({
 
         await this._super(...arguments);
 
-        // prevent the columns from becoming too small.
-        this.tableEl.classList.toggle('o_we_matrix_five_col', this.tableEl.querySelectorAll('tr:first-child th').length > 5);
-
         this.backSelectEl.querySelector('we-title').textContent = this._isPieChart() ? _t("Data Color") : _t("Dataset Color");
         this.borderSelectEl.querySelector('we-title').textContent = this._isPieChart() ? _t("Data Border") : _t("Dataset Border");
 
@@ -82,7 +76,7 @@ options.registry.InnerChart = options.Class.extend({
             const color = el.dataset.backgroundColor || el.dataset.borderColor;
             if (color) {
                 el.style.border = '2px solid';
-                el.style.borderColor = ColorpickerWidget.isCSSColor(color) ? color : weUtils.getCSSVariableValue(color, this.style);
+                el.style.borderColor = isCSSColor(color) ? color : weUtils.getCSSVariableValue(color, this.style);
             }
         });
     },
@@ -174,9 +168,13 @@ options.registry.InnerChart = options.Class.extend({
         } else {
             // Find max value from each row/column data
             const datasets = JSON.parse(dataset.data).datasets || [];
-            const dataValue = _.flatten(datasets.map(el => el.data.map(data => {
-                return !isNaN(parseInt(data)) ? parseInt(data) : 0;
-            })));
+            const dataValue = datasets
+                .map((el) => {
+                    return el.data.map((data) => {
+                        return !isNaN(parseInt(data)) ? parseInt(data) : 0;
+                    });
+                })
+                .flat();
             // When max value is not given and min value is greater than chart
             // data values
             if (minValue >= Math.max(...dataValue)) {
@@ -305,6 +303,9 @@ options.registry.InnerChart = options.Class.extend({
         const newEl = document.createElement(tag);
         const contentEl = document.createElement('input');
         contentEl.type = 'text';
+        if (tag === 'td') {
+            contentEl.type = 'number';
+        }
         contentEl.value = value || '';
         if (backgroundColor) {
             contentEl.dataset.backgroundColor = backgroundColor;
@@ -476,9 +477,8 @@ options.registry.InnerChart = options.Class.extend({
     },
     /**
      * @private
-     * @param {Event} ev
      */
-    _onMatrixInputFocusOut: function (ev) {
+    _onMatrixInputInput() {
         this._reloadGraph();
     },
     /**
@@ -505,4 +505,3 @@ options.registry.InnerChart = options.Class.extend({
         this.updateUI();
     },
 });
-});
