@@ -1,7 +1,7 @@
-odoo.define('website.s_dynamic_snippet_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const options = require('web_editor.snippets.options');
+import options from "@web_editor/js/editor/snippets.options";
+import { rpc } from "@web/core/network/rpc";
 
 const dynamicSnippetOptions = options.Class.extend({
     /**
@@ -52,16 +52,17 @@ const dynamicSnippetOptions = options.Class.extend({
         // Default values depend on the templates and filters available.
         // Therefore, they cannot be computed prior the start of the option.
         await this._setOptionsDefaultValues();
-        // TODO Remove in master: adapt dropped snippet template.
-        if (this.$target[0].classList.contains('d-none') && !this.$target[0].classList.contains('d-md-block')) {
-            // Remove the 'd-none' of the old template if it is not related to
-            // the visible on mobile option.
-            this.$target[0].classList.remove('d-none');
-        }
         // The target needs to be restarted when the correct
         // template values are applied (numberOfElements, rowPerSlide, etc.)
         return this._refreshPublicWidgets();
     },
+    /**
+     * @override
+     */
+    async start() {
+        await this._super(...arguments);
+        this.customTemplateData = JSON.parse(this.$target[0].dataset?.customTemplateData || "{}");
+    },
 
     //--------------------------------------------------------------------------
     // Options
@@ -81,10 +82,15 @@ const dynamicSnippetOptions = options.Class.extend({
         if (params.attributeName === 'templateKey' && previewMode === false) {
             this._templateUpdated(widgetValue, params.activeValue);
         }
-        // TODO adapt in master
-        if (params.attributeName === 'numberOfRecords' && previewMode === false) {
-            this.$target.get(0).dataset.forceMinimumMaxLimitTo16 = '1';
-        }
+    },
+    /**
+     * Saves the template data that will be handled later by the public widget.
+     *
+     * @see this.selectClass for parameters
+     */
+    customizeTemplateValues(previewMode, widgetValue, params) {
+        this.customTemplateData[params.customizeTemplateKey] = widgetValue === "true";
+        this.$target[0].dataset.customTemplateData = JSON.stringify(this.customTemplateData);
     },
 
     //--------------------------------------------------------------------------
@@ -139,6 +145,16 @@ const dynamicSnippetOptions = options.Class.extend({
 
         return this._super.apply(this, arguments);
     },
+    /**
+     * @override
+     * @private
+     */
+    _computeWidgetState(methodName, params) {
+        if (methodName === "customizeTemplateValues") {
+            return `${this.customTemplateData[params.customizeTemplateKey] || false}`;
+        }
+        return this._super(...arguments);
+    },
     /**
      * @override
      * @private
@@ -160,10 +176,14 @@ const dynamicSnippetOptions = options.Class.extend({
      * @returns {Promise}
      */
     async _fetchDynamicFilters() {
-        const dynamicFilters = await this._rpc({route: '/website/snippet/options_filters', params: {
+        const dynamicFilters = await rpc('/website/snippet/options_filters', {
             model_name: this.modelNameFilter,
             search_domain: this.contextualFilterDomain,
-        }});
+        });
+        if (!dynamicFilters.length) {
+            // Additional modules are needed for dynamic filters to be defined.
+            return;
+        }
         for (let index in dynamicFilters) {
             this.dynamicFilters[dynamicFilters[index].id] = dynamicFilters[index];
         }
@@ -181,9 +201,9 @@ const dynamicSnippetOptions = options.Class.extend({
         if (!filter) {
             return [];
         }
-        const dynamicFilterTemplates = await this._rpc({route: '/website/snippet/filter_templates', params: {
+        const dynamicFilterTemplates = await rpc('/website/snippet/filter_templates', {
             filter_name: filter.model_name.replaceAll('.', '_'),
-        }});
+        });
         for (let index in dynamicFilterTemplates) {
             this.dynamicFilterTemplates[dynamicFilterTemplates[index].key] = dynamicFilterTemplates[index];
         }
@@ -327,6 +347,11 @@ const dynamicSnippetOptions = options.Class.extend({
         } else {
             delete this.$target[0].dataset.extraClasses;
         }
+        if (template.columnClasses) {
+            this.$target[0].dataset.columnClasses = template.columnClasses;
+        } else {
+            delete this.$target[0].dataset.columnClasses;
+        }
     },
     /**
      * Sets the option value.
@@ -347,6 +372,8 @@ const dynamicSnippetOptions = options.Class.extend({
 });
 
 options.registry.dynamic_snippet = dynamicSnippetOptions;
-
-return dynamicSnippetOptions;
+options.registry.DynamicSnippetTitle = options.Class.extend({
+    forceNoDeleteButton: true,
 });
+
+export default dynamicSnippetOptions;
