@@ -1,19 +1,12 @@
-odoo.define('website.s_dynamic_snippet_carousel', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const config = require('web.config');
-const core = require('web.core');
-const publicWidget = require('web.public.widget');
-const DynamicSnippet = require('website.s_dynamic_snippet');
+import publicWidget from "@web/legacy/js/public/public_widget";
+import DynamicSnippet from "@website/snippets/s_dynamic_snippet/000";
+import { utils as uiUtils } from "@web/core/ui/ui_service";
 
 const DynamicSnippetCarousel = DynamicSnippet.extend({
     selector: '.s_dynamic_snippet_carousel',
-    xmlDependencies: (DynamicSnippet.prototype.xmlDependencies || []).concat(
-        ['/website/static/src/snippets/s_dynamic_snippet_carousel/000.xml']
-    ),
-
     /**
-     *
      * @override
      */
     init: function () {
@@ -26,21 +19,19 @@ const DynamicSnippetCarousel = DynamicSnippet.extend({
     //--------------------------------------------------------------------------
 
     /**
-     * Method to be overridden in child components in order to prepare QWeb
-     * options
-     * @private
+     * @override
      */
-    _getQWebRenderParams: function () {
+    _getQWebRenderOptions: function () {
         return Object.assign(
             this._super.apply(this, arguments),
             {
-                interval : parseInt(this.$target[0].dataset.carouselInterval),
+                interval: parseInt(this.el.dataset.carouselInterval),
+                rowPerSlide: parseInt(uiUtils.isSmall() ? 1 : this.el.dataset.rowPerSlide || 1),
+                arrowPosition: this.el.dataset.arrowPosition || '',
             },
         );
     },
-
 });
 publicWidget.registry.dynamic_snippet_carousel = DynamicSnippetCarousel;
 
-return DynamicSnippetCarousel;
-});
+export default DynamicSnippetCarousel;
