@@ -1,9 +1,16 @@
-odoo.define('website.s_facebook_page_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const options = require('web_editor.snippets.options');
+import { _t } from "@web/core/l10n/translation";
+import { pick } from "@web/core/utils/objects";
+import options from "@web_editor/js/editor/snippets.options";
 
 options.registry.facebookPage = options.Class.extend({
+    init() {
+        this._super(...arguments);
+        this.orm = this.bindService("orm");
+        this.notification = this.bindService("notification");
+    },
+
     /**
      * Initializes the required facebook page data to create the iframe.
      *
@@ -19,17 +26,13 @@ options.registry.facebookPage = options.Class.extend({
             width: 350,
             tabs: '',
             small_header: true,
-            hide_cover: true,
+            hide_cover: "true",
         };
-        this.fbData = _.defaults(_.pick(this.$target[0].dataset, _.keys(defaults)), defaults);
-
+        this.fbData = Object.assign({}, defaults, pick(this.$target[0].dataset, ...Object.keys(defaults)));
         if (!this.fbData.href) {
             // Fetches the default url for facebook page from website config
             var self = this;
-            defs.push(this._rpc({
-                model: 'website',
-                method: 'search_read',
-                args: [[], ['social_facebook']],
+            defs.push(this.orm.searchRead("website", [], ["social_facebook"], {
                 limit: 1,
             }).then(function (res) {
                 if (res) {
@@ -40,6 +43,12 @@ options.registry.facebookPage = options.Class.extend({
 
         return Promise.all(defs).then(() => this._markFbElement()).then(() => this._refreshPublicWidgets());
     },
+    /**
+     * @override
+     */
+    onBuilt() {
+        this.$target[0].querySelector('.o_facebook_page_preview')?.remove();
+    },
 
     //--------------------------------------------------------------------------
     // Options
@@ -69,7 +78,7 @@ options.registry.facebookPage = options.Class.extend({
             }
         } else {
             if (optionName === 'show_cover') {
-                this.fbData.hide_cover = !widgetValue;
+                this.fbData.hide_cover = widgetValue ? "false" : "true";
             } else {
                 this.fbData[optionName] = widgetValue;
             }
@@ -105,9 +114,9 @@ options.registry.facebookPage = options.Class.extend({
             } else {
                 this.fbData.height = 150;
             }
-            _.each(this.fbData, (value, key) => {
+            for (const [key, value] of Object.entries(this.fbData)) {
                 this.$target[0].dataset[key] = value;
-            });
+            }
         });
     },
     /**
@@ -121,7 +130,7 @@ options.registry.facebookPage = options.Class.extend({
                     return this.fbData.tabs.split(',').includes(optionName.replace(/^tab./, ''));
                 } else {
                     if (optionName === 'show_cover') {
-                        return !this.fbData.hide_cover;
+                        return this.fbData.hide_cover === "false";
                     }
                     return this.fbData[optionName];
                 }
@@ -149,7 +158,7 @@ options.registry.facebookPage = options.Class.extend({
         // The regex is kept as a huge one-liner for performance as it is
         // compiled once on script load. The only way to split it on several
         // lines is with the RegExp constructor, which is compiled on runtime.
-        const match = this.fbData.href.match(/^(https?:\/\/)?((www\.)?(fb|facebook)|(m\.)?facebook)\.com\/(((profile\.php\?id=|people\/[^/?#]+\/|(p\/)?[^/?#]+-)(?<id>[0-9]{15,16}))|(?<nameid>[\w.]+))($|[/?# ])/);
+        const match = this.fbData.href.trim().match(/^(https?:\/\/)?((www\.)?(fb|facebook)|(m\.)?facebook)\.com\/(((profile\.php\?id=|people\/([^/?#]+\/)?|(p\/)?[^/?#]+-)(?<id>[0-9]{12,16}))|(?<nameid>[\w.]+))($|[/?# ])/);
         if (match) {
             // Check if the page exists on Facebook or not
             const pageId = match.groups.nameid || match.groups.id;
@@ -160,12 +169,17 @@ options.registry.facebookPage = options.Class.extend({
                 } else {
                     this.fbData.id = "";
                     this.fbData.href = defaultURL;
+                    this.notification.add(_t("We couldn't find the Facebook page"), {
+                        type: "warning",
+                    });
                 }
             });
         }
         this.fbData.id = "";
         this.fbData.href = defaultURL;
+        this.notification.add(_t("You didn't provide a valid Facebook link"), {
+            type: "warning",
+        });
         return Promise.resolve();
     },
 });
-});
