@@ -1,7 +1,6 @@
-odoo.define('website.s_popup_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const options = require('web_editor.snippets.options');
+import options from "@web_editor/js/editor/snippets.options";
 
 options.registry.SnippetPopup = options.Class.extend({
     /**
@@ -32,6 +31,15 @@ options.registry.SnippetPopup = options.Class.extend({
         // sound). Stop the video, as the user can't do it (no button on video
         // in edit mode).
         this._removeIframeSrc();
+        if (!this.$target[0].parentElement.matches("#website_cookies_bar")) {
+            this.trigger_up("option_update", {
+                optionName: "anchor",
+                name: "modalAnchor",
+                data: {
+                    buttonEl: this._requestUserValueWidgets("onclick_opt")[0].el,
+                },
+            });
+        }
         return this._super(...arguments);
     },
     /**
@@ -95,14 +103,15 @@ options.registry.SnippetPopup = options.Class.extend({
     //--------------------------------------------------------------------------
 
     /**
-     * Moves the snippet in footer to be common to all pages
-     * or inside wrap to be on one page only
+     * Moves the snippet in #o_shared_blocks to be common to all pages or inside
+     * the first editable oe_structure in the main to be on current page only.
      *
      * @see this.selectClass for parameters
      */
     moveBlock: function (previewMode, widgetValue, params) {
-        const containerEl = this.$target[0].ownerDocument.querySelector(widgetValue === 'moveToFooter' ? 'footer' : 'main');
-        const whereEl = $(containerEl).find('.oe_structure:o_editable')[0];
+        const selector = widgetValue === 'allPages' ?
+            '#o_shared_blocks' : 'main .oe_structure:o_editable';
+        const whereEl = $(this.$target[0].ownerDocument).find(selector)[0];
         const popupEl = this.$target[0].closest('.s_popup');
         whereEl.prepend(popupEl);
     },
@@ -132,7 +141,7 @@ options.registry.SnippetPopup = options.Class.extend({
     _computeWidgetState: function (methodName, params) {
         switch (methodName) {
             case 'moveBlock':
-                return this.$target.closest('footer').length ? 'moveToFooter' : 'moveToBody';
+                return this.$target[0].closest('#o_shared_blocks') ? 'allPages' : 'currentPage';
         }
         return this._super(...arguments);
     },
@@ -148,4 +157,3 @@ options.registry.SnippetPopup = options.Class.extend({
         });
     },
 });
-});
