@@ -1,7 +1,6 @@
-odoo.define('website.s_table_of_content_options', function (require) {
-'use strict';
+/** @odoo-module **/
 
-const options = require('web_editor.snippets.options');
+import options from "@web_editor/js/editor/snippets.options";
 
 options.registry.TableOfContent = options.Class.extend({
     /**
@@ -10,6 +9,7 @@ options.registry.TableOfContent = options.Class.extend({
     start: function () {
         this.targetedElements = 'h1, h2';
         this.oldHeadingsEls = [];
+        this.oldHeadingsDesktopVisible = [];
         const $headings = this.$target.find(this.targetedElements);
         if ($headings.length > 0) {
             this._generateNav();
@@ -76,6 +76,19 @@ options.registry.TableOfContent = options.Class.extend({
             scrollSpyInstance.dispose();
         }
     },
+    /**
+     * Returns the TOC id and the heading id from a header element.
+     *
+     * @param {HTMLElement} headingEl - A header element of the TOC.
+     * @returns {Object}
+     */
+    _getTocAndHeadingId(headingEl) {
+        const match = /^table_of_content_heading_(\d+)_(\d+)$/.exec(headingEl.getAttribute("id"));
+        if (match) {
+            return { tocId: parseInt(match[1]), headingId: parseInt(match[2]) };
+        }
+        return { tocId: 0, headingId: 0 };
+    },
     /**
      * @private
      */
@@ -92,10 +105,12 @@ options.registry.TableOfContent = options.Class.extend({
         }
         this.options.wysiwyg && this.options.wysiwyg.odooEditor.unbreakableStepUnactive();
         const navEl = this.$target[0].querySelector('.s_table_of_content_navbar');
-        const headingsEls = this.$target.find(this.targetedElements).toArray()
-            .filter(el => !el.closest('.o_snippet_desktop_invisible'));
+        const headingsEls = this.$target.find(this.targetedElements).toArray();
         const areHeadingsEqual = this.oldHeadingsEls.length === headingsEls.length
-            && this.oldHeadingsEls.every((el, i) => el.isEqualNode(headingsEls[i]));
+            && this.oldHeadingsEls.every((el, i) =>
+                el.isEqualNode(headingsEls[i])
+                && this.oldHeadingsDesktopVisible[i] === !headingsEls[i].closest(".o_snippet_desktop_invisible")
+            );
         const areVisibilityIdsEqual = headingsEls.every((headingEl) => {
             const visibilityId = headingEl.closest('section').getAttribute('data-visibility-id');
             const matchingLinkEl = navEl.querySelector(`a[href="#${headingEl.getAttribute('id')}"]`);
@@ -117,21 +132,56 @@ options.registry.TableOfContent = options.Class.extend({
         }
         // We dispose the scrollSpy because the navbar will be updated.
         this._disposeScrollSpy();
+
+        const firstHeadingEl = headingsEls[0];
+        let tocId = firstHeadingEl ? this._getTocAndHeadingId(firstHeadingEl).tocId : 0;
+        const tocEls = this.$target[0].ownerDocument.body.querySelectorAll("[data-snippet='s_table_of_content']");
+        const otherTocEls = [...tocEls].filter(tocEl => tocEl !== this.$target[0]);
+        const otherTocIds = otherTocEls.map(tocEl => {
+            const firstHeadingEl = tocEl.querySelector(this.targetedElements);
+            return this._getTocAndHeadingId(firstHeadingEl).tocId;
+        });
+        if (!tocId || otherTocIds.includes(tocId)) {
+            tocId = 1 + Math.max(0, ...otherTocIds);
+        }
+        const headingIds = headingsEls.map(headingEl => this._getTocAndHeadingId(headingEl).headingId);
+        let maxHeadingIds = Math.max(0, ...headingIds);
+
         navEl.innerHTML = '';
-        _.each(headingsEls, el => {
+        const uniqueHeadingIds = new Set();
+        headingsEls.forEach((el) => {
             const $el = $(el);
-            const id = 'table_of_content_heading_' + _.now() + '_' + _.uniqueId();
-            const visibilityId = $el.closest('section').attr('data-visibility-id');
-            $('<a>').attr({ 'href': "#" + id, 'data-visibility-id': visibilityId })
-                    .addClass('table_of_content_link list-group-item list-group-item-action py-2 border-0 rounded-0')
-                    .text($el.text())
-                    .appendTo(navEl);
+            let headingId = this._getTocAndHeadingId(el).headingId;
+            if (headingId) {
+                // Reset headingId on duplicate.
+                if (uniqueHeadingIds.has(headingId)) {
+                    headingId = 0;
+                } else {
+                    uniqueHeadingIds.add(headingId);
+                }
+            }
+            if (!headingId) {
+                maxHeadingIds += 1;
+                headingId = maxHeadingIds;
+            }
+            // Generate stable ids so that external links to heading anchors do
+            // not get broken next time the navigation links are re-generated.
+            const id = `table_of_content_heading_${tocId}_${headingId}`;
             $el.attr('id', id);
-            $el[0].dataset.anchor = 'true';
+            if (!el.closest('.o_snippet_desktop_invisible')) {
+                // Generate navigation entry only for desktop.
+                const visibilityId = $el.closest('section').attr('data-visibility-id');
+                $('<a>').attr({ 'href': "#" + id, 'data-visibility-id': visibilityId })
+                        .addClass('table_of_content_link list-group-item list-group-item-action py-2 border-0 rounded-0')
+                        .text($el.text())
+                        .appendTo(navEl);
+                $el[0].dataset.anchor = 'true';
+            }
         });
         const exception = (tocEl) => !tocEl.querySelector('.s_table_of_content_navbar a');
         this._activateScrollSpy(exception);
         this.oldHeadingsEls = [...headingsEls.map(el => el.cloneNode(true))];
+        this.oldHeadingsDesktopVisible = [...headingsEls.map(el => !el.closest('.o_snippet_desktop_invisible'))];
     },
 });
 
@@ -192,14 +242,4 @@ options.registry.TableOfContentNavbar = options.Class.extend({
 
 options.registry.TableOfContentMainColumns = options.Class.extend({
     forceNoDeleteButton: true,
-
-    /**
-     * @override
-     */
-    start: function () {
-        const leftPanelEl = this.$overlay.data('$optionsSection')[0];
-        leftPanelEl.querySelector('.oe_snippet_clone').classList.add('d-none'); // TODO improve the way to do that
-        return this._super.apply(this, arguments);
-    },
-});
 });
