@@ -1,22 +1,21 @@
 /** @odoo-module **/
 
-import tour from "web_tour.tour";
-import wTourUtils from "website.tour_utils";
+import { registry } from "@web/core/registry";
+import { clickOnEditAndWaitEditModeInTranslatedPage } from "@website/js/tours/tour_utils";
 
-tour.register('edit_translated_page_redirect', {
-    test: true,
+registry.category("web_tour.tours").add('edit_translated_page_redirect', {
     url: '/nl/contactus',
-}, [
+    steps: () => [
     {
         content: "Enter backend",
         trigger: 'a.o_frontend_to_backend_edit_btn',
+        run: "click",
     },
     {
         content: "Check the data-for attribute",
-        trigger: 'iframe main:has([data-for="contactus_form"])',
-        run: () => {}, // it's a check
+        trigger: ':iframe main:has([data-for="contactus_form"])',
     },
-    ...wTourUtils.clickOnEditAndWaitEditMode(),
+    ...clickOnEditAndWaitEditModeInTranslatedPage(),
     {
         content: "Go to /nl",
         trigger: "body",
@@ -30,6 +29,7 @@ tour.register('edit_translated_page_redirect', {
     {
         content: "Enter backend",
         trigger: 'a.o_frontend_to_backend_edit_btn',
+        run: "click",
     },
-    ...wTourUtils.clickOnEditAndWaitEditMode(),
-]);
+    ...clickOnEditAndWaitEditModeInTranslatedPage(),
+]});
