@@ -1,46 +1,49 @@
 /** @odoo-module */
 
-import wTourUtils from "website.tour_utils";
+import {
+    insertSnippet,
+    registerWebsitePreviewTour,
+} from '@website/js/tours/tour_utils';
 
-wTourUtils.registerWebsitePreviewTour("text_animations", {
-    test: true,
+registerWebsitePreviewTour("text_animations", {
     url: "/",
     edition: true,
-}, [
-    wTourUtils.dragNDrop({
+}, () => [
+    ...insertSnippet({
         id: "s_cover",
         name: "Cover",
+        groupName: "Intro",
     }),
     {
         content: "Click on the snippet title",
-        trigger: "iframe .s_cover h1 > font",
+        trigger: ":iframe .s_cover h1",
         run: "dblclick", // Make sure the title is selected.
     },
     {
         content: "Click on the 'Animate Text' button to activate the option",
         trigger: "div.o_we_animate_text",
+        run: "click",
     },
     {
         content: "Check that the animation was applied",
-        trigger: "iframe .s_cover h1 span.o_animated_text",
-        run: () => null, // it's a check
+        trigger: ":iframe .s_cover h1 span.o_animated_text",
     },
     {
         content: "Click on the 'Animate Text' button",
         trigger: "div.o_we_animate_text",
+        run: "click",
     },
     {
         content: "Check that the animation was disabled for the title",
-        trigger: "iframe .s_cover:not(:has(.o_animated_text))",
-        run: () => null, // it's a check
+        trigger: ":iframe .s_cover:not(:has(.o_animated_text))",
     },
     {
         content: "Try to apply the text animation again",
         trigger: "div.o_we_animate_text",
+        run: "click",
     },
     {
         content: "Check that the animation was applied",
-        trigger: "iframe .s_cover:has(span.o_animated_text)",
-        run: () => null, // it's a check
+        trigger: ":iframe .s_cover:has(span.o_animated_text)",
     },
 ]);
