@@ -1,14 +1,15 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from unittest.mock import patch
-from odoo.http import root
 from lxml import html
+from unittest.mock import patch
 
+from odoo.addons.website.controllers.main import Website
+from odoo.addons.website.tools import MockRequest
+from odoo.fields import Command
+from odoo.http import root
 from odoo.tests import common, HttpCase, tagged
 from odoo.tests.common import HOST
 from odoo.tools import config, mute_logger
-from odoo.addons.website.tools import MockRequest
-from odoo.fields import Command
 
 
 @tagged('-at_install', 'post_install')
@@ -293,7 +294,7 @@ class WithContext(HttpCase):
         # Set another page (/page_1) as homepage
         website.write({
             'homepage_url': self.page.url,
-            'domain': f"http://{HOST}:{config['http_port']}",
+            'domain': self.base_url(),
         })
         assert self.page.url != '/'
 
@@ -305,12 +306,64 @@ class WithContext(HttpCase):
         canonical_url = root_html.xpath('//link[@rel="canonical"]')[0].attrib['href']
         self.assertIn(canonical_url, [f"{website.domain}/", f"{website.domain}/page_1"])
 
+    def test_website_homepage_url_change(self):
+        website = self.env['website'].browse([1])
+        self.assertFalse(website.homepage_url)
+
+        test_page = self.env['website.page'].with_context(website_id=website.id).create({
+            'name': 'HomepageUrlTest',
+            'type': 'qweb',
+            'arch': '<div>HomepageUrlTest</div>',
+            'key': 'test.homepage_url_test',
+            'url': '/homepage_url_test',
+            'is_published': True,
+            'website_id': website.id
+        })
+        self.assertURLEqual(test_page.url, '/homepage_url_test')
+
+        # If one has set the `homepage_url` to a specific page URL..
+        website.write({
+            'name': 'Test Website',
+            'domain': self.base_url(),
+            'homepage_url': test_page.url,
+        })
+        home_url_full = website.domain + '/'
+        r = self.url_open('/')
+        self.assertEqual(r.status_code, 200)
+        self.assertURLEqual(r.url, home_url_full)
+        self.assertIn(b"HomepageUrlTest", r.content)
+
+        # .. and then change that page URL ..
+        with MockRequest(self.env, website=website):
+            test_page.url = '/url-changed'
+
+        # .. the `homepage_url` should be changed to follow the new page URL
+        self.assertEqual(website.homepage_url, '/url-changed')
+        r = self.url_open('/')
+        self.assertEqual(r.status_code, 200)
+        self.assertEqual(
+            r.url, home_url_full, """URL should still be '/', note that if this
+            `assert` fail, the loaded URL will probably be the first available
+            menu different from '/', see homepage controller.""")
+        self.assertIn(b"HomepageUrlTest", r.content)
+
+        # Side test: ensure `slugify` and `get_unique_path` changes are
+        # correctly replicated in the synced website homepage_url
+        with MockRequest(self.env, website=website):
+            # `/url-changed_two` will become `/url-changed-two`
+            test_page.url = '/url-changed_two'
+        self.assertEqual(website.homepage_url, '/url-changed-two')
+        r = self.url_open('/')
+        self.assertEqual(r.status_code, 200)
+        self.assertURLEqual(r.url, home_url_full)
+        self.assertIn(b"HomepageUrlTest", r.content)
+
     def test_06_homepage_url(self):
         # Setup
         website = self.env['website'].browse([1])
         website.write({
             'name': 'Test Website',
-            'domain': f'http://{HOST}:{config["http_port"]}',
+            'domain': self.base_url(),
             'homepage_url': False,
         })
         contactus_url = '/contactus'
@@ -336,7 +389,7 @@ class WithContext(HttpCase):
         # -------------------------------------------
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertEqual(r.url, home_url_full)
+        self.assertURLEqual(r.url, home_url_full)
         self.assertIn(home_content, r.content)
 
         # Case 2: Another page as homepage
@@ -348,7 +401,7 @@ class WithContext(HttpCase):
         # -------------------------------------------
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertEqual(r.url, home_url_full)
+        self.assertURLEqual(r.url, home_url_full)
         self.assertIn(contactus_content, r.content)
 
         # Case 3: Check we don't fallback on first menu if there is a / page
@@ -361,7 +414,7 @@ class WithContext(HttpCase):
         # -------------------------------------------
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertEqual(r.url, home_url_full)
+        self.assertURLEqual(r.url, home_url_full)
         self.assertIn(home_content, r.content)
 
         # Case 6: Wrong URL should fallback on first non "/" menu
@@ -378,7 +431,7 @@ class WithContext(HttpCase):
         self.assertEqual(r.status_code, 404, "The website homepage_url should be a 404")
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertEqual(r.url, contactus_url_full, "Menu fallback should be a redirect, not a reroute")
+        self.assertURLEqual(r.url, contactus_url_full, "Menu fallback should be a redirect, not a reroute")
         self.assertIn(contactus_content, r.content)
 
         # Case 4: Check first menu fallback is a redirect (and not a reroute)
@@ -392,7 +445,7 @@ class WithContext(HttpCase):
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
         self.assertEqual(r.history[0].status_code, 303)
-        self.assertEqual(r.url, contactus_url_full)
+        self.assertURLEqual(r.url, contactus_url_full)
         self.assertIn(contactus_content, r.content)
 
         # Case 5: Check controller redirect and make sure it is a reroute
@@ -404,7 +457,7 @@ class WithContext(HttpCase):
         # -------------------------------------------
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertEqual(r.url, home_url_full)
+        self.assertURLEqual(r.url, home_url_full)
         self.assertIn(b'o_website_info', r.content)
 
         # Case 6: Check controller redirect which has different `auth` method
@@ -416,9 +469,9 @@ class WithContext(HttpCase):
         # -------------------------------------------
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertNotIn(b'<title> My Portal', r.content)
-        self.assertIn(b'<title> Contact Us', r.content)
-        self.assertEqual(r.url, contactus_url_full)
+        self.assertNotIn(b'<title>My Portal', r.content)
+        self.assertIn(b'<title>Contact Us', r.content)
+        self.assertURLEqual(r.url, contactus_url_full)
         self.assertEqual(r.history[0].status_code, 303)
         # Now with /contactus which is a public content
         self.env['website.menu'].create({
@@ -430,8 +483,8 @@ class WithContext(HttpCase):
         })
         r = self.url_open(home_url)
         self.assertEqual(r.status_code, 200)
-        self.assertNotIn(b'<title> My Portal', r.content)
-        self.assertIn(b'<title> Login', r.content)
+        self.assertNotIn(b'<title>My Portal', r.content)
+        self.assertIn(b'<title>Login', r.content)
         self.assertIn('/web/login?redirect', r.url)
         self.assertEqual(r.history[0].status_code, 303)
 
@@ -468,6 +521,36 @@ class WithContext(HttpCase):
             self.assertEqual(alternate_en_url, f'{self.base_url()}/page_1')
             self.assertEqual(alternate_fr_url, f'{self.base_url()}/fr/page_1')
 
+    def test_alternate_hreflang(self):
+        website = self.env['website'].browse(1)
+        lang_en = self.env.ref('base.lang_en')
+        lang_fr = self.env['res.lang']._activate_lang('fr_FR')
+        with MockRequest(self.env, website=website):
+            # Only one region per lang, the hreflang should be the short code
+            website.language_ids = [Command.set((lang_en + lang_fr).ids)]
+            langs = self.env['res.lang']._get_frontend()
+            self.assertEqual(langs['en_US']['hreflang'], 'en')
+            self.assertEqual(langs['fr_FR']['hreflang'], 'fr')
+            # Multiple regions per lang, one lang from the same region should be
+            # the short code, others should keep the full code
+            lang_be = self.env['res.lang']._activate_lang('fr_BE')
+            lang_ca = self.env['res.lang']._activate_lang('fr_CA')
+            website.language_ids = [Command.set((lang_en + lang_fr + lang_be + lang_ca).ids)]
+            langs = self.env['res.lang']._get_frontend()
+            self.assertEqual(langs['en_US']['hreflang'], 'en')
+            self.assertEqual(langs['fr_FR']['hreflang'], 'fr-fr')
+            self.assertEqual(langs['fr_BE']['hreflang'], 'fr')
+            self.assertEqual(langs['fr_CA']['hreflang'], 'fr-ca')
+            # Special case for es_419: if there is multiple regions for spanish,
+            # including es_419, es_419 should be the one shortened
+            lang_es = self.env['res.lang']._activate_lang('es_ES')
+            lang_419 = self.env['res.lang']._activate_lang('es_419')
+            website.language_ids = [Command.set((lang_en + lang_es + lang_419).ids)]
+            langs = self.env['res.lang']._get_frontend()
+            self.assertEqual(langs['en_US']['hreflang'], 'en')
+            self.assertEqual(langs['es_ES']['hreflang'], 'es-es')
+            self.assertEqual(langs['es_419']['hreflang'], 'es')
+
     def test_07_not_authorized(self):
         # Create page that requires specific user role.
         specific_page = self.page.copy({'website_id': self.env['website'].get_current_website().id})
@@ -489,4 +572,42 @@ class WithContext(HttpCase):
         self.assertEqual(r.status_code, 200, "Reaching page URL, common case")
         r2 = self.url_open('/Page_1', allow_redirects=False)
         self.assertEqual(r2.status_code, 303, "URL exists only in different casing, should redirect to it")
-        self.assertTrue(r2.headers.get('Location').endswith('/page_1'), "Should redirect /Page_1 to /page_1")
+        self.assertURLEqual(r2.headers.get('Location'), '/page_1', "Should redirect /Page_1 to /page_1")
+
+    def test_page_generic_diverged_url(self):
+        """ When a generic page is COW and the new COW has its url changed, the
+        generic should not be reachable anymore even if the COW page has a
+        different URL. Note that they will both still share the same key.
+        """
+        Page = self.env['website.page']
+        specific_arch = '<div>website 1 content</div>'
+        generic_page = self.page
+        generic_page.arch = '<div>content</div>'
+
+        specific_page = Page.search([('url', '=', self.page.url), ('website_id', '=', 1)])
+        self.assertFalse(specific_page, "For this test, the specific page should not exist yet")
+
+        # COW a generic page
+        generic_page.view_id.with_context(website_id=1).save(specific_arch, xpath='/div')
+        specific_page = Page.search([('url', '=', self.page.url), ('website_id', '=', 1)])
+        self.assertEqual(specific_page.arch.replace('\n', ''), specific_arch)
+        self.assertEqual(generic_page.arch, '<div>content</div>')
+        # Change the URL of the specific page
+        specific_page.url = '/page_1_specific'
+        # Check that the generic page is not reachable anymore
+        r = self.url_open(specific_page.url)
+        self.assertEqual(r.status_code, 200, "Specific should be reachable")
+        r = self.url_open(generic_page.url)
+        self.assertEqual(r.status_code, 404, "Generic should not be reachable")
+
+
+@tagged('-at_install', 'post_install')
+class TestNewPage(common.TransactionCase):
+    def test_new_page_used_key(self):
+        website = self.env.ref('website.default_website')
+        controller = Website()
+        with MockRequest(self.env, website=website):
+            controller.pagenew(path="snippets")
+        pages = self.env['website.page'].search([('url', '=', '/snippets')])
+        self.assertEqual(len(pages), 1, "Exactly one page should be at /snippets.")
+        self.assertNotEqual(pages.key, "website.snippets", "Page's key cannot be website.snippets.")
