@@ -1,4 +1,3 @@
-# -*- encoding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, fields, models
@@ -22,14 +21,10 @@ class BaseLanguageInstall(models.TransientModel):
 
     def lang_install(self):
         action = super(BaseLanguageInstall, self).lang_install()
-        lang = self.env['res.lang']._lang_get(self.lang)
-        if self.website_ids and lang:
-            self.website_ids.write({'language_ids': [(4, lang.id)]})
+        if self.website_ids and self.lang_ids:
+            self.website_ids.language_ids |= self.lang_ids
         params = self._context.get('params', {})
         if 'url_return' in params:
-            return {
-                'url': params['url_return'].replace('[lang]', self.lang),
-                'type': 'ir.actions.act_url',
-                'target': 'self'
-            }
+            url = params['url_return'].replace('[lang]', self.first_lang_id.code)
+            return self.env['website'].get_client_action(url)
         return action
