@@ -1,19 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import re
 import werkzeug
 import itertools
 import pytz
 import babel.dates
-from collections import OrderedDict
+from collections import defaultdict
 
-from odoo import http, fields, tools
-from odoo.addons.http_routing.models.ir_http import slug, unslug
+from odoo import http, fields, tools, models
 from odoo.addons.website.controllers.main import QueryURL
-from odoo.addons.portal.controllers.portal import _build_url_w_params
 from odoo.http import request
-from odoo.osv import expression
 from odoo.tools import html2plaintext
 from odoo.tools.misc import get_lang
 from odoo.tools import sql
@@ -30,33 +26,46 @@ class WebsiteBlog(http.Controller):
         else:
             tag_ids.append(current_tag)
         tag_ids = request.env['blog.tag'].browse(tag_ids)
-        return ','.join(slug(tag) for tag in tag_ids)
+        return ','.join(request.env['ir.http']._slug(tag) for tag in tag_ids)
 
     def nav_list(self, blog=None):
         dom = blog and [('blog_id', '=', blog.id)] or []
         if not request.env.user.has_group('website.group_website_designer'):
             dom += [('post_date', '<=', fields.Datetime.now())]
-        groups = request.env['blog.post']._read_group_raw(
-            dom,
-            ['name', 'post_date'],
-            groupby=["post_date"], orderby="post_date desc")
-        for group in groups:
-            (r, label) = group['post_date']
-            start, end = r.split('/')
-            group['post_date'] = label
-            group['date_begin'] = start
-            group['date_end'] = end
+        groups = request.env['blog.post']._read_group(
+            dom, groupby=['post_date:month'])
 
-            locale = get_lang(request.env).code
-            start = pytz.UTC.localize(fields.Datetime.from_string(start))
-            tzinfo = pytz.timezone(request.context.get('tz', 'utc') or 'utc')
+        locale = get_lang(request.env).code
+        tzinfo = pytz.timezone(request.context.get('tz', 'utc') or 'utc')
+        fmt = tools.DEFAULT_SERVER_DATETIME_FORMAT
 
-            group['month'] = babel.dates.format_datetime(start, format='MMMM', tzinfo=tzinfo, locale=locale)
-            group['year'] = babel.dates.format_datetime(start, format='yyyy', tzinfo=tzinfo, locale=locale)
+        res = defaultdict(list)
+        for [start] in groups:
+            year = babel.dates.format_datetime(start, format='yyyy', tzinfo=tzinfo, locale=locale)
+            res[year].append({
+                'date_begin': start.strftime(fmt),
+                'date_end': (start + models.READ_GROUP_TIME_GRANULARITY['month']).strftime(fmt),
+                'month': babel.dates.format_datetime(start, format='MMMM', tzinfo=tzinfo, locale=locale),
+                'year': year,
+            })
+        return res
 
-        return OrderedDict((year, [m for m in months]) for year, months in itertools.groupby(groups, lambda g: g['year']))
+    def _get_blog_post_search_options(self, blog=None, active_tags=None, date_begin=None, date_end=None, state=None, **post):
+        return {
+            'displayDescription': True,
+            'displayDetail': False,
+            'displayExtraDetail': False,
+            'displayExtraLink': False,
+            'displayImage': False,
+            'allowFuzzy': not post.get('noFuzzy'),
+            'blog': str(blog.id) if blog else None,
+            'tag': ','.join([str(id) for id in active_tags.ids]),
+            'date_begin': date_begin,
+            'date_end': date_end,
+            'state': state,
+        }
 
-    def _prepare_blog_values(self, blogs, blog=False, date_begin=False, date_end=False, tags=False, state=False, page=False, search=None):
+    def _prepare_blog_values(self, blogs, blog=False, date_begin=False, date_end=False, tags=False, state=False, page=False, search=None, **post):
         """ Prepare all values to display the blogs index page or one specific blog"""
         BlogPost = request.env['blog.post']
         BlogTag = request.env['blog.tag']
@@ -69,11 +78,11 @@ class WebsiteBlog(http.Controller):
 
         if date_begin and date_end:
             domain += [("post_date", ">=", date_begin), ("post_date", "<=", date_end)]
-        active_tag_ids = tags and [unslug(tag)[1] for tag in tags.split(',')] or []
+        active_tag_ids = tags and [request.env['ir.http']._unslug(tag)[1] for tag in tags.split(',')] or []
         active_tags = BlogTag
         if active_tag_ids:
             active_tags = BlogTag.browse(active_tag_ids).exists()
-            fixed_tag_slug = ",".join(slug(t) for t in active_tags)
+            fixed_tag_slug = ",".join(request.env['ir.http']._slug(t) for t in active_tags)
             if fixed_tag_slug != tags:
                 path = request.httprequest.full_path
                 new_url = path.replace("/tag/%s" % tags, fixed_tag_slug and "/tag/%s" % fixed_tag_slug or "", 1)
@@ -98,29 +107,20 @@ class WebsiteBlog(http.Controller):
 
         # if blog, we show blog title, if use_cover and not fullwidth_cover we need pager + latest always
         offset = (page - 1) * self._blog_post_per_page
-        if not blog:
-            if use_cover and not fullwidth_cover and not tags and not date_begin and not date_end:
-                offset += 1
+        if not blog and use_cover and not fullwidth_cover and not tags and not date_begin and not date_end and not search:
+            offset += 1
 
-        options = {
-            'displayDescription': True,
-            'displayDetail': False,
-            'displayExtraDetail': False,
-            'displayExtraLink': False,
-            'displayImage': False,
-            'allowFuzzy': not request.params.get('noFuzzy'),
-            'blog': str(blog.id) if blog else None,
-            'tag': ','.join([str(id) for id in active_tags.ids]),
-            'date_begin': date_begin,
-            'date_end': date_end,
-            'state': state,
-        }
+        options = self._get_blog_post_search_options(
+            blog=blog,
+            active_tags=active_tags,
+            date_begin=date_begin,
+            date_end=date_end,
+            state=state,
+            **post
+        )
         total, details, fuzzy_search_term = request.website._search_with_fuzzy("blog_posts_only", search,
             limit=page * self._blog_post_per_page, order="is_published desc, post_date desc, id asc", options=options)
         posts = details[0].get('results', BlogPost)
-        first_post = BlogPost
-        if posts and not blog and posts[0].website_published:
-            first_post = posts[0]
         posts = posts[offset:offset + self._blog_post_per_page]
 
         url_args = dict()
@@ -146,21 +146,18 @@ class WebsiteBlog(http.Controller):
         tag_category = tools.lazy(lambda: sorted(all_tags.mapped('category_id'), key=lambda category: category.name.upper()))
         other_tags = tools.lazy(lambda: sorted(all_tags.filtered(lambda x: not x.category_id), key=lambda tag: tag.name.upper()))
         nav_list = tools.lazy(self.nav_list)
-        # for performance prefetch the first post with the others
-        post_ids = (first_post | posts).ids
         # and avoid accessing related blogs one by one
         posts.blog_id
 
         return {
             'date_begin': date_begin,
             'date_end': date_end,
-            'first_post': first_post.with_prefetch(post_ids),
             'other_tags': other_tags,
             'tag_category': tag_category,
             'nav_list': nav_list,
             'tags_list': self.tags_list,
             'pager': pager,
-            'posts': posts.with_prefetch(post_ids),
+            'posts': posts.with_prefetch(),
             'tag': tags,
             'active_tag_ids': active_tags.ids,
             'domain': domain,
@@ -184,31 +181,13 @@ class WebsiteBlog(http.Controller):
     ], type='http', auth="public", website=True, sitemap=True)
     def blog(self, blog=None, tag=None, page=1, search=None, **opt):
         Blog = request.env['blog.blog']
-
-        # TODO adapt in master. This is a fix for templates wrongly using the
-        # 'blog_url' QueryURL which is defined below. Indeed, in the case where
-        # we are rendering a blog page where no specific blog is selected we
-        # define(d) that as `QueryURL('/blog', ['tag'], ...)` but then some
-        # parts of the template used it like this: `blog_url(blog=XXX)` thus
-        # generating an URL like "/blog?blog=blog.blog(2,)". Adding "blog" to
-        # the list of params would not be right as would create "/blog/blog/2"
-        # which is still wrong as we want "/blog/2". And of course the "/blog"
-        # prefix in the QueryURL definition is needed in case we only specify a
-        # tag via `blog_url(tab=X)` (we expect /blog/tag/X). Patching QueryURL
-        # or making blog_url a custom function instead of a QueryURL instance
-        # could be a solution but it was judged not stable enough. We'll do that
-        # in master. Here we only support "/blog?blog=blog.blog(2,)" URLs.
-        if isinstance(blog, str):
-            blog = Blog.browse(int(re.search(r'\d+', blog)[0]))
-            if not blog.exists():
-                raise werkzeug.exceptions.NotFound()
-
         blogs = tools.lazy(lambda: Blog.search(request.website.website_domain(), order="create_date asc, id asc"))
 
         if not blog and len(blogs) == 1:
-            return request.redirect('/blog/%s' % slug(blogs[0]), code=302)
+            url = QueryURL('/blog/%s' % request.env['ir.http']._slug(blogs[0]), search=search, **opt)()
+            return request.redirect(url, code=302)
 
-        date_begin, date_end, state = opt.get('date_begin'), opt.get('date_end'), opt.get('state')
+        date_begin, date_end = opt.get('date_begin'), opt.get('date_end')
 
         if tag and request.httprequest.method == 'GET':
             # redirect get tag-1,tag-2 -> get tag-1
@@ -217,7 +196,7 @@ class WebsiteBlog(http.Controller):
                 url = QueryURL('' if blog else '/blog', ['blog', 'tag'], blog=blog, tag=tags[0], date_begin=date_begin, date_end=date_end, search=search)()
                 return request.redirect(url, code=302)
 
-        values = self._prepare_blog_values(blogs=blogs, blog=blog, date_begin=date_begin, date_end=date_end, tags=tag, state=state, page=page, search=search)
+        values = self._prepare_blog_values(blogs=blogs, blog=blog, tags=tag, page=page, search=search, **opt)
 
         # in case of a redirection need by `_prepare_blog_values` we follow it
         if isinstance(values, werkzeug.wrappers.Response):
@@ -225,9 +204,7 @@ class WebsiteBlog(http.Controller):
 
         if blog:
             values['main_object'] = blog
-            values['blog_url'] = QueryURL('', ['blog', 'tag'], blog=blog, tag=tag, date_begin=date_begin, date_end=date_end, search=search)
-        else:
-            values['blog_url'] = QueryURL('/blog', ['tag'], date_begin=date_begin, date_end=date_end, search=search)
+        values['blog_url'] = QueryURL('/blog', ['blog', 'tag'], blog=blog, tag=tag, date_begin=date_begin, date_end=date_end, search=search)
 
         return request.render("website_blog.blog_post_short", values)
 
@@ -242,11 +219,11 @@ class WebsiteBlog(http.Controller):
         return r
 
     @http.route([
-        '''/blog/<model("blog.blog"):blog>/post/<model("blog.post", "[('blog_id','=',blog.id)]"):blog_post>''',
+        '''/blog/<model("blog.blog"):blog>/post/<model("blog.post"):blog_post>''',
     ], type='http', auth="public", website=True, sitemap=False)
-    def old_blog_post(self, blog, blog_post, tag_id=None, page=1, enable_editor=None, **post):
+    def old_blog_post(self, blog, blog_post, **post):
         # Compatibility pre-v14
-        return request.redirect(_build_url_w_params("/blog/%s/%s" % (slug(blog), slug(blog_post)), request.params), code=301)
+        return request.redirect("/blog/%s/%s" % (request.env['ir.http']._slug(blog), request.env['ir.http']._slug(blog_post)), code=301)
 
     @http.route([
         '''/blog/<model("blog.blog"):blog>/<model("blog.post", "[('blog_id','=',blog.id)]"):blog_post>''',
@@ -277,7 +254,7 @@ class WebsiteBlog(http.Controller):
         blog_url = QueryURL('', ['blog', 'tag'], blog=blog_post.blog_id, tag=tag, date_begin=date_begin, date_end=date_end)
 
         if not blog_post.blog_id.id == blog.id:
-            return request.redirect("/blog/%s/%s" % (slug(blog_post.blog_id), slug(blog_post)), code=301)
+            return request.redirect("/blog/%s/%s" % (request.env['ir.http']._slug(blog_post.blog_id), request.env['ir.http']._slug(blog_post)), code=301)
 
         tags = request.env['blog.tag'].search([])
 
@@ -289,7 +266,7 @@ class WebsiteBlog(http.Controller):
         all_post = BlogPost.search(blog_post_domain)
 
         if blog_post not in all_post:
-            return request.redirect("/blog/%s" % (slug(blog_post.blog_id)))
+            return request.redirect("/blog/%s" % (request.env['ir.http']._slug(blog_post.blog_id)))
 
         # should always return at least the current post
         all_post_ids = all_post.ids
@@ -318,5 +295,5 @@ class WebsiteBlog(http.Controller):
                 if not request.session.get('posts_viewed'):
                     request.session['posts_viewed'] = []
                 request.session['posts_viewed'].append(blog_post.id)
-                request.session.modified = True
+                request.session.touch()
         return response
