@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import models, _
-from odoo.addons.http_routing.models.ir_http import url_for
 
 
 class Website(models.Model):
@@ -10,7 +9,7 @@ class Website(models.Model):
 
     def get_suggested_controllers(self):
         suggested_controllers = super(Website, self).get_suggested_controllers()
-        suggested_controllers.append((_('Blog'), url_for('/blog'), 'website_blog'))
+        suggested_controllers.append((_('Blog'), self.env['ir.http']._url_for('/blog'), 'website_blog'))
         return suggested_controllers
 
     def configurator_set_menu_links(self, menu_company, module_data):
