@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, fields, models
-from odoo.osv import expression
 
 
 class ResPartner(models.Model):
@@ -42,16 +40,13 @@ class ResPartner(models.Model):
 
     @api.depends('implemented_partner_ids.is_published', 'implemented_partner_ids.active')
     def _compute_implemented_partner_count(self):
-        if not self.ids:
-            self.implemented_partner_count = 0
-            return
         rg_result = self.env['res.partner']._read_group(
             [('assigned_partner_id', 'in', self.ids),
              ('is_published', '=', True)],
             ['assigned_partner_id'],
-            ['assigned_partner_id']
+            ['__count'],
         )
-        rg_data = {rg_item['assigned_partner_id'][0]: rg_item['assigned_partner_id_count'] for rg_item in rg_result}
+        rg_data = {assigned_partner.id: count for assigned_partner, count in rg_result}
         for partner in self:
             partner.implemented_partner_count = rg_data.get(partner.id, 0)
 
@@ -62,13 +57,14 @@ class ResPartner(models.Model):
 
     def _compute_opportunity_count(self):
         super()._compute_opportunity_count()
-        assign_counts = {}
-        if self.ids:
-            opportunity_data = self.env['crm.lead'].with_context(active_test=False)._read_group(
-                [('partner_assigned_id', 'in', self.ids)],
-                ['partner_assigned_id'], ['partner_assigned_id']
-            )
-            assign_counts = {datum['partner_assigned_id'][0]: datum['partner_assigned_id_count'] for datum in opportunity_data}
+        if not self.env.user.has_group('sales_team.group_sale_salesman'):
+            return
+
+        opportunity_data = self.env['crm.lead'].with_context(active_test=False)._read_group(
+            [('partner_assigned_id', 'in', self.ids)],
+            ['partner_assigned_id'], ['__count']
+        )
+        assign_counts = {partner_assigned.id: count for partner_assigned, count in opportunity_data}
         for partner in self:
             partner.opportunity_count += assign_counts.get(partner.id, 0)
 
@@ -83,7 +79,7 @@ class ResPartner(models.Model):
             return action
         # perform searches independently as having OR with those leaves seems to
         # be counter productive
-        Lead = self.env['crm.lead'].with_context(**action_context_origin)
+        Lead = self.env['crm.lead'].with_context(**action_context_origin, active_test=False)
         ids_origin = Lead.search(action_domain_origin).ids
         ids_new = Lead.search(action_domain_assign).ids
         action['domain'] = [('id', 'in', sorted(list(set(ids_origin) | set(ids_new))))]
