@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import models, _
-from odoo.addons.http_routing.models.ir_http import url_for
 
 
 class Website(models.Model):
@@ -10,5 +9,5 @@ class Website(models.Model):
 
     def get_suggested_controllers(self):
         suggested_controllers = super(Website, self).get_suggested_controllers()
-        suggested_controllers.append((_('Resellers'), url_for('/partners'), 'website_crm_partner_assign'))
+        suggested_controllers.append((_('Resellers'), self.env['ir.http']._url_for('/partners'), 'website_crm_partner_assign'))
         return suggested_controllers
