@@ -8,8 +8,7 @@ class WebsiteMenu(models.Model):
     _inherit = "website.menu"
 
     def unlink(self):
-        """ Override to synchronize event configuration fields with menu deletion.
-        This should be cleaned in upcoming versions. """    
+        """ Override to synchronize event configuration fields with menu deletion. """
         event_updates = {}
         website_event_menus = self.env['website.event.menu'].search([('menu_id', 'in', self.ids)])
         for event_menu in website_event_menus:
@@ -18,7 +17,9 @@ class WebsiteMenu(models.Model):
                 if event_menu.menu_type == menu_type:
                     to_update.append(fname)
 
-        # call super that resumes the unlink of menus entries (including website event menus)
+        # manually remove website_event_menus to call their ``unlink`` method. Otherwise
+        # super unlinks at db level and skip model-specific behavior.
+        website_event_menus.unlink()
         res = super(WebsiteMenu, self).unlink()
 
         # update events
