@@ -2,5 +2,5 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import product_product
+from . import product_template
 from . import sale_order
-from . import website
