@@ -2,7 +2,6 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from collections import defaultdict
-from odoo import _
 from odoo.http import request, route
 
 from odoo.addons.website_event.controllers.main import WebsiteEventController
@@ -10,15 +9,6 @@ from odoo.addons.website_event.controllers.main import WebsiteEventController
 
 class WebsiteEventSaleController(WebsiteEventController):
 
-    @route()
-    def event_register(self, event, **post):
-        event = event.with_context(pricelist=request.website.id)
-        if not request.context.get('pricelist'):
-            pricelist = request.website.pricelist_id
-            if pricelist:
-                event = event.with_context(pricelist=pricelist.id)
-        return super().event_register(event, **post)
-
     def _process_tickets_form(self, event, form_details):
         """ Add price information on ticket order """
         res = super()._process_tickets_form(event, form_details)
@@ -31,7 +21,20 @@ class WebsiteEventSaleController(WebsiteEventController):
         if not any(info.get('event_ticket_id') for info in registration_data):
             return super()._create_attendees_from_registration_post(event, registration_data)
 
+        event_ticket_ids = [registration['event_ticket_id'] for registration in registration_data if registration.get('event_ticket_id')]
+        event_ticket_by_id = {
+            event_ticket.id: event_ticket
+            for event_ticket in request.env['event.event.ticket'].sudo().browse(event_ticket_ids)
+        }
+
+        if all(event_ticket.price == 0 for event_ticket in event_ticket_by_id.values()) and not request.website.sale_get_order().id:
+            # all chosen tickets are free AND no existing SO -> skip SO and payment process
+            return super()._create_attendees_from_registration_post(event, registration_data)
+
         order_sudo = request.website.sale_get_order(force_create=True)
+        if order_sudo.state != 'draft':
+            request.website.sale_reset()
+            order_sudo = request.website.sale_get_order(force_create=True)
 
         tickets_data = defaultdict(int)
         for data in registration_data:
@@ -41,7 +44,7 @@ class WebsiteEventSaleController(WebsiteEventController):
 
         cart_data = {}
         for ticket_id, count in tickets_data.items():
-            ticket_sudo = request.env['event.event.ticket'].sudo().browse(ticket_id)
+            ticket_sudo = event_ticket_by_id.get(ticket_id)
             cart_values = order_sudo._cart_update(
                 product_id=ticket_sudo.product_id.id,
                 add_qty=count,
@@ -51,7 +54,8 @@ class WebsiteEventSaleController(WebsiteEventController):
 
         for data in registration_data:
             event_ticket_id = data.get('event_ticket_id')
-            if event_ticket_id:
+            event_ticket = event_ticket_by_id.get(event_ticket_id)
+            if event_ticket:
                 data['sale_order_id'] = order_sudo.id
                 data['sale_order_line_id'] = cart_data[event_ticket_id]
 
@@ -64,11 +68,15 @@ class WebsiteEventSaleController(WebsiteEventController):
         res = super().registration_confirm(event, **post)
 
         registrations = self._process_attendees_form(event, post)
+        order_sudo = request.website.sale_get_order()
+        if not order_sudo.id:
+            # order does not contain any lines related to the event, meaning we are confirming only free tickets of this event
+            return res
 
         # we have at least one registration linked to a ticket -> sale mode activate
         if any(info['event_ticket_id'] for info in registrations):
-            order_sudo = request.website.sale_get_order()
             if order_sudo.amount_total:
+                request.session['sale_last_order_id'] = order_sudo.id
                 return request.redirect("/shop/checkout")
             # free tickets -> order with amount = 0: auto-confirm, no checkout
             elif order_sudo:
