@@ -69,6 +69,18 @@ class EventTrackController(http.Controller):
           * 'search': search string;
           * 'tags': list of tag IDs for filtering;
         """
+
+        if searches.get('tags', '[]').count(',') > 0 and request.httprequest.method == 'GET' and not searches.get('prevent_redirect'):
+            # Previously, the tags were searched using GET, which caused issues with crawlers (too many hits)
+            # We replaced those with POST to avoid that, but it's not sufficient as bots "remember" crawled pages for a while
+            # This permanent redirect is placed to instruct the bots that this page is no longer valid
+            # Note: We allow a single tag to be GET, to keep crawlers & indexes on those pages
+            # What we really want to avoid is combinatorial explosions
+            # (Tags are formed as a JSON array, so we count ',' to keep it simple)
+            # TODO: remove in a few stable versions (v19?), including the "prevent_redirect" param in templates
+            slug = request.env['ir.http']._slug
+            return request.redirect(f'/event/{slug(event)}/track', code=301)
+
         return request.render(
             "website_event_track.tracks_session",
             self._event_tracks_get_values(event, tag=tag, **searches)
@@ -158,6 +170,7 @@ class EventTrackController(http.Controller):
             'today_tz': today_tz,
             # search information
             'searches': searches,
+            'search_count': len(tracks_sudo),
             'search_key': searches['search'],
             'search_wishlist': searches['search_wishlist'],
             'search_tags': search_tags,
@@ -488,10 +501,7 @@ class EventTrackController(http.Controller):
         track = request.env['event.track'].browse(track_id).exists()
         if not track:
             raise NotFound()
-        try:
-            track.check_access_rights('read')
-            track.check_access_rule('read')
-        except exceptions.AccessError:
+        if not track.has_access('read'):
             if not allow_sudo:
                 raise Forbidden()
             track = track.sudo()
@@ -500,10 +510,7 @@ class EventTrackController(http.Controller):
         # JSON RPC have no website in requests
         if hasattr(request, 'website_id') and not event.can_access_from_current_website():
             raise NotFound()
-        try:
-            event.check_access_rights('read')
-            event.check_access_rule('read')
-        except exceptions.AccessError:
+        if not event.has_access('read'):
             raise Forbidden()
 
         return track
