@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import common
+from . import test_event_menus
 from . import test_track_internals
-from . import test_website
 from . import test_website_event
+from . import test_website_visitor
