@@ -8,13 +8,11 @@
     'sequence': 1006,
     'version': '1.0',
     'summary': 'Support live tracks: streaming, participation, youtube',
-    'website': 'https://www.odoo.com/page/events',
-    'description': "",
+    'website': 'https://www.odoo.com/app/events',
     'depends': [
         'website_event_track',
     ],
     'data': [
-        'views/assets.xml',
         'views/event_track_templates_list.xml',
         'views/event_track_templates_page.xml',
         'views/event_track_views.xml',
@@ -22,10 +20,15 @@
     'demo': [
         'data/event_track_demo.xml'
     ],
-    'qweb': [
-        'static/src/xml/website_event_track_live_templates.xml'
-    ],
-    'application': False,
     'installable': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_event_track_live/static/src/scss/website_event_track_live.scss',
+            'website_event_track_live/static/src/js/website_event_track_replay_suggestion.js',
+            'website_event_track_live/static/src/js/website_event_track_suggestion.js',
+            'website_event_track_live/static/src/js/website_event_track_live.js',
+            'website_event_track_live/static/src/xml/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
