@@ -1,9 +1,8 @@
-odoo.define('website_event_track_live.website_event_track_suggestion', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var Widget = require('web.Widget');
+import { PublicWidget } from "@web/legacy/js/public/public_widget";
 
-var WebsiteEventTrackSuggestion = Widget.extend({
+var WebsiteEventTrackSuggestion = PublicWidget.extend({
     template: 'website_event_track_live.website_event_track_suggestion',
     events: {
         'click .owevent_track_suggestion_next': '_onNextTrackClick',
@@ -75,6 +74,4 @@ var WebsiteEventTrackSuggestion = Widget.extend({
     }
 });
 
-return WebsiteEventTrackSuggestion;
-
-});
+export default WebsiteEventTrackSuggestion;
