@@ -8,15 +8,13 @@
     'sequence': 1007,
     'version': '1.0',
     'summary': 'Quizzes on tracks',
-    'website': 'https://www.odoo.com/page/events',
-    'description': "",
+    'website': 'https://www.odoo.com/app/events',
     'depends': [
         'website_profile',
         'website_event_track',
     ],
     'data': [
         'security/ir.model.access.csv',
-        'views/assets.xml',
         'views/event_leaderboard_templates.xml',
         'views/event_quiz_views.xml',
         'views/event_quiz_question_views.xml',
@@ -31,7 +29,14 @@
     'demo': [
         'data/quiz_demo.xml',
     ],
-    'application': False,
     'installable': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_event_track_quiz/static/src/scss/event_quiz.scss',
+            'website_event_track_quiz/static/src/js/event_quiz.js',
+            'website_event_track_quiz/static/src/js/event_quiz_leaderboard.js',
+            'website_event_track_quiz/static/src/xml/quiz_templates.xml',
+        ],
+    },
     'license': 'LGPL-3',
 }
