@@ -1,14 +1,10 @@
-odoo.define('website_forum.share', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var core = require('web.core');
-var publicWidget = require('web.public.widget');
-require('website.content.snippets.animation');
+import publicWidget from "@web/legacy/js/public/public_widget";
+import "@website/js/content/snippets.animation";
+import { renderToElement } from "@web/core/utils/render";
 
-var qweb = core.qweb;
-
-// FIXME There is no reason to inherit from socialShare here
-var ForumShare = publicWidget.registry.socialShare.extend({
+const ForumShare = publicWidget.Widget.extend({
     selector: '',
     events: {},
 
@@ -27,49 +23,22 @@ var ForumShare = publicWidget.registry.socialShare.extend({
      */
     start: function () {
         var def = this._super.apply(this, arguments);
-        this._onMouseEnter();
-        return def;
-    },
-
-    //--------------------------------------------------------------------------
-    // Private
-    //--------------------------------------------------------------------------
-
-    /**
-     * @private
-     */
-    _bindSocialEvent: function () {
-        this._super.apply(this, arguments);
-        $('.oe_share_bump').click($.proxy(this._postBump, this));
-    },
-    /**
-     * @private
-     */
-    _render: function () {
         var $question = this.$('article.question');
         if (!this.targetType) {
             this._super.apply(this, arguments);
-        } else if (this.targetType === 'social-alert') {
-            $question.before(qweb.render('website.social_alert', {medias: this.socialList}));
         } else {
-            $('body').append(qweb.render('website.social_modal', {
-                medias: this.socialList,
+            const el = renderToElement('website.social_modal', {
                 target_type: this.targetType,
                 state: $question.data('state'),
-            }));
+            });
+            $('body').append(el);
+            this.trigger_up('widgets_start_request', {
+                editableMode: false,
+                $target: $(el.querySelector(".s_share")),
+            });
             $('#oe_social_share_modal').modal('show');
         }
-    },
-    /**
-     * @private
-     */
-    _postBump: function () {
-        this._rpc({ // FIXME
-            route: '/forum/post/bump',
-            params: {
-                post_id: this.element.data('id'),
-            },
-        });
+        return def;
     },
 });
 
@@ -86,13 +55,7 @@ publicWidget.registry.websiteForumShare = publicWidget.Widget.extend({
             (new ForumShare(this, false, socialData.targetType)).attachTo($(document.body));
             sessionStorage.removeItem('social_share');
         }
-        // Display an alert if post has no reply and is older than 10 days
-        var $questionContainer = $('.oe_js_bump');
-        if ($questionContainer.length) {
-            new ForumShare(this, false, 'social-alert').attachTo($questionContainer);
-        }
 
         return this._super.apply(this, arguments);
     },
 });
-});
