@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo.tests import common
+from odoo.tools import mute_logger
 
 KARMA = {
     'ask': 5, 'ans': 10,
@@ -24,6 +25,10 @@ class TestForumCommon(common.TransactionCase):
     def setUpClass(cls):
         super(TestForumCommon, cls).setUpClass()
 
+        # default base data
+        cls.base_website = cls.env.ref("website.default_website")
+        cls.base_forum = cls.env.ref("website_forum.forum_help")
+
         Forum = cls.env['forum.forum']
         Post = cls.env['forum.post']
 
@@ -39,6 +44,13 @@ class TestForumCommon(common.TransactionCase):
             'karma': 0,
             'groups_id': [(6, 0, [group_employee_id])]
         })
+        cls.user_employee_2 = cls.env['res.users'].create({
+            'name': 'Merlin Employee',
+            'login': 'Merlin',
+            'email': 'merlin.employee@example.com',
+            'karma': KARMA['ask'],
+            'groups_id': [(6, 0, [cls.env.ref('base.group_user').id])],
+        })
         cls.user_portal = TestUsersEnv.create({
             'name': 'Beatrice Portal',
             'login': 'Beatrice',
@@ -53,6 +65,7 @@ class TestForumCommon(common.TransactionCase):
             'karma': 0,
             'groups_id': [(6, 0, [group_public_id])]
         })
+        cls.user_admin = cls.env.ref('base.user_admin')
 
         # Test forum
         cls.forum = Forum.create({
@@ -94,3 +107,20 @@ class TestForumCommon(common.TransactionCase):
             'forum_id': cls.forum.id,
             'parent_id': cls.post.id,
         })
+
+    @classmethod
+    def _activate_multi_website(cls):
+        cls.website_2 = cls.env['website'].create({
+            'name': 'Second Website on same company',
+        })
+
+    @mute_logger("odoo.models.unlink")
+    def _activate_tags_for_counts(self):
+        self.env['forum.tag'].search([]).unlink()
+        self.tags = self.env['forum.tag'].create(
+            [
+                {'forum_id': forum_id.id, 'name': f'Test Tag {tag_idx}'}
+                for forum_id in self.forum | self.base_forum
+                for tag_idx in range(1, 8)
+            ]
+        )
