@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+import contextlib
+
 from odoo import http
 from odoo.http import request
 from odoo.tools.json import scriptsafe
@@ -22,16 +24,30 @@ class GoogleMap(http.Controller):
     directives ``width`` and ``height``.
     '''
 
+    def _get_gmap_domains(self, **kw):
+        return [(0, '=', 1)]
+
     @http.route(['/google_map'], type='http', auth="public", website=True, sitemap=False)
     def google_map(self, *arg, **post):
+        PartnerSudo = request.env['res.partner'].sudo()
         clean_ids = []
-        for partner_id in post.get('partner_ids', "").split(","):
-            try:
-                clean_ids.append(int(partner_id))
-            except ValueError:
-                pass
-        partners = request.env['res.partner'].sudo().search([("id", "in", clean_ids),
-                                                             ('website_published', '=', True), ('is_company', '=', True)])
+        domain = []
+        if post.get('partner_ids'):
+            for partner_id in post['partner_ids'].split(","):
+                with contextlib.suppress(ValueError):
+                    clean_ids.append(int(partner_id))
+            domain += [("id", "in", clean_ids), ('is_company', '=', True)]
+        elif post.get('dom'):
+            domain = self._get_gmap_domains(**post)
+
+        limit = post.get('limit') and int(post['limit']) or 80
+
+        if domain:  # [] is not allowed
+            domain += [('website_published', '=', True)]
+            partners = PartnerSudo.search(domain, limit=limit)
+        else:
+            partners = PartnerSudo
+
         partner_data = {
             "counter": len(partners),
             "partners": []
@@ -40,7 +56,7 @@ class GoogleMap(http.Controller):
             partner_data["partners"].append({
                 'id': partner.id,
                 'name': partner.name,
-                'address': '\n'.join(partner.name_get()[0][1].split('\n')[1:]),
+                'address': '\n'.join(partner.display_name.split('\n')[1:]),
                 'latitude': str(partner.partner_latitude) if partner.partner_latitude else False,
                 'longitude': str(partner.partner_longitude) if partner.partner_longitude else False,
             })
