@@ -3,6 +3,9 @@
 
 from odoo import models, _
 
+from werkzeug import urls
+
+
 class LinkTracker(models.Model):
     _inherit = ['link.tracker']
 
@@ -13,3 +16,8 @@ class LinkTracker(models.Model):
             'url': '%s+' % (self.short_url),
             'target': 'new',
         }
+
+    def _compute_short_url_host(self):
+        for tracker in self:
+            base_url = self.env['website'].get_current_website().get_base_url()
+            tracker.short_url_host = urls.url_join(base_url, '/r/')
