@@ -1,12 +1,25 @@
-odoo.define('website_mail.follow', function (require) {
-'use strict';
+/** @odoo-module **/
 
-var publicWidget = require('web.public.widget');
+import publicWidget from "@web/legacy/js/public/public_widget";
+import { rpc } from "@web/core/network/rpc";
+import { ReCaptcha } from "@google_recaptcha/js/recaptcha";
+import { _t } from "@web/core/l10n/translation";
 
 publicWidget.registry.follow = publicWidget.Widget.extend({
-    selector: '#wrapwrap:has(.js_follow)',
+    selector: '#wrapwrap',
+    selectorHas: '.js_follow',
     disabledInEditableMode: false,
 
+    init() {
+        this._super(...arguments);
+        this._recaptcha = new ReCaptcha();
+        this.notification = this.bindService("notification");
+    },
+
+    async willStart() {
+        return this._recaptcha.loadLibs();
+    },
+
     /**
      * @override
      */
@@ -35,17 +48,14 @@ publicWidget.registry.follow = publicWidget.Widget.extend({
             records[model].push(parseInt(el.dataset.id));
         }
 
-        this._rpc({
-            route: '/website_mail/is_follower',
-            params: {
-                records: records,
-            },
-        }).then(always).guardedCatch(always);
+        rpc('/website_mail/is_follower', {
+            records: records,
+        }).then(always, always);
 
         // not if editable mode to allow designer to edit
         if (!this.editableMode) {
             $('.js_follow > .d-none').removeClass('d-none');
-            this.$target.find('.js_follow_btn, .js_unfollow_btn').on('click', function (event) {
+            this.$el.find('.js_follow_btn, .js_unfollow_btn').on('click', function (event) {
                 event.preventDefault();
                 self._onClick(event);
             });
@@ -99,7 +109,7 @@ publicWidget.registry.follow = publicWidget.Widget.extend({
      * @private
      * @param {Event} ev
      */
-    _onClick: function (ev) {
+    async _onClick(ev) {
         var self = this;
         var $jsFollow = $(ev.currentTarget).closest('.js_follow');
         var $email = $jsFollow.find(".js_follow_email");
@@ -112,18 +122,26 @@ publicWidget.registry.follow = publicWidget.Widget.extend({
 
         var email = $email.length ? $email.val() : false;
         if (email || this.isUser) {
-            this._rpc({
-                route: '/website_mail/follow',
-                params: {
-                    'id': +$jsFollow.data('id'),
-                    'object': $jsFollow.data('object'),
-                    'message_is_follower': $jsFollow.attr("data-follow") || "off",
-                    'email': email,
-                },
-            }).then(function (follow) {
+            const tokenCaptcha = await this._recaptcha.getToken("website_mail_follow");
+            const token = tokenCaptcha.token;
+
+            if (tokenCaptcha.error) {
+                self.notification.add(tokenCaptcha.error, {
+                    type: "danger",
+                    title: _t("Error"),
+                    sticky: true
+                });
+                return false;
+            }
+            rpc("/website_mail/follow", {
+                "id": +$jsFollow.data("id"),
+                "object": $jsFollow.data("object"),
+                "message_is_follower": $jsFollow.attr("data-follow") || "off",
+                "email": email,
+                "recaptcha_token_response": token
+            }).then(function(follow) {
                 self._toggleSubscription(follow, email, $jsFollow);
             });
         }
     },
 });
-});
