@@ -7,7 +7,6 @@ from odoo import fields
 
 from odoo import http
 from odoo.http import request
-from odoo.addons.http_routing.models.ir_http import unslug
 from odoo.tools.translate import _
 
 
@@ -63,7 +62,7 @@ class WebsiteMembership(http.Controller):
         if post_name:
             country_domain += ['|', ('name', 'ilike', post_name), ('website_description', 'ilike', post_name)]
 
-        countries = Partner.sudo().read_group(country_domain + [("website_published", "=", True)], ["id", "country_id"], groupby="country_id", orderby="country_id")
+        countries = Partner.sudo().read_group(country_domain + [("website_published", "=", True)], ["__count"], groupby="country_id")
         countries_total = sum(country_dict['country_id_count'] for country_dict in countries)
 
         line_domain = list(base_line_domain)
@@ -163,11 +162,14 @@ class WebsiteMembership(http.Controller):
     # Do not use semantic controller due to SUPERUSER_ID
     @http.route(['/members/<partner_id>'], type='http', auth="public", website=True)
     def partners_detail(self, partner_id, **post):
-        _, partner_id = unslug(partner_id)
+        current_slug = partner_id
+        _, partner_id = request.env['ir.http']._unslug(partner_id)
         if partner_id:
             partner = request.env['res.partner'].sudo().browse(partner_id)
             if partner.exists() and partner.website_published:  # TODO should be done with access rules
+                if request.env['ir.http']._slug(partner) != current_slug:
+                    return request.redirect('/members/%s' % request.env['ir.http']._slug(partner))
                 values = {}
                 values['main_object'] = values['partner'] = partner
                 return request.render("website_membership.partner", values)
-        return self.members(**post)
+        raise request.not_found()
