@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 from odoo import api, fields, models
-from odoo.addons.http_routing.models.ir_http import slug
 from odoo.tools.translate import html_translate
 
 
@@ -9,10 +8,10 @@ class WebsiteResPartner(models.Model):
     _name = 'res.partner'
     _inherit = ['res.partner', 'website.seo.metadata']
 
-    website_description = fields.Html('Website Partner Full Description', strip_style=True, translate=html_translate)
+    website_description = fields.Html('Website Partner Full Description', strip_style=True, sanitize_overridable=True, translate=html_translate)
     website_short_description = fields.Text('Website Partner Short Description', translate=True)
 
     def _compute_website_url(self):
         super(WebsiteResPartner, self)._compute_website_url()
         for partner in self:
-            partner.website_url = "/partners/%s" % slug(partner)
+            partner.website_url = "/partners/%s" % self.env['ir.http']._slug(partner)
