@@ -1,7 +1,9 @@
-odoo.define('website_sale.recently_viewed', function (require) {
+/** @odoo-module **/
 
-var publicWidget = require('web.public.widget');
-const {getCookie, setCookie} = require('web.utils.cookies');
+import { debounce } from "@web/core/utils/timing";
+import publicWidget from "@web/legacy/js/public/public_widget";
+import { cookie } from "@web/core/browser/cookie";;
+import { rpc } from "@web/core/network/rpc";
 
 publicWidget.registry.productsRecentlyViewedUpdate = publicWidget.Widget.extend({
     selector: '#product_detail',
@@ -15,7 +17,7 @@ publicWidget.registry.productsRecentlyViewedUpdate = publicWidget.Widget.extend(
      */
     init: function () {
         this._super.apply(this, arguments);
-        this._onProductChange = _.debounce(this._onProductChange, this.debounceValue);
+        this._onProductChange = debounce(this._onProductChange, this.debounceValue);
     },
 
     //--------------------------------------------------------------------------
@@ -33,19 +35,16 @@ publicWidget.registry.productsRecentlyViewedUpdate = publicWidget.Widget.extend(
         if (! parseInt(this.el.dataset.viewTrack, 10)) {
             return; // Is not tracked
         }
-        if (getCookie(cookieName)) {
+        if (cookie.get(cookieName)) {
             return; // Already tracked in the last 30min
         }
         if ($(this.el).find('.js_product.css_not_available').length) {
             return; // Variant not possible
         }
-        this._rpc({
-            route: '/shop/products/recently_viewed_update',
-            params: {
-                product_id: productId,
-            }
+        rpc('/shop/products/recently_viewed_update', {
+            product_id: productId,
         }).then(function (res) {
-            setCookie(cookieName, productId, 30 * 60, 'optional');
+            cookie.set(cookieName, productId, 30 * 60, 'optional');
         });
     },
 
@@ -62,4 +61,3 @@ publicWidget.registry.productsRecentlyViewedUpdate = publicWidget.Widget.extend(
         this._updateProductView($(ev.currentTarget));
     },
 });
-});
