@@ -5,11 +5,12 @@ import io
 
 from PIL import Image
 
-import odoo.tests
+from odoo.fields import Command
+from odoo.tests import HttpCase, tagged
 
 
-@odoo.tests.common.tagged('post_install', '-at_install')
-class TestWebsiteSaleImage(odoo.tests.HttpCase):
+@tagged('post_install', '-at_install')
+class TestWebsiteSaleImage(HttpCase):
 
     # registry_test_mode = False  # uncomment to save the product to test in browser
 
@@ -23,30 +24,31 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
         color_blue = '#4169E1'
         name_blue = 'Royal Blue'
 
+        self.env['product.pricelist'].sudo().search([]).action_archive()
+
         # create the color attribute
         product_attribute = self.env['product.attribute'].create({
             'name': 'Beautiful Color',
             'display_type': 'color',
+            'value_ids': [
+                Command.create({
+                    'name': name_red,
+                    'html_color': color_red,
+                    'sequence': 1,
+                }),
+                Command.create({
+                    'name': name_green,
+                    'html_color': color_green,
+                    'sequence': 2,
+                }),
+                Command.create({
+                    'name': name_blue,
+                    'html_color': color_blue,
+                    'sequence': 3,
+                }),
+            ]
         })
 
-        # create the color attribute values
-        attr_values = self.env['product.attribute.value'].create([{
-            'name': name_red,
-            'attribute_id': product_attribute.id,
-            'html_color': color_red,
-            'sequence': 1,
-        }, {
-            'name': name_green,
-            'attribute_id': product_attribute.id,
-            'html_color': color_green,
-            'sequence': 2,
-        }, {
-            'name': name_blue,
-            'attribute_id': product_attribute.id,
-            'html_color': color_blue,
-            'sequence': 3,
-        }])
-
         # first image (blue) for the template
         f = io.BytesIO()
         Image.new('RGB', (1920, 1080), color_blue).save(f, 'JPEG')
@@ -87,17 +89,21 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
         image_png = base64.b64encode(f.read())
 
         # create the template, without creating the variants
-        template = self.env['product.template'].with_context(create_product_product=True).create({
+        template = self.env['product.template'].create({
             'name': 'A Colorful Image',
-            'product_template_image_ids': [(0, 0, {'name': 'image 1', 'image_1920': image_gif}), (0, 0, {'name': 'image 4', 'image_1920': image_svg})],
+            'product_template_image_ids': [
+                Command.create({'name': 'image 1', 'image_1920': image_gif}),
+                Command.create({'name': 'image 4', 'image_1920': image_svg}),
+            ],
+            'attribute_line_ids': [
+                Command.create({
+                    'attribute_id': product_attribute.id,
+                    'value_ids': [Command.set(product_attribute.value_ids.ids)],
+                })
+            ]
         })
 
-        # set the color attribute and values on the template
-        line = self.env['product.template.attribute.line'].create([{
-            'attribute_id': product_attribute.id,
-            'product_tmpl_id': template.id,
-            'value_ids': [(6, 0, attr_values.ids)]
-        }])
+        line = template.attribute_line_ids
         value_red = line.product_template_value_ids[0]
         value_green = line.product_template_value_ids[1]
 
@@ -210,16 +216,9 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
             [('key', '=', 'website_sale.product_picture_magnify_click')]
         ).write({'active': True})
 
-        # Ensure that only one pricelist is available during the test, with the company currency.
+        # Ensure that no pricelist is available during the test.
         # This ensures that tours with triggers on the amounts will run properly.
-        # To this purpose, we will ensure that only the public_pricelist is available for the default_website.
-        public_pricelist = self.env.ref('product.list0')
-        default_website = self.env.ref('website.default_website')
-        self.env['product.pricelist'].search([
-            ('id', '!=', public_pricelist.id),
-            ('website_id', 'in', [False, default_website.id])]
-        ).website_id = self.env.ref('website.website2')
-        public_pricelist.currency_id = self.env.company.currency_id
+        self.env['product.pricelist'].search([]).action_archive()
 
         self.start_tour("/", 'shop_zoom', login="admin")
 
@@ -243,9 +242,9 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
         product_green.image_variant_1920 = False
         images = product_green._get_images()
         # images on fields are resized to max 1920
-        image = Image.open(io.BytesIO(base64.b64decode(images[0].image_1920)))
-        self.assertEqual(image.size, (1268, 1920))
-        self.assertEqual(images[1].image_1920, red_image)
+        image_png = Image.open(io.BytesIO(base64.b64decode(images[1].image_1920)))
+        self.assertEqual(images[0].image_1920, red_image)
+        self.assertEqual(image_png.size, (1268, 1920))
         self.assertEqual(images[2].image_1920, image_gif)
         self.assertEqual(images[3].image_1920, image_svg)
 
@@ -286,25 +285,24 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
         product_attribute = self.env['product.attribute'].create({
             'name': 'Beautiful Color',
             'display_type': 'color',
+            'value_ids': [
+                Command.create({
+                    'name': 'Red',
+                    'sequence': 1,
+                }),
+                Command.create({
+                    'name': 'Green',
+                    'sequence': 2,
+                }),
+                Command.create({
+                    'name': 'Blue',
+                    'sequence': 3,
+                }),
+            ]
         })
 
-        # create the color attribute values
-        attr_values = self.env['product.attribute.value'].create([{
-            'name': 'Red',
-            'attribute_id': product_attribute.id,
-            'sequence': 1,
-        }, {
-            'name': 'Green',
-            'attribute_id': product_attribute.id,
-            'sequence': 2,
-        }, {
-            'name': 'Blue',
-            'attribute_id': product_attribute.id,
-            'sequence': 3,
-        }])
-
         # create the template, without creating the variants
-        template = self.env['product.template'].with_context(create_product_product=True).create({
+        template = self.env['product.template'].with_context(create_product_product=False).create({
             'name': 'Test subject',
         })
 
@@ -315,7 +313,7 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
         line = self.env['product.template.attribute.line'].create([{
             'attribute_id': product_attribute.id,
             'product_tmpl_id': template.id,
-            'value_ids': [(6, 0, attr_values.ids)]
+            'value_ids': [Command.set(product_attribute.value_ids.ids)]
         }])
         value_red = line.product_template_value_ids[0]
         product_red = template._get_variant_for_combination(value_red)
@@ -337,3 +335,76 @@ class TestWebsiteSaleImage(odoo.tests.HttpCase):
 
         # when there is a template image, the image must be obtained from the template
         self.assertEqual(template, template._get_image_holder())
+
+@tagged('post_install', '-at_install')
+class TestWebsiteSaleRemoveImage(HttpCase):
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        # Attachment needed for the replacement of images
+        cls.env['ir.attachment'].create({
+            'public': True,
+            'name': 's_default_image.jpg',
+            'type': 'url',
+            'url': f'{cls.base_url()}/web/image/website.s_banner_default_image.jpg',
+        })
+
+        # First image (blue) for the template.
+        color_blue = '#4169E1'
+        name_blue = 'Royal Blue'
+        # Red for the variant.
+        color_red = '#CD5C5C'
+        name_red = 'Indian Red'
+
+        # Create the color attribute.
+        cls.product_attribute = cls.env['product.attribute'].create({
+            'name': 'Beautiful Color',
+            'display_type': 'color',
+        })
+
+        # create the color attribute values
+        cls.attr_values = cls.env['product.attribute.value'].create([{
+            'name': name_blue,
+            'attribute_id': cls.product_attribute.id,
+            'html_color': color_blue,
+            'sequence': 1,
+        }, {
+            'name': name_red,
+            'attribute_id': cls.product_attribute.id,
+            'html_color': color_red,
+            'sequence': 2,
+        },
+        ])
+        f = io.BytesIO()
+        Image.new('RGB', (1920, 1080), color_blue).save(f, 'JPEG')
+        f.seek(0)
+        blue_image = base64.b64encode(f.read())
+
+        cls.template = cls.env['product.template'].with_context(create_product_product=False).create({
+            'name': 'Test Remove Image',
+            'image_1920': blue_image,
+        })
+
+    def test_website_sale_add_and_remove_main_product_image_no_variant(self):
+        self.product = self.env['product.product'].create({
+            'product_tmpl_id': self.template.id,
+        })
+
+        self.start_tour(self.env['website'].get_client_action_url('/'), 'add_and_remove_main_product_image_no_variant', login='admin')
+        self.assertFalse(self.template.image_1920)
+        self.assertFalse(self.product.image_1920)
+
+    def test_website_sale_remove_main_product_image_with_variant(self):
+        # Set the color attribute and values on the template.
+        self.env['product.template.attribute.line'].create([{
+            'attribute_id': self.product_attribute.id,
+            'product_tmpl_id': self.template.id,
+            'value_ids': [(6, 0, self.attr_values.ids)]
+        }])
+        self.product = self.env['product.product'].create({
+            'product_tmpl_id': self.template.id,
+        })
+        self.start_tour(self.env['website'].get_client_action_url('/'), 'remove_main_product_image_with_variant', login='admin')
+        self.assertFalse(self.template.image_1920)
+        self.assertFalse(self.product.image_1920)
