@@ -1,14 +1,16 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+
 from unittest.mock import patch
 
 from odoo.fields import Command
-from odoo.tests import tagged, TransactionCase
+from odoo.tests import tagged
+from odoo.tools import SQL
 
-from odoo.addons.base.tests.common import TransactionCaseWithUserDemo, HttpCaseWithUserPortal
-from odoo.addons.website.tools import MockRequest
+from odoo.addons.base.tests.common import HttpCaseWithUserPortal, TransactionCaseWithUserDemo
+from odoo.addons.website_sale.tests.common import WebsiteSaleCommon
 
-''' /!\/!\
+
+r''' /!\/!\
 Calling `get_pricelist_available` after setting `property_product_pricelist` on
 a partner will not work as expected. That field will change the output of
 `get_pricelist_available` but modifying it will not invalidate the cache.
@@ -28,80 +30,86 @@ Try to keep one call to `get_pricelist_available` by test method.
 
 
 @tagged('post_install', '-at_install')
-class TestWebsitePriceList(TransactionCase):
+class TestWebsitePriceList(WebsiteSaleCommon):
 
-    def setUp(self):
-        super(TestWebsitePriceList, self).setUp()
-        self.env.user.partner_id.country_id = False  # Remove country to avoid property pricelist computed.
-        self.website = self.env.ref('website.default_website')
-        self.website.user_id = self.env.user
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.env.user.partner_id.country_id = False  # Remove country to avoid property pricelist computed.
+        cls.website.user_id = cls.env.user
 
-        (self.env['product.pricelist'].search([]) - self.env.ref('product.list0')).write({'website_id': False, 'active': False})
-        self.benelux = self.env['res.country.group'].create({
+        cls.pricelist.name = "Public Pricelist" # reduce diff in existing tests
+        cls.country_be = cls.env.ref('base.be')
+        cls.benelux = cls.env['res.country.group'].create({
             'name': 'BeNeLux',
-            'country_ids': [(6, 0, (self.env.ref('base.be') + self.env.ref('base.lu') + self.env.ref('base.nl')).ids)]
+            'country_ids': [Command.set((cls.country_be + cls.env.ref('base.lu') + cls.env.ref('base.nl')).ids)]
         })
-        self.list_benelux = self.env['product.pricelist'].create({
+        cls.curr_eur = cls._enable_currency('EUR')
+        cls.list_benelux = cls.env['product.pricelist'].create({
             'name': 'Benelux',
             'selectable': True,
-            'website_id': self.website.id,
-            'country_group_ids': [(4, self.benelux.id)],
+            'website_id': cls.website.id,
+            'country_group_ids': [Command.link(cls.benelux.id)],
+            'currency_id': cls.curr_eur.id,
             'sequence': 2,
-        })
-        item_benelux = self.env['product.pricelist.item'].create({
-            'pricelist_id': self.list_benelux.id,
-            'compute_price': 'percentage',
-            'base': 'list_price',
-            'percent_price': 10,
-            'currency_id': self.env.ref('base.EUR').id,
+            'item_ids': [
+                Command.create({
+                    'compute_price': 'percentage',
+                    'base': 'list_price',
+                    'percent_price': 10,
+                }),
+            ]
         })
 
-
-        self.list_christmas = self.env['product.pricelist'].create({
+        cls.europe = cls.env.ref('base.europe')
+        cls.list_christmas = cls.env['product.pricelist'].create({
             'name': 'Christmas',
             'selectable': False,
-            'website_id': self.website.id,
-            'country_group_ids': [(4, self.env.ref('base.europe').id)],
+            'website_id': cls.website.id,
+            'country_group_ids': [Command.link(cls.europe.id)],
             'sequence': 20,
-        })
-        item_christmas = self.env['product.pricelist.item'].create({
-            'pricelist_id': self.list_christmas.id,
-            'compute_price': 'formula',
-            'base': 'list_price',
-            'price_discount': 20,
+            'item_ids': [
+                Command.create({
+                    'compute_price': 'formula',
+                    'base': 'list_price',
+                    'price_discount': 20,
+                }),
+            ]
         })
 
-        list_europe = self.env['product.pricelist'].create({
+        cls.list_europe = cls.env['product.pricelist'].create({
             'name': 'EUR',
             'selectable': True,
-            'website_id': self.website.id,
-            'country_group_ids': [(4, self.env.ref('base.europe').id)],
+            'website_id': cls.website.id,
+            'country_group_ids': [Command.link(cls.europe.id)],
             'sequence': 3,
-            'currency_id': self.env.ref('base.EUR').id,
+            'currency_id': cls.curr_eur.id,
+            'item_ids': [
+                Command.create({
+                    'compute_price': 'formula',
+                    'base': 'list_price',
+                }),
+            ]
         })
-        item_europe = self.env['product.pricelist.item'].create({
-            'pricelist_id': list_europe.id,
-            'compute_price': 'formula',
-            'base': 'list_price',
-        })
-        self.env.ref('product.list0').website_id = self.website.id
-        self.website.pricelist_id = self.ref('product.list0')
 
-        ca_group = self.env['res.country.group'].create({
+        ca_group = cls.env['res.country.group'].create({
             'name': 'Canada',
-            'country_ids': [(6, 0, [self.ref('base.ca')])]
+            'country_ids': [Command.set([cls.env.ref('base.ca').id])]
         })
-        self.env['product.pricelist'].create({
+        cls.env['product.pricelist'].create({
             'name': 'Canada',
             'selectable': True,
-            'website_id': self.website.id,
-            'country_group_ids': [(6, 0, [ca_group.id])],
+            'website_id': cls.website.id,
+            'country_group_ids': [Command.set(ca_group.ids)],
             'sequence': 10
         })
-        self.args = {
+        cls.args = {
             'show': False,
             'current_pl': False,
         }
+
+    def setUp(self):
+        super().setUp()
         patcher = patch('odoo.addons.website_sale.models.website.Website.get_pricelist_available', wraps=self._get_pricelist_available)
         self.startPatcher(patcher)
 
@@ -155,11 +163,12 @@ class TestWebsitePriceList(TransactionCase):
     def test_get_pricelist_available_promocode(self):
         christmas_pl = self.list_christmas.id
 
+        # Christmas Pricelist only available for EU countries
         country_list = {
             False: True,
             'BE': True,
             'IT': True,
-            'US': True,
+            'US': False,
             'CA': False
         }
 
@@ -174,7 +183,7 @@ class TestWebsitePriceList(TransactionCase):
 
     def test_get_pricelist_available_show_with_auto_property(self):
         show = True
-        self.env.user.partner_id.country_id = self.env.ref('base.be')  # Add EUR pricelist auto
+        self.env.user.partner_id.country_id = self.country_be  # Add EUR pricelist auto
         current_pl = False
 
         country_list = {
@@ -190,16 +199,16 @@ class TestWebsitePriceList(TransactionCase):
                               % (country, len(pls), pls.mapped('name'), len(result), result))
 
     def test_pricelist_combination(self):
+        # Enable discounts to view discount in sale_order
+        self.env.user.groups_id += self.env.ref('sale.group_discount_per_so_line')
+
         product = self.env['product.product'].create({
             'name': 'Super Product',
             'list_price': 100,
             'taxes_id': False,
         })
-        current_website = self.env['website'].get_current_website()
-        website_pricelist = current_website.get_current_pricelist()
-        website_pricelist.write({
-            'discount_policy': 'with_discount',
-            'item_ids': [(5, 0, 0), (0, 0, {
+        self.website.pricelist_id.write({
+            'item_ids': [Command.clear(), Command.create({
                 'applied_on': '1_product',
                 'product_tmpl_id': product.product_tmpl_id.id,
                 'min_quantity': 500,
@@ -209,34 +218,33 @@ class TestWebsitePriceList(TransactionCase):
         })
         promo_pricelist = self.env['product.pricelist'].create({
             'name': 'Super Pricelist',
-            'discount_policy': 'without_discount',
-            'item_ids': [(0, 0, {
+            'item_ids': [Command.create({
                 'applied_on': '1_product',
                 'product_tmpl_id': product.product_tmpl_id.id,
                 'base': 'pricelist',
-                'base_pricelist_id': website_pricelist.id,
-                'compute_price': 'formula',
-                'price_discount': 25
+                'base_pricelist_id': self.website.pricelist_id.id,
+                'compute_price': 'percentage',
+                'percent_price': 25,
             })]
         })
         so = self.env['sale.order'].create({
             'partner_id': self.env.user.partner_id.id,
-            'order_line': [(0, 0, {
+            'website_id': self.website.id,
+            'order_line': [Command.create({
                 'name': product.name,
                 'product_id': product.id,
                 'product_uom_qty': 1,
                 'product_uom': product.uom_id.id,
                 'price_unit': product.list_price,
                 'tax_id': False,
-            })]
+            })],
         })
         sol = so.order_line
         self.assertEqual(sol.price_total, 100.0)
         so.pricelist_id = promo_pricelist
-        with MockRequest(self.env, website=current_website, sale_order_id=so.id):
-            so._cart_update(product_id=product.id, line_id=sol.id, set_qty=500)
-        self.assertEqual(sol.price_unit, 37.0, 'Both reductions should be applied')
-        self.assertEqual(sol.price_reduce, 27.75, 'Both reductions should be applied')
+        so._cart_update(product_id=product.id, line_id=sol.id, set_qty=500)
+        self.assertEqual(sol.price_unit, 100.0, 'Both reductions should be applied')
+        self.assertEqual(sol.discount, 72.25, 'Both reductions should be applied')
         self.assertEqual(sol.price_total, 13875)
 
     def test_pricelist_with_no_list_price(self):
@@ -245,21 +253,23 @@ class TestWebsitePriceList(TransactionCase):
             'list_price': 0,
             'taxes_id': False,
         })
-        current_website = self.env['website'].get_current_website()
-        website_pricelist = current_website.get_current_pricelist()
-        website_pricelist.write({
-            'discount_policy': 'without_discount',
-            'item_ids': [(5, 0, 0), (0, 0, {
-                'applied_on': '1_product',
-                'product_tmpl_id': product.product_tmpl_id.id,
-                'min_quantity': 0,
-                'compute_price': 'fixed',
-                'fixed_price': 10,
-            })]
+        self.website.pricelist_id.write({
+            'item_ids': [
+                Command.clear(),
+                Command.create({
+                    'applied_on': '1_product',
+                    'product_tmpl_id': product.product_tmpl_id.id,
+                    'min_quantity': 0,
+                    'compute_price': 'fixed',
+                    'fixed_price': 10,
+                }),
+            ]
         })
         so = self.env['sale.order'].create({
             'partner_id': self.env.user.partner_id.id,
-            'order_line': [(0, 0, {
+            'website_id': self.website.id,
+            'pricelist_id': self.website.pricelist_id.id,
+            'order_line': [Command.create({
                 'name': product.name,
                 'product_id': product.id,
                 'product_uom_qty': 5,
@@ -270,32 +280,11 @@ class TestWebsitePriceList(TransactionCase):
         })
         sol = so.order_line
         self.assertEqual(sol.price_total, 0)
-        so.pricelist_id = website_pricelist
-        with MockRequest(self.env, website=current_website, sale_order_id=so.id):
-            so._cart_update(product_id=product.id, line_id=sol.id, set_qty=6)
+        so._cart_update(product_id=product.id, line_id=sol.id, set_qty=6)
         self.assertEqual(sol.price_unit, 10.0, 'Pricelist price should be applied')
-        self.assertEqual(sol.price_reduce, 10.0, 'Pricelist price should be applied')
+        self.assertEqual(sol.discount, 0, 'Pricelist price should be applied')
         self.assertEqual(sol.price_total, 60.0)
 
-    def test_get_right_discount(self):
-        """ Test that `_get_sales_prices` from `product_template`
-        returns a dict with just `price_reduce` (no discount) as key
-        when the product is tax included.
-        """
-        tax = self.env['account.tax'].create({
-            'name': "Tax 10",
-            'amount': 10,
-        })
-
-        product = self.env['product.template'].create({
-            'name': 'Event Product',
-            'list_price': 10.0,
-            'taxes_id': tax,
-        })
-
-        prices = product._get_sales_prices(self.list_christmas)
-        self.assertFalse('base_price' in prices[product.id])
-
     def test_pricelist_item_based_on_cost_for_templates(self):
         """ Test that `_get_sales_prices` from `product_template` computes the correct price when
         the pricelist item is based on the cost of the product.
@@ -310,14 +299,18 @@ class TestWebsitePriceList(TransactionCase):
         })
 
         pa = self.env['product.attribute'].create({'name': 'Attribute'})
-        pav1 = self.env['product.attribute.value'].create({'name': 'Value1', 'attribute_id': pa.id})
-        pav2 = self.env['product.attribute.value'].create({'name': 'Value2', 'attribute_id': pa.id})
+        pav1, pav2 = self.env['product.attribute.value'].create([
+            {'name': 'Value1', 'attribute_id': pa.id},
+            {'name': 'Value2', 'attribute_id': pa.id},
+        ])
 
         product_template = self.env['product.template'].create({
             'name': 'Product Template', 'list_price': 10.0, 'standard_price': 5.0
         })
         self.assertEqual(product_template.standard_price, 5)
-        price = product_template._get_sales_prices(pricelist)[product_template.id]['price_reduce']
+        # Hack to enforce the use of this pricelist in the call to `_get_sales_price`
+        self.website.pricelist_id = pricelist
+        price = product_template._get_sales_prices(self.website)[product_template.id]['price_reduce']
         msg = "Template has no variants, the price should be computed based on the template's cost."
         self.assertEqual(price, 4.5, msg)
 
@@ -328,14 +321,54 @@ class TestWebsitePriceList(TransactionCase):
         self.assertEqual(product_template.standard_price, 0, msg)
         self.assertEqual(product_template.product_variant_ids[0].standard_price, 0)
 
-        price = product_template._get_sales_prices(pricelist)[product_template.id]['price_reduce']
+        self.website.pricelist_id = pricelist
+        price = product_template._get_sales_prices(self.website)[product_template.id]['price_reduce']
         msg = "Template has variants, the price should be computed based on the 1st variant's cost."
         self.assertEqual(price, 0, msg)
 
         product_template.product_variant_ids[0].standard_price = 20
-        price = product_template._get_sales_prices(pricelist)[product_template.id]['price_reduce']
+        self.website.pricelist_id = pricelist
+        price = product_template._get_sales_prices(self.website)[product_template.id]['price_reduce']
         self.assertEqual(price, 18, msg)
 
+    def test_base_price_with_discount_on_pricelist_tax_included(self):
+        """
+        Tests that the base price of a product with tax included
+        and discount from a price list is correctly displayed in the shop
+
+        ex: A product with a price of $61.98 ($75 tax incl. of 21%) and a discount of 20%
+        should display the base price of $75
+        """
+        self.env['res.config.settings'].create({                  # Set Settings:
+            'show_line_subtotals_tax_selection': 'tax_included',  # Set "Tax Included" on the "Display Product Prices"
+            'group_product_price_comparison': True,               # price comparison
+        }).execute()
+
+        product_tmpl = self.env['product.template'].create({
+            'name': 'Test Product',
+            'type': 'consu',
+            'list_price': 61.98,  # 75 tax incl.
+            'taxes_id': [
+                Command.create({
+                    'name': '21%',
+                    'type_tax_use': 'sale',
+                    'amount': 21,
+                })
+            ],
+            'is_published': True,
+        })
+        self.pricelist.write({
+            'item_ids': [Command.create({
+                'price_discount': 20,
+                'compute_price': 'formula',
+                'product_tmpl_id': product_tmpl.id,
+            })],
+        })
+        # Hack to enforce the use of this pricelist in the call to `_get_sales_price`
+        self.website.pricelist_id = self.pricelist
+        res = product_tmpl._get_sales_prices(self.website)
+        self.assertEqual(res[product_tmpl.id]['base_price'], 75)
+
 def simulate_frontend_context(self, website_id=1):
     # Mock this method will be enough to simulate frontend context in most methods
     def get_request_website():
@@ -345,66 +378,67 @@ def simulate_frontend_context(self, website_id=1):
 
 
 @tagged('post_install', '-at_install')
-class TestWebsitePriceListAvailable(TransactionCase):
-    def setUp(self):
-        super(TestWebsitePriceListAvailable, self).setUp()
-        Pricelist = self.env['product.pricelist']
-        Website = self.env['website']
+class TestWebsitePriceListAvailable(WebsiteSaleCommon):
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        Pricelist = cls.env['product.pricelist']
+        Website = cls.env['website']
 
         # Set up 2 websites
-        self.website = Website.browse(1)
-        self.website2 = Website.create({'name': 'Website 2'})
+        cls.website2 = Website.create({'name': 'Website 2'})
 
         # Remove existing pricelists and create new ones
         existing_pricelists = Pricelist.search([])
-        self.backend_pl = Pricelist.create({
+        cls.backend_pl = Pricelist.create({
             'name': 'Backend Pricelist',
             'website_id': False,
         })
-        self.generic_pl_select = Pricelist.create({
+        cls.generic_pl_select = Pricelist.create({
             'name': 'Generic Selectable Pricelist',
             'selectable': True,
             'website_id': False,
         })
-        self.generic_pl_code = Pricelist.create({
+        cls.generic_pl_code = Pricelist.create({
             'name': 'Generic Code Pricelist',
             'code': 'GENERICCODE',
             'website_id': False,
         })
-        self.generic_pl_code_select = Pricelist.create({
+        cls.generic_pl_code_select = Pricelist.create({
             'name': 'Generic Code Selectable Pricelist',
             'code': 'GENERICCODESELECT',
             'selectable': True,
             'website_id': False,
         })
-        self.w1_pl = Pricelist.create({
+        cls.w1_pl = Pricelist.create({
             'name': 'Website 1 Pricelist',
-            'website_id': self.website.id,
+            'website_id': cls.website.id,
         })
-        self.w1_pl_select = Pricelist.create({
+        cls.w1_pl_select = Pricelist.create({
             'name': 'Website 1 Pricelist Selectable',
-            'website_id': self.website.id,
+            'website_id': cls.website.id,
             'selectable': True,
         })
-        self.w1_pl_code_select = Pricelist.create({
+        cls.w1_pl_code_select = Pricelist.create({
             'name': 'Website 1 Pricelist Code Selectable',
-            'website_id': self.website.id,
+            'website_id': cls.website.id,
             'code': 'W1CODESELECT',
             'selectable': True,
         })
-        self.w1_pl_code = Pricelist.create({
+        cls.w1_pl_code = Pricelist.create({
             'name': 'Website 1 Pricelist Code',
-            'website_id': self.website.id,
+            'website_id': cls.website.id,
             'code': 'W1CODE',
         })
-        self.w2_pl = Pricelist.create({
+        cls.w2_pl = Pricelist.create({
             'name': 'Website 2 Pricelist',
-            'website_id': self.website2.id,
+            'website_id': cls.website2.id,
         })
-        existing_pricelists.write({'active': False})
-
-        self.website = self.env['website'].browse(1)
+        existing_pricelists.action_archive()
 
+    def setUp(self):
+        super().setUp()
         simulate_frontend_context(self)
 
     def test_get_pricelist_available(self):
@@ -425,7 +459,7 @@ class TestWebsitePriceListAvailable(TransactionCase):
         # Real case if for public user. His `property_product_pricelist` need to be set as it is passed
         # through `_get_pl_partner_order` as the `website_pl` when searching for available pricelists
         # for active users.
-        public_partner = self.env.ref('base.public_partner')
+        public_partner = self.public_partner
         self.assertFalse(public_partner.active, "Ensure public partner is inactive (purpose of this test)")
         pl = public_partner.property_product_pricelist
         self.assertEqual(len(pl), 1, "Inactive partner should still get a `property_product_pricelist`")
@@ -434,9 +468,36 @@ class TestWebsitePriceListAvailable(TransactionCase):
 @tagged('post_install', '-at_install')
 class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
     def setUp(self):
-        super(TestWebsitePriceListAvailableGeoIP, self).setUp()
+        super().setUp()
         # clean `property_product_pricelist` for partner for this test (clean setup)
-        self.env['ir.property'].search([('res_id', '=', 'res.partner,%s' % self.env.user.partner_id.id)]).unlink()
+        self.env.invalidate_all()
+        for field in self.env.registry.many2one_company_dependents['res.partner']:
+            self.env.cr.execute(SQL(
+                """
+                UPDATE %(table)s
+                SET %(field)s = (
+                    SELECT jsonb_object_agg(key, value)
+                    FROM jsonb_each(%(field)s)
+                    WHERE value != %(id_)s
+                )
+                WHERE %(field)s IS NOT NULL
+                """,
+                table=SQL.identifier(self.env[field.model_name]._table),
+                field=SQL.identifier(field.name),
+                id_=SQL('to_jsonb(%s::int)', self.env.user.partner_id.id),
+            ))
+        if fields_ := self.env.registry.many2one_company_dependents['res.partner']:
+            field_ids = [self.env['ir.model.fields']._get_ids(field.model_name).get(field.name) for field in fields_]
+            self.env.cr.execute(SQL(
+                """
+                DELETE FROM ir_default
+                WHERE field_id IN %(field_ids)s
+                AND json_value = %(id_text)s
+                """,
+                field_ids=tuple(field_ids),
+                id_text=str(self.env.user.partner_id.id),
+            ))
+        self.env.registry.clear_cache()
 
         # set different country groups on pricelists
         c_EUR = self.env.ref('base.europe')
@@ -446,14 +507,17 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
         })
 
         self.BE = self.env.ref('base.be')
+        self.US = self.env.ref('base.us')
         NL = self.env.ref('base.nl')
-        c_BE = self.env['res.country.group'].create({'name': 'Belgium', 'country_ids': [(6, 0, [self.BE.id])]})
-        c_NL = self.env['res.country.group'].create({'name': 'Netherlands', 'country_ids': [(6, 0, [NL.id])]})
+        c_BE, c_NL = self.env['res.country.group'].create([
+            {'name': 'Belgium', 'country_ids': [(6, 0, [self.BE.id])]},
+            {'name': 'Netherlands', 'country_ids': [(6, 0, [NL.id])]},
+        ])
 
-        (self.backend_pl + self.generic_pl_select + self.generic_pl_code + self.w1_pl_select).write({'country_group_ids': [(6, 0, [c_BE.id])]})
-        (self.generic_pl_code_select + self.w1_pl + self.w2_pl).write({'country_group_ids': [(6, 0, [c_BENELUX.id])]})
-        (self.w1_pl_code).write({'country_group_ids': [(6, 0, [c_EUR.id])]})
-        (self.w1_pl_code_select).write({'country_group_ids': [(6, 0, [c_NL.id])]})
+        (self.backend_pl + self.generic_pl_select + self.generic_pl_code + self.w1_pl_select).write({'country_group_ids': [Command.set(c_BE.ids)]})
+        (self.generic_pl_code_select + self.w1_pl + self.w2_pl).write({'country_group_ids': [Command.set(c_BENELUX.ids)]})
+        (self.w1_pl_code).write({'country_group_ids': [Command.set(c_EUR.ids)]})
+        (self.w1_pl_code_select).write({'country_group_ids': [Command.set(c_NL.ids)]})
 
         #        pricelist        | selectable | website | code | country group |
         # ----------------------------------------------------------------------|
@@ -471,7 +535,7 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
         self.website1_be_pl = self.generic_pl_select + self.generic_pl_code + self.w1_pl_select + self.generic_pl_code_select + self.w1_pl + self.w1_pl_code
 
     def test_get_pricelist_available_geoip(self):
-        # Test get all available pricelists with geoip and no partner pricelist (ir.property)
+        # Test get all available pricelists with geoip and no partner pricelist
 
         # property_product_pricelist will also be returned in the available pricelists
         self.website1_be_pl += self.env.user.partner_id.property_product_pricelist
@@ -481,14 +545,14 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
         self.assertEqual(pls, self.website1_be_pl, "Only pricelists for BE and accessible on website should be returned, and the partner pl")
 
     def test_get_pricelist_available_geoip2(self):
-        # Test get all available pricelists with geoip and a partner pricelist (ir.property) not website compliant
+        # Test get all available pricelists with geoip and a partner pricelist not website compliant
         self.env.user.partner_id.property_product_pricelist = self.backend_pl
         with patch('odoo.addons.website_sale.models.website.Website._get_geoip_country_code', return_value=self.BE.code):
             pls = self.website.get_pricelist_available()
         self.assertEqual(pls, self.website1_be_pl, "Only pricelists for BE and accessible on website should be returned as partner pl is not website compliant")
 
     def test_get_pricelist_available_geoip3(self):
-        # Test get all available pricelists with geoip and a partner pricelist (ir.property) website compliant (but not geoip compliant)
+        # Test get all available pricelists with geoip and a partner pricelist website compliant (but not geoip compliant)
         self.env.user.partner_id.property_product_pricelist = self.w1_pl_code_select
         with patch('odoo.addons.website_sale.models.website.Website._get_geoip_country_code', return_value=self.BE.code):
             pls = self.website.get_pricelist_available()
@@ -506,6 +570,13 @@ class TestWebsitePriceListAvailableGeoIP(TestWebsitePriceListAvailable):
             pls = self.website.get_pricelist_available(show_visible=True)
         self.assertEqual(pls, pls_to_return + current_pl, "Only pricelists for BE, accessible en website and selectable should be returned. It should also return the applied promo pl")
 
+    def test_get_pricelist_available_geoip5(self):
+        # Test get all available pricelists with geoip for a country not existing in any pricelists
+
+        with patch('odoo.addons.website_sale.models.website.Website._get_geoip_country_code', return_value=self.US.code):
+            pricelists = self.website.get_pricelist_available()
+        self.assertFalse(pricelists, "Pricelists specific to NL and BE should not be returned for US.")
+
 
 @tagged('post_install', '-at_install')
 class TestWebsitePriceListHttp(HttpCaseWithUserPortal):
@@ -542,7 +613,7 @@ class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
         - Set website's company to company 2
         - Demo user will still be in company 1
         '''
-        super(TestWebsitePriceListMultiCompany, self).setUp()
+        super().setUp()
 
         self.demo_user = self.user_demo
 
@@ -554,9 +625,6 @@ class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
         Website = self.env['website']
         self.website = self.env.ref('website.default_website')
         self.website.company_id = self.company2
-        # Delete unused website, it will make PL manipulation easier, avoiding
-        # UserError being thrown when a website wouldn't have any PL left.
-        Website.search([('id', '!=', self.website.id)]).unlink()
         self.website2 = Website.create({
             'name': 'Website 2',
             'company_id': self.company1.id,
@@ -580,16 +648,11 @@ class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
         # Ensure everything was done correctly
         self.assertEqual(self.demo_user.partner_id.with_company(self.company1.id).property_product_pricelist, self.c1_pl)
         self.assertEqual(self.demo_user.partner_id.with_company(self.company2.id).property_product_pricelist, self.c2_pl)
-        irp1 = self.env['ir.property'].with_company(self.company1)._get("property_product_pricelist", "res.partner", self.demo_user.partner_id.id)
-        irp2 = self.env['ir.property'].with_company(self.company2)._get("property_product_pricelist", "res.partner", self.demo_user.partner_id.id)
-        self.assertEqual((irp1, irp2), (self.c1_pl, self.c2_pl), "Ensure there is an `ir.property` for demo partner for every company, and that the pricelist is the company specific one.")
-        # ---------------------------------- IR.PROPERTY -------------------------------------
-        # id |            name              |     res_id    | company_id |   value_reference
-        # ------------------------------------------------------------------------------------
-        # 1  | 'property_product_pricelist' |               |      1     | product.pricelist,1
-        # 2  | 'property_product_pricelist' |               |      2     | product.pricelist,2
-        # 3  | 'property_product_pricelist' | res.partner,8 |      1     | product.pricelist,10
-        # 4  | 'property_product_pricelist' | res.partner,8 |      2     | product.pricelist,11
+        # property_product_pricelist has been cached
+        field = self.env['res.partner']._fields['property_product_pricelist']
+        cache_rp1 = self.env.cache.get(self.demo_user.partner_id.with_company(self.company1.id), field)
+        cache_rp2 = self.env.cache.get(self.demo_user.partner_id.with_company(self.company2.id), field)
+        self.assertEqual((cache_rp1, cache_rp2), (self.c1_pl.id, self.c2_pl.id), "Ensure the pricelist is the company specific one.")
 
     def test_property_product_pricelist_multi_company(self):
         ''' Test that the `property_product_pricelist` of `res.partner` is read
@@ -598,13 +661,13 @@ class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
             is not the same as its user's company.
 
             Here, as demo user (company1), we will visit website1 (company2).
-            It should return the ir.property for demo user for company2 and not
+            It should return the data for demo user for company2 and not
             for the company1 as we should get the website's company pricelist
             and not the demo user's current company pricelist.
         '''
         simulate_frontend_context(self, self.website.id)
 
-        # First check: It should return ir.property,4 as company_id is
+        # First check: It should return c2_pl as company_id is
         # website.company_id and not env.user.company_id
         company_id = self.website.company_id.id
         partner = self.demo_user.partner_id.with_company(company_id)
@@ -642,6 +705,7 @@ class TestWebsitePriceListMultiCompany(TransactionCaseWithUserDemo):
         # archive should not raise an error
         self.c2_pl.with_user(self.demo_user).with_context(allowed_company_ids=self.company2.ids).write({'active': False})
 
+
 @tagged('post_install', '-at_install')
 class TestWebsiteSaleSession(HttpCaseWithUserPortal):
 
@@ -656,10 +720,21 @@ class TestWebsiteSaleSession(HttpCaseWithUserPortal):
             'login': 'toto',
             'password': 'long_enough_password',
         })
-        user_pricelist = self.env['product.pricelist'].create({
-            'name': 'User Pricelist',
-            'website_id': website.id,
-            'code': 'User_pricelist',
-        })
+        user_pricelist, _ = self.env['product.pricelist'].create([
+            {
+                'name': 'User Pricelist',
+                'website_id': website.id,
+                'code': 'User_pricelist',
+                'selectable': True,
+                'sequence': 40, # Be sure not to use it by default
+            },
+            {
+                'name': 'Other Pricelist',
+                'website_id': website.id,
+                'code': 'Other_pricelist',
+                'selectable': True,
+                'sequence': 30,
+            }
+        ])
         test_user.partner_id.property_product_pricelist = user_pricelist
-        self.start_tour("/shop", 'website_sale_shop_pricelist_tour', login="")
+        self.start_tour("/shop", 'website_sale.website_sale_shop_pricelist_tour', login="")
