@@ -10,7 +10,6 @@ To configure product attributes, activate *Attributes & Variants* in the Website
 
 Finally, the module comes with an option to display an attribute summary table in product web pages (available in Customize menu).
     """,
-    'author': 'Odoo SA',
     'category': 'Website/Website',
     'version': '1.0',
     'depends': ['website_sale'],
@@ -18,11 +17,22 @@ Finally, the module comes with an option to display an attribute summary table i
         'security/ir.model.access.csv',
         'views/website_sale_comparison_template.xml',
         'views/website_sale_comparison_view.xml',
+        'views/snippets.xml',
     ],
     'demo': [
         'data/website_sale_comparison_data.xml',
         'data/website_sale_comparison_demo.xml',
     ],
     'installable': True,
+    'assets': {
+        'web.assets_frontend': [
+            'website_sale_comparison/static/src/scss/website_sale_comparison.scss',
+            'website_sale_comparison/static/src/js/**/*.js',
+            'website_sale_comparison/static/src/xml/comparison.xml',
+        ],
+        'web.assets_tests': [
+            'website_sale_comparison/static/tests/**/*',
+        ],
+    },
     'license': 'LGPL-3',
 }
