@@ -1,5 +1,6 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import variant
 from . import main
+from . import reorder
+from . import variant
+from . import website_sale
