@@ -1,5 +1,6 @@
-# -*- coding: utf-8 -*-
-from odoo import api, fields, models
+# Part of Odoo. See LICENSE file for full copyright and licensing details.
+
+from odoo import fields, models
 
 
 class Website(models.Model):
@@ -18,12 +19,15 @@ class Website(models.Model):
     def _get_warehouse_available(self):
         return (
             self.warehouse_id.id or
-            self.env['ir.default'].get('sale.order', 'warehouse_id', company_id=self.company_id.id) or
-            self.env['ir.default'].get('sale.order', 'warehouse_id') or
+            self.env['ir.default'].sudo()._get('sale.order', 'warehouse_id', company_id=self.company_id.id) or
+            self.env['ir.default'].sudo()._get('sale.order', 'warehouse_id') or
             self.env['stock.warehouse'].sudo().search([('company_id', '=', self.company_id.id)], limit=1).id
         )
 
     # FIXME VFE check if still needed
     def sale_get_order(self, *args, **kwargs):
         so = super().sale_get_order(*args, **kwargs)
-        return so.with_context(warehouse=so.warehouse_id.id) if so else so
+        return so.with_context(warehouse_id=so.warehouse_id.id) if so else so
+
+    def _get_product_available_qty(self, product, **kwargs):
+        return product.with_context(warehouse_id=self._get_warehouse_available()).free_qty
