@@ -1,7 +1,10 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from . import test_website_sale_stock_abandoned_cart_email
+from . import test_website_sale_stock_multilang
+from . import test_website_sale_stock_product_configurator
 from . import test_website_sale_stock_product_warehouse
+from . import test_website_sale_stock_delivery
 from . import test_website_sale_stock_stock_notification
 from . import test_website_sale_stock_reorder_from_portal
+from . import test_website_sale_stock_stock_message
