@@ -20,9 +20,10 @@ class Users(models.Model):
     def write(self, vals):
         """ Trigger automatic subscription based on updated user groups """
         res = super(Users, self).write(vals)
-        if vals.get('groups_id'):
-            added_group_ids = [command[1] for command in vals['groups_id'] if command[0] == 4]
-            added_group_ids += [id for command in vals['groups_id'] if command[0] == 6 for id in command[2]]
+        sanitized_vals = self._remove_reified_groups(vals)
+        if sanitized_vals.get('groups_id'):
+            added_group_ids = [command[1] for command in sanitized_vals['groups_id'] if command[0] == 4]
+            added_group_ids += [id for command in sanitized_vals['groups_id'] if command[0] == 6 for id in command[2]]
             self.env['slide.channel'].sudo().search([('enroll_group_ids', 'in', added_group_ids)])._action_add_members(self.mapped('partner_id'))
         return res
 
