@@ -2,10 +2,11 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import base64
 
+from odoo import http
 from odoo.addons.mail.tests.common import mail_new_test_user
 from odoo.addons.website_slides.tests import common
 from odoo.exceptions import AccessError
-from odoo.tests import tagged
+from odoo.tests import tagged, HttpCase
 from odoo.tools import mute_logger
 
 
@@ -42,6 +43,17 @@ class TestAccess(common.SlidesCase):
         self.slide.with_user(self.user_emp).read(['name'])
 
         # not member anymore -> cannot read
+        membership.action_archive()
+        self.channel.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_emp).read(['name'])
+
+        # re-activate member -> can read again
+        membership.action_unarchive()
+        self.channel.with_user(self.user_emp).read(['name'])
+        self.slide.with_user(self.user_emp).read(['name'])
+
+        # unlink membership -> cannot read
         membership.unlink()
         self.channel.with_user(self.user_emp).read(['name'])
         with self.assertRaises(AccessError):
@@ -152,6 +164,158 @@ class TestAccess(common.SlidesCase):
         self.slide.with_user(self.user_portal).read(['name'])
         self.slide.with_user(self.user_public).read(['name'])
 
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_visibility_public(self):
+        self.channel.write({'visibility': 'public'})
+        self.slide.write({'is_preview': True})
+        self.slide.flush_model()
+
+        self.channel.with_user(self.user_officer).read(['name'])
+        self.channel.with_user(self.user_manager).read(['name'])
+        self.channel.with_user(self.user_emp).read(['name'])
+        self.channel.with_user(self.user_portal).read(['name'])
+        self.channel.with_user(self.user_public).read(['name'])
+
+        self.slide.with_user(self.user_officer).read(['name'])
+        self.slide.with_user(self.user_manager).read(['name'])
+        self.slide.with_user(self.user_emp).read(['name'])
+        self.slide.with_user(self.user_portal).read(['name'])
+        self.slide.with_user(self.user_public).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_public_with_website_published(self):
+        self.channel.write({'visibility': 'public', 'website_published': False})
+
+        self.channel.with_user(self.user_officer).read(['name'])
+        self.channel.with_user(self.user_manager).read(['name'])
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_portal).read(['name'])
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_public).read(['name'])
+
+        self.slide.with_user(self.user_officer).read(['name'])
+        self.slide.with_user(self.user_manager).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_portal).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_public).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_visibility_members(self):
+        self.channel.write({'visibility': 'members'})
+        self.channel.flush_model()
+        user_emp_membership = self.env['slide.channel.partner'].create({
+            'channel_id': self.channel.id,
+            'partner_id': self.user_emp.partner_id.id,
+        })
+
+        self.channel.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_portal).read(['name'])
+
+        user_emp_membership.action_archive()
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_emp).read(['name'])
+
+        user_emp_membership.unlink()
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_emp).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_visiblilty_members_as_invited(self):
+        self.channel.visibility = 'members'
+        self.channel.flush_recordset()
+
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_portal).read(['name'])
+
+        user_portal_membership = self.env['slide.channel.partner'].create({
+            'channel_id': self.channel.id,
+            'partner_id': self.user_portal.partner_id.id,
+            'member_status': 'invited'
+        })
+        self.channel.with_user(self.user_portal).read(['name'])
+
+        user_portal_membership.action_archive()
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_portal).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_members_with_website_published(self):
+        self.channel.write({'visibility': 'members', 'website_published': False})
+        self.channel.flush_model()
+
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_emp).read(['name'])
+
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_portal).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_visibility_connected(self):
+        self.channel.write({'visibility': 'connected'})
+
+        self.channel.with_user(self.user_officer).read(['name'])
+        self.channel.with_user(self.user_manager).read(['name'])
+        self.channel.with_user(self.user_emp).read(['name'])
+        self.channel.with_user(self.user_portal).read(['name'])
+        with self.assertRaises(AccessError):
+            self.channel.with_user(self.user_public).read(['name'])
+
+        self.slide.with_user(self.user_officer).read(['name'])
+        self.slide.with_user(self.user_manager).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_emp).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_portal).read(['name'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_public).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_channel_visiblilty_connected_as_invited(self):
+        self.channel.visibility = 'connected'
+        self.channel.flush_recordset()
+
+        self.env['slide.channel.partner'].create({
+            'channel_id': self.channel.id,
+            'partner_id': self.user_emp.partner_id.id,
+            'member_status': 'invited'
+        })
+        self.channel.with_user(self.user_emp).read(['name'])
+
+    @mute_logger('odoo.models', 'odoo.addons.base.models.ir_rule')
+    def test_access_slide_slide_as_invited(self):
+        """ Check that preview slides are visible to logged invited attendees, but not others, nor non published ones."""
+        self.env['slide.channel.partner'].create({
+            'channel_id': self.channel.id,
+            'partner_id': self.user_portal.partner_id.id,
+            'member_status': 'invited'
+        })
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_portal).read(['name'])
+
+        self.slide.is_preview = True
+        self.slide.with_user(self.user_portal).read(['name'])
+
+        self.channel.visibility = 'connected'
+        self.channel.flush_recordset()
+        self.slide.with_user(self.user_portal).read(['name'])
+
+        self.channel.visibility = 'members'
+        self.channel.flush_recordset()
+        self.slide.with_user(self.user_portal).read(['name'])
+
+        self.slide.is_published = False
+        self.slide.flush_recordset(['is_published'])
+        with self.assertRaises(AccessError):
+            self.slide.with_user(self.user_portal).read(['name'])
+
 
 @tagged('functional', 'security')
 class TestRemoveMembership(common.SlidesCase):
@@ -325,10 +489,8 @@ class TestAccessFeatures(common.SlidesCase):
             resource1.with_user(self.user_public).write({'name': 'other name'})
             resource3.with_user(self.user_public).write({'name': 'other name'})
 
-        # public access to knowing if there are resources, also by type
-        self.assertTrue(self.slide_3.with_user(self.user_public)._has_additional_resources())
-        self.assertTrue(self.slide_3.with_user(self.user_public)._has_additional_resources('file'))
-        self.assertTrue(self.slide_3.with_user(self.user_public)._has_additional_resources('url'))
+        # public access to knowing if there are resources
+        self.assertTrue(self.slide_3.with_user(self.user_public).sudo().slide_resource_ids)
 
         # No random portal access
         with self.assertRaises(AccessError):
@@ -365,3 +527,45 @@ class TestAccessFeatures(common.SlidesCase):
         resource2.with_user(self.user_manager).write({'name': 'Another name'})
         resource2.with_user(self.user_manager).unlink()
         self.env['slide.slide.resource'].with_user(self.user_manager).create(resource_values)
+
+
+@tagged("functional")
+class TestReview(common.SlidesCase, HttpCase):
+    @mute_logger("odoo.addons.http_routing.models.ir_http", "odoo.http")
+    def test_channel_multiple_reviews(self):
+        self.authenticate("admin", "admin")
+
+        res1 = self.opener.post(
+            url="%s/mail/message/post" % self.base_url(),
+            json={
+                "params": {
+                    "thread_model": "slide.channel",
+                    "thread_id": self.channel.id,
+                    "post_data": {
+                        "body": "My first review :)",
+                        "subtype_xmlid": "mail.mt_comment",
+                        "rating_value": "2",
+                    },
+                    "pid": self.env.user.partner_id.id,
+                },
+            },
+        )
+        self.assertIn("My first review :)", res1.text)
+
+
+        res2 = self.opener.post(
+            url="%s/mail/message/post" % self.base_url(),
+            json={
+                "params": {
+                    "thread_model": "slide.channel",
+                    "thread_id": self.channel.id,
+                    "post_data": {
+                        "body": "My second review :)",
+                        "subtype_xmlid": "mail.mt_comment",
+                        "rating_value": "2",
+                    },
+                    "pid": self.env.user.partner_id.id,
+                },
+            },
+        )
+        self.assertIn("odoo.exceptions.ValidationError", res2.text)
