@@ -14,12 +14,14 @@
     'data': [
         'security/ir.model.access.csv',
         'security/website_slides_forum_security.xml',
-        'views/forum_views.xml',
+        'views/forum_forum_views.xml',
+        'views/forum_post_views.xml',
+        'views/res_config_settings_views.xml',
         'views/slide_channel_views.xml',
         'views/website_slides_menu_views.xml',
-        'views/assets.xml',
+        'views/forum_forum_templates.xml',
         'views/website_slides_templates.xml',
-        'views/website_slides_forum_templates.xml'
+        'views/snippets.xml',
     ],
     'demo': [
         'data/slide_channel_demo.xml',
