@@ -16,8 +16,8 @@ class Channel(models.Model):
 
     def action_redirect_to_forum(self):
         self.ensure_one()
-        action = self.env["ir.actions.actions"]._for_xml_id("website_forum.action_forum_post")
-        action['view_mode'] = 'tree'
+        action = self.env["ir.actions.actions"]._for_xml_id("website_forum.forum_post_action")
+        action['view_mode'] = 'list'
         action['context'] = {
             'create': False
         }
