@@ -2,6 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from odoo import api, fields, models
+from odoo.osv import expression
 
 
 class SlidePartnerRelation(models.Model):
@@ -27,6 +28,21 @@ class SlidePartnerRelation(models.Model):
                 'completed': True
             })
 
+    def _recompute_completion(self):
+        super(SlidePartnerRelation, self)._recompute_completion()
+        # Update certified partners
+        certification_success_slides = self.filtered(lambda slide: slide.survey_scoring_success)
+        if not certification_success_slides:
+            return
+        certified_channels_domain = expression.OR([
+            [('partner_id', '=', slide.partner_id.id), ('channel_id', '=', slide.channel_id.id)]
+            for slide in certification_success_slides
+        ])
+        self.env['slide.channel.partner'].search(expression.AND([
+            [("survey_certification_success", "=", False)],
+            certified_channels_domain]
+        )).survey_certification_success = True
+
 class Slide(models.Model):
     _inherit = 'slide.slide'
 
@@ -65,6 +81,12 @@ class Slide(models.Model):
             if slide.slide_category == 'certification' or not slide.is_preview:
                 slide.is_preview = False
 
+    @api.depends('slide_type')
+    def _compute_slide_icon_class(self):
+        certification = self.filtered(lambda slide: slide.slide_type == 'certification')
+        certification.slide_icon_class = 'fa-trophy'
+        super(Slide, self - certification)._compute_slide_icon_class()
+
     @api.depends('slide_category', 'source_type')
     def _compute_slide_type(self):
         super(Slide, self)._compute_slide_type()
