@@ -11,8 +11,8 @@ class TestSurvey(SlidesCase):
     def setUp(self):
         super(TestSurvey, self).setUp()
 
-        self.survey = self.env['survey.survey'].create({'title': 'TestSurvey'})
-        self.survey2 = self.env['survey.survey'].create({'title': 'TestSurvey'})
+        self.survey = self.env['survey.survey'].create({'title': 'TestSurvey', 'certification': True})
+        self.survey2 = self.env['survey.survey'].create({'title': 'TestSurvey', 'certification': True})
 
     @users('user_manager')
     def test_unlink(self):
