@@ -32,6 +32,7 @@ class IrConfigParameter(models.Model):
     _description = 'System Parameter'
     _rec_name = 'key'
     _order = 'key'
+    _allow_sudo_commands = False
 
     key = fields.Char(required=True)
     value = fields.Text(required=True)
@@ -64,7 +65,7 @@ class IrConfigParameter(models.Model):
         :return: The value of the parameter, or ``default`` if it does not exist.
         :rtype: string
         """
-        self.check_access_rights('read')
+        self.browse().check_access('read')
         return self._get_param(key) or default
 
     @api.model
@@ -103,7 +104,7 @@ class IrConfigParameter(models.Model):
 
     @api.model_create_multi
     def create(self, vals_list):
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return super(IrConfigParameter, self).create(vals_list)
 
     def write(self, vals):
@@ -111,11 +112,11 @@ class IrConfigParameter(models.Model):
             illegal = _default_parameters.keys() & self.mapped('key')
             if illegal:
                 raise ValidationError(_("You cannot rename config parameters with keys %s", ', '.join(illegal)))
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return super(IrConfigParameter, self).write(vals)
 
     def unlink(self):
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return super(IrConfigParameter, self).unlink()
 
     @api.ondelete(at_uninstall=False)
