@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import base64
+import warnings
 from collections import defaultdict, OrderedDict
 from decorator import decorator
 from operator import attrgetter
-import importlib
+from textwrap import dedent
 import io
 import logging
 import os
-import pkg_resources
 import shutil
-import tempfile
 import threading
 import zipfile
 
@@ -27,13 +26,13 @@ import psycopg2
 import odoo
 from odoo import api, fields, models, modules, tools, _
 from odoo.addons.base.models.ir_model import MODULE_UNINSTALL_FLAG
-from odoo.exceptions import AccessDenied, UserError
+from odoo.exceptions import AccessDenied, UserError, ValidationError
 from odoo.osv import expression
 from odoo.tools.parse_version import parse_version
-from odoo.tools.misc import topological_sort
-from odoo.tools.translate import TranslationImporter
+from odoo.tools.misc import topological_sort, get_flag
+from odoo.tools.translate import TranslationImporter, get_po_paths
 from odoo.http import request
-from odoo.modules import get_module_path, get_module_resource
+from odoo.modules import get_module_path
 
 _logger = logging.getLogger(__name__)
 
@@ -80,29 +79,11 @@ class ModuleCategory(models.Model):
     _name = "ir.module.category"
     _description = "Application"
     _order = 'name'
-
-    @api.depends('module_ids')
-    def _compute_module_nr(self):
-        self.env['ir.module.module'].flush_model(['category_id'])
-        self.flush_model(['parent_id'])
-        cr = self._cr
-        cr.execute('SELECT category_id, COUNT(*) \
-                      FROM ir_module_module \
-                     WHERE category_id IN %(ids)s \
-                        OR category_id IN (SELECT id \
-                                             FROM ir_module_category \
-                                            WHERE parent_id IN %(ids)s) \
-                     GROUP BY category_id', {'ids': tuple(self.ids)}
-                   )
-        result = dict(cr.fetchall())
-        for cat in self.filtered('id'):
-            cr.execute('SELECT id FROM ir_module_category WHERE parent_id=%s', (cat.id,))
-            cat.module_nr = sum([result.get(c, 0) for (c,) in cr.fetchall()], result.get(cat.id, 0))
+    _allow_sudo_commands = False
 
     name = fields.Char(string='Name', required=True, translate=True, index=True)
     parent_id = fields.Many2one('ir.module.category', string='Parent Application', index=True)
     child_ids = fields.One2many('ir.module.category', 'parent_id', string='Child Applications')
-    module_nr = fields.Integer(string='Number of Apps', compute='_compute_module_nr')
     module_ids = fields.One2many('ir.module.module', 'category_id', string='Modules')
     description = fields.Text(string='Description', translate=True)
     sequence = fields.Integer(string='Sequence')
@@ -118,6 +99,12 @@ class ModuleCategory(models.Model):
         for cat in self:
             cat.xml_id = xml_ids.get(cat.id, [''])[0]
 
+    @api.constrains('parent_id')
+    def _check_parent_not_circular(self):
+        if self._has_cycle():
+            raise ValidationError(_("Error ! You cannot create recursive categories."))
+
+
 class MyFilterMessages(Transform):
     """
     Custom docutils transform to remove `system message` for a document and
@@ -154,21 +141,20 @@ STATES = [
 ]
 
 
+XML_DECLARATION = (
+    '<?xml version='.encode('utf-8'),
+    '<?xml version='.encode('utf-16-be'),
+    '<?xml version='.encode('utf-16-le'),
+)
+
+
 class Module(models.Model):
     _name = "ir.module.module"
     _rec_name = "shortdesc"
     _rec_names_search = ['name', 'shortdesc', 'summary']
     _description = "Module"
     _order = 'application desc,sequence,name'
-
-    @api.model
-    def get_views(self, views, options=None):
-        res = super().get_views(views, options)
-        if res['views'].get('form', {}).get('toolbar'):
-            install_id = self.env.ref('base.action_server_module_immediate_install').id
-            action = [rec for rec in res['views']['form']['toolbar']['action'] if rec.get('id', False) != install_id]
-            res['views']['form']['toolbar'] = {'action': action}
-        return res
+    _allow_sudo_commands = False
 
     @classmethod
     def get_module_info(cls, name):
@@ -180,22 +166,41 @@ class Module(models.Model):
 
     @api.depends('name', 'description')
     def _get_desc(self):
+        def _apply_description_images(doc):
+            html = lxml.html.document_fromstring(doc)
+            for element, _attribute, _link, _pos in html.iterlinks():
+                if element.get('src') and not '//' in element.get('src') and not 'static/' in element.get('src'):
+                    element.set('src', "/%s/static/description/%s" % (module.name, element.get('src')))
+            return tools.html_sanitize(lxml.html.tostring(html, encoding='unicode'))
+
         for module in self:
             if not module.name:
                 module.description_html = False
                 continue
-            module_path = modules.get_module_path(module.name, display_warning=False)  # avoid to log warning for fake community module
-            if module_path:
-                path = modules.check_resource_path(module_path, 'static/description/index.html')
-            if module_path and path:
+            path = os.path.join(module.name, 'static/description/index.html')
+            try:
                 with tools.file_open(path, 'rb') as desc_file:
                     doc = desc_file.read()
-                    html = lxml.html.document_fromstring(doc)
-                    for element, attribute, link, pos in html.iterlinks():
-                        if element.get('src') and not '//' in element.get('src') and not 'static/' in element.get('src'):
-                            element.set('src', "/%s/static/description/%s" % (module.name, element.get('src')))
-                    module.description_html = tools.html_sanitize(lxml.html.tostring(html))
-            else:
+                    if doc.startswith(XML_DECLARATION):
+                        warnings.warn(
+                            f"XML declarations in HTML module descriptions are "
+                            f"deprecated since Odoo 17, {module.name} can just "
+                            f"have a UTF8 description with not need for a "
+                            f"declaration.",
+                            category=DeprecationWarning,
+                        )
+                    else:
+                        try:
+                            doc = doc.decode()
+                        except UnicodeDecodeError:
+                            warnings.warn(
+                                f"Non-UTF8 module descriptions are deprecated "
+                                f"since Odoo 17 ({module.name}'s description "
+                                f"is not utf-8)",
+                                category=DeprecationWarning,
+                            )
+                    module.description_html = _apply_description_images(doc)
+            except FileNotFoundError:
                 overrides = {
                     'embed_stylesheet': False,
                     'doctitle_xform': False,
@@ -204,7 +209,7 @@ class Module(models.Model):
                     'file_insertion_enabled': False,
                 }
                 output = publish_string(source=module.description if not module.application and module.description else '', settings_overrides=overrides, writer=MyWriter())
-                module.description_html = tools.html_sanitize(output)
+                module.description_html = _apply_description_images(output)
 
     @api.depends('name')
     def _get_latest_version(self):
@@ -246,18 +251,23 @@ class Module(models.Model):
 
     @api.depends('icon')
     def _get_icon_image(self):
+        self.icon_image = ''
         for module in self:
-            module.icon_image = ''
+            if not module.id:
+                continue
             if module.icon:
-                path_parts = module.icon.split('/')
-                path = modules.get_module_resource(path_parts[1], *path_parts[2:])
-            elif module.id:
-                path = modules.module.get_module_icon(module.name)
+                path = os.path.join(module.icon.lstrip("/"))
             else:
-                path = ''
+                path = modules.module.get_module_icon_path(module)
             if path:
-                with tools.file_open(path, 'rb') as image_file:
-                    module.icon_image = base64.b64encode(image_file.read())
+                try:
+                    with tools.file_open(path, 'rb', filter_ext=('.png', '.svg', '.gif', '.jpeg', '.jpg')) as image_file:
+                        module.icon_image = base64.b64encode(image_file.read())
+                except FileNotFoundError:
+                    module.icon_image = ''
+            countries = self.get_module_info(module.name).get('countries', [])
+            country_code = len(countries) == 1 and countries[0]
+            module.icon_flag = get_flag(country_code.upper()) if country_code else ''
 
     name = fields.Char('Technical Name', readonly=True, required=True)
     category_id = fields.Many2one('ir.module.category', string='Category', readonly=True, index=True)
@@ -282,6 +292,7 @@ class Module(models.Model):
     sequence = fields.Integer('Sequence', default=100)
     dependencies_id = fields.One2many('ir.module.module.dependency', 'module_id',
                                        string='Dependencies', readonly=True)
+    country_ids = fields.Many2many('res.country', 'module_country', 'module_id', 'country_id')
     exclusion_ids = fields.One2many('ir.module.module.exclusion', 'module_id',
                                     string='Exclusions', readonly=True)
     auto_install = fields.Boolean('Automatic Installation',
@@ -308,6 +319,7 @@ class Module(models.Model):
     application = fields.Boolean('Application', readonly=True)
     icon = fields.Char('Icon URL')
     icon_image = fields.Binary(string='Icon', compute='_get_icon_image')
+    icon_flag = fields.Char(string='Flag', compute='_get_icon_image')
     to_buy = fields.Boolean('Odoo Enterprise Module', default=False)
     has_iap = fields.Boolean(compute='_compute_has_iap')
 
@@ -326,63 +338,29 @@ class Module(models.Model):
                 raise UserError(_('You are trying to remove a module that is installed or will be installed.'))
 
     def unlink(self):
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return super(Module, self).unlink()
 
     def _get_modules_to_load_domain(self):
         """ Domain to retrieve the modules that should be loaded by the registry. """
         return [('state', '=', 'installed')]
 
-    @staticmethod
-    def _check_python_external_dependency(pydep):
+    def check_external_dependencies(self, module_name, newstate='to install'):
+        terp = self.get_module_info(module_name)
         try:
-            pkg_resources.get_distribution(pydep)
-        except pkg_resources.DistributionNotFound as e:
-            try:
-                importlib.import_module(pydep)
-                _logger.info("python external dependency on '%s' does not appear to be a valid PyPI package. Using a PyPI package name is recommended.", pydep)
-            except ImportError:
-                # backward compatibility attempt failed
-                _logger.warning("DistributionNotFound: %s", e)
-                raise Exception('Python library not installed: %s' % (pydep,))
-        except pkg_resources.VersionConflict as e:
-            _logger.warning("VersionConflict: %s", e)
-            raise Exception('Python library version conflict: %s' % (pydep,))
-        except Exception as e:
-            _logger.warning("get_distribution(%s) failed: %s", pydep, e)
-            raise Exception('Error finding python library %s' % (pydep,))
-
-    @staticmethod
-    def _check_external_dependencies(terp):
-        depends = terp.get('external_dependencies')
-        if not depends:
-            return
-        for pydep in depends.get('python', []):
-            Module._check_python_external_dependency(pydep)
-
-        for binary in depends.get('bin', []):
-            try:
-                tools.find_in_path(binary)
-            except IOError:
-                raise Exception('Unable to find %r in path' % (binary,))
-
-    @classmethod
-    def check_external_dependencies(cls, module_name, newstate='to install'):
-        terp = cls.get_module_info(module_name)
-        try:
-            cls._check_external_dependencies(terp)
+            modules.check_manifest_dependencies(terp)
         except Exception as e:
             if newstate == 'to install':
-                msg = _('Unable to install module "%s" because an external dependency is not met: %s')
+                msg = _('Unable to install module "%(module)s" because an external dependency is not met: %(dependency)s', module=module_name, dependency=e.args[0])
             elif newstate == 'to upgrade':
-                msg = _('Unable to upgrade module "%s" because an external dependency is not met: %s')
+                msg = _('Unable to upgrade module "%(module)s" because an external dependency is not met: %(dependency)s', module=module_name, dependency=e.args[0])
             else:
-                msg = _('Unable to process module "%s" because an external dependency is not met: %s')
-            raise UserError(msg % (module_name, e.args[0]))
+                msg = _('Unable to process module "%(module)s" because an external dependency is not met: %(dependency)s', module=module_name, dependency=e.args[0])
+            raise UserError(msg)
 
     def _state_update(self, newstate, states_to_update, level=100):
         if level < 1:
-            raise UserError(_('Recursion error in modules dependencies !'))
+            raise UserError(_('Recursion error in modules dependencies!'))
 
         # whether some modules are installed with demo data
         demo = False
@@ -396,7 +374,10 @@ class Module(models.Model):
             update_mods, ready_mods = self.browse(), self.browse()
             for dep in module.dependencies_id:
                 if dep.state == 'unknown':
-                    raise UserError(_("You try to install module '%s' that depends on module '%s'.\nBut the latter module is not available in your system.") % (module.name, dep.name,))
+                    raise UserError(_(
+                        'You try to install module "%(module)s" that depends on module "%(dependency)s".\nBut the latter module is not available in your system.',
+                        module=module.name, dependency=dep.name,
+                    ))
                 if dep.depend_id.state == newstate:
                     ready_mods += dep.depend_id
                 else:
@@ -416,16 +397,20 @@ class Module(models.Model):
 
     @assert_log_admin_access
     def button_install(self):
+        company_countries = self.env['res.company'].search([]).country_id
         # domain to select auto-installable (but not yet installed) modules
         auto_domain = [('state', '=', 'uninstalled'), ('auto_install', '=', True)]
 
         # determine whether an auto-install module must be installed:
         #  - all its dependencies are installed or to be installed,
         #  - at least one dependency is 'to install'
+        #  - if the module is country specific, at least one company is in one of the countries
         install_states = frozenset(('installed', 'to install', 'to upgrade'))
         def must_install(module):
             states = {dep.state for dep in module.dependencies_id if dep.auto_install_required}
-            return states <= install_states and 'to install' in states
+            return states <= install_states and 'to install' in states and (
+                not module.country_ids or module.country_ids & company_countries
+            )
 
         modules = self
         while modules:
@@ -443,8 +428,11 @@ class Module(models.Model):
         for module in install_mods:
             for exclusion in module.exclusion_ids:
                 if exclusion.name in install_names:
-                    msg = _('Modules "%s" and "%s" are incompatible.')
-                    raise UserError(msg % (module.shortdesc, exclusion.exclusion_id.shortdesc))
+                    raise UserError(_(
+                        'Modules "%(module)s" and "%(incompatible_module)s" are incompatible.',
+                        module=module.shortdesc,
+                        incompatible_module=exclusion.exclusion_id.shortdesc,
+                    ))
 
         # check category exclusions
         def closure(module):
@@ -462,12 +450,13 @@ class Module(models.Model):
             # the installation is valid if all installed modules in categories
             # belong to the transitive dependencies of one of them
             if modules and not any(modules <= closure(module) for module in modules):
-                msg = _('You are trying to install incompatible modules in category "%s":')
                 labels = dict(self.fields_get(['state'])['state']['selection'])
-                raise UserError("\n".join([msg % category.name] + [
-                    "- %s (%s)" % (module.shortdesc, labels[module.state])
-                    for module in modules
-                ]))
+                raise UserError(
+                    _('You are trying to install incompatible modules in category "%(category)s":%(module_list)s', category=category.name, module_list=''.join(
+                        f"\n- {module.shortdesc} ({labels[module.state]})"
+                        for module in modules
+                    ))
+                )
 
         return dict(ACTION_DICT, name=_('Install'))
 
@@ -488,7 +477,7 @@ class Module(models.Model):
         # configure the CoA on his own company, which makes no sense.
         if request:
             request.allowed_company_ids = self.env.companies.ids
-        return self._button_immediate_function(type(self).button_install)
+        return self._button_immediate_function(self.env.registry[self._name].button_install)
 
     @assert_log_admin_access
     def button_install_cancel(self):
@@ -586,7 +575,7 @@ class Module(models.Model):
         return {
             'type': 'ir.actions.act_url',
             'target': 'self',
-            'url': '/web',
+            'url': '/odoo',
         }
 
     def _button_immediate_function(self, function):
@@ -614,6 +603,10 @@ class Module(models.Model):
         self._cr.commit()
         registry = modules.registry.Registry.new(self._cr.dbname, update_module=True)
         self._cr.commit()
+        if request and request.registry is self.env.registry:
+            request.env.cr.reset()
+            request.registry = request.env.registry
+            assert request.env.registry is registry
         self._cr.reset()
         assert self.env.registry is registry
 
@@ -637,7 +630,7 @@ class Module(models.Model):
         returns the next res.config action to execute
         """
         _logger.info('User #%d triggered module uninstallation', self.env.uid)
-        return self._button_immediate_function(type(self).button_uninstall)
+        return self._button_immediate_function(self.env.registry[self._name].button_uninstall)
 
     @assert_log_admin_access
     def button_uninstall(self):
@@ -675,7 +668,7 @@ class Module(models.Model):
         Upgrade the selected module(s) immediately and fully,
         return the next res.config action to execute
         """
-        return self._button_immediate_function(type(self).button_upgrade)
+        return self._button_immediate_function(self.env.registry[self._name].button_upgrade)
 
     @assert_log_admin_access
     def button_upgrade(self):
@@ -700,7 +693,7 @@ class Module(models.Model):
             module = todo[i]
             i += 1
             if module.state not in ('installed', 'to upgrade'):
-                raise UserError(_("Can not upgrade module '%s'. It is not installed.") % (module.name,))
+                raise UserError(_("Cannot upgrade module “%s”. It is not installed.", module.name))
             if self.get_module_info(module.name).get("installable", True):
                 self.check_external_dependencies(module.name, 'to upgrade')
             for dep in Dependency.search([('name', '=', module.name)]):
@@ -719,7 +712,7 @@ class Module(models.Model):
                 continue
             for dep in module.dependencies_id:
                 if dep.state == 'unknown':
-                    raise UserError(_('You try to upgrade the module %s that depends on the module: %s.\nBut this module is not available in your system.') % (module.name, dep.name,))
+                    raise UserError(_('You try to upgrade the module %(module)s that depends on the module: %(dependency)s.\nBut this module is not available in your system.', module=module.name, dependency=dep.name))
                 if dep.state == 'uninstalled':
                     to_install += self.search([('name', '=', dep.name)]).ids
 
@@ -734,7 +727,7 @@ class Module(models.Model):
     @staticmethod
     def get_values_from_terp(terp):
         return {
-            'description': terp.get('description', ''),
+            'description': dedent(terp.get('description', '')),
             'shortdesc': terp.get('name', ''),
             'author': terp.get('author', 'Unknown'),
             'maintainer': terp.get('maintainer', False),
@@ -799,115 +792,15 @@ class Module(models.Model):
                 mod = self.create(dict(name=mod_name, state=state, **values))
                 res[1] += 1
 
-            mod._update_dependencies(terp.get('depends', []), terp.get('auto_install'))
-            mod._update_exclusions(terp.get('excludes', []))
-            mod._update_category(terp.get('category', 'Uncategorized'))
+            mod._update_from_terp(terp)
 
         return res
 
-    @assert_log_admin_access
-    def download(self, download=True):
-        return []
-
-    @assert_log_admin_access
-    @api.model
-    def install_from_urls(self, urls):
-        if not self.env.user.has_group('base.group_system'):
-            raise AccessDenied()
-
-        # One-click install is opt-in - cfr Issue #15225
-        ad_dir = tools.config.addons_data_dir
-        if not os.access(ad_dir, os.W_OK):
-            msg = (_("Automatic install of downloaded Apps is currently disabled.") + "\n\n" +
-                   _("To enable it, make sure this directory exists and is writable on the server:") +
-                   "\n%s" % ad_dir)
-            _logger.warning(msg)
-            raise UserError(msg)
-
-        apps_server = werkzeug.urls.url_parse(self.get_apps_server())
-
-        OPENERP = odoo.release.product_name.lower()
-        tmp = tempfile.mkdtemp()
-        _logger.debug('Install from url: %r', urls)
-        try:
-            # 1. Download & unzip missing modules
-            for module_name, url in urls.items():
-                if not url:
-                    continue    # nothing to download, local version is already the last one
-
-                up = werkzeug.urls.url_parse(url)
-                if up.scheme != apps_server.scheme or up.netloc != apps_server.netloc:
-                    raise AccessDenied()
-
-                try:
-                    _logger.info('Downloading module `%s` from OpenERP Apps', module_name)
-                    response = requests.get(url)
-                    response.raise_for_status()
-                    content = response.content
-                except Exception:
-                    _logger.exception('Failed to fetch module %s', module_name)
-                    raise UserError(_('The `%s` module appears to be unavailable at the moment, please try again later.', module_name))
-                else:
-                    zipfile.ZipFile(io.BytesIO(content)).extractall(tmp)
-                    assert os.path.isdir(os.path.join(tmp, module_name))
-
-            # 2a. Copy/Replace module source in addons path
-            for module_name, url in urls.items():
-                if module_name == OPENERP or not url:
-                    continue    # OPENERP is special case, handled below, and no URL means local module
-                module_path = modules.get_module_path(module_name, downloaded=True, display_warning=False)
-                bck = backup(module_path, False)
-                _logger.info('Copy downloaded module `%s` to `%s`', module_name, module_path)
-                shutil.move(os.path.join(tmp, module_name), module_path)
-                if bck:
-                    shutil.rmtree(bck)
-
-            # 2b.  Copy/Replace server+base module source if downloaded
-            if urls.get(OPENERP):
-                # special case. it contains the server and the base module.
-                # extract path is not the same
-                base_path = os.path.dirname(modules.get_module_path('base'))
-
-                # copy all modules in the SERVER/odoo/addons directory to the new "odoo" module (except base itself)
-                for d in os.listdir(base_path):
-                    if d != 'base' and os.path.isdir(os.path.join(base_path, d)):
-                        destdir = os.path.join(tmp, OPENERP, 'addons', d)    # XXX 'odoo' subdirectory ?
-                        shutil.copytree(os.path.join(base_path, d), destdir)
-
-                # then replace the server by the new "base" module
-                server_dir = tools.config['root_path']      # XXX or dirname()
-                bck = backup(server_dir)
-                _logger.info('Copy downloaded module `odoo` to `%s`', server_dir)
-                shutil.move(os.path.join(tmp, OPENERP), server_dir)
-                #if bck:
-                #    shutil.rmtree(bck)
-
-            self.update_list()
-
-            with_urls = [module_name for module_name, url in urls.items() if url]
-            downloaded = self.search([('name', 'in', with_urls)])
-            installed = self.search([('id', 'in', downloaded.ids), ('state', '=', 'installed')])
-
-            to_install = self.search([('name', 'in', list(urls)), ('state', '=', 'uninstalled')])
-            post_install_action = to_install.button_immediate_install()
-
-            if installed or to_install:
-                # in this case, force server restart to reload python code...
-                self._cr.commit()
-                odoo.service.server.restart()
-                return {
-                    'type': 'ir.actions.client',
-                    'tag': 'home',
-                    'params': {'wait': True},
-                }
-            return post_install_action
-
-        finally:
-            shutil.rmtree(tmp)
-
-    @api.model
-    def get_apps_server(self):
-        return tools.config.get('apps_server', 'https://apps.odoo.com/apps')
+    def _update_from_terp(self, terp):
+        self._update_dependencies(terp.get('depends', []), terp.get('auto_install'))
+        self._update_countries(terp.get('countries', []))
+        self._update_exclusions(terp.get('excludes', []))
+        self._update_category(terp.get('category', 'Uncategorized'))
 
     def _update_dependencies(self, depends=None, auto_install_requirements=()):
         self.env['ir.module.module.dependency'].flush_model()
@@ -922,6 +815,16 @@ class Module(models.Model):
         self.env['ir.module.module.dependency'].invalidate_model(['auto_install_required'])
         self.invalidate_recordset(['dependencies_id'])
 
+    def _update_countries(self, countries=()):
+        existing = set(self.country_ids.ids)
+        needed = set(self.env['res.country'].search([('code', 'in', [c.upper() for c in countries])]).ids)
+        for dep in (needed - existing):
+            self._cr.execute('INSERT INTO module_country (module_id, country_id) values (%s, %s)', (self.id, dep))
+        for dep in (existing - needed):
+            self._cr.execute('DELETE FROM module_country WHERE module_id = %s and country_id = %s', (self.id, dep))
+        self.invalidate_recordset(['country_ids'])
+        self.env['res.company'].invalidate_model(['uninstalled_l10n_module_ids'])
+
     def _update_exclusions(self, excludes=None):
         self.env['ir.module.module.exclusion'].flush_model()
         existing = set(excl.name for excl in self.exclusion_ids)
@@ -934,9 +837,14 @@ class Module(models.Model):
 
     def _update_category(self, category='Uncategorized'):
         current_category = self.category_id
+        seen = set()
         current_category_path = []
         while current_category:
             current_category_path.insert(0, current_category.name)
+            seen.add(current_category.id)
+            if current_category.parent_id.id in seen:
+                current_category.parent_id = False
+                _logger.warning('category %r ancestry loop has been detected and fixed', current_category)
             current_category = current_category.parent_id
 
         categs = category.split('/')
@@ -962,7 +870,7 @@ class Module(models.Model):
     def _check(self):
         for module in self:
             if not module.description_html:
-                _logger.warning('module %s: description is empty !', module.name)
+                _logger.warning('module %s: description is empty!', module.name)
 
     def _get(self, name):
         """ Return the (sudoed) `ir.module.module` record with the given name.
@@ -990,13 +898,18 @@ class Module(models.Model):
     def search_panel_select_range(self, field_name, **kwargs):
         if field_name == 'category_id':
             enable_counters = kwargs.get('enable_counters', False)
-            domain = [('parent_id', '=', False), ('child_ids.module_ids', '!=', False)]
+            domain = [
+                ('parent_id', '=', False),
+                '|',
+                ('module_ids.application', '!=', False),
+                ('child_ids.module_ids', '!=', False),
+            ]
 
             excluded_xmlids = [
                 'base.module_category_website_theme',
                 'base.module_category_theme',
             ]
-            if not self.user_has_groups('base.group_no_one'):
+            if not self.env.user.has_group('base.group_no_one'):
                 excluded_xmlids.append('base.module_category_hidden')
 
             excluded_category_ids = []
@@ -1035,46 +948,24 @@ class Module(models.Model):
         return super(Module, self).search_panel_select_range(field_name, **kwargs)
 
     @api.model
-    def _load_module_terms(self, modules, langs, overwrite=False):
+    def _load_module_terms(self, modules, langs, overwrite=False, imported_module=False):
         """ Load PO files of the given modules for the given languages. """
         # load i18n files
         translation_importer = TranslationImporter(self.env.cr, verbose=False)
 
         for module_name in modules:
-            modpath = get_module_path(module_name)
+            modpath = get_module_path(module_name, downloaded=imported_module)
             if not modpath:
                 continue
             for lang in langs:
-                lang_code = tools.get_iso_codes(lang)
-                base_lang_code = None
-                if '_' in lang_code:
-                    base_lang_code = lang_code.split('_')[0]
-
-                # Step 1: for sub-languages, load base language first (e.g. es_CL.po is loaded over es.po)
-                if base_lang_code:
-                    base_trans_file = get_module_resource(module_name, 'i18n', base_lang_code + '.po')
-                    if base_trans_file:
-                        _logger.info('module %s: loading base translation file %s for language %s', module_name, base_lang_code, lang)
-                        translation_importer.load_file(base_trans_file, lang)
-
-                    # i18n_extra folder is for additional translations handle manually (eg: for l10n_be)
-                    base_trans_extra_file = get_module_resource(module_name, 'i18n_extra', base_lang_code + '.po')
-                    if base_trans_extra_file:
-                        _logger.info('module %s: loading extra base translation file %s for language %s', module_name, base_lang_code, lang)
-                        translation_importer.load_file(base_trans_extra_file, lang)
-
-                # Step 2: then load the main translation file, possibly overriding the terms coming from the base language
-                trans_file = get_module_resource(module_name, 'i18n', lang_code + '.po')
-                if trans_file:
-                    _logger.info('module %s: loading translation file %s for language %s', module_name, lang_code, lang)
-                    translation_importer.load_file(trans_file, lang)
-                elif lang_code != 'en_US':
-                    _logger.info('module %s: no translation for language %s', module_name, lang_code)
-
-                trans_extra_file = get_module_resource(module_name, 'i18n_extra', lang_code + '.po')
-                if trans_extra_file:
-                    _logger.info('module %s: loading extra translation file %s for language %s', module_name, lang_code, lang)
-                    translation_importer.load_file(trans_extra_file, lang)
+                is_lang_imported = False
+                env = self.env if imported_module else None
+                for po_path in get_po_paths(module_name, lang, env=env):
+                    _logger.info('module %s: loading translation file %s for language %s', module_name, po_path, lang)
+                    translation_importer.load_file(po_path, lang)
+                    is_lang_imported = True
+                if lang != 'en_US' and not is_lang_imported:
+                    _logger.info('module %s: no translation for language %s', module_name, lang)
 
         translation_importer.save(overwrite=overwrite)
 
@@ -1085,6 +976,7 @@ class ModuleDependency(models.Model):
     _name = "ir.module.module.dependency"
     _description = "Module dependency"
     _log_access = False  # inserts are done manually, create and write uid, dates are always null
+    _allow_sudo_commands = False
 
     # the dependency name
     name = fields.Char(index=True)
@@ -1123,10 +1015,32 @@ class ModuleDependency(models.Model):
         for dependency in self:
             dependency.state = dependency.depend_id.state or 'unknown'
 
+    @api.model
+    def all_dependencies(self, module_names):
+        to_search = {key: True for key in module_names}
+        res = {}
+        def search_direct_deps(to_search, res):
+            to_search_list = list(to_search.keys())
+            dependencies = self.web_search_read(domain=[("module_id.name", "in", to_search_list)], specification={"module_id":{"fields":{"name":{}}}, "name": {}, })["records"]
+            to_search.clear()
+            for dependency in dependencies:
+                dep_name = dependency["name"]
+                mod_name = dependency["module_id"]["name"]
+                if dep_name not in res and dep_name not in to_search and dep_name not in to_search_list:
+                    to_search[dep_name] = True
+                if mod_name not in res:
+                    res[mod_name] = list()
+                res[mod_name].append(dep_name)
+        search_direct_deps(to_search, res)
+        while to_search:
+            search_direct_deps(to_search, res)
+        return res
+
 
 class ModuleExclusion(models.Model):
     _name = "ir.module.module.exclusion"
     _description = "Module exclusion"
+    _allow_sudo_commands = False
 
     # the exclusion name
     name = fields.Char(index=True)
