@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 import logging
-import warnings
 
-from odoo import api, fields, models, tools, SUPERUSER_ID, _
+from odoo import api, fields, models, tools, _
 from odoo.exceptions import AccessError, ValidationError
 from odoo.osv import expression
-from odoo.tools import config
+from odoo.tools import config, SQL
 from odoo.tools.safe_eval import safe_eval, time
 
 _logger = logging.getLogger(__name__)
@@ -15,21 +14,22 @@ class IrRule(models.Model):
     _description = 'Record Rule'
     _order = 'model_id DESC,id'
     _MODES = ['read', 'write', 'create', 'unlink']
+    _allow_sudo_commands = False
 
     name = fields.Char(index=True)
     active = fields.Boolean(default=True, help="If you uncheck the active field, it will disable the record rule without deleting it (if you delete a native record rule, it may be re-created when you reload the module).")
     model_id = fields.Many2one('ir.model', string='Model', index=True, required=True, ondelete="cascade")
     groups = fields.Many2many('res.groups', 'rule_group_rel', 'rule_group_id', 'group_id', ondelete='restrict')
     domain_force = fields.Text(string='Domain')
-    perm_read = fields.Boolean(string='Apply for Read', default=True)
-    perm_write = fields.Boolean(string='Apply for Write', default=True)
-    perm_create = fields.Boolean(string='Apply for Create', default=True)
-    perm_unlink = fields.Boolean(string='Apply for Delete', default=True)
+    perm_read = fields.Boolean(string='Read', default=True)
+    perm_write = fields.Boolean(string='Write', default=True)
+    perm_create = fields.Boolean(string='Create', default=True)
+    perm_unlink = fields.Boolean(string='Delete', default=True)
 
     _sql_constraints = [
         ('no_access_rights',
          'CHECK (perm_read!=False or perm_write!=False or perm_create!=False or perm_unlink!=False)',
-         'Rule must have at least one checked access right !'),
+         'Rule must have at least one checked access right!'),
     ]
 
     @api.model
@@ -60,6 +60,17 @@ class IrRule(models.Model):
         if any(rule.model_id.model == self._name for rule in self):
             raise ValidationError(_('Rules can not be applied on the Record Rules model.'))
 
+    @api.constrains('active', 'domain_force', 'model_id')
+    def _check_domain(self):
+        eval_context = self._eval_context()
+        for rule in self:
+            if rule.active and rule.domain_force:
+                try:
+                    domain = safe_eval(rule.domain_force, eval_context)
+                    expression.expression(domain, self.env[rule.model_id.model].sudo())
+                except Exception as e:
+                    raise ValidationError(_('Invalid domain: %s', e))
+
     def _compute_domain_keys(self):
         """ Return the list of context keys to use for caching ``_compute_domain``. """
         return ['allowed_company_ids']
@@ -108,16 +119,17 @@ class IrRule(models.Model):
         if self.env.su:
             return self.browse(())
 
-        query = """ SELECT r.id FROM ir_rule r JOIN ir_model m ON (r.model_id=m.id)
-                    WHERE m.model=%s AND r.active AND r.perm_{mode}
-                    AND (r.id IN (SELECT rule_group_id FROM rule_group_rel rg
-                                  JOIN res_groups_users_rel gu ON (rg.group_id=gu.gid)
-                                  WHERE gu.uid=%s)
-                         OR r.global)
-                    ORDER BY r.id
-                """.format(mode=mode)
-        self._cr.execute(query, (model_name, self._uid))
-        return self.browse(row[0] for row in self._cr.fetchall())
+        sql = SQL("""
+            SELECT r.id FROM ir_rule r
+            JOIN ir_model m ON (r.model_id=m.id)
+            WHERE m.model = %s AND r.active AND r.perm_%s
+                AND (r.global OR r.id IN (
+                    SELECT rule_group_id FROM rule_group_rel rg
+                    WHERE rg.group_id IN %s
+                ))
+            ORDER BY r.id
+        """, model_name, SQL(mode), tuple(self.env.user._get_group_ids()) or (None,))
+        return self.browse(v for v, in self.env.execute_query(sql))
 
     @api.model
     @tools.conditional(
@@ -126,14 +138,20 @@ class IrRule(models.Model):
                        'tuple(self._compute_domain_context_values())'),
     )
     def _compute_domain(self, model_name, mode="read"):
+        global_domains = []                     # list of domains
+
+        # add rules for parent models
+        for parent_model_name, parent_field_name in self.env[model_name]._inherits.items():
+            if domain := self._compute_domain(parent_model_name, mode):
+                global_domains.append([(parent_field_name, 'any', domain)])
+
         rules = self._get_rules(model_name, mode=mode)
         if not rules:
-            return
+            return expression.AND(global_domains) if global_domains else []
 
-        # browse user and rules as SUPERUSER_ID to avoid access errors!
+        # browse user and rules with sudo to avoid access errors!
         eval_context = self._eval_context()
         user_groups = self.env.user.groups_id
-        global_domains = []                     # list of domains
         group_domains = []                      # list of domains
         for rule in rules.sudo():
             # evaluate the domain for the current user
@@ -159,14 +177,9 @@ class IrRule(models.Model):
                 v = tuple(v)
             yield v
 
-    @api.model
-    def clear_cache(self):
-        warnings.warn("Deprecated IrRule.clear_cache(), use IrRule.clear_caches() instead", DeprecationWarning)
-        self.clear_caches()
-
     def unlink(self):
         res = super(IrRule, self).unlink()
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return res
 
     @api.model_create_multi
@@ -174,7 +187,7 @@ class IrRule(models.Model):
         res = super(IrRule, self).create(vals_list)
         # DLE P33: tests
         self.env.flush_all()
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return res
 
     def write(self, vals):
@@ -184,28 +197,28 @@ class IrRule(models.Model):
         # - odoo/addons/test_access_rights/tests/test_ir_rules.py
         # - odoo/addons/base/tests/test_orm.py (/home/dle/src/odoo/master-nochange-fp/odoo/addons/base/tests/test_orm.py)
         self.env.flush_all()
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return res
 
     def _make_access_error(self, operation, records):
         _logger.info('Access Denied by record rules for operation: %s on record ids: %r, uid: %s, model: %s', operation, records.ids[:6], self._uid, records._name)
+        self = self.with_context(self.env.user.context_get())
 
         model = records._name
         description = self.env['ir.model']._get(model).name or model
-        msg_heads = {
-            # Messages are declared in extenso so they are properly exported in translation terms
-            'read':   _("Due to security restrictions, you are not allowed to access '%(document_kind)s' (%(document_model)s) records.", document_kind=description, document_model=model),
-            'write':  _("Due to security restrictions, you are not allowed to modify '%(document_kind)s' (%(document_model)s) records.", document_kind=description, document_model=model),
-            'create': _("Due to security restrictions, you are not allowed to create '%(document_kind)s' (%(document_model)s) records.", document_kind=description, document_model=model),
-            'unlink': _("Due to security restrictions, you are not allowed to delete '%(document_kind)s' (%(document_model)s) records.", document_kind=description, document_model=model)
+        operations = {
+            'read':  _("read"),
+            'write': _("write"),
+            'create': _("create"),
+            'unlink': _("unlink"),
         }
-        operation_error = msg_heads[operation]
-        resolution_info = _("Contact your administrator to request access if necessary.")
+        user_description = f"{self.env.user.name} (id={self.env.user.id})"
+        operation_error = _("Uh-oh! Looks like you have stumbled upon some top-secret records.\n\n" \
+            "Sorry, %(user)s doesn't have '%(operation)s' access to:", user=user_description, operation=operations[operation])
+        failing_model = _("- %(description)s (%(model)s)", description=description, model=model)
 
-        if not self.env.user.has_group('base.group_no_one') or not self.env.user.has_group('base.group_user'):
-            return AccessError(f"{operation_error}\n\n{resolution_info}")
+        resolution_info = _("If you really, really need access, perhaps you can win over your friendly administrator with a batch of freshly baked cookies.")
 
-        # This extended AccessError is only displayed in debug mode.
         # Note that by default, public and portal users do not have
         # the group "base.group_no_one", even if debug mode is enabled,
         # so it is relatively safe here to include the list of rules and record names.
@@ -218,25 +231,36 @@ class IrRule(models.Model):
             # If the user has access to the company of the record, add this
             # information in the description to help them to change company
             if company_related and 'company_id' in rec and rec.company_id in self.env.user.company_ids:
-                return f'{rec.display_name} (id={rec.id}, company={rec.company_id.display_name})'
-            return f'{rec.display_name} (id={rec.id})'
+                return f'{description}, {rec.display_name} ({model}: {rec.id}, company={rec.company_id.display_name})'
+            return f'{description}, {rec.display_name} ({model}: {rec.id})'
 
-        records_description = ', '.join(get_record_description(rec) for rec in records_sudo)
-        failing_records = _("Records: %s", records_description)
-
-        user_description = f'{self.env.user.name} (id={self.env.user.id})'
-        failing_user = _("User: %s", user_description)
-
-        rules_description = '\n'.join(f'- {rule.name}' for rule in rules)
-        failing_rules = _("This restriction is due to the following rules:\n%s", rules_description)
+        context = None
         if company_related:
-            failing_rules += "\n\n" + _('Note: this might be a multi-company issue.')
+            suggested_companies = records_sudo._get_redirect_suggested_company()
+            if suggested_companies and len(suggested_companies) != 1:
+                resolution_info += _('\n\nNote: this might be a multi-company issue. Switching company may help - in Odoo, not in real life!')
+            elif suggested_companies and suggested_companies in self.env.user.company_ids:
+                context = {'suggested_company': {'id': suggested_companies.id, 'display_name': suggested_companies.display_name}}
+                resolution_info += _('\n\nThis seems to be a multi-company issue, you might be able to access the record by switching to the company: %s.', suggested_companies.display_name)
+            elif suggested_companies:
+                resolution_info += _('\n\nThis seems to be a multi-company issue, but you do not have access to the proper company to access the record anyhow.')
+
+        if not self.env.user.has_group('base.group_no_one') or not self.env.user._is_internal():
+            msg = f"{operation_error}\n{failing_model}\n\n{resolution_info}"
+        else:
+            # This extended AccessError is only displayed in debug mode.
+            failing_records = '\n'.join(f'- {get_record_description(rec)}' for rec in records_sudo)
+            rules_description = '\n'.join(f'- {rule.name}' for rule in rules)
+            failing_rules = _("Blame the following rules:\n%s", rules_description)
+            msg = f"{operation_error}\n{failing_records}\n\n{failing_rules}\n\n{resolution_info}"
 
         # clean up the cache of records prefetched with display_name above
         records_sudo.invalidate_recordset()
 
-        msg = f"{operation_error}\n\n{failing_records}\n{failing_user}\n\n{failing_rules}\n\n{resolution_info}"
-        return AccessError(msg)
+        exception = AccessError(msg)
+        if context:
+            exception.context = context
+        return exception
 
 
 #
