@@ -3,13 +3,13 @@
 
 import base64
 from collections import defaultdict
+from os.path import join as opj
 import operator
 import re
 
 from odoo import api, fields, models, tools, _
 from odoo.exceptions import ValidationError
 from odoo.http import request
-from odoo.modules import get_module_resource
 from odoo.osv import expression
 
 MENU_ITEM_SEPARATOR = "/"
@@ -21,13 +21,14 @@ class IrUiMenu(models.Model):
     _description = 'Menu'
     _order = "sequence,id"
     _parent_store = True
+    _allow_sudo_commands = False
 
     name = fields.Char(string='Menu', required=True, translate=True)
     active = fields.Boolean(default=True)
     sequence = fields.Integer(default=10)
     child_id = fields.One2many('ir.ui.menu', 'parent_id', string='Child IDs')
     parent_id = fields.Many2one('ir.ui.menu', string='Parent Menu', index=True, ondelete="restrict")
-    parent_path = fields.Char(index=True, unaccent=False)
+    parent_path = fields.Char(index=True)
     groups_id = fields.Many2many('res.groups', 'ir_ui_menu_group_rel',
                                  'menu_id', 'gid', string='Groups',
                                  help="If you have groups, the visibility of this menu will be based on these groups. "\
@@ -56,20 +57,20 @@ class IrUiMenu(models.Model):
         else:
             return self.name
 
-    def read_image(self, path):
+    def _read_image(self, path):
         if not path:
             return False
         path_info = path.split(',')
-        icon_path = get_module_resource(path_info[0], path_info[1])
-        icon_image = False
-        if icon_path:
-            with tools.file_open(icon_path, 'rb') as icon_file:
-                icon_image = base64.encodebytes(icon_file.read())
-        return icon_image
+        icon_path = opj(path_info[0], path_info[1])
+        try:
+            with tools.file_open(icon_path, 'rb', filter_ext=('.png',)) as icon_file:
+                return base64.encodebytes(icon_file.read())
+        except FileNotFoundError:
+            return False
 
     @api.constrains('parent_id')
     def _check_parent_id(self):
-        if not self._check_recursion():
+        if self._has_cycle():
             raise ValidationError(_('Error! You cannot create recursive menus.'))
 
     @api.model
@@ -78,14 +79,14 @@ class IrUiMenu(models.Model):
         """ Return the ids of the menu items visible to the user. """
         # retrieve all menus, and determine which ones are visible
         context = {'ir.ui.menu.full_list': True}
-        menus = self.with_context(context).search([]).sudo()
+        menus = self.with_context(context).search_fetch([], ['action', 'parent_id']).sudo()
 
-        groups = self.env.user.groups_id
-        if not debug:
-            groups = groups - self.env.ref('base.group_no_one')
         # first discard all menus with groups the user does not have
+        group_ids = set(self.env.user._get_group_ids())
+        if not debug:
+            group_ids = group_ids - {self.env['ir.model.data']._xmlid_to_res_id('base.group_no_one', raise_if_not_found=False)}
         menus = menus.filtered(
-            lambda menu: not menu.groups_id or menu.groups_id & groups)
+            lambda menu: not (menu.groups_id and group_ids.isdisjoint(menu.groups_id._ids)))
 
         # take apart menus that have an action
         actions_by_model = defaultdict(set)
@@ -138,9 +139,8 @@ class IrUiMenu(models.Model):
         return self.filtered(lambda menu: menu.id in visible_ids)
 
     @api.model
-    def _search(self, args, offset=0, limit=None, order=None, count=False, access_rights_uid=None):
-        menu_ids = super(IrUiMenu, self)._search(args, offset=0, limit=None, order=order, count=False, access_rights_uid=access_rights_uid)
-        menus = self.browse(menu_ids)
+    def search_fetch(self, domain, field_names, offset=0, limit=None, order=None):
+        menus = super().search_fetch(domain, field_names, order=order)
         if menus:
             # menu filtering is done only on main menu tree, not other menu lists
             if not self._context.get('ir.ui.menu.full_list'):
@@ -149,21 +149,28 @@ class IrUiMenu(models.Model):
                 menus = menus[offset:]
             if limit:
                 menus = menus[:limit]
-        return len(menus) if count else menus.ids
+        return menus
 
-    def name_get(self):
-        return [(menu.id, menu._get_full_name()) for menu in self]
+    @api.model
+    def search_count(self, domain, limit=None):
+        # to be consistent with search() above
+        return len(self.search(domain, limit=limit))
+
+    @api.depends('parent_id')
+    def _compute_display_name(self):
+        for menu in self:
+            menu.display_name = menu._get_full_name()
 
     @api.model_create_multi
     def create(self, vals_list):
-        self.clear_caches()
+        self.env.registry.clear_cache()
         for values in vals_list:
             if 'web_icon' in values:
                 values['web_icon_data'] = self._compute_web_icon_data(values.get('web_icon'))
         return super(IrUiMenu, self).create(vals_list)
 
     def write(self, values):
-        self.clear_caches()
+        self.env.registry.clear_cache()
         if 'web_icon' in values:
             values['web_icon_data'] = self._compute_web_icon_data(values.get('web_icon'))
         return super(IrUiMenu, self).write(values)
@@ -173,10 +180,10 @@ class IrUiMenu(models.Model):
             `web_icon` can either be:
               - an image icon [module, path]
               - a built icon [icon_class, icon_color, background_color]
-            and it only has to call `read_image` if it's an image.
+            and it only has to call `_read_image` if it's an image.
         """
         if web_icon and len(web_icon.split(',')) == 2:
-            return self.read_image(web_icon)
+            return self._read_image(web_icon)
 
     def unlink(self):
         # Detach children and promote them to top-level, because it would be unwise to
@@ -188,18 +195,19 @@ class IrUiMenu(models.Model):
         direct_children = self.with_context(**extra).search([('parent_id', 'in', self.ids)])
         direct_children.write({'parent_id': False})
 
-        self.clear_caches()
+        self.env.registry.clear_cache()
         return super(IrUiMenu, self).unlink()
 
     def copy(self, default=None):
-        record = super(IrUiMenu, self).copy(default=default)
-        match = NUMBER_PARENS.search(record.name)
-        if match:
-            next_num = int(match.group(1)) + 1
-            record.name = NUMBER_PARENS.sub('(%d)' % next_num, record.name)
-        else:
-            record.name = record.name + '(1)'
-        return record
+        new_menus = super().copy(default=default)
+        for new_menu in new_menus:
+            match = NUMBER_PARENS.search(new_menu.name)
+            if match:
+                next_num = int(match.group(1)) + 1
+                new_menu.name = NUMBER_PARENS.sub('(%d)' % next_num, new_menu.name)
+            else:
+                new_menu.name = new_menu.name + '(1)'
+        return new_menus
 
     @api.model
     @api.returns('self')
@@ -243,7 +251,7 @@ class IrUiMenu(models.Model):
         :return: the menu root
         :rtype: dict('children': menu_nodes)
         """
-        fields = ['name', 'sequence', 'parent_id', 'action', 'web_icon', 'web_icon_data']
+        fields = ['name', 'sequence', 'parent_id', 'action', 'web_icon']
         menu_roots = self.get_user_roots()
         menu_roots_data = menu_roots.read(fields) if menu_roots else []
         menu_root = {
@@ -273,6 +281,14 @@ class IrUiMenu(models.Model):
         # mapping, resulting in children being correctly set on the roots.
         menu_items.extend(menu_roots_data)
 
+        mi_attachments = self.env['ir.attachment'].sudo().search_read(
+            domain=[('res_model', '=', 'ir.ui.menu'),
+                    ('res_id', 'in', [menu_item['id'] for menu_item in menu_items if menu_item['id']]),
+                    ('res_field', '=', 'web_icon_data')],
+            fields=['res_id', 'datas', 'mimetype'])
+
+        mi_attachment_by_res_id = {attachment['res_id']: attachment for attachment in mi_attachments}
+
         # set children ids and xmlids
         menu_items_map = {menu_item["id"]: menu_item for menu_item in menu_items}
         for menu_item in menu_items:
@@ -282,6 +298,13 @@ class IrUiMenu(models.Model):
             if parent in menu_items_map:
                 menu_items_map[parent].setdefault(
                     'children', []).append(menu_item['id'])
+            attachment = mi_attachment_by_res_id.get(menu_item['id'])
+            if attachment:
+                menu_item['web_icon_data'] = attachment['datas'].decode()
+                menu_item['web_icon_data_mimetype'] = attachment['mimetype']
+            else:
+                menu_item['web_icon_data'] = False
+                menu_item['web_icon_data_mimetype'] = False
         all_menus.update(menu_items_map)
 
         # sort by sequence
