@@ -33,11 +33,13 @@ class Country(models.Model):
     _name = 'res.country'
     _description = 'Country'
     _order = 'name'
+    _rec_names_search = ['name', 'code']
 
     name = fields.Char(
         string='Country Name', required=True, translate=True)
     code = fields.Char(
         string='Country Code', size=2,
+        required=True,
         help='The ISO country code in two chars. \nYou can use this field for quick search.')
     address_format = fields.Text(string="Layout in Reports",
         help="Display format to use for addresses belonging to this country.\n\n"
@@ -76,25 +78,27 @@ class Country(models.Model):
 
     _sql_constraints = [
         ('name_uniq', 'unique (name)',
-            'The name of the country must be unique !'),
+            'The name of the country must be unique!'),
         ('code_uniq', 'unique (code)',
-            'The code of the country must be unique !')
+            'The code of the country must be unique!')
     ]
 
-    def _name_search(self, name='', args=None, operator='ilike', limit=100, name_get_uid=None):
-        if args is None:
-            args = []
-
-        ids = []
-        if len(name) == 2:
-            ids = list(self._search([('code', 'ilike', name)] + args, limit=limit))
-
-        search_domain = [('name', operator, name)]
-        if ids:
-            search_domain.append(('id', 'not in', ids))
-        ids += list(self._search(search_domain + args, limit=limit))
-
-        return ids
+    @api.model
+    def name_search(self, name='', args=None, operator='ilike', limit=100):
+        result = []
+        domain = args or []
+        # first search by code
+        if operator not in expression.NEGATIVE_TERM_OPERATORS and name and len(name) == 2:
+            countries = self.search_fetch(expression.AND([domain, [('code', operator, name)]]), ['display_name'], limit=limit)
+            result.extend((country.id, country.display_name) for country in countries.sudo())
+            domain = expression.AND([domain, [('id', 'not in', countries.ids)]])
+            if limit is not None:
+                limit -= len(countries)
+                if limit <= 0:
+                    return result
+        # normal search
+        result.extend(super().name_search(name, domain, operator, limit))
+        return result
 
     @api.model
     @tools.ormcache('code')
@@ -114,11 +118,11 @@ class Country(models.Model):
         res = super().write(vals)
         if ('code' in vals or 'phone_code' in vals):
             # Intentionally simplified by not clearing the cache in create and unlink.
-            self.clear_caches()
+            self.env.registry.clear_cache()
         if 'address_view_id' in vals:
             # Changing the address view of the company must invalidate the view cached for res.partner
             # because of _view_get_address
-            self.env['res.partner'].clear_caches()
+            self.env.registry.clear_cache('templates')
         return res
 
     def get_address_fields(self):
@@ -157,6 +161,7 @@ class CountryState(models.Model):
     _description = "Country state"
     _name = 'res.country.state'
     _order = 'code'
+    _rec_names_search = ['name', 'code']
 
     country_id = fields.Many2one('res.country', string='Country', required=True)
     name = fields.Char(string='State Name', required=True,
@@ -164,32 +169,54 @@ class CountryState(models.Model):
     code = fields.Char(string='State Code', help='The state code.', required=True)
 
     _sql_constraints = [
-        ('name_code_uniq', 'unique(country_id, code)', 'The code of the state must be unique by country !')
+        ('name_code_uniq', 'unique(country_id, code)', 'The code of the state must be unique by country!')
     ]
 
     @api.model
-    def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
-        args = args or []
-        if self.env.context.get('country_id'):
-            args = expression.AND([args, [('country_id', '=', self.env.context.get('country_id'))]])
-
-        if operator == 'ilike' and not (name or '').strip():
-            first_domain = []
-            domain = []
-        else:
-            first_domain = [('code', '=ilike', name)]
-            domain = [('name', operator, name)]
-
-        first_state_ids = self._search(expression.AND([first_domain, args]), limit=limit, access_rights_uid=name_get_uid) if first_domain else []
-        return list(first_state_ids) + [
-            state_id
-            for state_id in self._search(expression.AND([domain, args]),
-                                         limit=limit, access_rights_uid=name_get_uid)
-            if state_id not in first_state_ids
-        ]
-
-    def name_get(self):
+    def name_search(self, name='', args=None, operator='ilike', limit=100):
         result = []
-        for record in self:
-            result.append((record.id, "{} ({})".format(record.name, record.country_id.code)))
+        domain = args or []
+        # first search by code (with =ilike)
+        if operator not in expression.NEGATIVE_TERM_OPERATORS and name:
+            states = self.search_fetch(expression.AND([domain, [('code', '=like', name)]]), ['display_name'], limit=limit)
+            result.extend((state.id, state.display_name) for state in states.sudo())
+            domain = expression.AND([domain, [('id', 'not in', states.ids)]])
+            if limit is not None:
+                limit -= len(states)
+                if limit <= 0:
+                    return result
+        # normal search
+        result.extend(super().name_search(name, domain, operator, limit))
         return result
+
+    @api.model
+    def _search_display_name(self, operator, value):
+        domain = super()._search_display_name(operator, value)
+        if value and operator not in expression.NEGATIVE_TERM_OPERATORS:
+            if operator in ('ilike', '='):
+                domain = expression.OR([
+                    domain, self._get_name_search_domain(value, operator),
+                ])
+            elif operator == 'in':
+                domain = expression.OR([
+                    domain,
+                    *(self._get_name_search_domain(name, '=') for name in value),
+                ])
+        if country_id := self.env.context.get('country_id'):
+            domain = expression.AND([domain, [('country_id', '=', country_id)]])
+        return domain
+
+    def _get_name_search_domain(self, name, operator):
+        m = re.fullmatch(r"(?P<name>.+)\((?P<country>.+)\)", name)
+        if m:
+            return [
+                ('name', operator, m['name'].strip()),
+                '|', ('country_id.name', 'ilike', m['country'].strip()),
+                ('country_id.code', '=', m['country'].strip()),
+            ]
+        return [expression.FALSE_LEAF]
+
+    @api.depends('country_id')
+    def _compute_display_name(self):
+        for record in self:
+            record.display_name = f"{record.name} ({record.country_id.code})"
