@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from contextlib import contextmanager
-from unittest.mock import patch
+import threading
 
-from odoo.tests.common import TransactionCase, HttpCase
-from odoo import Command
+from contextlib import contextmanager
+from unittest.mock import patch, Mock
+
+from odoo.tests.common import new_test_user, TransactionCase, HttpCase
+from odoo import Command, modules
 
 DISABLED_MAIL_CONTEXT = {
     'tracking_disable': True,
@@ -22,28 +24,75 @@ class BaseCommon(TransactionCase):
     def setUpClass(cls):
         super().setUpClass()
 
-        # Enforce the use of USD as main currency unless modified in inherited class(es)
-        cls._use_currency('USD')
-
         # Mail logic won't be tested by default in other modules.
         # Mail API overrides should be tested with dedicated tests on purpose
         # Hack to use with_context and avoid manual context dict modification
-        cls.env = cls.env['base'].with_context(**DISABLED_MAIL_CONTEXT).env
+        cls.env = cls.env['base'].with_context(**cls.default_env_context()).env
+
+        independent_user = cls.setup_independent_user()
+        if independent_user:
+            cls.env = cls.env(user=independent_user)
+            cls.user = cls.env.user
+
+        independent_company = cls.setup_independent_company()
+        if independent_company:
+            # avoid using the context to assign companies
+            cls.env.user.company_id = independent_company
+            cls.env.user.company_ids = [Command.set(independent_company.ids)]
+        else:
+            cls.setup_main_company()
+
+        # Make sure all class variables have the same env.
+        # Do not specify any class variables before the env changes.
+        cls.company = cls.env.company
+        cls.currency = cls.env.company.currency_id
 
         cls.partner = cls.env['res.partner'].create({
             'name': 'Test Partner',
         })
 
+        cls.group_portal = cls.env.ref('base.group_portal')
+        cls.group_user = cls.env.ref('base.group_user')
+        cls.group_system = cls.env.ref('base.group_system')
+
     @classmethod
-    def _use_currency(cls, currency_code):
-        # Enforce constant currency
-        currency = cls._enable_currency(currency_code)
-        if not cls.env.company.currency_id == currency:
-            cls.env.transaction.cache.set(cls.env.company, type(cls.env.company).currency_id, currency.id, dirty=True)
-            # this is equivalent to cls.env.company.currency_id = currency but without triggering buisness code checks.
-            # The value is added in cache, and the cache value is set as dirty so that that
-            # the value will be written to the database on next flush.
-            # this was needed because some journal entries may exist when running tests, especially l10n demo data.
+    def default_env_context(cls):
+        """ To Override to reactivate the tracking """
+        return {**DISABLED_MAIL_CONTEXT}
+
+    @classmethod
+    def setup_other_currency(cls, code, **kwargs):
+        rates = kwargs.pop('rates', [])
+        currency = cls.env['res.currency'].with_context(active_test=False).search([('name', '=', code)], limit=1)
+        currency.rate_ids.unlink()
+        currency.write({
+            'active': True,
+            'rate_ids': [Command.create(
+                {
+                    'name': rate_date,
+                    'rate': rate,
+                    'company_id': cls.env.company.id,
+                }
+            ) for rate_date, rate in rates],
+            **kwargs,
+        })
+        return currency
+
+    @classmethod
+    def setup_independent_company(cls, **kwargs):
+        return None
+
+    @classmethod
+    def setup_independent_user(cls):
+        return None
+
+    @classmethod
+    def get_default_groups(cls):
+        return cls.env['res.users']._default_groups()
+
+    @classmethod
+    def setup_main_company(cls, currency_code='USD'):
+        cls._use_currency(cls.env.company, currency_code)
 
     @classmethod
     def _enable_currency(cls, currency_code):
@@ -53,6 +102,50 @@ class BaseCommon(TransactionCase):
         currency.action_unarchive()
         return currency
 
+    @classmethod
+    def _use_currency(cls, company, currency_code):
+        # Enforce constant currency
+        currency = cls._enable_currency(currency_code)
+        if company.currency_id != currency:
+            cls.env.transaction.cache.set(cls.env.company, type(cls.env.company).currency_id, currency.id, dirty=True)
+            # this is equivalent to cls.env.company.currency_id = currency but without triggering buisness code checks.
+            # The value is added in cache, and the cache value is set as dirty so that that
+            # the value will be written to the database on next flush.
+            # this was needed because some journal entries may exist when running tests, especially l10n demo data.
+
+    @classmethod
+    def _create_partner(cls, **create_values):
+        return cls.env['res.partner'].create({
+            'name': "Test Partner",
+            'company_id': False,
+            **create_values,
+        })
+
+    @classmethod
+    def _create_company(cls, **create_values):
+        company = cls.env['res.company'].create({
+            'name': "Test Company",
+            **create_values,
+        })
+        cls.env.user.company_ids = [Command.link(company.id)]
+        # cls.env.context['allowed_company_ids'].append(company.id)
+        return company
+
+    @classmethod
+    def _create_new_internal_user(cls, **kwargs):
+        return new_test_user(
+            cls.env,
+            **({'login': 'internal_user'} | kwargs),
+        )
+
+    @classmethod
+    def _create_new_portal_user(cls, **kwargs):
+        return new_test_user(
+            cls.env,
+            groups='base.group_portal',
+            **({'login': 'portal_user'} | kwargs),
+        )
+
 
 class BaseUsersCommon(BaseCommon):
 
@@ -60,24 +153,8 @@ class BaseUsersCommon(BaseCommon):
     def setUpClass(cls):
         super().setUpClass()
 
-        cls.group_portal = cls.env.ref('base.group_portal')
-        cls.group_user = cls.env.ref('base.group_user')
-
-        cls.user_portal = cls.env['res.users'].create({
-            'name': 'Test Portal User',
-            'login': 'portal_user',
-            'password': 'portal_user',
-            'email': 'portal_user@gladys.portal',
-            'groups_id': [Command.set([cls.group_portal.id])],
-        })
-
-        cls.user_internal = cls.env['res.users'].create({
-            'name': 'Test Internal User',
-            'login': 'internal_user',
-            'password': 'internal_user',
-            'email': 'mark.brown23@example.com',
-            'groups_id': [Command.set([cls.group_user.id])],
-        })
+        cls.user_portal = cls._create_new_portal_user()
+        cls.user_internal = cls._create_new_internal_user()
 
 
 class TransactionCaseWithUserDemo(TransactionCase):
@@ -92,7 +169,6 @@ class TransactionCaseWithUserDemo(TransactionCase):
 
         if not cls.user_demo:
             cls.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
-            # YTI TODO: This could be factorized between the different classes
             cls.partner_demo = cls.env['res.partner'].create({
                 'name': 'Marc Demo',
                 'email': 'mark.brown23@example.com',
@@ -107,25 +183,26 @@ class TransactionCaseWithUserDemo(TransactionCase):
 
 class HttpCaseWithUserDemo(HttpCase):
 
-    def setUp(self):
-        super(HttpCaseWithUserDemo, self).setUp()
-        self.user_admin = self.env.ref('base.user_admin')
-        self.user_admin.write({'name': 'Mitchell Admin'})
-        self.partner_admin = self.user_admin.partner_id
-        self.user_demo = self.env['res.users'].search([('login', '=', 'demo')])
-        self.partner_demo = self.user_demo.partner_id
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.user_admin = cls.env.ref('base.user_admin')
+        cls.user_admin.write({'name': 'Mitchell Admin'})
+        cls.partner_admin = cls.user_admin.partner_id
+        cls.user_demo = cls.env['res.users'].search([('login', '=', 'demo')])
+        cls.partner_demo = cls.user_demo.partner_id
 
-        if not self.user_demo:
-            self.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
-            self.partner_demo = self.env['res.partner'].create({
+        if not cls.user_demo:
+            cls.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
+            cls.partner_demo = cls.env['res.partner'].create({
                 'name': 'Marc Demo',
                 'email': 'mark.brown23@example.com',
             })
-            self.user_demo = self.env['res.users'].create({
+            cls.user_demo = cls.env['res.users'].create({
                 'login': 'demo',
                 'password': 'demo',
-                'partner_id': self.partner_demo.id,
-                'groups_id': [Command.set([self.env.ref('base.group_user').id, self.env.ref('base.group_partner_manager').id])],
+                'partner_id': cls.partner_demo.id,
+                'groups_id': [Command.set([cls.env.ref('base.group_user').id, cls.env.ref('base.group_partner_manager').id])],
             })
 
 
@@ -133,7 +210,7 @@ class SavepointCaseWithUserDemo(TransactionCase):
 
     @classmethod
     def setUpClass(cls):
-        super(SavepointCaseWithUserDemo, cls).setUpClass()
+        super().setUpClass()
 
         cls.user_demo = cls.env['res.users'].search([('login', '=', 'demo')])
         cls.partner_demo = cls.user_demo.partner_id
@@ -254,24 +331,48 @@ class SavepointCaseWithUserDemo(TransactionCase):
             }
         ])
 
-class HttpCaseWithUserPortal(HttpCase):
 
-    def setUp(self):
-        super(HttpCaseWithUserPortal, self).setUp()
-        self.user_portal = self.env['res.users'].search([('login', '=', 'portal')])
-        self.partner_portal = self.user_portal.partner_id
+class TransactionCaseWithUserPortal(TransactionCase):
 
-        if not self.user_portal:
-            self.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
-            self.partner_portal = self.env['res.partner'].create({
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.user_portal = cls.env['res.users'].sudo().search([('login', '=', 'portal')])
+        cls.partner_portal = cls.user_portal.partner_id
+
+        if not cls.user_portal:
+            cls.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
+            cls.partner_portal = cls.env['res.partner'].create({
                 'name': 'Joel Willis',
                 'email': 'joel.willis63@example.com',
             })
-            self.user_portal = self.env['res.users'].with_context(no_reset_password=True).create({
+            cls.user_portal = cls.env['res.users'].with_context(no_reset_password=True).create({
                 'login': 'portal',
                 'password': 'portal',
-                'partner_id': self.partner_portal.id,
-                'groups_id': [Command.set([self.env.ref('base.group_portal').id])],
+                'partner_id': cls.partner_portal.id,
+                'groups_id': [Command.set([cls.env.ref('base.group_portal').id])],
+            })
+
+
+class HttpCaseWithUserPortal(HttpCase):
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.user_portal = cls.env['res.users'].sudo().search([('login', '=', 'portal')])
+        cls.partner_portal = cls.user_portal.partner_id
+
+        if not cls.user_portal:
+            cls.env['ir.config_parameter'].sudo().set_param('auth_password_policy.minlength', 4)
+            cls.partner_portal = cls.env['res.partner'].create({
+                'name': 'Joel Willis',
+                'email': 'joel.willis63@example.com',
+            })
+            cls.user_portal = cls.env['res.users'].with_context(no_reset_password=True).create({
+                'login': 'portal',
+                'password': 'portal',
+                'partner_id': cls.partner_portal.id,
+                'groups_id': [Command.set([cls.env.ref('base.group_portal').id])],
             })
 
 
@@ -328,59 +429,97 @@ class MockSmtplibCase:
         self.testing_smtp_session = TestingSMTPSession()
 
         IrMailServer = self.env['ir.mail_server']
-        connect_origin = IrMailServer.connect
-        find_mail_server_origin = IrMailServer._find_mail_server
+        connect_origin = type(IrMailServer).connect
+        find_mail_server_origin = type(IrMailServer)._find_mail_server
+
+        # custom mock to avoid losing context
+        def mock_function(func):
+            mock = Mock()
+
+            def _call(*args, **kwargs):
+                mock(*args[1:], **kwargs)
+                return func(*args, **kwargs)
+
+            _call.mock = mock
+            return _call
 
         with patch('smtplib.SMTP_SSL', side_effect=lambda *args, **kwargs: self.testing_smtp_session), \
              patch('smtplib.SMTP', side_effect=lambda *args, **kwargs: self.testing_smtp_session), \
-             patch.object(type(IrMailServer), '_is_test_mode', lambda self: False), \
-             patch.object(type(IrMailServer), 'connect', wraps=IrMailServer, side_effect=connect_origin) as connect_mocked, \
-             patch.object(type(IrMailServer), '_find_mail_server', side_effect=find_mail_server_origin) as find_mail_server_mocked:
-            self.connect_mocked = connect_mocked
-            self.find_mail_server_mocked = find_mail_server_mocked
+             patch.object(modules.module, 'current_test', False), \
+             patch.object(type(IrMailServer), 'connect', mock_function(connect_origin)) as connect_mocked, \
+             patch.object(type(IrMailServer), '_find_mail_server', mock_function(find_mail_server_origin)) as find_mail_server_mocked:
+            self.connect_mocked = connect_mocked.mock
+            self.find_mail_server_mocked = find_mail_server_mocked.mock
             yield
 
-    def assert_email_sent_smtp(self, smtp_from=None, smtp_to_list=None, message_from=None, from_filter=None, emails_count=1):
-        """Check that the given email has been sent.
+    def _build_email(self, mail_from, return_path=None, **kwargs):
+        return self.env['ir.mail_server'].build_email(
+            mail_from,
+            kwargs.pop('email_to', 'dest@example-é.com'),
+            kwargs.pop('subject', 'subject'),
+            kwargs.pop('body', 'body'),
+            headers={'Return-Path': return_path} if return_path else None,
+            **kwargs,
+        )
 
-        If one of the parameter is None, it's just ignored and not used to retrieve the email.
+    def _send_email(self, msg, smtp_session):
+        with patch.object(modules.module, 'current_test', False):
+            self.env['ir.mail_server'].send_email(msg, smtp_session=smtp_session)
+        return smtp_session.messages.pop()
+
+    def assertSMTPEmailsSent(self, smtp_from=None, smtp_to_list=None, message_from=None,
+                             mail_server=None, from_filter=None,
+                             emails_count=1):
+        """Check that the given email has been sent. If one of the parameter is
+        None it is just ignored and not used to retrieve the email.
 
         :param smtp_from: FROM used for the authentication to the mail server
         :param smtp_to_list: List of destination email address
         :param message_from: FROM used in the SMTP headers
-        :param from_filter: from_filter of the <ir.mail_server> used to send the email
-            Can use a lambda to check the value
+        :arap mail_server: used to compare the 'from_filter' as an alternative
+          to using the from_filter parameter
+        :param from_filter: from_filter of the <ir.mail_server> used to send the
+          email. False means 'match everything';'
         :param emails_count: the number of emails which should match the condition
         :return: True if at least one email has been found with those parameters
         """
+        if from_filter is not None and mail_server:
+            raise ValueError('Invalid usage: use either from_filter either mail_server')
+        if from_filter is None and mail_server is not None:
+            from_filter = mail_server.from_filter
         matching_emails = filter(
             lambda email:
-                (smtp_from is None or (
-                    smtp_from(email['smtp_from'])
-                    if callable(smtp_from)
-                    else smtp_from == email['smtp_from'])
-                 )
+                (smtp_from is None or smtp_from == email['smtp_from'])
                 and (smtp_to_list is None or smtp_to_list == email['smtp_to_list'])
                 and (message_from is None or 'From: %s' % message_from in email['message'])
                 and (from_filter is None or from_filter == email['from_filter']),
             self.emails,
         )
 
+        debug_info = ''
         matching_emails_count = len(list(matching_emails))
-
-        self.assertTrue(
-            matching_emails_count == emails_count,
-            msg='Emails not sent, %i emails match the condition but %i are expected' % (matching_emails_count, emails_count),
+        if matching_emails_count != emails_count:
+            emails_from = []
+            for email in self.emails:
+                from_found = next((
+                    line.split('From:')[1].strip() for line in email['message'].splitlines()
+                    if line.startswith('From:')), '')
+                emails_from.append(from_found)
+            debug_info = '\n'.join(
+                f"SMTP-From: {email['smtp_from']}, SMTP-To: {email['smtp_to_list']}, Msg-From: {email_msg_from}, From_filter: {email['from_filter']})"
+                for email, email_msg_from in zip(self.emails, emails_from)
+            )
+        self.assertEqual(
+            matching_emails_count, emails_count,
+            msg=f'Incorrect emails sent: {matching_emails_count} found, {emails_count} expected'
+                f'\nConditions\nSMTP-From: {smtp_from}, SMTP-To: {smtp_to_list}, Msg-From: {message_from}, From_filter: {from_filter}'
+                f'\nNot found in\n{debug_info}'
         )
 
     @classmethod
-    def _init_mail_config(cls):
-        cls.alias_bounce = 'bounce.test'
-        cls.alias_domain = 'test.com'
-        cls.default_from = 'notifications'
-        cls.env['ir.config_parameter'].sudo().set_param('mail.catchall.domain', cls.alias_domain)
-        cls.env['ir.config_parameter'].sudo().set_param('mail.default.from', cls.default_from)
-        cls.env['ir.config_parameter'].sudo().set_param('mail.bounce.alias', cls.alias_bounce)
+    def _init_mail_gateway(cls):
+        cls.default_from_filter = False
+        cls.env['ir.config_parameter'].sudo().set_param('mail.default.from_filter', cls.default_from_filter)
 
     @classmethod
     def _init_mail_servers(cls):
@@ -390,27 +529,30 @@ class MockSmtplibCase:
             'smtp_host': 'smtp_host',
             'smtp_encryption': 'none',
         }
-        (
-            cls.server_domain,
-            cls.server_user,
-            cls.server_notification,
-            cls.server_default,
-        ) = cls.env['ir.mail_server'].create([
+        cls.mail_servers = cls.env['ir.mail_server'].create([
             {
                 'name': 'Domain based server',
-                'from_filter': 'test.com',
+                'from_filter': 'test.mycompany.com',
+                'sequence': 0,
                 ** ir_mail_server_values,
             }, {
                 'name': 'User specific server',
-                'from_filter': 'specific_user@test.com',
+                'from_filter': 'specific_user@test.mycompany.com',
+                'sequence': 1,
                 ** ir_mail_server_values,
             }, {
                 'name': 'Server Notifications',
-                'from_filter': 'notifications@test.com',
+                'from_filter': 'notifications.test@test.mycompany.com',
+                'sequence': 2,
                 ** ir_mail_server_values,
             }, {
                 'name': 'Server No From Filter',
                 'from_filter': False,
+                'sequence': 3,
                 ** ir_mail_server_values,
             },
         ])
+        (
+            cls.mail_server_domain, cls.mail_server_user,
+            cls.mail_server_notification, cls.mail_server_default
+        ) = cls.mail_servers
