@@ -1,21 +1,25 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+import json
 import logging
 from functools import partial
+from unittest.mock import patch
 
 import psycopg2
 from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
 
 import odoo
+from odoo.modules.registry import Registry
 from odoo.sql_db import db_connect, TestCursor
 from odoo.tests import common
-from odoo.tests.common import BaseCase
+from odoo.tests.common import BaseCase, HttpCase
 from odoo.tools.misc import config
 
 ADMIN_USER_ID = common.ADMIN_USER_ID
 
+
 def registry():
-    return odoo.registry(common.get_db_name())
+    return Registry(common.get_db_name())
 
 
 class TestRealCursor(BaseCase):
@@ -47,6 +51,78 @@ class TestRealCursor(BaseCase):
         with registry().cursor() as cr:
             self.assertEqual(cr.connection.isolation_level, ISOLATION_LEVEL_REPEATABLE_READ)
 
+    def test_connection_readonly(self):
+        # even without db_replica, we expect the connection to be readonly for consistency
+        registry_ = registry()
+        with registry_.cursor(readonly=False) as cr:
+            cr.execute('SHOW transaction_read_only')
+            self.assertEqual(cr.fetchone(), ('off',))
+            self.assertFalse(cr._cnx.readonly)
+
+        with registry_.cursor(readonly=True) as cr:
+            cr.execute('SHOW transaction_read_only')
+            self.assertEqual(cr.fetchone(), ('on',))
+            self.assertTrue(cr._cnx.readonly)
+
+
+class TestHTTPCursor(HttpCase):
+    def test_cursor_keeps_readwriteness(self):
+        with self.env.registry.cursor(readonly=False) as cr:
+            self.assertFalse(cr.readonly)
+            cr.execute("SELECT 1")
+            cr.rollback()
+            self.assertFalse(cr.readonly)
+            cr.execute("SELECT 1")
+            cr.commit()
+            self.assertFalse(cr.readonly)
+
+        with self.env.registry.cursor(readonly=True) as cr:
+            self.assertTrue(cr.readonly)
+            cr.execute("SELECT 1")
+            cr.rollback()
+            self.assertTrue(cr.readonly)
+            cr.execute("SELECT 1")
+            cr.commit()
+            self.assertTrue(cr.readonly)
+
+    def test_call_kw_readonly(self):
+        self.authenticate('admin', 'admin')
+        self.env.user.partner_id.id
+
+        # a generic patcher to check if the method was called with a readonly cursor or not.
+        def return_readonly(self, *args, **kwargs):
+            return ['ok', self.env.cr.readonly]
+
+        with patch.object(type(self.env['res.partner']), 'read', return_readonly):
+            result_read = self.url_open('/web/dataset/call_kw', data=json.dumps({
+                "params": {
+                    'model': 'res.partner',
+                    'method': 'read',
+                    'args': [self.env.user.partner_id.id, ['name']],
+                    'kwargs': {},
+                },
+            }), headers={"Content-Type": "application/json"})
+            self.assertEqual(result_read.status_code, 200)
+            ok, readonly = result_read.json()['result']
+            self.assertEqual(ok, 'ok')
+            self.assertEqual(readonly, True, 'Call to read are expecte to be read only')
+
+
+        with patch.object(type(self.env['res.partner']), 'write', return_readonly):
+            result_write = self.url_open('/web/dataset/call_kw', data=json.dumps({
+                "params": {
+                    'model': 'res.partner',
+                    'method': 'write',
+                    'args': [self.env.user.partner_id.id, {'name': 'Urgo'}],
+                    'kwargs': {},
+                },
+            }), headers={"Content-Type": "application/json"})
+            self.assertEqual(result_write.status_code, 200)
+            ok, readonly = result_write.json()['result']
+            self.assertEqual(ok, 'ok')
+            self.assertEqual(readonly, False, 'Call to write are expecte to be read write')
+
+
 class TestTestCursor(common.TransactionCase):
     def setUp(self):
         super().setUp()
