@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+import collections
+import datetime
 import time
-from xmlrpc.client import Binary
 
 from odoo.exceptions import AccessDenied, AccessError
 from odoo.http import _request_stack
 
+import odoo
 import odoo.tools
 from odoo.tests import common
 from odoo.service import common as auth, model
@@ -40,6 +42,28 @@ class TestXMLRPC(common.HttpCase):
         ids = o.execute(db_name, self.admin_uid, 'admin', 'ir.model', 'search', [], {})
         self.assertIsInstance(ids, list)
 
+    def test_xmlrpc_datetime(self):
+        """ Test that native datetime can be sent over xmlrpc
+        """
+        m = self.env.ref('base.model_res_device_log')
+        self.env['ir.model.access'].create({
+            'name': "w/e",
+            'model_id': m.id,
+            'perm_read': True,
+            'perm_create': True,
+        })
+
+        now = datetime.datetime.now()
+        ids = self.xmlrpc(
+            'res.device.log', 'create',
+            {'session_identifier': "abc", 'first_activity': now}
+        )
+        [r] = self.xmlrpc(
+            'res.device.log', 'read',
+            ids, ['first_activity'],
+        )
+        self.assertEqual(r['first_activity'], now.isoformat(" ", "seconds"))
+
     def test_xmlrpc_read_group(self):
         groups = self.xmlrpc_object.execute(
             common.get_db_name(), self.admin_uid, 'admin',
@@ -65,6 +89,7 @@ class TestXMLRPC(common.HttpCase):
 
     def test_xmlrpc_frozendict_marshalling(self):
         """ Test that the marshalling of a frozendict object works properly over XMLRPC """
+        self.env.ref('base.user_admin').tz = "Europe/Brussels"
         ctx = self.xmlrpc_object.execute(
             common.get_db_name(), self.admin_uid, 'admin',
             'res.users', 'context_get',
@@ -72,6 +97,24 @@ class TestXMLRPC(common.HttpCase):
         self.assertEqual(ctx['lang'], 'en_US')
         self.assertEqual(ctx['tz'], 'Europe/Brussels')
 
+    def test_xmlrpc_defaultdict_marshalling(self):
+        """
+        Test that the marshalling of a collections.defaultdict object
+        works properly over XMLRPC
+        """
+        self.patch(self.registry['res.users'], 'context_get',
+                   odoo.api.model(lambda *_: collections.defaultdict(int)))
+        self.assertEqual(self.xmlrpc('res.users', 'context_get'), {})
+
+    def test_xmlrpc_remove_control_characters(self):
+        record = self.env['res.users'].create({
+            'name': 'bob with a control character: \x03',
+            'login': 'bob',
+        })
+        self.assertEqual(record.name, 'bob with a control character: \x03')
+        [record_data] = self.xmlrpc('res.users', 'read', record.id, ['name'])
+        self.assertEqual(record_data['name'], 'bob with a control character: ')
+
     def test_jsonrpc_read_group(self):
         self._json_call(
             common.get_db_name(), self.admin_uid, 'admin',
@@ -87,7 +130,7 @@ class TestXMLRPC(common.HttpCase):
         )
 
     def _json_call(self, *args):
-        self.opener.post("http://%s:%s/jsonrpc" % (common.HOST, odoo.tools.config['http_port']), json={
+        self.opener.post(f"{self.base_url()}/jsonrpc", json={
             'jsonrpc': '2.0',
             'id': None,
             'method': 'call',
@@ -99,11 +142,12 @@ class TestXMLRPC(common.HttpCase):
         })
 
     def test_xmlrpc_attachment_raw(self):
-        ids = self.env['ir.attachment'].create({'name': 'n', 'raw': b'\x01\02\03'}).ids
+        ids = self.env['ir.attachment'].create({'name': 'n', 'raw': b'\x01\x09'}).ids
         [att] = self.xmlrpc_object.execute(
             common.get_db_name(), self.admin_uid, 'admin',
             'ir.attachment', 'read', ids, ['raw'])
-        self.assertEqual(att['raw'], '', "actual binary data should be blanked out on read")
+        self.assertEqual(att['raw'], '\t',
+            "on read, binary data should be decoded as a string and stripped from control character")
 
 # really just for the test cursor
 @common.tagged('post_install', '-at_install')
@@ -120,16 +164,22 @@ class TestAPIKeys(common.HttpCase):
 
     def setUp(self):
         super().setUp()
+
+        def get_json_data():
+            raise ValueError("There is no json here")
         # needs a fake request in order to call methods protected with check_identity
         fake_req = DotDict({
             # various things go and access request items
             'httprequest': DotDict({
                 'environ': {'REMOTE_ADDR': 'localhost'},
                 'cookies': {},
+                'args': {},
             }),
+            'cookies': {},
             # bypass check_identity flow
             'session': {'identity-check-last': time.time()},
             'geoip': {},
+            'get_json_data': get_json_data,
         })
         _request_stack.push(fake_req)
         self.addCleanup(_request_stack.pop)
