@@ -27,7 +27,11 @@ class BaseModuleUninstall(models.TransientModel):
     def _compute_module_ids(self):
         for wizard in self:
             modules = wizard._get_modules().sorted(lambda m: (not m.application, m.sequence))
-            wizard.module_ids = modules if wizard.show_all else modules.filtered('application')
+            wizard.module_ids = modules if wizard.show_all else wizard._modules_to_display(modules)
+
+    @api.model
+    def _modules_to_display(self, modules):
+        return modules.filtered('application')
 
     def _get_models(self):
         """ Return the models (ir.model) to consider for the impact. """
