@@ -3,16 +3,15 @@
 
 from ast import literal_eval
 from collections import defaultdict
-import functools
 import itertools
 import logging
 import psycopg2
 import datetime
 
 from odoo import api, fields, models, Command
-from odoo import SUPERUSER_ID, _
+from odoo import _
 from odoo.exceptions import ValidationError, UserError
-from odoo.tools import mute_logger
+from odoo.tools import mute_logger, SQL
 
 _logger = logging.getLogger('odoo.addons.base.partner.merge')
 
@@ -101,16 +100,15 @@ class MergePartnerAutomatic(models.TransientModel):
         return self._cr.fetchall()
 
     @api.model
-    def _update_foreign_keys(self, src_partners, dst_partner):
-        """ Update all foreign key from the src_partner to dst_partner. All many2one fields will be updated.
-            :param src_partners : merge source res.partner recordset (does not include destination one)
-            :param dst_partner : record of destination res.partner
+    def _update_foreign_keys_generic(self, model, src_records, dst_record):
+        """ Update all foreign key from the src_records to dst_record for any model.
+            :param model: model name as a string
+            :param src_records: merge source recordset (does not include destination one)
+            :param dst_record: record of destination
         """
-        _logger.debug('_update_foreign_keys for dst_partner: %s for src_partners: %s', dst_partner.id, str(src_partners.ids))
+        _logger.debug('_update_foreign_keys_generic for dst_record: %s for src_records: %s', dst_record.id, str(src_records.ids))
 
-        # find the many2one relation to a partner
-        Partner = self.env['res.partner']
-        relations = self._get_fk_on('res_partner')
+        relations = self._get_fk_on(self.env[model]._table)
 
         # this guarantees cache consistency
         self.env.invalidate_all()
@@ -147,68 +145,55 @@ class MergePartnerAutomatic(models.TransientModel):
                                 "%(column)s" = %%s AND
                                 ___tu.%(value)s = ___tw.%(value)s
                         )""" % query_dic
-                for partner in src_partners:
-                    self._cr.execute(query, (dst_partner.id, partner.id, dst_partner.id))
+                for record in src_records:
+                    self._cr.execute(query, (dst_record.id, record.id, dst_record.id))
             else:
                 try:
                     with mute_logger('odoo.sql_db'), self._cr.savepoint():
                         query = 'UPDATE "%(table)s" SET "%(column)s" = %%s WHERE "%(column)s" IN %%s' % query_dic
-                        self._cr.execute(query, (dst_partner.id, tuple(src_partners.ids),))
-
-                        # handle the recursivity with parent relation
-                        if column == Partner._parent_name and table == 'res_partner':
-                            query = """
-                                WITH RECURSIVE cycle(id, parent_id) AS (
-                                        SELECT id, parent_id FROM res_partner
-                                    UNION
-                                        SELECT  cycle.id, res_partner.parent_id
-                                        FROM    res_partner, cycle
-                                        WHERE   res_partner.id = cycle.parent_id AND
-                                                cycle.id != cycle.parent_id
-                                )
-                                SELECT id FROM cycle WHERE id = parent_id AND id = %s
-                            """
-                            self._cr.execute(query, (dst_partner.id,))
-                            # NOTE JEM : shouldn't we fetch the data ?
+                        self._cr.execute(query, (dst_record.id, tuple(src_records.ids)))
                 except psycopg2.Error:
                     # updating fails, most likely due to a violated unique constraint
                     # keeping record with nonexistent partner_id is useless, better delete it
                     query = 'DELETE FROM "%(table)s" WHERE "%(column)s" IN %%s' % query_dic
-                    self._cr.execute(query, (tuple(src_partners.ids),))
+                    self._cr.execute(query, (tuple(src_records.ids),))
 
     @api.model
-    def _update_reference_fields(self, src_partners, dst_partner):
-        """ Update all reference fields from the src_partner to dst_partner.
-            :param src_partners : merge source res.partner recordset (does not include destination one)
-            :param dst_partner : record of destination res.partner
+    def _update_reference_fields_generic(self, referenced_model, src_records, dst_record, additional_update_records=None):
+        """ Update all reference fields from the src_records to dst_record for any model.
+            :param referenced_model: model name as a string
+            :param src_records: merge source recordset (does not include destination one)
+            :param dst_record: record of destination
+            :param additional_update_records: list of tuples (model, field_model, field_id)
         """
-        _logger.debug('_update_reference_fields for dst_partner: %s for src_partners: %r', dst_partner.id, src_partners.ids)
+        _logger.debug('_update_reference_fields_generic for dst_record: %s for src_records: %r', dst_record.id, src_records.ids)
 
         def update_records(model, src, field_model='model', field_id='res_id'):
             Model = self.env[model] if model in self.env else None
             if Model is None:
                 return
-            records = Model.sudo().search([(field_model, '=', 'res.partner'), (field_id, '=', src.id)])
+            records = Model.sudo().search([(field_model, '=', referenced_model), (field_id, '=', src.id)])
             try:
                 with mute_logger('odoo.sql_db'), self._cr.savepoint():
-                    records.sudo().write({field_id: dst_partner.id})
+                    records.sudo().write({field_id: dst_record.id})
                     records.env.flush_all()
             except psycopg2.Error:
                 # updating fails, most likely due to a violated unique constraint
                 # keeping record with nonexistent partner_id is useless, better delete it
                 records.sudo().unlink()
 
-        update_records = functools.partial(update_records)
+        for record in src_records:
+            update_records('ir.attachment', src=record, field_model='res_model')
+            update_records('mail.followers', src=record, field_model='res_model')
+            update_records('mail.activity', src=record, field_model='res_model')
+            update_records('mail.message', src=record)
+            update_records('ir.model.data', src=record)
 
-        for partner in src_partners:
-            update_records('calendar', src=partner, field_model='model_id.model')
-            update_records('ir.attachment', src=partner, field_model='res_model')
-            update_records('mail.followers', src=partner, field_model='res_model')
-            update_records('mail.activity', src=partner, field_model='res_model')
-            update_records('mail.message', src=partner)
-            update_records('ir.model.data', src=partner)
+        additional_update_records = additional_update_records or []
+        for update_record in additional_update_records:
+            update_records(update_record['model'], src=record, field_model=update_record['field_model'])
 
-        records = self.env['ir.model.fields'].sudo().search([('ttype', '=', 'reference')])
+        records = self.env['ir.model.fields'].sudo().search([('ttype', '=', 'reference'), ('store', '=', True)])
         for record in records:
             try:
                 Model = self.env[record.model]
@@ -220,15 +205,84 @@ class MergePartnerAutomatic(models.TransientModel):
             if Model._abstract or field.compute is not None:
                 continue
 
-            for partner in src_partners:
-                records_ref = Model.sudo().search([(record.name, '=', 'res.partner,%d' % partner.id)])
+            for src_record in src_records:
+                records_ref = Model.sudo().search([(record.name, '=', '%s,%d' % (referenced_model, src_record.id))])
                 values = {
-                    record.name: 'res.partner,%d' % dst_partner.id,
+                    record.name: '%s,%d' % (referenced_model, dst_record.id),
                 }
                 records_ref.sudo().write(values)
+        # company_dependent fields referring the merged records
+        for field in self.env.registry.many2one_company_dependents[dst_record._name]:
+            self.env.cr.execute(SQL(
+                """
+                UPDATE %(table)s
+                SET %(field)s = (
+                    SELECT jsonb_object_agg(key,
+                        CASE
+                            WHEN value::int IN %(src_record_ids)s
+                            THEN %(dest_record_id)s
+                            ELSE value::int
+                        END
+                    )
+                    FROM jsonb_each_text(%(field)s)
+                )
+                WHERE %(field)s IS NOT NULL
+                """,
+                table=SQL.identifier(self.env[field.model_name]._table),
+                field=SQL.identifier(field.name),
+                src_record_ids=tuple(src_records.ids),
+                dest_record_id=dst_record.id,
+            ))
 
         self.env.flush_all()
 
+        # company_dependent fields of merged records
+        with self._cr.savepoint():
+            for fname, field in dst_record._fields.items():
+                if field.company_dependent:
+                    self.env.execute_query(SQL(
+                        # use the specific company dependent value of sources
+                        # to fill the non-specific value of destination. Source
+                        # values for rows with larger id have higher priority
+                        # when aggregated
+                        """
+                        WITH source AS (
+                            SELECT %(field)s
+                            FROM  %(table)s
+                            WHERE id IN %(source_ids)s
+                            ORDER BY id
+                        ), source_agg AS (
+                            SELECT jsonb_object_agg(key, value) AS value
+                            FROM  source, jsonb_each(%(field)s)
+                        )
+                        UPDATE %(table)s
+                        SET %(field)s = source_agg.value || COALESCE(%(table)s.%(field)s, '{}'::jsonb)
+                        FROM source_agg
+                        WHERE id = %(destination_id)s AND source_agg.value IS NOT NULL
+                        """,
+                        table=SQL.identifier(dst_record._table),
+                        field=SQL.identifier(fname),
+                        destination_id=dst_record.id,
+                        source_ids=tuple(src_records.ids),
+                    ))
+
+    @api.model
+    def _update_foreign_keys(self, src_partners, dst_partner):
+        """ Update all foreign key from the src_partner to dst_partner. All many2one fields will be updated.
+            :param src_partners : merge source res.partner recordset (does not include destination one)
+            :param dst_partner : record of destination res.partner
+        """
+        self._update_foreign_keys_generic('res.partner', src_partners, dst_partner)
+
+    @api.model
+    def _update_reference_fields(self, src_partners, dst_partner):
+        """ Update all reference fields from the src_partner to dst_partner.
+            :param src_partners : merge source res.partner recordset (does not include destination one)
+            :param dst_partner : record of destination res.partner
+        """
+        additional_update_records = [{'model': 'calendar', 'field_model': 'model_id.model'}]
+        self._update_reference_fields_generic('res.partner', src_partners, dst_partner, additional_update_records)
+
     def _get_summable_fields(self):
         """ Returns the list of fields that should be summed when merging partners
         """
@@ -285,6 +339,23 @@ class MergePartnerAutomatic(models.TransientModel):
             except ValidationError:
                 _logger.info('Skip recursive partner hierarchies for parent_id %s of partner: %s', parent_id, dst_partner.id)
 
+    @api.model
+    def _merge_bank_accounts(self, src_partners, dst_partner):
+        """ Merge bank accounts of src_partners into dst_partner.
+            :param src_partners: merge source res.partner recordset (does not include destination one)
+            :param dst_partner: record of destination res.partner
+        """
+        all_src_accounts = src_partners.bank_ids
+
+        for src_account in all_src_accounts:
+            duplicate_account = dst_partner.bank_ids.filtered(lambda a: a.sanitized_acc_number == src_account.sanitized_acc_number)
+            if duplicate_account:
+                self._update_foreign_keys_generic('res.partner.bank', src_account, duplicate_account)
+                self._update_reference_fields_generic('res.partner.bank', src_account, duplicate_account)
+                src_account.unlink()
+            else:
+                src_account.write({'partner_id': dst_partner.id})
+
     def _merge(self, partner_ids, dst_partner=None, extra_checks=True):
         """ private implementation of merge partner
             :param partner_ids : ids of partner to merge
@@ -310,6 +381,10 @@ class MergePartnerAutomatic(models.TransientModel):
         if partner_ids & child_ids:
             raise UserError(_("You cannot merge a contact with one of his parent."))
 
+        # check if the list of partners to merge are linked to more than one user
+        if len(partner_ids.with_context(active_test=False).user_ids) > 1:
+            raise UserError(_("You cannot merge contacts linked to more than one user even if only one is active."))
+
         if extra_checks and len(set(partner.email for partner in partner_ids)) > 1:
             raise UserError(_("All contacts must have the same email. Only the Administrator can merge contacts with different emails."))
 
@@ -329,11 +404,16 @@ class MergePartnerAutomatic(models.TransientModel):
                 'company_id': dst_partner.company_id.id
             })
 
+        # Merge bank accounts before merging partners
+        self._merge_bank_accounts(src_partners, dst_partner)
+
         # call sub methods to do the merge
         self._update_foreign_keys(src_partners, dst_partner)
         self._update_reference_fields(src_partners, dst_partner)
         self._update_values(src_partners, dst_partner)
 
+        self.env.add_to_compute(dst_partner._fields['partner_share'], dst_partner)
+
         self._log_merge_operation(src_partners, dst_partner)
 
         # delete source partner, since they are merged
@@ -400,7 +480,7 @@ class MergePartnerAutomatic(models.TransientModel):
 
         for field_name in self._fields:
             if field_name.startswith(group_by_prefix):
-                if getattr(self, field_name, False):
+                if field_name in self and self[field_name]:
                     groups.append(field_name[len(group_by_prefix):])
 
         if not groups:
