@@ -1,14 +1,14 @@
 /** @odoo-module */
 
 import { registry } from "@web/core/registry";
-
-const { Component } = owl;
+import { Component } from "@odoo/owl";
 
 export class LazyTestComponent extends Component {
+    static template = "test_assetsbundle.LazyTestComponent";
+    static props = ["*"];
     setup() {
         this.props.onCreated();
     }
 }
-LazyTestComponent.template = "test_assetsbundle.LazyTestComponent";
 
 registry.category("lazy_components").add("LazyTestComponent", LazyTestComponent);
