@@ -6,7 +6,6 @@
     'category': 'Hidden/Tests',
     'description': """A module to verify the inheritance using _inherits in non-original modules.""",
     'depends': ['test_inherits'],
-    'data': [],
     'installable': True,
-    'auto_install': False,
+    'license': 'LGPL-3',
 }
