@@ -1,9 +1,15 @@
 import astroid
-from pylint import checkers, interfaces
+import pylint.interfaces
+from pylint import checkers
 
+def parse_version(s):
+    # can't use odoo.tools.parse_version because pythonpath is screwed from
+    # inside pylint on runbot
+    return [s.rjust(3, '0') for s in s.split('.')]
 
 class OdooBaseChecker(checkers.BaseChecker):
-    __implements__ = interfaces.IAstroidChecker
+    if parse_version(pylint.__version__) < parse_version('2.14.0'):
+        __implements__ = pylint.interfaces.IAstroidChecker
     name = 'odoo'
 
     msgs = {
