@@ -12,6 +12,8 @@ class BaseModel(models.Model):
     value = fields.Integer(default=0)
     value_pc = fields.Float(compute="_value_pc", store=True)
     value_ctx = fields.Float(compute="_value_ctx")
+    computed_value = fields.Float(compute="_computed_value")
+    indirect_computed_value = fields.Float(compute="_indirect_computed_value")
     partner_id = fields.Many2one('res.partner', string='Customer')
 
     line_ids = fields.One2many('test_performance.line', 'base_id')
@@ -23,6 +25,16 @@ class BaseModel(models.Model):
         for record in self:
             record.value_pc = float(record.value) / 100
 
+    @api.depends('value')
+    def _computed_value(self):
+        for record in self:
+            record.computed_value = float(record.value) / 100
+
+    @api.depends('computed_value')
+    def _indirect_computed_value(self):
+        for record in self:
+            record.indirect_computed_value = record.computed_value / 100
+
     @api.depends_context('key')
     def _value_ctx(self):
         self.env.cr.execute('SELECT 42')  # one dummy query per batch
@@ -81,3 +93,13 @@ class Mozzarella(models.Model):
     def _value_plus_one(self):
         for record in self:
             record.value_plus_one = record.value + 1
+
+
+class SimpleMinded(models.Model):
+    _name = _description = 'test_performance.simple.minded'
+
+    name = fields.Char()
+    active = fields.Boolean(default=True)
+    parent_id = fields.Many2one('test_performance.simple.minded')
+
+    child_ids = fields.One2many('test_performance.simple.minded', 'parent_id')
