@@ -7,9 +7,10 @@
 This here module is useful to validate that they're doing what they're 
 supposed to do
     """,
-    'depends': ['base'],
+    'depends': ['base', 'web'],
     'data': [
         'ir.model.access.csv',
         'menu.xml',
-    ]
+    ],
+    'license': 'LGPL-3',
 }
