@@ -2,24 +2,53 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 """ Modules migration handling. """
-
-from collections import defaultdict
 import glob
 import importlib.util
+import inspect
+import itertools
 import logging
 import os
+import re
+from collections import defaultdict
 from os.path import join as opj
 
-from odoo.modules.module import get_resource_path
 import odoo.release as release
 import odoo.upgrade
 from odoo.tools.parse_version import parse_version
+from odoo.tools.misc import file_path
 
 _logger = logging.getLogger(__name__)
 
 
+VERSION_RE = re.compile(
+    r"""^
+        # Optional prefix with Odoo version
+        ((
+            6\.1|
+
+            # "x.0" version, with x >= 6.
+            [6-9]\.0|
+
+            # multi digits "x.0" versions
+            [1-9]\d+\.0|
+
+            # x.saas~y, where x >= 7 and x <= 10
+            (7|8|9|10)\.saas~[1-9]\d*|
+
+            # saas~x.y, where x >= 11 and y between 1 and 9
+            # FIXME handle version >= saas~100 (expected in year 2106)
+            saas~(1[1-9]|[2-9]\d+)\.[1-9]
+        )\.)?
+        # After Odoo version we allow precisely 2 or 3 parts
+        # note this will also allow 0.0.0 which has a special meaning
+        \d+\.\d+(\.\d+)?
+    $""",
+    re.VERBOSE | re.ASCII,
+)
+
+
 def load_script(path, module_name):
-    full_path = get_resource_path(*path.split(os.path.sep)) if not os.path.isabs(path) else path
+    full_path = file_path(path) if not os.path.isabs(path) else path
     spec = importlib.util.spec_from_file_location(module_name, full_path)
     module = importlib.util.module_from_spec(spec)
     spec.loader.exec_module(module)
@@ -71,23 +100,44 @@ class MigrationManager(object):
                 if os.path.exists(upgrade_path):
                     yield upgrade_path
 
+        def _verify_upgrade_version(path, version):
+            full_path = opj(path, version)
+            if not os.path.isdir(full_path):
+                return False
+
+            if version == "tests":
+                return False
+
+            if not VERSION_RE.match(version):
+                _logger.warning("Invalid version for upgrade script %r", full_path)
+                return False
+
+            return True
+
         def get_scripts(path):
             if not path:
                 return {}
             return {
                 version: glob.glob(opj(path, version, '*.py'))
                 for version in os.listdir(path)
-                if os.path.isdir(opj(path, version))
+                if _verify_upgrade_version(path, version)
             }
 
+        def check_path(path):
+            try:
+                return file_path(path)
+            except FileNotFoundError:
+                return False
+
         for pkg in self.graph:
             if not (hasattr(pkg, 'update') or pkg.state == 'to upgrade' or
                     getattr(pkg, 'load_state', None) == 'to upgrade'):
                 continue
 
+
             self.migrations[pkg.name] = {
-                'module': get_scripts(get_resource_path(pkg.name, 'migrations')),
-                'module_upgrades': get_scripts(get_resource_path(pkg.name, 'upgrades')),
+                'module': get_scripts(check_path(pkg.name + '/migrations')),
+                'module_upgrades': get_scripts(check_path(pkg.name + '/upgrades')),
             }
 
             scripts = defaultdict(list)
@@ -109,8 +159,10 @@ class MigrationManager(object):
             return
 
         def convert_version(version):
-            if version.count('.') >= 2:
-                return version  # the version number already contains the server version
+            if version == "0.0.0":
+                return version
+            if version.count(".") > 2:
+                return version  # the version number already contains the server version, see VERSION_RE for details
             return "%s.%s" % (release.major_version, version)
 
         def _get_migration_versions(pkg, stage):
@@ -148,12 +200,24 @@ class MigrationManager(object):
         parsed_installed_version = parse_version(installed_version)
         current_version = parse_version(convert_version(pkg.data['version']))
 
+        def compare(version):
+            if version == "0.0.0" and parsed_installed_version < current_version:
+                return True
+
+            full_version = convert_version(version)
+            majorless_version = (version != full_version)
+
+            if majorless_version:
+                # We should not re-execute major-less scripts when upgrading to new Odoo version
+                # a module in `9.0.2.0` should not re-execute a `2.0` script when upgrading to `10.0.2.0`.
+                # In which case we must compare just the module version
+                return parsed_installed_version[2:] < parse_version(full_version)[2:] <= current_version[2:]
+
+            return parsed_installed_version < parse_version(full_version) <= current_version
+
         versions = _get_migration_versions(pkg, stage)
-
         for version in versions:
-            if ((version == "0.0.0" and parsed_installed_version < current_version)
-               or parsed_installed_version < parse_version(convert_version(version)) <= current_version):
-
+            if compare(version):
                 strfmt = {'addon': pkg.name,
                           'stage': stage,
                           'version': stageformat[stage] % version,
@@ -163,18 +227,34 @@ class MigrationManager(object):
                     name, ext = os.path.splitext(os.path.basename(pyfile))
                     if ext.lower() != '.py':
                         continue
-                    mod = None
                     try:
                         mod = load_script(pyfile, name)
-                        _logger.info('module %(addon)s: Running migration %(version)s %(name)s' % dict(strfmt, name=mod.__name__))
-                        migrate = mod.migrate
-                    except ImportError:
-                        _logger.exception('module %(addon)s: Unable to load %(stage)s-migration file %(file)s' % dict(strfmt, file=pyfile))
-                        raise
-                    except AttributeError:
-                        _logger.error('module %(addon)s: Each %(stage)s-migration file must have a "migrate(cr, installed_version)" function' % strfmt)
-                    else:
-                        migrate(self.cr, installed_version)
-                    finally:
-                        if mod:
-                            del mod
+                    except ImportError as e:
+                        raise ImportError('module %(addon)s: Unable to load %(stage)s-migration file %(file)s' % dict(strfmt, file=pyfile)) from e
+
+                    if not hasattr(mod, 'migrate'):
+                        raise AttributeError(
+                            'module %(addon)s: Each %(stage)s-migration file must have a "migrate(cr, installed_version)" function, not found in %(file)s' % dict(
+                                strfmt,
+                                file=pyfile,
+                            ))
+
+                    try:
+                        sig = inspect.signature(mod.migrate)
+                    except TypeError as e:
+                        raise TypeError("module %(addon)s: `migrate` needs to be a function, got %(migrate)r" % dict(strfmt, migrate=mod.migrate)) from e
+
+                    if not (
+                            tuple(sig.parameters.keys()) in VALID_MIGRATE_PARAMS
+                        and all(p.kind in (p.POSITIONAL_ONLY, p.POSITIONAL_OR_KEYWORD) for p in sig.parameters.values())
+                    ):
+                        raise TypeError("module %(addon)s: `migrate`'s signature should be `(cr, version)`, %(func)s is %(sig)s" % dict(strfmt, func=mod.migrate, sig=sig))
+
+                    _logger.info('module %(addon)s: Running migration %(version)s %(name)s' % dict(strfmt, name=mod.__name__))  # noqa: G002
+                    mod.migrate(self.cr, installed_version)
+
+
+VALID_MIGRATE_PARAMS = list(itertools.product(
+    ['cr', '_cr'],
+    ['version', '_version'],
+))
