@@ -6,20 +6,35 @@ import collections.abc
 import copy
 import functools
 import importlib
+import importlib.metadata
 import logging
 import os
-import pkg_resources
 import re
 import sys
+import traceback
 import warnings
 from os.path import join as opj, normpath
 
 import odoo
 import odoo.tools as tools
 import odoo.release as release
-from odoo.tools import pycompat
 from odoo.tools.misc import file_path
 
+try:
+    from packaging.requirements import InvalidRequirement, Requirement
+except ImportError:
+    class InvalidRequirement(Exception):
+        ...
+
+    class Requirement:
+        def __init__(self, pydep):
+            if not re.fullmatch(r'\w+', pydep):  # check that we have no versions or marker in pydep
+                msg = f"Package `packaging` is required to parse `{pydep}` external dependency and is not installed"
+                raise Exception(msg)
+            self.marker = None
+            self.specifier = None
+            self.name = pydep
+
 
 MANIFEST_NAMES = ('__manifest__.py', '__openerp__.py')
 README = ['README.rst', 'README.md', 'README.txt']
@@ -32,12 +47,15 @@ _DEFAULT_MANIFEST = {
     'author': 'Odoo S.A.',
     'auto_install': False,
     'category': 'Uncategorized',
+    'cloc_exclude': [],
+    'configurator_snippets': {},  # website themes
+    'countries': [],
     'data': [],
     'demo': [],
     'demo_xml': [],
     'depends': [],
     'description': '',
-    'external_dependencies': [],
+    'external_dependencies': {},
     #icon: f'/{module}/static/description/icon.png',  # automatic
     'init_xml': [],
     'installable': True,
@@ -45,12 +63,12 @@ _DEFAULT_MANIFEST = {
     'images_preview_theme': {},  # website themes
     #license, mandatory
     'live_test_url': '',  # website themes
+    'new_page_templates': {},  # website themes
     #name, mandatory
     'post_init_hook': '',
-    'post_load': None,
+    'post_load': '',
     'pre_init_hook': '',
     'sequence': 100,
-    'snippet_lists': {},  # website themes
     'summary': '',
     'test': [],
     'update_xml': [],
@@ -60,103 +78,25 @@ _DEFAULT_MANIFEST = {
     'website': '',
 }
 
+# matches field definitions like
+#     partner_id: base.ResPartner = fields.Many2one
+#     partner_id = fields.Many2one[base.ResPartner]
+TYPED_FIELD_DEFINITION_RE = re.compile(r'''
+    \b (?P<field_name>\w+) \s*
+    (:\s*(?P<field_type>[^ ]*))? \s*
+    = \s*
+    fields\.(?P<field_class>Many2one|One2many|Many2many)
+    (\[(?P<type_param>[^\]]+)\])?
+''', re.VERBOSE)
+
 _logger = logging.getLogger(__name__)
 
-# addons path as a list
-# ad_paths is a deprecated alias, please use odoo.addons.__path__
-@tools.lazy
-def ad_paths():
-    warnings.warn(
-        '"odoo.modules.module.ad_paths" is a deprecated proxy to '
-        '"odoo.addons.__path__".', DeprecationWarning, stacklevel=2)
-    return odoo.addons.__path__
-
-# Modules already loaded
-loaded = []
-
-class AddonsHook(object):
-    """ Makes modules accessible through openerp.addons.* """
-
-    def find_module(self, name, path=None):
-        if name.startswith('openerp.addons.') and name.count('.') == 2:
-            warnings.warn(
-                '"openerp.addons" is a deprecated alias to "odoo.addons".',
-                DeprecationWarning, stacklevel=2)
-            return self
-
-    def find_spec(self, fullname, path=None, target=None):
-        if fullname.startswith('openerp.addons.') and fullname.count('.') == 2:
-            warnings.warn(
-                '"openerp.addons" is a deprecated alias to "odoo.addons".',
-                DeprecationWarning, stacklevel=2)
-            return importlib.util.spec_from_loader(fullname, self)
-
-    def load_module(self, name):
-        assert name not in sys.modules
-
-        odoo_name = re.sub(r'^openerp.addons.(\w+)$', r'odoo.addons.\g<1>', name)
-
-        odoo_module = sys.modules.get(odoo_name)
-        if not odoo_module:
-            odoo_module = importlib.import_module(odoo_name)
-
-        sys.modules[name] = odoo_module
-
-        return odoo_module
-
-class OdooHook(object):
-    """ Makes odoo package also available as openerp """
-
-    def find_module(self, name, path=None):
-        # openerp.addons.<identifier> should already be matched by AddonsHook,
-        # only framework and subdirectories of modules should match
-        if re.match(r'^openerp\b', name):
-            warnings.warn(
-                'openerp is a deprecated alias to odoo.',
-                DeprecationWarning, stacklevel=2)
-            return self
-
-    def find_spec(self, fullname, path=None, target=None):
-        # openerp.addons.<identifier> should already be matched by AddonsHook,
-        # only framework and subdirectories of modules should match
-        if re.match(r'^openerp\b', fullname):
-            warnings.warn(
-                'openerp is a deprecated alias to odoo.',
-                DeprecationWarning, stacklevel=2)
-            return importlib.util.spec_from_loader(fullname, self)
-
-    def load_module(self, name):
-        assert name not in sys.modules
-
-        canonical = re.sub(r'^openerp(.*)', r'odoo\g<1>', name)
-
-        if canonical in sys.modules:
-            mod = sys.modules[canonical]
-        else:
-            # probable failure: canonical execution calling old naming -> corecursion
-            mod = importlib.import_module(canonical)
-
-        # just set the original module at the new location. Don't proxy,
-        # it breaks *-import (unless you can find how `from a import *` lists
-        # what's supposed to be imported by `*`, and manage to override it)
-        sys.modules[name] = mod
-
-        return sys.modules[name]
-
 
 class UpgradeHook(object):
     """Makes the legacy `migrations` package being `odoo.upgrade`"""
 
-    def find_module(self, name, path=None):
-        if re.match(r"^odoo\.addons\.base\.maintenance\.migrations\b", name):
-            # We can't trigger a DeprecationWarning in this case.
-            # In order to be cross-versions, the multi-versions upgrade scripts (0.0.0 scripts),
-            # the tests, and the common files (utility functions) still needs to import from the
-            # legacy name.
-            return self
-
     def find_spec(self, fullname, path=None, target=None):
-        if re.match(r"^odoo.addons.base.maintenance.migrations\b", fullname):
+        if re.match(r"^odoo\.addons\.base\.maintenance\.migrations\b", fullname):
             # We can't trigger a DeprecationWarning in this case.
             # In order to be cross-versions, the multi-versions upgrade scripts (0.0.0 scripts),
             # the tests, and the common files (utility functions) still needs to import from the
@@ -190,7 +130,7 @@ def initialize_sys_path():
 
     # hook odoo.addons on addons paths
     for ad in tools.config['addons_path'].split(','):
-        ad = os.path.normcase(os.path.abspath(tools.ustr(ad.strip())))
+        ad = os.path.normcase(os.path.abspath(ad.strip()))
         if ad not in odoo.addons.__path__:
             odoo.addons.__path__.append(ad)
 
@@ -203,7 +143,7 @@ def initialize_sys_path():
     from odoo import upgrade
     legacy_upgrade_path = os.path.join(base_path, 'base', 'maintenance', 'migrations')
     for up in (tools.config['upgrade_path'] or legacy_upgrade_path).split(','):
-        up = os.path.normcase(os.path.abspath(tools.ustr(up.strip())))
+        up = os.path.normcase(os.path.abspath(up.strip()))
         if os.path.isdir(up) and up not in upgrade.__path__:
             upgrade.__path__.append(up)
 
@@ -217,8 +157,6 @@ def initialize_sys_path():
     # hook deprecated module alias from openerp to odoo and "crm"-like to odoo.addons
     if not getattr(initialize_sys_path, 'called', False): # only initialize once
         sys.meta_path.insert(0, UpgradeHook())
-        sys.meta_path.insert(0, OdooHook())
-        sys.meta_path.insert(0, AddonsHook())
         initialize_sys_path.called = True
 
 
@@ -244,39 +182,6 @@ def get_module_path(module, downloaded=False, display_warning=True):
         _logger.warning('module %s: module not found', module)
     return False
 
-def get_module_filetree(module, dir='.'):
-    warnings.warn(
-        "Since 16.0: use os.walk or a recursive glob or something",
-        DeprecationWarning,
-        stacklevel=2
-    )
-    path = get_module_path(module)
-    if not path:
-        return False
-
-    dir = os.path.normpath(dir)
-    if dir == '.':
-        dir = ''
-    if dir.startswith('..') or (dir and dir[0] == '/'):
-        raise Exception('Cannot access file outside the module')
-
-    files = odoo.tools.osutil.listdir(path, True)
-
-    tree = {}
-    for f in files:
-        if not f.startswith(dir):
-            continue
-
-        if dir:
-            f = f[len(dir)+int(not dir.endswith('/')):]
-        lst = f.split(os.sep)
-        current = tree
-        while len(lst) != 1:
-            current = current.setdefault(lst.pop(0), {})
-        current[lst.pop(0)] = None
-
-    return tree
-
 def get_resource_path(module, *args):
     """Return the full path of a resource of the given module.
 
@@ -286,21 +191,19 @@ def get_resource_path(module, *args):
     :rtype: str
     :return: absolute path to the resource
     """
+    warnings.warn(
+        f"Since 17.0: use tools.misc.file_path instead of get_resource_path({module}, {args})",
+        DeprecationWarning,
+    )
     resource_path = opj(module, *args)
     try:
         return file_path(resource_path)
     except (FileNotFoundError, ValueError):
         return False
 
-def check_resource_path(mod_path, *args):
-    resource_path = opj(mod_path, *args)
-    try:
-        return file_path(resource_path)
-    except (FileNotFoundError, ValueError):
-        return False
-
 # backwards compatibility
 get_module_resource = get_resource_path
+check_resource_path = get_resource_path
 
 def get_resource_from_path(path):
     """Tries to extract the module name and the resource's relative path
@@ -335,25 +238,34 @@ def get_resource_from_path(path):
     return None
 
 def get_module_icon(module):
-    iconpath = ['static', 'description', 'icon.png']
-    if get_module_resource(module, *iconpath):
-        return ('/' + module + '/') + '/'.join(iconpath)
-    return '/base/'  + '/'.join(iconpath)
+    fpath = f"{module}/static/description/icon.png"
+    try:
+        file_path(fpath)
+        return "/" + fpath
+    except FileNotFoundError:
+        return "/base/static/description/icon.png"
 
 def get_module_icon_path(module):
-    iconpath = ['static', 'description', 'icon.png']
-    path = get_module_resource(module.name, *iconpath)
-    if not path:
-        path = get_module_resource('base', *iconpath)
-    return path
+    try:
+        return file_path(f"{module}/static/description/icon.png")
+    except FileNotFoundError:
+        return file_path("base/static/description/icon.png")
 
 def module_manifest(path):
     """Returns path to module manifest if one can be found under `path`, else `None`."""
     if not path:
         return None
     for manifest_name in MANIFEST_NAMES:
-        if os.path.isfile(opj(path, manifest_name)):
-            return opj(path, manifest_name)
+        candidate = opj(path, manifest_name)
+        if os.path.isfile(candidate):
+            if manifest_name == '__openerp__.py':
+                warnings.warn(
+                    "__openerp__.py manifests are deprecated since 17.0, "
+                    f"rename {candidate!r} to __manifest__.py "
+                    "(valid since 10.0)",
+                    category=DeprecationWarning
+                )
+            return candidate
 
 def get_module_root(path):
     """
@@ -394,6 +306,7 @@ def load_manifest(module, mod_path=None):
         return {}
 
     manifest = copy.deepcopy(_DEFAULT_MANIFEST)
+
     manifest['icon'] = get_module_icon(module)
 
     with tools.file_open(manifest_file, mode='r') as f:
@@ -426,12 +339,15 @@ def load_manifest(module, mod_path=None):
     elif manifest['auto_install']:
         manifest['auto_install'] = set(manifest['depends'])
 
-    manifest['version'] = adapt_version(manifest['version'])
+    try:
+        manifest['version'] = adapt_version(manifest['version'])
+    except ValueError as e:
+        if manifest.get("installable", True):
+            raise ValueError(f"Module {module}: invalid manifest") from e
     manifest['addons_path'] = normpath(opj(mod_path, os.pardir))
 
     return manifest
 
-@functools.lru_cache(maxsize=None)
 def get_manifest(module, mod_path=None):
     """
     Get the module manifest.
@@ -444,13 +360,12 @@ def get_manifest(module, mod_path=None):
         when the manifest was not found.
     :rtype: dict
     """
+    return copy.deepcopy(_get_manifest_cached(module, mod_path))
+
+@functools.lru_cache(maxsize=None)
+def _get_manifest_cached(module, mod_path=None):
     return load_manifest(module, mod_path)
 
-def load_information_from_description_file(module, mod_path=None):
-    warnings.warn(
-        'load_information_from_description_file() is a deprecated '
-        'alias to get_manifest()', DeprecationWarning, stacklevel=2)
-    return get_manifest(module, mod_path)
 
 def load_openerp_module(module_name):
     """ Load an OpenERP module, if not already loaded.
@@ -460,27 +375,42 @@ def load_openerp_module(module_name):
     This is also used to load server-wide module (i.e. it is also used
     when there is no model to register).
     """
-    global loaded
-    if module_name in loaded:
+
+    qualname = f'odoo.addons.{module_name}'
+    if qualname in sys.modules:
         return
 
     try:
-        __import__('odoo.addons.' + module_name)
+        __import__(qualname)
 
         # Call the module's post-load hook. This can done before any model or
         # data has been initialized. This is ok as the post-load hook is for
         # server-wide (instead of registry-specific) functionalities.
         info = get_manifest(module_name)
         if info['post_load']:
-            getattr(sys.modules['odoo.addons.' + module_name], info['post_load'])()
+            getattr(sys.modules[qualname], info['post_load'])()
 
-    except Exception as e:
-        msg = "Couldn't load module %s" % (module_name)
-        _logger.critical(msg)
-        _logger.critical(e)
+    except AttributeError as err:
+        _logger.critical("Couldn't load module %s", module_name)
+        trace = traceback.format_exc()
+        match = TYPED_FIELD_DEFINITION_RE.search(trace)
+        if match and "most likely due to a circular import" in trace:
+            field_name = match['field_name']
+            field_class = match['field_class']
+            field_type = match['field_type'] or match['type_param']
+            if "." not in field_type:
+                field_type = f"{module_name}.{field_type}"
+            raise AttributeError(
+                f"{err}\n"
+                "To avoid circular import for the the comodel use the annotation syntax:\n"
+                f"    {field_name}: {field_type} = fields.{field_class}(...)\n"
+                "and add at the beggining of the file:\n"
+                "    from __future__ import annotations"
+            ).with_traceback(err.__traceback__) from None
+        raise
+    except Exception:
+        _logger.critical("Couldn't load module %s", module_name)
         raise
-    else:
-        loaded.append(module_name)
 
 def get_modules():
     """Returns the list of module names
@@ -508,7 +438,7 @@ def get_modules():
             _logger.warning("addons path does not exist: %s", ad)
             continue
         plist.extend(listdir(ad))
-    return list(set(plist))
+    return sorted(set(plist))
 
 def get_modules_with_version():
     modules = get_modules()
@@ -524,29 +454,48 @@ def get_modules_with_version():
 def adapt_version(version):
     serie = release.major_version
     if version == serie or not version.startswith(serie + '.'):
+        base_version = version
         version = '%s.%s' % (serie, version)
+    else:
+        base_version = version[len(serie) + 1:]
+
+    if not re.match(r"^[0-9]+\.[0-9]+(?:\.[0-9]+)?$", base_version):
+        raise ValueError(f"Invalid version {base_version!r}. Modules should have a version in format `x.y`, `x.y.z`,"
+                         f" `{serie}.x.y` or `{serie}.x.y.z`.")
+
     return version
 
-current_test = None
+
+current_test = False
 
 
 def check_python_external_dependency(pydep):
     try:
-        pkg_resources.get_distribution(pydep)
-    except pkg_resources.DistributionNotFound as e:
+        requirement = Requirement(pydep)
+    except InvalidRequirement as e:
+        msg = f"{pydep} is an invalid external dependency specification: {e}"
+        raise Exception(msg) from e
+    if requirement.marker and not requirement.marker.evaluate():
+        _logger.debug(
+            "Ignored external dependency %s because environment markers do not match",
+            pydep
+        )
+        return
+    try:
+        version = importlib.metadata.version(requirement.name)
+    except importlib.metadata.PackageNotFoundError as e:
         try:
+            # keep compatibility with module name but log a warning instead of info
             importlib.import_module(pydep)
-            _logger.info("python external dependency on '%s' does not appear to be a valid PyPI package. Using a PyPI package name is recommended.", pydep)
+            _logger.warning("python external dependency on '%s' does not appear o be a valid PyPI package. Using a PyPI package name is recommended.", pydep)
+            return
         except ImportError:
-            # backward compatibility attempt failed
-            _logger.warning("DistributionNotFound: %s", e)
-            raise Exception('Python library not installed: %s' % (pydep,))
-    except pkg_resources.VersionConflict as e:
-        _logger.warning("VersionConflict: %s", e)
-        raise Exception('Python library version conflict: %s' % (pydep,))
-    except Exception as e:
-        _logger.warning("get_distribution(%s) failed: %s", pydep, e)
-        raise Exception('Error finding python library %s' % (pydep,))
+            pass
+        msg = f"External dependency {pydep} not installed: {e}"
+        raise Exception(msg) from e
+    if requirement.specifier and not requirement.specifier.contains(version):
+        msg = f"External dependency version mismatch: {pydep} (installed: {version})"
+        raise Exception(msg)
 
 
 def check_manifest_dependencies(manifest):
