@@ -5,9 +5,6 @@ from . import db
 from . import model
 from . import server
 
-# deprecated since 15.3
-from . import wsgi_server
-
 #.apidoc title: RPC Services
 
 """ Classes of this module implement the network protocols that the
